@@ -6,22 +6,75 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/baseframework/app/basecontroller', [
     'io.ox/office/tk/utils',
+    'io.ox/office/tk/utils/driveutils',
     'io.ox/office/tk/keycodes',
     'io.ox/office/tk/dialogs',
     'io.ox/office/tk/object/baseobject',
     'io.ox/office/tk/object/triggerobject',
     'io.ox/office/tk/object/timermixin'
-], function (Utils, KeyCodes, Dialogs, BaseObject, TriggerObject, TimerMixin) {
+], function (Utils, DriveUtils, KeyCodes, Dialogs, BaseObject, TriggerObject, TimerMixin) {
 
     'use strict';
 
+    // class ItemState ========================================================
+
+    /**
+     * A descriptor for the complete state of a controller item.
+     *
+     * @constructor
+     *
+     * @property {String} key
+     *  The key of the controller item.
+     *
+     * @property {Boolean} enabled
+     *  The enabled state of the controller item.
+     *
+     * @property {Any} value
+     *  The current value (the result of the getter callback) of the controller
+     *  item.
+     *
+     * @property {Boolean} dirty
+     *  Whether the item has been set to dirty state. A dirty item will always
+     *  be updated, regardless if it has been changed since the last update.
+     *
+     * @property {Array<ItemState>} parentStates
+     *  The item states of all parent items of the controller item, in order of
+     *  the parent keys registered for the item. If the item does not contain
+     *  any parent items, this property will be an empty array.
+     *
+     * @property {Array<Any>} parentValues
+     *  The current values (results of all getter callbacks) of the parent
+     *  items, in order of the parent keys registered for the item. This
+     *  property is provided for convenience (the parent values are contained
+     *  in the property 'parentStates' of this instance).
+     *
+     * @property {Boolean} parentsEnabled
+     *  The accumulated enabled state of the parent items of the controller
+     *  item. Will be true, if all parent items are enabled.
+     */
+    function ItemState(key, parentStates) {
+
+        this.key = key;
+        this.enabled = false;
+        this.value = undefined;
+        this.dirty = false;
+
+        // parent states and values
+        this.parentStates = parentStates;
+        this.parentValues = _.pluck(parentStates, 'value');
+
+        // all parents must be enabled for 'parentsEnabled' being true
+        this.parentsEnabled = parentStates.every(function (state) { return state.enabled; });
+
+    } // class ItemState
+
     // class BaseController ===================================================
 
     /**
@@ -32,11 +85,11 @@ define('io.ox/office/baseframework/app/basecontroller', [
      * Triggers the following events:
      * - 'change:items'
      *      After the current state (value and/or enabled state) of at least
-     *      one controller item has been changed after the controller has
+     *      one controller item has been changed, after the controller has
      *      updated itself or the method BaseController.update() has been
-     *      invoked. Event listeners receive data about the changed items in a
-     *      map with item identifiers as keys, and objects containing the
-     *      optional properties 'value' and 'enable' as values.
+     *      invoked. Event listeners receive the descriptors of the changed
+     *      items in a map with item identifiers as keys, and instances of the
+     *      class ItemState as values.
      *
      * @constructor
      *
@@ -46,20 +99,23 @@ define('io.ox/office/baseframework/app/basecontroller', [
      * @param {BaseApplication} app
      *  The application that has created this controller instance.
      *
+     * @param {BaseModel} docModel
+     *  The document model created by the passed application.
+     *
+     * @param {BaseView} docView
+     *  The document view created by the passed application.
+     *
      * @param {Object} [initOptions]
      *  Optional parameters:
      *  @param {Number} [initOptions.updateDelay=50]
      *      The delay for the debounced Controller.update() method, in
      *      milliseconds. Must not be less than 50.
      */
-    function BaseController(app, initOptions) {
+    function BaseController(app, docModel, docView, initOptions) {
 
         var // self reference
             self = this,
 
-            // the application view
-            view = null,
-
             // all registered controller items, mapped by item key
             items = {},
 
@@ -75,6 +131,9 @@ define('io.ox/office/baseframework/app/basecontroller', [
             // cached item states, mapped by item keys
             stateCache = {},
 
+            // cached states of all controller items as notified the last time
+            notifiedStateCache = {},
+
             // number of item setters currently running (recursion counter)
             runningSetters = 0,
 
@@ -82,46 +141,7 @@ define('io.ox/office/baseframework/app/basecontroller', [
             keyShortcuts = {},
 
             // shortcut definitions for 'keypress', mapped by char code (for performance)
-            charShortcuts = {},
-
-            // definitions for items supported by all application types
-            BASE_ITEM_DEFINITIONS = {
-
-                // quit/destroy the application (do not return the promise,
-                // otherwise this controller would wait for its own destruction)
-                'app/quit': {
-                    set: function () { app.quit(); }
-                },
-
-                // disabled during import, enabled afterwards
-                'app/imported': {
-                    enable: function () { return app.isImportFinished(); }
-                },
-
-                // disabled in stand-alone mode
-                'app/bundled': {
-                    enable: function () { return !app.isStandalone(); }
-                },
-
-                // download the document: enabled in read-only mode, disabled during import
-                'document/download': {
-                    parent: 'app/imported',
-                    set: function () { return app.download(); }
-                },
-
-                // print the document (download as PDF): enabled in read-only mode, disabled during import
-                'document/print': {
-                    parent: 'app/imported',
-                    set: function () { return app.print(); },
-                    shortcut: { keyCode: 'P', ctrlOrMeta: true }
-                },
-
-                // send document as mail: enabled in read-only mode, disabled during import, disabled in stand-alone-mode
-                'document/sendmail': {
-                    parent: ['app/imported', 'app/bundled'],
-                    set: function () { return app.sendMail(); }
-                }
-            };
+            charShortcuts = {};
 
         // class Item ---------------------------------------------------------
 
@@ -135,6 +155,8 @@ define('io.ox/office/baseframework/app/basecontroller', [
                 getHandler = Utils.getFunctionOption(definition, 'get', _.identity),
                 // handler for value setter
                 setHandler = Utils.getFunctionOption(definition, 'set'),
+                // whether an asynchronous setter is cancelable
+                cancelable = Utils.getBooleanOption(definition, 'cancel', false),
                 // behavior for returning browser focus to application
                 itemPreserveFocus = Utils.getBooleanOption(definition, 'preserveFocus', false),
                 // custom focus target node or callback handler
@@ -148,9 +170,11 @@ define('io.ox/office/baseframework/app/basecontroller', [
              * Returns the state descriptor of the specified parent item.
              */
             function getParentState(parentKey) {
-                if (parentKey in items) { return items[parentKey].getState(); }
-                Utils.warn('BaseController.Item.getParentState(): item "' + key + '" refers to unknown parent "' + parentKey + '"');
-                return dummyItem.getState();
+                var parentItem = items[parentKey] || dummyItem;
+                if (parentItem === dummyItem) {
+                    Utils.warn('BaseController.Item.getParentState(): item "' + key + '" refers to unknown parent "' + parentKey + '"');
+                }
+                return parentItem.getState();
             }
 
             // methods  - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -166,53 +190,31 @@ define('io.ox/office/baseframework/app/basecontroller', [
             };
 
             /**
-             * Returns a raw descriptor for the current state of this item.
+             * Returns a descriptor for the current state of this item.
              *
-             * @returns {Object}
-             *  A state descriptor with the following properties:
-             *  - {Boolean|Null} parentsEnabled
-             *      The enabled state of all parents of this item. The value
-             *      null represents the undetermined state (no 'enable'
-             *      callback handlers found at the parents, or no parents
-             *      defined).
-             *  - {Boolean|Null} enabled
-             *      The enabled state of this item. The value null represents
-             *      the undetermined state (no 'enable' callback handlers
-             *      found).
-             *  - {Array} parentValues
-             *      The values (results of all getter callbacks) of the parent
-             *      items.
-             *  - {Array} value
-             *      The value (result of the getter callback) of the item.
+             * @returns {ItemState}
+             *  The state descriptor for this controller item.
              */
             this.getState = function () {
 
                 // return cached state
                 if (key in stateCache) { return stateCache[key]; }
 
-                var // the states of all parent items
-                    parentStates = _.map(parentKeys, getParentState),
-                    // determine whether all parent items are enabled
-                    parentsEnabled = _.all(parentStates, function (state) { return state.enabled; }),
-                    // all parent values, as array
-                    parentValues = _.pluck(parentStates, 'value'),
-                    // create the state descriptor of this item
-                    itemState = {
-                        parentStates: parentStates,
-                        parentsEnabled: parentsEnabled,
-                        parentValues: parentValues,
-                        enabled: false,
-                        value: undefined
-                    };
+                // create the state descriptor of this item
+                var itemState = new ItemState(key, _.map(parentKeys, getParentState));
 
                 // insert the state descriptor into the cache before resolving enabled state and value
                 stateCache[key] = itemState;
 
                 // resolve and store the value first (enable callback may want to query the own value)
-                itemState.value = getHandler.apply(this, parentValues);
+                itemState.value = getHandler.apply(this, itemState.parentValues);
 
                 // resolve and store the enabled state
-                itemState.enabled = parentsEnabled && (enableHandler.apply(this, parentValues) === true);
+                itemState.enabled = itemState.parentsEnabled && (enableHandler.apply(this, itemState.parentValues) === true);
+
+                // copy dirty flag from previous state cache
+                var oldItemState = notifiedStateCache[key];
+                itemState.dirty = !oldItemState || oldItemState.dirty || itemState.parentStates.some(function (parent) { return parent.dirty; });
 
                 // return the state descriptor
                 return itemState;
@@ -271,22 +273,36 @@ define('io.ox/office/baseframework/app/basecontroller', [
              *  for details.
              *
              * @returns {jQuery.Promise}
-             *  The Promise of a Deferred object that will be resolved or
-             *  rejected according to the result of the item setter function.
-             *  If the item is disabled, a rejected Promise will be returned.
-             *  If the item setter returns a Promise, this Promise will be
-             *  returned. Otherwise, the return value of the item setter
-             *  function will be wrapped and returned in a resolved Promise.
+             *  A promise that will be resolved or rejected according to the
+             *  result of the item setter function. If the item is disabled, a
+             *  rejected promise will be returned. If the item setter returns a
+             *  promise, that promise will be returned. Otherwise, the return
+             *  value of the item setter function will be wrapped and returned
+             *  in a resolved promise.
              */
             this.execute = function (value, options) {
 
-                var // the resulting promise
+                var // the current state and value of the item
+                    state = this.getState(),
+                    // the resulting promise
                     promise = null,
                     // whether the setter runs asynchronous code
                     pending = false,
                     // focus options after executing the item
                     focusOptions = _.extend({}, options);
 
+                // final processing after the set handler has finished
+                function finalize() {
+
+                    // bug 40673: mark the item in the state cache as dirty, to force updating
+                    // the item with the effective value as returned then by its getter
+                    var oldState = notifiedStateCache[key];
+                    if (oldState) { oldState.dirty = true; }
+
+                    // update the controller items
+                    self.update();
+                }
+
                 // mix item focus options and passed focus options
                 focusOptions.preserveFocus = itemPreserveFocus || Utils.getBooleanOption(options, 'preserveFocus', false);
                 focusOptions.focusTarget = Utils.getOption(options, 'focusTarget', itemFocusTarget);
@@ -296,8 +312,8 @@ define('io.ox/office/baseframework/app/basecontroller', [
                     focusOptions.focusTarget = _.bind(focusOptions.focusTarget, this);
                 }
 
-                // do nothing if item is disabled (but accept undetermined state)
-                if (this.isEnabled()) {
+                // do nothing if item is disabled
+                if (state.enabled) {
 
                     // execute the set handler
                     runningSetters += 1;
@@ -312,10 +328,11 @@ define('io.ox/office/baseframework/app/basecontroller', [
                     });
 
                     // show busy screen if setter is running asynchronously
-                    // (but not for modal dialogs which have their own busy blocker)
+                    // (but not for modal dialogs which have their own blocker overlay)
                     pending = promise.state() === 'pending';
                     if (pending && !Dialogs.isModalDialogOpen()) {
-                        view.enterBusy();
+                        var showCancel = cancelable && _.isFunction(promise.abort);
+                        docView.enterBusy({ cancelHandler: showCancel ? function () { promise.abort(); } : null });
                     }
                 } else {
                     // item is disabled
@@ -324,10 +341,10 @@ define('io.ox/office/baseframework/app/basecontroller', [
 
                 // focus back to application
                 if (!focusOptions.preserveFocus) {
-                    promise.always(function () {
+                    this.listenTo(promise, 'always', function () {
                         if (pending) {
                             // execute in a timeout, needed for dialogs which are closed *after* resolve/reject
-                            self.executeDelayed(function () { self.grabFocus(focusOptions); });
+                            self.executeDelayed(function () { self.grabFocus(focusOptions); }, undefined, 'BaseController: grabFocus');
                         } else {
                             self.grabFocus(focusOptions);
                         }
@@ -337,13 +354,13 @@ define('io.ox/office/baseframework/app/basecontroller', [
                 // post-processing after the setter is finished
                 Utils.logSelenium('time=' + _.now(), 'key=' + key, 'state=' + promise.state());
                 if (pending) {
-                    promise.always(function () {
-                        view.leaveBusy();
-                        self.update();
+                    this.listenTo(promise, 'always', function () {
+                        docView.leaveBusy();
+                        finalize();
                         Utils.logSelenium('time=' + _.now(), 'key=' + key, 'state=' + promise.state());
                     });
                 } else {
-                    self.update();
+                    finalize();
                 }
 
                 return promise;
@@ -372,13 +389,6 @@ define('io.ox/office/baseframework/app/basecontroller', [
 
         // private methods ----------------------------------------------------
 
-        /**
-         * Clears all cached item results.
-         */
-        function clearStateCache() {
-            stateCache = {};
-        }
-
         /**
          * Handles 'keydown' and 'keypress' events and calls the setter of this
          * item, if it contains a matching keyboard shortcut definition.
@@ -402,7 +412,7 @@ define('io.ox/office/baseframework/app/basecontroller', [
                     value = null;
 
                 // check whether to actually process the key event according to the event target node
-                if (_.isString(definition.selector) && ($(event.target).closest(definition.selector, app.getWindow().nodes.outer[0]).length === 0)) {
+                if (_.isString(definition.selector) && ($(event.target).closest(definition.selector, app.getRootNode()[0]).length === 0)) {
                     return;
                 }
 
@@ -449,41 +459,48 @@ define('io.ox/office/baseframework/app/basecontroller', [
         function initializeKeyHandler() {
 
             var // the root node of the application window
-                windowNode = app.getWindow().nodes.outer;
+                rootNode = app.getRootNode();
 
-            // Bug 27528: IE does not allow to prevent default actions of some
-            // special key events (especially CTRL+P which always opens the Print
-            // dialog, regardless whether the keydown event has been canceled in
-            // the bubbling phase). The 'official' hack is to bind a keydown event
-            // handler with IE's ancient 'attachEvent()' method, and to suppress
-            // the default action by changing the key code in the event object.
-            // (add smiley-with-big-eyes here). Note that the 'addEventListener()'
-            // method does NOT work for this hack.
-            // Bug 27528: IE 11 does not support the attachEvent() method anymore
-            // (hack for bug 27528 will fail).
-            if (_.browser.IE && _.isFunction(windowNode[0].attachEvent)) {
-                windowNode[0].attachEvent('onkeydown', function (event) {
+            // Bug 27528: IE does not allow to prevent default actions of some special key events
+            // (especially CTRL+P which always opens the Print dialog, regardless whether the keydown
+            // event has been canceled in the bubbling phase). The 'official' hack is to bind a
+            // keydown event handler with IE's ancient 'attachEvent()' method, and to suppress the
+            // default action by changing the key code in the event object (add smiley-with-big-eyes
+            // here). Note that the 'addEventListener()' method does NOT work for this hack.
+            // Bug 29544: IE 11 does not support the attachEvent() method anymore (hack for bug 27528 will fail).
+            if (_.browser.IE && _.isFunction(rootNode[0].attachEvent)) {
+                rootNode[0].attachEvent('onkeydown', function (event) {
                     var result = keyHandler(_.extend(event, { target: event.srcElement }));
                     // modify the key code to prevent the browser default action
                     if (result === false) { event.keyCode = 0; }
                     return result;
                 });
             } else {
-                windowNode.on('keydown', keyHandler);
+                rootNode.on('keydown', keyHandler);
             }
 
             // register 'keypress' event listener for shortcuts
-            windowNode.on('keypress', keyHandler);
+            rootNode.on('keypress', keyHandler);
 
             // prevent that Ctrl+A shortcut selects the entire HTML document
-            windowNode.on('keydown', function (event) {
+            rootNode.on('keydown', function (event) {
                 if (!$(event.target).is('input,textarea') && KeyCodes.matchKeyCode(event, 'A', { ctrlOrMeta: true })) {
                     return false;
                 }
             });
         }
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
+
+        /**
+         * Returns the application instance that owns this document controller.
+         *
+         * @returns {BaseApplication}
+         *  The application instance that owns this document controller.
+         */
+        this.getApp = function () {
+            return app;
+        };
 
         /**
          * Adds the definition for a new item to this controller.
@@ -495,7 +512,7 @@ define('io.ox/office/baseframework/app/basecontroller', [
          *  A map with callback functions defining the behavior of the item.
          *  All callback functions will be executed in the context of the Item
          *  class instance. The following attributes are supported:
-         *  @param {String|String[]} [definition.parent]
+         *  @param {String|Array<String>} [definition.parent]
          *      The key of an item that will be used to calculate intermediate
          *      results for the getter function and enabler function (see
          *      below), or an array of item keys used as parents for this item.
@@ -527,10 +544,16 @@ define('io.ox/office/baseframework/app/basecontroller', [
          *  @param {Function} [definition.set]
          *      Setter function changing the value of an item to the first
          *      parameter of the setter. Can be omitted for read-only items.
-         *      Defaults to an empty function. May return a Deferred object or
-         *      a Promise to indicate that the setter executes asynchronous
-         *      code. In this case, the application window will be blocked and
-         *      a busy indicator will be shown while the setter is running.
+         *      Defaults to an empty function. May return a promise to indicate
+         *      that the setter executes asynchronous code. In this case, the
+         *      application window will be blocked and a busy indicator will be
+         *      shown while the setter is running.
+         *  @param {Boolean} [definition.cancel=false]
+         *      If set to true, and if the setter function of the item has
+         *      returned a pending abortable promise, the busy blocker screen
+         *      will show a Cancel button that will abort that promise. It is
+         *      up to the implementation of the setter function to react
+         *      properly on aborting the promise.
          *  @param {Object|Array} [definition.shortcut]
          *      One or multiple keyboard shortcut definitions. If the window
          *      root node of the application receives a 'keydown' or 'keypress'
@@ -628,27 +651,32 @@ define('io.ox/office/baseframework/app/basecontroller', [
          *  A reference to this controller instance.
          */
         this.registerDefinition = function (key, definition) {
-            if (_.isString(key) && key && _.isObject(definition)) {
-                items[key] = new Item(key, definition);
-                if (_.isObject(definition.shortcut)) {
-                    _.chain(definition.shortcut).getArray().each(function (shortcut) {
-                        var keyCode = Utils.getOption(shortcut, 'keyCode'),
-                            charCode = Utils.getOption(shortcut, 'charCode');
-                        if (_.isString(keyCode)) {
-                            keyCode = KeyCodes[keyCode] || 0;
-                        }
-                        if (_.isNumber(keyCode) && (keyCode > 0)) {
-                            (keyShortcuts[keyCode] || (keyShortcuts[keyCode] = [])).push({ key: key, definition: shortcut });
-                        }
-                        if (_.isString(charCode)) {
-                            charCode = charCode.charCodeAt(0);
-                        }
-                        if (_.isNumber(charCode) && (charCode > 0)) {
-                            (charShortcuts[charCode] || (charShortcuts[charCode] = [])).push({ key: key, definition: shortcut });
-                        }
-                    });
-                }
+
+            // create a new item instance
+            items[key] = new Item(key, definition);
+
+            // special preparations for global keyboard shortcuts
+            if (_.isObject(definition.shortcut)) {
+                _.getArray(definition.shortcut).forEach(function (shortcut) {
+
+                    var keyCode = Utils.getOption(shortcut, 'keyCode'),
+                        charCode = Utils.getOption(shortcut, 'charCode');
+
+                    if (_.isString(keyCode)) {
+                        keyCode = KeyCodes[keyCode] || 0;
+                    }
+                    if (_.isNumber(keyCode) && (keyCode > 0)) {
+                        (keyShortcuts[keyCode] || (keyShortcuts[keyCode] = [])).push({ key: key, definition: shortcut });
+                    }
+                    if (_.isString(charCode)) {
+                        charCode = charCode.charCodeAt(0);
+                    }
+                    if (_.isNumber(charCode) && (charCode > 0)) {
+                        (charShortcuts[charCode] || (charShortcuts[charCode] = [])).push({ key: key, definition: shortcut });
+                    }
+                });
             }
+
             return this;
         };
 
@@ -656,9 +684,10 @@ define('io.ox/office/baseframework/app/basecontroller', [
          * Adds definitions for multiple items to this controller.
          *
          * @param {Object} definitions
-         *  A map of key/definition pairs for all new items. Each item will be
-         *  defined by calling the method BaseController.registerDefinition().
-         *  See this method for more details.
+         *  A map of definitions for all new items, mapped by item key. Each
+         *  new controller item will be defined by calling the method
+         *  BaseController.registerDefinition(). See this method for more
+         *  details.
          *
          * @returns {BaseController}
          *  A reference to this controller instance.
@@ -677,34 +706,55 @@ define('io.ox/office/baseframework/app/basecontroller', [
          * @returns {BaseController}
          *  A reference to this controller.
          */
-        this.update = this.createDebouncedMethod(Utils.NOOP, function () {
+        this.update = (function () {
 
-            // collect states of all matching items in resultCache
-            clearStateCache();
-
-            if (_.browser.IE) {
-
-                // early break out, 'getState' is only important for hud,
-                // but freezing to long can kill 'realtime' and the browser itself,
-                // so we cancel it after 1sec.
-                // (more than 3sec could also mean 5sec. or more, because we have to check later)
-
-                var startTime = _.now();
-                _.any(items, function (item) {
-                    item.getState();
-                    if ((_.now() - startTime) > 3000) { return true; }
-                });
-            } else {
-                _.invoke(items, 'getState');
+            // clear the entire cache state on every call of the update() method
+            function invalidateStates() {
+                stateCache = {};
             }
 
-            // notify all listeners (with locked GUI refresh for performance,
-            // view elements are listening to controller items to change their visibility)
-            view.lockPaneLayout(function () {
-                self.trigger('change:items', stateCache);
-            });
+            // after a delay, collect all item states, and notify change listeners
+            function updateStates() {
 
-        }, { delay: updateDelay, maxDelay: updateDelay * 5 });
+                // collect all item states missing in the state cache
+
+                // IE workaround: early exit after 3 seconds (only important for view update),
+                // but freezing too long can kill RT connection and the browser itself
+                if (_.browser.IE) {
+                    var t0 = _.now(), i = 0;
+                    _.any(items, function (item) {
+                        item.getState();
+                        i = (i + 1) % 10;
+                        if ((i === 0) && (_.now() - t0 > 3000)) {
+                            Utils.error('BaseController.update(): performance bottleneck (early exit after 3s), please fix item getters!');
+                            return true;
+                        }
+                    });
+                } else {
+                    _.invoke(items, 'getState');
+                }
+
+                // reduce to the properties that differ from previous state
+                var changedStates = _.omit(stateCache, function (state, key) {
+                    var prevState = notifiedStateCache[key];
+                    return prevState && !state.dirty && (prevState.enabled === state.enabled) && _.isEqual(prevState.value, state.value);
+                });
+
+                // store the current cache for next update() call
+                // bug 40770: create flat clones of all cache entries, to be able to change stateCache in-place later
+                notifiedStateCache = _.mapObject(stateCache, function (state) { return _.clone(state); });
+
+                // notify all listeners (with locked GUI refresh for performance,
+                // view elements are listening to controller items to change their visibility)
+                if (!_.isEmpty(changedStates)) {
+                    docView.lockPaneLayout(function () {
+                        self.trigger('change:items', changedStates);
+                    });
+                }
+            }
+
+            return self.createDebouncedMethod(invalidateStates, updateStates, { delay: updateDelay, infoString: 'BaseController: update' });
+        }());
 
         /**
          * Returns the current enabled state of the specified item.
@@ -716,8 +766,6 @@ define('io.ox/office/baseframework/app/basecontroller', [
          *  Whether the specified item exists and is enabled.
          */
         this.isItemEnabled = function (key) {
-            // do not clear the cache if currently running in a set handler
-            if (runningSetters === 0) { clearStateCache(); }
             return (key in items) && items[key].isEnabled();
         };
 
@@ -732,8 +780,6 @@ define('io.ox/office/baseframework/app/basecontroller', [
          *  exist.
          */
         this.getItemValue = function (key) {
-            // do not clear the cache if currently running in a set handler
-            if (runningSetters === 0) { clearStateCache(); }
             return (key in items) ? items[key].getValue() : undefined;
         };
 
@@ -744,7 +790,7 @@ define('io.ox/office/baseframework/app/basecontroller', [
          * @param {String} key
          *  The key of the item to be changed.
          *
-         * @param {Any} value
+         * @param {Any} [value]
          *  The new value of the item.
          *
          * @param {Object} [options]
@@ -780,19 +826,17 @@ define('io.ox/office/baseframework/app/basecontroller', [
          *      Group class.
          *
          * @returns {jQuery.Promise}
-         *  The Promise of a Deferred object that will be resolved or rejected
-         *  according to the result of the item setter function. If the item is
-         *  disabled or does not exist, a rejected Promise will be returned. If
-         *  the item setter returns a Promise, this Promise will be returned.
-         *  Otherwise, the return value of the item setter function will be
-         *  wrapped and returned in a resolved Promise.
+         *  A promise that will be resolved or rejected according to the result
+         *  of the item setter function. If the item is disabled or does not
+         *  exist, a rejected promise will be returned. If the item setter
+         *  returns a promise, that promise will be returned. Otherwise, the
+         *  return value of the item setter function will be wrapped and
+         *  returned in a resolved promise.
          */
         this.executeItem = function (key, value, options) {
 
             // execute setter of existing item
             if (key in items) {
-                // do not clear cache if called recursively from another setter
-                if (runningSetters === 0) { clearStateCache(); }
                 return items[key].execute(value, options);
             }
 
@@ -836,12 +880,13 @@ define('io.ox/office/baseframework/app/basecontroller', [
 
             // move focus to specified target, or to the view
             if (_.isObject(focusTarget) && _.isFunction(focusTarget.grabFocus) && _.isFunction(focusTarget.isVisible) && focusTarget.isVisible()) {
-                focusTarget.grabFocus();
+                focusTarget.grabFocus(options);
             } else if (((focusTarget instanceof $) || (focusTarget instanceof HTMLElement)) && $(focusTarget).is(':visible')) {
                 $(focusTarget).focus();
             } else {
-                view.grabFocus();
+                docView.grabFocus(options);
             }
+
             return this;
         };
 
@@ -850,18 +895,55 @@ define('io.ox/office/baseframework/app/basecontroller', [
         // dummy item singleton, used e.g. for undefined parent items
         dummyItem = new Item('\x00');
 
-        // register item definitions
-        this.registerDefinitions(BASE_ITEM_DEFINITIONS);
+        // register all controller items
+        this.registerDefinitions({
 
-        // initialize class members when the application is alive
-        app.onInit(function () {
-            view = app.getView();
+            // quit/destroy the application (do not return the promise,
+            // otherwise this controller would wait for its own destruction)
+            'app/quit': {
+                set: function () { _.defer(function () { app.quit(); }); }
+            },
+
+            // disabled during import, enabled afterwards
+            'app/imported': {
+                enable: function () { return app.isImportFinished(); }
+            },
+
+            // disabled in stand-alone mode
+            'app/bundled': {
+                enable: function () { return !app.isStandalone(); }
+            },
+
+            // download the document: enabled in read-only mode, disabled during import
+            'document/download': {
+                parent: ['app/imported', 'system/filesystem'],
+                set: function () { return app.download(); }
+            },
+
+            // print the document (download as PDF): enabled in read-only mode, disabled during import
+            'document/print': {
+                parent: 'app/imported',
+                set: function () { return app.print(); },
+                shortcut: { keyCode: 'P', ctrlOrMeta: true }
+            },
+
+            // send document as mail: enabled in read-only mode, disabled during import, disabled in stand-alone-mode
+            'document/sendmail': {
+                parent: ['app/imported', 'app/bundled'],
+                enable: function () { return !DriveUtils.isGuest(); },
+                set: function () { return app.sendMail(); }
+            },
+
+            // all supported operation systems have a file system where user can save in, but not on IOS
+            'system/filesystem': {
+                enable: function () { return !Utils.IOS; }
+            }
         });
 
         // update once after import (successful and failed)
         app.onImport(function () {
             initializeKeyHandler();
-            self.listenTo(view, 'refresh:layout', function () { self.update(); });
+            self.listenTo(docView, 'refresh:layout', function () { self.update(); });
             self.update();
         });
 
@@ -869,8 +951,8 @@ define('io.ox/office/baseframework/app/basecontroller', [
         this.registerDestructor(function () {
             _.invoke(items, 'destroy');
             dummyItem.destroy();
-            self = items = dummyItem = view = stateCache = keyShortcuts = charShortcuts = null;
-            BASE_ITEM_DEFINITIONS = null;
+            self = docModel = docView = null;
+            items = dummyItem = stateCache = keyShortcuts = charShortcuts = null;
         });
 
     } // class BaseController
