@@ -5,45 +5,45 @@
  * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/baseframework/app/extensionregistry',
-    ['io.ox/core/capabilities',
-     'io.ox/office/tk/utils',
-     'io.ox/office/tk/config'
-    ], function (Capabilities, Utils, Config) {
+define('io.ox/office/baseframework/app/extensionregistry', [
+    'io.ox/core/capabilities',
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/config'
+], function (Capabilities, Utils, Config) {
 
     'use strict';
 
-    var // which file formats are editable (maps format identifier to Booleans)
+    var // which file formats are editable (maps format identifier to booleans)
         EDITABLE_FILE_FORMATS = {
             ooxml: true,
-            odf: Utils.getBooleanOption(Config.get(), 'odfsupport', true)
+            odf: Config.getFlag('odfsupport', true)
         },
 
         // the extension suffix for files with an internal error
         INTERNAL_ERROR_SUFFIX = '_ox',
 
-        // file extension configuration (all extensions are viewable by OX Preview)
+        // file extension configuration
         FILE_EXTENSION_CONFIGURATION = {
 
             text: {
                 module: 'io.ox/office/text',
                 requires: 'text',
-                editable: !_.browser.Android || Utils.supportedChromeVersionOnAndroidForText(),
+                editable: !_.browser.Android || Utils.CHROME_ON_ANDROID,
                 extensions: {
                     docx: { format: 'ooxml' },
                     docm: { format: 'ooxml', macros: true },
                     dotx: { format: 'ooxml', template: true },
                     dotm: { format: 'ooxml', template: true, macros: true },
-                    odt:  { format: 'odf',   macros: true },
-                    ott:  { format: 'odf',   template: true, macros: true },
-                    doc:  { format: 'ooxml',   macros: true, convert: true },
-                    dot:  { format: 'ooxml',   template: true, templateLocked: true, macros: true, convert: true },
-                    rtf:  { format: 'ooxml',   convert: true }
+                    odt:  { format: 'odf', macros: true },
+                    ott:  { format: 'odf', template: true, macros: true },
+                    doc:  { format: 'ooxml', macros: true, convert: true },
+                    dot:  { format: 'ooxml', template: true, templateLocked: true, macros: true, convert: true },
+                    rtf:  { format: 'ooxml', convert: true }
                 }
             },
 
@@ -56,37 +56,34 @@ define('io.ox/office/baseframework/app/extensionregistry',
                     xlsm: { format: 'ooxml', macros: true },
                     xltx: { format: 'ooxml', template: true },
                     xltm: { format: 'ooxml', template: true, macros: true },
-                    xlam: {                  macros: true }, // add-in
+                    xlam: { macros: true }, // add-in
                     xls:  { format: 'ooxml', macros: true, convert: true },
                     xlt:  { format: 'ooxml', template: true, templateLocked: true, macros: true, convert: true },
-                    xla:  {                  macros: true }, // add-in
+                    xla:  { macros: true }, // add-in
                     xlsb: { format: 'ooxml', macros: true, convert: true },
-                    csv:  {}, //empty only for viewing
-
-                    //disable ods for release!
-                    ods:  { format: 'odf', macros: true, convert: !Config.DEBUG }, //spreadsheet
-                    ots:  { format: 'odf', template: true, macros: true, convert: !Config.DEBUG }, //spreadsheet
+                    ods:  { format: 'odf', macros: true }, //spreadsheet
+                    ots:  { format: 'odf', template: true, macros: true } //spreadsheet
                 }
             },
 
             presentation: {
                 module: 'io.ox/office/presentation',
                 requires: 'presentation',
-                editable: true,
+                editable: false,
                 extensions: {
                     pptx: { format: 'ooxml' },
                     pptm: { format: 'ooxml', macros: true },
                     potx: { format: 'ooxml', template: true },
                     potm: { format: 'ooxml', template: true, macros: true },
                     ppsx: {}, // slide show
-                    ppsm: {                  macros: true }, // slide show
-                    ppam: {                  macros: true }, // add-in
-                    odp:  { format: 'odf',   macros: true },
-                    otp:  { format: 'odf',   template: true, macros: true },
-                    ppt:  { format: 'odf',   macros: true, convert: true },
-                    pot:  { format: 'odf',   template: true, templateLocked: true, macros: true, convert: true },
-                    pps:  {                  macros: true }, // slide show
-                    ppa:  {                  macros: true } // add-in
+                    ppsm: { macros: true }, // slide show
+                    ppam: { macros: true }, // add-in
+                    odp:  { format: 'odf', macros: true },
+                    otp:  { format: 'odf', template: true, macros: true },
+                    ppt:  { format: 'odf', macros: true, convert: true },
+                    pot:  { format: 'odf', template: true, templateLocked: true, macros: true, convert: true },
+                    pps:  { macros: true }, // slide show
+                    ppa:  { macros: true } // add-in
                 }
             },
 
@@ -139,7 +136,7 @@ define('io.ox/office/baseframework/app/extensionregistry',
      */
     var ExtensionRegistry = {};
 
-    // methods ----------------------------------------------------------------
+    // static methods ---------------------------------------------------------
 
     /**
      * Returns whether the passed module name specifies an OX Documents
@@ -272,12 +269,16 @@ define('io.ox/office/baseframework/app/extensionregistry',
      *
      * @returns {String}
      *  The module name of the edit application that is able to modify the file
-     *  with the passed file name; or an empty string, if the file is not
+     *  with the passed file name; or null, if the file is not
      *  editable.
      */
     ExtensionRegistry.getEditModule = function (fileName) {
         var extensionSettings = ExtensionRegistry.getExtensionSettings(fileName);
-        return Utils.getStringOption(extensionSettings, 'module', '');
+        if (extensionSettings && extensionSettings.editable) {
+            return Utils.getStringOption(extensionSettings, 'module', '');
+        } else {
+            return null;
+        }
     };
 
     /**
