@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -31,13 +31,37 @@ define('io.ox/office/baseframework/model/basemodel', [
      * @param {BaseApplication} app
      *  The application that has created this model instance.
      */
-    function BaseModel(/*app*/) {
+    function BaseModel(app) {
 
         // base constructors --------------------------------------------------
 
         TriggerObject.call(this);
         TimerMixin.call(this);
 
+        // public methods -----------------------------------------------------
+
+        /**
+         * Returns the application instance that owns this document model.
+         *
+         * @returns {BaseApplication}
+         *  The application instance that owns this document model.
+         */
+        this.getApp = function () {
+            return app;
+        };
+
+        /**
+         * Returns a reference to this document model. Provided to allow to
+         * implement functions that can take any model object (instances of the
+         * class ModelObject), including this document model itself.
+         *
+         * @returns {BaseModel}
+         *  A reference to this instance.
+         */
+        this.getDocModel = function () {
+            return this;
+        };
+
     } // class BaseModel
 
     // exports ================================================================
