@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -29,8 +29,8 @@ define('io.ox/office/baseframework/model/modelobject', [
      *
      * @extends TriggerObject
      *
-     * @param {EditApplication} app
-     *  The application instance containing this attributed model object.
+     * @param {BaseModel} docModel
+     *  The document model containing this model object.
      *
      * @param {Object} [initOptions]
      *  Optional parameters:
@@ -50,34 +50,57 @@ define('io.ox/office/baseframework/model/modelobject', [
      *          permanently silent although the classes derive indirectly from
      *          this class.
      */
-    function ModelObject(app, initOptions) {
-
-        var // the trigger mode
-            triggerMode = Utils.getStringOption(initOptions, 'trigger', 'runtime');
+    function ModelObject(docModel, initOptions) {
 
         // base constructor ---------------------------------------------------
 
         TriggerObject.call(this);
 
+        // public methods -----------------------------------------------------
+
+        /**
+         * Returns the application that owns this instance.
+         *
+         * @returns {BaseApplication}
+         *  The application that owns this instance.
+         */
+        this.getApp = function () {
+            return docModel.getApp();
+        };
+
+        /**
+         * Returns the document model that owns this instance.
+         *
+         * @returns {BaseModel}
+         *  The document model that owns this instance.
+         */
+        this.getDocModel = function () {
+            return docModel;
+        };
+
         // initialization -----------------------------------------------------
 
-        // set dummy event methods for silent objects
-        if (triggerMode === 'never') {
-            this.on = this.one = this.off = this.trigger = Utils.NOOP;
-        }
+        (function (triggerMode) {
 
-        // disable triggering until import is finished
-        else if ((triggerMode !== 'always') && !app.isImportFinished()) {
+            // set dummy event methods for silent objects
+            if (triggerMode === 'never') {
+                this.on = this.one = this.off = this.trigger = Utils.NOOP;
+                return;
+            }
 
-            // overwrite the trigger() method with a method that triggers
-            // events only if the document has been imported already
-            this.trigger = (function (trigger) {
-                this.listenTo(app.getImportPromise(), 'always', _.bind(function () {
-                    this.trigger = trigger; // back to original method
-                }, this));
-                return Utils.NOOP;
-            }.call(this, this.trigger));
-        }
+            // disable triggering until import is finished
+            if ((triggerMode !== 'always') && !docModel.getApp().isImportFinished()) {
+
+                // overwrite the trigger() method with a dummy, restore original method after import
+                this.trigger = (function (trigger) {
+                    this.listenTo(docModel.getApp().getImportPromise(), 'always', _.bind(function () {
+                        this.trigger = trigger; // back to original method
+                    }, this));
+                    return Utils.NOOP;
+                }.call(this, this.trigger));
+            }
+
+        }.call(this, Utils.getStringOption(initOptions, 'trigger', 'runtime')));
 
     } // class ModelObject
 
