@@ -6,14 +6,15 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/baseframework/view/baselabels', [
-    'gettext!io.ox/office/main'
-], function (gt) {
+    'io.ox/office/settings/view/settingslabels',
+    'gettext!io.ox/office/baseframework/main'
+], function (SettingsLabels, gt) {
 
     'use strict';
 
@@ -22,8 +23,10 @@ define('io.ox/office/baseframework/view/baselabels', [
     /**
      * Provides generic caption texts for control groups, menu headers, and
      * other GUI elements in a single map for convenience.
+     *
+     * @extends SettingsLabels
      */
-    var BaseLabels = {};
+    var BaseLabels = _.extend({}, SettingsLabels);
 
     // header labels ----------------------------------------------------------
 
@@ -76,7 +79,7 @@ define('io.ox/office/baseframework/view/baselabels', [
      *
      * @constant
      */
-    BaseLabels.DOWNLOAD_ICON = 'fa-cloud-download';
+    BaseLabels.DOWNLOAD_ICON = 'fa-download';
 
     /**
      * Standard label for a 'Print document' button.
