@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,8 +15,9 @@ define('io.ox/office/baseframework/view/component', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/forms',
     'io.ox/office/tk/object/timermixin',
-    'io.ox/office/baseframework/model/modelobject'
-], function (Utils, Forms, TimerMixin, ModelObject) {
+    'io.ox/office/baseframework/model/modelobject',
+    'io.ox/office/baseframework/view/viewobjectmixin'
+], function (Utils, Forms, TimerMixin, ModelObject, ViewObjectMixin) {
 
     'use strict';
 
@@ -47,14 +48,18 @@ define('io.ox/office/baseframework/view/component', [
      * - 'component:blur'
      *      After a control group in this component has lost the browser focus,
      *      after focusing any other DOM node outside the group.
+     * - 'component:change'
+     *      After a control group in this component has been activated in the
+     *      GUI (i.e. has triggered a 'group:change' event).
      *
      * @constructor
      *
      * @extends ModelObject
      * @extends TimerMixin
+     * @extends ViewObjectMixin
      *
-     * @param {BaseApplication} app
-     *  The application containing this view component instance.
+     * @param {BaseView} docView
+     *  The document view instance containing this view component.
      *
      * @param {Object} [initOptions]
      *  Optional parameters:
@@ -66,35 +71,39 @@ define('io.ox/office/baseframework/view/component', [
      *      the chain of focusable landmark nodes reachable with specific
      *      global keyboard shortcuts (usually the F6 key with platform
      *      dependent modifier keys).
-     *  @param {Boolean} [initOptions.hoverEffect=false]
-     *      If set to true, all control groups in this view component will be
-     *      displayed half-transparent as long as the mouse does not hover the
-     *      view component. Has no effect, if the current device is a touch
-     *      device.
+     *  @param {Boolean} [initOptions.autoHideGroups=false]
+     *      If set to true, all disabled control groups contained in this view
+     *      component will be hidden automatically.
      */
-    function Component(app, initOptions) {
+    function Component(docView, initOptions) {
 
         var // self reference
             self = this,
 
+            // the application instance
+            app = docView.getApp(),
+
             // the container element representing the pane
-            rootNode = Utils.createContainerNode('view-component', initOptions),
+            rootNode = $('<div class="view-component">').addClass(Utils.getStringOption(initOptions, 'classes', '')),
 
             // the last cached size of the root node, used to detect layout changes
             nodeSize = null,
 
-            // all control groups, as plain array
-            groups = [],
+            // all control groups, as descriptor objects with key and other data, mapped by group UID
+            groupRegistry = {},
 
-            // all control groups, as arrays mapped by key
+            // all control groups, as sub-maps mapped by controller key, then by group UID
             groupsByKey = {},
 
-            // all control groups with bound visibility, as arrays mapped by key
-            visibleGroupsByKey = {},
+            // all control groups with bound visibility, as sub-maps mapped by visibility key, then by group UID
+            groupsByVisibleKey = {},
 
             // handler called to insert a new group into this view component
             groupInserters = [],
 
+            // whether to hide all disabled control groups automatically
+            autoHideGroups = Utils.getBooleanOption(initOptions, 'autoHideGroups', false),
+
             // whether the view component is set to visible (regardless of effective visibility)
             visible = true,
 
@@ -106,33 +115,26 @@ define('io.ox/office/baseframework/view/component', [
 
         // base constructors --------------------------------------------------
 
-        ModelObject.call(this, app);
+        ModelObject.call(this, docView.getDocModel());
         TimerMixin.call(this);
+        ViewObjectMixin.call(this, docView);
 
         // private methods ----------------------------------------------------
 
-        /**
-         * Removes the passed object from the array in-place.
-         */
-        function removeFromArray(array, object) {
-            var index = _.indexOf(array, object);
-            if (index >= 0) { array.splice(index, 1); }
-        }
-
         /**
          * Inserts the passed group into the map under a specific group key.
          */
-        function addGroupToMap(map, key, group) {
-            (map[key] || (map[key] = [])).push(group);
+        function addGroupToKeyMap(keyMap, key, group) {
+            var subMap = keyMap[key] || (keyMap[key] = {});
+            subMap[group.getUid()] = group;
         }
 
         /**
-         * Removes the passed group from the map.
+         * Removes the passed group from the key map.
          */
-        function removeGroupFromMap(map, group) {
-            _.each(map, function (array) {
-                removeFromArray(array, group);
-            });
+        function removeGroupFromKeyMap(keyMap, group) {
+            var groupUid = group.getUid();
+            _.each(keyMap, function (subMap) { delete subMap[groupUid]; });
         }
 
         /**
@@ -144,8 +146,8 @@ define('io.ox/office/baseframework/view/component', [
          */
         function updateNodeVisibility() {
 
-            var // whether the component is visible and contains at least one visible group
-                makeVisible = visible && _.any(groups, function (group) { return group.isVisible(); });
+            // whether the component is visible and contains at least one visible group
+            var makeVisible = visible && _.some(groupRegistry, function (data) { return data.group.isVisible(); });
 
             // update node visibility and notify listeners, if really changed
             if (self.isVisible() !== makeVisible) {
@@ -183,10 +185,10 @@ define('io.ox/office/baseframework/view/component', [
                 triggerLayout = updateLayout || Utils.getBooleanOption(options, 'triggerLayout', false);
 
             // update layout of all control groups if specified
-            if (updateLayout) { _.invoke(groups, 'layout'); }
+            if (updateLayout) { _.each(groupRegistry, function (data) { data.group.layout(); }); }
 
-            var // current size of the DOM root node
-                newSize = { width: rootNode.outerWidth(), height: rootNode.outerHeight() };
+            // current size of the DOM root node
+            var newSize = { width: rootNode.outerWidth(), height: rootNode.outerHeight() };
 
             // notify listeners if size has changed
             if (!_.isEqual(nodeSize, newSize)) {
@@ -254,20 +256,84 @@ define('io.ox/office/baseframework/view/component', [
         }
 
         /**
-         * Handler for application controller 'change:items' events.
+         * Updates the enabled state of the specified group.
+         *
+         * @param {Group} group
+         *  The control group to be updated.
+         *
+         * @param {Boolean} enabled
+         *  The initial enabled state for the control group. If this value is
+         *  true, the 'enableHandler' callback handler passed to the method
+         *  Component.addGroup() will be invoked to receive the resulting
+         *  enabled state for the group.
          */
-        function controllerChangeHandler(event, changedItems) {
+        function updateGroupEnabledState(group, enabled) {
+
+            // if the group is initially enabled, ask its registered callback handler
+            if (enabled) {
+                var groupData = groupRegistry[group.getUid()];
+                if (groupData.enableHandler) {
+                    enabled = groupData.enableHandler.call(self, group) === true;
+                }
+            }
+
+            // enable/disable the group
+            group.enable(enabled);
+        }
+
+        /**
+         * Updates the visibility, enabled state, and value of all groups that
+         * are bound to the specified controller item.
+         *
+         * @param {String} key
+         *  The identifier of a controller item.
+         *
+         * @param {Boolean} enabled
+         *  Whether the controller item is enabled.
+         *
+         * @param {Any} value
+         *  The current value of the controller item.
+         */
+        function updateGroupState(key, enabled, value) {
+
+            // the application controller
+            var controller = app.getController();
+
+            // show/hide the passed control group
+            function updateGroupVisibility(group) {
+                var visibleKey = groupRegistry[group.getUid()].visibleKey;
+                if (autoHideGroups || visibleKey) {
+                    group.toggle((!autoHideGroups || enabled) && (!visibleKey || controller.isItemEnabled(visibleKey)));
+                }
+            }
+
+            // update all control groups with the passed key (may be more than one)
+            _.each(groupsByKey[key], function (group) {
+
+                // update enabled state and value of the controls
+                updateGroupEnabledState(group, enabled);
+                group.setValue(value);
+
+                // show/hide all groups with bound visibility
+                updateGroupVisibility(group);
+            });
+
+            // show/hide all controls whose visibility is bound to the passed key (regardless of their own key)
+            _.each(groupsByVisibleKey[key], updateGroupVisibility);
+        }
+
+        /**
+         * Debounced version of the method updateAllGroups().
+         */
+        var updateAllGroupsDebounced = this.createDebouncedMethod(_.noop, function () { self.updateAllGroups(); }, { delay: 50 });
+
+        /**
+         * Handler for changed controller items. Updates the state and value of
+         * all control groups registered for this view component.
+         */
+        function controllerChangeHandler(changedItems) {
             _.each(changedItems, function (itemState, key) {
-                if (key in visibleGroupsByKey) {
-                    // show/hide all controls bound to the item
-                    _.invoke(visibleGroupsByKey[key], 'toggle', itemState.enabled);
-                }
-                if (key in groupsByKey) {
-                    // enable/disable all controls bound to the item
-                    _.invoke(groupsByKey[key], 'enable', itemState.enabled);
-                    // change value of all controls bound to the item
-                    _.invoke(groupsByKey[key], 'setValue', itemState.value);
-                }
+                updateGroupState(key, itemState.enabled, itemState.value);
             });
         }
 
@@ -320,7 +386,8 @@ define('io.ox/office/baseframework/view/component', [
          *  A reference to this instance.
          */
         this.unregisterGroupInserter = function (inserter) {
-            removeFromArray(groupInserters, inserter);
+            var index = groupInserters.indexOf(inserter);
+            if (index >= 0) { groupInserters.splice(index, 1); }
             return this;
         };
 
@@ -431,6 +498,13 @@ define('io.ox/office/baseframework/view/component', [
          *      to the enabled state of the controller item with the key passed
          *      in this option. If the item gets disabled, the group will be
          *      hidden automatically, and vice versa.
+         *  @param {Function} [options.enableHandler]
+         *      A predicate callback function that can be used to influence the
+         *      enabled state of the control group. MUST return a boolean value
+         *      stating whether the control group is enabled. If the group is
+         *      bound to a controller item (parameter 'key' is a string), the
+         *      state of the controller item can only be changed from enabled
+         *      to disabled by this callback function.
          *  @param {HTMLElement|jQuery|Object|Function} [options.focusTarget]
          *      A DOM node that will receive the browser focus after the
          *      inserted group has been activated, an object that provides a
@@ -442,27 +516,32 @@ define('io.ox/office/baseframework/view/component', [
          */
         this.addGroup = function (key, group, options) {
 
-            var // the key of a controller item bound to the visibility of the group
-                visibleKey = Utils.getStringOption(options, 'visibleKey'),
-                // target focus node after activating the group
-                focusTarget = Utils.getOption(options, 'focusTarget');
+            var // the unique identifier of the group
+                groupUid = group.getUid(),
+                // the registry entry for the group
+                groupData = {
+                    group: group,
+                    key: key,
+                    visibleKey: Utils.getStringOption(options, 'visibleKey', null),
+                    enableHandler: Utils.getFunctionOption(options, 'enableHandler', null),
+                    focusTarget: Utils.getOption(options, 'focusTarget')
+                };
 
             // builds a focus options object to be passed to the controller
             function getFocusOptions(event) {
                 var options = { changeEvent: event, preserveFocus: event.preserveFocus === true };
                 if ('sourceType' in event) { options.sourceType = event.sourceType; }
-                if (!_.isUndefined(focusTarget)) { options.focusTarget = focusTarget; }
+                if (!_.isUndefined(groupData.focusTarget)) { options.focusTarget = groupData.focusTarget; }
                 return options;
             }
 
             // returns browser focus to the application
             function groupCancelHandler(event) {
-                app.getController().grabFocus(getFocusOptions(event));
-                app.getController().update();
+                app.getController().grabFocus(getFocusOptions(event)).update();
             }
 
             // remember the group object
-            groups.push(group);
+            groupRegistry[groupUid] = groupData;
 
             // insert the group into this view component
             if (_.all(groupInserters, function (inserter) { return inserter.call(self, group, options) === false; })) {
@@ -474,33 +553,45 @@ define('io.ox/office/baseframework/view/component', [
             this.listenTo(group, 'group:show group:hide group:resize', groupResizeHandler);
             this.listenTo(group, 'group:focus group:blur', groupFocusHandler);
 
-            // bind group to controller item
-            if (_.isString(key)) {
+            // special handling if the group has been activated in the GUI
+            this.listenTo(group, 'group:change', function (event, value) {
 
-                // store group reference in internal map
-                addGroupToMap(groupsByKey, key, group);
-
-                // execute controller item for 'group:change' events
-                this.listenTo(group, 'group:change', function (event, value) {
-                	var executeOptions = {
-                        changeEvent: event
-                    };
+                if (key) {
+                    // execute controller item for 'group:change' events
+                    var executeOptions = { changeEvent: event };
                     if (event.sourceType) { executeOptions.sourceType = event.sourceType; }
                     if (event.preserveFocus) { executeOptions.preserveFocus = event.preserveFocus; }
-                    if (focusTarget) { executeOptions.focusTarget = focusTarget; }
+                    if (groupData.focusTarget) { executeOptions.focusTarget = groupData.focusTarget; }
+                    docView.executeControllerItem(key, value, executeOptions);
+                } else {
+                    // update all groups after an unbound group has been activated (custom enabled states etc.)
+                    self.updateAllGroups();
+                }
 
-                    app.getController().executeItem(key, value, executeOptions);
-                });
+                // notify listeners
+                self.trigger('component:change', key, value);
+            });
+
+            // special handling for groups bound to controller items
+            if (key) {
+
+                // store group reference in internal map
+                addGroupToKeyMap(groupsByKey, key, group);
 
                 // set the key as DOM data attribute at the root node of the group
                 group.getNode().attr('data-key', key);
+
+                // bug 37236: update enabled state of all new groups, e.g. when initializing
+                // a view component lazily (e.g. directly before showing a pop-up menu)
+                app.onImportSuccess(updateAllGroupsDebounced);
             }
 
             // bind group visibility to controller item
-            if (_.isString(visibleKey)) {
-
-                // store group reference in internal map
-                addGroupToMap(visibleGroupsByKey, visibleKey, group);
+            if (groupData.visibleKey) {
+                addGroupToKeyMap(groupsByVisibleKey, groupData.visibleKey, group);
+                // bug 39986: hide the group initially, better than showing these groups during
+                // application initialization and hiding them afterwards on first controller update
+                group.hide();
             }
 
             // update internal layout (component visibility and size)
@@ -522,19 +613,23 @@ define('io.ox/office/baseframework/view/component', [
          */
         this.removeGroup = function (group) {
 
+            // the unique identifier of the group
+            var groupUid = group.getUid();
+
             // unregister all event handlers for the group
             this.stopListeningTo(group);
 
             // remove group from internal containers
-            removeFromArray(groups, group);
-            removeGroupFromMap(groupsByKey, group);
-            removeGroupFromMap(visibleGroupsByKey, group);
+            delete groupRegistry[groupUid];
+            removeGroupFromKeyMap(groupsByKey, group);
+            removeGroupFromKeyMap(groupsByVisibleKey, group);
 
             // remove group root node from DOM
             group.getNode().detach();
 
             // update internal layout (component visibility and size)
             updateNode();
+
             return this;
         };
 
@@ -543,9 +638,8 @@ define('io.ox/office/baseframework/view/component', [
          *
          * @param {Function} iterator
          *  The iterator called for each control group. Receives the reference
-         *  to the control group, and its insertion index. If the iterator
-         *  returns the Utils.BREAK object, the iteration process will be
-         *  stopped immediately.
+         *  to the control group. If the iterator returns the Utils.BREAK
+         *  object, the iteration process will be stopped immediately.
          *
          * @param {Object} [context]
          *  If specified, the iterator will be called with this context (the
@@ -557,8 +651,8 @@ define('io.ox/office/baseframework/view/component', [
          *  Utils.BREAK to stop the iteration process, otherwise undefined.
          */
         this.iterateGroups = function (iterator, context) {
-            return _.any(groups, function (group, index) {
-                return iterator.call(context, group, index) === Utils.BREAK;
+            return _.some(groupRegistry, function (data) {
+                return iterator.call(context, data.group) === Utils.BREAK;
             }) ? Utils.BREAK : undefined;
         };
 
@@ -569,7 +663,69 @@ define('io.ox/office/baseframework/view/component', [
          *  Whether this view component contains visible groups.
          */
         this.hasVisibleGroups = function () {
-            return _.any(groups, function (group) { return group.isVisible(); });
+            return _.some(groupRegistry, function (data) { return data.group.isVisible(); });
+        };
+
+        /**
+         * Updates the visibility, enabled state, and value of all groups in
+         * this view component.
+         *
+         * @returns {Component}
+         *  A reference to this instance.
+         */
+        this.updateAllGroups = function () {
+
+            // the application controller
+            var controller = app.getController();
+
+            // use the 'groupsByKey' map for iteration, to resolve each controller item
+            // once, even if multiple groups have been registered for that item
+            _.each(groupsByKey, function (map, key) {
+                updateGroupState(key, controller.isItemEnabled(key), controller.getItemValue(key));
+            });
+
+            // update enabled state of all groups not bound to a controller item
+            _.each(groupRegistry, function (data) {
+                if (!data.key) { updateGroupEnabledState(data.group, true); }
+            });
+
+            // update all groups not bound to a controller item, but with a 'visibleKey'
+            _.each(groupsByVisibleKey, function (map, key) {
+                var visible = controller.isItemEnabled(key);
+                _.each(map, function (group, groupUid) {
+                    if (!groupRegistry[groupUid].key) {
+                        group.toggle(visible);
+                    }
+                });
+            });
+
+            return this;
+        };
+
+        /**
+         * Removes and destroys all control groups from this view component,
+         * and completely clears its root node.
+         *
+         * @returns {Component}
+         *  A reference to this instance.
+         */
+        this.destroyAllGroups = function () {
+
+            // destroy all group instances (remove themselves from the DOM)
+            _.each(groupRegistry, function (data) { data.group.destroy(); });
+
+            // clean the group containers (arrays/maps)
+            groupRegistry = {};
+            groupsByKey = {};
+            groupsByVisibleKey = {};
+
+            // remove all remaining DOM nodes (separators etc.)
+            this.getNode().empty();
+
+            // update internal layout (component visibility and size)
+            updateNode();
+
+            return this;
         };
 
         /**
@@ -584,7 +740,7 @@ define('io.ox/office/baseframework/view/component', [
          *  browser focus.
          */
         this.hasFocus = function () {
-            return _.any(groups, function (group) { return group.hasFocus(); });
+            return _.some(groupRegistry, function (data) { return data.group.hasFocus(); });
         };
 
         /**
@@ -603,22 +759,17 @@ define('io.ox/office/baseframework/view/component', [
         // initialization -----------------------------------------------------
 
         // update all registered groups after change events of the application controller
-        app.onInit(function () {
-            self.listenTo(app.getController(), 'change:items', controllerChangeHandler);
-        });
+        this.handleChangedControllerItems(controllerChangeHandler);
 
         // initially hidden until the first visible group will be inserted
         Forms.showNodes(rootNode, false);
 
         // marker for touch devices
-        rootNode.toggleClass('touch', Modernizr.touch);
+        rootNode.toggleClass('touch', Utils.TOUCHDEVICE);
 
         // whether the component will be a land mark for F6 focus traveling
         rootNode.toggleClass('f6-target', Utils.getBooleanOption(initOptions, 'landmark', false));
 
-        // hover effect for groups embedded in the view component
-        rootNode.toggleClass('hover-effect', Utils.getBooleanOption(initOptions, 'hoverEffect', false));
-
         // log focus and layout events
         Utils.FocusLogger.logEvent(this, 'component:focus component:blur');
         Utils.LayoutLogger.logEvent(this, 'component:beforeshow component:show component:beforehide component:hide component:layout');
@@ -626,10 +777,10 @@ define('io.ox/office/baseframework/view/component', [
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            _.invoke(groups, 'destroy');
+            _.each(groupRegistry, function (data) { data.group.destroy(); });
             rootNode.remove();
-            app = initOptions = self = rootNode = null;
-            groups = groupsByKey = visibleGroupsByKey = groupInserters = null;
+            self = app = docView = initOptions = rootNode = null;
+            groupRegistry = groupsByKey = groupsByVisibleKey = groupInserters = null;
         });
 
     } // class Component
