@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -37,39 +37,36 @@ define('io.ox/office/baseframework/view/control/compoundbutton', [
      * @extends WidthMixin
      * @extends CompoundMenuMixin
      *
-     * @param {BaseApplication} app
-     *  The application containing this control.
+     * @param {BaseView} docView
+     *  The document view instance containing this control.
      *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options also supported by the base
      *  class Group, the mix-in class WidthMixin, and the mix-in class
      *  CompoundMenuMixin.
      */
-    function CompoundButton(app, initOptions) {
+    function CompoundButton(docView, initOptions) {
 
         var // the drop-down button
-            menuButton = $(Forms.createButtonMarkup(Utils.extendOptions({
-                tooltip: Utils.getStringOption(initOptions, 'label', null, true),
-                attributes: { 'aria-haspopup': true }
-            }, initOptions)));
-
-        Utils.addToPreventFocusChange(menuButton);
+            menuButton = $(Forms.createButtonMarkup(Utils.extendOptions({ attributes: { 'aria-haspopup': true } }, initOptions)));
 
         // base constructors --------------------------------------------------
 
         Group.call(this, initOptions);
         CaptionMixin.call(this, menuButton, initOptions);
         WidthMixin.call(this, menuButton, initOptions);
-        CompoundMenuMixin.call(this, app, Utils.extendOptions(initOptions, { button: menuButton }));
+        CompoundMenuMixin.call(this, docView, Utils.extendOptions(initOptions, { button: menuButton }));
 
         // initialization -----------------------------------------------------
 
+        Forms.setToolTip(menuButton, initOptions);
+
         // add the drop-down button to the group
         this.addChildNodes(menuButton);
 
         // destroy all class members
         this.registerDestructor(function () {
-            app = initOptions = menuButton = null;
+            docView = initOptions = menuButton = null;
         });
 
     } // class CompoundButton
