@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -32,31 +32,33 @@ define('io.ox/office/baseframework/view/control/compoundcheckbox', [
      * @extends CheckBox
      * @extends CompoundMenuMixin
      *
-     * @param {BaseApplication} app
-     *  The application containing this control.
+     * @param {BaseView} docView
+     *  The document view instance containing this control.
      *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options also supported by the base
      *  class CheckBox, and the mix-in class CompoundMenuMixin.
      */
-    function CompoundCheckBox(app, initOptions) {
+    function CompoundCheckBox(docView, initOptions) {
 
         var // the drop-down button
-            menuButton = $(Forms.createButtonMarkup({ tooltip: Utils.getOption(initOptions, 'tooltip', '') }));
+            menuButton = $(Forms.createButtonMarkup());
 
         // base constructors --------------------------------------------------
 
         CheckBox.call(this, initOptions);
-        CompoundMenuMixin.call(this, app, Utils.extendOptions(initOptions, { button: menuButton }));
+        CompoundMenuMixin.call(this, docView, Utils.extendOptions(initOptions, { button: menuButton }));
 
         // initialization -----------------------------------------------------
 
+        Forms.setToolTip(menuButton, { tooltip: Utils.getOption(initOptions, 'tooltip', '') });
+
         // add the drop-down button to the group
         this.addChildNodes(menuButton);
 
         // destroy all class members
         this.registerDestructor(function () {
-            app = initOptions = menuButton = null;
+            docView = initOptions = menuButton = null;
         });
 
     } // class CompoundCheckBox
