@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -35,18 +35,18 @@ define('io.ox/office/baseframework/view/control/compoundmenumixin', [
      *
      * @extends MenuMixin
      *
-     * @param {BaseApplication} app
-     *  The application containing this control.
+     * @param {BaseView} docView
+     *  The document view instance containing this control.
      *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options also supported by the base
      *  class MenuMixin (visibility of caret icon), and the class CompoundMenu
      *  (settings for the drop-down menu).
      */
-    function CompoundMenuMixin(app, initOptions) {
+    function CompoundMenuMixin(docView, initOptions) {
 
         var // the pop-up menu instance
-            menu = new CompoundMenu(app, Utils.extendOptions({ expandWidth: true }, initOptions, { anchor: this.getNode() }));
+            menu = new CompoundMenu(docView, Utils.extendOptions({ expandWidth: true }, initOptions, { anchor: this.getNode() }));
 
         // base constructors --------------------------------------------------
 
@@ -55,7 +55,7 @@ define('io.ox/office/baseframework/view/control/compoundmenumixin', [
         // public methods -----------------------------------------------------
 
         // export methods of the menu
-        _.each(['addGroup', 'removeGroup', 'addSeparator', 'addSectionLabel'], function (methodName) {
+        ['addGroup', 'removeGroup', 'addSeparator', 'addSectionLabel'].forEach(function (methodName) {
             this[methodName] = function () {
                 var result = menu[methodName].apply(menu, arguments);
                 return (result === menu) ? this : result;
@@ -66,7 +66,7 @@ define('io.ox/office/baseframework/view/control/compoundmenumixin', [
 
         // destroy all class members
         this.registerDestructor(function () {
-            app = initOptions = menu = null;
+            docView = initOptions = menu = null;
         });
 
     } // class CompoundMenuMixin
