@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -33,33 +33,35 @@ define('io.ox/office/baseframework/view/control/compoundsplitbutton', [
      * @extends Button
      * @extends CompoundMenuMixin
      *
-     * @param {BaseApplication} app
-     *  The application containing this control.
+     * @param {BaseView} docView
+     *  The document view instance containing this control.
      *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options also supported by the base
      *  class Button, and the mix-in class CompoundMenuMixin.
      */
-    function CompoundSplitButton(app, initOptions) {
+    function CompoundSplitButton(docView, initOptions) {
 
         var // the drop-down button
-            menuButton = $(Forms.createButtonMarkup({
-                tooltip: Utils.getStringOption(initOptions, 'caretTooltip', Utils.getStringOption(initOptions, 'tooltip', ''))
-            }));
+            menuButton = $(Forms.createButtonMarkup());
 
         // base constructors --------------------------------------------------
 
         Button.call(this, initOptions);
-        CompoundMenuMixin.call(this, app, Utils.extendOptions(initOptions, { button: menuButton }));
+        CompoundMenuMixin.call(this, docView, Utils.extendOptions(initOptions, { button: menuButton }));
 
         // initialization -----------------------------------------------------
 
+        Forms.setToolTip(menuButton, {
+            tooltip: Utils.getStringOption(initOptions, 'caretTooltip', Utils.getStringOption(initOptions, 'tooltip', ''))
+        });
+
         // add the drop-down button to the group
         this.addChildNodes(menuButton);
 
         // destroy all class members
         this.registerDestructor(function () {
-            app = initOptions = menuButton = null;
+            docView = initOptions = menuButton = null;
         });
 
     } // class CompoundSplitButton
