@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,7 +14,7 @@
 define('io.ox/office/baseframework/view/control/percentagelabel', [
     'io.ox/office/tk/utils',
     'io.ox/office/baseframework/view/control/dynamiclabel',
-    'gettext!io.ox/office/main'
+    'gettext!io.ox/office/baseframework/main'
 ], function (Utils, DynamicLabel, gt) {
 
     'use strict';
