@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -16,9 +16,11 @@ define('io.ox/office/baseframework/view/pane', [
     'io.ox/office/tk/keycodes',
     'io.ox/office/tk/forms',
     'io.ox/office/tk/config',
+    'io.ox/office/tk/utils/tracking',
     'io.ox/office/tk/object/triggerobject',
-    'io.ox/office/tk/object/timermixin'
-], function (Utils, KeyCodes, Forms, Config, TriggerObject, TimerMixin) {
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/baseframework/view/viewobjectmixin'
+], function (Utils, KeyCodes, Forms, Config, Tracking, TriggerObject, TimerMixin, ViewObjectMixin) {
 
     'use strict';
 
@@ -55,9 +57,10 @@ define('io.ox/office/baseframework/view/pane', [
      *
      * @extends TriggerObject
      * @extends TimerMixin
+     * @extends ViewObjectMixin
      *
-     * @param {BaseApplication} app
-     *  The application containing this pane element.
+     * @param {BaseView} docView
+     *  The document view instance containing this pane element.
      *
      * @param {Object} [initOptions]
      *  Optional parameters:
@@ -80,53 +83,27 @@ define('io.ox/office/baseframework/view/pane', [
      *  @param {String} [initOptions.maxSize=0x7FFFFFFF]
      *      The maximum size of resizable panes (when the option 'resizable' is
      *      set to true).
-     *  @param {Boolean} [initOptions.overlay=false]
-     *      If set to true, the pane will overlay the application pane instead
-     *      of reserving and consuming the space needed for its size.
-     *  @param {Boolean} [initOptions.transparent=false]
-     *      If set to true, the background of an overlay pane will be
-     *      transparent. Has no effect if the pane is not in overlay mode.
-     *  @param {Boolean} [initOptions.hoverEffect=false]
-     *      If set to true, all control groups in all view components will be
-     *      displayed half-transparent as long as the mouse does not hover the
-     *      view pane. Has no effect, if the current device is a touch device.
      *  @param {Boolean} [initOptions.cursorNavigate=false]
      *      If set to true, the cursor keys can be used to move the browser
      *      focus to the next/previous focusable form control in this pane.
-     *  @param {Boolean} [initOptions.enableContextMenu=false]
-     *      If set to true, the view pane will enable the browser context menu.
-     *      Otherwise, the context menu will be disabled for all DOM nodes but
-     *      for text input fields and text areas.
-     *  @param {Boolean} [initOptions.hideOnEscape=false]
-     *      If set to true, and the ESCAPE key is pressed while browser focus
-     *      is inside the view pane, it will hide itself automatically.
      */
-    function Pane(app, initOptions) {
+    function Pane(docView, initOptions) {
 
         var // self reference
             self = this,
 
             // the container element representing the pane
-            rootNode = Utils.createContainerNode('view-pane', initOptions),
+            rootNode = $('<div class="view-pane">').addClass(Utils.getStringOption(initOptions, 'classes', '')),
 
             // position of the pane in the application window
             position = Utils.getStringOption(initOptions, 'position', 'top'),
 
-            // overlay pane or fixed pane
-            overlay = Utils.getBooleanOption(initOptions, 'overlay', false),
-
-            // transparent overlay pane
-            transparent = overlay && Utils.getBooleanOption(initOptions, 'transparent', false),
-
             // minimum size of the view pane (for resizable panes)
             minSize = Utils.getIntegerOption(initOptions, 'minSize', 1, 1),
 
             // maximum size of the view pane (for resizable panes)
             maxSize = Utils.getIntegerOption(initOptions, 'maxSize', 0x7FFFFFFF, minSize),
 
-            // whether to hide the pane on ESCAPE key
-            hideOnEscape = Utils.getBooleanOption(initOptions, 'hideOnEscape', false),
-
             // the last cached size of the root node, used to detect layout changes
             nodeSize = null,
 
@@ -149,6 +126,7 @@ define('io.ox/office/baseframework/view/pane', [
 
         TriggerObject.call(this);
         TimerMixin.call(this);
+        ViewObjectMixin.call(this, docView);
 
         // private methods ----------------------------------------------------
 
@@ -282,13 +260,13 @@ define('io.ox/office/baseframework/view/pane', [
                 case 'tracking:move':
                     var size = origSize + (self.isLeadingPosition() ? 1 : -1) * (self.isVerticalPosition() ? event.offsetY : event.offsetX);
                     self.setSize(Utils.minMax(size, minSize, maxSize));
-                break;
+                    break;
                 case 'tracking:end':
-                    app.getView().grabFocus();
+                    docView.grabFocus();
                     break;
                 case 'tracking:cancel':
                     self.setSize(origSize);
-                    app.getView().grabFocus();
+                    docView.grabFocus();
                     break;
                 }
             }
@@ -303,8 +281,7 @@ define('io.ox/office/baseframework/view/pane', [
 
             // ESCAPE key
             if (event.keyCode === KeyCodes.ESCAPE) {
-                app.getView().grabFocus();
-                if (hideOnEscape) { self.hide(); }
+                docView.grabFocus();
                 return false;
             }
 
@@ -403,26 +380,6 @@ define('io.ox/office/baseframework/view/pane', [
             return Utils.isLeadingPosition(position);
         };
 
-        /**
-         * Returns whether this pane is an overlay pane.
-         *
-         * @returns {Boolean}
-         *  Whether this view pane is an overlay pane.
-         */
-        this.isOverlay = function () {
-            return overlay;
-        };
-
-        /**
-         * Returns whether this pane is a transparent overlay pane.
-         *
-         * @returns {Boolean}
-         *  Whether this view pane is a transparent overlay pane.
-         */
-        this.isTransparent = function () {
-            return transparent;
-        };
-
         /**
          * Checks the current outer size of the root node of this view pane.
          * Triggers a 'pane:resize' event if the pane is visible and its size
@@ -479,9 +436,6 @@ define('io.ox/office/baseframework/view/pane', [
          */
         this.toggle = function (state) {
 
-            // always cancel node tracking if the visibility of a view pane changes
-            $.cancelTracking();
-
             // resolve missing parameter for toggling visibility
             visible = (state === true) || ((state !== false) && !visible);
 
@@ -606,7 +560,7 @@ define('io.ox/office/baseframework/view/pane', [
             return _.any(components, function (component) { return component.hasFocus(); });
         };
 
-		/**
+        /**
          * Sets the focus to the first enabled control group object in this
          * view pane, unless the pane already contains the browser focus.
          *
@@ -624,15 +578,14 @@ define('io.ox/office/baseframework/view/pane', [
         rootNode.attr('data-pos', position);
 
         // marker for touch devices and browser types
-        Utils.addDeviceMarkers(rootNode);
+        Forms.addDeviceMarkers(rootNode);
 
         // fixed size if specified (zero will be ignored)
         this.setSize(Utils.getIntegerOption(initOptions, 'size', null, 1));
 
         // no size tracking for transparent view panes
-        if (!transparent && Utils.getBooleanOption(initOptions, 'resizable', false)) {
-            $('<div class="resizer" tabindex="-1">')
-                .enableTracking()
+        if (Utils.getBooleanOption(initOptions, 'resizable', false)) {
+            Tracking.enableTracking($('<div class="resizer" tabindex="-1">'))
                 .on('tracking:start tracking:move tracking:end tracking:cancel', trackingHandler)
                 .appendTo(rootNode);
         }
@@ -640,21 +593,6 @@ define('io.ox/office/baseframework/view/pane', [
         // whether the pane will be a land mark for F6 focus traveling
         rootNode.toggleClass('f6-target', Utils.getBooleanOption(initOptions, 'landmark', true));
 
-        // overlay mode
-        rootNode.toggleClass('overlay', overlay);
-        if (transparent) {
-            rootNode.css(this.isVerticalPosition() ? 'height' : 'width', 0);
-            // hover effect for view components embedded in the pane
-            rootNode.toggleClass('hover-effect', Utils.getBooleanOption(initOptions, 'hoverEffect', false));
-        }
-
-        // context menu support
-        if (!Utils.getBooleanOption(initOptions, 'enableContextMenu', false)) {
-            rootNode.on('contextmenu', function (event) {
-                if (!$(event.target).is('input,textarea')) { return false; }
-            });
-        }
-
         // keyboard handler
         rootNode.on('keydown', keyDownHandler);
         if (Utils.getBooleanOption(initOptions, 'cursorNavigate', false)) {
@@ -674,7 +612,7 @@ define('io.ox/office/baseframework/view/pane', [
         this.registerDestructor(function () {
             _.invoke(components, 'destroy');
             rootNode.remove();
-            app = initOptions = self = rootNode = components = componentInserters = null;
+            self = docView = initOptions = rootNode = components = componentInserters = null;
         });
 
     } // class Pane
