@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -17,11 +17,18 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
     'io.ox/office/tk/popup/basemenu',
     'io.ox/office/tk/control/button',
     'io.ox/office/baseframework/view/component',
-    'gettext!io.ox/office/main'
-], function (Utils, Forms, BaseMenu, Button, Component, gt) {
+    'io.ox/office/baseframework/view/viewobjectmixin',
+    'gettext!io.ox/office/baseframework/main'
+], function (Utils, Forms, BaseMenu, Button, Component, ViewObjectMixin, gt) {
 
     'use strict';
 
+    var // CSS class name for separator lines
+        SEPARATOR_LINE_CLASS = 'separator-line',
+
+        // CSS class name for header labels
+        HEADER_LABEL_CLASS = 'header-label';
+
     // class CompoundMenu =====================================================
 
     /**
@@ -37,24 +44,31 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
      * @constructor
      *
      * @extends BaseMenu
+     * @extends ViewObjectMixin
      *
-     * @param {BaseApplication} app
-     *  The application containing this pop-up menu.
+     * @param {BaseView} docView
+     *  The document view instance containing this menu.
      *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options also supported by the base
      *  class BaseMenu. Additionally, the following options are supported:
+     *  @param {Boolean} [initOptions.autoHideGroups=false]
+     *      If set to true, all disabled control groups contained in this menu
+     *      will be hidden automatically.
+     *  @param {Function} [initOptions.lazyInitHandler]
+     *      A callback function that will be invoked exactly once before this
+     *      compound menu will be shown the first time.
      *  @param {String} [initOptions.actionKey]
      *      If specified, the menu will contain a footer section with an action
      *      button, and a cancel button. The action button will be bound to the
      *      controller item with the key specified in this option. The enabled
      *      state of the button element will be bound to the state of the
      *      controller item.
-     *  @param {String} [options.actionLabel]
+     *  @param {String} [initOptions.actionLabel]
      *      A different text label for the action button in the footer section.
      *      If omitted, the button will contain the label 'OK'. This option has
      *      no effect, if the option 'actionKey' has been omitted.
-     *  @param {Any} [options.actionValue]
+     *  @param {Any} [initOptions.actionValue]
      *      A value, object, or function that will be used as fixed value for
      *      the action button when it has been clicked. Must not be null. A
      *      function will be evaluated every time the button has been
@@ -62,28 +76,48 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
      *      been passed, the current value of that group will be used and
      *      forwarded as button value instead. This option has no effect, if
      *      the option 'actionKey' has been omitted.
+     *  @param {Function} [options.enableActionHandler]
+     *      A predicate callback function that can be used to influence the
+     *      enabled state of the action button manually. MUST return a boolean
+     *      value stating whether the control group is enabled.
      */
-    function CompoundMenu(app, initOptions) {
+    function CompoundMenu(docView, initOptions) {
 
         var // self reference
             self = this,
 
+            // lazy initialization callback function
+            lazyInitHandler = Utils.getFunctionOption(initOptions, 'lazyInitHandler'),
+
+            // callback predicate returning whether the OK button in the footer is currently enabled
+            enableActionHandler = Utils.getFunctionOption(initOptions, 'enableActionHandler'),
+
+            // whether to hide all disabled control groups automatically
+            autoHideGroups = Utils.getBooleanOption(initOptions, 'autoHideGroups', false),
+
             // the view component in the drop-down menu for dynamic contents
             innerComponent = null,
 
-            // the optional footer component
-            footerComponent = null,
+            // the root node of the menu footer
+            footerNode = null,
+
+            // the OK button shown in the footer
+            actionButton = null,
+
+            // the Cancel button shown in the footer
+            cancelButton = null,
 
             // saved scroll position of inner component during 2-step layouting
             scrollTop = 0;
 
-        // base constructor ---------------------------------------------------
+        // base constructors --------------------------------------------------
 
         BaseMenu.call(this, Utils.extendOptions(initOptions, {
             suppressScrollBars: true,
             prepareLayoutHandler: prepareLayoutHandler,
             updateLayoutHandler: updateLayoutHandler
         }));
+        ViewObjectMixin.call(this, docView);
 
         // private methods ----------------------------------------------------
 
@@ -120,7 +154,7 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
             //  - insert before specific element (save separator?)
             } else if (insertBefore instanceof $) {
                 // save separator
-                if (saveSeparator && insertBefore.prev().is('.separator-line')) { insertBefore = insertBefore.prev(); }
+                if (saveSeparator && insertBefore.prev().is('.' + SEPARATOR_LINE_CLASS)) { insertBefore = insertBefore.prev(); }
                 // insert
                 insertBefore.before(groupNode);
 
@@ -136,14 +170,50 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
          */
         function prepareLayoutHandler() {
 
-            var // DOM node of the inner component
-                innerCompNode = innerComponent ? innerComponent.getNode() : null;
+            // DOM node of the inner component
+            var innerCompNode = innerComponent ? innerComponent.getNode() : null;
+            if (!innerCompNode) { return; }
 
-            if (innerCompNode) {
-                // bug 35832: save scroll position before removing overflow-y attribute
-                scrollTop = innerCompNode.scrollTop();
-                innerCompNode.css({ height: '', overflowY: '' });
+            // bug 35832: save scroll position before removing overflow-y attribute
+            scrollTop = innerCompNode.scrollTop();
+            innerCompNode.css({ height: '', overflowY: '' });
+
+            // separate all nodes into sections
+            var allSectionData = [], currData = null;
+
+            function newSectionData(separator) {
+                allSectionData.push(currData = { separator: separator, header: null, groups: [] });
             }
+
+            innerCompNode.children().each(function () {
+                var childNode = $(this);
+                if (childNode.is('.' + SEPARATOR_LINE_CLASS)) {
+                    newSectionData(childNode);
+                } else if (childNode.is('.' + HEADER_LABEL_CLASS)) {
+                    if (!currData || (currData.groups.length > 0)) { newSectionData(null); }
+                    currData.header = childNode;
+                } else if (childNode.is('.group')) {
+                    if (!currData) { newSectionData(null); }
+                    currData.groups.push(childNode);
+                }
+            });
+
+            // update visibility of the separator lines and header labels according to visible groups
+            allSectionData.forEach(function (sectionData, index) {
+
+                // detect sections with visible groups
+                sectionData.visible = sectionData.groups.some(Forms.isVisibleNode);
+
+                // separator line needs a visible group before and one after
+                if (sectionData.separator) {
+                    Forms.showNodes(sectionData.separator, (index > 0) && sectionData.visible && allSectionData[index - 1].visible);
+                }
+
+                // header label needs a visible group after
+                if (sectionData.header) {
+                    Forms.showNodes(sectionData.header, sectionData.visible);
+                }
+            });
         }
 
         /**
@@ -250,7 +320,7 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
          *  A reference to this instance.
          */
         this.addSeparator = function () {
-            innerComponent.getNode().append('<div class="separator-line">');
+            innerComponent.getNode().append('<div class="' + SEPARATOR_LINE_CLASS + '">');
             return this;
         };
 
@@ -273,7 +343,7 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
                 this.addSeparator();
             }
 
-            innerComponent.getNode().append('<div class="header-label" aria-hidden="true">' + Utils.escapeHTML(label) + '</div>');
+            innerComponent.getNode().append('<div class="' + HEADER_LABEL_CLASS + '" aria-hidden="true">' + Utils.escapeHTML(label) + '</div>');
             return this;
         };
 
@@ -282,9 +352,8 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
          *
          * @param {Function} iterator
          *  The iterator called for each control group. Receives the reference
-         *  to the control group, and its insertion index. If the iterator
-         *  returns the Utils.BREAK object, the iteration process will be
-         *  stopped immediately.
+         *  to the control group. If the iterator returns the Utils.BREAK
+         *  object, the iteration process will be stopped immediately.
          *
          * @param {Object} [context]
          *  If specified, the iterator will be called with this context (the
@@ -310,6 +379,30 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
             return _.isObject(innerComponent) && innerComponent.hasVisibleGroups();
         };
 
+        /**
+         * Updates the visibility, enabled state, and value of all groups in
+         * this pop-up menu.
+         *
+         * @returns {CompoundMenu}
+         *  A reference to this instance.
+         */
+        this.updateAllGroups = function () {
+            innerComponent.updateAllGroups();
+            return this;
+        };
+
+        /**
+         * Removes and destroys all control groups from this pop-up menu, and
+         * completely clears its root node.
+         *
+         * @returns {CompoundMenu}
+         *  A reference to this instance.
+         */
+        this.destroyAllGroups = function () {
+            innerComponent.destroyAllGroups();
+            return this;
+        };
+
         /**
          * Releases ownership of the current view component used as container
          * for the form controls shown in this pop-up menu, and detaches its
@@ -353,8 +446,8 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
             // if it's already there, get out here
             if (innerComponent === newComponent) { return this; }
 
-            var // release (unregister) the current view component
-                oldComponent = this.releaseComponent();
+            // release (unregister) the current view component
+            var oldComponent = this.releaseComponent();
 
             // destroy old component (removes itself from the DOM)
             if (_.isObject(oldComponent)) {
@@ -363,8 +456,10 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
 
             // register the new view component
             innerComponent = newComponent;
-            if (footerComponent) {
-                innerComponent.getNode().insertBefore(footerComponent.getNode());
+            if (footerNode) {
+                innerComponent.getNode().insertBefore(footerNode);
+                // keep OK button in the footer section up-to-date
+                this.listenTo(innerComponent, 'component:change', this.updateFooter.bind(this));
             } else {
                 this.appendContentNodes(innerComponent.getNode());
             }
@@ -378,51 +473,74 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
             return this;
         };
 
+        /**
+         * Updates the OK/Cancel buttons contained in the footer, if existing.
+         *
+         * @returns {CompoundMenu}
+         *  A reference to this instance.
+         */
+        this.updateFooter = function () {
+
+            // enable/disable the OK button according to the callback function
+            if (actionButton) {
+                var enabled = !enableActionHandler || (enableActionHandler.call(self) === true);
+                Forms.enableBSButton(actionButton, enabled);
+            }
+
+            return this;
+        };
+
         // initialization -----------------------------------------------------
 
         // do not allow to clear the menu manually
         delete this.clearContents;
 
+        // call lazy initialization handler before showing this menu
+        if (_.isFunction(lazyInitHandler)) {
+            this.one('popup:beforeshow', function () {
+                lazyInitHandler.call(self);
+                innerComponent.updateAllGroups();
+            });
+        }
+
+        // update OK/Cancel buttons after showing the menu
+        this.on('popup:show popup:idle', function () {
+            self.updateFooter();
+        });
+
         // create the header and footer sections
         (function () {
 
-            var // the controller key for the OK button in the footer section
-                actionKey = Utils.getStringOption(initOptions, 'actionKey', '');
+            // the controller key for the OK button in the footer section
+            var actionKey = Utils.getStringOption(initOptions, 'actionKey', '');
+            if (actionKey.length === 0) { return; }
 
-            // create and insert the footer component
-            if (actionKey.length > 0) {
-
-                var // the text label and value for the action button
-                    actionLabel = Utils.getStringOption(initOptions, 'actionLabel', gt('OK')),
-                    actionValue = Utils.getOption(initOptions, 'actionValue'),
-                    // the action and cancel button controls
-                    actionButton = new Button({ nodeName: 'button', label: actionLabel, value: actionValue }),
-                    cancelButton = new Button({ nodeName: 'button', label: gt('Cancel') });
-
-                // additional standard Bootstrap styling for the buttons
-                actionButton.getButtonNode().removeClass(Forms.BUTTON_CLASS).addClass('btn btn-primary');
-                cancelButton.getButtonNode().removeClass(Forms.BUTTON_CLASS).addClass('btn btn-default');
-
-                // create the footer view component
-                footerComponent = new Component(app);
-                self.appendContentNodes(footerComponent.getNode().addClass('footer-section'));
-
-                // insert the buttons into the footer component
-                footerComponent.addGroup(null, cancelButton).addGroup(actionKey, actionButton);
-
-                // close the menu when hitting the cancel button
-                self.listenTo(cancelButton, 'group:change', function () {
-                    self.hide();
-                    app.getView().grabFocus();
-                });
+            function actionHandler() {
+                var actionValue = Utils.getOption(initOptions, 'actionValue');
+                if (_.isFunction(actionValue)) { actionValue = actionValue.call(self); }
+                docView.executeControllerItem(actionKey, actionValue);
             }
+
+            // close the menu when hitting the cancel button
+            function cancelHandler() {
+                self.hide();
+                docView.grabFocus();
+            }
+
+            // the action and cancel button controls
+            actionButton = $.button({ label: Utils.getStringOption(initOptions, 'actionLabel', gt('OK')), click: actionHandler }).addClass('btn-primary');
+            cancelButton = $.button({ label: gt('Cancel'), click: cancelHandler }).addClass('btn-default');
+
+            // create the footer section node with OK and Cancel button
+            footerNode = $('<div class="footer-section">').append(cancelButton, actionButton);
+            self.appendContentNodes(footerNode);
         }());
 
         // create and insert the inner view component into the drop-down menu
-        this.replaceComponent(new Component(app));
+        this.replaceComponent(new Component(docView, { autoHideGroups: autoHideGroups }));
 
         // hide pop-up menu when application is not active
-        this.listenTo(app.getWindow(), 'hide', function () { self.hide(); });
+        this.listenTo(docView.getApp().getWindow(), 'hide', function () { self.hide(); });
 
         // additional cursor key shortcuts for focus navigation
         Forms.enableCursorFocusNavigation(this.getNode(), { homeEnd: true });
@@ -430,8 +548,8 @@ define('io.ox/office/baseframework/view/popup/compoundmenu', [
         // destroy all class members on destruction
         this.registerDestructor(function () {
             if (_.isObject(innerComponent)) { innerComponent.destroy(); }
-            if (_.isObject(footerComponent)) { footerComponent.destroy(); }
-            app = initOptions = self = innerComponent = footerComponent = null;
+            self = docView = initOptions = lazyInitHandler = innerComponent = null;
+            footerNode = actionButton = cancelButton = null;
         });
 
     } // class CompoundMenu
