@@ -6,15 +6,16 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/baseframework/view/popup/contextmenu', [
     'io.ox/office/tk/utils',
+    'io.ox/office/tk/object/timermixin',
     'io.ox/office/baseframework/view/popup/compoundmenu'
-], function (Utils, CompoundMenu) {
+], function (Utils, TimerMixin, CompoundMenu) {
 
     'use strict';
 
@@ -39,9 +40,10 @@ define('io.ox/office/baseframework/view/popup/contextmenu', [
      * @constructor
      *
      * @extends CompoundMenu
+     * @extends TimerMixin
      *
-     * @param {BaseApplication} app
-     *  The application containing this context menu.
+     * @param {BaseView} docView
+     *  The document view instance containing this context menu.
      *
      * @param {HTMLObject|jQuery} sourceNode
      *  The DOM element used as event source for context events (right mouse
@@ -50,12 +52,12 @@ define('io.ox/office/baseframework/view/popup/contextmenu', [
      *  contains.
      *
      * @param {Object} [initOptions]
-     *  Initial options for this context menu.
+     *  Optional parameters:
      *  @param {String} [initOptions.selector]
      *      A jQuery selector for specific descendant elements of the passed
      *      event source node. The context menu will only be shown, if the
      *      mouse or keyboard events originate from the selected nodes.
-     *  @param {String|String[]} [initOptions.enableKey]
+     *  @param {String|Array<String>} [initOptions.enableKey]
      *      The key of a controller item, or an array of controller item keys,
      *      that specify whether the context menu is enabled. Uses the enabled
      *      state of the controller items registered for the specified keys.
@@ -67,7 +69,7 @@ define('io.ox/office/baseframework/view/popup/contextmenu', [
      *      false, the context menu will be opened after the 'mouseup' and its
      *      related 'contextmenu' event.
      */
-    function ContextMenu(app, sourceNode, initOptions) {
+    function ContextMenu(docView, sourceNode, initOptions) {
 
         var // self reference
             self = this,
@@ -78,12 +80,20 @@ define('io.ox/office/baseframework/view/popup/contextmenu', [
             // controller keys specifying whether the context menu is enabled
             enableKeys = Utils.getOption(initOptions, 'enableKey'),
 
-            // whether to show the context menu instantly after the 'mousedown' event
-            mouseDown = Utils.getBooleanOption(initOptions, 'mouseDown', false);
+            // delay to open the context menu
+            delay = Utils.getNumberOption(initOptions, 'delay'),
+
+            // special handler to invoke
+            contextMenuHandlers = [],
+            detectHandlers = [],
+            resetHandlers = [];
 
         // base constructor ---------------------------------------------------
 
-        CompoundMenu.call(this, app, { anchorPadding: 0 });
+        CompoundMenu.call(this, docView, _.extend({ anchorPadding: 0, autoHideGroups: true }, initOptions));
+
+        // add timer methods (needs the registerDestructor() method)
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
@@ -92,9 +102,9 @@ define('io.ox/office/baseframework/view/popup/contextmenu', [
          * the context menu at the correct position.
          */
         function showContextMenu(event) {
-
             var // the (cancelable) jQuery prepare event
-                prepareEvent = new $.Event('contextmenu:prepare', { sourceEvent: event });
+                prepareEvent = new $.Event('contextmenu:prepare', { sourceEvent: event.sourceEvent }),
+                xy = self.getXY(event);
 
             // notify listeners (also if the context menu is disabled, e.g. to select
             // an element on click, although the context menu will not occur)
@@ -104,19 +114,64 @@ define('io.ox/office/baseframework/view/popup/contextmenu', [
             // the controller item bound to the context menu is disabled
             if (prepareEvent.isDefaultPrevented() || !self.isEnabled()) { return; }
 
-            // set the click position as anchor, and show the context menu
-            self.hide().setAnchor({ left: event.pageX, top: event.pageY, width: 1, height: 1 }).show();
+            function executeShow() {
+                // set the click position as anchor, and show the context menu
+                self.hide().setAnchor({ left: xy.pageX, top: xy.pageY, width: 1, height: 1 }).show();
+            }
+
+            if (delay) {
+                self.executeDelayed(executeShow, delay);
+            } else {
+                executeShow();
+            }
+        }
+
+        /**
+         * Detects some states we must know in the following event stack
+         *
+         * @param {String} defaultEvent
+         *  The type which triggers the event (mouse, keyboard or touch)
+         */
+        function detectState() {
+            detectHandlers.forEach(function (func) {
+                func.call(this);
+            }, this);
+        }
+
+        /**
+         * Resets all detected states
+         *
+         * @param {Object} [options]
+         *  Optional parameters. Used in some resetHandlers.
+         */
+        function resetState(options) {
+            resetHandlers.forEach(function (func) {
+                func.call(this, options);
+            }, this);
+        }
+
+        // EVENT-Handler ------------------------------------------------------
+
+        /**
+         * Handles 'keydown' events of the source node.
+         */
+        function keyDownHandler() {
+            detectState();
+        }
+
+        /**
+         * Handles 'keyup' events of the source node.
+         */
+        function keyUpHandler() {
+            resetState();
         }
 
         /**
          * Handles 'mousedown' events of the source node. Shows the context
          * menu, if the option 'mouseDown' has been passed to the constructor.
          */
-        function mouseDownHandler(event) {
-            if (mouseDown && (event.button === 2)) {
-                showContextMenu(event);
-                return false;
-            }
+        function mouseDownHandler() {
+            detectState();
         }
 
         /**
@@ -125,28 +180,66 @@ define('io.ox/office/baseframework/view/popup/contextmenu', [
          * showing a context menu.
          */
         function contextMenuHandler(event) {
+
+            // bug 38770: fail-safety: prevent context menus in busy state
+            if (docView.isBusy()) { return false; }
+
+            var sourceEvent = event.sourceEvent;
+
+            // if x/y-coordinate not set, calculate them both by last known
+            // touch position
+            if (!event.pageX || !event.pageY) {
+                if (!_.isNull(sourceEvent)) {
+                    event.pageX = sourceEvent.pageX;
+                    event.pageY = sourceEvent.pageY;
+
+                } else {
+                    var clickedElement = $(sourceEvent.target),
+                        offset = clickedElement.offset(),
+                        width = clickedElement.width(),
+                        height = clickedElement.height();
+
+                    event.pageX = (offset.left + (width / 2));
+                    event.pageY = (offset.top + (height / 2));
+                }
+            }
+
+            contextMenuHandlers.forEach(function (func) {
+                func.call(self, event);
+            });
+
             showContextMenu(event);
+
             event.preventDefault();
+            event.stopPropagation();
+            return false;
         }
 
         /**
-         * Handles 'change:items' events of the controller. Hides the context
-         * menu if any of the controller items specified with the constructor
-         * option 'enableKey' has been disabled.
+         * Handles 'mousedown' events of the source node. Shows the context
+         * menu, if the option 'mouseDown' has been passed to the constructor.
          */
-        function controllerChangeHandler(event, changedItems) {
+        function mouseUpHandler() {
+            resetState({ resetTextSelection: true });
+        }
+
+        /**
+         * Hides the context menu if any of the controller items specified with
+         * the constructor option 'enableKey' has been disabled.
+         */
+        function controllerChangeHandler(changedItems) {
 
             // nothing to do, if the context menu is not visible
             if (!self.isVisible()) { return; }
 
             var // whether to hide the context menu due to changed controller item states
-                hideMenu = _.any(enableKeys, function (enableKey) {
+                hideMenu = enableKeys.some(function (enableKey) {
                     return (enableKey in changedItems) && (changedItems[enableKey].enabled === false);
                 });
 
             if (hideMenu) {
                 self.hide();
-                app.getView().grabFocus();
+                docView.grabFocus();
             }
         }
 
@@ -161,31 +254,82 @@ define('io.ox/office/baseframework/view/popup/contextmenu', [
          *  Whether this context menu is currently enabled.
          */
         this.isEnabled = function () {
-            var controller = app.getController();
-            return _.all(enableKeys, _.bind(controller.isItemEnabled, controller));
+            var controller = docView.getApp().getController();
+            return enableKeys.every(controller.isItemEnabled, controller);
+        };
+
+        /**
+         * Register handler for 'contextmenu' event. Invoked in the
+         * contextMenuHandler.
+         *
+         * @param  {Function} func
+         *   The function which should be invoked.
+         */
+        this.registerContextMenuHandler = function (func) {
+            contextMenuHandlers.push(func);
+        };
+
+        /**
+         * Register handler for 'detect' method. Invoked in the
+         * detectState method.
+         *
+         * @param  {Function} func
+         *   The function which should be invoked.
+         */
+        this.registerDetectHandler = function (func) {
+            detectHandlers.push(func);
+        };
+
+        /**
+         * Register handler for 'reset' method. Invoked in the
+         * resetState method.
+         *
+         * @param  {Function} func
+         *   The function which should be invoked.
+         */
+        this.registerResetHandler = function (func) {
+            resetHandlers.push(func);
+        };
+
+        /**
+         * Return the default pageX / pageY of the event
+
+         * @return {Object}
+         *   Object with 'pageX' and 'pageY' values
+         */
+        this.getXY = function (event) {
+            return {
+                pageX: event.pageX,
+                pageY: event.pageY
+            };
         };
 
         // initialization -----------------------------------------------------
 
+        this.getNode().addClass('context-menu');
+
         // listen to relevant context events on the source node
-        sourceNode.on({ mousedown: mouseDownHandler, contextmenu: contextMenuHandler }, selector);
+        sourceNode.on({
+            mousedown: mouseDownHandler,
+            mouseup: mouseUpHandler,
+            keydown: keyDownHandler,
+            keyup: keyUpHandler,
+            'documents:contextmenu': contextMenuHandler
+        }, selector);
 
         // always use an array of parent keys (as strings)
         enableKeys =
             _.isArray(enableKeys) ? _.filter(enableKeys, _.isString) :
-            _.isString(enableKeys) ? [enableKeys] :
-            null;
+            _.isString(enableKeys) ? [enableKeys] : [];
 
         // hide displayed context menu, if controller item changes to disabled state
-        if (_.isArray(enableKeys) && (enableKeys.length > 0)) {
-            app.onInit(function () {
-                self.listenTo(app.getController(), 'change:items', controllerChangeHandler);
-            });
+        if (enableKeys.length > 0) {
+            this.handleChangedControllerItems(controllerChangeHandler);
         }
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = sourceNode = initOptions = self = null;
+            docView = sourceNode = initOptions = self = null;
         });
 
     } // class ContextMenu
