@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Michael Nimz <michael.nimz@open-xchange.com>
  */
@@ -14,8 +14,10 @@
 define('io.ox/office/baseframework/view/popup/layermenu', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/forms',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/utils/tracking',
     'io.ox/office/baseframework/view/popup/compoundmenu'
-], function (Utils, Forms, CompoundMenu) {
+], function (Utils, Forms, KeyCodes, Tracking, CompoundMenu) {
 
     'use strict';
 
@@ -29,8 +31,8 @@ define('io.ox/office/baseframework/view/popup/layermenu', [
      *
      * @extends CompoundMenu
      *
-     * @param {BaseApplication} app
-     *  The application containing this menu instance.
+     * @param {BaseView} docView
+     *  The document view instance containing this menu.
      *
      * @param {String} titleLabel
      *  The title shown on top of the menu.
@@ -42,7 +44,7 @@ define('io.ox/office/baseframework/view/popup/layermenu', [
      *  top-level global AppSuite tab bar); and the option 'autoClose' will be
      *  set to false (the browser focus may be located outside the menu node).
      */
-    function LayerMenu(app, titleLabel, initOptions) {
+    function LayerMenu(docView, titleLabel, initOptions) {
 
         var // self reference
             self = this,
@@ -64,8 +66,8 @@ define('io.ox/office/baseframework/view/popup/layermenu', [
 
         // base constructor ---------------------------------------------------
 
-        CompoundMenu.call(this, app, Utils.extendOptions({
-            boundingBox: app.getWindowNode(),
+        CompoundMenu.call(this, docView, Utils.extendOptions({
+            boundingBox: docView.getApp().getWindowNode(),
             autoClose: false
         }, initOptions));
 
@@ -76,7 +78,7 @@ define('io.ox/office/baseframework/view/popup/layermenu', [
          * will be hidden.
          */
         function popupBeforeHideHandler() {
-            app.getView().grabFocus();
+            docView.grabFocus();
         }
 
         /**
@@ -104,11 +106,16 @@ define('io.ox/office/baseframework/view/popup/layermenu', [
          * Click handler for the closer button in the header section. Triggers
          * a 'menu:userclose' event, and closes the menu afterwards.
          */
-        function closeButtonHandler() {
-            self.trigger('menu:userclose').hide();
+        function closeButtonHandler(e) {
+            var type = e.type,
+                key = e.keyCode;
 
-            //activator buttons should know the changes
-            app.getController().update();
+            if ((type === 'keydown' && (key === KeyCodes.SPACE || key === KeyCodes.ENTER)) || type !== 'keydown') {
+                self.trigger('menu:userclose').hide();
+
+                //activator buttons should know the changes
+                docView.getApp().getController().update();
+            }
         }
 
         // initialization -----------------------------------------------------
@@ -123,15 +130,16 @@ define('io.ox/office/baseframework/view/popup/layermenu', [
         this.prependContentNodes(headerNode);
 
         // make the menu node draggable
-        titleNode.css('cursor', 'move').enableTracking();
+        titleNode.css('cursor', 'move');
+        Tracking.enableTracking(titleNode);
         titleNode.on({ 'tracking:start': trackingStartHandler, 'tracking:move': trackingMoveHandler });
 
         // notify listeners and close the menu on click
-        closeButton.on('click', closeButtonHandler);
+        closeButton.on('click keydown', closeButtonHandler);
 
         // destroy all class members
         this.registerDestructor(function () {
-            app = initOptions = self = headerNode = titleNode = closeButton = null;
+            self = docView = initOptions = headerNode = titleNode = closeButton = null;
         });
 
     } // class LayerMenu
