@@ -6,15 +6,15 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/baseframework/view/toolpane',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/view/pane'
-    ], function (Utils, Pane) {
+define('io.ox/office/baseframework/view/toolpane', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/baseframework/view/pane'
+], function (Utils, Pane) {
 
     'use strict';
 
@@ -34,14 +34,14 @@ define('io.ox/office/baseframework/view/toolpane',
      *
      * @extends Pane
      *
-     * @param {BaseApplication} app
-     *  The application containing this tool pane.
+     * @param {BaseView} docView
+     *  The document view instance containing this tool pane.
      *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options supported by the base class
      *  Pane.
      */
-    function ToolPane(app, initOptions) {
+    function ToolPane(docView, initOptions) {
 
         var // self reference
             self = this,
@@ -51,7 +51,7 @@ define('io.ox/office/baseframework/view/toolpane',
 
         // base constructor ---------------------------------------------------
 
-        Pane.call(this, app, initOptions);
+        Pane.call(this, docView, initOptions);
 
         // private methods ----------------------------------------------------
 
@@ -68,13 +68,11 @@ define('io.ox/office/baseframework/view/toolpane',
             // insert the component root node (fall-back to center area)
             if (!(targetArea in areaNodes)) { targetArea = 'center'; }
 
-            if (!specialPosition) {
-            	areaNodes[targetArea].append(component.getNode());
-
+            if (specialPosition) {
+                specialPosition.getNode().before(component.getNode());
             } else {
-            	specialPosition.getNode().before(component.getNode());
+                areaNodes[targetArea].append(component.getNode());
             }
-
         }
 
         // methods ------------------------------------------------------------
@@ -163,12 +161,12 @@ define('io.ox/office/baseframework/view/toolpane',
         // add CSS marker class for special formatting
         this.getNode().addClass('tool-pane');
 
-        this.getNode().on('touchmove', function(e) {
-            e.preventDefault();
+        this.getNode().on('touchmove', function (event) {
+            event.preventDefault();
         });
 
         // create component area nodes
-        _.each(['leading', 'center', 'trailing'], function (areaType) {
+        ['leading', 'center', 'trailing'].forEach(function (areaType) {
             areaNodes[areaType] = $('<div class="area-container">');
             self.getNode().append($('<div class="float-wrapper ' + areaType + '">').append(areaNodes[areaType]));
         });
@@ -182,7 +180,7 @@ define('io.ox/office/baseframework/view/toolpane',
         // destroy all class members on destruction
         this.registerDestructor(function () {
             _.invoke(areaNodes, 'remove');
-            app = initOptions = self = areaNodes = null;
+            self = docView = initOptions = areaNodes = null;
         });
 
     } // class ToolPane
