@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -61,8 +61,8 @@ define('io.ox/office/drawinglayer/model/drawingcollection', [
      * @extends TriggerObject
      * @extends TimerMixin
      *
-     * @param {EditApplication} app
-     *  The application instance containing this drawing collection.
+     * @param {EditModel} docModel
+     *  The document model containing this instance.
      *
      * @param {Object} initOptions
      *  Optional parameters:
@@ -93,7 +93,7 @@ define('io.ox/office/drawinglayer/model/drawingcollection', [
      *      using its DrawingModel.setAttributes() method. Receives the drawing
      *      model instance.
      */
-    function DrawingCollection(app, initOptions) {
+    function DrawingCollection(docModel, initOptions) {
 
         var // self reference
             self = this,
@@ -182,18 +182,49 @@ define('io.ox/office/drawinglayer/model/drawingcollection', [
             return null;
         }
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the number of drawings inserted directly into this drawing
          * collection (excluding the drawing objects embedded in other drawing
          * objects).
          *
+         * @param {Object} [options]
+         *  Optional parameters:
+         *  @param {Boolean} [options.deep=false]
+         *      If set to true, all drawing objects embedded in other drawing
+         *      objects will be counted too.
+         *  @param {String} [options.type]
+         *      If specified, only drawing models of the specified type will be
+         *      counted.
+         *
          * @returns {Number}
          *  The number of drawing objects contained in this collection.
          */
-        this.getModelCount = function () {
-            return modelCount;
+        this.getModelCount = function (options) {
+
+            var // whether to search deeply in embedded collections
+                deep = Utils.getBooleanOption(options, 'deep', false),
+                // whether to restrict to a specific type
+                type = Utils.getStringOption(options, 'type'),
+                // the resulting model count
+                count = 0;
+
+            // fast access of model count without deep option and type filtering
+            if (!deep && !type) { return modelCount; }
+
+            // count all models according to options
+            _.each(drawingModels, function (model) {
+
+                // type filtering
+                if (!type || (type === model.getType())) { count += 1; }
+
+                // deep option: count models in nested drawing collections
+                var collection = deep ? model.getCollection() : null;
+                if (collection) { count += collection.getModelCount(options); }
+            });
+
+            return count;
         };
 
         /**
