@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -29,8 +29,8 @@ define('io.ox/office/drawinglayer/model/drawingmodel', [
      * @extends AttributedModel
      * @extends TimerMixin
      *
-     * @param {EditApplication} app
-     *  The application instance containing this drawing object.
+     * @param {EditModel} docModel
+     *  The document model containing this drawing object.
      *
      * @param {String} type
      *  The type of this drawing object.
@@ -40,25 +40,36 @@ define('io.ox/office/drawinglayer/model/drawingmodel', [
      *  object.
      *
      * @param {Object} [initOptions]
-     *  A map with additional options for the drawing object. The following
-     *  options are supported:
-     *  @param {String} [options.families]
+     *  Optional parameters:
+     *  @param {String} [initOptions.families]
      *      Additional explicit attribute families supported by this drawing
      *      model object (space-separated). The main attribute family 'drawing'
      *      supported by all drawing objects will be registered implicitly, and
      *      does not have to be added here.
      */
-    function DrawingModel(app, type, initAttributes, initOptions) {
+    function DrawingModel(docModel, type, initAttributes, initOptions) {
 
         // base constructors --------------------------------------------------
 
-        AttributedModel.call(this, app, initAttributes, {
+        AttributedModel.call(this, docModel, initAttributes, {
             styleFamily: 'drawing',
             families: Utils.getStringOption(initOptions, 'families')
         });
 
         TimerMixin.call(this);
 
+        // protected methods --------------------------------------------------
+
+        /**
+         * Handler for the document operation 'setDrawingAttributes'.
+         *
+         * @param {OperationContext} context
+         *  A wrapper representing the 'setDrawingAttributes' operation.
+         */
+        this.applySetAttributesOperation = function (context) {
+            this.setAttributes(context.getObj('attrs'));
+        };
+
         // public methods -----------------------------------------------------
 
         /**
@@ -72,6 +83,17 @@ define('io.ox/office/drawinglayer/model/drawingmodel', [
             return type;
         };
 
+        /**
+         * Returns whether this drawing model is in visible state (the 'hidden'
+         * attribute is not set to true).
+         *
+         * @returns {Boolean}
+         *  Whether this drawing model is in visible state.
+         */
+        this.isVisible = function () {
+            return !this.getMergedAttributes().drawing.hidden;
+        };
+
         this.getCollection = function () {
             return null;
         };
@@ -80,7 +102,7 @@ define('io.ox/office/drawinglayer/model/drawingmodel', [
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initAttributes = initOptions = null;
+            docModel = initAttributes = initOptions = null;
         });
 
     } // class DrawingModel
