@@ -6,14 +6,15 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/drawinglayer/model/imagemodel', [
+    'io.ox/office/editframework/utils/border',
     'io.ox/office/drawinglayer/model/drawingmodel'
-], function (DrawingModel) {
+], function (Border, DrawingModel) {
 
     'use strict';
 
@@ -26,23 +27,23 @@ define('io.ox/office/drawinglayer/model/imagemodel', [
      *
      * @extends DrawingModel
      *
-     * @param {EditApplication} app
-     *  The application instance containing this image object.
+     * @param {EditModel} docModel
+     *  The document model containing this drawing object.
      *
      * @param {Object} [initAttributes]
      *  An attribute set with initial formatting attributes for the object.
      */
-    function ImageModel(app, initAttributes) {
+    function ImageModel(docModel, initAttributes) {
 
         // base constructor ---------------------------------------------------
 
-        DrawingModel.call(this, app, 'image', initAttributes, { families: 'image fill line' });
+        DrawingModel.call(this, docModel, 'image', initAttributes, { families: 'image fill line' });
 
         // initialization -----------------------------------------------------
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initAttributes = null;
+            docModel = initAttributes = null;
         });
 
     } // class ImageModel
