@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -26,23 +26,23 @@ define('io.ox/office/drawinglayer/model/shapemodel', [
      *
      * @extends DrawingModel
      *
-     * @param {EditApplication} app
-     *  The application instance containing this shape object.
+     * @param {EditModel} docModel
+     *  The document model containing this drawing object.
      *
      * @param {Object} [initAttributes]
      *  An attribute set with initial formatting attributes for the object.
      */
-    function ShapeModel(app, initAttributes) {
+    function ShapeModel(docModel, initAttributes) {
 
         // base constructor ---------------------------------------------------
 
-        DrawingModel.call(this, app, 'shape', initAttributes, { families: 'shape fill line' });
+        DrawingModel.call(this, docModel, 'shape', initAttributes, { families: 'shape fill line' });
 
         // initialization -----------------------------------------------------
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initAttributes = null;
+            docModel = initAttributes = null;
         });
 
     } // class ShapeModel
