@@ -6,47 +6,48 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
-define('io.ox/office/drawinglayer/view/chartstyleutil',
-        ['io.ox/office/tk/utils',
-         'io.ox/office/editframework/utils/color',
-         'gettext!io.ox/office/drawing'
-        ], function (Utils, Color, gt) {
+define('io.ox/office/drawinglayer/view/chartstyleutil', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/utils/dateutils',
+    'io.ox/office/editframework/utils/color',
+    'gettext!io.ox/office/drawinglayer/main'
+], function (Utils, DateUtils, Color, gt) {
 
     'use strict';
 
-    var BACKGROUNDTRANSFORMATION = [{type: 'satMod', value: 25000}, {type: 'lumOff', value: 35000}];
+    var BACKGROUNDTRANSFORMATION = [{ type: 'satMod', value: 25000 }, { type: 'lumOff', value: 35000 }];
 
     var STYLESET = [{
-        bg: {type: 'scheme', value: 'light1'},
-        bevelEnabled: false
-    }, {
-        bg: {type: 'scheme', value: 'light1'},
-        bevelEnabled: false
-    }, {
-        bg: {type: 'scheme', value: 'light1'},
-        bevelEnabled: false
-    }, {
-        bg: {type: 'scheme', value: 'light1'},
-        bevelEnabled: true
-    }, {
-        bg: null,
-        bevelEnabled: false
-    }, {
-        bg: {type: 'scheme', value: 'background1', transformations: [{type: 'lumOff', value: -80000}]},
-        bevelEnabled: true
-    }];
+            bg: { type: 'scheme', value: 'light1' },
+            bevelEnabled: false
+        }, {
+            bg: { type: 'scheme', value: 'light1' },
+            bevelEnabled: false
+        }, {
+            bg: { type: 'scheme', value: 'light1' },
+            bevelEnabled: false
+        }, {
+            bg: { type: 'scheme', value: 'light1' },
+            bevelEnabled: true
+        }, {
+            bg: null,
+            bevelEnabled: false
+        }, {
+            bg: { type: 'scheme', value: 'text1', transformations: [{ type: 'lumOff', value: -80000 }] },
+            bevelEnabled: true
+        }];
 
     var COLORSET = [];
 
     (function () {
         var allAccents = [];
         for (var i = 1; i <= 6; i++) {
-            allAccents.push({type: 'scheme', value: 'accent' + i});
+            allAccents.push({ type: 'scheme', value: 'accent' + i });
         }
 
         function getAccentColorSetName(accent) {
@@ -62,27 +63,21 @@ define('io.ox/office/drawinglayer/view/chartstyleutil',
 
         COLORSET.push({
             name: /*#. predefined formatting style for chart objects: black/gray/white only */ gt.pgettext('chart-format', 'Grayscale'),
-            colors: [{type: 'rgb', value: '555555'}, {type: 'rgb', value: '9E9E9E'}, {type: 'rgb', value: '727272'}, {type: 'rgb', value: '464646'}, {type: 'rgb', value: '838383'}, {type: 'rgb', value: 'C1C1C1'}],
-            variations: []
+            colors: [{ type: 'rgb', value: '555555' }, { type: 'rgb', value: '9E9E9E' }, { type: 'rgb', value: '727272' }, { type: 'rgb', value: '464646' }, { type: 'rgb', value: '838383' }, { type: 'rgb', value: 'C1C1C1' }],
+            type: 'group'
         });
 
         COLORSET.push({
             name: /*#. predefined formatting style for chart objects: all accent colors from the current color scheme */ gt.pgettext('chart-format', 'Accents'),
             colors: allAccents,
-            variations: [{transformations: [{type: 'lumOff', value: -1000}]}]
+            type: 'group'
         });
 
-        var accentVariations = [];//[{transformations: [{type: 'lumOff', value: -10000}]}, null, {transformations: [{type: 'lumOff', value: 10000}]}, {transformations: [{type: 'lumOff', value: 20000}]}, {transformations: [{type: 'lumOff', value: 30000}]}, {transformations: [{type: 'lumOff', value: 40000}]}];
-
-        for (var i = -2; i <= 3; i++) {
-            accentVariations.push({transformations: [{type: 'lumOff', value: i * 8000}]});
-        }
-
         _.each(allAccents, function (color, index) {
             COLORSET.push({
                 name: getAccentColorSetName(index + 1),
                 colors: [color],
-                variations: accentVariations
+                type: 'single'
             });
         });
     })();
@@ -115,35 +110,33 @@ define('io.ox/office/drawinglayer/view/chartstyleutil',
         return BACKGROUNDTRANSFORMATION;
     };
 
-    function getCorrectLineColor(chart, styles) {
-        var attrs = chart.getMergedAttributes();
+    function getCorrectLineColor(docModel, chartModel) {
+        var attrs = chartModel.getMergedAttributes();
         var bg = attrs.fill.color;
-        if (bg.type === 'auto') {
-            return Color.BLACK;
+        if (Color.parseJSON(bg).isAuto()) {
+            chartModel.updateBackgroundColor();
+            attrs = chartModel.getMergedAttributes();
+            bg = attrs.fill.color;
         }
+        var colorDesc = docModel.resolveColor(Color.parseJSON(bg), 'line');
         if (!_.isString(bg.fallbackValue)) {
-            bg.fallbackValue = styles.getCssColor(bg);
-        }
-        if (Color.isDark(bg.fallbackValue)){
-            return Color.WHITE;
-        } else {
-            return Color.BLACK;
+            bg.fallbackValue = colorDesc.hex;
         }
+        return { type: 'preset', value: colorDesc.dark ? 'white' : 'black' };
     }
 
     /**
      * sets all character-model-properties to the target,
      * target is a CanvasJS-data-model
-     * @app {BaseApplication}
-     * @character {object} character-model comes from operations
-     * @target {object} target-object is a inner object of CanvasJS
-     * @prefix {object} necessary if there are more than one character-holder on the target object
-     *
+     * @param {Object} character character-model comes from operations
+     * @param {Object} target target-object is a inner object of CanvasJS
+     * @param {Object} prefix necessary if there are more than one character-holder on the target object
      */
-    ChartStyleUtil.handleCharacterProps = function (chart, character, target, prefix) {
-        var app = chart.getApp();
-        var view = app.getView();
-        var styles = app.getModel().getDocumentStyles();
+    ChartStyleUtil.handleCharacterProps = function (chartModel, character, target, prefix) {
+
+        var app = chartModel.getApp();
+        var docModel = app.getModel();
+        var docView = app.getView();
 
         var usePrefix;
         if (prefix && prefix.length) {
@@ -152,9 +145,9 @@ define('io.ox/office/drawinglayer/view/chartstyleutil',
             usePrefix = 'font';
         }
 
-        target[usePrefix + 'Color'] = styles.getCssColor(character.color, getCorrectLineColor(chart, styles));
-        target[usePrefix + 'Family'] = styles.getCssFontFamily(character.fontName);
-        target[usePrefix + 'Size'] = view.getEffectiveZoom() * Utils.convertLength(character.fontSize, 'pt', 'px');
+        target[usePrefix + 'Color'] = docModel.getCssColor(character.color, getCorrectLineColor(docModel, chartModel));
+        target[usePrefix + 'Family'] = docModel.getCssFontFamily(character.fontName);
+        target[usePrefix + 'Size'] = docView.getZoomFactor() * Utils.convertLength(character.fontSize, 'pt', 'px');
         target[usePrefix + 'Weight'] = character.bold ? 'bold' : 'normal';
         target[usePrefix + 'Style'] = character.italic ? 'italic' : 'normal';
     };
@@ -167,22 +160,23 @@ define('io.ox/office/drawinglayer/view/chartstyleutil',
      * @target {object} target-object is a inner object of CanvasJS
      * @prefix {object} necessary for more than one character-holder on the target object
      */
-    ChartStyleUtil.handleLineProps = function (chart, line, target, prefix) {
-        var app = chart.getApp();
-        var view = app.getView();
-        var styles = app.getModel().getDocumentStyles();
-        var w = line.width ? line.width*(72/2540) : 1;
+    ChartStyleUtil.handleLineProps = function (chartModel, line, target, prefix) {
+
+        var app = chartModel.getApp();
+        var docModel = app.getModel();
+        var docView = app.getView();
+        var w = line.width ? line.width * (72 / 2540) : 1;
         var usePrefix = prefix;
 
         if (line.type === 'none') {
             target[usePrefix + 'Color'] = 'transparent';
         } else {
-            target[usePrefix + 'Color'] = styles.getCssColor(line.color, getCorrectLineColor(chart, styles));
-            target[usePrefix + 'Thickness'] = Math.max(1, Math.round(view.getEffectiveZoom() * w));
+            target[usePrefix + 'Color'] = docModel.getCssColor(line.color, getCorrectLineColor(docModel, chartModel));
+            target[usePrefix + 'Thickness'] = Math.max(1, Math.round(docView.getZoomFactor() * w));
         }
     };
 
-    ChartStyleUtil.getNumberFormat = function (app, value) {
+    ChartStyleUtil.getNumberFormat = function (model, value) {
         if (!value) {
             return;
         }
@@ -193,9 +187,9 @@ define('io.ox/office/drawinglayer/view/chartstyleutil',
 
         var format = attrs.cell.numberFormat.code;
         if (!format || !format.length) {
-            if(_.isNumber(attrs.cell.numberFormat.id)) {
-                format = app.getModel().getNumberFormatter().getBuiltInFormatCode(attrs.cell.numberFormat.id);
-            }else{
+            if (_.isNumber(attrs.cell.numberFormat.id)) {
+                format = model.getNumberFormatter().getBuiltInFormatCode(attrs.cell.numberFormat.id);
+            } else {
                 format = null;
             }
         }
@@ -206,6 +200,10 @@ define('io.ox/office/drawinglayer/view/chartstyleutil',
         return format;
     };
 
+    ChartStyleUtil.isDate = function (format) {
+        return DateUtils.isDateFormat(format) || DateUtils.isTimeFormat(format);
+    };
+
     // exports ================================================================
 
     return ChartStyleUtil;
