@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
@@ -17,7 +17,7 @@ define('io.ox/office/drawinglayer/view/control/chartcolorsetpicker', [
     'io.ox/office/tk/forms',
     'io.ox/office/tk/control/radiolist',
     'io.ox/office/drawinglayer/view/chartstyleutil',
-    'gettext!io.ox/office/drawing'
+    'gettext!io.ox/office/drawinglayer/main'
 ], function (Utils, Forms, RadioList, ChartStyleUtil, gt) {
 
     'use strict';
@@ -37,13 +37,13 @@ define('io.ox/office/drawinglayer/view/control/chartcolorsetpicker', [
      *
      * @extends RadioList
      */
-    function ChartColorSetPicker(app, initOptions) {
+    function ChartColorSetPicker(docView, initOptions) {
 
         var // self reference
             self = this,
 
-            // the document style collection
-            documentStyles = null;
+            // the document model
+            docModel = docView.getDocModel();
 
         // base constructor ---------------------------------------------------
 
@@ -75,12 +75,17 @@ define('io.ox/office/drawinglayer/view/control/chartcolorsetpicker', [
             _.times(COLOR_BOX_COUNT, function (index) {
 
                 var // extract the base color from the color set
-                    color = _.copy(colorSet.colors[index % colorSet.colors.length], true),
-                    // extract the variation from the color set
-                    variation = (colorSet.variations.length > 0) ? colorSet.variations[index % colorSet.variations.length] : null;
+                    color = _.copy(colorSet.colors[index % colorSet.colors.length], true);
 
-                if (variation) { color.transformations = variation.transformations; }
-                markup += '<span class="color-node" style="background-color:' + documentStyles.getCssColor(color, 'fill') + ';"></span>';
+                if (colorSet.type === 'single') {
+                    if (index === 0) {
+                        color.transformations = [{ type: 'shade', value: 80000 }];
+                    } else if (index === 2) {
+                        color.transformations = [{ type: 'tint', value: 80000 }];
+                    }
+                }
+
+                markup += '<span class="color-node" style="background-color:' + docModel.getCssColor(color, 'fill') + ';"></span>';
             });
             markup += '</div>';
 
@@ -103,20 +108,15 @@ define('io.ox/office/drawinglayer/view/control/chartcolorsetpicker', [
         this.getMenuNode().addClass('color-set-picker');
         this.getMenu().on('create:item', createItemHandler);
 
-        // initialize class members
-        app.onInit(function () {
-            documentStyles = app.getModel().getDocumentStyles();
-        });
-
         // insert color buttons in the drop-down menu after import and changed themes
-        this.listenTo(app.getImportPromise(), 'done', function () {
+        this.listenTo(docView.getApp().getImportPromise(), 'done', function () {
             initializeColorTable();
-            self.listenTo(app.getModel().getThemeCollection(), 'triggered', initializeColorTable);
+            self.listenTo(docModel.getThemeCollection(), 'triggered', initializeColorTable);
         });
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initOptions = self = documentStyles = null;
+            self = initOptions = docModel = docView = null;
         });
 
     } // class ChartColorSetPicker
