@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,7 +14,7 @@
 define('io.ox/office/drawinglayer/view/control/chartlegendpicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/radiolist',
-    'gettext!io.ox/office/drawing'
+    'gettext!io.ox/office/drawinglayer/main'
 ], function (Utils, RadioList, gt) {
 
     'use strict';
@@ -29,7 +29,7 @@ define('io.ox/office/drawinglayer/view/control/chartlegendpicker', [
      * @extends RadioList
      *
      * @param {Object} [initOptions]
-     *  Additional options. Supports all options of the base class RadioList.
+     *  Optional parameters. Supports all options of the base class RadioList.
      */
     function ChartLegendPicker(initOptions) {
 
