@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
@@ -18,7 +18,7 @@ define('io.ox/office/drawinglayer/view/control/charttypepicker', [
     'io.ox/office/tk/control/radiolist',
     'io.ox/office/drawinglayer/view/drawinglabels',
     'io.ox/office/drawinglayer/lib/canvasjs.min',
-    'gettext!io.ox/office/drawing'
+    'gettext!io.ox/office/drawinglayer/main'
 ], function (Utils, Forms, RadioList, Labels, CanvasJS, gt) {
 
     'use strict';
@@ -35,11 +35,14 @@ define('io.ox/office/drawinglayer/view/control/charttypepicker', [
      *
      * @extends RadioList
      */
-    function ChartTypePicker(app, initOptions) {
+    function ChartTypePicker(docView, initOptions) {
 
         var // self reference
             self = this,
 
+            // the document model
+            docModel = docView.getDocModel(),
+
             // data-model for CanvasJS to drop all changed value before drawing
             data = {},
 
@@ -74,9 +77,7 @@ define('io.ox/office/drawinglayer/view/control/charttypepicker', [
          */
         var updatePreviewCharts = this.createDebouncedMethod($.noop, function () {
 
-            var // the style collection of the document
-                documentStyles = app.getModel().getDocumentStyles(),
-                // unique identifier used by CanvasJS
+            var // unique identifier used by CanvasJS
                 id = _.uniqueId('io-ox-office-chart-dummy-'),
                 // container node for the CanvasJS chart
                 containerNode = $('<div id="' + id + '" style="position:fixed;visibility:hidden;left:-100px;top:-100px;width:100px;height:100px;">').appendTo('body'),
@@ -100,13 +101,13 @@ define('io.ox/office/drawinglayer/view/control/charttypepicker', [
                 _.each(data.data, function (dataSeries, index) {
                     dataSeries.type = cjs;
                     var cl = { type: 'scheme', value: 'accent' + (index + 1) };
-                    dataSeries.color = documentStyles.getCssColor(cl, 'fill');
+                    dataSeries.color = docModel.getCssColor(cl, 'fill');
                 });
 
                 _.each(data.data[0].dataPoints, function (dataPoint, index) {
                     if (single) {
                         var cl = { type: 'scheme', value: 'accent' + (index + 1) };
-                        dataPoint.color = documentStyles.getCssColor(cl, 'fill');
+                        dataPoint.color = docModel.getCssColor(cl, 'fill');
                     } else {
                         dataPoint.color = null;
                     }
@@ -151,7 +152,7 @@ define('io.ox/office/drawinglayer/view/control/charttypepicker', [
                 var dataPoints = [];
                 data.data[series] = { markerSize: 2, indexLabelFontColor: 'transparent', indexLabelLineColor: 'transparent', showInLegend: false, dataPoints: dataPoints, sourcePoints: {}};
                 _.each(points, function (point, index) {
-                    dataPoints.push({ label: 'bla ' + (index + 1), y: point, z: (2 + Math.random() * 100)|0});
+                    dataPoints.push({ label: 'bla ' + (index + 1), y: point, z: Math.floor(2 + Math.random() * 100) });
                 });
             });
 
@@ -173,8 +174,11 @@ define('io.ox/office/drawinglayer/view/control/charttypepicker', [
         this.getMenu().one('popup:beforeshow', updatePreviewCharts);
 
         // refresh preview charts in the drop-down menu after changed theme (color scheme)
-        this.listenTo(app.getImportPromise(), 'done', function () {
-            self.listenTo(app.getModel().getThemeCollection(), 'triggered', updatePreviewCharts);
+        this.listenTo(docModel.getThemeCollection(), 'triggered', updatePreviewCharts);
+
+        // destroy class members on destruction
+        this.registerDestructor(function () {
+            self = initOptions = docModel = docView = data = null;
         });
 
     } // class ChartTypePicker
