@@ -6,23 +6,24 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/drawinglayer/view/drawingframe',
-    ['io.ox/core/date',
-     'io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/io',
-     'io.ox/office/tk/canvaswrapper',
-     'io.ox/office/editframework/utils/attributeutils',
-     'io.ox/office/drawinglayer/view/drawinglabels',
-     'io.ox/office/drawinglayer/lib/canvasjs.min',
-     'io.ox/office/editframework/utils/border',
-     'less!io.ox/office/drawinglayer/view/drawingstyle'
-    ], function (CoreDate, Utils, Forms, IO, CanvasWrapper, AttributeUtils, Labels, CanvasJS, Border) {
+define('io.ox/office/drawinglayer/view/drawingframe', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/io',
+    'io.ox/office/tk/utils/dateutils',
+    'io.ox/office/tk/locale/localedata',
+    'io.ox/office/tk/canvas/canvas',
+    'io.ox/office/editframework/utils/attributeutils',
+    'io.ox/office/drawinglayer/view/drawinglabels',
+    'io.ox/office/drawinglayer/lib/canvasjs.min',
+    'io.ox/office/editframework/utils/border',
+    'less!io.ox/office/drawinglayer/view/drawingstyle'
+], function (Utils, Forms, IO, DateUtils, LocaleData, Canvas, AttributeUtils, Labels, CanvasJS, Border) {
 
     'use strict';
 
@@ -133,13 +134,6 @@ define('io.ox/office/drawinglayer/view/drawingframe',
      */
     DrawingFrame.NODE_SELECTOR = '.' + DrawingFrame.NODE_CLASS;
 
-    /**
-     * A jQuery selector that matches nodes representing a content element.
-     *
-     * @constant
-     */
-    DrawingFrame.CONTENT_SELECTOR = 'div.' + CONTENT_CLASS;
-
     /**
      * The CSS class used to mark drawing content nodes as place holder.
      *
@@ -156,19 +150,11 @@ define('io.ox/office/drawinglayer/view/drawingframe',
     DrawingFrame.PLACEHOLDER_SELECTOR = 'div.' + DrawingFrame.PLACEHOLDER_CLASS;
 
     /**
-     * The CSS class used to mark drawing content nodes as container for further drawings.
+     * A jQuery selector that matches content nodes of group drawing objects.
      *
      * @constant
      */
-    DrawingFrame.GROUPCONTENT_CLASS = 'groupcontent';
-
-    /**
-     * A jQuery selector that matches nodes representing a group content element
-     * of type 'div'.
-     *
-     * @constant
-     */
-    DrawingFrame.GROUPCONTENT_SELECTOR = 'div.' + DrawingFrame.GROUPCONTENT_CLASS;
+    DrawingFrame.GROUPCONTENT_SELECTOR = DrawingFrame.NODE_SELECTOR + '[data-type="group"]>.' + CONTENT_CLASS;
 
     /**
      * The CSS class used to mark drawing content nodes inside text frames
@@ -210,14 +196,6 @@ define('io.ox/office/drawinglayer/view/drawingframe',
      */
     DrawingFrame.DATA_UID = 'uid';
 
-    /**
-     * The name of the jQuery data attribute that stores the object type of a
-     * drawing frame node.
-     *
-     * @constant
-     */
-    DrawingFrame.DATA_TYPE = 'type';
-
     /**
      * The name of the jQuery data attribute that stores the drawing model
      * instance for a drawing frame node.
@@ -300,31 +278,38 @@ define('io.ox/office/drawinglayer/view/drawingframe',
     // methods ----------------------------------------------------------------
 
     /**
-     * Returns a new drawing frame node.
+     * Returns a new drawing frame node without specific contents.
      *
      * @param {String} type
      *  The type of the drawing.
      *
      * @param {DrawingModel|String} model
-     *  The drawing model instance this drawing frame is based on.
-     *  Alternatively (for applications not supporting drawing models), this
-     *  value can be a string with the type of the drawing object.
+     *  The drawing model the new drawing frame will be based on. Alternatively
+     *  (for applications not supporting drawing models), this value can be a
+     *  string with the type of the drawing object.
      *
      * @returns {jQuery}
      *  A new drawing frame with empty content node, as jQuery object.
      */
     DrawingFrame.createDrawingFrame = function (model) {
 
-        var // whether the drawing frame is a drawing group
-            isGroup = (_.isString(model) && model === 'group') || (_.isObject(model) && model.getType() === 'group');
+        var // the drawing type identifier
+            type = _.isObject(model) ? model.getType() : model,
+            // whether the drawing object is a group
+            isGroup = type === 'group',
+            // the content node
+            contentNode = $('<div class="' + CONTENT_CLASS + '">'),
+            // the resulting drawing frame node
+            drawingFrame = $('<div class="' + DrawingFrame.NODE_CLASS + '" data-type="' + type + '" contenteditable="' + (isGroup && !_.browser.IE) + '">');
 
-        return $('<div>', { contenteditable: (isGroup && !_.browser.IE) ? true : false })
-            .addClass(DrawingFrame.NODE_CLASS)
+        // add data attributes, event handlers, and the content node
+        drawingFrame
             .data(DrawingFrame.DATA_UID, 'frame' + _.uniqueId())
-            .data(DrawingFrame.DATA_TYPE, _.isObject(model) ? model.getType() : model)
             .data(DrawingFrame.DATA_MODEL, _.isObject(model) ? model : null)
             .on('dragstart', false)
-            .append($('<div>').addClass(CONTENT_CLASS).addClass(isGroup ? DrawingFrame.GROUPCONTENT_CLASS : ''));
+            .append(contentNode);
+
+        return drawingFrame;
     };
 
     /**
@@ -353,7 +338,7 @@ define('io.ox/office/drawinglayer/view/drawingframe',
      *  has been created.
      */
     DrawingFrame.getDrawingType = function (drawingFrame) {
-        return $(drawingFrame).first().data(DrawingFrame.DATA_TYPE);
+        return $(drawingFrame).first().attr('data-type');
     };
 
     /**
@@ -367,7 +352,7 @@ define('io.ox/office/drawinglayer/view/drawingframe',
      *  Whether the passed node is a content node inside a drawing frame.
      */
     DrawingFrame.isDrawingContentNode = function (node) {
-        return $(node).is(DrawingFrame.CONTENT_SELECTOR);
+        return $(node).is('.' + CONTENT_CLASS);
     };
 
     /**
@@ -383,7 +368,7 @@ define('io.ox/office/drawinglayer/view/drawingframe',
      *  no content.
      */
     DrawingFrame.isEmptyDrawingContentNode = function (node) {
-        return $(node).is(DrawingFrame.CONTENT_SELECTOR) && $(node).children().not(DrawingFrame.BORDER_NODE_SELECTOR).length === 0;
+        return DrawingFrame.isDrawingContentNode(node) && $(node).children().not(DrawingFrame.BORDER_NODE_SELECTOR).length === 0;
     };
 
     /**
@@ -694,6 +679,21 @@ define('io.ox/office/drawinglayer/view/drawingframe',
         return $(node).is(DrawingFrame.TEXTFRAME_NODE_SELECTOR);
     };
 
+    /**
+     * Returns whether the passed node is a text frame content <div> element, that is
+     * used inside drawing frames
+     *
+     * @param {Node|jQuery|Null} [node]
+     *  The DOM node to be checked. If this object is a jQuery collection, uses
+     *  the first DOM node it contains. If missing or null, returns false.
+     *
+     * @returns {Boolean}
+     *  Whether the passed node is a div textframecontent element.
+     */
+    DrawingFrame.isTextFrameContentNode = function (node) {
+        return $(node).is(DrawingFrame.TEXTFRAMECONTENT_NODE_SELECTOR);
+    };
+
     /**
      * Returns whether the passed node is a drawing node, that is a text frame, that
      * automatically resizes its height. In this case the content node contains a
@@ -801,23 +801,6 @@ define('io.ox/office/drawinglayer/view/drawingframe',
         return $(drawingFrame).first().children('.' + CONTENT_CLASS);
     };
 
-    /**
-     * Appends a new border canvas element to the DOM and returns it.
-     *
-     * @param {HTMLElement|jQuery} drawingFrame
-     *  The root node of the drawing frame. If this object is a jQuery
-     *  collection, uses the first DOM node it contains.
-     *
-     * @returns {jQuery}
-     *  A new CanvasWrapper element for the given drawigFrame.
-     */
-    DrawingFrame.addBorderNode = function (drawingFrame, canvas) {
-        var contentNode = DrawingFrame.getContentNode(drawingFrame);
-
-        contentNode.prepend(canvas);
-        return DrawingFrame;
-    };
-
     /**
      * Returns the border node (canvas) of the specified drawing frame
      *
@@ -832,6 +815,20 @@ define('io.ox/office/drawinglayer/view/drawingframe',
         return $(drawingFrame).find(DrawingFrame.BORDER_NODE_SELECTOR);
     };
 
+    /**
+     * Returns whether the specified node is a border node (canvas)
+     *
+     * @param {HTMLElement|jQuery} drawingFrame
+     *  The root node of the drawing frame. If this object is a jQuery
+     *  collection, uses the first DOM node it contains.
+     *
+     * @returns {jQuery}
+     *  Whether the specified node is a border node.
+     */
+    DrawingFrame.isBorderNode = function (node) {
+        return $(node).is(DrawingFrame.BORDER_NODE_SELECTOR);
+    };
+
     /**
      * Checks, whether the passed node contains a border element
      *
@@ -861,6 +858,70 @@ define('io.ox/office/drawinglayer/view/drawingframe',
         return DrawingFrame.getContentNode(drawingFrame).empty().removeClass(DrawingFrame.PLACEHOLDER_CLASS).attr('style', '');
     };
 
+    /**
+     * Increasing the height of a group drawing frame, that contains text frames
+     * with auto resize height functionality. Those text frames expand automatically
+     * the height of the drawing, if required.
+     *
+     * @param {HTMLElement|jQuery} drawingNode
+     *  The text frame drawing node with auto resize height functionality.
+     */
+    DrawingFrame.updateDrawingGroupHeight = function (drawingNode) {
+
+        // comparing lower border of auto resize text frame node with the group
+
+        var // the offset of the bottom border in pixel
+            drawingBottom = 0,
+            // an optional group node surrounding the specified text frame drawing node
+            groupNode = null,
+            // the offset of the group border in pixel
+            groupBottom = 0,
+            // the new height of the group drawing node
+            newHeight = 0,
+            // whether the group iteration need to be continued
+            doContinue = true;
+
+        // helper function to find the pixel positions of all bottom borders inside a group
+        function getLowestChildDrawingBottom(node) {
+
+            var // the lowest bottom pixel position of all child drawings inside the specified node
+                lowestBottom = null;
+
+            // iterating over all children of the specified (group) node
+            _.each(node.children(), function (oneDrawing) {
+
+                var // the bottom pixel position of one child node
+                    localBottom = Utils.round($(oneDrawing).offset().top + $(oneDrawing).height(), 1);
+
+                // the larger the value, the deeper the drawing
+                if (lowestBottom === null || localBottom > lowestBottom) { lowestBottom = localBottom; }
+            });
+
+            return lowestBottom;
+        }
+
+        // iterating over all groups
+        while (doContinue && DrawingFrame.isGroupedDrawingFrame(drawingNode)) {
+
+            // drawingBottom = $(drawingNode).offset().top + $(drawingNode).height();
+            groupNode = DrawingFrame.getGroupNode(drawingNode);
+            groupBottom = groupNode ? Utils.round(groupNode.offset().top + groupNode.height(), 1) : null;
+
+            drawingBottom = getLowestChildDrawingBottom($(drawingNode).parent());
+
+            // comparing the bottom borders of group and specified text frame
+            if (_.isNumber(groupBottom) && _.isNumber(drawingBottom) && (groupBottom !== drawingBottom)) {
+                // increasing or decreasing the height of the group without operation
+                newHeight = Utils.round(groupNode.height() + (drawingBottom - groupBottom), 1);
+                groupNode.height(newHeight);
+                drawingNode = groupNode;
+            } else {
+                doContinue = false;
+            }
+
+        }
+    };
+
     /**
      * Prepares a specified drawing frame for text insertion. This requires
      * that the content node inside drawing frame is cleared and then a new
@@ -968,7 +1029,9 @@ define('io.ox/office/drawinglayer/view/drawingframe',
      */
     DrawingFrame.updateFormatting = function (app, drawingFrame, mergedAttributes) {
 
-        var // the drawing attributes of the passed attribute set
+        var // the document model
+            docModel = app.getModel(),
+            // the drawing attributes of the passed attribute set
             drawingAttributes = mergedAttributes.drawing,
             // value of the 'transform' CSS attribute, e.g. for flipped drawings
             transformations = '',
@@ -981,16 +1044,14 @@ define('io.ox/office/drawinglayer/view/drawingframe',
             // rendering result (false to fall-back to placeholder graphic)
             rendered = false;
 
-        // forehanded, remove class which marks this drawing as unsupported
+        // first remove the class which marks this drawing as unsupported
         drawingFrame.removeClass(DrawingFrame.UNSUPPORTED_CLASS);
 
-        // helper function to get all node attributes (TODO: This needs to be valid for all apps)
+        // helper function to get all node attributes (TODO: This needs to be valid for all applications)
         function getAllDrawingAttributes(node, family) {
 
-            var // the style collection of the document
-                documentStyles = app.getModel().getDocumentStyles(),
-                // the drawing style collection
-                styleCollection = documentStyles ? documentStyles.getStyleCollection('drawing') : null,
+            var // the drawing style collection
+                styleCollection = docModel.getStyleCollection('drawing'),
                 // the element specific attributes
                 elementAttributes = styleCollection ? styleCollection.getElementAttributes(node) : null,
                 // the family specific attributes of the element
@@ -1139,12 +1200,12 @@ define('io.ox/office/drawinglayer/view/drawingframe',
                 // localization for CanvasJS
                 if (!CULTURE_INFO) {
                     CULTURE_INFO = {
-                        decimalSeparator: app.getDecimalSeparator(),
-                        digitGroupSeparator: app.getGroupSeparator(),
-                        shortDays: CoreDate.locale.daysShort,
-                        days: CoreDate.locale.days,
-                        months: CoreDate.locale.months,
-                        shortMonths: CoreDate.locale.monthsShort
+                        decimalSeparator: LocaleData.DEC,
+                        digitGroupSeparator: LocaleData.GROUP,
+                        shortDays: DateUtils.getLocaleDaysShort(),
+                        days: DateUtils.getLocaleDays(),
+                        months: DateUtils.getLocaleMonths(),
+                        shortMonths: DateUtils.getLocaleMonthsShort()
                     };
                     CanvasJS.addCultureInfo('en', CULTURE_INFO);
                 }
@@ -1187,14 +1248,15 @@ define('io.ox/office/drawinglayer/view/drawingframe',
          *  for drawing groups, where all children need to be modified.
          */
         function updateFillFormatting() {
-            var // the CSS properties for the drawing frame
+
+            var // the document model
+                docModel = app.getModel(),
+                // the CSS properties for the drawing frame
                 cssProps = {},
                 // the node, whose properties will be set
                 cssNode = contentNode,
                 // the fill attributes
-                fillAttrs = mergedAttributes.fill,
-                // the style collection of the document
-                documentStyles = app.getModel().getDocumentStyles();
+                fillAttrs = mergedAttributes.fill;
 
             // calculate the CSS fill attributes
             switch (fillAttrs.type) {
@@ -1204,7 +1266,7 @@ define('io.ox/office/drawinglayer/view/drawingframe',
                 break;
             case 'solid':
                 // use 'background' compound attribute to clear bitmaps
-                cssProps.background = documentStyles.getCssColor(fillAttrs.color, 'fill');
+                cssProps.background = docModel.getCssColor(fillAttrs.color, 'fill');
                 break;
             default:
                 Utils.warn('DrawingFrame.updateFillFormatting(): unknown fill type "' + fillAttrs.type + '"');
@@ -1226,9 +1288,9 @@ define('io.ox/office/drawinglayer/view/drawingframe',
         function updateLineFormatting(node) {
 
             var // gets the correct node to work on (parent() used at grouped drawings)
-                currentNode = (node) ? DrawingFrame.getDrawingNode(node): drawingFrame,
+                currentNode = node ? DrawingFrame.getDrawingNode(node) : drawingFrame,
                 // the line attributes
-                lineAttrs = (node) ? AttributeUtils.getExplicitAttributes(currentNode).line : mergedAttributes.line;
+                lineAttrs = node ? AttributeUtils.getExplicitAttributes(currentNode).line : mergedAttributes.line;
 
             // Explicit attributes are not sufficient, if default line attributes are used (36892)
             if (node && (!lineAttrs || !lineAttrs.type)) { lineAttrs = getAllDrawingAttributes(node, 'line'); }
@@ -1270,6 +1332,8 @@ define('io.ox/office/drawinglayer/view/drawingframe',
             // setting the value for automatic vertical resizing of the shape.
             if (shapeAttrs.autoResizeHeight) {
                 contentNode.addClass(DrawingFrame.AUTORESIZEHEIGHT_CLASS);
+                // updating the height of a surrounding group node
+                if (DrawingFrame.isGroupedDrawingFrame(contentNode.parent())) { DrawingFrame.updateDrawingGroupHeight(contentNode.parent()); }
             } else {
                 contentNode.removeClass(DrawingFrame.AUTORESIZEHEIGHT_CLASS);
             }
@@ -1337,7 +1401,10 @@ define('io.ox/office/drawinglayer/view/drawingframe',
                 if (_.isNumber(drawingAttrs.left)) { cssAttrs.left = Utils.convertHmmToCssLength(drawingAttrs.left * horzResizeFactor, 'px', 1); }
                 if (_.isNumber(drawingAttrs.top)) { cssAttrs.top = Utils.convertHmmToCssLength(drawingAttrs.top * vertResizeFactor, 'px', 1); }
                 if (_.isNumber(drawingAttrs.width)) { cssAttrs.width = Utils.convertHmmToCssLength(drawingAttrs.width * horzResizeFactor, 'px', 1); }
-                if (_.isNumber(drawingAttrs.height)) { cssAttrs.height = Utils.convertHmmToCssLength(drawingAttrs.height * vertResizeFactor, 'px', 1); }
+
+                if (_.isNumber(drawingAttrs.height) && !DrawingFrame.isAutoResizeHeightDrawingFrame(drawing)) { // not setting height for auto resize drawings
+                    cssAttrs.height = Utils.convertHmmToCssLength(drawingAttrs.height * vertResizeFactor, 'px', 1);
+                }
 
                 drawing.css(cssAttrs);
 
@@ -1394,7 +1461,7 @@ define('io.ox/office/drawinglayer/view/drawingframe',
         drawingFrame = $(drawingFrame).first();
 
         // set text attribute to support tooltip for the drawing
-        Forms.setToolTip(drawingFrame, drawingAttributes.name);
+        Forms.setToolTip(drawingFrame, { tooltip: drawingAttributes.name });
 
         // vertically or horizontally flipped drawings
         if (drawingAttributes.flipH && drawingAttributes.flipV) {
@@ -1453,12 +1520,13 @@ define('io.ox/office/drawinglayer/view/drawingframe',
 
     // draws the border to the canvas
     DrawingFrame.drawBorder = function (app, currentNode, lineAttrs) {
-        var // the border node inside the drawing frame
+
+        var // the document model
+            docModel = app.getModel(),
+            // the border node inside the drawing frame
             borderNode = DrawingFrame.getBorderNode(currentNode),
             // the canvas wrapper of the border node
             borderCanvas = null,
-            // the style collection of the document
-            documentStyles = app.getModel().getDocumentStyles(),
             // border width
             lineWidth = Utils.convertHmmToLength(lineAttrs.width, 'px', 1),
             // line width to calculate the border correct
@@ -1470,68 +1538,52 @@ define('io.ox/office/drawinglayer/view/drawingframe',
             cTop = 0,
             cLeft = 0;
 
-        if (borderNode.length === 0) {
-            borderCanvas = new CanvasWrapper({classes: DrawingFrame.BORDER_CLASS});
-            borderNode = borderCanvas.getNode();
-            DrawingFrame.addBorderNode(currentNode, borderNode);
+        if (borderNode.length > 0) {
+            borderCanvas = new Canvas({ classes: DrawingFrame.BORDER_CLASS, node: borderNode });
         } else {
-            borderCanvas = new CanvasWrapper({classes: DrawingFrame.BORDER_CLASS, canvasElement: borderNode});
+            borderCanvas = new Canvas({ classes: DrawingFrame.BORDER_CLASS });
+            DrawingFrame.getContentNode(currentNode).prepend(borderCanvas.getNode());
         }
 
         // lineWidth cannot be "0" (min. "1")
         lineWidth = (lineWidth === 0) ? 1 : lineWidth;
 
         // calculate the dimension of the canvas
-        calcLineWidth = (lineWidth % 2 === 0) ? lineWidth : (lineWidth+1);
+        calcLineWidth = (lineWidth % 2 === 0) ? lineWidth : (lineWidth + 1);
 
         // calculate the position of the line in the canvas
         // special behavior for text frames. Their border covers their boundary
-        cWidth  = (DrawingFrame.isTextFrameShapeDrawingFrame(currentNode)) ? (currentNode.first().width() + calcLineWidth) : (currentNode.first().width() + (lineWidth*2));
-        cHeight = (DrawingFrame.isTextFrameShapeDrawingFrame(currentNode)) ? (currentNode.first().height() + calcLineWidth) : (currentNode.first().height() + (lineWidth*2));
-        cTop    = (DrawingFrame.isTextFrameShapeDrawingFrame(currentNode)) ? '-'+Math.ceil(lineWidth / 2)+'px' : '-'+lineWidth+'px';
-        cLeft   = (DrawingFrame.isTextFrameShapeDrawingFrame(currentNode)) ? '-'+Math.ceil(lineWidth / 2)+'px' : '-'+lineWidth+'px';
+        cWidth  = currentNode.first().width() + (DrawingFrame.isTextFrameShapeDrawingFrame(currentNode) ? calcLineWidth : (2 * lineWidth));
+        cHeight = currentNode.first().height() + (DrawingFrame.isTextFrameShapeDrawingFrame(currentNode) ? calcLineWidth : (2 * lineWidth));
+        cTop    = (DrawingFrame.isTextFrameShapeDrawingFrame(currentNode) ? -Math.ceil(lineWidth / 2) : -lineWidth) + 'px';
+        cLeft   = (DrawingFrame.isTextFrameShapeDrawingFrame(currentNode) ? -Math.ceil(lineWidth / 2) : -lineWidth) + 'px';
 
         // set dimension of the canvas element
-        borderCanvas.initialize({
-            top: 0,
-            left: 0,
-            width:  cWidth,
-            height: cHeight
-        });
+        borderCanvas.initialize({ width: cWidth, height: cHeight });
+
         // set position of the canvas element
-        borderCanvas.getNode().css({
-            top: cTop,
-            left: cLeft
-        });
+        borderCanvas.getNode().css({ top: cTop, left: cLeft });
 
         borderCanvas.render(function (context, width, height) {
-            var color = documentStyles.getCssColor(lineAttrs.color, 'line'),
+            var color = docModel.getCssColor(lineAttrs.color, 'line'),
                 pattern = Border.getBorderPattern(lineAttrs.style, lineWidth);
 
             context.setLineStyle({ style: color, width: lineWidth, pattern: pattern });
-            context.drawRect( (lineWidth / 2), (lineWidth / 2), (width - lineWidth), (height - lineWidth));
+            context.drawRect(lineWidth / 2, lineWidth / 2, width - lineWidth, height - lineWidth);
         });
     };
 
     // clears the canvas
     DrawingFrame.clearBorder = function (currentNode) {
-        var // the border node inside the drawing frame
+
+        var // the border canvas node inside the drawing frame
             borderNode = DrawingFrame.getBorderNode(currentNode),
             // the canvas wrapper of the border node
             borderCanvas = null;
 
         if (borderNode.length > 0) {
-            borderCanvas = new CanvasWrapper({classes: DrawingFrame.BORDER_CLASS, canvasElement: borderNode});
-        }
-
-        if (!_.isNull(borderCanvas)) {
-            borderCanvas.initialize({
-                top: 0,
-                left: 0,
-                width:  borderNode.width(),
-                height: borderNode.height()
-            });
-            borderCanvas.clear();
+            borderCanvas = new Canvas({ classes: DrawingFrame.BORDER_CLASS, node: borderNode });
+            borderCanvas.initialize({ width: borderNode.width(), height: borderNode.height() });
         }
     };
 
