@@ -6,13 +6,13 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/drawinglayer/view/drawinglabels', [
-    'gettext!io.ox/office/drawing'
+    'gettext!io.ox/office/drawinglayer/main'
 ], function (gt) {
 
     'use strict';
@@ -50,7 +50,7 @@ define('io.ox/office/drawinglayer/view/drawinglabels', [
         };
 
     // set missing Boolean properties explicitly to false
-    _.each(CHART_TYPE_STYLES, function(CHART_TYPE) {
+    _.each(CHART_TYPE_STYLES, function (CHART_TYPE) {
         CHART_TYPE.curved = CHART_TYPE.curved || false;
         CHART_TYPE.varyColors = CHART_TYPE.varyColors || false;
     });
@@ -91,6 +91,36 @@ define('io.ox/office/drawinglayer/view/drawinglabels', [
         return (DRAWING_TYPE_INFOS[type] || DRAWING_TYPE_DEFAULT_INFO).label;
     };
 
+    // constant labels and tooltips -------------------------------------------
+
+    /**
+     * Standard options for the 'Delete drawing' label.
+     *
+     * @constant
+     */
+    DrawingLabels.DELETE_DRAWING_LABEL = /*#. delete a drawing object from the document */ gt.pgettext('drawing', 'Delete');
+
+    /**
+     * Standard options for the 'Delete drawing' tooltip.
+     *
+     * @constant
+     */
+    DrawingLabels.DELETE_DRAWING_TOOLTIP = gt.pgettext('drawing', 'Delete the drawing object');
+
+    /**
+     * Standard options for the 'Insert drawing' label.
+     *
+     * @constant
+     */
+    DrawingLabels.INSERT_DRAWING_LABEL = DrawingLabels.getDrawingTypeLabel('image');
+
+    /**
+     * Standard options for the 'Insert drawing' tooltip.
+     *
+     * @constant
+     */
+    DrawingLabels.INSERT_DRAWING_TOOLTIP = gt.pgettext('drawing', 'Insert an image');
+
     // constants for controls -------------------------------------------------
 
     /**
@@ -100,8 +130,17 @@ define('io.ox/office/drawinglayer/view/drawinglabels', [
      */
     DrawingLabels.DELETE_DRAWING_BUTTON_OPTIONS = {
         icon: 'fa-trash-o',
-        // label: /*#. delete a drawing object from the document */ gt.pgettext('drawing', 'Delete'),
-        tooltip: gt.pgettext('drawing', 'Delete the drawing object')
+        tooltip: DrawingLabels.DELETE_DRAWING_TOOLTIP
+    };
+
+    /**
+     * Standard options for insert image.
+     *
+     * @constant
+     */
+    DrawingLabels.INSERT_IMAGE_OPTIONS = {
+        label: DrawingLabels.INSERT_DRAWING_LABEL,
+        tooltip: DrawingLabels.INSERT_DRAWING_TOOLTIP
     };
 
     /**
@@ -111,8 +150,8 @@ define('io.ox/office/drawinglayer/view/drawinglabels', [
      */
     DrawingLabels.INSERT_IMAGE_BUTTON_OPTIONS = {
         icon: 'docs-image-insert',
-        label: DrawingLabels.getDrawingTypeLabel('image'),
-        tooltip: gt.pgettext('drawing', 'Insert an image'),
+        label: DrawingLabels.INSERT_DRAWING_LABEL,
+        tooltip: DrawingLabels.INSERT_DRAWING_TOOLTIP,
         smallerVersion: {
             css: { width: 35 },
             hideLabel: true
@@ -120,20 +159,50 @@ define('io.ox/office/drawinglayer/view/drawinglabels', [
     };
 
     /**
-     * Standard options for the 'Insert image' button.
+     * Standard options for insert textframe.
+     *
+     * @constant
+     */
+    DrawingLabels.INSERT_TEXTFRAME_OPTIONS = {
+        label: DrawingLabels.getDrawingTypeLabel('textframe'),
+        tooltip: gt.pgettext('drawing', 'Insert a text frame')
+    };
+
+    /**
+     * Standard options for the 'Insert textframe' button.
      *
      * @constant
      */
     DrawingLabels.INSERT_TEXTFRAME_BUTTON_OPTIONS = {
         icon: DrawingLabels.getDrawingTypeIcon('textframe'),
-        label: DrawingLabels.getDrawingTypeLabel('textframe'),
-        tooltip: gt.pgettext('drawing', 'Insert a text frame'),
+        label: DrawingLabels.INSERT_TEXTFRAME_OPTIONS.label,
+        tooltip: DrawingLabels.INSERT_TEXTFRAME_OPTIONS.tooltip,
         smallerVersion: {
             css: { width: 35 },
             hideLabel: true
         }
     };
 
+    /**
+     * Standard options for insert comment.
+     *
+     * @constant
+     */
+    DrawingLabels.INSERT_COMMENT_OPTIONS = {
+        label: gt('Comment'),
+        tooltip: /*#. insert a comment into the text */ gt('Insert a comment')
+    };
+
+    /**
+     * Standard options for a 'Insert chart' button.
+     *
+     * @constant
+     */
+    DrawingLabels.INSERT_CHART_OPTIONS = {
+        label: DrawingLabels.getDrawingTypeLabel('chart'),
+        tooltip: gt.pgettext('drawing', 'Insert a chart')
+    };
+
     /**
      * Standard options for the 'Insert chart' button.
      *
@@ -142,9 +211,10 @@ define('io.ox/office/drawinglayer/view/drawinglabels', [
     DrawingLabels.INSERT_CHART_BUTTON_OPTIONS = {
         width: null,
         icon: DrawingLabels.getDrawingTypeIcon('chart'),
-        label: DrawingLabels.getDrawingTypeLabel('chart'),
-        tooltip: gt.pgettext('drawing', 'Insert a chart'),
-        updateCaptionMode: 'none'
+        label: DrawingLabels.INSERT_CHART_OPTIONS.label,
+        tooltip: DrawingLabels.INSERT_CHART_OPTIONS.tooltip,
+        updateCaptionMode: 'none',
+        value: 'column standard'
     };
 
     /**
@@ -171,6 +241,14 @@ define('io.ox/office/drawinglayer/view/drawinglabels', [
             gt.pgettext('chart-format', 'Vary colors by point')
     };
 
+    /**
+     * @constant
+     */
+    DrawingLabels.CHART_MARKER_ONLY_BUTTON_OPTIONS = {
+        label: //#. check box label: for line & scatter charts, show/hide lines between points (markers)
+            gt.pgettext('chart-format', 'Markers only')
+    };
+
     /**
      * @constant
      */
