@@ -6,25 +6,19 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
 define('io.ox/office/drawinglayer/view/imageutil', [
-    'io.ox/core/config',
-    'io.ox/core/tk/folderviews',
-    'io.ox/core/folder/api',
-    'io.ox/core/cache',
-    'io.ox/core/tk/selection',
-    'io.ox/files/api',
     'io.ox/office/tk/utils',
     'io.ox/office/tk/io',
-    'io.ox/office/tk/keycodes',
-    'io.ox/office/editframework/view/editdialogs',
+    'io.ox/office/tk/dialogs',
+    'io.ox/office/tk/utils/driveutils',
     'settings!io.ox/office',
-    'gettext!io.ox/office/main'
-], function (Config, FolderViews, FolderAPI, CoreCache, CoreSelection, FilesAPI, Utils, IO, KeyCodes, EditDialogs, Settings, gt) {
+    'gettext!io.ox/office/drawinglayer/main'
+], function (Utils, IO, Dialogs, DriveUtils, Settings, gt) {
 
     'use strict';
 
@@ -107,14 +101,11 @@ define('io.ox/office/drawinglayer/view/imageutil', [
         var dataLength = data.length - offset;
 
         // data length must be at least two bytes to encode one character (if no pad bytes are used)
-        if (dataLength < 2)
-            return null;
+        if (dataLength < 2) { return null; }
 
         // taking care of padbytes
-        if (data.charAt(offset + dataLength - 1) === '=')
-            dataLength--;
-        if (data.charAt(offset + dataLength - 1) === '=')
-            dataLength--;
+        if (data[offset + dataLength - 1] === '=') { dataLength--; }
+        if (data[offset + dataLength - 1] === '=') { dataLength--; }
 
         var groups = dataLength >> 2;
         var hangover = dataLength - (groups << 2);
@@ -153,97 +144,41 @@ define('io.ox/office/drawinglayer/view/imageutil', [
         return iCRC ^ 0xFFFFFFFF;
     }
 
+    function insertDataUrl(app, dataUrl, fileName) {
+        var offset = dataUrl && dataUrl.indexOf('base64,') + 7,
+            crc32 = null,
+            extension = null;
+
+        if (!_.isString(fileName) || (fileName.length < 1)) {
+            fileName = 'image.img';
+        }
+
+        crc32 = getCRC32fromBase64(dataUrl, offset);
+
+        // if no crc32 value could be determined, the loading process must be stopped now.
+        if (!crc32) { return $.Deferred().reject(); }
+
+        crc32 = toHex32(crc32);
+        extension = fileName.substring(fileName.lastIndexOf('.') + 1);
+
+        var params = {
+                add_crc32: crc32,
+                add_ext: extension,
+                alt_filename: fileName,
+                add_filedata: dataUrl,
+                requestdata: 'hallo'
+            };
+
+        return app.sendActionRequest('addfile', params);
+    }
+
     function insertFile(app, file) {
 
         return IO.readClientFileAsDataUrl(file).then(function (dataUrl) {
-
-            var offset = dataUrl && dataUrl.indexOf('base64,') + 7,
-                crc32 = null,
-                extension = null;
-
-            crc32 = getCRC32fromBase64(dataUrl, offset);
-
-            // if no crc32 value could be determined, the loading process must be stopped now.
-            if (!crc32) { return $.Deferred().reject(); }
-
-            crc32 = toHex32(crc32),
-            extension = file.name.substring(file.name.lastIndexOf('.') + 1);
-
-            var params = {
-                    action: 'addfile',
-                    add_crc32: crc32,
-                    add_ext: extension,
-                    alt_filename: file.name,
-                    add_filedata: dataUrl,
-                    requestdata: 'hallo'
-                };
-
-            return app.sendFileRequest(IO.FILTER_MODULE_NAME, params, { method: 'POST' });
+            return insertDataUrl(app, dataUrl, file.name);
         });
     }
 
-    /**
-     * Special handling for IE 9 for inserting the image from the specified
-     * file into a document.
-     *
-     * @param {ox.ui.App} app
-     *  The application object representing the edited document.
-     *
-     * @param {Object} file
-     *  The simulated file object describing the image. It contains only the
-     *  name.
-     *
-     * @param {Object} form
-     *  The form used to upload the file.
-     */
-    function insertFileIE9(app, file, form) {
-
-        var def = $.Deferred(),
-            name = 'formpost_' + _.now(),
-            description = '</body>',
-            extension = _.isString(file.name) ? file.name.substring(file.name.lastIndexOf('.') + 1) : '',
-            options = {
-                form: form,
-                data: { description: description },
-                field: 'json'
-            },
-            data = JSON.stringify(options.data);
-
-        $('#tmp').append(
-            $('<iframe>', { name: name, id: name, height: 1, width: 1, src: ox.base + '/blank.html' })
-        );
-
-        // callback function, that is executed after the file is completely uploaded.
-        // This is a process similar to the defereds, that are not supported by form submit.
-        // Instead the server writes some JavaScript code into the <iframe> to start this callback.
-        window.callback_addfile = function (response) {
-            window.callback_addfile = null;
-            $('#' + name).remove();
-            if (response && !response.error) {
-                def.resolve(response);
-            } else {
-                def.reject(response);
-            }
-        };
-
-
-        var action = app.getServerModuleUrl(IO.FILTER_MODULE_NAME, { action: 'addfile', add_ext: extension, alt_filename: file.name });
-
-        form.append(
-                $('<input type="hidden" name="' + options.field + '">').val(data)
-            ).attr({
-                method: 'post',
-                enctype: 'multipart/form-data',
-                action: action,
-                target: name
-            }).submit();
-
-        // extract the data object from the response
-        return def.then(IO.createDeferredFilter(function (response) {
-            return Utils.getObjectOption(response, 'data');
-        })).promise();
-    }
-
     /**
      * Inserts a file from OX Drive.
      *
@@ -261,14 +196,13 @@ define('io.ox/office/drawinglayer/view/imageutil', [
         if (!fileDescriptor) { return $.Deferred().reject().promise(); }
 
         var params = {
-                action: 'addfile',
                 add_ext: fileDescriptor.filename.substring(fileDescriptor.filename.lastIndexOf('.') + 1),
                 add_fileid: fileDescriptor.id,
                 add_folderid: fileDescriptor.folder_id,
                 alt_filename: fileDescriptor.filename
             };
 
-        return app.sendFileRequest(IO.FILTER_MODULE_NAME, params, { method: 'POST' });
+        return app.sendActionRequest('addfile', params);
     }
 
     // static class Image =====================================================
@@ -316,10 +250,6 @@ define('io.ox/office/drawinglayer/view/imageutil', [
      *
      * @param {String} uri
      *  A hierarchical uri
-     *
-     * @returns {Boolean}
-     *  TRUE if the uri references a file which has a supported image extensions,
-     *  otherwise FALSE.
      */
     Image.getFileNameFromImageUri = function (uri) {
 
@@ -413,6 +343,32 @@ define('io.ox/office/drawinglayer/view/imageutil', [
         delete imageAttrs.sessionId;
     };
 
+    /**
+     * Inserts the image from the specified data URL into a document.
+     *
+     * @param {ox.ui.App} app
+     *  The application object representing the edited document.
+     *
+     * @param {String} dataURL
+     *  The image, Base64 encoded as data URL.
+     *
+     * @returns {jQuery.Promise}
+     *  The Promise of a Deferred object that will be resolved if the image
+     *  has been uploaded to the server; or rejected, in case of an error.
+     */
+    Image.insertDataUrl = function (app, dataUrl, fileName) {
+        // the callback methods for insertFile
+        return insertDataUrl(app, dataUrl, fileName)
+        .then(function (data) {
+            return {
+                url: Utils.getStringOption(data, 'added_filename', ''),
+                name: Image.getFileNameWithoutExt(fileName || 'image.img')
+            };
+        }, function () {
+            app.getView().yell({ type: 'warning', message: gt('Image could not be loaded.') });
+        });
+    };
+
     /**
      * Inserts the image from the specified file into a document.
      *
@@ -422,16 +378,13 @@ define('io.ox/office/drawinglayer/view/imageutil', [
      * @param {File} file
      *  The file object describing the image file to be inserted.
      *
-     * @param {jQuery} form
-     *  The form node (required for IE 9 file upload).
-     *
      * @returns {jQuery.Promise}
      *  The Promise of a Deferred object that will be resolved if the image
      *  has been uploaded to the server; or rejected, in case of an error.
      */
-    Image.insertFile = function (app, file, form) {
-        // the callback methods for insertFile/insertFileIE9
-        return (form ? insertFileIE9(app, file, form) : insertFile(app, file))
+    Image.insertFile = function (app, file) {
+        // the callback methods for insertFile
+        return insertFile(app, file)
         .then(function (data) {
             return {
                 url: Utils.getStringOption(data, 'added_filename', ''),
@@ -464,7 +417,6 @@ define('io.ox/office/drawinglayer/view/imageutil', [
         });
     };
 
-
     /**
      * Inserts the image specified by a URL into a document.
      *
@@ -495,10 +447,11 @@ define('io.ox/office/drawinglayer/view/imageutil', [
     Image.showInsertImageDialog = function (app) {
 
         var // the document model
-            model = app.getModel(),
+            docModel = app.getModel(),
 
-            // do not use asynchronous mode for IE9 file upload
-            async = !Utils.IE9,
+            recentImageFolderKey = 'recentImageFolder' + app.getDocumentType(),
+            recentImageFolder = Settings.get(recentImageFolderKey) || DriveUtils.getStandardPicturesFolderId(),
+            async = true,
 
             // the dialog instance, and the tab panel nodes
             dialog = null,
@@ -511,12 +464,19 @@ define('io.ox/office/drawinglayer/view/imageutil', [
             fileInput = filePicker.find('input[type="file"]'),
 
             // the drive file picker
-            driveFilePicker = new DriveFilePicker(),
+            driveFilePicker = new DriveUtils.FilePicker({
+                filter: function (file) {
+                    return Image.hasFileImageExtension(file.filename);
+                },
+                emptyText: gt('This folder has no images.'),
+                preselect: recentImageFolder
+            }),
 
             // the 'Enter URL' widgets
             urlInput = $('<input>', {
                 tabindex: 0,
-                role: 'textbox'
+                role: 'textbox',
+                'aria-label': gt('enter image URL')
             }).addClass('form-control'),
 
             // the file descriptor of the file currently selected
@@ -527,6 +487,10 @@ define('io.ox/office/drawinglayer/view/imageutil', [
             // the resulting promise
             promise = null;
 
+        driveFilePicker.on('change', function () {
+            selectedDriveFile = driveFilePicker.getSelectedFile();
+            checkOkButton();
+        });
 
         // create a file picker widget node
         function filePickerWidget(options) {
@@ -537,7 +501,6 @@ define('io.ox/office/drawinglayer/view/imageutil', [
                 accept: 'image/*'
             }, options);
 
-
             var guid = _.uniqueId('form-control-'),
 
                 node = $('<div class="file-picker">').addClass((options.wrapperClass ? options.wrapperClass : 'form-group')),
@@ -549,15 +512,15 @@ define('io.ox/office/drawinglayer/view/imageutil', [
                     .attr('for', guid)
                     .text(options.buttontext),
 
-                inputFile = $('<input name="file" type="file">')
+                inputFile = $('<input name="file" type="file" style="margin-top: -20px; margin-bottom: -20px;">')
                     .prop({ multiple: options.multi })
                     .attr({ accept: options.accept, id: guid }),
 
                 buttonGroupFile = $('<span class="input-group-btn btn-file" role="button">')
                   .append(buttonFile, labelFile, inputFile),
 
-                inputText = $('<input type="text" class="form-control" readonly="" aria-readonly="true">')
-                    .attr({ placeholder: options.placeholder });
+                inputText = $('<input type="text" class="form-control" readonly="" aria-readonly="true" style="pointer-events: none;">')
+                    .attr({ placeholder: options.placeholder, 'aria-label': gt('file name') });
 
             // set tabindex depending on browser
             ((_.browser.IE) ? buttonFile : buttonGroupFile).attr('tabindex', '1');
@@ -565,10 +528,10 @@ define('io.ox/office/drawinglayer/view/imageutil', [
             node.append(($('<div class="input-group">').append(buttonGroupFile, inputText)));
 
             node.on('keypress', function (e) {
-            	// press SPACE (ENTER isn't active, cause of the special 'send'-occupancy)
+                // press SPACE (ENTER isn't active, cause of the special 'send'-occupancy)
                 if (e.which === 32) {
-            		inputFile.focus(); // BUG #34034: FF needs to focus the input-element first
-                	inputFile.trigger('click');
+                    inputFile.focus(); // BUG #34034: FF needs to focus the input-element first
+                    inputFile.trigger('click');
                 }
             });
 
@@ -580,166 +543,18 @@ define('io.ox/office/drawinglayer/view/imageutil', [
             return node;
         }
 
-        // create infostore (Drive) file system browser
-        function DriveFilePicker() {
-
-            var folderCache = new CoreCache.SimpleCache('folder-all'),
-                subFolderCache = new CoreCache.SimpleCache('subfolder-all'),
-                treeNode = $('<div>').addClass('drive-folder-list'),
-                filesPane = $('<ul>').addClass('io-ox-fileselection list-unstyled drive-image-list singleselection f6-target').attr({ tabindex: 0}),
-                filePickerNode = $('<div>').addClass('drive-image-picker').append(treeNode, filesPane),
-                folderTree = new FolderViews.ApplicationFolderTree(treeNode, {
-                    type: 'infostore',
-                    tabindex: 0,
-                    rootFolderId: '9',
-                    all: true,
-                    storage: {
-                        folderCache: folderCache,
-                        subFolderCache: subFolderCache
-                    },
-                    filter: function (folder) {
-                        return !FolderAPI.is('trash', folder);
-                    }
-                }),
-                self = this;
-
-            // extend filesPane with CoreSelection functionalities, plus keyboard selection support
-            CoreSelection.extend(this, filesPane, {});
-            self.selection.keyboard(filesPane, true);
-
-
-            // on foldertree change update file selection
-            folderTree.selection.on('select', function (e, folderId) {
-
-                filesPane.empty();
-
-                FilesAPI.getAll({ folder: folderId }, false).then(function (files) {
-
-                    // reset selected file after every folder load file action
-                    selectedDriveFile = null;
-
-                    // filter folder content for image files
-                    var imageFiles = _.filter(files, function (file) {
-                        return Image.hasFileImageExtension(file.filename);
-                    });
-
-                    // disable ok button and quit immediately if no image files is found in this folder
-                    if (imageFiles.length === 0) {
-                        dialog.enableOkButton(false);
-                        filesPane.append($('<p>').addClass('bg-info').text(gt('This folder has no images.')));
-                        return;
-                    }
-
-                    // prepare filespane node and insert into root filepickernode
-                    _.each(imageFiles, function (file) {
-                        var fileNameNode = $('<div>').addClass('name').text(file.filename || file.title),
-                            fileListItem = $('<li>').addClass('file selectable').attr('data-obj-id', _.cid(file));
-                        filesPane.append(fileListItem.append(fileNameNode));
-                    });
-                    filePickerNode.append(filesPane);
-
-                    // init the CoreSelection object of the filespane, select first item initially
-                    self.selection.clear();
-                    self.selection.init(imageFiles);
-
-                    // save initially selected image file to global variable of insertImageDialog
-                    var selectedList = self.selection.get();
-                    selectedDriveFile = selectedList.length > 0 ? selectedList[0] : null;
-
-                    // update selected file after change event on the filespane selection object
-                    self.selection.on('change', function (e, list) {
-                        selectedDriveFile = list.length > 0 ? list[0] : null;
-                        // activate/deactivate dialogs ok button according to selected file status and edit mode
-                        checkOkButton();
-                    });
-
-                });
-
-                // remember user's last insert image folder location, to be saved later while leaving dialog
-                filePickerNode.data('recent-folder', folderId);
-
-            });
-
-            // draw and init application folder tree
-            folderTree.paint().done(function () {
-                //preselect user's recent image folder if exists
-                var recentImageFolder = Settings.get('recentImageFolder');
-                if (recentImageFolder) {
-                    folderTree.select(recentImageFolder);
-                    folderTree.selection.select(recentImageFolder);
-                }
-                else {
-                    folderTree.selection.updateIndex().selectFirst();
-                }
-                treeNode.focus();
-            });
-
-			// grab "TAB" and "shift+TAB" keystrokes in FolderTree
-			// to prevent default handling an jump to prev/next Control
-			treeNode.on('keydown', function(event){
-                if (filesPane.find('.file').length === 0) { return; }
-				if (KeyCodes.matchKeyCode(event, 'TAB', { shift: false })) {
-
-					self.selection.selectFirst();
-					filesPane.focus();
-					$('li[data-obj-id="'+selectedDriveFile.folder_id+'.'+selectedDriveFile.id+'"]').first().focus();
-					checkOkButton();
-
-					event.preventDefault();
-					event.stopPropagation();
-					return false;
-
-				} else if (KeyCodes.matchKeyCode(event, 'TAB', { shift: true })) {
-					var arrFolders = folderTree.selection.getObservedItems(),
-						activeFolder = folderTree.selection.get()[0],
-						firstFolder = null;
-
-						_.each(arrFolders, function(i, ele){
-							if (i===0) { firstFolder = ele; }
-						});
-
-					if( arrFolders[activeFolder] !== 0 ){
-						$('div[data-obj-id="'+firstFolder+'"]').focus();
-					}
-				}
-			});
-
-			// grab "shift+TAB" in FilesPane
-			// to jump to prev Control respectivily to the "selected" Folder in FolderTree
-			filesPane.on('keydown', function(event){
-                if (filesPane.find('.file').length === 0) { return; }
-                if (KeyCodes.matchKeyCode(event, 'TAB', { shift: true })) {
-
-					self.selection.deselect(selectedDriveFile.folder_id+'.'+selectedDriveFile.id);
-
-					var folderId = folderTree.selection.get()[0];
-
-					// treeNode.focus();
-					$('div[data-obj-id="'+folderId+'"]').focus();
-
-					checkOkButton();
-
-					event.preventDefault();
-					event.stopPropagation();
-					return false;
-				}
-			});
-
-            return filePickerNode;
-        }
-
         // enable / disable OK button
         function checkOkButton() {
             switch (dialog.getActiveTabId()) {
             case 'drive':
                 // enable Ok button when we have file descriptor ready
-                dialog.enableOkButton(model.getEditMode() && selectedDriveFile);
+                dialog.enableOkButton(docModel.getEditMode() && selectedDriveFile);
                 break;
             case 'file':
-                dialog.enableOkButton(model.getEditMode() && (uploadFile || uploadFileName));
+                dialog.enableOkButton(docModel.getEditMode() && (uploadFile || uploadFileName));
                 break;
             case 'url':
-                dialog.enableOkButton(model.getEditMode() && (urlInput.val().length > 0));
+                dialog.enableOkButton(docModel.getEditMode() && (urlInput.val().length > 0));
                 break;
             default:
                 dialog.enableOkButton(false);
@@ -747,14 +562,14 @@ define('io.ox/office/drawinglayer/view/imageutil', [
         }
 
         // create the dialog
-        dialog = new EditDialogs.ModalTabDialog({ title: gt('Insert Image'), width: 600, async: async });
+        dialog = new Dialogs.ModalTabDialog({ title: gt('Insert Image'), async: async, width: Dialogs.getBestDialogWidth(800) });
 
         // close dialog automatically after losing edit rights
         app.getView().closeDialogOnReadOnlyMode(dialog);
 
         // prepare the 'Drive' tab panel
         drivePanel = dialog.addTabPanel('drive', { label: gt('Drive') });
-        drivePanel.append(driveFilePicker);
+        drivePanel.append(driveFilePicker.getNode());
 
         // prepare the 'Upload File' tab panel
         filePanel = dialog.addTabPanel('file', { label: gt('Upload image') });
@@ -783,21 +598,46 @@ define('io.ox/office/drawinglayer/view/imageutil', [
                 }
                 break;
             case 'url':
-                urlInput.focus();
+                // us 102078426: dialogs on small devices should come up w/o keyboard,
+                // so the focus shoud not be in an input element
+                if (Utils.SMALL_DEVICE) {
+                    dialog.getOkButton().focus();
+                } else {
+                    urlInput.focus();
+                }
+
                 break;
             case 'drive':
-            	var folderList = $(driveFilePicker).find('.drive-folder-list');
-            	if (folderList.length > 0) {
-            		var selectedItem = $(folderList[0]).find('.selected');
-            		if (selectedItem.length > 0) {
-            			selectedItem[0].focus();
-            		}
-            	}
+                var folderList = driveFilePicker.getNode().find('.drive-folder-list').first();
+                var selectedItem = folderList.find('.selected')[0];
+
+                if (selectedItem) {
+                    app.executeDelayed(function () {
+                        selectedItem.focus();
+                    });
+                }
                 break;
             }
             checkOkButton();
         });
 
+        if (Utils.TOUCHDEVICE) {
+            //set a label over the "tab" button, which start directly the filehandler when user changes tab to file-upload!
+            var uploadTab = dialog.getPopup().find('a[data-user-id="file"]');
+            var tabParent = uploadTab.parent();
+
+            var uploadLabel = $('<label>');
+            uploadLabel.attr('for', fileInput.attr('id'));
+            uploadLabel.text(uploadTab.text());
+            uploadLabel.css({ color: 'transparent', position: 'absolute', width: '100%', height: '100%', left: 0, top: 0 });
+
+            tabParent.append(uploadLabel);
+
+            uploadLabel.on('click', function () {
+                uploadTab.trigger('click');
+            });
+        }
+
         // register a change handler at the file input that extracts the file descriptor
         fileInput.on('change', function (event) {
             uploadFile = (event.target && event.target.files && event.target.files[0]) || null;  // requires IE 10+
@@ -861,14 +701,18 @@ define('io.ox/office/drawinglayer/view/imageutil', [
         });
 
         promise = promise.then(function (file, isDriveFile, isFileUpload, form) {
-            if (isDriveFile) { return Image.insertDriveImage(app, file); }
+            if (isDriveFile) {
+
+                // save user's last selected folder
+                var userRecentFile = driveFilePicker.getSelectedFile();
+                if (userRecentFile) { Settings.set(recentImageFolderKey, userRecentFile.folder_id).save(); }
+
+                return Image.insertDriveImage(app, file);
+            }
             return isFileUpload ? Image.insertFile(app, file, form) : Image.insertURL(file.trim());
         });
 
         return promise.always(function () {
-            // save user's last selected folder
-            var userRecentFolder = driveFilePicker.data('recent-folder');
-            if (userRecentFolder) { Settings.set('recentImageFolder', userRecentFolder).save(); }
             //close dialog manually in asynchronous mode
             dialog.close();
             // remove all event listeners
