@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -16,7 +16,7 @@ define('io.ox/office/drawinglayer/view/popup/chartlabelsmenu', [
     'io.ox/office/tk/control/checkbox',
     'io.ox/office/tk/control/textfield',
     'io.ox/office/drawinglayer/view/popup/drawingframemenu',
-    'gettext!io.ox/office/drawing'
+    'gettext!io.ox/office/drawinglayer/main'
 ], function (Utils, CheckBox, TextField, DrawingFrameMenu, gt) {
 
     'use strict';
@@ -50,12 +50,15 @@ define('io.ox/office/drawinglayer/view/popup/chartlabelsmenu', [
      * @constructor
      *
      * @extends DrawingFrameMenu
+     *
+     * @param {EditView} docView
+     *  The document view that contains this instance.
      */
-    function ChartLabelsMenu(app, initOptions) {
+    function ChartLabelsMenu(docView, initOptions) {
 
         // base constructor ---------------------------------------------------
 
-        DrawingFrameMenu.call(this, app, 'chart', /*#. menu title: modify settings for chart objects */ gt.pgettext('menu-title', 'Labels & Axes'), initOptions);
+        DrawingFrameMenu.call(this, docView, 'chart', /*#. menu title: modify settings for chart objects */ gt.pgettext('menu-title', 'Labels & Axes'), initOptions);
 
         // initialization -----------------------------------------------------
 
@@ -91,7 +94,7 @@ define('io.ox/office/drawinglayer/view/popup/chartlabelsmenu', [
 
         // destroy all class members
         this.registerDestructor(function () {
-            app = initOptions = null;
+            docView = initOptions = null;
         });
 
     } // class ChartLabelsMenu
