@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -28,28 +28,27 @@ define('io.ox/office/drawinglayer/view/popup/drawingframemenu', [
      *
      * @extends LayerMenu
      *
-     * @param {EditApplication} app
-     *  The application containing this menu instance.
+     * @param {EditView} docView
+     *  The document view that contains this instance.
      *
      * @param {String} drawingType
      *  The types of drawing objects supported by this menu, as space-separated
      *  list.
      */
-    function DrawingFrameMenu(app, drawingType, titleLabel, initOptions) {
+    function DrawingFrameMenu(docView, drawingType, titleLabel, initOptions) {
 
         var // self reference
             self = this,
 
             // the model and view instance of the document
-            docModel = app.getModel(),
-            docView = app.getView(),
+            docModel = docView.getDocModel(),
 
             // the current drawing frame used as menu anchor
             drawingFrame = null;
 
         // base constructor ---------------------------------------------------
 
-        LayerMenu.call(this, app, titleLabel, Utils.extendOptions({
+        LayerMenu.call(this, docView, titleLabel, Utils.extendOptions({
             anchorBorder: 'right left'
         }, initOptions, {
             anchor: function () { return drawingFrame; },
@@ -84,23 +83,17 @@ define('io.ox/office/drawinglayer/view/popup/drawingframemenu', [
         // convert space-separated list of drawing object types to array
         drawingType = drawingType.split(/\s+/);
 
-        // initialize class members when application is ready
-        app.onInit(function () {
-            docModel = app.getModel(),
-            docView = app.getView();
-
-            // hide the menu automatically in read-only mode
-            self.listenTo(docModel, 'change:editmode', function (event, editMode) {
-                if (!editMode) { self.hide(); }
-            });
-
-            // adjust position and visibility of the menu according to the current selection
-            self.listenTo(docView, 'render:drawingselection', renderDrawingSelectionHandler);
+        // hide the menu automatically in read-only mode
+        this.listenTo(docModel, 'change:editmode', function (event, editMode) {
+            if (!editMode) { self.hide(); }
         });
 
+        // adjust position and visibility of the menu according to the current selection
+        this.listenTo(docView, 'render:drawingselection', renderDrawingSelectionHandler);
+
         // destroy all class members
         this.registerDestructor(function () {
-            app = initOptions = self = docModel = docView = null;
+            self = initOptions = docModel = docView = null;
         });
 
     } // class DrawingFrameMenu
