@@ -5,21 +5,23 @@
  * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Oliver Specht <oliver.specht@open-xchange.com>
  */
 
-define('io.ox/office/editframework/app/mailactions',
-    ['io.ox/core/extensions',
-     'io.ox/core/extPatterns/links',
-     'io.ox/core/notifications',
-     'io.ox/mail/api',
-     'io.ox/office/tk/utils',
-     'io.ox/office/tk/io',
-     'io.ox/office/baseframework/app/extensionregistry',
-     'gettext!io.ox/office/files'
-    ], function (ext, links, CoreNotification, MailApi, Utils, IO, ExtensionRegistry, gt) {
+define('io.ox/office/editframework/app/mailactions', [
+    'io.ox/core/extensions',
+    'io.ox/core/extPatterns/links',
+    'io.ox/core/notifications',
+    'io.ox/mail/api',
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/utils/driveutils',
+    'io.ox/office/tk/io',
+    'io.ox/office/baseframework/utils/infomessages',
+    'io.ox/office/baseframework/app/extensionregistry',
+    'gettext!io.ox/office/editframework/actions'
+], function (ext, links, CoreNotification, MailApi, Utils, DriveUtils, IO, InfoMessages, ExtensionRegistry, gt) {
 
     'use strict';
 
@@ -60,7 +62,7 @@ define('io.ox/office/editframework/app/mailactions',
         // initialization -----------------------------------------------------
 
         // Copy mail attachment to Files and edit
-        new links.Action(ACTION_POINT + '/edit-as-new', {
+        new links.Action(ACTION_POINT + '-edit-asnew', {
             id: 'save',
             capabilities: 'infostore',
             requires: function (e) {
@@ -73,21 +75,21 @@ define('io.ox/office/editframework/app/mailactions',
                 return isEditable;
             },
             multiple: function (list) {
-                MailApi.saveAttachments(list).done(function (result) {
+                MailApi.saveAttachments(list,  DriveUtils.getStandardDocumentsFolderId()).done(function (result) {
 
                     var // the baton data of the created Drive file returned by the mail API
                         baton = result[0],
                         // the file descriptor passed to the OX Documents application launcher
                         file = _.clone(baton.data);
 
-                    // add the origin property to the file descriptor
+                    // Bug 40376 add the origin property to the file descriptor
+                    file.source = 'drive';
                     file.origin = { source: 'mail', folder_id: file.mailFolder, id: file.mailUID };
                     delete file.mailFolder;
                     delete file.mailUID;
 
                     // show a notification to the user that a new file has been created in Drive
-                    CoreNotification.yell('success', gt('Document has been stored in your default folder as "%1$s".', _.noI18n(file.filename)));
-
+                    CoreNotification.yell('success', InfoMessages.getMessageData('INFO_DOC_STORED_IN_DEFAULTFOLDER_AS', { fullFileName: file.filename }).message);
                     // launch the OX Documents editor application
                     if (ExtensionRegistry.isNative(file.filename)) {
                         launchApplication({ action: 'load', file: file });
@@ -99,14 +101,21 @@ define('io.ox/office/editframework/app/mailactions',
             }
         });
 
-        // an action link to edit files attached to an e-mail
-        ext.point('io.ox/mail/attachment/links').extend(new links.Link({
-            id: appBaseName + '_edit',
-            index: 600,
+        var link = {
+            id: appBaseName + '-edit',
+            index: 1000,
+            prio: 'lo',
             mobile: 'lo',
             label: gt('Edit as new'),
-            ref: ACTION_POINT + '/edit-as-new'
-        }));
+            ref: ACTION_POINT + '-edit-asnew'
+        };
+
+        // an action link to edit files attached to an e-mail
+        ext.point('io.ox/mail/attachment/links').extend(new links.Link(link));
+
+        // an action link to edit files attached to an e-mail
+        ext.point('io.ox/core/viewer/toolbar/links/mail').extend(new links.Link(link));
+
     };
 
     // exports ================================================================
