@@ -6,23 +6,23 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Kai Ahrens <kai.ahrens@open-xchange.com>
  * @author Carsten Driesner <carsten.driesner@open-xchange.com>
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/app/rtconnection',
-    ['io.ox/core/event',
-     'io.ox/realtime/groups',
-     'io.ox/office/tk/utils',
-     'io.ox/office/tk/logger',
-     'io.ox/office/tk/object/triggerobject',
-     'io.ox/office/tk/object/timermixin',
-     'io.ox/office/editframework/utils/editconfig',
-     'io.ox/office/tk/filedescriptor'
-    ], function (Events, RTGroups, Utils, Logger, TriggerObject, TimerMixin, Config, FileDescriptor) {
+define('io.ox/office/editframework/app/rtconnection', [
+    'io.ox/core/event',
+    'io.ox/realtime/groups',
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/utils/logger',
+    'io.ox/office/tk/object/triggerobject',
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/editframework/utils/editconfig',
+    'io.ox/office/tk/restore/sessionrestore'
+], function (Events, RTGroups, Utils, Logger, TriggerObject, TimerMixin, Config, SessionRestore) {
 
     'use strict';
 
@@ -33,7 +33,7 @@ define('io.ox/office/editframework/app/rtconnection',
         REQUEST_EVENTS = 'addresource getdocument',
 
         // names of runtime push messages
-        PUSH_EVENTS = 'update flusherrorinfo hangup preparelosingeditrights declinedacquireeditrights acceptedacquireeditrights',
+        PUSH_EVENTS = 'update flusherrorinfo hangup preparelosingeditrights declinedacquireeditrights acceptedacquireeditrights infomessage renameandreload',
 
         // max number of action send calls until an update must be received
         MAX_NUM_OF_APPLYACTIONS_UNTIL_UPDATE = 50,
@@ -41,6 +41,25 @@ define('io.ox/office/editframework/app/rtconnection',
         // repeated time to check for pending ACKs
         CHECK_PENDING_INTERVAL = 30000;
 
+    // private static functions ===============================================
+
+    /**
+     * Encodes file properties of the passed file descriptor to a resource
+     * identifier that can be used to initialize a real-time connection.
+     *
+     * @param {Object} file
+     *  A file descriptor object which contains file specific properties, like
+     *  'id' and 'folder_id'.
+     *
+     * @returns {String}
+     *  A string which references a file and complies to the resource id
+     *  encoding scheme; or null, if the file descriptor doesn't describe a
+     *  valid file.
+     */
+    function encodeFileAsResourceId(file) {
+        return encodeURIComponent(file['com.openexchange.realtime.resourceID'] || file.id);
+    }
+
     // class RTConnection =====================================================
 
     /**
@@ -72,8 +91,11 @@ define('io.ox/office/editframework/app/rtconnection',
             // file descriptor of the edited document
             file = app.getFileDescriptor(),
 
+            // rt channel id
+            channelId = app.getDocumentType(),
+
             // create the core real-time group instance for the document
-            rtGroup = RTGroups.getGroup('synthetic.office://operations/' + FileDescriptor.getAsResourceId(file)),
+            rtGroup = RTGroups.getGroup('synthetic.' + channelId + '://operations/' + encodeFileAsResourceId(file)),
 
             // private event hub for internal event handling
             eventHub = new Events(),
@@ -96,7 +118,13 @@ define('io.ox/office/editframework/app/rtconnection',
             joining = false,
 
             // rejoining is possible
-            rejoinPossible = false;
+            rejoinPossible = false,
+
+            // old rtid of the user for restore support
+            oldrtId = SessionRestore.state('documents.rtid');
+
+        // store rtid to retrieve it with a possible browser reload
+        SessionRestore.state('documents.rtid', RTGroups.rtId);
 
         // base constructor ---------------------------------------------------
 
@@ -121,8 +149,10 @@ define('io.ox/office/editframework/app/rtconnection',
                 activeClients = Utils.getArrayOption(data, 'activeUsers', []).length,
                 // write protected
                 writeProtected = Utils.getBooleanOption(data, 'writeProtected', false),
-                // editing user id
+                // locking user
                 lockedByUser = Utils.getStringOption(data, 'lockedByUser', ''),
+                // locking user id
+                lockedByUserId = Utils.getIntegerOption(data, 'lockedByUserId', -1),
                 // output string
                 output = '';
 
@@ -132,7 +162,8 @@ define('io.ox/office/editframework/app/rtconnection',
             output += 'active users: ' + activeClients + '; ';
             output += 'hasErrors: ' + hasErrors + '; ';
             output += 'writeProtected: ' + writeProtected + '; ';
-            output += 'locked by user: ' + lockedByUser;
+            output += 'locked by user: ' + lockedByUser + '; ';
+            output += 'locked by id: ' + lockedByUserId;
             Utils.log(output);
 
             // check that all array elements are objects
@@ -155,8 +186,8 @@ define('io.ox/office/editframework/app/rtconnection',
         function registerRTEvents(eventNames) {
             _.each(eventNames.split(/\s+/), function (eventName) {
                 rtGroup.on('receive', function (event, stanza) {
-                    _.each(stanza.getAll('office', eventName), function (payload) {
-                        RTConnection.log('RTConnection.eventHub.trigger: office:' + eventName);
+                    _.each(stanza.getAll(channelId, eventName), function (payload) {
+                        RTConnection.log('RTConnection.eventHub.trigger (' + channelId + '), ' + eventName);
                         eventHub.trigger(eventName, payload.data);
                     });
                 });
@@ -195,7 +226,7 @@ define('io.ox/office/editframework/app/rtconnection',
             });
         }
 
-       /**
+        /**
          * Send a message to the connected RT object,
          * waiting for the acknowledge to this call.
          *
@@ -211,7 +242,7 @@ define('io.ox/office/editframework/app/rtconnection',
 
             if (_.isString(element) && !_.isUndefined(data)) {
                 payloads.push({
-                    namespace: 'office',
+                    namespace: channelId,
                     element: element,
                     data: data,
                     verbatim: true
@@ -281,12 +312,20 @@ define('io.ox/office/editframework/app/rtconnection',
 
             if (_.isString(element) && !_.isUndefined(data)) {
                 payloads.push({
-                    namespace: 'office',
+                    namespace: channelId,
                     element: element,
                     data: data,
                     verbatim: true
                 });
             }
+            // Add necessary payload to provide the real-time id for
+            // correct rights management on the backend
+            payloads.push({
+                namespace: 'office',
+                element: 'rtdata',
+                data: { rtid: RTGroups.rtId, session: ox.session },
+                verbatim: true
+            });
 
             defer = rtGroup.query({
                 element: 'message',
@@ -299,7 +338,9 @@ define('io.ox/office/editframework/app/rtconnection',
             defer.fail(function () {
                 RTConnection.error('Send query message with RT failed, payloads: ' + payloads.toString());
                 // RT calls us even after connection, rtgroup are destroyed and references are cleared.
-                if (!self.destroyed) { self.trigger('timeout'); }
+                if (!self.destroyed) {
+                    self.trigger('timeout');
+                }
             });
 
             return defer.promise();
@@ -340,7 +381,7 @@ define('io.ox/office/editframework/app/rtconnection',
                         RTConnection.log('Timeout reached for RT request, timeout=' + timeout + 'ms.');
                         def.reject({ cause: 'timeout' });
                     }
-                }, timeout);
+                }, timeout, 'RTConnection: handleResponse');
                 def.always(function () { timer.abort(); timer = null; });
             }
 
@@ -371,7 +412,7 @@ define('io.ox/office/editframework/app/rtconnection',
 
             // prevent endless waiting
             if (_.isNumber(timeout) && (timeout > 0)) {
-                self.executeDelayed(function () { def.reject({ cause: 'timeout' }); }, timeout);
+                self.executeDelayed(function () { def.reject({ cause: 'timeout' }); }, timeout, 'RTConnection: handleGenericResponse');
             }
 
             return def.promise();
@@ -392,7 +433,7 @@ define('io.ox/office/editframework/app/rtconnection',
 
             if (self.destroyed) {
                 // reject deferred after destroy
-                def.reject({cause: 'closing'});
+                def.reject({ cause: 'closing' });
             }
             if (payloads.length > 0) {
                 // Extract the data and provide it
@@ -426,6 +467,113 @@ define('io.ox/office/editframework/app/rtconnection',
             }
         }
 
+        /**
+         * Handles the resolved response of the RT low-level join request
+         * and makes checks for certain aspects (errors).
+         *
+         * @param {Object} {jQuery.Deferred} def
+         *
+         * @paran {Object} response
+         *  The original response from a join-request provided by the RT
+         *  low-level code.
+         */
+        function handleJoinResponse(def, response) {
+            var // the payloads of the join response
+                payloads = Utils.getArrayOption(response, 'payloads', []),
+                // the data to resolve/reject the deferred
+                data = null,
+                // do we found an error
+                hasError = false;
+
+            if (self.destroyed) {
+                // reject deferred after destroy
+                def.reject({ cause: 'closing' });
+            }
+
+            if (payloads.length > 0) {
+                // Extract the answer payload from the backend answer, look
+                // for possible errors sent by RT via payloads, too. Should be
+                // handled by RT-core (see #39911).
+                _.find(payloads, function (payload) {
+                    if (_.isObject(payload) && _.isObject(payload.data) && _.isString(payload.element)) {
+                        switch (payload.element) {
+                        case 'error': hasError = true; data = { error: payload.data }; break;
+                        case 'getactions': data = payload.data; break;
+                        default: /* ignore payload */ break;
+                        }
+                    }
+                    return hasError;
+                });
+
+                data = data || payloads[0].data; // as fallback
+                if (hasError) { def.reject(data); } else { def.resolve(data); }
+            } else {
+                // we must receive at least one payload - otherwise we
+                // assume an error
+                def.reject(response);
+            }
+        }
+
+        /**
+         * Handles the rejected response of the RT low-level join request
+         * and makes checks for certain aspects (errors). In special cases
+         * a re-join is started to re-enroll the client.
+         *
+         * @param {Object} {jQuery.Deferred} def
+         *
+         * @paran {Object} response
+         *  The original response from a join-request provided by the RT
+         *  low-level code.
+         */
+        function handleRejectedJoinRequest(def, response, retryHandler) {
+            var // do we found error data
+                hasError = false;
+
+            if (self.destroyed) {
+                // reject deferred after destroy
+                def.reject({ cause: 'closing' });
+            }
+
+            // we assume that we receive the error data as a property in the response
+            if (_.isObject(response) && _.isObject(response.error)) {
+                hasError = true;
+            }
+
+            // check error code to decide, if we can try to re-join
+            if (hasError && response.error.code === 1007) {
+                // try to re-join in case of code 1007 (STATE_MISSING)
+                handleRejoin(def, response, retryHandler);
+            } else {
+                joining = false;
+                def.reject(response);
+            }
+        }
+
+        /**
+         * Handles the re-join of the client, in case a re-enroll
+         * is necessary.
+         *
+         * @param {Function} retryHandler
+         *  A handler which is responsible to start a re-join of the client.
+         *  Must be valid function.
+         */
+        function handleRejoin(def, response, retryHandler) {
+            // We have to wait for possible notifications from the
+            // low-level RT part to determine, if a retry is possible.
+            self.executeDelayed(function () {
+                // now check the states and decide, if we can try to
+                // start a reconnect using a possible retry handler
+                if (rejoinPossible) {
+                    rejoinPossible = false;
+                    retryHandler.call(this, def);
+                    joining = false;
+                } else {
+                    handleJoinResponse(def, response);
+                    joining = false;
+                }
+            }, 1000, 'RTConnection: joinGroup');
+        }
+
         /**
          * Join a group using the provided options and set the asynchronous
          * state. An optional retryHandler is called, if there is a chance
@@ -449,38 +597,25 @@ define('io.ox/office/editframework/app/rtconnection',
 
                 if (self.destroyed) {
                     // reject deferred after destroy
-                    def.reject({cause: 'closing'});
+                    def.reject({ cause: 'closing' });
                     joining = false;
                 } else if (payloads.length > 0) {
-                    handleResolvedResponse(def, response);
+                    handleJoinResponse(def, response);
                     joining = false;
                 } else {
                     // We encountered an illegal state for our join request.
                     if (_.isFunction(retryHandler)) {
-                        // We have to wait for possible notifications from the
-                        // low-level RT part to determine, if a retry is possible.
-                        self.executeDelayed(function () {
-                            // now check the states and decide, if we can try to
-                            // start a reconnect using a possible retry handler
-                            if (rejoinPossible) {
-                                rejoinPossible = false;
-                                retryHandler.call(this, def);
-                                joining = false;
-                            } else {
-                                handleResolvedResponse(def, response);
-                                joining = false;
-                            }
-                        }, 1000);
+                        // try to re-join if handler is available
+                        handleRejoin(def, response, retryHandler);
                     } else {
                         // without retry handler just process the original
                         // response
-                        handleResolvedResponse(def, response);
+                        handleJoinResponse(def, response);
                         joining = false;
                     }
                 }
             }, function (response) {
-                handleRejectedResponse(def, response);
-                joining = false;
+                handleRejectedJoinRequest(def, response, retryHandler);
             });
         }
 
@@ -501,16 +636,22 @@ define('io.ox/office/editframework/app/rtconnection',
         this.connect = function (connectData) {
             var connectDeferred = $.Deferred(),
                 // optional connect data
-                connectOptions = {expectWelcomeMessage: true};
+                connectOptions = { expectWelcomeMessage: true },
+                // connect data
+                sendConnectData = connectData || {};
 
-            if (connectData) {
-                _.extend(connectOptions, {additionalPayloads: [{element: 'connectData', namespace: 'office', data: connectData, verbatim: true }]});
-            }
+            // ATTENTION:
+            // We have to send an additional payload with context information for the join.
+            // Due to sharing we need to provide folder & file-id to enable the backend to
+            // load the document & check the permissions in the context of every user.
+            // The session alone is not enough anymore. Please be aware that this is now
+            // MANDATORY and changing this can break the whole load/save process!!
+            _.extend(connectOptions, { additionalPayloads: [{ element: 'connectData', namespace: channelId, data: _.extend(sendConnectData, { oldrtId: oldrtId, folder_id: file.folder_id, id: file.id, session: ox.session }), verbatim: true }] });
 
             if (!rtGroup.isRTWorking()) {
                 // the server doesn't support real-time communication, must be
                 // incorrect configured.
-                return $.Deferred().reject({cause: 'bad server component' });
+                return $.Deferred().reject({ cause: 'bad server component' });
             }
 
             RTConnection.log('RTConnection.connect called');
@@ -650,6 +791,16 @@ define('io.ox/office/editframework/app/rtconnection',
             return send('logMessage', 'state', { message: data });
         };
 
+        /**
+         * Sends a sync request to the server, which will answer with a update
+         * notification. This can be used to synchronize client and server
+         * after an off-/online scenario.
+         */
+        this.sync = function () {
+            RTConnection.log('RTConnection.sync called');
+            return send('sync', 'state', { osn: app.getModel().getOperationStateNumber() });
+        };
+
         /**
          * Close the document at the connected RT object
          *
@@ -658,20 +809,30 @@ define('io.ox/office/editframework/app/rtconnection',
          *  closing document status of the request.
          */
         this.closeDocument = function (recentFileData) {
-            var leaveOptions = {expectSignOffMessage: true};
+            var leaveOptions = { expectSignOffMessage: true };
 
             RTConnection.log('RTConnection.closeDocument called');
             if (recentFileData) {
-                _.extend(leaveOptions, {additionalPayloads: [{element: 'recentfile', namespace: 'office', data: recentFileData, verbatim: true }]});
+                _.extend(leaveOptions, { additionalPayloads: [{ element: 'recentfile', namespace: channelId, data: recentFileData, verbatim: true }] });
             }
             return handleResponse(rtGroup.leave(leaveOptions), 20000);
         };
 
+        /**
+         * Provides the unique real-time id used by this connection.
+         *
+         * @returns {String}
+         *  The unique real-time id used by this connection.
+         */
+        this.getUuid = function () {
+            return RTGroups.rtId;
+        };
+
         // initialization -----------------------------------------------------
 
         RTConnection.log('RTConnection initialization');
         if (RTConnection.debug) {
-            this.repeatDelayed(checkPendingAcks, CHECK_PENDING_INTERVAL);
+            this.repeatDelayed(checkPendingAcks, CHECK_PENDING_INTERVAL, 'RTConnection: checkPendingAcks');
         }
 
         // read traceable actions from page URL
@@ -694,12 +855,12 @@ define('io.ox/office/editframework/app/rtconnection',
                     // This must be limited to error code 1007 which means that
                     // we have to re-enroll. A reset after we received a reset with
                     // data.code 1007 is also normal, ignore that, too.
+                    // See #40096, the error structure changed
                     if ((!rejoinPossible && (data && data.code === 1007)) || rejoinPossible) {
                         if (!rejoinPossible) {
                             rejoinPossible = true;
                         }
-                    }
-                    else {
+                    } else {
                         // trigger reset otherwise
                         self.trigger(event.type);
                     }
@@ -759,14 +920,11 @@ define('io.ox/office/editframework/app/rtconnection',
     // extend statically with logging methods
     Logger.extend(RTConnection, { enable: RTConnection.debug, prefix: 'RT' });
 
-    // add time stamps for RT debugging by default
-    if (RTConnection.debug && Config.getUrlFlag('office:log-timestamp', true)) {
-        Logger.LOG_TIMESTAMP = true;
-    }
-
     // enable DOM console for RT debugging by default
     if (RTConnection.debug && Config.getUrlFlag('office:debug-console', true)) {
-        Logger.enableDOMConsole();
+        require(['io.ox/office/tk/utils/domconsole']).done(function (DOMConsole) {
+            DOMConsole.enable();
+        });
     }
 
     // enable logging for real-time core
