@@ -6,16 +6,16 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/model/attributedmodel',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/utils/attributeutils',
-     'io.ox/office/baseframework/model/modelobject'
-    ], function (Utils, AttributeUtils, ModelObject) {
+define('io.ox/office/editframework/model/attributedmodel', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/utils/attributeutils',
+    'io.ox/office/baseframework/model/modelobject'
+], function (Utils, AttributeUtils, ModelObject) {
 
     'use strict';
 
@@ -44,8 +44,8 @@ define('io.ox/office/editframework/model/attributedmodel',
      *
      * @extends ModelObject
      *
-     * @param {EditApplication} app
-     *  The application instance containing this attributed model object.
+     * @param {EditModel} docModel
+     *  The document model containing this attributed model object.
      *
      * @param {Object} [initAttributes]
      *  An attribute set with initial formatting attributes for the attributed
@@ -89,23 +89,8 @@ define('io.ox/office/editframework/model/attributedmodel',
      *      AttributedModel.setAttributes(). This option has no effect, if this
      *      model instance uses style sheets (see option 'styleFamily'), or
      *      refers to a parent model object (see option 'parentModel').
-     *  @param {Function} [initOptions.changeStyleHandler]
-     *      A callback function that will be invoked after changing the style
-     *      sheet reference via the method AttributedModel.setAttributes().
-     *      Will be called in the context of this instance. Receives the
-     *      following parameters:
-     *      (1) {Object} attributeSet
-     *          (in/out) The (incomplete) attribute set passed to the method
-     *          AttributedModel.setAttributes(), which can be modified
-     *          in-place.
-     *      (2) {Object} styleAttributeSet
-     *          The merged (complete) attribute set of the new style sheet.
-     *      (3) {String} styleId
-     *          The identifier of the new style sheet.
-     *      After running this callback function, the resulting explicit
-     *      attributes will be applied at this instance.
      */
-    function AttributedModel(app, initAttributes, initOptions) {
+    function AttributedModel(docModel, initAttributes, initOptions) {
 
         var // self reference
             self = this,
@@ -119,18 +104,15 @@ define('io.ox/office/editframework/model/attributedmodel',
             // whether to remove explicit attributes equal to the attribute defaults
             autoClear = !styleFamily && !parentModel && Utils.getBooleanOption(initOptions, 'autoClear', false),
 
-            // callback function to add custom behavior when changing the style sheet reference
-            changeStyleHandler = Utils.getFunctionOption(initOptions, 'changeStyleHandler'),
-
-            // the global style sheets container of the document model
-            documentStyles = null,
-
             // the collection of style sheets for the style family
             styleCollection = null,
 
             // all attribute families supported by instances of this class
             supportedFamilies = null,
 
+            // the current auto style
+            autoStyleId = null,
+
             // the (incomplete) explicit attributes of this model object
             explicitAttributeSet = {},
 
@@ -139,7 +121,7 @@ define('io.ox/office/editframework/model/attributedmodel',
 
         // base constructor ---------------------------------------------------
 
-        ModelObject.call(this, app, initOptions);
+        ModelObject.call(this, docModel, initOptions);
 
         // private methods ----------------------------------------------------
 
@@ -148,7 +130,60 @@ define('io.ox/office/editframework/model/attributedmodel',
          * sheets are supported.
          */
         function getEffectiveStyleId() {
-            return styleCollection ? styleCollection.getEffectiveStyleId(Utils.getStringOption(explicitAttributeSet, 'styleId', '')) : null;
+            return styleCollection ? styleCollection.getEffectiveStyleId(Utils.getStringOption(explicitAttributeSet, 'styleId', ''), true) : null;
+        }
+
+        /**
+         * Adds or removes an attribute value in the specified attribute map.
+         *
+         * @param {Object} attributes
+         *  A map with formatting attributes to be updated.
+         *
+         * @param {String} name
+         *  The name of the formatting attribute to be changed.
+         *
+         * @param {Any} value
+         *  The new value of the formatting attribute. If set to null, the
+         *  attribute will be removed from the map.
+         *
+         * @param {Object} [definition]
+         *  The definition descriptor of the attribute, if available. Used to
+         *  check the new attribute value against the default value, if the
+         *  option 'autoClear' has been passed to the constructor.
+         */
+        function updateAttributeValue(attributes, name, value, definition) {
+
+            var // whether this attribute has changed
+                attrChanged = false;
+
+            if (_.isNull(value) || (autoClear && _.isObject(definition) && _.isEqual(value, definition.def))) {
+                if (name in attributes) {
+                    delete attributes[name];
+                    attrChanged = true;
+                }
+            } else {
+                if (!_.isEqual(attributes[name], value)) {
+                    attributes[name] = _.copy(value, true);
+                    attrChanged = true;
+                }
+            }
+
+            // delete child attributes according to current attribute value
+            if (attrChanged && autoClear && _.isObject(definition) && _.isArray(definition.childAttrs)) {
+                if (_.isNull(value)) { value = definition.def; }
+                _.each(definition.childAttrs, function (childAttr) {
+
+                    var // the expected value of the parent attribute
+                        expectedValue = childAttr.value,
+                        // whether the current value of the primary attribute matches the expected value
+                        valueMatches = _.isFunction(expectedValue) ? expectedValue.call(self, value) : _.isEqual(value, expectedValue);
+
+                    // delete secondary attribute if expected value does not match
+                    if (!valueMatches) { delete attributes[childAttr.name]; }
+                });
+            }
+
+            return attrChanged;
         }
 
         /**
@@ -162,23 +197,39 @@ define('io.ox/office/editframework/model/attributedmodel',
                 styleId = getEffectiveStyleId();
 
             // start with defaults and style sheet attributes
-            mergedAttributeSet = documentStyles.getDefaultAttributes(supportedFamilies);
+            mergedAttributeSet = docModel.getDefaultAttributes(supportedFamilies);
 
             // add merged attributes of parent model object
             if (parentModel) {
-                documentStyles.extendAttributes(mergedAttributeSet, parentModel.getMergedAttributes());
+                docModel.extendAttributes(mergedAttributeSet, parentModel.getMergedAttributes());
                 _.chain(mergedAttributeSet).keys().difference(supportedFamilies).each(function (family) {
                     delete mergedAttributeSet[family];
                 });
             }
 
-            // add style sheet attributes to resulting merged attribute set
+            // additional processing, if style sheet support is enabled
             if (styleCollection) {
-                documentStyles.extendAttributes(mergedAttributeSet, styleCollection.getStyleSheetAttributes(styleId));
+
+                // set default auto-style identifier if existing
+                if (!autoStyleId && styleCollection.isAuto(styleId)) {
+                    autoStyleId = styleId;
+                }
+
+                // add auto style attributes to resulting merged attribute set
+                if (_.isString(autoStyleId)) {
+                    docModel.extendAttributes(mergedAttributeSet, styleCollection.getStyleAttributeSet(autoStyleId));
+                    styleId = mergedAttributeSet.styleId;
+                }
+
+                // add style sheet attributes to resulting merged attribute set
+                if (!autoStyleId || ('styleId' in explicitAttributeSet)) {
+                    docModel.extendAttributes(mergedAttributeSet, styleCollection.getStyleAttributeSet(explicitAttributeSet.styleId || ''));
+                    styleId = mergedAttributeSet.styleId;
+                }
             }
 
             // add the passed explicit attributes
-            documentStyles.extendAttributes(mergedAttributeSet, explicitAttributeSet);
+            docModel.extendAttributes(mergedAttributeSet, explicitAttributeSet);
 
             // restore the effective style sheet identifier
             if (styleCollection) {
@@ -191,18 +242,84 @@ define('io.ox/office/editframework/model/attributedmodel',
             }
         }
 
-        // methods ------------------------------------------------------------
+        // protected methods --------------------------------------------------
 
         /**
-         * Returns whether this model object contains any explicit attributes.
+         * Clones all contents from the passed attributed model into this model
+         * instance.
+         *
+         * @internal
+         *  Used during clone construction. DO NOT CALL from external code!
+         *
+         * @param {AttributedModel} sourceModel
+         *  The source model instance whose attributes will be cloned into this
+         *  model instance.
+         *
+         * @returns {AttributedModel}
+         *  A reference to this instance.
+         */
+        this.cloneFrom = function (sourceModel) {
+
+            // copy the auto style identifier of the source model
+            autoStyleId = sourceModel.getAutoStyleId();
+
+            // deep copy of all explicit attributes
+            explicitAttributeSet = sourceModel.getExplicitAttributes();
+
+            // recalculate the merged attributes (without notifications)
+            calculateMergedAttributeSet(false);
+
+            return this;
+        };
+
+        // public methods -----------------------------------------------------
+
+        /**
+         * Returns whether this model object contains a reference to an auto
+         * style.
          *
          * @returns {Boolean}
-         *  Whether this model object contains any explicit attributes.
+         *  Whether this model object contains a reference to an auto style.
+         */
+        this.hasAutoStyle = function () {
+            return _.isString(autoStyleId);
+        };
+
+        /**
+         * Returns whether this model object contains any explicit attributes,
+         * or a reference to a style sheet.
+         *
+         * @returns {Boolean}
+         *  Whether this model object contains any explicit attributes, or a
+         *  reference to a style sheet.
          */
         this.hasExplicitAttributes = function () {
             return !_.isEmpty(explicitAttributeSet);
         };
 
+        /**
+         * Returns whether this model object contains any explicit attributes,
+         * a reference to a style sheet, or a reference to an auto style.
+         *
+         * @returns {Boolean}
+         *  Whether this model object contains any explicit attributes, a
+         *  reference to a style sheet, or a reference to an auto style.
+         */
+        this.isFormatted = function () {
+            return this.hasAutoStyle() || this.hasExplicitAttributes();
+        };
+
+        /**
+         * Returns the identifier of the auto style referred by this model.
+         *
+         * @returns {String|Null}
+         *  The identifier of the auto style referred by this model, if
+         *  available; otherwise null.
+         */
+        this.getAutoStyleId = function () {
+            return autoStyleId;
+        };
+
         /**
          * Returns the explicit attribute set of this model object.
          *
@@ -231,12 +348,30 @@ define('io.ox/office/editframework/model/attributedmodel',
          *  style sheet reference in the 'styleId' property), containing values
          *  for all known attributes, collected from the document default
          *  values, the style sheet currently referenced by this model object,
-         *  and the explicit attributes of this model object.
+         *  and the explicit attributes of this model object. For efficiency,
+         *  a reference to the cached attribute set will be returned. This
+         *  object MUST NOT be changed!
          */
         this.getMergedAttributes = function () {
             return mergedAttributeSet;
         };
 
+        /**
+         * Returns whether this attributed model contains the same auto style
+         * identifier and explicit attributes as the passed attributed model.
+         *
+         * @param {AttributedModel} attribModel
+         *  The other attributed model to be compared to this model.
+         *
+         * @returns {Boolean}
+         *  Whether both attributed models contain the same auto style
+         *  identifier and explicit formatting attributes.
+         */
+        this.hasEqualAttributes = function (attribModel) {
+            return (this === attribModel) ||
+                ((autoStyleId === attribModel.getAutoStyleId()) && _.isEqual(explicitAttributeSet, attribModel.getExplicitAttributes(true)));
+        };
+
         /**
          * Changes and/or removes specific explicit formatting attributes, or
          * the style sheet reference of this model object.
@@ -268,94 +403,82 @@ define('io.ox/office/editframework/model/attributedmodel',
          *      recognized by this method.
          *
          * @returns {Boolean}
-         *  Whether any attributes have actually been changed.
+         *  Whether the attributes of this model object have actually changed.
          */
         this.setAttributes = function (attributeSet, options) {
 
             var // whether to trigger the event
                 notify = Utils.getStringOption(options, 'notify', 'auto'),
                 // whether any explicit attribute or the style identifier has been changed
-                changed = false,
-                // the style sheet identifier from the passed attributes
-                styleId = attributeSet.styleId;
+                changed = false;
 
-            // adds or removes an attribute value, updates the changed flag
-            function updateAttributeValue(attributes, name, value, definition) {
+            // set new style sheet or auto style
+            if ('styleId' in attributeSet) {
 
-                var // whether this attribute has changed
-                    attrChanged = false;
-
-                if (_.isNull(value) || (autoClear && _.isObject(definition) && _.isEqual(value, definition.def))) {
-                    if (name in attributes) {
-                        delete attributes[name];
-                        attrChanged = true;
-                    }
-                } else {
-                    if (!_.isEqual(attributes[name], value)) {
-                        attributes[name] = _.copy(value, true);
-                        attrChanged = true;
-                    }
+                // style collection must exist (this model must support style sheets at all)
+                if (!styleCollection) {
+                    Utils.error('AttributedModel.setAttributes(): setting style sheets not supported');
+                    return false;
                 }
 
-                // delete child attributes according to current attribute value
-                if (attrChanged && autoClear && _.isObject(definition) && _.isArray(definition.childAttrs)) {
-                    if (_.isNull(value)) { value = definition.def; }
-                    _.each(definition.childAttrs, function (childAttr) {
+                var // the style sheet identifier from the passed attributes
+                    styleId = attributeSet.styleId;
 
-                        var // the expected value of the parent attribute
-                            expectedValue = childAttr.value,
-                            // whether the current value of the primary attribute matches the expected value
-                            valueMatches = _.isFunction(expectedValue) ? expectedValue.call(self, value) : _.isEqual(value, expectedValue);
+                // remove explicit reference to default style sheet (missing style
+                // reference will automatically fall-back to the default style sheet)
+                if (!_.isString(styleId) || (styleId === '') || (styleId === styleCollection.getDefaultStyleId())) {
+                    styleId = null;
+                }
 
-                        // delete secondary attribute if expected value does not match
-                        if (!valueMatches) { delete attributes[childAttr.name]; }
+                // special handling for auto styles
+                if (_.isString(styleId) && styleCollection.isAuto(styleId)) {
+
+                    // store the new auto style identifier in a special attribute
+                    if (autoStyleId !== styleId) {
+                        autoStyleId = styleId;
+                        changed = true;
+                    }
+
+                    // remove explicit reference to a style sheet (will be resolved via auto style)
+                    if ('styleId' in explicitAttributeSet) {
+                        delete explicitAttributeSet.styleId;
+                        changed = true;
+                    }
+
+                    // remove all explicit attributes supported by style sheets
+                    styleCollection.getSupportedFamilies().forEach(function (family) {
+                        if (family in explicitAttributeSet) {
+                            delete explicitAttributeSet[family];
+                            changed = true;
+                        }
                     });
-                }
 
-                changed = changed || attrChanged;
-                return attrChanged;
-            }
+                } else {
 
-            // check style sheet support
-            if (!styleCollection && ('styleId' in attributeSet)) {
-                Utils.error('AttributedModel.setAttributes(): setting style sheets not supported');
-            }
-
-            // remove explicit reference to default style sheet (missing style
-            // reference will automatically fall-back to the default style sheet)
-            if (styleCollection && _.isString(styleId) && ((styleId === '') || (styleId === styleCollection.getDefaultStyleId()))) {
-                styleId = null;
-            }
-
-            // update the style sheet identifier
-            if (styleCollection && (_.isString(styleId) || _.isNull(styleId))) {
-
-                var // change the style sheet identifier in the internal map
-                    styleChanged = updateAttributeValue(explicitAttributeSet, 'styleId', styleId);
-
-                // invoke callback after changing the style sheet identifier
-                if (styleChanged && _.isFunction(changeStyleHandler)) {
-                    styleId = getEffectiveStyleId();
-                    attributeSet = _.copy(attributeSet, true);
-                    changeStyleHandler.call(this, attributeSet, styleCollection.getStyleSheetAttributes(styleId), styleId);
+                    // change the style sheet identifier in the internal map, update the resulting changed flag
+                    if (updateAttributeValue(explicitAttributeSet, 'styleId', styleId)) {
+                        changed = true;
+                    }
                 }
             }
 
             // update the remaining explicit attributes of all supported attribute families
-            _.each(supportedFamilies, function (family) {
+            supportedFamilies.forEach(function (family) {
 
                 // check if new attributes exist in the passed attribute set
                 if (!_.isObject(attributeSet[family])) { return; }
 
                 var // attribute definitions of the current family
-                    definitions = documentStyles.getAttributeDefinitions(family),
+                    definitions = docModel.getAttributeDefinitions(family),
                     // target attribute map for the current family
                     explicitAttributes = explicitAttributeSet[family] || (explicitAttributeSet[family] = {});
 
                 // update the attribute map with the new attribute values
                 _.each(attributeSet[family], function (value, name) {
                     if (AttributeUtils.isRegisteredAttribute(definitions, name, options)) {
-                        updateAttributeValue(explicitAttributes, name, value, definitions[name]);
+                        if (updateAttributeValue(explicitAttributes, name, value, definitions[name])) {
+                            changed = true;
+                        }
                     }
                 });
 
@@ -388,50 +511,43 @@ define('io.ox/office/editframework/model/attributedmodel',
 
         // initialization -----------------------------------------------------
 
-        app.onInit(function () {
+        // initialize class members
+        styleCollection = styleFamily ? docModel.getStyleCollection(styleFamily) : null;
 
-            // initialize class members
-            documentStyles = app.getModel().getDocumentStyles();
-            if (styleFamily) {
-                styleCollection = documentStyles.getStyleCollection(styleFamily);
-            }
+        // get all supported attribute families
+        supportedFamilies = Utils.getTokenListOption(initOptions, 'families', []);
+        if (styleCollection) { supportedFamilies = supportedFamilies.concat(styleCollection.getSupportedFamilies()); }
+        supportedFamilies = _.unique(supportedFamilies);
 
-            // get all supported attribute families
-            supportedFamilies = Utils.getTokenListOption(initOptions, 'families', []);
-            if (styleCollection) { supportedFamilies = supportedFamilies.concat(styleCollection.getSupportedFamilies()); }
-            supportedFamilies = _.unique(supportedFamilies);
+        // set the attributes passed to the constructor (filters by supported attributes)
+        this.setAttributes(_.isObject(initAttributes) ? initAttributes : {}, { notify: 'never' });
+        initAttributes = null;
 
-            // set the attributes passed to the constructor (filters by supported attributes)
-            self.setAttributes(_.isObject(initAttributes) ? initAttributes : {}, { notify: 'never' });
-            initAttributes = null;
-            calculateMergedAttributeSet(false);
+        // listen to changes in the parent object, update the merged attributes and notify own listeners
+        if (parentModel && Utils.getBooleanOption(initOptions, 'listenToParent', true)) {
+            this.listenTo(parentModel, 'change:attributes', function () {
+                calculateMergedAttributeSet(true);
+            });
+        }
 
-            // listen to changes in the parent object, update the merged attributes and notify own listeners
-            if (parentModel && Utils.getBooleanOption(initOptions, 'listenToParent', true)) {
-                self.listenTo(parentModel, 'change:attributes', function () {
-                    calculateMergedAttributeSet(true);
-                });
-            }
+        // listen to style sheet changes, update the merged attributes and notify own listeners
+        if (styleCollection) {
+            this.listenTo(styleCollection, 'triggered', function () {
+                calculateMergedAttributeSet(true);
+            });
+        }
 
-            // listen to style sheet changes, update the merged attributes and notify own listeners
-            if (styleCollection) {
-                self.listenTo(styleCollection, 'triggered', function () {
-                    calculateMergedAttributeSet(true);
-                });
-            }
-
-            // update merged attributes silently once after import
-            if (!app.isImportFinished()) {
-                // object may be destroyed before import finishes
-                self.listenTo(app.getImportPromise(), 'done', function () {
-                    calculateMergedAttributeSet(false);
-                });
-            }
-        });
+        // update merged attributes silently once after import
+        if (!docModel.getApp().isImportFinished()) {
+            // object may be destroyed before import finishes
+            this.listenTo(docModel.getApp().getImportPromise(), 'done', function () {
+                calculateMergedAttributeSet(false);
+            });
+        }
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            self = parentModel = documentStyles = styleCollection = null;
+            self = docModel = parentModel = styleCollection = null;
             explicitAttributeSet = mergedAttributeSet = null;
         });
 
