@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -25,15 +25,12 @@ define('io.ox/office/editframework/model/operationsgenerator', [
      *
      * @constructor
      *
-     * @param {EditApplication} app
-     *  The application instance containing this generator.
+     * @param {EditModel} docModel
+     *  The document model containing this generator.
      */
-    function OperationsGenerator(app) {
+    function OperationsGenerator(docModel) {
 
-        var // the document styles (style sheet containers etc.)
-            documentStyles = app.getModel().getDocumentStyles(),
-
-            // the operations buffer
+        var // the operations buffer
             operations = [];
 
         // methods ------------------------------------------------------------
@@ -49,6 +46,17 @@ define('io.ox/office/editframework/model/operationsgenerator', [
             return operations;
         };
 
+        /**
+         * Returns the last generated operation, but does not remove that
+         * operation from this generator instance.
+         *
+         * @returns {Object|Null}
+         *  The last generated operation, if available; otherwise null.
+         */
+        this.getLastOperation = function () {
+            return _.last(operations) || null;
+        };
+
         /**
          * Reverses the entire operations array.
          *
@@ -60,38 +68,47 @@ define('io.ox/office/editframework/model/operationsgenerator', [
             return this;
         };
 
+        /**
+         * Appends the passed JSON operations to the operations array.
+         *
+         * @param {Object|Array<Object>|OperationsGenerator} operations
+         *  A single JSON operation, or an array of JSON operations, or an
+         *  operations generator instance with operations to be appended to the
+         *  operations array. Note that an operations generator passed to this
+         *  method will NOT be cleared after its operations have been appended.
+         *
+         * @returns {OperationsGenerator}
+         *  A reference to this instance.
+         */
+        this.appendOperations = function (ops) {
+            if (ops instanceof OperationsGenerator) {
+                operations = operations.concat(ops.getOperations());
+            } else if (_.isArray(ops)) {
+                operations = operations.concat(ops);
+            } else {
+                operations.push(ops);
+            }
+            return this;
+        };
+
         /**
          * Creates and appends a new operation to the operations array.
          *
          * @param {String} name
          *  The name of the operation.
          *
-         * @param {Object} [operationOptions]
-         *  Additional options that will be stored in the operation.
+         * @param {Object} [properties]
+         *  Additional properties that will be stored in the JSON operation.
          *
          * @returns {Object}
          *  The created operation object.
          */
-        this.generateOperation = function (name, operationOptions) {
-            var operation = _.extend({ name: name }, operationOptions);
+        this.generateOperation = function (name, properties) {
+            var operation = _.extend({ name: name }, properties);
             operations.push(operation);
             return operation;
         };
 
-        /**
-         * Adding operations to this operations generator.
-         *
-         * @param {String[]} ops
-         *  The additional operations.
-         *
-         * @returns {Number}
-         *  The number of operations in the operations generator.
-         */
-        this.addOperations = function (ops) {
-            operations = operations.concat(ops);
-            return operations.length;
-        };
-
         /**
          * Generates an 'insertStyleSheet' operation, if the specified style
          * sheet exists in the style sheet collection, and is marked dirty.
@@ -109,7 +126,7 @@ define('io.ox/office/editframework/model/operationsgenerator', [
         this.generateMissingStyleSheetOperation = function (styleFamily, styleId) {
 
             var // the style sheet container
-                styleSheets = documentStyles.getStyleCollection(styleFamily),
+                styleSheets = docModel.getStyleCollection(styleFamily),
                 // additional properties for the operation
                 operationOptions = null;
 
@@ -123,7 +140,9 @@ define('io.ox/office/editframework/model/operationsgenerator', [
                     styleId: styleId,
                     styleName: styleSheets.getName(styleId),
                     parent: styleSheets.getParentId(styleId),
-                    uiPriority: styleSheets.getUIPriority(styleId)
+                    uiPriority: styleSheets.getUIPriority(styleId),
+                    hidden: styleSheets.isHidden(styleId),
+                    custom: styleSheets.isCustom(styleId)
                 };
                 if (!_.isString(operationOptions.parent)) { delete operationOptions.parent; }
                 if (!_.isNumber(operationOptions.uiPriority)) { delete operationOptions.uiPriority; }
