@@ -6,15 +6,15 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Oliver Specht <oliver.specht@open-xchange.com>
  */
 
-define('io.ox/office/editframework/model/themecollection',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/model/modelobject'
-    ], function (Utils, ModelObject) {
+define('io.ox/office/editframework/model/themecollection', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/baseframework/model/modelobject'
+], function (Utils, ModelObject) {
 
     'use strict';
 
@@ -75,7 +75,8 @@ define('io.ox/office/editframework/model/themecollection',
          *  existing, otherwise null.
          */
         this.getSchemeColor = function (name) {
-            return _.isString(colorScheme[name]) ? colorScheme[name] : null;
+            var color = colorScheme[name];
+            return (typeof color === 'string') ? color : null;
         };
 
     } // class Theme
@@ -89,10 +90,10 @@ define('io.ox/office/editframework/model/themecollection',
      *
      * @extends ModelObject
      *
-     * @param {EditApplication} app
-     *  The root application instance.
+     * @param {EditModel} docModel
+     *  The document model containing this instance.
      */
-    function ThemeCollection(app) {
+    function ThemeCollection(docModel) {
 
         var // themes, mapped by identifier
             themes = {},
@@ -105,9 +106,9 @@ define('io.ox/office/editframework/model/themecollection',
 
         // base constructor ---------------------------------------------------
 
-        ModelObject.call(this, app);
+        ModelObject.call(this, docModel);
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
 
         /**
          * Adds a new theme to this container. An existing theme with the
@@ -154,6 +155,13 @@ define('io.ox/office/editframework/model/themecollection',
             return (name in themes) ? themes[name] : defaultTheme;
         };
 
+        // initialization -----------------------------------------------------
+
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            docModel = themes = defaultTheme = null;
+        });
+
     } // class ThemeCollection
 
     // exports ================================================================
