@@ -6,17 +6,17 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Malte Timmermann <malte.timmermann@open-xchange.com>
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/model/undomanager',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/object/triggerobject'
-    ], function (Utils, TriggerObject) {
+define('io.ox/office/editframework/model/undomanager', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/object/triggerobject'
+], function (Utils, TriggerObject) {
 
     'use strict';
 
@@ -99,8 +99,8 @@ define('io.ox/office/editframework/model/undomanager',
      *
      * @extends TriggerObject
      *
-     * @param {EditApplication} app
-     *  The application containing this undo manager instance.
+     * @param {EditModel} docModel
+     *  The document model containing this undo manager instance.
      *
      * @param {Object} [initOptions]
      *  Optional parameters:
@@ -125,13 +125,13 @@ define('io.ox/office/editframework/model/undomanager',
      *      discarded. All other return values result in pushing the new undo
      *      action onto the undo stack.
      */
-    function UndoManager(app, initOptions) {
+    function UndoManager(docModel, initOptions) {
 
         var // self reference
             self = this,
 
-            // all undo actions
-            actions = [],
+            // the application instance
+            app = docModel.getApp(),
 
             // whether the application manages undo/redo operations on the server
             remoteUndo = Utils.getBooleanOption(initOptions, 'remoteUndo', false),
@@ -139,6 +139,9 @@ define('io.ox/office/editframework/model/undomanager',
             // callback function that implements merging two actions to one action
             mergeUndoActionHandler = Utils.getFunctionOption(initOptions, 'mergeUndoActionHandler', $.noop),
 
+            // all undo actions
+            actions = [],
+
             // index of the next redo action in the 'actions' array
             currentAction = 0,
 
@@ -194,7 +197,9 @@ define('io.ox/office/editframework/model/undomanager',
             // push existing group action to action stack on last group level
             groupLevel -= 1;
             if ((groupLevel === 0) && groupAction) {
-                pushAction(groupAction);
+                if (!app.isOperationsBlockActive()) { // the operations were not cancelled
+                    pushAction(groupAction);
+                }
                 groupAction = null;
             }
 
@@ -267,8 +272,8 @@ define('io.ox/office/editframework/model/undomanager',
          *  The number of undo/redo actions whose operations will be applied.
          *
          * @returns {jQuery.Promise}
-         *  The Promise of a Deferred object that will be resolved or rejected
-         *  after the undo/redo actions have been applied completely.
+         *  A promise that will be resolved or rejected after the undo/redo
+         *  actions have been applied completely.
          */
         function requestAction(type/*, count*/) {
 
@@ -289,18 +294,14 @@ define('io.ox/office/editframework/model/undomanager',
         // methods ------------------------------------------------------------
 
         /**
-         * Clears all undo actions from the stack of the undo manager.
+         * Returns whether the undo stack manages its undo actions locally (the
+         * constructor option 'remoteUndo' was not set).
          *
-         * @returns {UndoManager}
-         *  A reference to this instance.
+         * @returns {Boolean}
+         *  Whether the undo stack manages its undo actions locally.
          */
-        this.clearUndoActions = function () {
-            if (actions.length > 0) {
-                actions = [];
-                currentAction = 0;
-                triggerChangeCountEvent();
-            }
-            return this;
+        this.isLocalUndo = function () {
+            return !remoteUndo;
         };
 
         /**
@@ -315,6 +316,36 @@ define('io.ox/office/editframework/model/undomanager',
             return !processing && app.isImportFinished();
         };
 
+        /**
+         * Clears all undo actions from the stack of the undo manager.
+         *
+         * @returns {UndoManager}
+         *  A reference to this instance.
+         */
+        this.clearUndoActions = function () {
+            if (actions.length > 0) {
+                actions = [];
+                currentAction = 0;
+                triggerChangeCountEvent();
+            }
+            return this;
+        };
+
+        /**
+         * Clears all undo actions in the stack that are above the current
+         * pointer position. This disables the redo stack.
+         *
+         * @returns {UndoManager}
+         *  A reference to this instance.
+         */
+        this.disableRedo = function () {
+
+            if (currentAction < actions.length) {
+                actions.splice(currentAction);
+            }
+            return this;
+        };
+
         /**
          * Opens an undo action group and executes the specified callback
          * function. All undo operations added by the callback function will be
@@ -323,17 +354,16 @@ define('io.ox/office/editframework/model/undomanager',
          *
          * @param {Function} callback
          *  The callback function that will be executed while the undo action
-         *  group is open. If the function returns a Deferred object or a
-         *  Promise, the undo group remains opened asynchronously until this
-         *  object will be resolved or rejected by the callback.
+         *  group is open. If the function returns a promise, the undo group
+         *  remains opened asynchronously until this object will be resolved or
+         *  rejected by the callback.
          *
          * @param {Object} [context]
          *  The context the callback function will be bound to.
          *
          * @returns {Any}
          *  The return value of the callback function. In asynchronous mode,
-         *  this will be the Deferred object or Promise returned by the
-         *  callback function.
+         *  this will be the promise returned by the callback function.
          */
         this.enterUndoGroup = function (callback, context) {
 
@@ -408,13 +438,15 @@ define('io.ox/office/editframework/model/undomanager',
          *  The number of undo actions whose undo operations will be applied.
          *
          * @returns {jQuery.Promise}
-         *  The Promise of a Deferred object that will be resolved or rejected
-         *  after the undo action has been applied completely.
+         *  A promise that will be resolved or rejected after the undo action
+         *  has been applied completely.
          */
         this.undo = function (count) {
 
             var // the resulting Promise
-                promise = null;
+                promise = null,
+                // collect all undo operations into a single array
+                operations = [];
 
             this.trigger('undo:before');
             processing = true;
@@ -423,20 +455,25 @@ define('io.ox/office/editframework/model/undomanager',
                 // request undo from remote undo manager
                 promise = requestAction('undo', count);
             } else {
-                // apply the undo operations directly
-                promise = $.when();
+
+                // collect the undo operations into a single array
                 count = _.isNumber(count) ? count : 1;
                 while ((currentAction > 0) && (count > 0)) {
                     currentAction -= 1;
-                    // Fix for 30954: Create a deep copy of the undo operations
-                    app.getModel().applyOperations(_.copy(actions[currentAction].undoOperations, true));
+                    operations = operations.concat(actions[currentAction].undoOperations);
                     count -= 1;
                 }
+
+                // apply all collected operations
+                // bug 30954: always create a deep copy of the operations
+                operations = _.copy(operations, true);
+                promise = docModel.applyOperations(operations, { async: true });
             }
 
+            // notify listeners after all operations have been applied
             return promise.always(function () {
                 processing = false;
-                self.trigger('undo:after');
+                self.trigger('undo:after', operations);
                 triggerChangeCountEvent();
             });
         };
@@ -458,13 +495,15 @@ define('io.ox/office/editframework/model/undomanager',
          *  The number of undo actions whose redo operations will be applied.
          *
          * @returns {jQuery.Promise}
-         *  The Promise of a Deferred object that will be resolved or rejected
-         *  after the redo action has been applied completely.
+         *  A promise that will be resolved or rejected after the redo action
+         *  has been applied completely.
          */
         this.redo = function (count) {
 
             var // the resulting Promise
-                promise = null;
+                promise = null,
+                // collect all redo operations into a single array
+                operations = [];
 
             this.trigger('redo:before');
             processing = true;
@@ -473,37 +512,48 @@ define('io.ox/office/editframework/model/undomanager',
                 // request redo from remote undo manager
                 promise = requestAction('redo', count);
             } else {
-                // apply the redo operations directly
-                promise = $.when();
+
+                // collect the redo operations into a single array
                 count = _.isNumber(count) ? count : 1;
                 while ((currentAction < actions.length) && (count > 0)) {
-                    // Fix for 30954: Create a deep copy of the redo operations
-                    app.getModel().applyOperations(_.copy(actions[currentAction].redoOperations, true));
+                    operations = operations.concat(actions[currentAction].redoOperations);
                     currentAction += 1;
                     count -= 1;
                 }
+
+                // apply all collected operations
+                // bug 30954: always create a deep copy of the operations
+                operations = _.copy(operations, true);
+                promise = docModel.applyOperations(operations, { async: true });
             }
 
+            // notify listeners after all operations have been applied
             return promise.always(function () {
                 processing = false;
-                self.trigger('redo:after');
+                self.trigger('redo:after', operations);
                 triggerChangeCountEvent();
             });
         };
 
         // initialization -----------------------------------------------------
 
+        // initial notification about undo stack size
+        app.onImportSuccess(triggerChangeCountEvent);
+
         // remote undo stack: listen to update push messages from application
         if (remoteUndo) {
             this.listenTo(app, 'docs:update', remoteUpdateHandler);
         }
 
-        // initial notification about undo stack size
-        app.onImportSuccess(triggerChangeCountEvent);
+        // destroy class members on destruction
+        this.registerDestructor(function () {
+            self = initOptions = null;
+            app = docModel = mergeUndoActionHandler = actions = null;
+        });
 
     } // class UndoManager
 
-   // exports =================================================================
+    // exports ================================================================
 
     // derive this class from class TriggerObject
     return TriggerObject.extend({ constructor: UndoManager });
