@@ -6,16 +6,16 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/utils/mixedborder',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/utils/color',
-     'io.ox/office/editframework/utils/border'
-    ], function (Utils, Color, Border) {
+define('io.ox/office/editframework/utils/mixedborder', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/utils/color',
+    'io.ox/office/editframework/utils/border'
+], function (Utils, Color, Border) {
 
     'use strict';
 
@@ -260,11 +260,12 @@ define('io.ox/office/editframework/utils/mixedborder',
                 // whether the original border is visible
                 origVisible = MixedBorder.isVisibleBorder(origBorder);
 
-            // toggle visibility if specified
-            visible = _.isNull(visible) ? !origVisible : visible;
+            // toggle visibility if specified (but switch from 'mixed' state to visible)
+            visible = _.isNull(visible) ? ((origVisible && origBorder.mixed) || !origVisible) : visible;
 
             // build the border attribute value according to visibility and original borders
             borderAttributes[borderAttrName] = _.copy(visible ? (origVisible ? origBorder : defaultBorder) : Border.NONE, true);
+            delete borderAttributes[borderAttrName].mixed;
         }
 
         if (_.isObject(borderMode)) {
