@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -67,6 +67,13 @@ define('io.ox/office/editframework/utils/operations', [
          */
         DELETE_STYLESHEET: 'deleteStyleSheet',
 
+        /**
+         * :)
+         *
+         * @constant
+         */
+        CHANGE_STYLESHEET: 'changeStyleSheet',
+
         /**
          * Name of the operation that does nothing (internal usage).
          *
