@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/editframework/view/control/acquireeditbutton', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/button',
     'io.ox/office/editframework/view/editlabels',
-    'gettext!io.ox/office/editframework'
+    'gettext!io.ox/office/editframework/main'
 ], function (Utils, Button, Labels, gt) {
 
     'use strict';
@@ -28,8 +28,11 @@ define('io.ox/office/editframework/view/control/acquireeditbutton', [
      * @constructor
      *
      * @extends Button
+     *
+     * @param {EditView} docView
+     *  The document view instance containing this control.
      */
-    function AcquireEditButton(app, initOptions) {
+    function AcquireEditButton(docView, initOptions) {
 
         var // self reference
             self = this,
@@ -55,7 +58,6 @@ define('io.ox/office/editframework/view/control/acquireeditbutton', [
         function startAnimation() {
             stopAnimation();
             self.setIcon('fa-refresh', { classes: 'fa-spin' });
-            timer = self.executeDelayed(stopAnimation, 30000);
         }
 
         /**
@@ -70,16 +72,14 @@ define('io.ox/office/editframework/view/control/acquireeditbutton', [
         // initialization -----------------------------------------------------
 
         // start and stop the spinning animation
-        app.onInit(function () {
-            self.listenTo(app, 'docs:editrights:acquire', startAnimation);
-            self.listenTo(app, 'docs:editrights:decline', stopAnimation);
-            self.listenTo(app.getModel(), 'change:editmode', stopAnimation);
-        });
+        this.listenTo(docView.getApp(), 'docs:editrights:acquire', startAnimation);
+        this.listenTo(docView.getApp(), 'docs:editrights:decline', stopAnimation);
+        this.listenTo(docView.getDocModel(), 'change:editmode', stopAnimation);
 
         // destroy class members on destruction
         this.registerDestructor(function () {
             stopAnimation();
-            app = initOptions = self = null;
+            docView = initOptions = self = null;
         });
 
     } // class AcquireEditButton
