@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,7 +14,7 @@
 define('io.ox/office/editframework/view/control/applicationstatuslabel', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/label',
-    'gettext!io.ox/office/editframework'
+    'gettext!io.ox/office/editframework/main'
 ], function (Utils, Label, gt) {
 
     'use strict';
@@ -27,8 +27,11 @@ define('io.ox/office/editframework/view/control/applicationstatuslabel', [
      * @constructor
      *
      * @extends Label
+     *
+     * @param {EditView} docView
+     *  The document view instance containing this control.
      */
-    function ApplicationStatusLabel(app, initOptions) {
+    function ApplicationStatusLabel(docView, initOptions) {
 
         var // self reference
             self = this,
@@ -56,13 +59,13 @@ define('io.ox/office/editframework/view/control/applicationstatuslabel', [
         });
 
         // update caption according to application status
-        this.listenTo(app, 'docs:state', function (state) {
+        this.listenTo(docView.getApp(), 'docs:state', function (state) {
             self.setCaption(captions[state]);
         });
 
         // destroy class members on destruction
         this.registerDestructor(function () {
-            app = self = null;
+            docView = self = null;
         });
 
     } // class ApplicationStatusLabel
