@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,7 +14,7 @@
 define('io.ox/office/editframework/view/control/bordermodepicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/radiolist',
-    'gettext!io.ox/office/editframework'
+    'gettext!io.ox/office/editframework/main'
 ], function (Utils, RadioList, gt) {
 
     'use strict';
@@ -100,7 +100,7 @@ define('io.ox/office/editframework/view/control/bordermodepicker', [
             dropDownVersion: { label: Utils.getStringOption(initOptions, 'tooltip') }
         }, initOptions, {
             itemMatcher: matchBorder,
-            updateCaptionMode: 'none',
+            updateCaptionMode: 'none'
         }));
 
         // private methods ----------------------------------------------------
@@ -163,7 +163,7 @@ define('io.ox/office/editframework/view/control/bordermodepicker', [
 
             // update single-border list items
             if (_.isString(entryValue)) {
-             // return false for null values
+                // return false for null values
                 return (entryValue in value) && (value[entryValue] === true);
             }
 
