@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,11 +14,11 @@
 define('io.ox/office/editframework/view/control/borderstylepicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/forms',
-    'io.ox/office/tk/canvaswrapper',
+    'io.ox/office/tk/canvas/canvas',
     'io.ox/office/tk/control/radiolist',
     'io.ox/office/editframework/utils/border',
     'io.ox/office/editframework/view/editlabels'
-], function (Utils, Forms, CanvasWrapper, RadioList, Border, Labels) {
+], function (Utils, Forms, Canvas, RadioList, Border, Labels) {
 
     'use strict';
 
@@ -53,7 +53,7 @@ define('io.ox/office/editframework/view/control/borderstylepicker', [
     var getBorderStyleBitmapUrl = (function () {
 
         var // the canvas element used to generate the bitmaps
-            canvasWrapper = new CanvasWrapper(),
+            canvas = new Canvas(),
 
             // maps unique bitmap keys values to the data URLs
             bitmapUrls = {};
@@ -72,11 +72,11 @@ define('io.ox/office/editframework/view/control/borderstylepicker', [
             }
 
             // initialize the canvas with the passed bitmap size (implicitly clears the canvas)
-            canvasWrapper.initialize({ width: width, height: height });
+            canvas.initialize({ width: width, height: height });
 
+            // create the bitmap
             if (borderStyle.style !== 'none') {
-                // create the bitmap
-                canvasWrapper.render(function (context) {
+                canvas.render(function (context) {
 
                     var // the path with all line segments (of double lines)
                         path = context.createPath(),
@@ -98,7 +98,7 @@ define('io.ox/office/editframework/view/control/borderstylepicker', [
             }
 
             // convert the bitmap to a data URL
-            return (bitmapUrls[bitmapKey] = canvasWrapper.getDataURL());
+            return (bitmapUrls[bitmapKey] = canvas.getDataURL());
         }
 
         return getBorderStyleBitmapUrl;
@@ -138,10 +138,7 @@ define('io.ox/office/editframework/view/control/borderstylepicker', [
      */
     function BorderStylePicker(listEntries, initOptions) {
 
-        var // the icon-like element in the drop-down button
-            styleBoxNode = null,
-
-            // maps control values to border style descriptors
+        var // maps control values to border style descriptors
             borderStyles = {};
 
         // base constructor ---------------------------------------------------
@@ -194,7 +191,7 @@ define('io.ox/office/editframework/view/control/borderstylepicker', [
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            initOptions = styleBoxNode = null;
+            listEntries = initOptions = null;
         });
 
     } // class BorderStylePicker
