@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/editframework/view/control/borderwidthpicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/combofield',
     'io.ox/office/editframework/view/editlabels',
-    'gettext!io.ox/office/editframework'
+    'gettext!io.ox/office/editframework/main'
 ], function (Utils, ComboField, EditLabels, gt) {
 
     'use strict';
@@ -50,7 +50,7 @@ define('io.ox/office/editframework/view/control/borderwidthpicker', [
 
         // initialization -----------------------------------------------------
 
-        _.each([0.5, 1, 1.5, 2, 2.5, 3, 4, 6], function (size) {
+        [0.5, 1, 1.5, 2, 2.5, 3, 4, 6].forEach(function (size) {
             this.createListEntry(size, {
                 label: gt.format(
                     //#. A single list entry of a GUI border width picker.
