@@ -6,17 +6,19 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
+ * @author Jonas Regier <jonas.regier@open-xchange.com>
  */
 
 define('io.ox/office/editframework/view/control/colorpicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/radiolist',
     'io.ox/office/editframework/utils/color',
-    'gettext!io.ox/office/editframework'
-], function (Utils, RadioList, Color, gt) {
+    'io.ox/office/tk/control/colorslider',
+    'gettext!io.ox/office/editframework/main'
+], function (Utils, RadioList, Color, ColorSlider, gt) {
 
     'use strict';
 
@@ -37,16 +39,16 @@ define('io.ox/office/editframework/view/control/colorpicker', [
         // definitions for the theme color table (in order as shown in the GUI color picker)
         // transformation table: 0 = pure color; negative values = 'shade'; positive values = 'tint'
         SCHEME_COLOR_DEFINITIONS = [
-            { name: 'background1', label: getBackgroundColorName(1), transformations: [    0, -242, -217, -191, -166, -128 ] },
-            { name: 'text1',       label: getTextColorName(1),       transformations: [  128,  166,  191,  217,  242,    0 ] },
-            { name: 'background2', label: getBackgroundColorName(2), transformations: [    0, -230, -191, -128,  -64,  -26 ] },
-            { name: 'text2',       label: getTextColorName(2),       transformations: [   51,  102,  153,    0, -191, -128 ] },
-            { name: 'accent1',     label: getAccentColorName(1),     transformations: [   51,  102,  153,    0, -191, -128 ] },
-            { name: 'accent2',     label: getAccentColorName(2),     transformations: [   51,  102,  153,    0, -191, -128 ] },
-            { name: 'accent3',     label: getAccentColorName(3),     transformations: [   51,  102,  153,    0, -191, -128 ] },
-            { name: 'accent4',     label: getAccentColorName(4),     transformations: [   51,  102,  153,    0, -191, -128 ] },
-            { name: 'accent5',     label: getAccentColorName(5),     transformations: [   51,  102,  153,    0, -191, -128 ] },
-            { name: 'accent6',     label: getAccentColorName(6),     transformations: [   51,  102,  153,    0, -191, -128 ] }
+            { name: 'background1', label: getBackgroundColorName(1), transformations: [0, -242, -217, -191, -166, -128] },
+            { name: 'text1',       label: getTextColorName(1),       transformations: [128, 166, 191,  217, 242, 0] },
+            { name: 'background2', label: getBackgroundColorName(2), transformations: [0, -230, -191, -128, -64, -26] },
+            { name: 'text2',       label: getTextColorName(2),       transformations: [51, 102, 153, 0, -191, -128] },
+            { name: 'accent1',     label: getAccentColorName(1),     transformations: [51, 102, 153, 0, -191, -128] },
+            { name: 'accent2',     label: getAccentColorName(2),     transformations: [51, 102, 153, 0, -191, -128] },
+            { name: 'accent3',     label: getAccentColorName(3),     transformations: [51, 102, 153, 0, -191, -128] },
+            { name: 'accent4',     label: getAccentColorName(4),     transformations: [51, 102, 153, 0, -191, -128] },
+            { name: 'accent5',     label: getAccentColorName(5),     transformations: [51, 102, 153, 0, -191, -128] },
+            { name: 'accent6',     label: getAccentColorName(6),     transformations: [51, 102, 153, 0, -191, -128] }
         ];
 
     // private global functions ===============================================
@@ -98,43 +100,65 @@ define('io.ox/office/editframework/view/control/colorpicker', [
     /**
      * Creates a control with a drop-down menu used to choose a color from a
      * set of color items. Shows a selection of standard colors, and a table of
-     * shaded scheme colors from the current document theme.
+     * shaded scheme colors from the current document theme. It has two display
+     * modes. One used on small devices with a colorslider to browse
+     * through the scheme colors and a mode for all other devices, which has a
+     * color table to display all scheme colors.
      *
      * @constructor
      *
      * @extends RadioList
      *
-     * @param {EditApplication} app
-     *  The application containing this control group.
+     * @param {EditView} docView
+     *  The document view instance containing this control.
      *
-     * @param {String|Object} autoColor
-     *  Additional information needed to resolve the automatic color. Can be a
-     *  string for predefined automatic colors, or an explicit color object
-     *  with a value other than the automatic color. See method
-     *  Color.getCssColor() for more details.
+     * @param {String|Null} colorType
+     *  The color type used to resolve the automatic color entry. See method
+     *  Color.resolve() for a list of supported color types. If set to null,
+     *  the button for the automatic color will not be created at all.
      *
      * @param {Object} [initOptions]
-     *  Optional parameters. Supports all options of the RadioList base class.
+     *  Optional parameters. Supports all options of the RadioList base class,
+     *  and the following options:
+     *  @param.
      */
-    function ColorPicker(app, autoColor, initOptions) {
+    function ColorPicker(docView, colorType, initOptions) {
 
         var // self reference
             self = this,
 
+            // the document model
+            docModel = docView.getDocModel(),
+
             // the color box in the drop-down button
             colorBox = $('<div>').addClass('color-preview-box'),
 
-            // the style sheet containers of the document
-            documentStyles = app.getModel().getDocumentStyles(),
+            // the replacement color for the automatic color entry
+            autoColor = new Color('auto'),
 
-            // show the 'auto-color' entry in the drop down
-            showAutoColor = Utils.getBooleanOption(initOptions, 'showAutoColor', true);
+            // reference to 'theme' section menu
+            themeMenu = null,
+
+            // the color slider
+            colorSlider = null,
+
+            // shown color values for all colorFields in the colorSlider
+            shownSchemeColor = [],
+
+            // the default handle position in the colorSlider
+            defaultHandlePosition = 2,
+
+            // array with all used scheme colors in the same order as the buttons are created
+            colorTable = [],
+
+            // reference to all scheme color rows in the DOM
+            schemeColorRows = null;
 
         // base constructor ---------------------------------------------------
 
         RadioList.call(this, Utils.extendOptions({
             itemDesign: 'grid',
-            gridColumns: 10,
+            gridColumns: (Utils.SMALL_DEVICE) ? 6 : 10,
             updateCaptionMode: 'none',
             itemMatcher: sameColors,
             dropDownVersion: { label: Utils.getStringOption(initOptions, 'tooltip') }
@@ -145,25 +169,44 @@ define('io.ox/office/editframework/view/control/colorpicker', [
         /**
          * Converts the passed color value to a CSS color string.
          */
-        function setBackgroundColor(boxNode, color) {
-            var cssColor = _.isObject(color) ? documentStyles.getCssColor(color, autoColor) : 'transparent';
+        function setBackgroundColor(boxNode, jsonColor) {
+            var cssColor = _.isObject(jsonColor) ? docModel.getCssColor(jsonColor, autoColor.isAuto() ? colorType : autoColor.toJSON()) : 'transparent';
             boxNode.css('background-color', cssColor);
         }
 
         /**
          * Initializes a new color item button.
          */
-        function createItemHandler(event, buttonNode, color) {
-            var boxNode = $('<span class="color-button">');
-            setBackgroundColor(boxNode, color);
+        function createItemHandler(event, buttonNode, jsonColor) {
+            var
+                className = (Utils.SMALL_DEVICE) ? 'color-button-mobile' : 'color-button',
+                boxNode = $('<span class="' + className + '">');
+            setBackgroundColor(boxNode, jsonColor);
             buttonNode.prepend(boxNode);
         }
 
         /**
          * Updates the color box in the drop-down menu button.
          */
-        function updateHandler(color) {
-            setBackgroundColor(colorBox, color);
+        function updateHandler(jsonColor) {
+            setBackgroundColor(colorBox, jsonColor);
+        }
+
+        /**
+         * Get the corresponding scheme color row to a given color object.
+         */
+        function getColorRowNumber (color) {
+            var colorRow = defaultHandlePosition;
+            var temp;
+            for (var n = 0; n < 10; n++) {
+                temp = colorTable[n];
+                for (var i = 0; i < 6; i++) {
+                    if (sameColors(temp[i], color)) {
+                        colorRow = n;
+                    }
+                }
+            }
+            return colorRow;
         }
 
         /**
@@ -223,7 +266,7 @@ define('io.ox/office/editframework/view/control/colorpicker', [
             }
 
             // special treatment for fall-back values in ODF mode
-            return app.isODF() &&
+            return docView.getApp().isODF() &&
                 ((color1.type === 'scheme') && _.isString(color1.fallbackValue) && (color1.fallbackValue.toLowerCase() === color2Value)) ||
                 ((color2.type === 'scheme') && _.isString(color2.fallbackValue) && (color2.fallbackValue.toLowerCase() === color1Value));
         }
@@ -240,7 +283,7 @@ define('io.ox/office/editframework/view/control/colorpicker', [
                     tint = encoded > 0,
                     value = Math.round(Math.abs(encoded) / 255 * 100000),
                     // the theme color object
-                    color = { type: 'scheme', value: definition.name },
+                    color = new Color('scheme', definition.name),
                     // the data value added at the button
                     dataValue = definition.name,
                     // description of the color (tool tip)
@@ -251,7 +294,7 @@ define('io.ox/office/editframework/view/control/colorpicker', [
                     fallbackValue = null;
 
                 if (value !== 0) {
-                    Color.appendTransformation(color, tint ? 'tint' : 'shade', value);
+                    color.transform(tint ? 'tint' : 'shade', value);
                     dataValue += (tint ? '-lighter' : '-darker') + percent;
                     label = tint ?
                         //#. The full name of a light theme color (a base color lightened by a specific percentage value)
@@ -269,24 +312,77 @@ define('io.ox/office/editframework/view/control/colorpicker', [
                 }
 
                 // store the resulting RGB color in the color value object
-                if (Color.isThemeColor(color)) {
-                    fallbackValue = documentStyles.getCssColor(color, 'fill');
-                    if (fallbackValue[0] === '#') {
-                        color.fallbackValue = fallbackValue.slice(1);
-                    } else {
-                        Utils.warn('ColorPicker.fillSchemeColorRow(): unexpected CSS fallback color: ', fallbackValue);
-                    }
-                }
-
+                fallbackValue = docModel.resolveColor(color, 'fill').css.slice(1);
+                color = color.toJSON();
+                color.fallbackValue = fallbackValue;
                 self.createOptionButton(color, { section: 'theme', tooltip: label, dataValue: dataValue });
             });
         }
 
+        /**
+         * Creates the option buttons for one table row for a given scheme color number on small devices.
+         * The orientation is different compared to fillSchemeColorRow(). It returns an array with color
+         * object in the same order as the buttons are created.
+         */
+        function fillSchemeColorRowMobile(colorNumber) {
+            var definition = SCHEME_COLOR_DEFINITIONS[colorNumber];
+            // saved colors in the same order as the buttons are created
+            var colorRowTable = [];
+
+            for (var i = 0; i < 6; i++) {
+                var // the encoded transformation
+                    encoded = definition.transformations[i],
+                    // decoded tint/shade value
+                    tint = encoded > 0,
+                    value = Math.round(Math.abs(encoded) / 255 * 100000),
+                    // the theme color object
+                    color = new Color('scheme', definition.name),
+                    // the data value added at the button
+                    dataValue = definition.name,
+                    // description of the color (tool tip)
+                    label = definition.label,
+                    // shade/tint value as integral percentage
+                    percent = (100 - Math.round(value / 1000)),
+                    // fall-back value for format without theme support
+                    fallbackValue = null;
+
+                if (value !== 0) {
+                    color.transform(tint ? 'tint' : 'shade', value);
+                    dataValue += (tint ? '-lighter' : '-darker') + percent;
+                    label = tint ?
+                        //#. The full name of a light theme color (a base color lightened by a specific percentage value)
+                        //#. Example result: "Green, lighter 20%"
+                        //#. %1$s is the name of the base color
+                        //#. %2$d is the percentage value, followed by a literal percent sign
+                        //#, c-format
+                        gt('%1$s, lighter %2$d%', label, _.noI18n(percent)) :
+                        //#. The full name of a dark theme color (a base color darkened by a specific percentage value)
+                        //#. Example result: "Green, darker 20%"
+                        //#. %1$s is the name of the base color
+                        //#. %2$d is the percentage value, followed by a literal percent sign
+                        //#, c-format
+                        gt('%1$s, darker %2$d%', label, _.noI18n(percent));
+                }
+
+                // store the resulting RGB color in the color value object
+                fallbackValue = docModel.resolveColor(color, 'fill').css.slice(1);
+                color = color.toJSON();
+                color.fallbackValue = fallbackValue;
+                colorRowTable[i] = color;
+                self.createOptionButton(color, { section: 'theme', tooltip: label, dataValue: dataValue });
+
+                // get the colors values from the current scheme for colorSlider.js colorFields
+                if (i === 3) {
+                    shownSchemeColor[colorNumber] = fallbackValue;
+                }
+            }
+            return colorRowTable;
+        }
         /**
          * Returns the button label for the automatic color.
          */
         function getAutoColorLabel() {
-            return Color.isTransparentColor(Color.AUTO, autoColor) ?
+            return (autoColor === 'transparent') ?
                 /*#. no fill color, transparent */ gt('No color') :
                 /*#. automatic text color (white on dark backgrounds, otherwise black) */ gt('Automatic color');
         }
@@ -296,25 +392,56 @@ define('io.ox/office/editframework/view/control/colorpicker', [
          */
         function initializeColorTable() {
 
-            self.clearMenu();
+            self.clearMenu({ except: '.title' });
 
-            // add automatic color
-            if (showAutoColor) {
-                self.createMenuSection('auto', { gridColumns: 1 })
-                    .createOptionButton(Color.AUTO, { section: 'auto', label: getAutoColorLabel(), dataValue: 'auto' });
+            function addAutoColor() {
+                // add automatic color
+                if (_.isString(colorType)) {
+                    self.createMenuSection('auto', { gridColumns: 1 })
+                        .createOptionButton(Color.AUTO, { section: 'auto', label: getAutoColorLabel(), dataValue: 'auto' });
+                }
             }
 
-            // add scheme colors
-            if (documentStyles.hasSchemeColors()) {
-                self.createMenuSection('theme', { label: gt('Theme Colors') });
-                _.times(SCHEME_COLOR_DEFINITIONS[0].transformations.length, fillSchemeColorRow);
+            function addSchemeColors() {
+                // different creation method on small devices
+                var method = (Utils.SMALL_DEVICE) ? fillSchemeColorRowMobile : fillSchemeColorRow,
+                // different length due to a different orientation on small devices
+                    length = (Utils.SMALL_DEVICE) ? SCHEME_COLOR_DEFINITIONS.length : SCHEME_COLOR_DEFINITIONS[0].transformations.length;
+
+                // add scheme colors
+                if (docModel.hasSchemeColors()) {
+                    themeMenu = self.createMenuSection('theme', { label: gt('Theme Colors') });
+                    colorTable = _.times(length, method);
+                }
             }
 
-            // add predefined colors
-            self.createMenuSection('standard', { label: gt('Standard Colors') });
-            _.each(BUILTIN_COLOR_DEFINITIONS, function (definition) {
-                self.createOptionButton(definition.color, { section: 'standard', tooltip: definition.label, dataValue: definition.name });
-            });
+            function addPreColors() {
+                // add predefined colors
+                self.createMenuSection('standard', { label: gt('Standard Colors') });
+                _.each(BUILTIN_COLOR_DEFINITIONS, function (definition) {
+                    self.createOptionButton(definition.color, { section: 'standard', tooltip: definition.label, dataValue: definition.name });
+                });
+            }
+            // section order according to the device type
+            if (Utils.SMALL_DEVICE) {
+                addAutoColor();
+                addPreColors();
+                addSchemeColors();
+            } else {
+                addAutoColor();
+                addSchemeColors();
+                addPreColors();
+            }
+        }
+
+        /**
+         *  unhides a color row for a given color and hides the last color.
+         */
+        function showActiveSchemeColorRow (colorNumber, lastColor) {
+            // hides the last selected row in the mobile color picker
+            schemeColorRows.eq(lastColor)[0].style.display = 'none';
+            // show the row from the given color number
+            schemeColorRows.eq(colorNumber)[0].style.display = 'block';
         }
 
         /**
@@ -325,22 +452,18 @@ define('io.ox/office/editframework/view/control/colorpicker', [
         // public methods -----------------------------------------------------
 
         /**
-         * Changes the information needed to resolve the automatic color, and
-         * repaints the menu list item for the automatic color.
+         * Sets the specified color to render the color box in the menu list
+         * item representing the automatic color, and repaints that list item.
          *
-         * @param {String|Object} newAutoColor
-         *   Can be a string for predefined automatic colors, or an explicit
-         *   color object with a value other than the automatic color. See
-         *   method Color.getCssColor() for more details.
+         * @param {Color} color
+         *  The new replacement color for the automatic color entry.
          *
          * @returns {ColorPicker}
          *  A reference to this instance.
          */
-        this.setAutoColor = function (newAutoColor) {
-            if (!_.isEqual(autoColor, newAutoColor)) {
-                autoColor = newAutoColor;
-                setBackgroundColor(this.getMenu().findItemNodes(Color.AUTO).find('.color-button'), Color.AUTO);
-            }
+        this.setAutoColor = function (color) {
+            autoColor = color;
+            setBackgroundColor(this.getMenu().findItemNodes(Color.AUTO).find('.color-button'), Color.AUTO);
             return this;
         };
 
@@ -361,14 +484,37 @@ define('io.ox/office/editframework/view/control/colorpicker', [
         // lazy initialization of the color table
         this.getMenu().one('popup:beforeshow', initializeColorTable);
 
+        if (Utils.SMALL_DEVICE) {
+            // creates the colorSlider
+            this.getMenu().one('popup:beforeshow', function () {
+                colorSlider = new ColorSlider(shownSchemeColor, { defaultHandlePosition: defaultHandlePosition });
+                // add slider markup to menu
+                self.getMenu().appendContentNodes((colorSlider.getColorSliderMarkUp()));
+                // get all scheme color rows in the DOM
+                schemeColorRows = themeMenu.getMenu().getSectionNode('theme').find('.grid-row');
+                // initial hide for all scheme color buttons
+                schemeColorRows.hide();
+                // unhides the row for the currently active color
+                showActiveSchemeColorRow(defaultHandlePosition, defaultHandlePosition);
+
+                self.listenTo(colorSlider, 'colorSlider:showRow', function (event, colorNumber) {
+                    showActiveSchemeColorRow(colorNumber.colorNumber, colorNumber.lastColor);
+                });
+            });
+            // updates the handle position in the colorSlider
+            this.getMenu().on('popup:beforeshow', function () {
+                colorSlider.moveHandleToCorrespondingColor(getColorRowNumber(self.getValue()));
+            });
+        }
+
         // refresh color buttons in the drop-down menu after theme has changed
-        this.listenTo(app.getImportPromise(), 'done', function () {
-            self.listenTo(app.getModel().getThemeCollection(), 'triggered', initializeColorTableDebounced);
+        this.listenTo(docView.getApp().getImportPromise(), 'done', function () {
+            self.listenTo(docModel.getThemeCollection(), 'triggered', initializeColorTableDebounced);
         });
 
         // destroy all members on destruction
         this.registerDestructor(function () {
-            self = colorBox = documentStyles = null;
+            self = docView = docModel = colorBox = themeMenu = colorSlider = shownSchemeColor = defaultHandlePosition = colorTable = schemeColorRows = null;
         });
 
     } // class ColorPicker
