@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/editframework/view/control/filenamefield', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/forms',
     'io.ox/office/tk/control/textfield',
-    'gettext!io.ox/office/editframework'
+    'gettext!io.ox/office/editframework/main'
 ], function (Utils, Forms, TextField, gt) {
 
     'use strict';
@@ -28,12 +28,18 @@ define('io.ox/office/editframework/view/control/filenamefield', [
      * @constructor
      *
      * @extends TextField
+     *
+     * @param {EditView} docView
+     *  The document view instance containing this control.
      */
-    function FileNameField(app) {
+    function FileNameField(docView) {
 
         var // self reference
             self = this,
 
+            // the document model
+            docModel = docView.getDocModel(),
+
             // a special busy spinner visible while committing the file name
             busyOverlayNode = $('<div class="busy-overlay">').hide().busy();
 
@@ -55,7 +61,7 @@ define('io.ox/office/editframework/view/control/filenamefield', [
          * read-only mode.
          */
         function updateToolTip() {
-            Forms.setToolTip(self.getNode(), app.getModel().getEditMode() ? gt('Rename document') : _.noI18n(app.getFullFileName()));
+            Forms.setToolTip(self.getNode(), { tooltip: docModel.getEditMode() ? gt('Rename document') : _.noI18n(docView.getApp().getFullFileName()) });
         }
 
         // initialization -----------------------------------------------------
@@ -67,14 +73,12 @@ define('io.ox/office/editframework/view/control/filenamefield', [
         this.on('group:focus group:blur', function () { busyOverlayNode.hide(); });
 
         // set document name as tool tip in read-only mode
-        app.onInit(function () {
-            app.getModel().on('change:editmode', updateToolTip);
-            updateToolTip();
-        });
+        this.listenTo(docModel, 'change:editmode', updateToolTip);
+        updateToolTip();
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = self = busyOverlayNode = null;
+            self = docView = docModel = busyOverlayNode = null;
         });
 
     } // class FileNameField
