@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,7 +14,7 @@
 define('io.ox/office/editframework/view/control/fontfamilypicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/combofield',
-    'gettext!io.ox/office/editframework'
+    'gettext!io.ox/office/editframework/main'
 ], function (Utils, ComboField, gt) {
 
     'use strict';
@@ -28,16 +28,19 @@ define('io.ox/office/editframework/view/control/fontfamilypicker', [
      *
      * @extends ComboField
      *
+     * @param {EditView} docView
+     *  The document view instance containing this control.
+     *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options of the base class ComboField.
      */
-    function FontFamilyPicker(app, initOptions) {
+    function FontFamilyPicker(docView, initOptions) {
 
         var // self reference
             self = this,
 
             // the collection of fonts of the edited document
-            fontCollection = null;
+            fontCollection = docView.getDocModel().getFontCollection();
 
         // base constructor ---------------------------------------------------
 
@@ -71,12 +74,8 @@ define('io.ox/office/editframework/view/control/fontfamilypicker', [
 
         // initialization -----------------------------------------------------
 
-        app.onInit(function () {
-            fontCollection = app.getModel().getFontCollection();
-        });
-
         // add all known fonts after import and changed fonts
-        this.listenTo(app.getImportPromise(), 'done', function () {
+        this.listenTo(docView.getApp().getImportPromise(), 'done', function () {
             fillList();
             self.listenTo(fontCollection, 'triggered', fillList);
         });
