@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/editframework/view/control/languagepicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/io',
     'io.ox/office/tk/control/radiolist',
-    'gettext!io.ox/office/editframework'
+    'gettext!io.ox/office/editframework/main'
 ], function (Utils, IO, RadioList, gt) {
 
     'use strict';
@@ -23,6 +23,7 @@ define('io.ox/office/editframework/view/control/languagepicker', [
     var // all supported languages, mapped by locale code
         LOCALE_DEFINITIONS = [
             { locale: 'en-US', label: gt('English (US)') },
+            { locale: 'en-GB', label: gt('English (UK)') },
             { locale: 'de-DE', label: gt('German') },
             { locale: 'fr-FR', label: gt('French') },
             { locale: 'es-ES', label: gt('Spanish') },
@@ -61,6 +62,9 @@ define('io.ox/office/editframework/view/control/languagepicker', [
      *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options of the RadioList base class.
+     *  Additionally, the following options are supported:
+     *  @param {Boolean} [initOptions.showListIcons=true]
+     *      Whether to show the check boxes in the list.
      */
     function LanguagePicker(initOptions) {
 
@@ -79,11 +83,17 @@ define('io.ox/office/editframework/view/control/languagepicker', [
         this.getMenuNode().addClass('language-picker');
 
         this.listenTo(supportedLocalesPromise, 'always', function () {
-            _.each(LOCALE_DEFINITIONS, function (definition) {
-                self.createOptionButton(definition.locale, {
-                    label: definition.label,
-                    icon: definition.supported ? 'fa-check-square-o' : 'fa-square-o'
-                });
+
+            var showListIcons = Utils.getBooleanOption(initOptions, 'showListIcons', true);
+
+            LOCALE_DEFINITIONS.forEach(function (definition) {
+
+                var options = { label: definition.label };
+                if (showListIcons) {
+                    options.icon = definition.supported ? 'fa-check-square-o' : 'fa-square-o';
+                }
+
+                self.createOptionButton(definition.locale, options);
             });
         });
 
