@@ -6,16 +6,16 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/editframework/view/control/stylesheetpicker', [
     'io.ox/office/tk/utils',
-    'io.ox/office/tk/io',
+    'io.ox/office/tk/locale/localedata',
     'io.ox/office/tk/control/radiolist'
-], function (Utils, IO, RadioList) {
+], function (Utils, LocaleData, RadioList) {
 
     'use strict';
 
@@ -30,14 +30,14 @@ define('io.ox/office/editframework/view/control/stylesheetpicker', [
      *
      * @extends RadioList
      *
-     * @param {EditApplication} app
-     *  The application containing this control group.
+     * @param {EditView} docView
+     *  The document view instance containing this control.
      *
      * @param {String} family
      *  The attribute family of the style sheets visualized by this control.
      *
      * @param {Object} [initOptions]
-     *  Additional options passed to the RadioList constructor. Supports all
+     *  Optional parameters passed to the RadioList constructor. Supports all
      *  options of the RadioList class, and the following additional options:
      *  @param {String|String[]} [initOptions.previewFamilies]
      *      The attribute families used to get the formatting options of the
@@ -56,16 +56,16 @@ define('io.ox/office/editframework/view/control/stylesheetpicker', [
      *      translated style sheet names. The placeholders '$1', '$2', etc.
      *      will match the capturing groups defined in the key patterns.
      */
-    function StyleSheetPicker(app, family, initOptions) {
+    function StyleSheetPicker(docView, family, initOptions) {
 
         var // self reference
             self = this,
 
             // the model instance
-            docModel = null,
+            docModel = docView.getDocModel(),
 
             // the style collection
-            styleCollection = null,
+            styleCollection = docModel.getStyleCollection(family),
 
             // the path to the translated style sheet names
             modulePath = Utils.getStringOption(initOptions, 'i18nModulePath', ''),
@@ -109,32 +109,16 @@ define('io.ox/office/editframework/view/control/stylesheetpicker', [
                 translatedName = null;
 
             // first, check the cache of the translation database
-            if (styleKey in translationCache) { return translationCache[styleKey]; }
+            translatedName = translationCache[styleKey];
 
-            // try to find a database entry for the base style name
-            _.any(translationDatabase, function (value, key) {
-
-                var // the matches for the pattern of the current entry
-                    matches = new RegExp('^' + key + '$', 'i').exec(styleName);
-
-                // continue with next database entry, if key does not match
-                if (!_.isArray(matches)) { return; }
-
-                // matching entry found, translate it and replace the placeholders
-                translatedName = value;
-                for (var index = matches.length - 1; index > 0; index -= 1) {
-                    translatedName = translatedName.replace(new RegExp('\\$' + index, 'g'), matches[index]);
-                }
-
-                // exit the _.any() loop
-                return true;
-            });
-
-            // adjust the resulting style name for GUI display
-            translatedName = (translatedName || Utils.capitalizeWords(styleName)).replace(/ (\d)/g, '\xa0$1');
-
-            // put the translated name into the cache
-            return (translationCache[styleKey] = _.noI18n(translatedName));
+            if (!translatedName) {
+                translatedName = Utils.translateWordFromDatabase(styleName, translationDatabase);
+                // adjust the resulting style name for GUI display
+                translatedName = translatedName.replace(/ (\d)/g, '\xa0$1');
+                // put the translated name into the cache
+                translationCache[styleKey] = translatedName;
+            }
+            return _.noI18n(translatedName);
         }
 
         /**
@@ -152,6 +136,9 @@ define('io.ox/office/editframework/view/control/stylesheetpicker', [
          */
         function createOptionButton(styleName, styleId) {
 
+            // nothing to do for hidden styles
+            if (styleCollection.isHidden(styleId)) { return; }
+
             var // the section identifier for the style sheet
                 category = styleCollection.getUICategory(styleId) || '',
                 // sorting priority
@@ -159,8 +146,12 @@ define('io.ox/office/editframework/view/control/stylesheetpicker', [
                 // the sort index stored at the button for lexicographical sorting
                 sortIndex = String((priority < 0) ? (priority + 0x7FFFFFFF) : priority);
 
-            // translate and adjust style name
-            styleName = translateStyleName(styleName);
+            //TODO: hacked code for functions like "create new paragraph stylesheet"
+            //we could use the category for
+            if (styleId.indexOf('\x00') < 0) {
+                // translate and adjust style name
+                styleName = translateStyleName(styleName);
+            }
 
             // build a sorting index usable for lexicographical comparison:
             // 1 digit for priority sign, 10 digits positive priority,
@@ -185,58 +176,79 @@ define('io.ox/office/editframework/view/control/stylesheetpicker', [
                     self.createMenuSection(sectionId);
                 });
                 // insert the style sheets
-                _.each(styleCollection.getStyleSheetNames(), createOptionButton);
+                _.each(styleCollection.getStyleSheetNames({ skipHidden: true }), createOptionButton);
             });
         }, { delay: 1000 });
 
         /**
          * Handles inserted or modified style sheets in the style collection.
          */
-        function insertStyleSheetHandler(event, styleId) {
+        function insertStyleHandler(event, styleId) {
             // bug 33737, 33747: restore browser focus
             self.getMenu().guardFocusedListItem(function () {
+
+                //Bug 39578 indices are temporary wrong because new button is always at the end and sorting is debounced
+                // collecting old index
+                //Bug 40403 double quotes dont work for attribute-selector
+                var btnSelector = 'a[data-value="' + styleId.replace(/\"/g, '\\"') + '"]';
+                var menuNode = self.getMenu().getNode();
+                var oldCellNode = menuNode.find(btnSelector).parent();
+                var oldIndex = oldCellNode.index();
+
+                //-----------------------------------------------------
+
                 self.deleteOptionButton(styleId);
                 createOptionButton(styleCollection.getName(styleId), styleId);
+
+                //-----------------------------------------------------
+
+                //compare old with new index an correct if required
+                var newCellNode = menuNode.find(btnSelector).parent();
+                var newIndex = newCellNode.index();
+
+                if (newIndex !== oldIndex) {
+                    var parent = newCellNode.parent()[0];
+                    if (oldIndex === 0) {
+                        newCellNode.insertBefore(parent.childNodes[oldIndex]);
+                    } else {
+                        newCellNode.insertAfter(parent.childNodes[oldIndex - 1]);
+                    }
+                }
             });
         }
 
         /**
          * Handles deleted style sheets in the style collection.
          */
-        function deleteStyleSheetHandler(event, styleId) {
+        function deleteStyleHandler(event, styleId) {
             self.deleteOptionButton(styleId);
         }
 
         // initialization -----------------------------------------------------
 
-        app.onInit(function () {
-            docModel = app.getModel();
-            styleCollection = docModel.getStyleCollection(family);
-        });
-
         // add CSS class to menu button and drop-down list for special formatting
-        self.getNode().addClass('style-picker family-' + family);
-        self.getMenuNode().addClass('app-' + app.getDocumentType() + ' style-picker family-' + family);
+        this.getNode().addClass('style-picker family-' + family);
+        this.getMenuNode().addClass('app-' + docView.getApp().getDocumentType() + ' style-picker family-' + family);
 
         // update translated style name in drop-down button
         this.registerUpdateHandler(updateHandler);
 
         // add all known style sheets after import and changed style sheets
-        this.listenTo(app.getImportPromise(), 'done', function () {
-            self.executeDelayed(fillList, 1000);
-            self.listenTo(styleCollection, 'insert:stylesheet', insertStyleSheetHandler);
-            self.listenTo(styleCollection, 'delete:stylesheet', deleteStyleSheetHandler);
-            self.listenTo(styleCollection, 'modify:stylesheet', insertStyleSheetHandler);
+        this.listenTo(docView.getApp().getImportPromise(), 'done', function () {
+            self.executeDelayed(fillList, 1000, 'EditFramework: StyleSheetPicker');
+            self.listenTo(styleCollection, 'insert:stylesheet', insertStyleHandler);
+            self.listenTo(styleCollection, 'delete:stylesheet', deleteStyleHandler);
+            self.listenTo(styleCollection, 'change:stylesheet', insertStyleHandler);
             // also reinitialize the preview if theme settings have been changed
             self.listenTo(docModel.getThemeCollection(), 'triggered', fillList);
         });
 
         // load the translated style names if specified
         if (modulePath.length > 0) {
-            this.listenTo(IO.loadResource(modulePath), 'done', function (data) {
+            this.listenTo(LocaleData.loadResource(modulePath, { merge: true }), 'done', function (data) {
                 translationDatabase = data;
                 translationCache = {};
-                if (app.isImportSucceeded()) {
+                if (docView.getApp().isImportSucceeded()) {
                     fillList();
                 }
             });
@@ -244,7 +256,7 @@ define('io.ox/office/editframework/view/control/stylesheetpicker', [
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initOptions = self = docModel = styleCollection = null;
+            self = initOptions = docView = docModel = styleCollection = null;
         });
 
     } // class StyleSheetPicker
