@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,21 +15,24 @@ define('io.ox/office/editframework/view/control/tablesizepicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/keycodes',
     'io.ox/office/tk/forms',
-    'io.ox/office/tk/canvaswrapper',
+    'io.ox/office/tk/utils/tracking',
+    'io.ox/office/tk/canvas/canvas',
     'io.ox/office/tk/popup/basemenu',
     'io.ox/office/tk/popup/tooltip',
-    'io.ox/office/tk/control/button',
+    'io.ox/office/tk/control/group',
+    'io.ox/office/tk/control/captionmixin',
+    'io.ox/office/tk/control/widthmixin',
     'io.ox/office/tk/control/menumixin',
-    'gettext!io.ox/office/editframework'
-], function (Utils, KeyCodes, Forms, CanvasWrapper, BaseMenu, ToolTip, Button, MenuMixin, gt) {
+    'gettext!io.ox/office/editframework/main'
+], function (Utils, KeyCodes, Forms, Tracking, Canvas, BaseMenu, ToolTip, Group, CaptionMixin, WidthMixin, MenuMixin, gt) {
 
     'use strict';
 
     var // width of a single grid cell, in pixels
-        CELL_WIDTH = Modernizr.touch ? 27 : 18,
+        CELL_WIDTH = Utils.TOUCHDEVICE ? 27 : 18,
 
         // height of a single grid cell, in pixels
-        CELL_HEIGHT = Modernizr.touch ? 24 : 16;
+        CELL_HEIGHT = Utils.TOUCHDEVICE ? 24 : 16;
 
     // class TableSizePicker ==================================================
 
@@ -39,7 +42,9 @@ define('io.ox/office/editframework/view/control/tablesizepicker', [
      *
      * @constructor
      *
-     * @extends Button
+     * @extends Group
+     * @extends CaptionMixin
+     * @extends WidthMixin
      * @extends MenuMixin
      *
      * @param {Object} [initOptions]
@@ -50,7 +55,7 @@ define('io.ox/office/editframework/view/control/tablesizepicker', [
      *  @param {Integer} [initOptions.maxRows=15]
      *      The maximal number of rows to be displayed in the control.
      */
-    function TableSizePicker(app, initOptions) {
+    function TableSizePicker(initOptions) {
 
         var // self reference
             self = this,
@@ -71,12 +76,15 @@ define('io.ox/office/editframework/view/control/tablesizepicker', [
             // the drop-down menu instance (must be created after Button base constructor!)
             menu = null,
 
+            // the drop-down button
+            menuButton = $(Forms.createButtonMarkup(Utils.extendOptions({ attributes: { 'aria-haspopup': true } }, initOptions))),
+
             // the button control to be inserted into the drop-down menu
             // since 'aria-label' would override 'aria-describedby' we set the title attribute only
             gridButtonNode = $(Forms.createButtonMarkup({ attributes: { title: gt('Table size'), role: 'menuitem' } })),
 
             // canvas element for rendering the grid lines
-            canvasWrapper = new CanvasWrapper(),
+            canvas = new Canvas(),
 
             // the tooltip node showing the current grid size
             sizeToolTip = null,
@@ -89,17 +97,18 @@ define('io.ox/office/editframework/view/control/tablesizepicker', [
 
         // base constructors --------------------------------------------------
 
-        Button.call(this, Utils.extendOptions({
-            icon: 'docs-table-insert',
-            tooltip: gt('Insert table'),
-            attributes: { 'aria-haspopup': true, 'aria-expanded': false }
+        Group.call(this, Utils.extendOptions({
+            icon: 'docs-table-insert'
         }, initOptions));
+        CaptionMixin.call(this, menuButton, initOptions);
+        WidthMixin.call(this, menuButton, initOptions);
+
         menu = new BaseMenu({
             classes: 'table-size-picker',
             anchor: this.getNode(),
             prepareLayoutHandler: prepareLayoutHandler
         });
-        MenuMixin.call(this, menu, { button: this.getButtonNode() });
+        MenuMixin.call(this, menu, { button: menuButton });
 
         // private methods ----------------------------------------------------
 
@@ -167,7 +176,7 @@ define('io.ox/office/editframework/view/control/tablesizepicker', [
                 (function () {
 
                     var // position/size of the grid
-                        gridPosition = Utils.getNodePositionInPage(canvasWrapper.getNode()),
+                        gridPosition = Utils.getNodePositionInPage(canvas.getNode()),
                         // mouse position relative to grid origin
                         mouseRelX = growLeft ? (gridPosition.left + gridPosition.width - mousePos.x) : (mousePos.x - gridPosition.left),
                         mouseRelY = growTop ? (gridPosition.top + gridPosition.height - mousePos.y) : (mousePos.y - gridPosition.top);
@@ -181,12 +190,14 @@ define('io.ox/office/editframework/view/control/tablesizepicker', [
             gridSize.width = Utils.minMax(gridSize.width, MIN_SIZE.width, maxGridSize.width);
             gridSize.height = Utils.minMax(gridSize.height, MIN_SIZE.height, maxGridSize.height);
 
-            // render the grid
-            canvasWrapper.initialize({
+            // initialize size of the canvas element (clears the canvas implicitly)
+            canvas.initialize({
                 width: Math.max(gridSize.width, MIN_VIEW_SIZE.width) * CELL_WIDTH + 1,
                 height: Math.max(gridSize.height, MIN_VIEW_SIZE.height) * CELL_HEIGHT + 1
             });
-            canvasWrapper.render(function (context, width, height) {
+
+            // render the grid
+            canvas.render(function (context, width, height) {
 
                 var // the pixel size of the highlighted area
                     highlightWidth = gridSize.width * CELL_WIDTH + 1,
@@ -233,7 +244,7 @@ define('io.ox/office/editframework/view/control/tablesizepicker', [
         function toolTipPrepareLayoutHandler() {
             var anchorBorder = growTop ? 'top bottom' : 'bottom top',
                 anchorAlign = growLeft ? 'trailing' : 'leading';
-            if (Modernizr.touch) {
+            if (Utils.TOUCHDEVICE) {
                 anchorBorder = (growLeft ? 'left' : 'right') + ' ' + anchorBorder;
                 anchorAlign = (growTop ? 'trailing' : 'leading') + ' ' + anchorAlign;
             }
@@ -278,8 +289,13 @@ define('io.ox/office/editframework/view/control/tablesizepicker', [
 
         // initialization -----------------------------------------------------
 
+        Forms.setToolTip(menuButton, initOptions);
+
+        // add the drop-down button to the group
+        this.addChildNodes(menuButton);
+
         // embed the button control in the drop-down menu
-        gridButtonNode.append(canvasWrapper.getNode());
+        gridButtonNode.append(canvas.getNode());
         menu.appendContentNodes(gridButtonNode);
 
         // create the tooltip that shows the table size currently selected
@@ -312,22 +328,22 @@ define('io.ox/office/editframework/view/control/tablesizepicker', [
         menu.getContentNode().on('keydown', gridKeyDownHandler);
 
         // bug 28583: enable tracking for touch devices
-        gridButtonNode.enableTracking();
+        Tracking.enableTracking(gridButtonNode);
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
             unbindGlobalEventHandlers();
-            canvasWrapper.destroy();
+            canvas.destroy();
             sizeToolTip.destroy();
             gridButtonNode.remove();
-            app = self = menu = gridButtonNode = canvasWrapper = sizeToolTip = null;
+            self = menu = gridButtonNode = canvas = sizeToolTip = null;
         });
 
     } // class TableSizePicker
 
     // exports ================================================================
 
-    // derive this class from class Button
-    return Button.extend({ constructor: TableSizePicker });
+    // derive this class from class Group
+    return Group.extend({ constructor: TableSizePicker });
 
 });
