@@ -6,28 +6,33 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/view/control/toolbartabgroup',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/control/radiogroup'
-    ], function (Utils, KeyCodes, Forms, RadioGroup) {
+define('io.ox/office/editframework/view/control/toolbartabgroup', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/control/radiogroup'
+], function (Utils, KeyCodes, Forms, RadioGroup) {
 
     'use strict';
 
     // class ToolBarTabGroup ==================================================
 
     /**
+     * A button group in the top-pane used to select the active tool bar.
+     *
      * @constructor
      *
      * @extends RadioGroup
+     *
+     * @param {ToolBarTabCollection} toolBarTabs
+     *  The collection with all tool bars registered for the application.
      */
-    function ToolBarTabGroup(app, toolBarTabs) {
+    function ToolBarTabGroup(toolBarTabs) {
 
         var // self reference
             self = this;
@@ -67,7 +72,7 @@ define('io.ox/office/editframework/view/control/toolbartabgroup',
         // initialization -----------------------------------------------------
 
         // update control according to the tab collection
-        toolBarTabs.on({
+        this.listenTo(toolBarTabs, {
             // create button element for a new tab
             'tab:create': tabCreateHandler,
             'tab:label': tabLabelHandler,
@@ -81,6 +86,11 @@ define('io.ox/office/editframework/view/control/toolbartabgroup',
         // add own keyboard handling
         this.getNode().on('keydown', keyDownHandler);
 
+        // destroy class members on destruction
+        this.registerDestructor(function () {
+            self = toolBarTabs = null;
+        });
+
     } // class ToolBarTabGroup
 
     // exports ================================================================
