@@ -6,28 +6,33 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/view/control/toolbartablist',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/control/radiolist',
-     'gettext!io.ox/office/editframework'
-    ], function (Utils, Forms, RadioList, gt) {
+define('io.ox/office/editframework/view/control/toolbartablist', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/control/radiolist',
+    'gettext!io.ox/office/editframework/main'
+], function (Utils, Forms, RadioList, gt) {
 
     'use strict';
 
     // class ToolBarTabList ===================================================
 
     /**
+     * A drop-down list in the top-pane used to select the active tool bar.
+     *
      * @constructor
      *
      * @extends RadioList
+     *
+     * @param {ToolBarTabCollection} toolBarTabs
+     *  The collection with all tool bars registered for the application.
      */
-    function ToolBarTabList(app, toolBarTabs) {
+    function ToolBarTabList(toolBarTabs) {
 
         var // self reference
             self = this;
@@ -59,7 +64,7 @@ define('io.ox/office/editframework/view/control/toolbartablist',
         // initialization -----------------------------------------------------
 
         // update control according to the tab collection
-        toolBarTabs.on({
+        this.listenTo(toolBarTabs, {
             // create button element for a new tab
             'tab:create': tabCreateHandler,
             'tab:label': tabLabelHandler,
@@ -70,6 +75,11 @@ define('io.ox/office/editframework/view/control/toolbartablist',
             'tab:activate': function (event, tabId) { self.setValue(tabId); }
         });
 
+        // destroy class members on destruction
+        this.registerDestructor(function () {
+            self = toolBarTabs = null;
+        });
+
     } // class ToolBarTabList
 
     // exports ================================================================
