@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,6 +15,7 @@ define('io.ox/office/editframework/view/editcontrols', [
     'io.ox/office/tk/utils',
     'io.ox/office/baseframework/view/basecontrols',
     'io.ox/office/drawinglayer/view/drawingcontrols',
+    'io.ox/office/editframework/view/editlabels',
     'io.ox/office/editframework/view/control/applicationstatuslabel',
     'io.ox/office/editframework/view/control/acquireeditbutton',
     'io.ox/office/editframework/view/control/filenamefield',
@@ -30,8 +31,8 @@ define('io.ox/office/editframework/view/editcontrols', [
     'io.ox/office/editframework/view/control/userbadge',
     'io.ox/office/editframework/view/control/toolbartabgroup',
     'io.ox/office/editframework/view/control/toolbartablist',
-    'gettext!io.ox/office/editframework'
-], function (Utils, BaseControls, DrawingControls, ApplicationStatusLabel, AcquireEditButton, FileNameField, ColorPicker, BorderModePicker, BorderStylePicker, BorderWidthPicker, FontFamilyPicker, FontSizePicker, StyleSheetPicker, LanguagePicker, TableSizePicker, UserBadge, ToolBarTabGroup, ToolBarTabList, gt) {
+    'gettext!io.ox/office/editframework/main'
+], function (Utils, BaseControls, DrawingControls, Editlabels, ApplicationStatusLabel, AcquireEditButton, FileNameField, ColorPicker, BorderModePicker, BorderStylePicker, BorderWidthPicker, FontFamilyPicker, FontSizePicker, StyleSheetPicker, LanguagePicker, TableSizePicker, UserBadge, ToolBarTabGroup, ToolBarTabList, gt) {
 
     'use strict';
 
@@ -67,75 +68,94 @@ define('io.ox/office/editframework/view/editcontrols', [
 
     /**
      * A color picker control for selecting a text color. The list entry for
-     * the automatic color will be shown as 'Black'.
+     * the automatic color will be shown as 'Black' by default, but can be
+     * changed with the method ColorPicker.setAutoColor().
      *
      * @constructor
      *
      * @extends ColorPicker
      */
-   EditControls.TextColorPicker = ColorPicker.extend({ constructor: function (app, initOptions) {
+    EditControls.TextColorPicker = ColorPicker.extend({ constructor: function (docView, initOptions) {
 
-       ColorPicker.call(this, app, 'text', Utils.extendOptions({
-           icon: 'docs-font-color',
-           tooltip: gt('Text color')
-       }, initOptions));
+        ColorPicker.call(this, docView, 'text', Utils.extendOptions({
+            icon: 'docs-font-color',
+            tooltip: gt('Text color'),
+            title:  gt('Text color')
+        }, initOptions));
 
-   }}); // class TextColorPicker
+    }}); // class TextColorPicker
 
-     // class FillColorPicker ==================================================
+    // class FillColorPicker ==================================================
 
-     /**
-      * A color picker control for selecting a fill color. The list entry for
-     * the automatic color will be shown as 'No fill'.
-      *
-      * @constructor
-      *
-      * @extends ColorPicker
-      */
-    EditControls.FillColorPicker = ColorPicker.extend({ constructor: function (app, initOptions) {
+    /**
+     * A color picker control for selecting a fill color. The list entry for
+     * the automatic color will be shown as 'No fill' by default, but can be
+     * changed with the method ColorPicker.setAutoColor().
+     *
+     * @constructor
+     *
+     * @extends ColorPicker
+     */
+    EditControls.FillColorPicker = ColorPicker.extend({ constructor: function (docView, initOptions) {
 
-        ColorPicker.call(this, app, 'fill', Utils.extendOptions({
+        ColorPicker.call(this, docView, 'fill', Utils.extendOptions({
             icon: 'docs-cell-fill-color',
-            tooltip: gt('Fill color')
+            tooltip: gt('Fill color'),
+            title: gt('Fill color')
         }, initOptions));
 
     }}); // class FillColorPicker
 
     // class BorderColorPicker ==================================================
 
-     /**
-      * A color picker control for selecting a border color. The list entry for
-     * the automatic color will be shown as 'Black'.
-      *
-      * @constructor
-      *
-      * @extends ColorPicker
-      */
-    EditControls.BorderColorPicker = ColorPicker.extend({ constructor: function (app, initOptions) {
+    /**
+     * A color picker control for selecting a border color.
+     *
+     * @constructor
+     *
+     * @extends ColorPicker
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters. Supports all options of the RadioList base class,
+     *  and the following options:
+     *  @param {Boolean} [initOptions.showAutoColor=false]
+     *      Whether to show a button for the automatic color on top of the
+     *      droop-down menu. Will be shown as 'Black' by default, but can be
+     *      changed with the method ColorPicker.setAutoColor().
+     */
+    EditControls.BorderColorPicker = ColorPicker.extend({ constructor: function (docView, initOptions) {
 
-        ColorPicker.call(this, app, null, Utils.extendOptions({
+        ColorPicker.call(this, docView, Utils.getBooleanOption(initOptions, 'showAutoColor', false) ? 'line' : null, Utils.extendOptions({
             icon: 'docs-drawing-line-color',
             tooltip: gt.pgettext('borders', 'Border color'),
-            showAutoColor: false
+            title: gt.pgettext('borders', 'Border color')
         }, initOptions));
 
     }}); // class BorderColorPicker
 
-    // class AccessRightsPicker ===============================================
+    // class BorderPresetStylePicker ==========================================
 
-    EditControls.AccessRightsPicker = BaseControls.RadioList.extend({ constructor: function (app, initOptions) {
+    /**
+     * A drop-down list control for border line styles used in OOXML files.
+     *
+     * @constructor
+     *
+     * @extends EditControls.BorderStylePicker
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters. Supports all options supported by the base class
+     *  EditControls.BorderStylePicker.
+     */
+    EditControls.BorderPresetStylePicker = EditControls.BorderStylePicker.extend({ constructor: function (listEntries) {
 
-        BaseControls.RadioList.call(this, Utils.extendOptions(initOptions, {
-            label: _.noI18n('Access Rights'),
-            sortItems: true
-        }));
+        // base constructor ---------------------------------------------------
 
-        _.times(4, function (index) {
-            var mail = 'user' + (index + 1) + '@example.com';
-            this.createOptionButton(mail, { label: _.noI18n('User ' + (index + 1)), tooltip: _.noI18n(mail) });
-        }, this);
+        EditControls.BorderStylePicker.call(this, listEntries, {
+            label: Editlabels.BORDER_STYLE_LABEL,
+            smallerVersion: { hideLabel: true }
+        });
 
-    }}); // class AccessRightsPicker
+    }}); // class BorderPresetStylePicker
 
     // exports ================================================================
 
