@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,7 +14,7 @@
 define('io.ox/office/editframework/view/editlabels', [
     'io.ox/office/baseframework/view/baselabels',
     'io.ox/office/drawinglayer/view/drawinglabels',
-    'gettext!io.ox/office/editframework'
+    'gettext!io.ox/office/editframework/main'
 ], function (BaseLabels, DrawingLabels, gt) {
 
     'use strict';
@@ -185,6 +185,43 @@ define('io.ox/office/editframework/view/editlabels', [
         justify: /*#. vertical alignment of text in table cells */ gt.pgettext('v-alignment', 'Justify')
     };
 
+    // hyperlink labels -------------------------------------------------------
+
+    /**
+     * @constant
+     */
+    EditLabels.HYPERLINK_LABEL = gt('Hyperlink');
+
+    /**
+     * @constant
+     */
+    EditLabels.EDIT_HYPERLINK_LABEL = gt('Edit hyperlink');
+
+    /**
+     * @constant
+     */
+    EditLabels.REMOVE_HYPERLINK_LABEL = gt('Remove hyperlink');
+
+    /**
+     * @constant
+     */
+    EditLabels.OPEN_HYPERLINK_LABEL = gt('Open hyperlink');
+
+    /**
+     * @constant
+     */
+    EditLabels.INSERT_HYPERLINK_LABEL = gt('Insert hyperlink');
+
+    /**
+     * @constant
+     */
+    EditLabels.INSERT_OR_EDIT_HYPERLINK_LABEL = gt('Insert or edit a hyperlink');
+
+    /**
+     * @constant
+     */
+    EditLabels.INVALID_HYPERLINK_LABEL = gt('This hyperlink is invalid.');
+
     // options for controls ---------------------------------------------------
 
     /**
@@ -246,6 +283,16 @@ define('io.ox/office/editframework/view/editlabels', [
         dropDownVersion: { label: EditLabels.CLEAR_FORMAT_LABEL }
     };
 
+    /**
+     * Standard options for a 'Insert hyperlink' button.
+     *
+     * @constant
+     */
+    EditLabels.INSERT_HYPERLINK_OPTIONS = {
+        label: EditLabels.HYPERLINK_LABEL,
+        tooltip: EditLabels.INSERT_HYPERLINK_LABEL
+    };
+
     /**
      * Standard options for a 'Insert/edit hyperlink' button.
      *
@@ -253,8 +300,8 @@ define('io.ox/office/editframework/view/editlabels', [
      */
     EditLabels.HYPERLINK_BUTTON_OPTIONS = {
         icon: 'docs-hyperlink',
-        label: gt('Hyperlink'),
-        tooltip: gt('Insert or edit a hyperlink'),
+        label: EditLabels.HYPERLINK_LABEL,
+        tooltip: EditLabels.INSERT_OR_EDIT_HYPERLINK_LABEL,
         smallerVersion: {
             css: { width: 35 },
             hideLabel: true
@@ -282,6 +329,77 @@ define('io.ox/office/editframework/view/editlabels', [
         tooltip: /*#. check box tooltip: show/hide the collaborator list */ gt('Show or hide list of collaborators')
     };
 
+    /**
+     * Standard options for search TextField
+     *
+     * @constant
+     */
+    EditLabels.SEARCH_TEXT = { placeholder: gt('Search ...'), tooltip: gt('Find text'), smallerVersion: { css: { width: 110 }}};
+
+    /**
+     * Standard options for start search button
+     *
+     * @constant
+     */
+    EditLabels.SEARCH_START = { icon: 'fa-search', tooltip: gt('Start search') };
+
+    /**
+     * Standard options for search previous button
+     *
+     * @constant
+     */
+    EditLabels.SEARCH_PREV = { icon: 'fa-chevron-left', tooltip: gt('Select previous search result') };
+
+    /**
+     * Standard options for search next button
+     *
+     * @constant
+     */
+    EditLabels.SEARCH_NEXT = { icon: 'fa-chevron-right', tooltip: gt('Select next search result') };
+
+    /**
+     * Standard options for replace TextField
+     *
+     * @constant
+     */
+    EditLabels.REPLACE_TEXT = { placeholder: gt('Replace with ...'), tooltip: gt('Replacement text'), smallerVersion: { css: { width: 110 }}};
+
+    /**
+     * Standard options for replace next button
+     *
+     * @constant
+     */
+    EditLabels.REPLACE_NEXT = { label: /*#. in search/replace: button label */ gt('Replace'), tooltip: gt('Replace selected search result and select the next result'), smallerVersion: { css: { maxWidth: 50, textOverflow: 'ellipsis', overflow: 'hidden' }}};
+
+    /**
+     * Standard options for replace all button
+     *
+     * @constant
+     */
+    EditLabels.REPLACE_ALL = { label: /*#. in search/replace: replace all occurrences */ gt('Replace all'), tooltip: gt('Replace all search results'), smallerVersion: { css: { maxWidth: 50, textOverflow: 'ellipsis', overflow: 'hidden' }}};
+
+    /**
+     * Standard options for search tab entry
+     *
+     * @constant
+     */
+    EditLabels.TAB_SEARCH = { label: gt('Search'), icon: 'fa-search' };
+
+    /**
+     * Standard options for search & replace tab entry
+     *
+     * @constant
+     */
+    EditLabels.TAB_SEARCH_REPLACE = { label: gt('Replace'), icon: 'fa-search' };
+
+    /**
+     * @constant
+     */
+    EditLabels.NO_SEARCHRESULT_OPTIONS = {
+        type: 'info',
+        message: gt('No search result found.')
+    };
+
     // exports ================================================================
 
     return EditLabels;
