@@ -6,20 +6,20 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/view/edittoolpane',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
-     'io.ox/office/baseframework/view/toolpane',
-     'io.ox/office/baseframework/view/toolbar',
-     'io.ox/office/editframework/view/editlabels',
-     'io.ox/office/editframework/view/editcontrols',
-     'gettext!io.ox/office/editframework'
-    ], function (Utils, Forms, ToolPane, ToolBar, Labels, Controls, gt) {
+define('io.ox/office/editframework/view/edittoolpane', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms',
+    'io.ox/office/baseframework/view/toolpane',
+    'io.ox/office/baseframework/view/toolbar',
+    'io.ox/office/editframework/view/editlabels',
+    'io.ox/office/editframework/view/editcontrols',
+    'gettext!io.ox/office/editframework/main'
+], function (Utils, Forms, ToolPane, ToolBar, Labels, Controls, gt) {
 
     'use strict';
 
@@ -34,20 +34,17 @@ define('io.ox/office/editframework/view/edittoolpane',
      *
      * @extends ToolPane
      *
-     * @param {EditApplication} app
-     *  The application containing this tool pane.
+     * @param {EditView} docView
+     *  The document view instance containing this tool pane.
      *
      * @param {ToolBarTabCollection} toolBarTabs
      *  The collection of all registered tool bar tabs.
      */
-    function EditToolPane(app, toolBarTabs, initOptions) {
+    function EditToolPane(docView, toolBarTabs, initOptions) {
 
         var // self reference
             self = this,
 
-            // view reference
-            view = app.getView(),
-
             // the tool bars, mapped by tab identifier
             toolBarsMap = {},
 
@@ -70,15 +67,15 @@ define('io.ox/office/editframework/view/edittoolpane',
             searchBtn = null,
 
             // reference to the topPane
-            topPane = view.getTopPane(),
-            // reference to the view-menu-group
+            topPane = docView.getTopPane(),
+            // reference to the View menu group
             viewMenuGroup = topPane.getViewMenuGroup(),
-            // reference to the view-menu
+            // reference to the view menu
             viewMenu = viewMenuGroup.getMenu();
 
         // base constructor ---------------------------------------------------
 
-        ToolPane.call(this, app, Utils.extendOptions({
+        ToolPane.call(this, docView, Utils.extendOptions({
             position: 'top',
             classes: 'edit-pane standard-design',
             cursorNavigate: true
@@ -115,6 +112,7 @@ define('io.ox/office/editframework/view/edittoolpane',
          * tool pane.
          */
         var paneLayoutHandler = this.createDebouncedMethod(Utils.NOOP, function () {
+
             // check if another process/trigger is running this already.
             // cancel in this case, to prevent flashing controls
             if (reLayout) { return; }
@@ -144,9 +142,8 @@ define('io.ox/office/editframework/view/edittoolpane',
             if (paneWidthStore === paneWidth && overflowWidth < 0 && !forceLayoutHandler) {
                 reLayout = false;
                 return;
-            } else {
-                paneWidthStore = paneWidth;
             }
+            paneWidthStore = paneWidth;
 
             // catch new widths
             function resetDimensions() {
@@ -171,8 +168,8 @@ define('io.ox/office/editframework/view/edittoolpane',
              *  whether add or remove the classes
              */
             function toolbarClass(addClass, visible) {
-                var node = view.getToolPane().getNode();
-                    node.toggleClass(addClass, visible);
+                var node = docView.getToolPane().getNode();
+                node.toggleClass(addClass, visible);
             }
 
             /**
@@ -186,7 +183,7 @@ define('io.ox/office/editframework/view/edittoolpane',
              */
             function makeVerticalList(i, state) {
                 var popupWrapper = shrinkableToolbars[i].shrinkedMenu.getNode();
-                    popupWrapper.toggleClass('vertical-list drop-down', state);
+                popupWrapper.toggleClass('vertical-list drop-down', state);
             }
 
             /**
@@ -226,8 +223,6 @@ define('io.ox/office/editframework/view/edittoolpane',
                     makeVerticalList(i, true);
                     return true;
 
-
-
                 } else if (type === 'unshrink' && !self.hasViewComponent(shrinkableToolbars[i].toolBar)) {
 
                     // remove component from shrunken drop-down menu
@@ -253,8 +248,7 @@ define('io.ox/office/editframework/view/edittoolpane',
                 }
             }
 
-
-            view.lockPaneLayout(function() {
+            docView.lockPaneLayout(function () {
                 // returns whether the toolbar(s) are too big for the pane
                 function tooBig() {
                     overflowWidth = resetDimensions();
@@ -262,23 +256,22 @@ define('io.ox/office/editframework/view/edittoolpane',
                         // reset force-option to stop invoking paneLayoutHandler here
                         forceLayoutHandler = false;
                         return true;
-                    } else {
-                        return false;
                     }
+                    return false;
                 }
                 // returns whether the toolbar(s) fit in the pane
                 function fitIn() {
                     overflowWidth = resetDimensions();
                     return (overflowWidth < 0);
                 }
-                // returns the sum of all shrinked toolbars of this pane
+                // returns the sum of all shrunken tool bars of this pane
                 function getCountShrinked() {
-                    countShrinked = _.filter(toolBarsMap[toolBarTabs.getActiveTabId()], function(bar) { return bar.shrinked; }).length;
+                    countShrinked = _.filter(toolBarsMap[toolBarTabs.getActiveTabId()], function (bar) { return bar.shrinked; }).length;
                     return countShrinked;
                 }
-                // returns the sum of all hidden toolbars of this pane
+                // returns the sum of all hidden tool bars of this pane
                 function getCountHidden() {
-                    countHidden = _.filter(toolBarsMap[toolBarTabs.getActiveTabId()], function(bar) { return bar.hidden; }).length;
+                    countHidden = _.filter(toolBarsMap[toolBarTabs.getActiveTabId()], function (bar) { return bar.hidden; }).length;
                     return countHidden;
                 }
 
@@ -288,61 +281,60 @@ define('io.ox/office/editframework/view/edittoolpane',
                     toolbarClass('small-distance', true);
 
                     // activate small versions of groups
-                    if ( tooBig() ) {
-                        Utils.iterateArray(toolbars, function(toolbar) {
+                    if (tooBig()) {
+                        Utils.iterateArray(toolbars, function (toolbar) {
                             // only activate the small version of the groups, if their were not shrinked
                             if (!toolbar.shrinked) {
-                                toolbar.toolBar.iterateGroups(function(group) {
+                                toolbar.toolBar.iterateGroups(function (group) {
                                     group.activateSmallVersion();
                                 });
                                 if (fitIn()) { return Utils.BREAK; }
                             }
-                        }, {reverse: true});
+                        }, { reverse: true });
 
                         // shrink complete toolbar to dropdown-menu
                         if (tooBig()) {
                             // iterate over all toolbars (reversed)
-                            Utils.iterateArray(shrinkableToolbars, function(toolbar, i) {
+                            Utils.iterateArray(shrinkableToolbars, function (toolbar, i) {
                                 if (toolbar.shrinked) { return; }
 
-                                if (i===(shrinkableToolbars.length-1) || (i < (shrinkableToolbars.length-1) && shrinkableToolbars[(i+1)].shrinked)) {
+                                if ((i === shrinkableToolbars.length - 1) || ((i < shrinkableToolbars.length - 1) && shrinkableToolbars[i + 1].shrinked)) {
                                     if (moveToolBar(i, 'shrink')) {
                                         toolbar.shrinked = true;
                                         if (fitIn()) { return Utils.BREAK; }
                                     }
                                 }
-                            }, {reverse: true});
+                            }, { reverse: true });
 
                             // hide toolbars (last step to fit in the given area)
                             if (tooBig()) {
                                 // iterate backwards through all hideable toolbars
-                                Utils.iterateArray(hideableToolbars, function(toolbar, i) {
+                                Utils.iterateArray(hideableToolbars, function (toolbar, i) {
                                     if (toolbar.hidden) { return; }
 
-                                    if (i===(hideableToolbars.length-1) || (i < (hideableToolbars.length-1) && hideableToolbars[(i+1)].hidden)) {
-                                        var toolBarNode = (_.has(toolbar, 'shrinkedToolBar'))? toolbar.shrinkedToolBar.getNode():toolbar.toolBar.getNode();
-                                            toolBarNode.hide();
+                                    if ((i === hideableToolbars.length - 1) || ((i < hideableToolbars.length - 1) && hideableToolbars[i + 1].hidden)) {
+                                        var toolBarNode = _.has(toolbar, 'shrinkedToolBar') ? toolbar.shrinkedToolBar.getNode() : toolbar.toolBar.getNode();
+                                        toolBarNode.hide();
 
                                         toolbar.hidden = true;
                                         if (fitIn()) { return Utils.BREAK; }
                                     }
-                                }, {reverse: true});
+                                }, { reverse: true });
                             }
                         }
                     }
 
-
                 // Controls have free space in the pane
                 } else {
 
                     // show hidden toolbars
-                    Utils.iterateArray(hideableToolbars, function(toolbar, i) {
+                    Utils.iterateArray(hideableToolbars, function (toolbar, i) {
                         if (!toolbar.hidden) { return; }
 
-                        if (i===0 || (i > 0 && !hideableToolbars[(i-1)].hidden)) {
-                            var toolbarNode = (_.has(toolbar, 'shrinkedToolBar'))?toolbar.shrinkedToolBar.getNode():toolbar.toolBar.getNode();
+                        if ((i === 0) || ((i > 0) && !hideableToolbars[i - 1].hidden)) {
+                            var toolbarNode = _.has(toolbar, 'shrinkedToolBar') ? toolbar.shrinkedToolBar.getNode() : toolbar.toolBar.getNode();
 
-                            toolbarNode.css('display','');
+                            toolbarNode.css('display', '');
                             toolbar.hidden = false;
 
                             // if it was too much, revert to last state
@@ -358,12 +350,12 @@ define('io.ox/office/editframework/view/edittoolpane',
                     if (fitIn() && forceStop === false && getCountHidden() === 0) {
 
                         // unshrink toolbars
-                        Utils.iterateArray(shrinkableToolbars, function(toolbar, i) {
+                        Utils.iterateArray(shrinkableToolbars, function (toolbar, i) {
                             if (!toolbar.shrinked) { return; }
 
                             // toolbar have to be shrinked
                             // and the toolbar before have to be unshrinked (or do not exists "===0")
-                            if (i===0 || (i > 0 && !shrinkableToolbars[(i-1)].shrinked)) {
+                            if ((i === 0) || ((i > 0) && !shrinkableToolbars[i - 1].shrinked)) {
                                 if (moveToolBar(i, 'unshrink')) {
                                     toolbar.shrinked = false;
                                     overflowWidth = resetDimensions();
@@ -385,14 +377,14 @@ define('io.ox/office/editframework/view/edittoolpane',
                         // maximize groups
                         if (fitIn() && forceStop === false && getCountShrinked() === 0) {
                             // iterate through all groups of all toolbars
-                            Utils.iterateArray(toolbars, function(toolbar) {
-                                toolbar.toolBar.iterateGroups(function(group) {
+                            Utils.iterateArray(toolbars, function (toolbar) {
+                                toolbar.toolBar.iterateGroups(function (group) {
                                     group.deactivateSmallVersion();
                                 });
 
                                 // if it was too much, revert to last state
                                 if (tooBig()) {
-                                    toolbar.toolBar.iterateGroups(function(group) {
+                                    toolbar.toolBar.iterateGroups(function (group) {
                                         group.activateSmallVersion();
                                     });
                                     forceStop = true;
@@ -409,25 +401,26 @@ define('io.ox/office/editframework/view/edittoolpane',
 
             // reset status (actually not in layout-progess)
             reLayout = false;
-        }, { delay: 500 });
+        }, { delay: 500, infoString: 'EditToolPane: paneLayoutHandler' });
 
         /**
          *  Checks the height of the window-node and (de-)activates the flat-pane
          */
         function setPaneHeight() {
-            var isStandAlone    = app.isStandalone();
+
+            var isStandAlone = docView.getApp().isStandalone();
 
             self.getNode().toggleClass('flat-pane', true);  // add special flat-pane css-class
 
             viewMenu.iterateGroups(function (group) {
+
                 // add sub-menus to focusable nodes, so that the parent-menu don't close automatically
                 if (_.isFunction(group.getMenu)) {
                     flatTabView.getMenu().registerFocusableNodes(group.getMenu().getNode());
                 }
 
                 // check whether something should happen with this view-menu-group if its moved to the flatPane
-                var options = group.getOptions();
-                if (_.has(options, 'flatPane') && options.flatPane === 'hide') {
+                if (Utils.getStringOption(group.getOptions(), 'flatPane') === 'hide') {
                     group.hide();
                 }
             });
@@ -440,11 +433,11 @@ define('io.ox/office/editframework/view/edittoolpane',
 
         function tabCreateHandler(event, tabId, options) {
 
-            var tabButton = new Controls.Button({
-                    label: Utils.getStringOption(options, 'label', ''),
-                    value: tabId,
-                    highlight: function (value) { return value === tabId; }
-                });
+            var tabButton = new Controls.Button(Utils.extendOptions(options, {
+                label: Utils.getStringOption(options, 'label', ''),
+                value: tabId,
+                highlight: function (value) { return value === tabId; }
+            }));
 
             flatTabView.addGroup('view/toolbars/tab', tabButton, {
                 visibleKey: options.visibleKey,
@@ -470,7 +463,7 @@ define('io.ox/office/editframework/view/edittoolpane',
          * Updates the visibility of the tool bars according to the active tab.
          */
         function tabActivateHandler(event, newTabId, oldTabId) {
-            view.lockPaneLayout(function () {
+            docView.lockPaneLayout(function () {
                 toggleToolBars(oldTabId, false);
                 if (_.isString(newTabId)) { self.show(); }
                 toggleToolBars(newTabId, true);
@@ -484,7 +477,7 @@ define('io.ox/office/editframework/view/edittoolpane',
          * Updates the visibility of the active tool bars that are bound to the
          * enabled state of a controller item.
          */
-        function controllerChangeHandler(event, changedItems) {
+        function controllerChangeHandler(changedItems) {
             _.each(toolBarsMap, function (toolBarInfos) {
                 _.each(toolBarInfos, function (toolBarInfo) {
                     var key = toolBarInfo.visibleKey,
@@ -513,7 +506,7 @@ define('io.ox/office/editframework/view/edittoolpane',
 
             // without split button in toolbar
             if (splitBtn === false) {
-                smenu = new Controls.CompoundButton(app, {
+                smenu = new Controls.CompoundButton(docView, {
                     tooltip:    Utils.getStringOption(options, 'tooltip', ''),
                     label:      Utils.getStringOption(options, 'label', ''),
                     icon:       Utils.getStringOption(options, 'icon', ''),
@@ -522,7 +515,7 @@ define('io.ox/office/editframework/view/edittoolpane',
 
             // with split button in toolbar
             } else {
-                smenu = new Controls.CompoundSplitButton(app, Utils.extendOptions({
+                smenu = new Controls.CompoundSplitButton(docView, Utils.extendOptions({
                     tooltip:        Utils.getStringOption(options, 'tooltip', ''),
                     caretTooltip:   Utils.getStringOption(options, 'caretTooltip', ''),
                     label:          Utils.getStringOption(options, 'label', ''),
@@ -534,7 +527,7 @@ define('io.ox/office/editframework/view/edittoolpane',
             }
 
             shrinkObj.shrinkedMenu = smenu.getMenu();
-            shrinkObj.shrinkedToolBar = new ToolBar(app).addGroup(controllerKey, smenu).hide();
+            shrinkObj.shrinkedToolBar = new ToolBar(docView).addGroup(controllerKey, smenu).hide();
 
             self.addViewComponent(shrinkObj.shrinkedToolBar);
 
@@ -571,7 +564,7 @@ define('io.ox/office/editframework/view/edittoolpane',
         this.createToolBar = function (tabId, options) {
 
             var // the new tool bar instance
-                toolBar = new ToolBar(app, options),
+                toolBar = new ToolBar(docView, options),
                 // prepare shrunken toolbar
                 shrinkObj = (Utils.getBooleanOption(options, 'prepareShrink', false)) ? addShrinkedVersion(options) : {};
 
@@ -593,26 +586,33 @@ define('io.ox/office/editframework/view/edittoolpane',
             return toolBar;
         };
 
+        /**
+         *  return the flat-pane-menu for small device gui
+         * @returns {CompoundButton}
+         */
+        this.getFlatTabView = function () {
+            return flatTabView;
+        };
+
         // initialization -----------------------------------------------------
 
         // new menu for flat-pane (in leading-container of toolbar)
-        flatTabView = new Controls.CompoundButton(app, { icon: 'fa-bars', caret: false, anchorPadding: -3 });
+        flatTabView = new Controls.CompoundButton(docView, { icon: 'fa-bars', caret: false, anchorPadding: -3 });
 
         // add static controls to new flat-pane-menu
         flatTabView
-            .addSeparator()
             .addGroup('view/searchpane', searchBtn = new Controls.Button({ icon: 'fa-search', label: gt('Search'), tooltip: gt('Toggle search'), toggle: true }), { visibleKey: 'app/valid' })
             .addSeparator()
             .addGroup('document/undo', new Controls.Button({ icon: 'docs-undo', tooltip: gt('Revert last operation') }), { visibleKey: 'document/editable', inline: true })
             .addGroup('document/redo', new Controls.Button({ icon: 'docs-redo', tooltip: gt('Restore last operation') }), { visibleKey: 'document/editable', inline: true });
 
         // create the tool bar containing the tab buttons and add it to the leading-container of the toolBar
-        this.addViewComponent(new ToolBar(app)
+        this.addViewComponent(new ToolBar(docView)
             .addGroup(null, flatTabView.hide()),
         { targetArea: 'leading' });
 
         // create the toolbar containing the close button and add it to the trailing-container of the toolBar
-        this.addViewComponent(new ToolBar(app)
+        this.addViewComponent(new ToolBar(docView)
             .addGroup('app/quit', closeBtn = new Controls.Button(Labels.QUIT_BUTTON_OPTIONS).hide()),
         { targetArea: 'trailing' });
 
@@ -623,13 +623,13 @@ define('io.ox/office/editframework/view/edittoolpane',
         this.listenTo(toolBarTabs, 'tab:create', tabCreateHandler);
         this.listenTo(toolBarTabs, 'tab:label', tabLabelHandler);
         this.listenTo(toolBarTabs, 'tab:activate', tabActivateHandler);
-        this.listenTo(app.getController(), 'change:items', controllerChangeHandler);
 
-        // activate a tool bar tab, if the tool pane becomes visible
+        // handle changed controller items
+        this.handleChangedControllerItems(controllerChangeHandler);
+
+        // bug 39071: reactivate last tool bar tab, if the tool pane becomes visible
         this.on('pane:show', function () {
-            if (!toolBarTabs.getActiveTabId()) {
-                toolBarTabs.activateFirstTab();
-            }
+            toolBarTabs.restoreActivateTab();
         });
 
         // get the container nodes for the center tool bar area
@@ -637,19 +637,21 @@ define('io.ox/office/editframework/view/edittoolpane',
         centerAreaNode = centerWrapperNode.find('>.area-container');
 
         // decide whether to show the combined tool/tab bar
-        if (view.panesCombined()){
+        if (docView.panesCombined()) {
             setPaneHeight();
         }
-        app.registerWindowResizeHandler(function(){
+
+        docView.getApp().registerWindowResizeHandler(function () {
             // only remove the 'small-distance' marker, if the width has changed
             if (centerWrapperNode.width() !== paneWidthStore) {
-                app.getView().getToolPane().getNode().removeClass('small-distance');
+                docView.getToolPane().getNode().removeClass('small-distance');
             }
         });
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = toolBarTabs = self = toolBarsMap = centerWrapperNode = centerAreaNode = null;
+            self = docView = toolBarTabs = initOptions = toolBarsMap = null;
+            centerWrapperNode = centerAreaNode = null;
         });
 
     } // class ToolPane
