@@ -6,37 +6,56 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/view/editview',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/baseframework/app/extensionregistry',
-     'io.ox/office/baseframework/view/baseview',
-     'io.ox/office/editframework/utils/editconfig',
-     'io.ox/office/editframework/view/editlabels',
-     'io.ox/office/editframework/view/toolbartabcollection',
-     'io.ox/office/editframework/view/toppane',
-     'io.ox/office/editframework/view/edittoolpane',
-     'io.ox/office/editframework/view/searchpane',
-     'io.ox/office/editframework/view/operationspane',
-     'io.ox/office/editframework/view/clipboardpane',
-     'io.ox/office/editframework/view/editlabels',
-     'io.ox/office/editframework/view/editcontrols',
-     'io.ox/office/editframework/view/editdialogs',
-     'io.ox/office/editframework/view/popup/userslayermenu',
-     'gettext!io.ox/office/editframework',
-     'less!io.ox/office/editframework/view/editstyle'
-    ], function (Utils, KeyCodes, ExtensionRegistry, BaseView, Config, EditLabels, ToolBarTabCollection, TopPane, EditToolPane, SearchPane, OperationsPane, ClipboardPane, Labels, Controls, Dialogs, UsersLayerMenu, gt) {
+define('io.ox/office/editframework/view/editview', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/baseframework/app/extensionregistry',
+    'io.ox/office/baseframework/view/baseview',
+    'io.ox/office/editframework/utils/editconfig',
+    'io.ox/office/editframework/view/toolbartabcollection',
+    'io.ox/office/editframework/view/toppane',
+    'io.ox/office/editframework/view/edittoolpane',
+    'io.ox/office/editframework/view/searchpane',
+    'io.ox/office/editframework/view/operationspane',
+    'io.ox/office/editframework/view/clipboardpane',
+    'io.ox/office/editframework/view/editlabels',
+    'io.ox/office/editframework/view/editcontrols',
+    'io.ox/office/editframework/view/editdialogs',
+    'io.ox/office/editframework/view/mobilesearchgroup',
+    'io.ox/office/editframework/view/popup/userslayermenu',
+    'gettext!io.ox/office/editframework/main',
+    'less!io.ox/office/editframework/view/editstyle'
+], function (Utils, KeyCodes, ExtensionRegistry, BaseView, Config, ToolBarTabCollection, TopPane, EditToolPane, SearchPane, OperationsPane, ClipboardPane, Labels, Controls, Dialogs, MobileSearchGroup, UsersLayerMenu, gt) {
 
     'use strict';
 
+    var // default search configuration
+        DEFAULT_SEARCH_SETTINGS = {
+            // current search query
+            query: '',
+            // whether 'query' is a regular expression
+            regexp: false,
+            // the replacement string
+            replace: ''
+        },
+
+        // beneath this minimum screen width or height, the tool/tab bars should be combined
+        // Nexus 7 = min resolution 601
+        PANES_COMBINED_MIN_SIZE = 610,
+
+        // combined panes mode via URL flag in debug mode
+        DEBUG_PANES_COMBINED = Config.getDebugUrlFlag('office:panes-combined');
+
     // class EditView =========================================================
 
     /**
+     * Base class for the application view of all editor applications.
+     *
      * @constructor
      *
      * @extends BaseView
@@ -44,6 +63,28 @@ define('io.ox/office/editframework/view/editview',
      * @param {EditApplication} app
      *  The application containing this view instance.
      *
+     * @param {EditModel} docModel
+     *  The document model created by the passed application.
+     *
+     * @param {Function} searchHandler
+     *  A callback function that will implement searching for a specific query
+     *  string entered in the search/replace tool bar. Receives the following
+     *  parameters:
+     *  (1) {String} command
+     *      The search command to be executed. See description of the method
+     *      EditView.executeSearchOperation() for details.
+     *  (2) {Object} settings
+     *      The current search settings. See the description of the method
+     *      EditView.getSearchSettings() for details about the properties
+     *      contained in this object.
+     *  The function may return the promise of a Deferred object, if the search
+     *  action involves asynchronous operations. The view will be locked until
+     *  the promise has been resolved or rejected. The promise may be rejected
+     *  with an error code string which may result in an alert message box
+     *  shown in the user interface. Alternatively, the function may return an
+     *  explicit error code string synchronously (without using a promise). The
+     *  function will be called in the context of this view instance.
+     *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options that are supported by the
      *  constructor of the base class BaseView, with the following additions
@@ -82,21 +123,18 @@ define('io.ox/office/editframework/view/editview',
      *          containing action commands for debugging.
      *      May return a promise to be able to run asynchronous code during
      *      initialization.
+     *  @param {Boolean} [initOptions.enablePageSettings=false]
+     *      If set to true, the file toolbar tab will contain a page toolbar,
+     *      containing a page settings button to show a page settings dialog
+     *      for the current document.
      */
-    function EditView(app, initOptions) {
+    function EditView(app, docModel, searchHandler, initOptions) {
 
         var // self reference
             self = this,
 
-            // beneath this min height, the tool-/tabbars should be combined
-            // Nexus 7 = min resolution 601
-            minHeight = 610,
-
-            // show combined Panes
-            panesCombined = false,
-
             // the tool bar tab collection
-            toolBarTabs = new ToolBarTabCollection(app),
+            toolBarTabs = null,
 
             // the top-level view pane containing global controls and the tool bar tabs
             topPane = null,
@@ -105,14 +143,22 @@ define('io.ox/office/editframework/view/editview',
             toolPane = null,
 
             // the search/replace pane below the tool pane
-            searchPane = null,
+            searchGroup = null,
+
+            // the current search configuration
+            searchSettings = _.clone(DEFAULT_SEARCH_SETTINGS),
 
             // collaborator list
-            collaboratorPopup = null;
+            collaboratorPopup = null,
+
+            // show combined Panes
+            panesCombined = false,
+
+            userLayerShowOwnColor = Utils.getBooleanOption(initOptions, 'userLayerShowOwnColor', false);
 
         // base constructor ---------------------------------------------------
 
-        BaseView.call(this, app, Utils.extendOptions(initOptions, {
+        BaseView.call(this, app, docModel, Utils.extendOptions(initOptions, {
             initHandler: initHandler,
             initGuiHandler: initGuiHandler,
             classes: 'io-ox-office-edit-main'
@@ -134,17 +180,21 @@ define('io.ox/office/editframework/view/editview',
                 clipboardPane = null;
 
             // determine whether the panes are combined or not
-            if ((Utils.TOUCHDEVICE && Math.min(Utils.getScreenWidth(), Utils.getScreenHeight()) <= minHeight) || (Config.DEBUG && Config.getUrlFlag('office:panes-combined'))) {
-                panesCombined = true;
-            }
+            panesCombined = DEBUG_PANES_COMBINED || (Utils.TOUCHDEVICE && (Math.min(Utils.getScreenWidth(), Utils.getScreenHeight()) <= PANES_COMBINED_MIN_SIZE));
 
             // disable internal browser handles on tables and drawings after edit mode has changed
-            self.listenTo(app.getModel(), 'change:editmode', function () { self.disableBrowserEditControls(); });
+            self.listenTo(docModel, 'change:editmode', function () { self.disableBrowserEditControls(); });
 
             // create the tool bars (initialize themselves internally)
-            self.addPane(topPane = new TopPane(app, toolBarTabs));
-            self.addPane(toolPane = new EditToolPane(app, toolBarTabs, { landmark: false }));
-            self.addPane(searchPane = new SearchPane(app).hide());
+            self.addPane(topPane = new TopPane(self, toolBarTabs));
+            self.addPane(toolPane = new EditToolPane(self, toolBarTabs, { landmark: false }));
+
+            if (panesCombined) {
+                searchGroup = new MobileSearchGroup(self);
+            } else {
+                searchGroup = new SearchPane(self);
+                self.addPane(searchGroup);
+            }
 
             // deactivate tool bar tabs while tool pane is hidden
             toolPane.on('pane:hide', function () { toolBarTabs.activateTab(null); });
@@ -154,6 +204,9 @@ define('io.ox/office/editframework/view/editview',
                 toolPane.hide();
             });
 
+            // hide search pane when application is in internal error state
+            this.listenTo(app, 'docs:state:error', function () { searchGroup.hide(); });
+
             // update controller immediately when toggling the collaborator pop-up menu
             this.listenTo(collaboratorPopup, 'popup:show popup:hide', function () {
                 app.getController().update();
@@ -163,12 +216,12 @@ define('io.ox/office/editframework/view/editview',
             if (Config.DEBUG) {
 
                 // create and insert the clipboard pane, hide it initially
-                clipboardPane = new ClipboardPane(app);
+                clipboardPane = new ClipboardPane(self);
                 clipboardPane.hide();
                 self.addPane(clipboardPane);
 
                 // create and insert the operations pane, hide it initially
-                operationsPane = new OperationsPane(app);
+                operationsPane = new OperationsPane(self);
                 operationsPane.hide();
                 self.addPane(operationsPane);
 
@@ -192,11 +245,6 @@ define('io.ox/office/editframework/view/editview',
                         get: function () { return app.getWindowNode().hasClass('debug-highlight'); },
                         set: function (state) { app.getWindowNode().toggleClass('debug-highlight', state); }
                     },
-                    'debug/toolbars/white': {
-                        parent: 'debug/enabled',
-                        get: function () { return app.getWindowNode().hasClass('debug-white'); },
-                        set: function (state) { app.getWindowNode().toggleClass('debug-white', state); }
-                    },
                     'debug/realtime/trigger': {
                         parent: 'debug/enabled',
                         set: function (type) { app.debugTriggerRealtimeEvent(type); }
@@ -207,7 +255,18 @@ define('io.ox/office/editframework/view/editview',
                     },
                     'debug/operations/apply': {
                         parent: ['debug/enabled', 'document/editable'],
-                        set: function (operation) { app.getModel().applyOperations(operation); }
+                        set: function (operation) { docModel.applyOperations(operation); }
+                    },
+                    'debug/slowsave': {
+                        parent: 'debug/enabled',
+                        enable: function () { return Config.getFlag('debugslowsave'); },
+                        get: function () { return Config.getFlag('debugslowsave') && Config.getFlag('debugslowsaveuser'); },
+                        set: function (state) { Config.set('debugslowsaveuser', state); }
+                    },
+                    'debug/view/busy': {
+                        parent: 'debug/enabled',
+                        set: function (cancelable) { return cancelable ? self.createAbortablePromise($.Deferred()) : $.Deferred(); },
+                        cancel: true
                     },
                     'debug/quit': {
                         parent: 'debug/enabled',
@@ -238,21 +297,16 @@ define('io.ox/office/editframework/view/editview',
             var
                 renameToolbar   = null,
                 saveToolbar     = null,
+                pageToolbar     = null,
                 commonToolbar   = null,
                 actionToolbar   = null,
 
-                btnSaveAs           = new Controls.Button({ label: gt('Save as'), value: 'file' }),
-                btnSaveAsTemplate   = new Controls.Button({ label: gt('Save as template'), value: 'template' }),
-                btnDownload         = new Controls.Button({ icon: 'fa-cloud-download', tooltip: gt('Download'), classes: 'link-style', dropDownVersion: { label: gt('Download') } }),
+                btnDownload         = new Controls.Button({ icon: Labels.DOWNLOAD_ICON, tooltip: gt('Download'), classes: 'link-style', dropDownVersion: { label: gt('Download') } }),
                 btnPrint            = new Controls.Button({ icon: 'fa-print', tooltip: gt('Print as PDF'), classes: 'link-style', dropDownVersion: { label: gt('Print as PDF') } }),
                 btnSendAsMail       = new Controls.Button({ icon: 'fa-envelope-o', tooltip: gt('Send as mail'), classes: 'link-style', dropDownVersion: { label: gt('Send as mail') } }),
                 btnReload           = new Controls.Button({ icon: 'fa-repeat', label: gt('Reload'), tooltip: gt('Reload document'), classes: 'link-style' }),
-                btnAcquireEdit      = new Controls.AcquireEditButton(app, { classes: 'link-style' }),
-
-                cBoxAutosave        = new Controls.CheckBox({ label: gt('AutoSave') }),
-
-                fileNameField       = new Controls.FileNameField(app);
-
+                btnAcquireEdit      = new Controls.AcquireEditButton(self, { classes: 'link-style' }),
+                enablePageSettings  = Utils.getBooleanOption(initOptions, 'enablePageSettings', false);
 
             // -----------------------------------------------------
             // TABS
@@ -266,27 +320,49 @@ define('io.ox/office/editframework/view/editview',
             // -----------------------------------------------------
             if (!self.panesCombined()) {
                 renameToolbar   = self.createToolBar('file');
-                saveToolbar     = self.createToolBar('file', {priority: 2, visibleKey: 'app/bundled'});
+                saveToolbar     = self.createToolBar('file', { priority: 2, visibleKey: 'app/bundled' });
             }
-            commonToolbar       = self.createToolBar('file', {priority: 3, label: gt('Actions'), classes: 'link-style', prepareShrink: true });
-            actionToolbar       = self.createToolBar('file', {priority: 1, tooltip: gt('actions'), classes: 'link-style' });
+
+            if (enablePageSettings) {
+                pageToolbar = self.createToolBar('file', { priority: 3, label: gt('Page'), classes: 'link-style', prepareShrink: true });
+            }
+
+            commonToolbar       = self.createToolBar('file', { priority: 3, label: gt('Actions'), classes: 'link-style', prepareShrink: true });
+            actionToolbar       = self.createToolBar('file', { priority: 1, tooltip: gt('actions'), classes: 'link-style' });
 
             // -----------------------------------------------------
             // CONTROLS
             //      add all controls
             // -----------------------------------------------------
             if (!self.panesCombined()) {
+                var fileNameField       = new Controls.FileNameField(self),
+                    cBoxAutosave        = new Controls.CheckBox({ label: gt('AutoSave') }),
+                    btnSaveAs           = new Controls.Button({ label: gt('Save as'), value: 'file' }),
+                    btnSaveAsTemplate   = new Controls.Button({ label: gt('Save as template'), value: 'template' });
+
                 renameToolbar
                     .addGroup('document/rename', fileNameField);
 
                 saveToolbar
-                    .addGroup(null, new Controls.CompoundButton(app, { label: gt('Save in Drive'), classes: 'link-style', smallerVersion:{ icon: 'fa-save', hideLabel: true } })
+                    .addGroup('view/saveas/menu', new Controls.CompoundButton(self, { label: gt('Save in Drive'), classes: 'link-style', smallerVersion: { icon: 'fa-save', hideLabel: true } })
                         .addGroup('document/saveas/dialog', btnSaveAs)
                         .addGroup('document/saveas/dialog', btnSaveAsTemplate)
                         .addSeparator()
                         .addGroup('document/autosave', cBoxAutosave)
                     );
             }
+
+            if (pageToolbar) {
+                if (enablePageSettings) {
+                    pageToolbar.addGroup('document/pagesettings', new Controls.Button({
+                            icon: 'docs-page-layout',
+                            tooltip: gt('Page settings'),
+                            classes: 'link-style',
+                            dropDownVersion: { label: gt('Page settings') }
+                        }));
+                }
+            }
+
             commonToolbar
                 .addGroup('document/download', btnDownload)
                 .addGap()
@@ -298,14 +374,13 @@ define('io.ox/office/editframework/view/editview',
                 .addGroup('document/acquireedit', btnAcquireEdit, { visibleKey: 'document/acquireedit' })
                 .addGroup('document/reload', btnReload, { visibleKey: 'document/reload' });
 
-
             // register double-click handler for the application launcher
             self.listenTo(app.getImportPromise(), 'done', function () {
                 $('#io-ox-topbar .launcher[data-app-guid="' + app.guid + '"] a.apptitle').on('dblclick', function () {
                     if (fileNameField.isEnabled()) {
-                        app.getController().executeItem('view/toolbars/tab', 'file', { focusTarget: fileNameField });
+                        self.executeControllerItem('view/toolbars/tab', 'file', { focusTarget: fileNameField });
                     } else {
-                        app.getView().grabFocus();
+                        self.grabFocus();
                     }
                 });
             });
@@ -337,11 +412,10 @@ define('io.ox/office/editframework/view/editview',
                 .addSectionLabel(_.noI18n('Debug'))
                 .addGroup('debug/operationspane', new Controls.CheckBox({ label: _.noI18n('Show operations panel') }))
                 .addGroup('debug/clipboardpane',  new Controls.CheckBox({ label: _.noI18n('Show clipboard panel') }))
-                .addGroup('debug/highlight',      new Controls.CheckBox({ label: _.noI18n('Highlight DOM elements') }))
-                .addGroup('debug/toolbars/white', new Controls.CheckBox({ label: _.noI18n('White toolbar panels') }));
+                .addGroup('debug/highlight',      new Controls.CheckBox({ label: _.noI18n('Highlight DOM elements') }));
 
             // create the drop-down list for the debug actions
-            actionMenuGroup = new Controls.CompoundButton(app, { icon: 'fa-wrench', label: EditLabels.ACTIONS_LABEL, tooltip: _.noI18n('Additional debug actions'), classes: 'link-style' })
+            actionMenuGroup = new Controls.CompoundButton(self, { icon: 'fa-wrench', label: Labels.ACTIONS_LABEL, tooltip: _.noI18n('Additional debug actions'), classes: 'link-style' })
                 // register realtime debug actions
                 .addSectionLabel(_.noI18n('Offline mode'))
                 .addGroup('debug/realtime/trigger', new Controls.Button({ label: _.noI18n('Switch to offline mode'),      value: 'offline' }))
@@ -354,22 +428,32 @@ define('io.ox/office/editframework/view/editview',
                 .addSectionLabel(_.noI18n('Operations'))
                 .addGroup('debug/operations/apply',              new Controls.Button({ label: _.noI18n('Apply operation with invalid name'), value: { name: '_invalid_operation_' } }))
                 .addGroup('debug/operations/apply',              new Controls.Button({ label: _.noI18n('Apply operation with invalid position'), value: { name: 'insertText', start: [987654321, 0], text: 'test' } }))
-                .addGroup('debug/realtime/sendinvalidoperation', new Controls.Button({ label: _.noI18n('Send operation with invalid OSN'), value: { operation: { name: 'delete', start: [987654321, 0]}, badOsn: true }}))
-                .addGroup('debug/realtime/sendinvalidoperation', new Controls.Button({ label: _.noI18n('Send invalid operation'), value: { operation: { name: 'createError', start: [987654321, 0]}}}))
+                .addGroup('debug/realtime/sendinvalidoperation', new Controls.Button({ label: _.noI18n('Send operation with invalid OSN'), value: { operation: { name: 'delete', start: [987654321, 0] }, badOsn: true } }))
+                .addGroup('debug/realtime/sendinvalidoperation', new Controls.Button({ label: _.noI18n('Send invalid operation'), value: { operation: { name: 'createError', start: [987654321, 0] } } }))
+                // register debug actions for busy screen
+                .addSectionLabel(_.noI18n('Busy blocker'))
+                .addGroup('debug/view/busy', new Controls.Button({ label: _.noI18n('Show permanent busy screen'), value: false }))
+                .addGroup('debug/view/busy', new Controls.Button({ label: _.noI18n('Show busy screen with Cancel button'), value: true }))
                 // register debug actions for application quit
                 .addSectionLabel(_.noI18n('Quit'))
                 .addGroup('debug/quit', new Controls.Button({ label: _.noI18n('Close document with unsaved changes'), value: 'unsaved' }))
                 .addGroup('debug/quit', new Controls.Button({ label: _.noI18n('Close document with server error'),    value: 'error' }));
 
+            searchGroup.handleTabListButtons();
+
             // create a new tool bar tab for debugging
             self.createToolBarTab('debug', { label: _.noI18n('Debug'), visibleKey: 'debug/enabled', priority: 9999 });
             self.createToolBar('debug').addGroup(null, actionMenuGroup);
 
+            // create a new tool bar for debugging containing debug options
+            self.createToolBar('debug')
+                .addGroup('debug/slowsave', new Controls.Button({ icon: 'fa-floppy-o', tooltip: _.noI18n('Slow down save document (' + Config.get('debugslowsavetime', 0) + 's)'), toggle: true }));
+
             // call initialization handler passed by sub class
             return Utils.getFunctionOption(initOptions, 'initDebugGuiHandler', $.noop).call(self, viewMenuGroup, actionMenuGroup);
         }
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the top pane containing the tool bar tab buttons, and other
@@ -392,26 +476,6 @@ define('io.ox/office/editframework/view/editview',
             return toolPane;
         };
 
-        /**
-         * Returns the search pane.
-         *
-         * @returns {Pane}
-         *  The search pane of this application.
-         */
-        this.getSearchPane = function () {
-            return searchPane;
-        };
-
-        /**
-         * Returns whether the search pane is currently visible.
-         *
-         * @returns {Boolean}
-         *  Whether the search pane is currently visible.
-         */
-        this.isSearchActive = function () {
-            return searchPane.isVisible();
-        };
-
         /**
          * Returns the collection containing data for all registered tool bar
          * tabs.
@@ -463,6 +527,36 @@ define('io.ox/office/editframework/view/editview',
             return toolPane.createToolBar(tabId, options);
         };
 
+        /**
+         * Returns the search group.
+         *
+         * @returns {SearchPane or MobileSearchGroup}
+         *  The search pane of this application.
+         */
+        this.getSearchGroup = function () {
+            return searchGroup;
+        };
+
+        /**
+         * Returns whether the search pane is currently visible.
+         *
+         * @returns {Boolean}
+         *  Whether the search pane is currently visible.
+         */
+        this.isSearchActive = function () {
+            return searchGroup.isVisible();
+        };
+
+        /**
+         * Returns the tool pane.
+         *
+         * @returns {EditToolPane}
+         *  The tool pane of this application.
+         */
+        this.getEditToolPane = function () {
+            return toolPane;
+        };
+
         /**
          * Creates a new editable container node for clipboard functionality,
          * and inserts it into the hidden container node of the application.
@@ -473,7 +567,7 @@ define('io.ox/office/editframework/view/editview',
         this.createClipboardNode = function () {
             // clipboard node must be content-editable to allow system copy-to-clipboard
             // clipboard node must contain 'user-select-text' to be focusable in Chrome
-            var clipboardNode = $('<div>', { contenteditable: true, 'data-focus-role': 'clipboard' }).addClass('clipboard user-select-text');
+            var clipboardNode = $('<div class="clipboard user-select-text" contenteditable="true" data-focus-role="clipboard">');
             this.insertHiddenNodes(clipboardNode);
             return clipboardNode;
         };
@@ -495,7 +589,7 @@ define('io.ox/office/editframework/view/editview',
                     window.document.execCommand('enableInlineTableEditing', false, false);
                 } catch (ex) {
                 }
-            });
+            }, undefined, 'EditFramework: disableBrowserEditControls');
             return this;
         };
 
@@ -503,28 +597,59 @@ define('io.ox/office/editframework/view/editview',
          * Closes the passed dialog automatically, if the document switches to
          * read-only mode.
          *
-         * @constructor
-         *
          * @param {ModalDialog} dialog
          *  The dialog instance that will be closed on read-only mode.
+         *
+         * @returns {EditView}
+         *  A reference to this instance.
          */
         this.closeDialogOnReadOnlyMode = function (dialog) {
 
-            var // the document model
-                model = app.getModel();
-
             // close the dialog if document switches to read-only mode
             function changeEditModeHandler(event, editMode) {
                 if (!editMode) { dialog.close(); }
             }
 
             // listen to read-only mode, close dialog automatically
-            model.on('change:editmode', changeEditModeHandler);
+            this.listenTo(docModel, 'change:editmode', changeEditModeHandler);
 
             // unregister listener after closing the dialog
             dialog.on('close', function () {
-                model.off('change:editmode', changeEditModeHandler);
+                self.stopListeningTo(docModel, 'change:editmode', changeEditModeHandler);
             });
+
+            return this;
+        };
+
+        /**
+         * Shows a modal query dialog with Yes/No buttons (an instance of the
+         * class ModalQueryDialog). The dialog will be closed automatically, if
+         * the application loses edit rights.
+         *
+         * @param {String} title
+         *  The title text for the dialog.
+         *
+         * @param {String} message
+         *  The message text shown in the dialog body.
+         *
+         * @param {Object} [options]
+         *  Additional options passed to the dialog constructor.
+         *
+         * @returns {jQuery.Promise}
+         *  A promise representing the dialog. Will be resolved after pressing
+         *  the Yes button; or rejected after pressing the No button, or losing
+         *  edit rights.
+         */
+        this.showQueryDialog = function (title, message, options) {
+
+            // create a query dialog instance
+            var dialog = new Dialogs.ModalQueryDialog(message, _.extend({}, options, { title: title }));
+
+            // close dialog automatically when losing edit rights
+            this.closeDialogOnReadOnlyMode(dialog);
+
+            // show the dialog, return its promise
+            return dialog.show();
         };
 
         /**
@@ -603,27 +728,33 @@ define('io.ox/office/editframework/view/editview',
 
             // prepare file extension and dialog title
             if (type === 'template') {
+
                 newFileName = oldFileName;
+
                 // use original extension as fallback - possible if a template
                 // document should be stored via save as as template.
-                templateExt = app.getFileExtension();
-                templateExt = ExtensionRegistry.getTemplateExtensionForFileName(app.getFullFileName()) || templateExt;
-                //#. %1$s is the new file extension used for the template file
+                templateExt = ExtensionRegistry.getTemplateExtensionForFileName(app.getFullFileName()) || app.getFileExtension();
+
+                //#. %1$s is the file extension used for the template file
                 //#, c-format
                 title = gt('Save as template (%1$s)', _.noI18n(templateExt));
+
+                // create the dialog
+                dialog = new Dialogs.SaveAsTemplateDialog(this, {
+                    title: title,
+                    value: newFileName
+                });
             } else {
-                //#. %1$s is the original file name that should be prefixed by 'Copy of' to create a new file that is a copy of the original file
+
+                //#. %1$s is the name of the original file used to create a new copy
                 //#, c-format
                 newFileName = gt('Copy of %1$s', _.noI18n(oldFileName));
-                title = gt('Save as');
-            }
 
-            // create the dialog
-            dialog = new Dialogs.ModalSaveAsDialog({
-                title: title,
-                value: newFileName,
-                folderPicker: type !== 'template'
-            });
+                title = gt('Save as');
+
+                // create the dialog
+                dialog = new Dialogs.SaveAsFileDialog({ title: title, value: newFileName, preselect: app.getFileParameters().folder_id });
+            }
 
             // show the dialog, and perform the save action
             return dialog.show().then(function () {
@@ -634,13 +765,29 @@ define('io.ox/office/editframework/view/editview',
         /**
          * return whether tool-/tabbars should be combined or not
          *
-         * @return {Boolean}
+         * @returns {Boolean}
          *  'true' for combining the bars
          */
-        this.panesCombined = function(){
+        this.panesCombined = function () {
             return panesCombined;
         };
 
+        /**
+         * Returns the effective zoom factor of the document. This method MUST
+         * be overwritten by the respective applications sub classes.
+         *
+         * @attention
+         *  Existence of this method is expected by code in base modules, e.g.
+         *  the drawing layer.
+         *
+         * @returns {Number}
+         *  The effective zoom factor for the document.
+         */
+        this.getZoomFactor = function () {
+            Utils.error('EditView.getZoomFactor(): missing implementation');
+            throw new Error('abstract method called');
+        };
+
         // undo manager -------------------------------------------------------
 
         /**
@@ -651,7 +798,7 @@ define('io.ox/office/editframework/view/editview',
          *  Whether at least one undo action is available on the stack.
          */
         this.isUndoAvailable = function () {
-            return app.getModel().getUndoManager().getUndoCount() > 0;
+            return docModel.getUndoManager().getUndoCount() > 0;
         };
 
         /**
@@ -662,7 +809,7 @@ define('io.ox/office/editframework/view/editview',
          *  applied.
          */
         this.undo = function () {
-            return app.getModel().getUndoManager().undo(1);
+            return docModel.getUndoManager().undo(1);
         };
 
         /**
@@ -673,7 +820,7 @@ define('io.ox/office/editframework/view/editview',
          *  Whether at least one redo action is available on the stack.
          */
         this.isRedoAvailable = function () {
-            return app.getModel().getUndoManager().getRedoCount() > 0;
+            return docModel.getUndoManager().getRedoCount() > 0;
         };
 
         /**
@@ -684,21 +831,121 @@ define('io.ox/office/editframework/view/editview',
          *  applied.
          */
         this.redo = function () {
-            return app.getModel().getUndoManager().redo(1);
+            return docModel.getUndoManager().redo(1);
+        };
+
+        // search/replace -----------------------------------------------------
+
+        /**
+         * Returns the current search settings, as currently configured in the
+         * search pane.
+         *
+         * @returns {Object}
+         *  The current search settings, with the following properties:
+         *  - {String} query
+         *      The search query string.
+         *  - {Boolean} regexp
+         *      Whether the query string shall be interpreted as regular
+         *      expression, instead of literal search string.
+         */
+        this.getSearchSettings = function () {
+            return _.clone(searchSettings);
+        };
+
+        /**
+         * Changes a property in the search configuration, and initiates a new
+         * search session with the current settings by invoking the search
+         * handler passed to the constructor.
+         *
+         * @param {String} name
+         *  The name of the property to be changed in the search configuration.
+         *  MUST be the name of a supported search configuration property.
+         *
+         * @param {Any} value
+         *  The new value for the configuration property.
+         *
+         * @returns {jQuery.Promise}
+         *  The promise of a Deferred object that will be resolved when
+         *  searching with the new configuration has finished, or rejected with
+         *  an error code that may lead to an alert message box.
+         */
+        this.changeSearchSettings = function (newSettings) {
+            _.extend(searchSettings, newSettings);
+            return this;
+        };
+
+        /**
+         * Invokes the search/replace handler passed to the constructor of this
+         * instance.
+         *
+         * @param {String} command
+         *  The search/replace command to be executed. Will be one of the
+         *  following commands:
+         *  - 'search:start'
+         *      Start a new search/replace session. Will be sent after entering
+         *      some text in the query input field, or changing any search
+         *      options which influence the matching document contents.
+         *  - 'search:prev'
+         *      Search for the previous occurrence of the current query string.
+         *  - 'search:next'
+         *      Search for the next occurrence of the current query string.
+         *  - 'search:end'
+         *      Finish the current search/replace session, remove all feedback
+         *      for search/replace in the user interface. Will be sent after
+         *      closing the search pane.
+         *  - 'replace:next'
+         *      Replace the next occurrence of the current query string with
+         *      the replacement text.
+         *  - 'replace:all'
+         *      Replace all occurrences of the current query string with the
+         *      replacement text.
+         *
+         * @param {Object} [newSettings]
+         *  New properties for the search configuration object. Will be
+         *  inserted into the configuration before invoking the serahc handler.
+         *
+         * @returns {jQuery.Promise}
+         *  The promise of a Deferred object that will be resolved when the
+         *  search handler has finished the specified operation, or that will
+         *  be rejected on any error.
+         */
+        this.executeSearchOperation = function (command, newSettings) {
+
+            var // the original result of the search handler
+                result = null,
+                // the promise returned by this method
+                promise = null;
+
+            // change the specified configuration properties
+            if (_.isObject(newSettings)) { this.changeSearchSettings(newSettings); }
+
+            // invoke the search handler
+            result = searchHandler.call(this, command, _.clone(searchSettings));
+
+            // convert error code (non-empty strings) to rejected promise
+            promise = (_.isString(result) && result) ? $.Deferred().reject(result) : $.when(result);
+
+            // TODO: show alert messages for generic errors
+            promise.fail(function (result) {
+                Utils.error('executeSearchOperation failed', result);
+            });
+
+            return promise;
         };
 
         // initialization -----------------------------------------------------
 
-        // early initialization after MVC construction
-        app.onInit(function () {
-            collaboratorPopup = new UsersLayerMenu(app);
-        });
+        // create class members relying on complete view
+        toolBarTabs = new ToolBarTabCollection(this);
+        collaboratorPopup = new UsersLayerMenu(this, { showOwnColor: userLayerShowOwnColor });
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
             toolBarTabs.destroy();
             collaboratorPopup.destroy();
-            app = initOptions = self = toolBarTabs = topPane = toolPane = collaboratorPopup = null;
+            app = searchHandler = initOptions = self = docModel = null;
+            toolBarTabs = topPane = toolPane = searchGroup = null;
+            collaboratorPopup = null;
         });
 
     } // class EditView
