@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/editframework/view/popup/userslayermenu', [
     'io.ox/office/tk/utils',
     'io.ox/office/baseframework/view/popup/layermenu',
     'io.ox/office/editframework/view/control/userbadge',
-    'gettext!io.ox/office/editframework'
+    'gettext!io.ox/office/editframework/main'
 ], function (Utils, LayerMenu, UserBadge, gt) {
 
     'use strict';
@@ -29,14 +29,17 @@ define('io.ox/office/editframework/view/popup/userslayermenu', [
      *
      * @extends LayerMenu
      *
-     * @param {BaseApplication} app
-     *  The application containing this menu instance.
+     * @param {EditView} docView
+     *  The document view containing this menu instance.
      */
-    function UsersLayerMenu(app) {
+    function UsersLayerMenu(docView, initOptions) {
 
-        var // the layerMenu-class
+        var // self reference
             self = this,
 
+            // the application instance
+            app = docView.getApp(),
+
             // all existing user badges, mapped by unique user index
             bagdeGroups = {},
 
@@ -44,11 +47,13 @@ define('io.ox/office/editframework/view/popup/userslayermenu', [
             visibleState = false,
 
             // whether to automatically show the menu for at least two active users
-            autoShow = true;
+            autoShow = true,
+
+            showOwnColor = Utils.getBooleanOption(initOptions, 'showOwnColor', false);
 
         // base constructor ---------------------------------------------------
 
-        LayerMenu.call(this, app, gt('Collaborators'), {
+        LayerMenu.call(this, docView, gt('Collaborators'), {
             tooltip: gt('List of collaborators of this document'),
             autoFocus: false,
             rootContainerNode: app.getWindowNode()
@@ -66,17 +71,17 @@ define('io.ox/office/editframework/view/popup/userslayermenu', [
                 localGroups = _.clone(bagdeGroups);
 
             // process all active users (returned array is sorted by unique client index)
-            _.each(app.getActiveClients(), function (client) {
+            _.each(app.getActiveClients(), function (clientData) {
 
                 var // existing badge group
-                    badgeGroup = localGroups[client.clientId];
+                    badgeGroup = localGroups[clientData.clientId];
 
                 if (badgeGroup) {
-                    badgeGroup.updateClient(client).show();
-                    delete localGroups[client.clientId];
+                    badgeGroup.setValue(clientData).show();
+                    delete localGroups[clientData.clientId];
                 } else {
-                    badgeGroup = new UserBadge(app, client);
-                    bagdeGroups[client.clientId] = badgeGroup;
+                    badgeGroup = new UserBadge(docView, clientData, { showOwnColor: showOwnColor });
+                    bagdeGroups[clientData.clientId] = badgeGroup;
                     self.addGroup(null, badgeGroup);
                 }
             });
@@ -88,6 +93,11 @@ define('io.ox/office/editframework/view/popup/userslayermenu', [
             self.refresh();
         }
 
+        /**
+         * Debounced version of the method UsersLayerMenu.fillMenu().
+         */
+        var fillMenuDebounced = this.createDebouncedMethod(_.noop, fillMenu);
+
         /**
          * Sets the default position of the collaborators list layer menu
          * (right top corner of the application content area).
@@ -100,7 +110,7 @@ define('io.ox/office/editframework/view/popup/userslayermenu', [
         // initialization -----------------------------------------------------
 
         // lazy initialization of the drop-down menu
-        this.one({'popup:beforeshow': fillMenu, 'popup:show': setDefaultPosition });
+        this.one({ 'popup:beforeshow': fillMenu, 'popup:show': setDefaultPosition });
 
         // disable auto-open functionality if user has closed this pop-up
         this.on('menu:userclose', function () { autoShow = false; });
@@ -109,7 +119,7 @@ define('io.ox/office/editframework/view/popup/userslayermenu', [
         this.listenTo(app, 'docs:users', function (activeClients) {
 
             // redraw user badges
-            fillMenu();
+            fillMenuDebounced();
 
             // auto show once if somebody joins
             if (activeClients.length < 2) {
@@ -124,6 +134,9 @@ define('io.ox/office/editframework/view/popup/userslayermenu', [
             }
         });
 
+        // bug 36482: repaint when application state changes, e.g. offline mode
+        this.listenTo(app, 'docs:state', fillMenuDebounced);
+
         // bug 35534: restore collaborators pop-up
         this.listenTo(app.getWindow(), 'show', function () {
             if (visibleState && autoShow && app.getActiveClients().length >= 2) {
@@ -138,7 +151,7 @@ define('io.ox/office/editframework/view/popup/userslayermenu', [
 
         // destroy all class members
         this.registerDestructor(function () {
-            app = self = bagdeGroups = null;
+            self = app = docView = bagdeGroups = null;
         });
 
     } // class UsersLayerMenu
