@@ -6,15 +6,16 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/view/toolbartabcollection',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/object/triggerobject'
-    ], function (Utils, TriggerObject) {
+define('io.ox/office/editframework/view/toolbartabcollection', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/object/triggerobject',
+    'io.ox/office/baseframework/view/viewobjectmixin'
+], function (Utils, TriggerObject, ViewObjectMixin) {
 
     'use strict';
 
@@ -71,8 +72,9 @@ define('io.ox/office/editframework/view/toolbartabcollection',
      * @constructor
      *
      * @extends TriggerObject
+     * @extends ViewObjectMixin
      */
-    function ToolBarTabCollection(app) {
+    function ToolBarTabCollection(docView) {
 
         var // self reference
             self = this,
@@ -81,11 +83,15 @@ define('io.ox/office/editframework/view/toolbartabcollection',
             tabs = [],
 
             // identifier of the active tab
-            activeTabId = null;
+            activeTabId = null,
 
-        // base constructor ---------------------------------------------------
+            // identifier of the last active tab, while no tab is active (bug 39071)
+            lastTabId = null;
+
+        // base constructors --------------------------------------------------
 
         TriggerObject.call(this);
+        ViewObjectMixin.call(this, docView);
 
         // private methods ----------------------------------------------------
 
@@ -134,9 +140,11 @@ define('io.ox/office/editframework/view/toolbartabcollection',
         /**
          * Updates the visibility of the tab buttons.
          */
-        function controllerChangeHandler(event, changedItems) {
+        function controllerChangeHandler() {
 
-            var // all tabs currently visible
+            var // the application controller
+                controller = docView.getApp().getController(),
+                // all tabs currently visible
                 visibleTabs = _.where(tabs, { visible: true }),
                 // all tabs currently hidden
                 hiddenTabs = _.where(tabs, { visible: false }),
@@ -147,20 +155,20 @@ define('io.ox/office/editframework/view/toolbartabcollection',
             _.each(tabs, function (tab) {
 
                 var // tabs without visibility key are always visible
-                    visible = true;
+                    visible = _.isString(tab.visibleKey) ? controller.isItemEnabled(tab.visibleKey) : true,
+                    // get dynamic label text
+                    label = _.isString(tab.labelKey) ? controller.getItemValue(tab.labelKey) : true;
 
-                // show or hide the tabs, if the controller item is contained in the event data
-                if (_.isString(tab.visibleKey)) {
-                    visible = (tab.visibleKey in changedItems) ? changedItems[tab.visibleKey].enabled : null;
-                }
-                if (_.isBoolean(visible) && (tab.visible !== visible)) {
+                // show or hide the tabs, if the state has changed
+                if (tab.visible !== visible) {
                     tab.visible = visible;
                     self.trigger(visible ? 'tab:show' : 'tab:hide', tab.tabId);
                 }
 
                 // change the button text, if the item is contained in the event data
-                if (_.isString(tab.labelKey) && (tab.labelKey in changedItems)) {
-                    self.trigger('tab:label', tab.tabId, changedItems[tab.labelKey].value);
+                if (_.isString(label) && (tab.label !== label)) {
+                    tab.label = label;
+                    self.trigger('tab:label', tab.tabId, label);
                 }
             });
 
@@ -223,7 +231,8 @@ define('io.ox/office/editframework/view/toolbartabcollection',
                     labelKey: Utils.getStringOption(options, 'labelKey'),
                     priority: Utils.getIntegerOption(options, 'priority', 0),
                     sticky: Utils.getBooleanOption(options, 'sticky', false),
-                    visible: true
+                    visible: true,
+                    label: null
                 };
 
             // insert new tab and notify listeners
@@ -266,7 +275,9 @@ define('io.ox/office/editframework/view/toolbartabcollection',
                 var oldTabId = activeTabId;
                 activeTabId = tabId;
                 this.trigger('tab:activate', tabId, oldTabId);
-                app.getController().update();
+                docView.getApp().getController().update();
+                // bug 39071: rescue last tab identifier, to be able to restore later
+                if (!tabId) { lastTabId = oldTabId; }
             }
             return this;
         };
@@ -282,16 +293,32 @@ define('io.ox/office/editframework/view/toolbartabcollection',
             return this;
         };
 
+        /**
+         * Restores the last visible tab, if no tab is currently visible.
+         *
+         * @returns {ToolBarTabCollection}
+         *  A reference to this instance.
+         */
+        this.restoreActivateTab = function () {
+            if (!activeTabId) {
+                if (lastTabId) {
+                    this.activateTab(lastTabId);
+                    lastTabId = null;
+                } else {
+                    activateFirstTab();
+                }
+            }
+            return this;
+        };
+
         // initialization -----------------------------------------------------
 
         // update visibility of the tabs
-        app.onInit(function () {
-            self.listenTo(app.getController(), 'change:items', controllerChangeHandler);
-        });
+        this.handleChangedControllerItems(controllerChangeHandler);
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = self = tabs = null;
+            self = docView = tabs = null;
         });
 
     } // class ToolBarTabCollection
