@@ -6,19 +6,19 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/view/toppane',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/view/toolpane',
-     'io.ox/office/baseframework/view/toolbar',
-     'io.ox/office/editframework/view/editlabels',
-     'io.ox/office/editframework/view/editcontrols',
-     'gettext!io.ox/office/editframework'
-    ], function (Utils, ToolPane, ToolBar, Labels, Controls, gt) {
+define('io.ox/office/editframework/view/toppane', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/baseframework/view/toolpane',
+    'io.ox/office/baseframework/view/toolbar',
+    'io.ox/office/editframework/view/editlabels',
+    'io.ox/office/editframework/view/editcontrols',
+    'gettext!io.ox/office/editframework/main'
+], function (Utils, ToolPane, ToolBar, Labels, Controls, gt) {
 
     'use strict';
 
@@ -33,13 +33,13 @@ define('io.ox/office/editframework/view/toppane',
      *
      * @extends ToolPane
      *
-     * @param {EditApplication} app
-     *  The application containing this view pane.
+     * @param {EditView} docView
+     *  The document view instance containing this view pane.
      *
      * @param {ToolBarTabCollection} toolBarTabs
      *  The collection of all registered tool bar tabs.
      */
-    function TopPane(app, toolBarTabs) {
+    function TopPane(docView, toolBarTabs) {
 
         var // self reference
             self = this,
@@ -67,10 +67,9 @@ define('io.ox/office/editframework/view/toppane',
             // shrink controls in multiple states
             shrinkState = 0;
 
-
         // base constructor ---------------------------------------------------
 
-        ToolPane.call(this, app, { position: 'top', classes: 'top-pane' });
+        ToolPane.call(this, docView, { position: 'top', classes: 'top-pane' });
 
         // private methods ----------------------------------------------------
 
@@ -91,9 +90,8 @@ define('io.ox/office/editframework/view/toppane',
                 centerWidth         = centerAreaNode.outerWidth(),
                 trailWidth          = trailAreaNode.outerWidth();
 
-
             // set the new widths of the nodes (wrapper + content nodes)
-            function widthUpdate(){
+            function widthUpdate() {
                 leadWrapperWidth    = leadWrapperNode.width();
                 centerWrapperWidth  = centerWrapperNode.width();
                 trailWrapperWidth   = trailWrapperNode.width();
@@ -104,12 +102,12 @@ define('io.ox/office/editframework/view/toppane',
             }
 
             // get total needed width of all content-nodes
-            function getTotalNeededWidth(){
+            function getTotalNeededWidth() {
                 return (leadWidth + centerWidth + trailWidth);
             }
 
             // shrink controls in specific steps (self prio)
-            function shrink(){
+            function shrink () {
                 switch (shrinkState) {
 
                     case 0:                 // on shrink state 0
@@ -134,7 +132,7 @@ define('io.ox/office/editframework/view/toppane',
             }
 
             // unshrink controls in opposite direction as shrink does
-            function unshrink(){
+            function unshrink () {
                 switch (shrinkState) {
 
                     case 1:                 // on shrink state 1
@@ -158,7 +156,6 @@ define('io.ox/office/editframework/view/toppane',
                 }
             }
 
-
             // NOT enough place
             if (getTotalNeededWidth() > paneWidth) {
                 // reduce the place between the controls
@@ -226,33 +223,33 @@ define('io.ox/office/editframework/view/toppane',
         trailAreaNode = trailWrapperNode.find('>.area-container');
 
         // create the tab button/list controls
-        tabGroup = new Controls.ToolBarTabGroup(app, toolBarTabs);
-        tabList = new Controls.ToolBarTabList(app, toolBarTabs);
+        tabGroup = new Controls.ToolBarTabGroup(toolBarTabs);
+        tabList = new Controls.ToolBarTabList(toolBarTabs);
 
         // create the tool bar containing the tab buttons for the edit tool pane
-        this.addViewComponent(new ToolBar(app)
+        this.addViewComponent(new ToolBar(docView)
             .addGroup('view/toolbars/tab', tabGroup)
             .addGroup('view/toolbars/tab', tabList.hide()),
         { targetArea: 'leading' });
 
         // create the tool bar containing the application status label
-        statusLabel = new Controls.ApplicationStatusLabel(app, { smallerVersion: { css: { width: 20 }, hideLabel: true } });
-        this.addViewComponent(new ToolBar(app).addGroup(null, statusLabel));
+        statusLabel = new Controls.ApplicationStatusLabel(docView, { smallerVersion: { css: { width: 20 }, hideLabel: true } });
+        this.addViewComponent(new ToolBar(docView).addGroup(null, statusLabel));
 
         // center status label in top bar when loading a document
-        this.listenTo(app, 'docs:state', function (state) {
+        this.listenTo(docView.getApp(), 'docs:state', function (state) {
             centerWrapperNode.css('text-align', (state === 'preview' ? 'center' : ''));
         });
 
         // create the 'View' drop-down menu
-        viewMenuGroup = new Controls.CompoundButton(app, { label: Labels.VIEW_LABEL, anchorPadding: -3 });
+        viewMenuGroup = new Controls.CompoundButton(docView, { label: Labels.VIEW_LABEL, anchorPadding: -3 });
 
         // create the tool bar with the standard controls
-        this.addViewComponent(new ToolBar(app)
+        this.addViewComponent(new ToolBar(docView)
             .addGroup('document/undo', new Controls.Button({ icon: 'docs-undo', tooltip: gt('Revert last operation') }), { visibleKey: 'document/editable' })
             .addGroup('document/redo', new Controls.Button({ icon: 'docs-redo', tooltip: gt('Restore last operation') }), { visibleKey: 'document/editable' })
             .addGap()
-            .addGroup('view/searchpane', new Controls.Button({ icon: 'fa-search', tooltip: gt('Toggle search'), toggle: true }), { visibleKey: 'app/valid' })
+            .addGroup('view/searchgroup', new Controls.Button({ icon: 'fa-search', tooltip: gt('Toggle search'), toggle: true }), { visibleKey: 'app/valid' })
             .addGap()
             .addGroup(null, viewMenuGroup)
             .addSeparator()
@@ -264,14 +261,13 @@ define('io.ox/office/editframework/view/toppane',
         this.listenTo(toolBarTabs, 'tab:show', function () { self.updateAreaSizes(); });
         this.listenTo(toolBarTabs, 'tab:hide', function () { self.updateAreaSizes(); });
 
-
         // show tab buttons, hide drop-down list
         tabGroup.show();
         tabList.hide();
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = toolBarTabs = self = tabGroup = tabList = statusLabel = viewMenuGroup = null;
+            self = docView = toolBarTabs = tabGroup = tabList = statusLabel = viewMenuGroup = null;
         });
 
     } // class TopPane
