@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Edy Haryono <edy.haryono@open-xchange.com>
  */
@@ -14,17 +14,16 @@
 /**
  *  Defines extensions and actions of the recent documents pane
  */
-define('io.ox/office/portal/action/recentsactions',
-    ['io.ox/core/extensions',
-     'io.ox/core/extPatterns/links',
-     'io.ox/core/extPatterns/actions',
-     'io.ox/office/portal/portalutils'],
-function (Ext, Links, CoreActions, PortalUtils) {
+define('io.ox/office/portal/action/recentsactions', [
+    'io.ox/core/extensions',
+    'io.ox/core/extPatterns/links',
+    'io.ox/core/extPatterns/actions',
+    'io.ox/office/portal/portalutils'
+], function (Ext, Links, CoreActions, PortalUtils) {
 
     'use strict';
 
-
-    var RecentsExtensions = function () {
+    function RecentsExtensions() {
 
         var recentDocumentExtension = 'io.ox/portal/office/recents/document';
 
@@ -33,8 +32,12 @@ function (Ext, Links, CoreActions, PortalUtils) {
         new Links.Action(recentDocumentExtension + '/open', {
 
             action: function (baton) {
-                var appModuleName = PortalUtils.getModuleName(baton.data.filename);
+                var appModuleName = PortalUtils.getModuleNameForFile(baton.data.filename);
+
+                PortalUtils.trackEvent({ moduleName: appModuleName, target: 'recentlist', action: 'edit' });
+
                 ox.launch(appModuleName + '/main', { action: 'load', file: baton.data });
+                //FIXME: TODO: combine me with normal file-actions
             }
 
         });
@@ -48,18 +51,14 @@ function (Ext, Links, CoreActions, PortalUtils) {
                 var link = $('<a>').addClass('document-link row')
                     .attr({
                         'data-toggle': 'tooltip',
-                        'data-placement': 'top',
-                        'title': _.noI18n(baton.data.filename),
-                        'tabindex': 0
+                        title: _.noI18n(baton.data.filename),
+                        tabindex: 0
                     }),
                     metadata = $('<div>').addClass('document-metadata'),
                     name = $('<div>').addClass('document-name col-xs-8'),
                     date = $('<div>').addClass('document-date col-xs-4')
                         .text(_.noI18n(PortalUtils.formatDate(baton.data.last_modified ? baton.data.last_modified : baton.data.last_opened)));
 
-                // enable Bootstrap tooltip except for touch devices, since it is buggy (see https://github.com/twbs/bootstrap/issues/6232)
-                if (!Modernizr.touch) { link.tooltip(); }
-
                 name.append(PortalUtils.createDocumentIcon(baton.data.filename), $('<div>').text(_.noI18n(baton.data.filename)));
                 link.append(metadata.append(name, date));
 
@@ -74,8 +73,8 @@ function (Ext, Links, CoreActions, PortalUtils) {
 
         });
 
-    };
+    } // class RecentExtensions
 
     return new RecentsExtensions();
 
-});
\ No newline at end of file
+});
