@@ -6,20 +6,20 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Edy Haryono <edy.haryono@open-xchange.com>
  */
 
-define('io.ox/office/portal/collection/documentcollection',
-    ['io.ox/office/portal/model/documentmodel'],
-    function (DocumentModel) {
+define('io.ox/office/portal/collection/documentcollection', [
+    'io.ox/office/portal/model/documentmodel'
+], function (DocumentModel) {
 
     'use strict';
 
     var documentCollection = Backbone.Collection.extend({
 
-        model : DocumentModel,
+        model: DocumentModel,
 
         setData: function (recentsArray) {
 
@@ -27,7 +27,7 @@ define('io.ox/office/portal/collection/documentcollection',
 
             // populate the collection with recent documents
             _.each(recentsArray, function (recentDocument) {
-                self.add(new DocumentModel(recentDocument), {merge: true});
+                self.add(new DocumentModel(recentDocument), { merge: true });
             });
 
         }
@@ -36,4 +36,4 @@ define('io.ox/office/portal/collection/documentcollection',
 
     return documentCollection;
 
-});
\ No newline at end of file
+});
