@@ -6,26 +6,27 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
-define('io.ox/office/portal/mainhelper',
-    ['io.ox/core/date',
-     'io.ox/core/capabilities',
-     'io.ox/core/commons',
-     'io.ox/core/notifications',
-     'io.ox/office/portal/collection/documentcollection',
-     'io.ox/office/portal/collection/templatecollection',
-     'io.ox/office/portal/view/documentview',
-     'io.ox/office/portal/view/templateview',
-     'io.ox/office/portal/view/toolbarview',
-     'io.ox/office/tk/io',
-     'io.ox/office/tk/utils',
-     'settings!io.ox/office',
-     'gettext!io.ox/office/portal',
-     'less!io.ox/office/portal/style'
-    ], function (CoreDate, Capabilities, Commons, Notifications, DocumentCollection, TemplateCollection, DocumentView, TemplateView, ToolbarView, IO, Utils, Settings, gt) {
+define('io.ox/office/portal/mainhelper', [
+    'io.ox/core/commons',
+    'io.ox/core/notifications',
+    'io.ox/office/tk/io',
+    'io.ox/office/tk/utils/driveutils',
+    'io.ox/office/portal/portalutils',
+    'io.ox/office/portal/collection/documentcollection',
+    'io.ox/office/portal/collection/templatecollection',
+    'io.ox/office/portal/view/documentview',
+    'io.ox/office/portal/view/templateview',
+    'io.ox/office/portal/view/toolbarview',
+    'io.ox/office/portal/view/tooltip',
+    'io.ox/office/baseframework/app/guidedtour',
+    'settings!io.ox/office',
+    'gettext!io.ox/office/portal/main',
+    'less!io.ox/office/portal/style'
+], function (Commons, Notifications, IO, DriveUtils, Utils, DocumentCollection, TemplateCollection, DocumentView, TemplateView, ToolbarView, ToolTip, GuidedTour, Settings, gt) {
 
     'use strict';
 
@@ -36,14 +37,16 @@ define('io.ox/office/portal/mainhelper',
 
     // methods ----------------------------------------------------------------
 
-    MainHelper.createLauncher = function (appBaseName, appTitle) {
+    MainHelper.createLauncher = function (appBaseName, appTitle, appOptions) {
 
         var // the module name of the portal application
-            PORTAL_MODULE_NAME = 'io.ox/office/portal/' + appBaseName,
+            PORTAL_MODULE_NAME = Utils.getPortalModuleName(appBaseName),
             // the module name of the edit application
-            EDIT_MODULE_NAME = 'io.ox/office/' + appBaseName,
+            EDIT_MODULE_NAME = Utils.getEditorModuleName(appBaseName),
             // the application instance
-            app = ox.ui.createApp({ name: PORTAL_MODULE_NAME, title: appTitle });
+            app = ox.ui.createApp({ name: PORTAL_MODULE_NAME, title: appTitle, trackingId: 'io.ox/' + appBaseName + '-portal' }),
+            // RequireJS path of the application welcome tour
+            appTourPath = Utils.getStringOption(appOptions, 'appTourPath', '');
 
         app.setLauncher(function (options) {
 
@@ -54,8 +57,11 @@ define('io.ox/office/portal/mainhelper',
                 chromeless: true
             });
 
+            new ToolTip(portalWindow.nodes.main);
+
             // the bootstrap of the OX Documents app
             function initOXDocuments() {
+
                 // prepare app skeleton. Maybe this could be the AppView in the future.
                 var rootNode = portalWindow.nodes.main,
                     windowBodyNode = portalWindow.nodes.body,
@@ -65,40 +71,39 @@ define('io.ox/office/portal/mainhelper',
                     // the document collection for the recent documents list
                     documentCollection = new DocumentCollection(),
                     // the template collection for the listed templates
-                    templateCollection = new TemplateCollection(),
-                    // whether this is an Android device and unsupported Chrome version or other browser => reduce functionality
-                    reducedFunctionality = (_.browser.Android) && (appTitle === 'Text') && !Utils.supportedChromeVersionOnAndroidForText();
+                    templateCollection = new TemplateCollection();
 
                 // prefetch application source code
                 IO.prefetchModuleSource(EDIT_MODULE_NAME);
 
                 rootNode.empty().append(recentsNode.append(recentsList), templatesNode);
 
-                // disable templates in Text App on Androids
-                if (reducedFunctionality) {
-                    recentsNode.addClass('reduced-recents-functionality');
-                    rootNode.find('.office-portal-templates').remove();
-                }
-
                 // create application views
-                new ToolbarView({ el: windowBodyNode, model: app});
-                var documentView = new DocumentView({collection: documentCollection, el: recentsList, model: app}),
-                    templateView = new TemplateView({collection: templateCollection, el: templatesNode, model: app});
-
-                // try to get recent files and templates
-                portalWindow.busy();
-                IO.sendRequest(IO.FILTER_MODULE_NAME, { action: 'gettemplateandrecentlist', type: appBaseName })
-                .always(function () {
-                    portalWindow.idle();
-                }).done(function (data) {
-                    // populate the document and template collection with data from server
-                    documentCollection.setData(data.recents || []);
-                    templateCollection.setData(data.templates || []);
-                    // render views with fresh data
-                    documentView.render(appBaseName);
-                    templateView.render(appBaseName);
-                }).fail(appFail);
+                new ToolbarView({ el: windowBodyNode, model: app });
+                var documentView = new DocumentView({ collection: documentCollection, el: recentsList, model: app }),
+                    templateView = new TemplateView({ collection: templateCollection, el: templatesNode, model: app });
 
+                if (DriveUtils.isGuest()) {
+                    // guests & anonymous users
+                    documentCollection.setData([]);
+                    templateCollection.setData([]);
+                } else {
+                    // try to get recent files and templates
+                    portalWindow.busy();
+                    IO.sendRequest(IO.FILTER_MODULE_NAME, { action: 'gettemplateandrecentlist', type: appBaseName })
+                    .always(function () {
+                        portalWindow.idle();
+                    }).done(function (data) {
+                        // populate the document and template collection with data from server
+                        documentCollection.setData(data.recents || []);
+                        templateCollection.setData(data.templates || []);
+                        // render views with fresh data
+                        documentView.render(appBaseName);
+                        templateView.render(appBaseName);
+                        // start the welcome tour for this application once
+                        GuidedTour.runAppTour(PORTAL_MODULE_NAME, { once: true, auto: true });
+                    }).fail(appFail);
+                }
             }
 
             // yells if something went wrong...
@@ -134,13 +139,14 @@ define('io.ox/office/portal/mainhelper',
             ox.on('refresh^', refresh);
 
             // init OX Documents in folder support addition success.
-            Commons.addFolderSupport(app, null, 'infostore', options.folder)
-                .always(function always() {
-                    initOXDocuments();
-                })
-                .fail(appFail);
+            Commons.addFolderSupport(app, null, 'infostore', options.folder).always(initOXDocuments).fail(appFail);
         });
 
+        // register as application tour (will create an entry in the settings menu to start the tour)
+        if (appTourPath && !_.device('smartphone')) {
+            GuidedTour.registerAppTour(PORTAL_MODULE_NAME, appTourPath);
+        }
+
         return { getApp: app.getInstance };
     };
 
