@@ -6,31 +6,30 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Edy Haryono <edy.haryono@open-xchange.com>
  */
 
-define('io.ox/office/portal/model/documentmodel', ['io.ox/files/api'], function (FilesAPI) {
+define('io.ox/office/portal/model/documentmodel', [
+'io.ox/office/tk/utils/driveutils'
+], function (DriveUtils) {
 
     'use strict';
 
     var documentModel = Backbone.Model.extend({
 
-        defaults : function () {
-            return {
-                previewWidth: 40
-            };
+        defaults: function () {
+            return { previewWidth: 40 };
         },
 
         initialize: function () {
-
-            this.set('previewURL', FilesAPI.getUrl(this.attributes, 'preview'));
-
+            this.set('previewURL', DriveUtils.getFileUrl(this.attributes, 'preview'));
+            this.set('source', 'drive');
         }
 
     });
 
     return documentModel;
 
-});
\ No newline at end of file
+});
