@@ -6,15 +6,19 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
-define('io.ox/office/portal/spreadsheet/main', ['io.ox/office/portal/mainhelper'], function (MainHelper) {
+define('io.ox/office/portal/spreadsheet/main', [
+    'io.ox/office/portal/mainhelper'
+], function (MainHelper) {
 
     'use strict';
 
-    return MainHelper.createLauncher('spreadsheet', 'Spreadsheet');
+    return MainHelper.createLauncher('spreadsheet', _.noI18n('Spreadsheet'), {
+        appTourPath: 'io.ox/office/portal/spreadsheet/fulltour'
+    });
 
 });
