@@ -6,19 +6,17 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Edy Haryono <edy.haryono@open-xchange.com>
  */
 
-define('io.ox/office/portal/view/documentview',
-    ['gettext!io.ox/office/portal',
-     'io.ox/office/portal/portalutils',
-     'io.ox/core/extensions',
-     'io.ox/office/portal/action/recentsactions',
-     'io.ox/core/capabilities'
-    ],
-function (gt, PortalUtils, Ext, RecentsActions) {
+define('io.ox/office/portal/view/documentview', [
+    'io.ox/office/portal/portalutils',
+    'io.ox/core/extensions',
+    'io.ox/office/portal/action/recentsactions',
+    'gettext!io.ox/office/portal/main'
+], function (PortalUtils, Ext, RecentsActions, gt) {
 
     'use strict';
 
@@ -78,7 +76,7 @@ function (gt, PortalUtils, Ext, RecentsActions) {
                 // or if the file name ends with the error extension '_ox'
                 if ((appBaseName !== fileAppName) || PortalUtils.stringEndsWith(model.get('filename'), errorSuffix)) { return; }
                 // draw recent document items and bind actions  through extension invoke
-                RecentsActions.documentPoint.invoke('draw', self.$el, Ext.Baton({ data : model.attributes }));
+                RecentsActions.documentPoint.invoke('draw', self.$el, Ext.Baton({ data: model.attributes }));
             });
 
             self.showOptimalCount(self);
@@ -154,4 +152,4 @@ function (gt, PortalUtils, Ext, RecentsActions) {
 
     return documentView;
 
-});
\ No newline at end of file
+});
