@@ -6,18 +6,19 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Sven Jacobi <sven.jacobi@open-xchange.com>
  */
 
 define('io.ox/office/presentation/app/application',
-    ['io.ox/office/tk/utils',
+[
+    'io.ox/office/tk/utils',
      'io.ox/office/editframework/app/editapplication',
      'io.ox/office/presentation/model/model',
      'io.ox/office/presentation/view/view',
      'io.ox/office/presentation/app/controller'
-    ], function (Utils, EditApplication, PresentationModel, PresentationView, PresentationController) {
+], function (Utils, EditApplication, PresentationModel, PresentationView, PresentationController) {
 
     'use strict';
 
@@ -32,22 +33,18 @@ define('io.ox/office/presentation/app/application',
      *
      * @param {Object} launchOptions
      *  All options passed to the core launcher (the ox.launch() method).
-     *
-     * @param {Object} [appOptions]
-     *  Static application options that have been passed to the static method
-     *  BaseApplication.createLauncher().
      */
-    var PresentationApplication = EditApplication.extend({ constructor: function (launchOptions, appOptions) {
+    var PresentationApplication = EditApplication.extend({ constructor: function (launchOptions) {
 
         // base constructor ---------------------------------------------------
 
-        EditApplication.call(this, PresentationModel, PresentationView, PresentationController, launchOptions, appOptions);
+        EditApplication.call(this, PresentationModel, PresentationView, PresentationController, launchOptions);
 
         // initialization -----------------------------------------------------
 
         // destroy all class members
         this.registerDestructor(function () {
-            launchOptions = appOptions = null;
+            launchOptions = null;
         });
 
     }}); // class PresentationApplication
