@@ -5,15 +5,16 @@
  * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/presentation/app/fileactions',
-    ['io.ox/office/editframework/app/fileactions',
-     'gettext!io.ox/office/files'
-    ], function (FileActions, gt) {
+[
+    'io.ox/office/editframework/app/fileactions',
+     'gettext!io.ox/office/presentation/actions'
+], function (FileActions, gt) {
 
     'use strict';
 
