@@ -6,18 +6,17 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/presentation/model/model',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/model/editmodel',
-     'io.ox/office/editframework/model/undomanager',
-     'io.ox/office/editframework/model/operationsgenerator',
-     'io.ox/office/presentation/model/format/documentstyles'
-    ], function (Utils, EditModel, UndoManager, OperationsGenerator, PresentationDocumentStyles) {
+[
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/model/editmodel',
+    'io.ox/office/presentation/model/modelattributesmixin'
+], function (Utils, EditModel, ModelAttributesMixin) {
 
     'use strict';
 
@@ -31,25 +30,21 @@ define('io.ox/office/presentation/model/model',
      * @constructor
      *
      * @extends EditModel
+     * @extends ModelAttributesMixin
      *
      * @param {PresentationApplication} app
      *  The application containing this document model.
      */
     function PresentationModel(app) {
 
-        var // the undo manager of this document
-            undoManager = new UndoManager(app),
-
-            // container for all style sheets of all attribute families
-            documentStyles = new PresentationDocumentStyles(app);
-
         // base constructor ---------------------------------------------------
 
-        EditModel.call(this, app, OperationsGenerator, undoManager, documentStyles);
+        EditModel.call(this, app);
+        ModelAttributesMixin.call(this);
 
         // private methods ----------------------------------------------------
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
 
         // initialization -----------------------------------------------------
 
