@@ -6,16 +6,17 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/presentation/view/view',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/view/editview',
-     'less!io.ox/office/presentation/view/style'
-    ], function (Utils, EditView) {
+[
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/view/editview',
+    'less!io.ox/office/presentation/view/style'
+], function (Utils, EditView) {
 
     'use strict';
 
@@ -26,16 +27,15 @@ define('io.ox/office/presentation/view/view',
      *
      * @extends EditView
      */
-    function PresentationView(app) {
+    function PresentationView(app, docModel) {
 
         // base constructor ---------------------------------------------------
 
-        EditView.call(this, app, {
+        EditView.call(this, app, docModel, searchHandler, {
             initHandler: initHandler,
             initDebugHandler: initDebugHandler,
             initGuiHandler: initGuiHandler,
-            initDebugGuiHandler: initDebugGuiHandler,
-            overlayMargin: { right: Utils.SCROLLBAR_WIDTH, bottom: Utils.SCROLLBAR_HEIGHT }
+            initDebugGuiHandler: initDebugGuiHandler
         });
 
         // private methods ----------------------------------------------------
@@ -67,6 +67,15 @@ define('io.ox/office/presentation/view/view',
         function initDebugGuiHandler(/*viewMenuGroup, actionMenuGroup*/) {
         }
 
+        /**
+         * Executes a search/replace operation, as requested by the base class
+         * EditView.
+         */
+        function searchHandler(command/*, settings*/) {
+            Utils.error('PresentationView.searchHandler(): unsupported search command "' + command + '"');
+            return 'internal';
+        }
+
         // methods ------------------------------------------------------------
 
     } // class PresentationView
