@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
@@ -15,11 +15,13 @@ define('io.ox/office/settings/register', [
     'io.ox/core/extensions',
     'io.ox/core/manifests',
     'io.ox/office/tk/io',
+    'io.ox/office/tk/utils/driveutils',
     'io.ox/office/settings/templatefolders',
     'io.ox/office/settings/templatemodel',
     'io.ox/office/settings/contexttemplatemodel',
-    'gettext!io.ox/office/main'
-], function (ext, Manifests, IO, TemplateFolders, TemplateModel, ContextTemplateModel, gt) {
+    'io.ox/office/settings/view/settingslabels',
+    'io.ox/office/settings/units'
+], function (ext, Manifests, IO, DriveUtils, TemplateFolders, TemplateModel, ContextTemplateModel, Labels, Units) {
 
     'use strict';
 
@@ -41,38 +43,43 @@ define('io.ox/office/settings/register', [
         inited = true;
 
         //extend the global settings with office settings
-        ext.point('io.ox/settings/pane').extend({
+        ext.point('io.ox/settings/pane/main').extend({
             id: 'io.ox/office',
             title: 'Documents',
             ref: 'io.ox/office',
             loadSettingPane: false,
-            index: 500
+            index: 4000
         });
 
         var settingsObject = {};
         var settingsNode = null;
         var subSettings = [];
 
-        subSettings.push(new TemplateFolders(new TemplateModel('UserTemplateFolders'), { title: gt('My template folders'), useMyFiles: true }));
+        subSettings.push(new Units());
 
-        checkAdmin().always(function () {
-            var resolved = this.state() === 'resolved';
-            var model = null;
+        if (!DriveUtils.isGuest()) {
 
-            var init = function () {
-                subSettings.push(new TemplateFolders(model, { title: gt('Global template folders'), readOnly: !resolved, showReadAccess: resolved }));
-            };
+            subSettings.push(new TemplateFolders(new TemplateModel('MyTemplateFolders'), { title: Labels.USER_TEMPLATE_FOLDER, useMyFiles: true }));
 
-            if (resolved) {
-                model = new TemplateModel('ContextTemplateFolders');
-                init();
-            } else {
-                IO.sendRequest(IO.FILTER_MODULE_NAME, { action: 'getcontexttemplatefolders' }).done(function (data) {
-                    model = new ContextTemplateModel(data);
+            checkAdmin().always(function () {
+                var resolved = this.state() === 'resolved';
+                var model = null;
+
+                var init = function () {
+                    subSettings.push(new TemplateFolders(model, { title: Labels.GLOBAL_TEMPLATE_FOLDER, readOnly: !resolved, showReadAccess: resolved }));
+                };
+
+                if (resolved) {
+                    model = new TemplateModel('ContextTemplateFolders');
                     init();
-                });
-            }
-        });
+                } else {
+                    IO.sendRequest(IO.FILTER_MODULE_NAME, { action: 'getcontexttemplatefolders' }).done(function (data) {
+                        model = new ContextTemplateModel(data);
+                        init();
+                    });
+                }
+            });
+        }
 
         /**
          * standard options section-title
@@ -103,7 +110,7 @@ define('io.ox/office/settings/register', [
             settingsNode = this;
 
             settingsNode.empty();
-            settingsNode.append($('<h1>').text(gt('Documents')));
+            settingsNode.append($('<h1>').text(Labels.DOCUMENTS_HEADER_LABEL));
 
             _.each(subSettings, function (subSetting) {
                 subSetting.refresh(settingsObject);
