@@ -6,22 +6,23 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
 define('io.ox/office/settings/templatefolders', [
-    'io.ox/core/folder/picker',
-    'io.ox/core/folder/api',
+    'io.ox/office/tk/dialogs',
     'io.ox/mail/detail/links',
+    'io.ox/core/notifications',
     'io.ox/office/tk/utils',
-    'gettext!io.ox/office/main'
-], function (FolderPicker, FolderAPI, DeepLinks, Utils, gt) {
+    'io.ox/office/tk/utils/driveutils',
+    'io.ox/office/settings/view/settingslabels'
+], function (Dialogs, DeepLinks, Notifications, Utils, DriveUtils, Labels) {
 
     'use strict';
 
-    // class TemplateFolders ====================================================
+    // class TemplateFolders ==================================================
 
     /**
      * @param {TemplateModel} templateModel
@@ -36,28 +37,27 @@ define('io.ox/office/settings/templatefolders', [
      */
     function TemplateFolders(templateModel, options) {
         var readOnly = Utils.getBooleanOption(options, 'readOnly', false);
+        var showReadAccess = Utils.getBooleanOption(options, 'showReadAccess', false);
+
+        var myFiles = DriveUtils.getStandardTemplateFolderId();
 
         this.refresh = function (settings) {
-            settings.appendTitle(Utils.getStringOption(options, 'title', gt('My template folders')));
+            settings.appendTitle(Utils.getStringOption(options, 'title', ''));
 
             var folders = $('<ul class="list-unstyled publications list-group widget-list"></ul>');
 
             settings.append(folders);
 
-
             /**
-             * calls FolderApi.path(id), but checks if the folder is deleted,
-             * if not callback is called
+             * calls DriveUtils.getTrashState() and if not
+             * it calls DriveUtils.getPath(id)
+             * and ssignes this to callback
              */
             function path(id, callback) {
-                FolderAPI.path(id).done(function(path) {
-                    for (var i = 1; i < path.length; i++) {
-                        if (FolderAPI.is('trash', path[i])) {
-                            templateModel.remove(id);
-                            return;
-                        }
+                DriveUtils.getTrashState(id).done(function (state) {
+                    if (!state) {
+                        DriveUtils.getPath(id).done(callback);
                     }
-                    callback(path);
                 });
             }
 
@@ -73,6 +73,10 @@ define('io.ox/office/settings/templatefolders', [
                 }
             }
 
+            function handlePermissions(/*btn, id*/) {
+
+            }
+
             /**
              * add new li-element with:
              *
@@ -80,39 +84,50 @@ define('io.ox/office/settings/templatefolders', [
              * folder-link (opens Drive)
              * delete button
              */
-            function add(path, noRemove, oldLi) {
+            function add(path, noRemove, oldLi, click) {
                 var folder = _.last(path);
                 var id = folder.id;
 
-
                 var li = $('<li class="widget-settings-view">');
 
                 var ctrl = $('<div class="widget-controls">');
                 li.append(ctrl);
 
                 if (!noRemove && !readOnly) {
-//                    var edit = $('<a href="#" role="button" tabindex="1" data-action="edit" class="action" style="position: absolute; right: 45px; top: 20px;"></a>');
-//                    edit.text(gt('Edit'));
-//                    edit.click(function(evt) {
-//                        evt.preventDefault();
-//                        clickAdd(evt, li, id);
-//                    });
 
-
-
-                    var rem = $('<a class="remove" href="#" tabindex="1" role="button" data-action="remove">');
-                    //FIXME: aria-label
-
-                    rem.append('<i class="fa fa-trash-o">');
-                    rem.attr('title', gt('Delete'));
-                    rem.click(function(evt) {
+                    var rem = $('<a href="#" tabindex="1" role="button" data-action="remove" style="right: 0;">');
+                    var icn = $('<i class="fa fa-trash-o">');
+                    rem.append(icn);
+                    rem.attr('title', title);
+                    rem.click(function (evt) {
                         evt.preventDefault();
                         remove(path, li);
                     });
-//                    ctrl.append(edit);
+
                     ctrl.append(rem);
                 }
 
+                if (showReadAccess) {
+                    var changeListener = function () {
+                        DriveUtils.showPermissionsDialog(id).always(function () {
+                            handlePermissions(perm, id);
+                        });
+                    };
+
+                    var perm = $('<a href="#" tabindex="1" role="button" style="right: 45px; position: absolute; margin-top: -5px; padding-right: 10px;">');
+                    perm.text(Labels.PERMISSIONS_LABEL);
+                    ctrl.append(perm);
+                    perm.click(function (evt) {
+                        evt.preventDefault();
+                        changeListener();
+                    });
+                    handlePermissions(perm, id);
+
+                    if (click) {
+                        Notifications.yell({ type: 'info', message: Labels.PERMISSIONS_MESSAGE_LABEL });
+                    }
+                }
+
                 var cnt = $('<span class="content">');
                 li.append(cnt);
 
@@ -122,17 +137,17 @@ define('io.ox/office/settings/templatefolders', [
 
                 var link = $('<a tabindex="1" class="sectiontitle" target="_blank" style="text-align: left; display: block; max-width: 80%;" href="#"></a>');
 
-                link.data({folder: id});
+                link.data({ folder: id });
                 link.addClass('deep-link-files');
 
                 var displayPath = '';
-                for (var i = 1; i < path.length-1; i++) {
-                    displayPath += FolderAPI.getFolderTitle(path[i].title, 30) + ' / ';
+                for (var i = 1; i < path.length - 1; i += 1) {
+                    displayPath += DriveUtils.getShortenedTitle(path[i].title, 30) + ' / ';
                 }
-                displayPath += FolderAPI.getFolderTitle(folder.title, 30);
+                displayPath += DriveUtils.getShortenedTitle(folder.title, 30);
 
-                link.text(gt.noI18n(displayPath));
-                link.attr('title', gt.noI18n(displayPath));
+                link.text(_.noI18n(displayPath));
+                link.attr('title', _.noI18n(displayPath));
 
                 cnt.append(link);
 
@@ -141,28 +156,19 @@ define('io.ox/office/settings/templatefolders', [
                 } else {
                     folders.append(li);
                 }
-
-
             }
 
             if (Utils.getBooleanOption(options, 'useMyFiles', false)) {
-                var myFiles = String(FolderAPI.getDefaultFolder('infostore'));
-
-                path(myFiles, function(path) {
+                path(myFiles, function (path) {
                     add(path, true);
                 });
             }
 
             //load all folders saved on account
             //generates its gui elements
-            _.each(templateModel.getAll(), function(id) {
-                try {
-                    delete FolderAPI.pool.models[id];
-                    //on this way i force a refresh of the folder, if its deleted
-                } catch (e) {
-                    if (ox.debug) { console.warn('cannot delete FolderAPI.pool.models', e); }
-                }
-                path(id, function(path) {
+            _.each(templateModel.getAll(), function (id) {
+                DriveUtils.deleteFromCache(id);
+                path(id, function (path) {
                     add(path);
                 });
             });
@@ -177,34 +183,17 @@ define('io.ox/office/settings/templatefolders', [
                 }
                 busy = true;
 
-                var theTree = null;
-                var dia = null;
-
-                new FolderPicker({
-                    addClass: '',
-                    button: gt('OK'),
+                var dialog = new Dialogs.ModalFolderPicker({
+                    button: Labels.OK_LABEL,
                     height: 350,
-                    module: 'infostore',
-                    persistent: 'folderpopup/filepicker',
-                    root: '9',
-                    title: Utils.getStringOption(options, 'folderPicker', gt('Select template folder')),
-
-                    initialize: function(dialog, tree) {
-                        theTree = tree;
-                        dia = dialog;
-                    },
-
-                    filter: function (parent, model) {
-                        if (FolderAPI.is('trash', model.toJSON())) { return false; }
-                    }
-                }).done(function (reason) {
+                    title: Utils.getStringOption(options, 'folderPicker', Labels.SELECT_TEMPLATE_FOLDER)
+                });
+                dialog.show().always(function () {
                     busy = false;
-                    if (reason !== 'ok') {
-                        return;
-                    }
-                    var folder_id = theTree.selection.get();
-                    if (folder_id !== undefined && folder_id !== myFiles) {
-                        path(folder_id, function(path) {
+                }).done(function (folder_id) {
+                    var folder_id = dialog.getSelectedFolderId();
+                    if (folder_id && folder_id !== myFiles) {
+                        path(folder_id, function (path) {
                             var id = _.last(path).id;
                             if (!templateModel.add(id)) {
                                 if (ox.debug) { console.warn('folder already in list', id); }
@@ -213,11 +202,10 @@ define('io.ox/office/settings/templatefolders', [
                             if (oldId) {
                                 templateModel.remove(oldId);
                             }
-                            add(path, false, oldLi);
+                            add(path, false, oldLi, true);
                         });
                     }
                 });
-                dia.getPopup().focus();
             }
 
             if (!readOnly) {
@@ -225,7 +213,7 @@ define('io.ox/office/settings/templatefolders', [
                  * standard button for opening FolderPicker
                  * if selected folder is new, it will be added to account & to gui
                  */
-                settings.appendButton(Utils.getStringOption(options, 'newFolder', gt('Add new template folder')), clickAdd);
+                settings.appendButton(Utils.getStringOption(options, 'newFolder', Labels.ADD_NEW_TEMPLATE_FOLDER), clickAdd);
             }
 
             settings.append($('<div style="display: block; height: 20px">'));
