@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
@@ -27,11 +27,16 @@ define('io.ox/office/settings/templatemodel', [
     function Model(key) {
         var self = this;
 
+        function set(list) {
+            Settings.set(key, list).save();
+        }
+
         /**
          * returns all template folder-ids in one array
          */
-        this.getAll = function() {
-            return Settings.get(key, []);
+        this.getAll = function () {
+            var list = Settings.get(key, []);
+            return list;
         };
 
         /**
@@ -39,14 +44,12 @@ define('io.ox/office/settings/templatemodel', [
          * double entries will be ignored and return false
          * new entries return true
          */
-        this.add = function(folderId) {
+        this.add = function (folderId) {
             var list = self.getAll();
-//            console.warn('lest before', list);
             var len = list.length;
             list.push(folderId);
             list = _.unique(list);
-//            console.warn('lest after', list);
-            Settings.set(key, list).save();
+            set(list);
 
             return len !== list.length;
         };
@@ -56,14 +59,14 @@ define('io.ox/office/settings/templatemodel', [
          * if id is not in the list, false will be returned and no changes are saved
          * if deletion is successful, it is saved and returns true
          */
-        this.remove = function(folderId) {
+        this.remove = function (folderId) {
             var list = self.getAll();
             var index = list.indexOf(folderId);
-            if (index<0) {
+            if (index < 0) {
                 return false;
             }
             list.splice(index, 1);
-            Settings.set(key, list).save();
+            set(list);
             return true;
         };
     }
