@@ -6,36 +6,46 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
 define('io.ox/office/spreadsheet/app/drawingcontroller', [
     'io.ox/office/tk/utils',
+    'io.ox/office/editframework/utils/color',
+    'io.ox/office/drawinglayer/utils/drawingutils',
     'io.ox/office/drawinglayer/view/drawinglabels',
     'io.ox/office/spreadsheet/utils/operations',
+    'io.ox/office/spreadsheet/utils/sheetutils',
     'io.ox/office/spreadsheet/model/formula/tokenarray',
     'io.ox/office/spreadsheet/view/chartcreator',
-    'gettext!io.ox/office/spreadsheet'
-], function (Utils, DrawingLabels, Operations, TokenArray, ChartCreator, gt) {
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, Color, DrawingUtils, DrawingLabels, Operations, SheetUtils, TokenArray, ChartCreator, gt) {
 
     'use strict';
 
-    // class DrawingController ================================================
+    // class DrawingControllerMixin ===========================================
 
     /**
-     * DrawingController (will) encapsulate(s) all Drawing-Features of the Controller
+     * Drawing controller (will) encapsulate(s) all drawing features of the controller
      * For now Chart are part of Drawings!
+     *
+     * @constructor
      */
-    function DrawingController(app) {
+    function DrawingControllerMixin(app, docModel, docView) {
 
         var self = this,
-            docView = null,
+
             sourceSelector = false,
+
             chartSourceToken = null,
+
             // the token arrays containing the highlighted ranges of the selected drawing object
-            tokenArrays = [];
+            tokenArrays = [],
+
+            // the unique identifier of the range highlighting mode
+            highlightUid = null;
 
         // private methods ----------------------------------------------------
 
@@ -67,74 +77,74 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
             default:
                 Utils.warn('DrawingController.yellChartDirection(): unknown warning type: "' + type + '"');
             }
+
+            return $.Deferred().reject();
         }
 
         /**
          * changeSelection
-         * if only one Drawing is selected and this is a Chart, the function highlights its source-cells via TokenArray
+         * if only one drawing is selected and this is a chart, the function highlights its source cells via token array
          */
-        var changeSelection = (function () {
+        function changeSelectionHandler(evt, selection) {
 
-            return function (evt, selection) {
-                if (tokenArrays.length > 0) {
-                    docView.endRangeHighlighting();
-                    tokenArrays = [];
+            if (highlightUid) {
+                docView.endRangeHighlighting(highlightUid);
+                highlightUid = null;
+            }
+
+            tokenArrays = [];
+
+            if (selection.drawings.length === 1) {
+
+                var chartModel = docView.getDrawingCollection().findModel(selection.drawings[0], { type: 'chart' });
+
+                if (chartModel) {
+                    chartModel.getTokenArrays(tokenArrays);
                 }
+            }
 
-                if (selection.drawings.length === 1) {
-
-                    var chartModel = docView.getDrawingCollection().findModel(selection.drawings[0], { type: 'chart' });
-
-                    if (chartModel) {
-                        chartModel.getTokenArrays(tokenArrays);
-                    }
-                }
-
-                if (tokenArrays.length > 0) {
-                    docView.startRangeHighlighting(tokenArrays, { draggable: false, resolveNames: true });
-                }
-            };
-        }());
+            // highlight the source ranges of the chart in the active sheet
+            if (tokenArrays.length > 0) {
+                highlightUid = docView.startRangeHighlighting(tokenArrays, { priority: true, resolveNames: true });
+            }
+        }
 
         /**
-         * @return the ID of the current selected Drawing
+         * Returns the position of the selected drawing object, if exactly one
+         * drawing object is currently selected.
+         *
+         * @returns {Array<Number>|Null}
+         *  The position of the selected drawing object in the active sheet
+         *  (without leading sheet index), if available; otherwise null.
          */
-        function getDrawingIndices() {
-            var selection = docView.getSelection();
-            if (selection.drawings.length === 1) {
-                return selection.drawings[0];
-            }
+        function getDrawingPosition() {
+            var drawingPositions = docView.getSelectedDrawings();
+            return (drawingPositions.length === 1) ? drawingPositions[0] : null;
         }
 
         /**
          * @return the current selected Drawing
          */
         function getDrawingModel(type) {
-            var drawingIndices = getDrawingIndices();
+            var drawingIndices = getDrawingPosition();
             if (drawingIndices) {
                 return docView.getDrawingCollection().findModel(drawingIndices, { type: type });
             }
         }
 
-        function generateDrawingOperation(opName, params, disableAxes) {
-
-            var docModel = app.getModel(),
-                generator = docView.getSheetModel().createOperationsGenerator(),
-                position = getDrawingIndices();
-
-            if (disableAxes) {
-                var noShape = {line: ChartCreator.getNoneShape(), label: false};
-                generator.generateDrawingOperation(Operations.SET_CHART_AXIS_ATTRIBUTES,        position, { attrs: noShape, axis: 'x' });
-                generator.generateDrawingOperation(Operations.SET_CHART_AXIS_ATTRIBUTES,        position, { attrs: noShape, axis: 'y' });
-                generator.generateDrawingOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES,    position, { attrs: noShape, axis: 'x' });
-                generator.generateDrawingOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES,    position, { attrs: noShape, axis: 'y' });
-                generator.generateDrawingOperation(Operations.SET_CHART_TITLE_ATTRIBUTES,       position, { attrs: { text: { link: []}}, axis: 'x' });
-                generator.generateDrawingOperation(Operations.SET_CHART_TITLE_ATTRIBUTES,       position, { attrs: { text: { link: []}}, axis: 'y' });
-            }
-
-            generator.generateDrawingOperation(opName, position, params);
-
-            docModel.applyOperations(generator.getOperations());
+        /**
+         * Generates and applies a drawing operation.
+         *
+         * @returns {jQuery.Promise}
+         *  A promise that will be resolved after the drawing operations have
+         *  been applied successfully, or that will be rejected on any error.
+         */
+        function createAndApplyDrawingOperation(opName, properties, callback) {
+            return docView.getSheetModel().createAndApplyOperations(function (generator) {
+                var position = getDrawingPosition();
+                if (callback) { callback(generator, position); }
+                generator.generateDrawingOperation(opName, position, properties);
+            });
         }
 
         /**
@@ -161,9 +171,13 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
          *
          * @param {Object} attrs
          *  The attribute set to be set for the title.
+         *
+         * @returns {jQuery.Promise}
+         *  A promise that will be resolved after the operation has been
+         *  applied successfully, or that will be rejected on any error.
          */
         function setChartTitleAttributes(titleId, attrs) {
-            generateDrawingOperation(Operations.SET_CHART_TITLE_ATTRIBUTES, { axis: titleId, attrs: attrs });
+            return createAndApplyDrawingOperation(Operations.SET_CHART_TITLE_ATTRIBUTES, { axis: titleId, attrs: attrs });
         }
 
         /**
@@ -177,23 +191,16 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
          * @param {String} title
          *  The new text contents of the title. An empty string will remove the
          *  title from the chart.
+         *
+         * @returns {jQuery.Promise}
+         *  A promise that will be resolved after the operation has been
+         *  applied successfully, or that will be rejected on any error.
          */
         function setChartTitle(titleId, title) {
-            setChartTitleAttributes(titleId, { text: { link: [title] }, character: ChartCreator.getHeadChar() });
+            return setChartTitleAttributes(titleId, { text: { link: [title] }, character: ChartCreator.getHeadChar() });
         }
 
-        /**
-         * generates an insert-Chart-Op and Insert-Chart-DataSeries chosen by
-         * current Selection
-         *
-         * @returns {jQuery.Deferred}
-         *  a deferred object, which is already resolved
-         */
-        function insertChart(id) {
-            return new ChartCreator(app, DrawingLabels.CHART_TYPE_STYLES[id]).deferred();
-        }
-
-        function setChartSource() {
+        function selectNewChartSource() {
 
             var // enter custom selection mode
                 promise = docView.enterCustomSelectionMode('chartsource', {
@@ -207,19 +214,19 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
                 var sheet = docView.getActiveSheet();
 
                 if (!chartSourceToken) {
-                    chartSourceToken = new TokenArray(app, docView.getSheetModel());
+                    chartSourceToken = new TokenArray(docView.getSheetModel());
                 }
 
                 chartSourceToken.clear().appendRange(range, { sheet: sheet, abs: true });
 
                 var chart = getDrawingModel('chart');
 
-                var possSources = chart.getExchangeInfo();
+                var possSources = chart.getDataSourceInfo();
                 if (possSources.warn) {
-                    ChartCreator.generateOperations(app, range, sheet, chart);
-                } else {
-                    ChartCreator.generateOperations(app, range, sheet, chart, 1 - possSources.axis);
+                    return ChartCreator.updateSeries(app, range, sheet, chart);
                 }
+
+                return ChartCreator.updateSeries(app, range, sheet, chart, possSources.axis);
             });
         }
 
@@ -229,22 +236,37 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
          *
          * There is a special behavior for bubble-chart, change to bubble or from bubble.
          * all series-data will be removed an initialized complete new by the ChartCreator
+         *
+         * @returns {jQuery.Promise}
+         *  A promise that will be resolved after the operation has been
+         *  applied successfully, or that will be rejected on any error.
          */
         function changeChartType(id) {
+
             var chart = getDrawingModel('chart');
             var data = DrawingLabels.CHART_TYPE_STYLES[id];
             var oldData = chart.getMergedAttributes().chart;
+
             if (data.type !== oldData.type && (data.type.indexOf('bubble') === 0 || oldData.type.indexOf('bubble') === 0)) {
                 //special behavior for bubble charts!!!
-                var possSources = chart.getExchangeInfo();
+                var possSources = chart.getDataSourceInfo();
                 if (possSources.warn) {
-                    yellChartDirection(possSources.warn);
-                } else {
-                    ChartCreator.generateOperations(app, possSources.range, possSources.sheet, chart, 1 - possSources.axis, data);
+                    return yellChartDirection(possSources.warn);
                 }
-            } else {
-                generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, { attrs: { chart: data} }, (data.type === 'pie2d' || data.type === 'donut2d'));
+                return ChartCreator.updateSeries(app, possSources.range, possSources.sheet, chart, possSources.axis, data);
             }
+
+            return createAndApplyDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, { attrs: { chart: data } }, function (generator, position) {
+                if ((data.type === 'pie2d' || data.type === 'donut2d')) {
+                    var noShape = { line: ChartCreator.getNoneShape(), label: false };
+                    generator.generateDrawingOperation(Operations.SET_CHART_AXIS_ATTRIBUTES,     position, { attrs: noShape, axis: 'x' });
+                    generator.generateDrawingOperation(Operations.SET_CHART_AXIS_ATTRIBUTES,     position, { attrs: noShape, axis: 'y' });
+                    generator.generateDrawingOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, position, { attrs: noShape, axis: 'x' });
+                    generator.generateDrawingOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, position, { attrs: noShape, axis: 'y' });
+                    generator.generateDrawingOperation(Operations.SET_CHART_TITLE_ATTRIBUTES,    position, { attrs: { text: { link: [] } }, axis: 'x' });
+                    generator.generateDrawingOperation(Operations.SET_CHART_TITLE_ATTRIBUTES,    position, { attrs: { text: { link: [] } }, axis: 'y' });
+                }
+            });
         }
 
         function addAxisDef(axisId, defs) {
@@ -261,11 +283,11 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
             }
 
             function setAxisAttributes(chartModel, attrs) {
-                generateDrawingOperation(Operations.SET_CHART_AXIS_ATTRIBUTES, { axis: getCorrectAxisId(chartModel), attrs: attrs });
+                return createAndApplyDrawingOperation(Operations.SET_CHART_AXIS_ATTRIBUTES, { axis: getCorrectAxisId(chartModel), attrs: attrs });
             }
 
             function setGridLineAttributes(chartModel, attrs) {
-                generateDrawingOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, { axis: getCorrectAxisId(chartModel), attrs: attrs });
+                return createAndApplyDrawingOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, { axis: getCorrectAxisId(chartModel), attrs: attrs });
             }
 
             // *** axis items ***
@@ -280,21 +302,21 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
             defs[keyPath + 'attributes'] = {
                 parent: [keyPath + 'model', 'drawing/chart'],
                 get: function (axisModel) { return axisModel ? axisModel.getMergedAttributes() : null; },
-                set: function (attributes) { setAxisAttributes(this.getParentValue(1), attributes); }
+                set: function (attributes) { return setAxisAttributes(this.getParentValue(1), attributes); }
             };
 
             // return or modify the visibility of the axis caption labels
             defs[keyPath + 'labels/visible'] = {
                 parent: [keyPath + 'attributes', 'drawing/chart'],
                 get: function (attributes) { return _.isObject(attributes) && (attributes.axis.label === true) && this.areParentsEnabled(); },
-                set: function (visible) { setAxisAttributes(this.getParentValue(1), { axis: { label: visible } }); }
+                set: function (visible) { return setAxisAttributes(this.getParentValue(1), { axis: { label: visible } }); }
             };
 
             // return or modify the visibility of the axis line
             defs[keyPath + 'line/visible'] = {
                 parent: [keyPath + 'attributes', 'drawing/chart'],
                 get: function (attributes) { return _.isObject(attributes) && (attributes.line.type !== 'none'); },
-                set: function (visible) { setAxisAttributes(this.getParentValue(1), getStandardLineAttributes(visible)); }
+                set: function (visible) { return setAxisAttributes(this.getParentValue(1), getStandardLineAttributes(visible)); }
             };
 
             // *** grid line items ***
@@ -309,14 +331,14 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
             defs[keyPath + 'grid/attributes'] = {
                 parent: [keyPath + 'grid/model', 'drawing/chart'],
                 get: function (gridModel) { return gridModel ? gridModel.getMergedAttributes() : null; },
-                set: function (attributes) { setGridLineAttributes(this.getParentValue(1), attributes); }
+                set: function (attributes) { return setGridLineAttributes(this.getParentValue(1), attributes); }
             };
 
             // return or modify the visibility of the axis grid lines
             defs[keyPath + 'grid/visible'] = {
                 parent: [keyPath + 'grid/attributes', 'drawing/chart'],
                 get: function (attributes) { return _.isObject(attributes) && (attributes.line.type !== 'none'); },
-                set: function (visible) { setGridLineAttributes(this.getParentValue(1), getStandardLineAttributes(visible)); }
+                set: function (visible) { return setGridLineAttributes(this.getParentValue(1), getStandardLineAttributes(visible)); }
             };
 
             // *** title items ***
@@ -331,25 +353,24 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
             defs[keyPath + 'title/attributes'] = {
                 parent: [keyPath + 'title/model', 'drawing/chart'],
                 get: function (titleModel) { return titleModel ? titleModel.getMergedAttributes() : null; },
-                set: function (attributes) { setChartTitleAttributes(getCorrectAxisId(this.getParentValue(1)), attributes); }
+                set: function (attributes) { return setChartTitleAttributes(getCorrectAxisId(this.getParentValue(1)), attributes); }
             };
 
             // return or modify the text contents of a chart axis title
             defs[keyPath + 'title/text'] = {
                 parent: [keyPath + 'title/attributes', 'drawing/chart'],
                 get: function (attributes) { return getChartTitle(attributes); },
-                set: function (title) { setChartTitle(getCorrectAxisId(this.getParentValue(1)), title); }
+                set: function (title) { return setChartTitle(getCorrectAxisId(this.getParentValue(1)), title); }
             };
 
             // remove the main title from a chart (disabled if title does not exist)
             defs[keyPath + 'title/delete'] = {
                 parent: [keyPath + 'title/text', 'drawing/chart'],
                 enable: function () { return this.getValue().length > 0; },
-                set: function () { setChartTitle(getCorrectAxisId(this.getParentValue(1)), ''); }
+                set: function () { return setChartTitle(getCorrectAxisId(this.getParentValue(1)), ''); }
             };
         }
 
-
         /**
          * private registerDefinitions handles all controller definitions for Drawings and Charts.
          * Including the Insert-Drawing button which is physical part of the original Sidepane but is better placed here
@@ -357,31 +378,32 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
         function registerDefinitions() {
             var defs = {};
 
-            defs['drawing/operations'] = {
-                parent: ['sheet/unlocked', 'view/cell/editmode/off'] // allow insertion of drawings only if sheet is not locked
+            defs['image/insert'] = {
+                parent: 'sheet/operation/unlocked',
+                set: function (imageDesc) { return docView.insertImages(imageDesc); }
             };
 
             defs['image/insert/dialog'] = {
-                parent: 'drawing/operations',
+                parent: 'sheet/operation/unlocked',
                 set: function () { return docView.showInsertImageDialog(); }
             };
 
             defs['chart/insert'] = {
-                parent: 'drawing/operations',
+                parent: 'sheet/operation/unlocked',
                 enable: function () {
                     var ranges = docView.getSelectedRanges();
                     return (ranges.length === 1) && !docView.getSheetModel().isSingleCellInRange(ranges[0]);
                 },
-                set: insertChart
+                set: function (id) { return ChartCreator.createChart(app, DrawingLabels.CHART_TYPE_STYLES[id]); }
             };
 
-            defs['document/editable/drawing'] = {
-                parent: 'sheet/unlocked',
+            defs['drawing/operation'] = {
+                parent: 'sheet/operation/unlocked',
                 enable: function () { return docView.hasDrawingSelection(); }
             };
 
             defs['drawing/model'] = {
-                parent: 'document/editable/drawing',
+                parent: 'drawing/operation',
                 get: getDrawingModel
             };
 
@@ -391,12 +413,18 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
             };
 
             defs['drawing/delete'] = {
-                parent: 'document/editable/drawing',
-                set: function () { docView.deleteDrawings(); }
+                parent: 'drawing/operation',
+                set: function () { return docView.deleteDrawings(); }
+            };
+
+            defs['drawing/image'] = {
+                parent: 'drawing/operation',
+                enable: function () { return _.isObject(this.getValue()); },
+                get: function () { return getDrawingModel('image'); }
             };
 
             defs['drawing/chart'] = {
-                parent: 'document/editable/drawing',
+                parent: 'drawing/operation',
                 enable: function () { return _.isObject(this.getValue()); },
                 get: function () { return getDrawingModel('chart'); }
             };
@@ -405,19 +433,18 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
                 parent: 'drawing/chart',
                 get: function () { return docView.getChartLabelsMenu().isVisible(); },
                 set: function (state) { docView.getChartLabelsMenu().toggle(state); }
-            },
+            };
 
             defs['drawing/chartexchange'] = {
                 parent: 'drawing/chart',
                 enable: function (chartModel) { return chartModel.getSeriesCount() > 0; },
                 set: function () {
                     var chart = getDrawingModel('chart');
-                    var possSources = chart.getExchangeInfo();
+                    var possSources = chart.getDataSourceInfo();
                     if (possSources.warn) {
-                        yellChartDirection(possSources.warn);
-                    } else {
-                        ChartCreator.generateOperations(app, possSources.range, possSources.sheet, chart, possSources.axis);
+                        return yellChartDirection(possSources.warn);
                     }
+                    return ChartCreator.updateSeries(app, possSources.range, possSources.sheet, chart, 1 - possSources.axis);
                 }
             };
 
@@ -427,12 +454,11 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
                 get: function (chartModel) { return chartModel ? chartModel.isFirstColLabel() : null; },
                 set: function (state) {
                     var chart = getDrawingModel('chart');
-                    var possSources = chart.getExchangeInfo();
+                    var possSources = chart.getDataSourceInfo();
                     if (possSources.warn) {
-                        yellChartDirection(possSources.warn);
-                    } else {
-                        ChartCreator.generateOperations(app, possSources.range, possSources.sheet, chart, 1 - possSources.axis, undefined, state, chart.isFirstRowLabel());
+                        return yellChartDirection(possSources.warn);
                     }
+                    return ChartCreator.updateSeries(app, possSources.range, possSources.sheet, chart, possSources.axis, undefined, state, chart.isFirstRowLabel());
                 }
             };
 
@@ -442,12 +468,11 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
                 get: function (chartModel) { return chartModel ? chartModel.isFirstRowLabel() : null; },
                 set: function (state) {
                     var chart = getDrawingModel('chart');
-                    var possSources = chart.getExchangeInfo();
+                    var possSources = chart.getDataSourceInfo();
                     if (possSources.warn) {
-                        yellChartDirection(possSources.warn);
-                    } else {
-                        ChartCreator.generateOperations(app, possSources.range, possSources.sheet, chart, 1 - possSources.axis, undefined, chart.isFirstColLabel(), state);
+                        return yellChartDirection(possSources.warn);
                     }
+                    return ChartCreator.updateSeries(app, possSources.range, possSources.sheet, chart, possSources.axis, undefined, chart.isFirstColLabel(), state);
                 }
             };
 
@@ -473,9 +498,15 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
 
             defs['drawing/chartdatalabel'] = {
                 parent: ['drawing/chart', 'drawing/chart/attributes'],
-                enable: function (chartModel) { return chartModel.isDataLabelEnabled(); },
                 get: function (chartModel, attributes) { return attributes ? attributes.chart.dataLabel : false; },
-                set: function (state) {  generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, { attrs: { chart: { dataLabel: state } } }); }
+                set: function (state) { return createAndApplyDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, { attrs: { chart: { dataLabel: state } } }); }
+            };
+
+            defs['drawing/chartmarkeronly'] = {
+                parent: ['drawing/chart'],
+                enable: function (chartModel) { var data = chartModel ? chartModel.getMergedAttributes().chart : {}; return data.type === 'line2d' || data.type === 'scatter2d'; },
+                get: function (chartModel) { return chartModel ? chartModel.isMarkerOnly() : false; },
+                set: function (state) { this.getParentValue().setMarkerOnly(state); }
             };
 
             defs['drawing/chartcolorset'] = {
@@ -493,13 +524,13 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
             defs['drawing/chartsource'] = {
                 parent: 'drawing/chart',
                 get: function () { return sourceSelector; },
-                set: setChartSource
+                set: selectNewChartSource
             };
 
             defs['drawing/chartlegend/pos'] = {
                 parent: 'drawing/chart',
                 get: function (chartModel) { return chartModel ? chartModel.getLegendModel().getMergedAttributes().legend.pos : null; },
-                set: function (pos) { generateDrawingOperation(Operations.SET_CHART_LEGEND_ATTRIBUTES, { attrs: { legend: { pos: pos }, character: ChartCreator.getStandardChar() } }); }
+                set: function (pos) { return createAndApplyDrawingOperation(Operations.SET_CHART_LEGEND_ATTRIBUTES, { attrs: { legend: { pos: pos } } }); }
             };
 
             defs['drawing/chart/axes/enabled'] = {
@@ -520,21 +551,53 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
             defs['drawing/chart/title/attributes'] = {
                 parent: 'drawing/chart/title/model',
                 get: function (titleModel) { return titleModel ? titleModel.getMergedAttributes() : null; },
-                set: function (attributes) { setChartTitleAttributes('main', attributes); }
+                set: function (attributes) { return setChartTitleAttributes('main', attributes); }
             };
 
             // return or modify the text contents of the main title of a chart
             defs['drawing/chart/title/text'] = {
                 parent: 'drawing/chart/title/attributes',
                 get: function (attributes) { return getChartTitle(attributes); },
-                set: function (title) { setChartTitle('main', title); }
+                set: function (title) { return setChartTitle('main', title); }
             };
 
             // remove the main title from a chart (disabled if title does not exist)
             defs['drawing/chart/title/delete'] = {
                 parent: 'drawing/chart/title/text',
                 enable: function () { return this.getValue().length > 0; },
-                set: function () { setChartTitle('main', ''); }
+                set: function () { return setChartTitle('main', ''); }
+            };
+
+            // IMAGE
+
+            defs['drawing/border/style'] = {
+                parent: 'drawing/image',
+                get: function (imageModel) { return DrawingUtils.getPresetBorder(imageModel ? imageModel.getMergedAttributes().line : null); },
+                set: function (preset) {
+                    var attributes = this.getParentValue().getMergedAttributes(),
+                        options = { line: DrawingUtils.resolvePresetBorder(preset) };
+
+                    if (attributes.line.color.type === 'auto') {
+                        options.line.color = { type: 'rgb', value: '000000' };
+                    }
+
+                    return docView.setDrawingAttributes(options);
+                }
+            };
+
+            defs['drawing/border/color'] = {
+                parent: 'drawing/image',
+                get: function (imageModel) { return imageModel ? imageModel.getMergedAttributes().line.color : Color.AUTO; },
+                set: function (color) {
+                    var attributes = this.getParentValue().getMergedAttributes(),
+                        options = { line: { color: color }};
+
+                    if (attributes.line.type !== 'solid') {
+                        options.line.type = 'solid';
+                    }
+
+                    return docView.setDrawingAttributes(options);
+                }
             };
 
             self.registerDefinitions(defs);
@@ -544,20 +607,16 @@ define('io.ox/office/spreadsheet/app/drawingcontroller', [
 
         registerDefinitions();
 
-        // initialization
-        app.onInit(function () {
-            docView = app.getView();
-            docView.on('change:selection', changeSelection);
-        });
+        this.listenTo(docView, 'change:selection', changeSelectionHandler);
 
         this.registerDestructor(function () {
-            app = self = docView = chartSourceToken = tokenArrays = null;
+            self = app = docModel = docView = chartSourceToken = tokenArrays = null;
         });
 
-    } // class DrawingController
+    } // class DrawingControllerMixin
 
     // exports ================================================================
 
-    return DrawingController;
+    return DrawingControllerMixin;
 
 });
