@@ -5,14 +5,14 @@
  * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/spreadsheet/app/fileactions', [
     'io.ox/office/editframework/app/fileactions',
-    'gettext!io.ox/office/files'
+    'gettext!io.ox/office/spreadsheet/actions'
 ], function (FileActions, gt) {
 
     'use strict';
