@@ -6,19 +6,19 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/drawing/axismodel',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/config',
-     'io.ox/office/drawinglayer/view/chartstyleutil',
-     'io.ox/office/editframework/model/attributedmodel',
-     'io.ox/office/spreadsheet/model/drawing/gridlinemodel',
-     'io.ox/office/spreadsheet/model/drawing/axistitlemodel'
-    ], function (Utils, Config, ChartStyleUtil, AttributedModel, GridlineModel, AxisTitleModel) {
+define('io.ox/office/spreadsheet/model/drawing/axismodel', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/config',
+    'io.ox/office/drawinglayer/view/chartstyleutil',
+    'io.ox/office/editframework/model/attributedmodel',
+    'io.ox/office/spreadsheet/model/drawing/gridlinemodel',
+    'io.ox/office/spreadsheet/model/drawing/axistitlemodel'
+], function (Utils, Config, ChartStyleUtil, AttributedModel, GridlineModel, AxisTitleModel) {
 
     'use strict';
 
@@ -35,8 +35,12 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
     /**
      * visualization model for chart axes,
      * it defines colors and textual colors
+     *
+     * @constructor
+     *
+     * @extends AttributedModel
      */
-    function AxisModel(chart, type, otherType) {
+    function AxisModel(chartModel, type, otherType) {
         var self = this;
         var title = null;
         var zeroLine = null;
@@ -44,24 +48,23 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
         var format = null;
         var labelLen = 0;
 
-        var dataAxis = chart.getModelData()['axis' + type.toUpperCase()];
-        if(dataAxis){
-            grid = new GridlineModel(chart, {}, dataAxis);
-            title = new AxisTitleModel(chart);
+        var dataAxis = chartModel.getModelData()['axis' + type.toUpperCase()];
+        if (dataAxis) {
+            grid = new GridlineModel(chartModel, {}, dataAxis);
+            title = new AxisTitleModel(chartModel);
 
-            var otherAxis = chart.getModelData()['axis' + otherType.toUpperCase()];
-            if(otherAxis.stripLines){
+            var otherAxis = chartModel.getModelData()['axis' + otherType.toUpperCase()];
+            if (otherAxis.stripLines) {
                 zeroLine = otherAxis.stripLines[0];
-                if(zeroLine && !zeroLine.zero){
+                if (zeroLine && !zeroLine.zero) {
                     zeroLine = null;
                 }
             }
         }
 
-
         // base constructor ---------------------------------------------------
 
-        AttributedModel.call(this, chart.getApp(), null, { families: 'axis line character' });
+        AttributedModel.call(this, chartModel.getDocModel(), null, { families: 'axis line character' });
 
         // methods ------------------------------------------------------------
 
@@ -76,11 +79,11 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
 
             var attrs = this.getMergedAttributes();
 
-            ChartStyleUtil.handleLineProps(chart, attrs.line, dataAxis, 'line');
-            ChartStyleUtil.handleLineProps(chart, attrs.line, dataAxis, 'tick');
+            ChartStyleUtil.handleLineProps(chartModel, attrs.line, dataAxis, 'line');
+            ChartStyleUtil.handleLineProps(chartModel, attrs.line, dataAxis, 'tick');
 
             if (attrs.axis.label) {
-                ChartStyleUtil.handleCharacterProps(chart, attrs.character, dataAxis, 'label');
+                ChartStyleUtil.handleCharacterProps(chartModel, attrs.character, dataAxis, 'label');
             } else {
                 dataAxis.labelFontColor = 'transparent';
                 dataAxis.labelFontSize = 1;
@@ -89,13 +92,13 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
             dataAxis.labelAutoFit = false;
             dataAxis.labelAngle = 0;
 
-            var drawingType = chart.getAxisIdForDrawing(type);
+            var drawingType = chartModel.getAxisIdForDrawing(type);
             if (drawingType === 'x') {
-                if (labelLen > 10 ) {
+                if (labelLen > 10) {
                     dataAxis.labelAngle = -20;
                 }
             } else {
-                if (labelLen > 10 ) {
+                if (labelLen > 10) {
                     dataAxis.labelAutoFit = true;
                 }
             }
@@ -107,10 +110,10 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
 
             var overwriteLabels = false;
             var stripLines = dataAxis.stripLines;
-            _.each(stripLines, function(stripLine) {
+            _.each(stripLines, function (stripLine) {
                 if (!stripLine.zero) {
                     if (attrs.axis.label) {
-                        ChartStyleUtil.handleCharacterProps(chart, attrs.character, stripLine, 'label');
+                        ChartStyleUtil.handleCharacterProps(chartModel, attrs.character, stripLine, 'label');
                     } else {
                         stripLine.labelFontColor = 'transparent';
                         stripLine.labelFontSize = 1;
@@ -127,12 +130,12 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
             var titleAtt = title.getMergedAttributes();
             dataAxis.title = titleAtt.text.link[0];
             if (dataAxis.title) {
-                ChartStyleUtil.handleCharacterProps(chart, titleAtt.character, dataAxis, 'title');
+                ChartStyleUtil.handleCharacterProps(chartModel, titleAtt.character, dataAxis, 'title');
             }
 
         };
 
-        this.setFormat = function(newFormat) {
+        this.setFormat = function (newFormat) {
             format = newFormat;
         };
 
@@ -154,25 +157,23 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
             grid = title = null;
         });
 
-
         function refreshMinMax() {
-            var data = chart.getModelData();
+            var data = chartModel.getModelData();
             if (!data.data.length) {
                 return;
             }
-            if(type === 'x'){
+            if (type === 'x') {
                 refreshAxisX(data);
-            }else{
+            } else {
                 refreshAxisY(data);
             }
         }
 
         function makeMin(min, max) {
-            if ((!format || !format.date) && (Math.abs(max - min) < 0.1 || (min / max < 5 / 6))) {
+            if ((!format || !format.date) && (max === min || (min / max < 5 / 6))) {
                 return 0;
-            } else {
-                return min - ((max - min) / 2);
             }
+            return min - ((max - min) / 2);
         }
 
         function makeMax(min, max) {
@@ -183,18 +184,71 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
             return res;
         }
 
-        function setMinMax(min, max, interval, interlaced) {
+        function distance(a, b) {
+            return Math.abs(a - b);
+        }
+        function updateInterval(number) {
+            var normalized = Utils.normalizeNumber(number);
+
+            var dist1 = distance(normalized.mant, 1);
+            var dist2 = distance(normalized.mant, 2);
+            var dist5 = distance(normalized.mant, 5);
+
+            var newMant = null;
+            if (dist1 < dist2) {
+                if (dist1 < dist5) {
+                    newMant = 1;
+                } else {
+                    newMant = 5;
+                }
+            } else {
+                if (dist2 < dist5) {
+                    newMant = 2;
+                } else {
+                    newMant = 5;
+                }
+            }
+            return newMant * Math.pow(10, normalized.exp);
+        }
+
+        function updateMinMax(minValue, maxValue, precision, intervalCount, xAxis) {
+            if (minValue < maxValue) {
+                var interval = (maxValue - minValue) / intervalCount;
+                interval = round(interval, precision - 1);
+                interval = updateInterval(interval);
+
+                if (interval === 0) {
+                    interval = null;
+                    dataAxis.minimum = minValue;
+                    dataAxis.maximum = maxValue;
+                    dataAxis.interval = null;
+                } else {
+                    var intCount = parseFloat((maxValue - minValue) / interval);
+
+                    if (Math.abs(maxValue) < Math.abs(minValue)) {
+                        maxValue = Utils.roundUp(maxValue, interval);
+                        minValue = maxValue - (interval * intCount);
+                    } else {
+                        minValue = Utils.roundDown(minValue, interval);
+                        maxValue = minValue + (interval * intCount);
+                    }
+                }
+                setMinMax(minValue, maxValue, interval, undefined, xAxis);
+            }
+        }
+
+        function setMinMax(min, max, interval, interlaced, xAxis) {
             dataAxis.minimum = min;
             dataAxis.maximum = max;
             dataAxis.interval = interval;
 
             var stripLines = dataAxis.stripLines;
 
-            if(!interval){
-                if(stripLines.length>1){
-                    for (var i = stripLines.length-1; i >= 0 ; i--) {
+            if (!interval) {
+                if (stripLines.length > 1) {
+                    for (var i = stripLines.length - 1; i >= 0; i -= 1) {
                         var stripLine = stripLines[i];
-                        if(!stripLine.zero){
+                        if (!stripLine.zero) {
                             stripLines.splice(i, 1);
                         }
                     }
@@ -203,24 +257,24 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
             }
 
             var index = 0;
-            var formatter = chart.getApp().getModel().getNumberFormatter();
+            var formatter = chartModel.getDocModel().getNumberFormatter();
             var steps = (dataAxis.maximum - dataAxis.minimum) / dataAxis.interval;
             var lastLine = null;
             labelLen = 0;
 
-            for (var i = 0; i <= steps; i ++) {
+            for (var i = 0; i <= steps; i += 1) {
                 var value = dataAxis.minimum + i * dataAxis.interval;
 
                 var line = stripLines[index];
-                if(line && line.zero){
+                if (line && line.zero) {
                     index++;
                     line = stripLines[index];
                 }
-                if(!line){
+                if (!line) {
                     line = {
                         value: 0,
                         color: 'transparent',
-                        label : '',
+                        label: '',
                         labelBackgroundColor: 'transparent'
                     };
                     stripLines[index] = line;
@@ -229,18 +283,27 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
                 line.label = '';
                 line.value = value;
 
-                if(!interlaced || index%2){
+                if (!interlaced || (index % 2)) {
                     var realValue = value * Math.pow(10, format.exp);
-                    if(format && format.format){
+                    if (format && format.format) {
                         line.label = formatter.formatValue(realValue, format.format);
-                    }else{
+                    } else {
                         line.label = formatter.formatStandardNumber(realValue, 8);
                     }
-                    if(lastLine && lastLine.label === line.label){
+                    if (lastLine && lastLine.label === line.label) {
                         line.label = '';
-                    }else{
+                        lastLine = null;
+                    } else {
                         lastLine = line;
                     }
+
+                    if (xAxis && lastLine && lastLine.label.length > 1) {
+                        if (value === max && i === steps) {
+                            line.label = '';
+                        } else if (value === min && !i && dataAxis.labelAngle) {
+                            line.label = '';
+                        }
+                    }
                 }
                 labelLen = Math.max(labelLen, line.label.length);
                 index++;
@@ -253,14 +316,10 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
         }
 
         function refreshAxisX(data) {
-            var area = chart.getMergedAttributes().chart.type.indexOf('area') === 0;
-            if (area && (!format || !format.format)) {
-                area = false;
-            }
-            if (!data.linearX && (chart.isXYType() || area)) {
+            var area = chartModel.getMergedAttributes().chart.type.indexOf('area') === 0;
+            if (!data.linearX && (chartModel.isXYType() || area)) {
                 var minValue = Number.POSITIVE_INFINITY;
                 var maxValue = Number.NEGATIVE_INFINITY;
-                var intervalCount = Math.max(data.data.length, 8);
 
                 _.each(data.data, function (dd) {
                     _.each(dd.dataPoints, function (dataPoint) {
@@ -270,16 +329,14 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
                     });
                 });
 
-//                if (chart.isXYType()) {
-//                    var diff = maxValue - minValue;
-//                    maxValue += 1 + diff / intervalCount;
-//                    minValue -= 1 + diff / intervalCount;
-//                } else {
-//                    minValue -= 0.25;
-//                    maxValue += 0.25;
-//                }
+                if (chartModel.isXYType()) {
+                    var intervalCount = Utils.minMax(data.data.length, 2, 8);
+                    var diff = maxValue - minValue;
+                    maxValue +=  diff / intervalCount;
+                    minValue -=  diff / intervalCount;
+                }
 
-                if(!isFinite(minValue) || !isFinite(maxValue)){
+                if (!isFinite(minValue) || !isFinite(maxValue)) {
                     setMinMax();
                     return;
                 }
@@ -288,23 +345,21 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
                     minValue -= 0.5;
                 }
 
-                var interval = (maxValue - minValue) / intervalCount;
-                setMinMax(minValue, maxValue, interval, true);
+                updateMinMax(minValue, maxValue, 2, 4, true);
             } else {
-                setMinMax(null, null, null);
+                setMinMax(null, null, null, null, true);
             }
         }
 
         function round(value, precision) {
-            if(precision>0&&precision<20){
+            if ((precision > 0) && (precision < 20)) {
                 return parseFloat(value.toPrecision(precision));
-            }else{
-                return value;
             }
+            return value;
         }
 
         function refreshAxisY(data) {
-            var chartAttrs = chart.getMergedAttributes().chart;
+            var chartAttrs = chartModel.getMergedAttributes().chart;
             var stacking = chartAttrs.stacking;
 
             var givenMin = null;
@@ -346,39 +401,40 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
                 });
             }
 
-            var intervalCount = 8;
+            var ignoreMax = false;
 
-            if (chart.isXYType()) {
-//                var diff = maxValue - minValue;
-//                maxValue += 0.005 + diff / intervalCount;
-//                minValue -= 0.005 + diff / intervalCount;
+            if (chartModel.isXYType()) {
+                var diff = maxValue - minValue;
+                maxValue += diff / 8;
+                minValue -= diff / 8;
             } else if (stacking === 'percentStacked') {
+                ignoreMax = true;
                 format = PERCENTFORMAT;
                 //Trial & error...
                 if (maxValue >= 0 && minValue >= 0) {
-                    //Szenario 1
+                    // scenario 1
                     minValue = givenMin || 0;
                     maxValue = givenMax || 110;
                 } else if (maxValue <= 0 && minValue <= 0) {
-                    //Szenario 2
+                    // scenario 2
                     maxValue = givenMax || 0;
                     minValue = givenMin || -110;
                 } else {
-                    //Szenario 3
+                    // scenario 3
                     maxValue =  givenMax || 110;
                     minValue =  givenMin || -110;
                 }
             } else {
                 if (maxValue >= 0 && minValue >= 0) {
-                    //Szenario 1
+                    // scenario 1
                     minValue = givenMin || makeMin(minValue, maxValue);
                     maxValue = givenMax || makeMax(minValue, maxValue);
                 } else if (maxValue <= 0 && minValue <= 0) {
-                    //Szenario 2
+                    // scenario 2
                     maxValue = givenMax || -makeMin(-maxValue, -minValue);
                     minValue = givenMin || -makeMax(-maxValue, -minValue);
                 } else {
-                    //Szenario 3
+                    // scenario 3
                     var ma = maxValue;
                     var mi = minValue;
                     maxValue =  givenMax || makeMax(mi, ma);
@@ -386,7 +442,7 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
                 }
             }
 
-            if(!isFinite(minValue) || !isFinite(maxValue)){
+            if (!isFinite(minValue) || !isFinite(maxValue)) {
                 setMinMax();
                 return;
             }
@@ -396,43 +452,23 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
                 minValue -= 0.5;
             }
 
-
             var precision = 2;
-            if(format.date){
+            if (format.date) {
                 precision = 0;
             }
 
-            minValue = round(minValue, precision);
-            maxValue = round(maxValue, precision);
-
-
-            if (minValue < maxValue) {
-                var interval = (maxValue - minValue) / intervalCount;
-                interval = round(interval, precision - 1);
-
-                if (interval === 0) {
-                    interval = null;
-                    dataAxis.minimum = minValue;
-                    dataAxis.maximum = maxValue;
-                    dataAxis.interval = null;
-                } else {
-                    var intCount = parseFloat((maxValue - minValue) / interval);
-
-                    if (Math.abs(maxValue) < Math.abs(minValue)) {
-                        maxValue = Utils.roundUp(maxValue, interval);
-                        minValue = maxValue - (interval * intCount);
-                    } else {
-                        minValue = Utils.roundDown(minValue, interval);
-                        maxValue = minValue + (interval * intCount);
-                    }
-                }
-                setMinMax(minValue, maxValue, interval);
+            if (Math.abs(maxValue - minValue) > 0.00000000001) {
+                minValue = round(minValue, precision);
+                maxValue = round(maxValue, precision);
             }
+
+            updateMinMax(minValue, maxValue, precision, 8, ignoreMax);
         }
 
     } // class AxisModel
 
     // exports ================================================================
+
     return AttributedModel.extend({ constructor: AxisModel });
 
 });
