@@ -6,14 +6,14 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/drawing/axistitlemodel',
-    ['io.ox/office/editframework/model/attributedmodel'
-    ], function (AttributedModel) {
+define('io.ox/office/spreadsheet/model/drawing/axistitlemodel', [
+    'io.ox/office/editframework/model/attributedmodel'
+], function (AttributedModel) {
 
     'use strict';
 
@@ -22,12 +22,16 @@ define('io.ox/office/spreadsheet/model/drawing/axistitlemodel',
     /**
      * visualization model for chart axes,
      * it defines colors and textual colors
+     *
+     * @constructor
+     *
+     * @extends AttributedModel
      */
-    function AxisTitleModel(chart) {
+    function AxisTitleModel(chartModel) {
 
         // base constructor ---------------------------------------------------
 
-        AttributedModel.call(this, chart.getApp(), null, { families: 'text character' });
+        AttributedModel.call(this, chartModel.getDocModel(), null, { families: 'text character' });
 
     } // class AxisTitleModel
 
