@@ -6,17 +6,17 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/drawing/dataseriesmodel',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/model/attributedmodel',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/model/formula/tokenarray'
-    ], function (Utils, AttributedModel, SheetUtils, TokenArray) {
+define('io.ox/office/spreadsheet/model/drawing/dataseriesmodel', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/model/attributedmodel',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/model/formula/tokenarray'
+], function (Utils, AttributedModel, SheetUtils, TokenArray) {
 
     'use strict';
 
@@ -35,10 +35,7 @@ define('io.ox/office/spreadsheet/model/drawing/dataseriesmodel',
      *
      * @constructor
      *
-     * @extends DrawingModel
-     *
-     * @param {SpreadsheetApplication} app
-     *  The application containing this chart model.
+     * @extends AttributedModel
      *
      * @param {SheetModel} sheetModel
      *  The sheet model that contains the chart object with this data series.
@@ -46,17 +43,20 @@ define('io.ox/office/spreadsheet/model/drawing/dataseriesmodel',
      * @param {Object} [initAttributes]
      *  Initial formatting attribute set for this data series model.
      */
-    function DataSeriesModel(app, sheetModel, initAttributes) {
+    function DataSeriesModel(sheetModel, initAttributes) {
 
         var // self reference
             self = this,
 
+            // the document model
+            docModel = sheetModel.getDocModel(),
+
             // token arrays for dynamic source ranges, mapped by source link type
             tokenArrays = {};
 
         // base constructor ---------------------------------------------------
 
-        AttributedModel.call(this, app, initAttributes, { families: 'series fill line' });
+        AttributedModel.call(this, docModel, initAttributes, { families: 'series fill line' });
 
         // private methods ----------------------------------------------------
 
@@ -67,52 +67,49 @@ define('io.ox/office/spreadsheet/model/drawing/dataseriesmodel',
         function changeAttributesHandler() {
 
             var // the merged series attributes
-                attributes = self.getMergedAttributes().series;
+                allAttributes = self.getMergedAttributes() || {},
+                attributes = allAttributes.series || {};
 
             _(SOURCE_LINK_TYPES).each(function (type) {
 
                 var sourceLink = attributes[type];
+                var tokenArray = tokenArrays[type];
 
                 // create, update, or delete token array according to type of source data
                 if (_.isString(sourceLink)) {
                     // string: parse link formula
-                    if (!(type in tokenArrays)) {
-                        var tokenArray = new TokenArray(app, sheetModel);
+                    if (!tokenArray) {
+                        tokenArray = new TokenArray(sheetModel);
                         tokenArrays[type] = tokenArray;
                         tokenArray.on('triggered', function () {
-                            self.setAttributes({ series: Utils.makeSimpleObject(type, tokenArray.getFormula()) });
+                            self.setAttributes({ series: Utils.makeSimpleObject(type, tokenArray.getFormula('op')) });
                         });
                     }
-                    tokenArrays[type].invokeSilently('parseFormula', sourceLink);
-                } else if (type in tokenArrays) {
+                    tokenArray.invokeSilently('parseFormula', 'op', sourceLink);
+                } else if (tokenArray) {
                     // else: constant source data, delete token array
-                    tokenArrays[type].destroy();
+                    tokenArray.destroy();
                     delete tokenArrays[type];
                 }
             });
         }
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
 
-        this.getFormula = function () {
-            var formula = '';
-
-            this.iterateTokens(function (token) {
-                if (formula.length) {
-                    formula += ',';
+        this.iterateTokenArrays = function (callback, context) {
+            _.each(tokenArrays, function (tokenArray, name) {
+                if (tokenArray) {
+                    callback.call(context, tokenArray, name);
                 }
-                formula += token.getFormula();
             });
-
-            return formula;
         };
 
-        this.checkRangeInsideInterval = function (sheetModelIndex, interval, insert, columns) {
+        this.transformRanges = function (sheet, interval, insert, columns) {
 
             var changed = false;
 
-            this.iterateTokens(function (tokenArray) {
-                if (tokenArray.transformFormula(sheetModelIndex, interval, insert, columns)) {
+            this.iterateTokenArrays(function (tokenArray) {
+                if (tokenArray.transformRanges(sheet, interval, insert, columns)) {
                     changed = true;
                 }
             });
@@ -120,34 +117,26 @@ define('io.ox/office/spreadsheet/model/drawing/dataseriesmodel',
             return changed;
         };
 
-        this.iterateTokens = function (fn) {
-            _.each(tokenArrays, function (token, name) {
-                if (token) {
-                    fn(token, name);
-                }
-            });
+        this.getRange = function (name) {
+            return tokenArrays[name].resolveRangeList({ resolveNames: true })[0];
         };
 
         /**
          * Returns whether any of the source links contained in this data
          * series overlaps with any of the passed cell ranges.
          *
-         * @param {Array} ranges
-         *  The logical addresses of the cell ranges to be checked. Each cell
-         *  range address object MUST contain an additional property 'sheet'
-         *  with the zero-based sheet index of the range.
+         * @param {Range3DArray|Range3D} ranges
+         *  The addresses of the cell ranges, or a single cell range address,
+         *  to be checked. The cell range addresses MUST be instances of the
+         *  class Range3D with sheet indexes.
          *
          * @returns {Boolean}
          *  Whether any of the passed ranges overlaps with the source links of
          *  this data series.
          */
         this.rangesOverlap = function (ranges) {
-            return _(tokenArrays).any(function (tokenArray) {
-                return _(tokenArray.resolveRangeList()).any(function (linkRange) {
-                    return _(ranges).any(function (range) {
-                        return (linkRange.sheet === range.sheet) && SheetUtils.rangeOverlapsRange(linkRange, range);
-                    });
-                });
+            return _.some(tokenArrays, function (tokenArray) {
+                return tokenArray.resolveRangeList({ resolveNames: true }).overlaps(ranges);
             });
         };
 
@@ -158,8 +147,8 @@ define('io.ox/office/spreadsheet/model/drawing/dataseriesmodel',
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            _(tokenArrays).invoke('destroy');
-            tokenArrays = self = null;
+            _.invoke(tokenArrays, 'destroy');
+            self = docModel = sheetModel = tokenArrays = null;
         });
 
     } // class DataSeriesModel
