@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -17,14 +17,18 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
     'io.ox/office/drawinglayer/model/shapemodel',
     'io.ox/office/drawinglayer/model/imagemodel',
     'io.ox/office/drawinglayer/model/drawingcollection',
+    'io.ox/office/spreadsheet/utils/operations',
     'io.ox/office/spreadsheet/utils/sheetutils',
     'io.ox/office/spreadsheet/model/drawing/drawingmodelmixin',
     'io.ox/office/spreadsheet/model/drawing/chartmodel'
-], function (Utils, DrawingModel, ShapeModel, ImageModel, DrawingCollection, SheetUtils, DrawingModelMixin, SheetChartModel) {
+], function (Utils, DrawingModel, ShapeModel, ImageModel, DrawingCollection, Operations, SheetUtils, DrawingModelMixin, SheetChartModel) {
 
     'use strict';
 
-    var // horizontal drawing anchor attribute names
+    var // convenience shortcuts
+        Interval = SheetUtils.Interval,
+
+        // horizontal drawing anchor attribute names
         HORIZONTAL_ATTRIBUTE_NAMES = {
             offset: 'left',
             size: 'width',
@@ -72,9 +76,6 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
      * @extends DrawingModel
      * @extends DrawingModelMixin
      *
-     * @param {EditApplication} app
-     *  The application instance containing this drawing object.
-     *
      * @param {SheetModel} sheetModel
      *  The sheet model instance containing this drawing object.
      *
@@ -84,20 +85,20 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
      * @param {Object} [initAttributes]
      *  An attribute set with initial formatting attributes for the object.
      */
-    var SheetDrawingModel = DrawingModel.extend({ constructor: function (app, sheetModel, type, initAttributes) {
+    var SheetDrawingModel = DrawingModel.extend({ constructor: function (sheetModel, type, initAttributes) {
 
         // base constructors --------------------------------------------------
 
-        DrawingModel.call(this, app, type, initAttributes);
-        DrawingModelMixin.call(this, app, sheetModel, function cloneConstructor(targetModel) {
-            return new SheetDrawingModel(app, targetModel, this.getType(), this.getExplicitAttributes(true));
+        DrawingModel.call(this, sheetModel.getDocModel(), type, initAttributes);
+        DrawingModelMixin.call(this, sheetModel, function cloneConstructor(targetModel) {
+            return new SheetDrawingModel(targetModel, this.getType(), this.getExplicitAttributes(true));
         });
 
         // initialization -----------------------------------------------------
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = sheetModel = initAttributes = null;
+            sheetModel = initAttributes = null;
         });
 
     }}); // class SheetImageModel
@@ -112,29 +113,26 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
      * @extends ShapeModel
      * @extends DrawingModelMixin
      *
-     * @param {EditApplication} app
-     *  The application instance containing this shape object.
-     *
      * @param {SheetModel} sheetModel
      *  The sheet model instance containing this shape object.
      *
      * @param {Object} [initAttributes]
      *  An attribute set with initial formatting attributes for the object.
      */
-    var SheetShapeModel = ImageModel.extend({ constructor: function (app, sheetModel, initAttributes) {
+    var SheetShapeModel = ImageModel.extend({ constructor: function (sheetModel, initAttributes) {
 
         // base constructors --------------------------------------------------
 
-        ShapeModel.call(this, app, initAttributes);
-        DrawingModelMixin.call(this, app, sheetModel, function cloneConstructor(targetModel) {
-            return new SheetShapeModel(app, targetModel, this.getExplicitAttributes(true));
+        ShapeModel.call(this, sheetModel.getDocModel(), initAttributes);
+        DrawingModelMixin.call(this, sheetModel, function cloneConstructor(targetModel) {
+            return new SheetShapeModel(targetModel, this.getExplicitAttributes(true));
         });
 
         // initialization -----------------------------------------------------
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = sheetModel = initAttributes = null;
+            sheetModel = initAttributes = null;
         });
 
     }}); // class SheetShapeModel
@@ -149,29 +147,26 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
      * @extends ImageModel
      * @extends DrawingModelMixin
      *
-     * @param {EditApplication} app
-     *  The application instance containing this image object.
-     *
      * @param {SheetModel} sheetModel
      *  The sheet model instance containing this image object.
      *
      * @param {Object} [initAttributes]
      *  An attribute set with initial formatting attributes for the object.
      */
-    var SheetImageModel = ImageModel.extend({ constructor: function (app, sheetModel, initAttributes) {
+    var SheetImageModel = ImageModel.extend({ constructor: function (sheetModel, initAttributes) {
 
         // base constructors --------------------------------------------------
 
-        ImageModel.call(this, app, initAttributes);
-        DrawingModelMixin.call(this, app, sheetModel, function cloneConstructor(targetModel) {
-            return new SheetImageModel(app, targetModel, this.getExplicitAttributes(true));
+        ImageModel.call(this, sheetModel.getDocModel(), initAttributes);
+        DrawingModelMixin.call(this, sheetModel, function cloneConstructor(targetModel) {
+            return new SheetImageModel(targetModel, this.getExplicitAttributes(true));
         });
 
         // initialization -----------------------------------------------------
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = sheetModel = initAttributes = null;
+            sheetModel = initAttributes = null;
         });
 
     }}); // class SheetImageModel
@@ -186,30 +181,31 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
      *
      * @extends DrawingCollection
      *
-     * @param {SpreadsheetApplication} app
-     *  The application instance with the document model containing this sheet.
-     *
      * @param {SheetModel} sheetModel
      *  The sheet model instance containing this collection.
      */
-    function SheetDrawingCollection(app, sheetModel) {
+    function SheetDrawingCollection(sheetModel) {
 
         var // self reference
             self = this,
 
             // the document model
-            docModel = app.getModel(),
+            docModel = sheetModel.getDocModel(),
 
             // the column and row collections (size and formatting of all columns/rows)
             colCollection = sheetModel.getColCollection(),
             rowCollection = sheetModel.getRowCollection(),
 
             // maps drawing model indexes (used as positions) to drawing models
-            drawingModels = [];
+            drawingModels = [],
+
+            // the anchor mode to be used when updating anchor after attribute changes (will be
+            // set to true while updating drawing anchors after inserting/deleting columns/rows)
+            updateDisplayAnchor = false;
 
         // base constructor ---------------------------------------------------
 
-        DrawingCollection.call(this, app, {
+        DrawingCollection.call(this, docModel, {
             clientCreateModelHandler: createDrawingModel,
             clientResolveModelHandler: resolveDrawingModel,
             clientResolvePositionHandler: resolveDrawingPosition,
@@ -235,17 +231,17 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
 
             switch (type) {
             case 'shape':
-                return new SheetShapeModel(app, sheetModel, attributes);
+                return new SheetShapeModel(sheetModel, attributes);
 
             case 'image':
-                return new SheetImageModel(app, sheetModel, attributes);
+                return new SheetImageModel(sheetModel, attributes);
 
             case 'chart':
-                return new SheetChartModel(app, sheetModel, attributes);
+                return new SheetChartModel(sheetModel, attributes);
 
             default:
                 if (/^(group|diagram|ole|undefined)$/.test(type)) {
-                    return new SheetDrawingModel(app, sheetModel, type, attributes);
+                    return new SheetDrawingModel(sheetModel, type, attributes);
                 }
             }
 
@@ -256,7 +252,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
          * Tries to return a drawing model contained in this sheet at the
          * specified position.
          *
-         * @param {Number[]} position
+         * @param {Array<Number>} position
          *  An arbitrary document position.
          *
          * @returns {DrawingModel|Null}
@@ -273,7 +269,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
          * @param {DrawingModel} drawingModel
          *  An arbitrary drawing model.
          *
-         * @param {Number[]|Null}
+         * @param {Array<Number>|Null}
          *  The position of the passed drawing model, if the model is part of
          *  this drawing collection, otherwise null.
          */
@@ -289,7 +285,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
          * @param {DrawingModel} drawingModel
          *  The new drawing model.
          *
-         * @param {Number[]} position
+         * @param {Array<Number>} position
          *  The position of the new drawing object.
          *
          * @returns {Boolean}
@@ -297,7 +293,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
          *  been registered successfully.
          */
         function registerDrawingModel(drawingModel, position) {
-            if ((position.length === 1) && (0 <= position[0]) && (position[0] <= drawingModels.length)) {
+            if ((position.length === 1) && (position[0] >= 0) && (position[0] <= drawingModels.length)) {
                 drawingModels.splice(position[0], 0, drawingModel);
                 return true;
             }
@@ -453,8 +449,8 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
             }
 
             // whether the drawing is located entirely in hidden columns or rows
-            position.hiddenCols = colCollection.isIntervalHidden({ first: position.startCol, last: position.endCol });
-            position.hiddenRows = rowCollection.isIntervalHidden({ first: position.startRow, last: position.endRow });
+            position.hiddenCols = colCollection.isIntervalHidden(new Interval(position.startCol, position.endCol));
+            position.hiddenRows = rowCollection.isIntervalHidden(new Interval(position.startRow, position.endRow));
 
             return position;
         }
@@ -575,7 +571,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
          * display anchor of the drawing model.
          */
         function setDrawingAttributesHandler(drawingModel) {
-            updateInactiveAnchorAttributes(drawingModel);
+            updateInactiveAnchorAttributes(drawingModel, { displayAnchor: updateDisplayAnchor });
         }
 
         /**
@@ -589,12 +585,19 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
                 // the maximum column/row
                 maxIndex = docModel.getMaxIndex(columns),
                 // the number of columns/rows to be inserted
-                move = SheetUtils.getIntervalSize(interval);
+                move = interval.size();
 
-            _.each(drawingModels, function (drawingModel) {
+            // use display anchor mode during runtime (e.g. the anchor mode 'twoCellAsOneCell'
+            // is expected to work as 'oneCell' when inserting columns or rows)
+            updateDisplayAnchor = true;
+
+            // process all drawing objects: simply calculate new column/row indexes; if
+            // the drawing object has one-cell or absolute anchor mode, these attributes
+            // will be overwritten when recalculating the inactive anchor attributes
+            drawingModels.forEach(function (drawingModel) {
 
                 var // the column/row interval covered by the drawing model
-                    drawingInterval = SheetUtils.getInterval(drawingModel.getRange(), columns),
+                    drawingInterval = drawingModel.getRange().interval(columns),
                     // the maximum number of columns/rows the drawing can be moved
                     maxMove = Math.min(move, maxIndex - drawingInterval.last),
                     // the new anchor attributes
@@ -615,6 +618,9 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
                     drawingModel.setAttributes({ drawing: attributes });
                 }
             });
+
+            // back to default anchor mode
+            updateDisplayAnchor = false;
         }
 
         /**
@@ -626,23 +632,31 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
             var // the attribute names according to the passed direction
                 ATTR_NAMES = columns ? HORIZONTAL_ATTRIBUTE_NAMES : VERTICAL_ATTRIBUTE_NAMES,
                 // the number of columns/rows to be deleted
-                move = SheetUtils.getIntervalSize(interval);
+                move = interval.size();
 
+            // use display anchor mode during runtime (e.g. the anchor mode 'twoCellAsOneCell'
+            // is expected to work as 'oneCell' when deleting columns or rows)
+            updateDisplayAnchor = true;
+
+            // process all drawing objects: simply calculate new column/row indexes; if
+            // the drawing object has one-cell or absolute anchor mode, these attributes
+            // will be overwritten when recalculating the inactive anchor attributes;
             // iterate in reverse order to be able to delete drawings from the collection
             Utils.iterateArray(drawingModels, function (drawingModel, position) {
 
                 var // the current anchor attributes
                     drawingAttributes = drawingModel.getMergedAttributes().drawing,
+                    // the anchor type of the drawing object that influences how to calculate new position and size
+                    anchorType = DISPLAY_ANCHOR_MODES[drawingAttributes.anchorType],
                     // the column/row interval covered by the drawing model
-                    drawingInterval = { first: drawingAttributes[ATTR_NAMES.startIndex], last: drawingAttributes[ATTR_NAMES.endIndex] },
+                    drawingInterval = new Interval(drawingAttributes[ATTR_NAMES.startIndex], drawingAttributes[ATTR_NAMES.endIndex]),
                     // the new anchor attributes
                     anchorAttributes = null;
 
                 // Delete drawing object with two-cell display anchor mode, if
                 // it is contained in the deleted column/row interval, or if it
                 // ends at offset zero in the first column/row after the interval.
-                if ((DISPLAY_ANCHOR_MODES[drawingAttributes.anchorType] === 'twoCell') &&
-                    (interval.first <= drawingInterval.first) &&
+                if ((anchorType === 'twoCell') && (interval.first <= drawingInterval.first) &&
                     ((drawingInterval.last <= interval.last) || ((drawingInterval.last + 1 === interval.last) && (drawingAttributes[ATTR_NAMES.endOffset] === 0)))
                 ) {
                     self.deleteModel([position]);
@@ -677,6 +691,9 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
                 drawingModel.setAttributes({ drawing: anchorAttributes });
 
             }, { reverse: true });
+
+            // back to default anchor mode
+            updateDisplayAnchor = false;
         }
 
         /**
@@ -685,10 +702,10 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
          */
         function changeInterval(interval, columns) {
 
-            _.each(drawingModels, function (drawingModel) {
+            drawingModels.forEach(function (drawingModel) {
 
                 var // the column/row interval covered by the drawing model
-                    drawingInterval = SheetUtils.getInterval(drawingModel.getRange(), columns);
+                    drawingInterval = drawingModel.getRange().interval(columns);
 
                 if (interval.first <= drawingInterval.last) {
                     updateInactiveAnchorAttributes(drawingModel, { displayAnchor: true, notify: true });
@@ -699,77 +716,116 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
         // protected methods --------------------------------------------------
 
         /**
-         * Creates and returns a cloned instance of this drawing collection for
-         * the specified sheet.
+         * Clones all contents from the passed collection into this collection.
          *
          * @internal
          *  Used by the class SheetModel during clone construction. DO NOT CALL
          *  from external code!
          *
-         * @param {SheetModel} targetModel
-         *  The model instance of the new cloned sheet that will own the clone
-         *  returned by this method.
+         * @param {SheetDrawingCollection} collection
+         *  The source collection whose contents will be cloned into this
+         *  collection.
          *
          * @returns {SheetDrawingCollection}
-         *  A clone of this drawing collection, initialized for ownership by
-         *  the passed sheet model.
+         *  A reference to this instance.
          */
-        this.clone = function (targetModel) {
-            // construct a new collection, pass own drawing models as hidden parameter
-            return new SheetDrawingCollection(app, targetModel, drawingModels);
+        this.cloneFrom = function (collection) {
+
+            // clone the contents of the source collection
+            collection.iterateModelsByPosition(function (drawingModel, position) {
+                self.insertModelInstance(position, drawingModel.clone(sheetModel));
+            });
+
+            return this;
+        };
+
+        /**
+         * Callback handler for the document operation 'insertDrawing'.
+         *
+         * @param {SheetOperationContext} context
+         *  A wrapper representing the 'insertDrawing' document operation.
+         *
+         * @param {Array<Number>} position
+         *  The insertion position of the new drawing object in this collection
+         *  (without leading sheet index).
+         */
+        this.applyInsertDrawingOperation = function (context, position) {
+            var result = this.insertModel(position, context.getStr('type'), context.getOptObj('attrs'));
+            context.ensure(result, 'cannot insert drawing');
+        };
+
+        /**
+         * Callback handler for the document operation 'deleteDrawing'.
+         *
+         * @param {SheetOperationContext} context
+         *  A wrapper representing the 'deleteDrawing' document operation.
+         *
+         * @param {Array<Number>} position
+         *  The position of the drawing object to be deleted in this collection
+         *  (without leading sheet index).
+         */
+        this.applyDeleteDrawingOperation = function (context, position) {
+            var result = this.deleteModel(position);
+            context.ensure(result, 'cannot delete drawing');
         };
 
         // public methods -----------------------------------------------------
 
         /**
-         * Invokes the passed iterator function for all drawing models in this
+         * Invokes the passed callback function for all drawing models in this
          * sheet. The drawing models will be visited in order of their document
          * positions (Z order).
          *
-         * @param {Function} iterator
-         *  The iterator function called for all drawing models. Receives the
+         * @param {Function} callback
+         *  The callback function called for all drawing models. Receives the
          *  current drawing model as first parameter, and its document position
          *  (single-element array with the index) as second parameter. If the
-         *  iterator returns the Utils.BREAK object, the iteration process will
-         *  be stopped immediately.
+         *  callback function returns the Utils.BREAK object, the iteration
+         *  process will be stopped immediately.
          *
          * @param {Object} [options]
          *  Optional parameters:
          *  @param {Object} [options.context]
-         *      If specified, the iterator will be called with this context
-         *      (the symbol 'this' will be bound to the context inside the
-         *      iterator function).
+         *      If specified, the callback function will be called with this
+         *      context (the symbol 'this' will be bound to the context inside
+         *      the iterator function).
          *  @param {Boolean} [options.reverse=false]
          *      If set to true, the drawing models will be visited in reversed
          *      order.
          *
          * @returns {Utils.BREAK|Undefined}
-         *  A reference to the Utils.BREAK object, if the iterator has returned
+         *  A reference to the Utils.BREAK object, if the callback has returned
          *  Utils.BREAK to stop the iteration process, otherwise undefined.
          */
-        this.iterateModelsByPosition = function (iterator, options) {
+        this.iterateModelsByPosition = function (callback, options) {
             return Utils.iterateArray(drawingModels, function (drawingModel, index) {
-                return iterator.call(this, drawingModel, [index]);
+                return callback.call(this, drawingModel, [index]);
             }, options);
         };
 
         /**
-         * Returns the sheet rectangle in pixels covered by a drawing object
-         * with the passed formatting attributes.
+         * Returns the sheet rectangle in pixels covered by the passed drawing
+         * object.
          *
-         * @param {Object} attributes
-         *  The merged attribute set of a drawing object.
+         * @param {DrawingModel} drawingModel
+         *  The model of the drawing object.
          *
          * @returns {Object|Null}
-         *  The position of the drawing object in the sheet in pixels,
+         *  The position of the passed drawing object in the sheet in pixels,
          *  according to the current sheet zoom factor, in the properties
          *  'left', 'top', 'width', and 'height'; or null if the drawing object
-         *  is located entirely in hidden columns or rows and thus invisible.
+         *  is invisible by itself, or is located entirely in hidden columns or
+         *  rows and thus invisible.
          */
-        this.getRectangleForAttributes = function (attributes) {
+        this.getModelRectangle = function (drawingModel) {
 
-            var // the complete position information object
-                positionInfo = getAnchorInformation(attributes.drawing, INTERNAL_ANCHOR_MODES[attributes.drawing.anchorType]);
+            // always return null for invisible drawing objects
+            if (!drawingModel.isVisible()) { return null; }
+
+            var // the anchor attributes of the drawing model
+                anchorAttrs = drawingModel.getMergedAttributes().drawing,
+                // the complete position information object
+                positionInfo = getAnchorInformation(anchorAttrs, INTERNAL_ANCHOR_MODES[anchorAttrs.anchorType]);
 
             // return null for hidden drawing objects
             if (positionInfo.hiddenCols || positionInfo.hiddenRows) {
@@ -791,19 +847,21 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
         };
 
         /**
-         * Returns the drawing attributes needed to set the position of a
-         * drawing object to the passed sheet rectangle.
+         * Returns a drawing attribute set containing the anchor attributes
+         * needed to set the position of a drawing object to the passed sheet
+         * rectangle.
          *
          * @param {Object} rectangle
-         *  The new position for a drawing object in the sheet in pixels,
-         *  according to the current sheet zoom factor, in the properties
-         *  'left', 'top', 'width', and 'height'.
+         *  The position of a drawing object in the sheet in pixels, according
+         *  to the current sheet zoom factor, in the properties 'left', 'top',
+         *  'width', and 'height'.
          *
          * @returns {Object}
-         *  The attribute set containing the anchor attributes for the new
-         *  position of the drawing object.
+         *  A drawing attribute set with the anchor attributes (contained in
+         *  the sub-object 'drawing' of the resulting attribute set) for the
+         *  new position of the drawing object.
          */
-        this.getAttributesForRectangle = function (rectangle) {
+        this.getAttributeSetForRectangle = function (rectangle) {
 
             var // collection entry of the column/row collection
                 entryData = null,
@@ -838,27 +896,39 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
         };
 
         /**
-         * Adds missing generic anchor attributes ('left', 'top', 'width', and
-         * 'height') to the passed (incomplete) attribute set of a drawing
-         * object.
+         * Generates the missing generic anchor attributes ('left', 'top',
+         * 'width', and 'height') for the passed (incomplete) attribute set of
+         * a drawing object.
          *
-         * @param {Object} attributes
-         *  A drawing attribute set. Must contain a property 'drawing' with
-         *  some anchor properties.
+         * @param {Object} attributeSet
+         *  A drawing attribute set. May contain an object property 'drawing'
+         *  with some anchor properties.
+         *
+         * @returns {Object}
+         *  A copy of the passed attribute set that has been extended with the
+         *  missing generic anchor attributes.
          */
-        this.addGenericAnchorAttributes = function (attributes) {
+        this.addGenericAnchorAttributes = function (attributeSet) {
 
             var // the merged attribute set (with default values for all missing attributes)
-                mergedAttributes = docModel.getStyleCollection('drawing').getMergedAttributes(attributes),
+                mergedAttributes = docModel.getStyleCollection('drawing').getMergedAttributes(attributeSet),
                 // the inactive anchor attributes
                 anchorAttributes = getInactiveAnchorAttributes(mergedAttributes.drawing);
 
+            // create a deep copy of the passed attributes
+            attributeSet = _.copy(attributeSet, true);
+
+            // the drawing attributes in the attribute set (create on demand)
+            var drawingAttrrs = attributeSet.drawing || (attributeSet.drawing = {});
+
             // add the generic anchor attributes to the passed attribute set
-            _.each(['left', 'top', 'width', 'height'], function (attrName) {
+            ['left', 'top', 'width', 'height'].forEach(function (attrName) {
                 if (attrName in anchorAttributes) {
-                    (attributes.drawing || (attributes.drawing = {}))[attrName] = anchorAttributes[attrName];
+                    drawingAttrrs[attrName] = anchorAttributes[attrName];
                 }
             });
+
+            return attributeSet;
         };
 
         // initialization -----------------------------------------------------
@@ -883,17 +953,10 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
             'change:entries': function (event, interval) { changeInterval(interval, false); }
         });
 
-        // clone drawing models passed as hidden argument by the clone() method
-        if (_.isArray(arguments[SheetDrawingCollection.length])) {
-            _.each(arguments[SheetDrawingCollection.length], function (drawingModel, index) {
-                self.insertModelInstance([index], drawingModel.clone(sheetModel));
-            });
-        }
-
         // destroy all class members
         this.registerDestructor(function () {
-            // drawing models are owned by the base class DrawingCollection
-            docModel = colCollection = rowCollection = drawingModels = null;
+            // drawing models are owned by the base class DrawingCollection, do not destroy them here!
+            self = docModel = sheetModel = colCollection = rowCollection = drawingModels = null;
         });
 
     } // class SheetDrawingCollection
