@@ -6,15 +6,15 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/drawing/gridlinemodel',
-    ['io.ox/office/editframework/model/attributedmodel',
-     'io.ox/office/drawinglayer/view/chartstyleutil'
-    ], function (AttributedModel, ChartStyleUtil) {
+define('io.ox/office/spreadsheet/model/drawing/gridlinemodel', [
+    'io.ox/office/editframework/model/attributedmodel',
+    'io.ox/office/drawinglayer/view/chartstyleutil'
+], function (AttributedModel, ChartStyleUtil) {
 
     'use strict';
 
@@ -23,12 +23,16 @@ define('io.ox/office/spreadsheet/model/drawing/gridlinemodel',
     /**
      * visualization model for chart axes,
      * it defines colors and textual colors
+     *
+     * @constructor
+     *
+     * @extends AttributedModel
      */
-    function GridlineModel(chart, attrs, dataAxis) {
+    function GridlineModel(chartModel, attrs, dataAxis) {
 
         // base constructor ---------------------------------------------------
 
-        AttributedModel.call(this, chart.getApp(), attrs, { families: 'line' });
+        AttributedModel.call(this, chartModel.getDocModel(), attrs, { families: 'line' });
 
         // methods ------------------------------------------------------------
 
@@ -38,13 +42,13 @@ define('io.ox/office/spreadsheet/model/drawing/gridlinemodel',
                 return;
             }
 
-            ChartStyleUtil.handleLineProps(chart, att.line, dataAxis, 'grid');
+            ChartStyleUtil.handleLineProps(chartModel, att.line, dataAxis, 'grid');
         };
 
-
     } // class GridlineModel
 
     // exports ================================================================
+
     return AttributedModel.extend({ constructor: GridlineModel });
 
 });
