@@ -6,25 +6,33 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/drawing/legendmodel',
-    ['io.ox/office/drawinglayer/view/chartstyleutil',
-     'io.ox/office/editframework/model/attributedmodel'
-    ], function (ChartStyleUtil, AttributedModel) {
+define('io.ox/office/spreadsheet/model/drawing/legendmodel', [
+    'io.ox/office/drawinglayer/view/chartstyleutil',
+    'io.ox/office/editframework/model/attributedmodel'
+], function (ChartStyleUtil, AttributedModel) {
 
     'use strict';
 
     // class LegendModel ======================================================
 
-    function LegendModel(chart, attrs, dataLegend) {
+    /**
+     *
+     * @constructor
+     *
+     * @extends AttributedModel
+     */
+    function LegendModel(chartModel, attrs, dataLegend) {
+
         var self = this;
+
         // base constructor ---------------------------------------------------
 
-        AttributedModel.call(this, chart.getApp(), attrs, { families: 'legend character' });
+        AttributedModel.call(this, chartModel.getDocModel(), attrs, { families: 'legend character' });
 
         // methods ------------------------------------------------------------
 
@@ -49,17 +57,17 @@ define('io.ox/office/spreadsheet/model/drawing/legendmodel',
             }
 
             if (legendPos !== 'off') {
-                ChartStyleUtil.handleCharacterProps(chart, att.character, dataLegend);
+                ChartStyleUtil.handleCharacterProps(chartModel, att.character, dataLegend);
                 if (dataLegend.forceSmall) {
-                    dataLegend.fontSize = Math.round(dataLegend.fontSize/4*3);
+                    dataLegend.fontSize = Math.round(dataLegend.fontSize / 4 * 3);
                 }
             }
         };
 
-
     } // class LegendModel
 
     // exports ================================================================
+
     return AttributedModel.extend({ constructor: LegendModel });
 
 });
