@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,21 +14,57 @@
 define('io.ox/office/spreadsheet/model/formula/compiler', [
     'io.ox/office/tk/utils',
     'io.ox/office/baseframework/model/modelobject',
-    'io.ox/office/spreadsheet/model/formula/tokenutils',
-    'io.ox/office/spreadsheet/model/formula/matrix'
-], function (Utils, ModelObject, TokenUtils, Matrix) {
+    'io.ox/office/spreadsheet/model/formula/formulautils',
+    'io.ox/office/spreadsheet/model/formula/matrix',
+    'io.ox/office/spreadsheet/model/formula/tokens'
+], function (Utils, ModelObject, FormulaUtils, Matrix, Tokens) {
 
     'use strict';
 
+    var // maps operator precedence categories to native operator names
+        OPERATOR_PRECEDENCE_CATEGORIES = {
+            cmp: Utils.makeSet(['eq', 'ne', 'lt', 'le', 'gt', 'ge']),
+            con: Utils.makeSet('con'),
+            pct: Utils.makeSet('pct'),
+            add: Utils.makeSet(['add', 'sub']),
+            mul: Utils.makeSet(['mul', 'div']),
+            pow: Utils.makeSet('pow'),
+            list: Utils.makeSet('list'),
+            isect: Utils.makeSet('isect'),
+            range: Utils.makeSet('range')
+        };
+
+    // private global functions ===============================================
+
+    /**
+     * Returns whether the passed formula token is an operator of the specified
+     * precedence category.
+     *
+     * @param {BaseToken|Null} token
+     *  The parser token to be checked. May be null, in this case this function
+     *  will return false.
+     *
+     * @param {String} category
+     *  An operator precedence category. Must be an existing key in the map
+     *  OPERATOR_PRECEDENCE_CATEGORIES.
+     *
+     * @returns {Boolean}
+     *  Whether the passed value is an existing operator token of the specified
+     *  precedence category.
+     */
+    function isOperatorToken(token, category) {
+        return _.isObject(token) && token.isType('op') && (token.getValue() in OPERATOR_PRECEDENCE_CATEGORIES[category]);
+    }
+
     // class OperatorToken ====================================================
 
     /**
      * Represents an operator or function call in the compiled token array.
      *
      * @property {String} type
-     *  The string 'op'.
+     *  The constant string 'op'.
      *
-     * @property {Token} token
+     * @property {BaseToken} token
      *  A reference to the original parser token describing the operator or
      *  function call.
      *
@@ -36,13 +72,17 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
      *  The number of operands associated to the operator.
      */
     function OperatorToken(token, params) {
+
         this.type = 'op';
         this.token = token;
         this.params = params;
-    }
+
+    } // class OperatorToken
+
+    // public methods ---------------------------------------------------------
 
     OperatorToken.prototype.toString = function () {
-        return 'op[' + this.token.getText()  + ']' + ':' + this.params;
+        return this.token + ':' + this.params;
     };
 
     // class OperandToken =====================================================
@@ -52,38 +92,51 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
      * token array.
      *
      * @property {String} type
-     *  The string 'val'.
+     *  The constant string 'val'.
      *
-     * @property {Token} token
+     * @property {BaseToken} token
      *  A reference to the original parser token describing the operand.
      */
     function OperandToken(token) {
+
         this.type = 'val';
         this.token = token;
-    }
+
+    } // class OperandToken
+
+    // public methods ---------------------------------------------------------
 
     OperandToken.prototype.toString = function () {
-        return 'val[' + this.token.getType() + ':' + this.token.getText() + ']';
+        return this.token.toString();
     };
 
     // class Compiler =========================================================
 
     /**
-     * Compiles arrays of formula tokens from infix to prefix notation (a.k.a.
-     * Polish Notation).
+     * Compiles arrays of formula tokens to a specific notation. This is needed
+     * as preparation for interpretation of the formula in order to get its
+     * current result. Unary and binary operators will be converted from infix
+     * or postfix notation to prefix notation (a.k.a. Polish Notation). All
+     * tokens that are not necessary for interpretation will be removed, e.g.
+     * whitespace and parentheses.
+     *
+     * Example: The formula =2+3*(4+5) has been tokenized to the token array
+     *  2, PLUS, 3, TIMES, OPEN, 4, PLUS, 5, CLOSE.
+     * The compiler will convert this token array to the prefix notation
+     *  PLUS, 2, TIMES, 3, PLUS, 4, 5.
      *
      * @constructor
      *
      * @extends ModelObject
      *
-     * @param {SpreadsheetApplication} app
-     *  The spreadsheet application instance.
+     * @param {SpreadsheetModel} docModel
+     *  The spreadsheet document model containing this instance.
      */
-    function Compiler(app) {
+    function Compiler(docModel) {
 
         // base constructor ---------------------------------------------------
 
-        ModelObject.call(this, app);
+        ModelObject.call(this, docModel);
 
         // public methods -----------------------------------------------------
 
@@ -93,18 +146,13 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
          * white-space tokens, parentheses and parameter separators of function
          * calls, and all other regular parentheses.
          *
-         * @param {Array} tokens
-         *  The array of parser tokens, in infix notation, as received from the
-         *  method Tokenizer.parseFormula().
-         *
-         * @param {Tokenizer} tokenizer
-         *  The tokenizer instance used as token factory for additional tokens
-         *  created during compilation.
+         * @param {Array<BaseToken>} tokens
+         *  The array of formula tokens, in infix notation.
          *
          * @returns {Object}
          *  The result descriptor of the compilation process, with the
          *  following properties:
-         *  - {Array|Null} tokens
+         *  - {Array<OperatorToken|OperandToken>|Null} tokens
          *      An array of specific token descriptors in prefix notation:
          *      instances of class OperatorToken for operators, and instances
          *      of class OperandToken for operands; or null, if compilation has
@@ -116,7 +164,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
          *          unexpectedly (e.g. incomplete formula).
          *      - 'unexpected': A token with an invalid type has been found.
          */
-        this.compileTokens = function (tokens, tokenizer) {
+        this.compileTokens = function (tokens) {
 
             var // the current array index
                 index = 0,
@@ -144,7 +192,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
             /**
              * Returns the next token, and increases the array index.
              *
-             * @returns {Token|Null}
+             * @returns {BaseToken|Null}
              *  The next available token; or null, if the end of the array has
              *  been reached.
              */
@@ -162,7 +210,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
              *  If set to true, a following token must exist in the passed
              *  token array, otherwise an exception will be thrown (see below).
              *
-             * @returns {Token|Null}
+             * @returns {BaseToken|Null}
              *  The next available token; or null, if the end of the array has
              *  been reached (and the parameter 'required' is not set).
              *
@@ -185,7 +233,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
              * @param {Function} compilerFunc
              *  The callback function invoked with the next available token.
              *
-             * @param {Token} opToken
+             * @param {BaseToken} opToken
              *  The operator token to be inserted into the token array.
              *
              * @param {Number} insert
@@ -195,7 +243,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
              * @param {Number} params
              *  The number of parameters expected by the operator token.
              *
-             * @returns {Token|Null}
+             * @returns {BaseToken|Null}
              *  The token returned by the passed callback function.
              */
             function compileTermAndPush(compilerFunc, opToken, insert, params) {
@@ -204,13 +252,6 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
                 return token;
             }
 
-            /**
-             * Returns whether the token is an operator of the specified type.
-             */
-            function isOperatorToken(token, opType) {
-                return _.isObject(token) && token.isType('op') && (token.getOpType() === opType);
-            }
-
             /**
              * Compiles a subexpression starting at the passed token, until the
              * end of the formula, or the specified terminator token.
@@ -229,7 +270,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
             function compileCompareTerm(token) {
                 var index = compiled.length;
                 token = compileConcatTerm(token);
-                while (isOperatorToken(token, '=')) {
+                while (isOperatorToken(token, 'cmp')) {
                     token = compileTermAndPush(compileConcatTerm, token, index, 2);
                 }
                 return token;
@@ -241,7 +282,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
             function compileConcatTerm(token) {
                 var index = compiled.length;
                 token = compileAddSubTerm(token);
-                while (isOperatorToken(token, '&')) {
+                while (isOperatorToken(token, 'con')) {
                     token = compileTermAndPush(compileAddSubTerm, token, index, 2);
                 }
                 return token;
@@ -253,7 +294,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
             function compileAddSubTerm(token) {
                 var index = compiled.length;
                 token = compileMulDivTerm(token);
-                while (isOperatorToken(token, '+')) {
+                while (isOperatorToken(token, 'add')) {
                     token = compileTermAndPush(compileMulDivTerm, token, index, 2);
                 }
                 return token;
@@ -265,7 +306,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
             function compileMulDivTerm(token) {
                 var index = compiled.length;
                 token = compilePowTerm(token);
-                while (isOperatorToken(token, '*')) {
+                while (isOperatorToken(token, 'mul')) {
                     token = compileTermAndPush(compilePowTerm, token, index, 2);
                 }
                 return token;
@@ -277,7 +318,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
             function compilePowTerm(token) {
                 var index = compiled.length;
                 token = compileUnaryTerm(token);
-                while (isOperatorToken(token, '^')) {
+                while (isOperatorToken(token, 'pow')) {
                     token = compileTermAndPush(compileUnaryTerm, token, index, 2);
                 }
                 return token;
@@ -288,12 +329,12 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
              */
             function compileUnaryTerm(token) {
                 var index = compiled.length;
-                if (isOperatorToken(token, '+')) {
+                if (isOperatorToken(token, 'add')) {
                     token = compileTermAndPush(compileUnaryTerm, token, compiled.length, 1);
                 } else {
                     token = compileListTerm(token);
                 }
-                while (isOperatorToken(token, '%')) {
+                while (isOperatorToken(token, 'pct')) {
                     pushOperator(token, index, 1);
                     token = getNextToken();
                 }
@@ -306,8 +347,9 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
             function compileListTerm(token) {
                 var index = compiled.length;
                 token = compileIntersectTerm(token, index);
-                while (token && token.isType('sep') && !stopAtSep) {
-                    token = compileTermAndPush(compileIntersectTerm, tokenizer.createOperatorToken(',', ','), index, 2);
+                while (token && (isOperatorToken('list') || (token.isType('sep') && !stopAtSep))) {
+                    if (token.isType('sep')) { token = new Tokens.OperatorToken('list'); }
+                    token = compileTermAndPush(compileIntersectTerm, token, index, 2);
                 }
                 return token;
             }
@@ -318,7 +360,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
             function compileIntersectTerm(token) {
                 var index = compiled.length;
                 token = compileRangeTerm(token);
-                while (isOperatorToken(token, '!')) {
+                while (isOperatorToken(token, 'isect')) {
                     token = compileTermAndPush(compileRangeTerm, token, index, 2);
                 }
                 return token;
@@ -330,7 +372,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
             function compileRangeTerm(token) {
                 var index = compiled.length;
                 token = compileOperand(token);
-                while (isOperatorToken(token, ':')) {
+                while (isOperatorToken(token, 'range')) {
                     token = compileTermAndPush(compileOperand, token, index, 2);
                 }
                 return token;
@@ -349,7 +391,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
                 }
 
                 // bug 33526: reference and name tokens must contain valid sheet references
-                if (token.isType(/^(ref|name)$/) && !token.hasSheetError()) {
+                if (token.matchesType(/^(ref|name)$/) && !token.hasSheetError()) {
                     pushOperand(token);
                     return getNextToken();
                 }
@@ -408,13 +450,13 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
                     case 'lit':
                         // literal value: a separator or closing parenthesis must follow
                         token = getNextToken(true);
-                        if (!token.isType(/^mat_(col|row|close)$/)) { throw 'unexpected'; }
+                        if (!token.matchesType(/^mat_(col|row|close)$/)) { throw 'unexpected'; }
                         state = token.getType();
                         break;
                     case 'mat_close':
                         // closing parenthesis: finalize the matrix literal (check equal length of all rows first)
                         if (_.chain(array).pluck('length').unique().value().length > 1) { throw 'unexpected'; }
-                        pushOperand(tokenizer.createMatrixToken(new Matrix(array)));
+                        pushOperand(new Tokens.MatrixToken(new Matrix(array)));
                         state = 'end';
                         break;
                     }
@@ -455,14 +497,14 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
                         // start of parameter subexpression
                         params += 1;
                         // accept missing closing parenthesis after separator at end of formula, TODO: notify?
-                        if (!token || token.isType(/^(sep|close)$/)) {
+                        if (!token || token.matchesType(/^(sep|close)$/)) {
                             // empty parameter, e.g. in SUM(1,,2)
-                            pushOperand(tokenizer.createLiteralToken(null));
+                            pushOperand(new Tokens.LiteralToken(null));
                         } else {
                             // compile the parameter subexpression (stop at next separator token)
                             token = compileExpression(token, true);
                             // accept missing closing parenthesis after parameter at end of formula, TODO: notify?
-                            if (token && !token.isType(/^(sep|close)$/)) { throw 'unexpected'; }
+                            if (token && !token.matchesType(/^(sep|close)$/)) { throw 'unexpected'; }
                         }
                         state = token ? token.getType() : 'close';
                         break;
@@ -483,18 +525,18 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
             }
 
             try {
-                TokenUtils.info('Compiler.compileTokens()');
-                TokenUtils.logTokens('  compiling', tokens);
+                FormulaUtils.info('Compiler.compileTokens()');
+                FormulaUtils.logTokens('\xa0 compiling', tokens);
                 // compile the entire token array as subexpression
                 resultToken = compileExpression(getNextToken(true));
                 // double-check the compiled tokens
                 if (compiled.length === 0) { throw 'missing'; }
                 // error, when tokens are left in the passed token array
                 if (_.isObject(resultToken) || (index < tokens.length)) { throw 'unexpected'; }
-                TokenUtils.logTokens('  compiled', compiled);
+                FormulaUtils.logTokens('\xa0 compiled', compiled);
             } catch (error) {
                 if (_.isString(error)) {
-                    TokenUtils.warn('Compiler.compileTokens(): error: ' + error + ' token, index=' + index);
+                    FormulaUtils.warn('Compiler.compileTokens(): error: ' + error + ' token, index=' + index);
                     return { tokens: null, error: error };
                 }
                 throw error;
@@ -507,7 +549,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler', [
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = null;
+            docModel = null;
         });
 
     } // class Compiler
