@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
@@ -20,6 +20,10 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
 
     'use strict';
 
+    var // convenience shortcuts
+        Range = SheetUtils.Range,
+        RangeArray = SheetUtils.RangeArray;
+
     // class MergeCollection ==================================================
 
     /**
@@ -32,52 +36,49 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
      *      Event handlers receive the following parameters:
      *      (1) {jQuery.Event} event
      *          The jQuery event object.
-     *      (2) {Array} ranges
+     *      (2) {RangeArray} ranges
      *          An array with the addresses of all new merged ranges.
      *      (3) {Object} [options]
      *          Optional parameters:
      *          - {Boolean} [options.implicit=false]
-     *              If set to true, the merged ranges have been inserted
-     *              implicitly, for example while rows, columns, or cell ranges
-     *              have been inserted or deleted in the sheet.
+     *              If set to true, the merged ranges have been inserted (or
+     *              moved) implicitly, after rows, columns, or cell ranges have
+     *              been inserted or deleted in the sheet.
      * - 'delete:merged'
      *      After merged ranges have been deleted from the collection. Event
      *      handlers receive the following parameters:
      *      (1) {jQuery.Event} event
      *          The jQuery event object.
-     *      (2) {Array} ranges
+     *      (2) {RangeArray} ranges
      *          An array with the addresses of all deleted merged ranges.
      *      (3) {Object} [options]
      *          Optional parameters:
      *          - {Boolean} [options.implicit=false]
-     *              If set to true, the merged ranges have been deleted
-     *              implicitly, for example while rows, columns, or cell ranges
-     *              have been inserted or deleted in the sheet.
+     *              If set to true, the merged ranges have been deleted (or
+     *              moved) implicitly, after rows, columns, or cell ranges have
+     *              been inserted or deleted in the sheet.
      *
      * @constructor
      *
      * @extends ModelObject
      *
-     * @param {SpreadsheetApplication} app
-     *  The application that contains this collection instance.
-     *
      * @param {SheetModel} sheetModel
      *  The sheet model instance containing this collection.
      */
-    function MergeCollection(app, sheetModel) {
+    function MergeCollection(sheetModel) {
 
         var // self reference
             self = this,
 
             // the spreadsheet document model
-            docModel = app.getModel(),
+            docModel = sheetModel.getDocModel(),
 
             // all merged ranges
-            mergedRanges = [];
+            mergedRanges = new RangeArray();
 
         // base constructor ---------------------------------------------------
 
-        ModelObject.call(this, app);
+        ModelObject.call(this, docModel);
 
         // private methods ----------------------------------------------------
 
@@ -86,8 +87,8 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          * merged. The merged columns/rows must exactly fit into the range, and
          * all columns/rows in the range must be merged.
          *
-         * @param {Object} range
-         *  The address of a range.
+         * @param {Range} range
+         *  The address of a cell range.
          *
          * @returns {Boolean}
          *  Whether the columns/rows of the specified range are merged.
@@ -95,23 +96,21 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
         function isColRowMerged(range, columns) {
 
             var // all merged ranges that are contained in the passed range
-                containedRanges = self.getMergedRanges(range, 'contain'),
-                // the array index in cell addresses
-                addrIndex = columns ? 0 : 1;
+                containedRanges = self.getMergedRanges(range, 'contain');
 
             // the number of merged ranges contained by the passed range must be equal to the
             // column/row count of the range, otherwise there is no need to iterate at all
-            if (containedRanges.length !== SheetUtils.getIndexCount(range, columns)) {
+            if (containedRanges.length !== range.size(columns)) {
                 return false;
             }
 
             // check size and position of the merged ranges (if all merged ranges start and
             // end at the boundaries of the passed range, and are single-sized in the other
             // direction, the entire range is filled by single-sized merged ranges)
-            return _.all(containedRanges, function (mergedRange) {
-                return (mergedRange.start[addrIndex] === mergedRange.end[addrIndex]) &&
-                    (mergedRange.start[1 - addrIndex] === range.start[1 - addrIndex]) &&
-                    (mergedRange.end[1 - addrIndex] === range.end[1 - addrIndex]);
+            return containedRanges.every(function (mergedRange) {
+                return (mergedRange.size(columns) === 1) &&
+                    (mergedRange.getStart(!columns) === range.getStart(!columns)) &&
+                    (mergedRange.getEnd(!columns) === range.getEnd(!columns));
             });
         }
 
@@ -120,7 +119,7 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          * is not empty.
          */
         function triggerEvent(type, ranges, options) {
-            if (ranges.length > 0) {
+            if (!ranges.empty()) {
                 self.trigger(type, ranges, options);
             }
         }
@@ -132,9 +131,9 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
         function transformationHandler(interval, insert, columns) {
 
             var // all deleted merged ranges
-                deletedRanges = [],
+                deletedRanges = new RangeArray(),
                 // all inserted merged ranges
-                insertedRanges = [];
+                insertedRanges = new RangeArray();
 
             // visit all merged ranges in reversed order, to be able to delete in-place
             Utils.iterateArray(mergedRanges, function (mergedRange, index) {
@@ -146,8 +145,8 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
                 if (!newRange) {
                     deletedRanges.push(mergedRange);
                     mergedRanges.splice(index, 1);
-                } else if (!_.isEqual(mergedRange, newRange)) {
-                    insertedRanges.push(_.copy(newRange, true));
+                } else if (mergedRange.differs(newRange)) {
+                    insertedRanges.push(newRange.clone());
                     deletedRanges.push(mergedRange);
                     mergedRanges[index] = newRange;
                 }
@@ -161,24 +160,123 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
         // protected methods --------------------------------------------------
 
         /**
-         * Creates and returns a cloned instance of this collection for the
-         * specified sheet.
+         * Returns the internal contents of this collection, needed for cloning
+         * into another collection.
+         *
+         * @internal
+         *  Used during clone construction. DO NOT CALL from external code!
+         *
+         * @returns {Object}
+         *  The internal contents of this collection.
+         */
+        this.getCloneData = function () {
+            return { mergedRanges: mergedRanges };
+        };
+
+        /**
+         * Clones all contents from the passed collection into this collection.
          *
          * @internal
          *  Used by the class SheetModel during clone construction. DO NOT CALL
          *  from external code!
          *
-         * @param {SheetModel} targetModel
-         *  The model instance of the new cloned sheet that will own the clone
-         *  returned by this method.
+         * @param {MergeCollection} collection
+         *  The source collection whose contents will be cloned into this
+         *  collection.
          *
          * @returns {MergeCollection}
-         *  A clone of this collection, initialized for ownership by the passed
-         *  sheet model.
+         *  A reference to this instance.
          */
-        this.clone = function (targetModel) {
-            // construct a new collection, pass all own ranges as hidden parameter
-            return new MergeCollection(app, targetModel, mergedRanges);
+        this.cloneFrom = function (collection) {
+
+            var // the internal contents of the source collection
+                cloneData = collection.getCloneData();
+
+            // clone the contents of the source collection
+            mergedRanges = cloneData.mergedRanges.clone(true);
+
+            return this;
+        };
+
+        /**
+         * Callback handler for the document operation 'mergeCells'. Merges the
+         * cell range, or removes any merged ranges from the range covered by
+         * the operation.
+         *
+         * @param {SheetOperationContext} context
+         *  A wrapper representing the 'mergeCells' document operation.
+         */
+        this.applyMergeCellsOperation = function (context) {
+
+            var // the range covered by the operation
+                range = context.getRange(),
+                // the method how to merge/unmerge the operation range
+                type = context.getEnum('type', /^(merge|horizontal|vertical|unmerge)$/),
+                // all existing merged ranges deleted from the collection
+                deletedRanges = new RangeArray(),
+                // all merged ranges added to the collection
+                insertedRanges = new RangeArray();
+
+            // inserts the passed range into the collection, and into the 'insertedRanges' array
+            function insertMergedRange(mergedRange) {
+
+                var // the array index of the passed range in the 'deletedRanges' array
+                    deletedIndex = Utils.findFirstIndex(deletedRanges, mergedRange.equals.bind(mergedRange));
+
+                // if the range has been deleted, remove it from the array, otherwise
+                // insert it into the 'insertedRanges' array (this prevents to notify
+                // ranges that have been unmerged *and* merged)
+                if (deletedIndex >= 0) {
+                    deletedRanges.splice(deletedIndex, 1);
+                } else {
+                    insertedRanges.push(mergedRange);
+                }
+
+                // push a copy of the range into the collection
+                mergedRanges.push(mergedRange.clone());
+            }
+
+            // always remove all merged ranges that overlap with the passed range,
+            // regardless of passed type (needed to merge over merged ranges)
+            mergedRanges = RangeArray.reject(mergedRanges, function (mergedRange) {
+                var overlaps = range.overlaps(mergedRange);
+                if (overlaps) { deletedRanges.push(mergedRange); }
+                return overlaps;
+            });
+
+            // handle the different merge types
+            switch (type) {
+
+            case 'merge':
+                if (range.cells() > 1) {
+                    insertMergedRange(range);
+                }
+                break;
+
+            case 'horizontal':
+                if (range.cols() > 1) {
+                    Utils.iterateRange(range.start[1], range.end[1] + 1, function (row) {
+                        insertMergedRange(Range.create(range.start[0], row, range.end[0], row));
+                    });
+                }
+                break;
+
+            case 'vertical':
+                if (range.rows() > 1) {
+                    Utils.iterateRange(range.start[0], range.end[0] + 1, function (col) {
+                        insertMergedRange(Range.create(col, range.start[1], col, range.end[1]));
+                    });
+                }
+                break;
+
+            case 'unmerge':
+                // all merged ranges have been removed already, see above
+                break;
+            }
+
+            // trigger change events with array of changed merged ranges
+            triggerEvent('delete:merged', deletedRanges);
+            triggerEvent('insert:merged', insertedRanges);
         };
 
         // public methods -----------------------------------------------------
@@ -187,7 +285,7 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          * Returns, whether the cell at the specified address is the upper left
          * corner of a merged cell range.
          *
-         * @param {Number[]} address
+         * @param {Address} address
          *  The address of a cell.
          *
          * @returns {Boolean}
@@ -202,7 +300,7 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          * Returns, whether the cell at the specified address is a hidden cell
          * inside a merged cell range.
          *
-         * @param {Number[]} address
+         * @param {Address} address
          *  The address of a cell.
          *
          * @returns {Boolean}
@@ -217,7 +315,7 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          * Returns, whether the cell at the specified address is part of a
          * merged cell. It can be the reference cell or a hidden cell.
          *
-         * @param {Number[]} address
+         * @param {Address} address
          *  The address of a cell.
          *
          * @returns {Boolean}
@@ -230,14 +328,14 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
         /**
          * Returns whether the passed range is contained in this collection.
          *
-         * @param {Object} range
-         *  The address of a range.
+         * @param {Range} range
+         *  The cell range address to be checked.
          *
          * @returns {Boolean}
          *  Whether the specified range is contained in this collection.
          */
         this.isMergedRange = function (range) {
-            return _.any(mergedRanges, function (mergedRange) { return _.isEqual(range, mergedRange); });
+            return mergedRanges.some(function (mergedRange) { return range.equals(mergedRange); });
         };
 
         /**
@@ -245,8 +343,8 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          * The merged columns must exactly fit into the range, and all columns
          * in the range must be merged.
          *
-         * @param {Object} range
-         *  The address of a range.
+         * @param {Range} range
+         *  The cell range address to be checked.
          *
          * @returns {Boolean}
          *  Whether the specified range is entirely vertically merged.
@@ -260,8 +358,8 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          * merged rows must exactly fit into the range, and all rows in the
          * range must be merged.
          *
-         * @param {Object} range
-         *  The address of a range.
+         * @param {Range} range
+         *  The cell range address to be checked.
          *
          * @returns {Boolean}
          *  Whether the specified range is entirely horizontally merged.
@@ -274,7 +372,7 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          * Returns the range address of a merged cell range for a specified
          * cell address.
          *
-         * @param {Number[]} address
+         * @param {Address} address
          *  The address of a cell.
          *
          * @param {String} [type='all']
@@ -287,35 +385,35 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          *  the three types 'all', 'reference' or 'hidden', it defaults to
          *  'all'.
          *
-         * @returns {Object|Null}
+         * @returns {Range|Null}
          *  The range address of a merged range that covers the passed cell
          *  address; or null, if no merged range exists at the address.
          */
         this.getMergedRange = function (address, type) {
 
             var // the merged range containing the cell with the passed address
-                mergedRange = SheetUtils.findFirstRange(mergedRanges, address);
+                mergedRange = mergedRanges.findByAddress(address);
 
             if (mergedRange) {
                 switch (type) {
                 case 'reference':
-                    if (!_.isEqual(mergedRange.start, address)) { mergedRange = null; }
+                    if (!mergedRange.startAt(address)) { mergedRange = null; }
                     break;
                 case 'hidden':
-                    if (_.isEqual(mergedRange.start, address)) { mergedRange = null; }
+                    if (mergedRange.startsAt(address)) { mergedRange = null; }
                     break;
                 }
             }
 
-            return mergedRange ? _.copy(mergedRange, true) : null;
+            return mergedRange ? mergedRange.clone() : null;
         };
 
         /**
          * Returns the range addresses of all merged cell ranges that overlap
          * with one of the passed ranges.
          *
-         * @param {Object|Array} ranges
-         *  A single cell range address, or an array with range addresses.
+         * @param {RangeArray|Range} ranges
+         *  An array of cell range addresses, or a single cell range address.
          *
          * @param {String} [type='all']
          *  If set to 'all' or omitted, returns all merged ranges that overlap
@@ -324,33 +422,33 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          *  ranges. If set to 'reference', returns only merged ranges whose
          *  reference cell is contained in any of the passed ranges.
          *
-         * @returns {Array}
+         * @returns {RangeArray}
          *  The range addresses of all merged ranges that overlap with one of
          *  the passed ranges.
          */
         this.getMergedRanges = function (ranges, type) {
 
             var // the merged ranges not matching any passed range yet
-                pendingRanges = _.clone(mergedRanges),
+                pendingRanges = mergedRanges.clone(),
                 // the resulting merged ranges
-                resultRanges = [];
+                resultRanges = new RangeArray();
 
-            // process all passed ranges
-            _.any(_.getArray(ranges), function (range) {
+            // process all passed ranges, until 'pendingRanges' is empty
+            RangeArray.some(ranges, function (range) {
 
                 // iterate backwards through pendingRanges to be able to remove array elements
                 Utils.iterateArray(pendingRanges, function (mergedRange, index) {
 
                     // move matching merged range from pendingRanges to resultRanges
-                    if (SheetUtils.rangeOverlapsRange(range, mergedRange)) {
+                    if (range.overlaps(mergedRange)) {
                         switch (type) {
                         case 'reference':
-                            if (SheetUtils.rangeContainsCell(range, mergedRange.start)) {
+                            if (range.containsAddress(mergedRange.start)) {
                                 resultRanges.push(mergedRange);
                             }
                             break;
                         case 'contain':
-                            if (SheetUtils.rangeContainsRange(range, mergedRange)) {
+                            if (range.contains(mergedRange)) {
                                 resultRanges.push(mergedRange);
                             }
                             break;
@@ -362,8 +460,8 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
                     }
                 }, { reverse: true });
 
-                // early exit of the _.any() loop, if all merged ranges will be returned
-                return pendingRanges.length === 0;
+                // early exit of the loop, if all merged ranges will be returned
+                return pendingRanges.empty();
             });
 
             return resultRanges;
@@ -372,18 +470,18 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
         /**
          * Returns the merged range that completely contains or is equal to the
          * passed cell range. Note the different to the method
-         * getMergedRanges() which returns all merged ranges that are contained
-         * IN the passed ranges.
+         * MergeCollection.getMergedRanges() which returns all merged ranges
+         * that are contained IN the passed ranges.
          *
-         * @param {Object} range
-         *  A range address.
+         * @param {Range} range
+         *  A cell range address.
          *
-         * @returns {Object|Null}
+         * @returns {Range|Null}
          *  The address of a merged range that contains the passed range if
          *  existing, otherwise null
          */
         this.getBoundingMergedRange = function (range) {
-            var boundingRange = _.find(mergedRanges, function (mergedRange) { return SheetUtils.rangeContainsRange(mergedRange, range); });
+            var boundingRange = _.find(mergedRanges, function (mergedRange) { return mergedRange.contains(range); });
             return boundingRange ? boundingRange : null;
         };
 
@@ -391,44 +489,47 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          * Returns whether any of the passed cell ranges overlaps with at least
          * one merged cell range.
          *
-         * @param {Object|Array} ranges
-         *  Address of a single cell range, or an array of range addresses.
+         * @param {RangeArray|Range} ranges
+         *  An array of cell range addresses, or a single cell range address.
          *
          * @returns {Boolean}
          *  Whether any of the passed cell ranges contains or overlaps with at
          *  least one merged cell range.
          */
         this.rangesOverlapAnyMergedRange = function (ranges) {
-            return SheetUtils.rangesOverlapRanges(ranges, mergedRanges);
+            return mergedRanges.overlaps(ranges);
         };
 
         /**
          * Expands the passed range address, so that it will include all merged
          * ranges it is currently covering partly.
          *
-         * @param {Object} range
+         * @param {Range} range
          *  The address of a cell range that will be expanded to the merged
          *  ranges contained in this collection.
          *
-         * @returns {Object}
-         *  The address of the expanded range.
+         * @returns {Range}
+         *  The address of the expanded cell range.
          */
         this.expandRangeToMergedRanges = function (range) {
 
             var // flat copy of the merged ranges, to prevent processing them multiple times
-                pendingRanges = _.clone(mergedRanges),
+                pendingRanges = mergedRanges.clone(),
                 // the length of the pending ranges array
                 pendingLength = pendingRanges.length;
 
             // loop iterator function must be defined outside the do-while loop
             function expandToMergedRange(mergedRange, index) {
                 // process all merged ranges that overlap the current range
-                if (SheetUtils.rangeOverlapsRange(range, mergedRange)) {
-                    range = SheetUtils.getBoundingRange(range, mergedRange);
+                if (range.overlaps(mergedRange)) {
+                    range = range.boundary(mergedRange);
                     pendingRanges.splice(index, 1);
                 }
             }
 
+            // ensure to keep passed range intact
+            range = range.clone();
+
             // loop as long as the range will still be expanded with other merged ranges
             if (pendingRanges.length > 0) {
                 do {
@@ -440,14 +541,14 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
             }
 
             // return the fully expanded range
-            return _.copy(range, true);
+            return range;
         };
 
         /**
          * Shrinks the passed range address, so that it will not include any
          * merged ranges it is currently covering partly.
          *
-         * @param {Object} range
+         * @param {Range} range
          *  The address of a cell range that will be shrunk until it does not
          *  partly contain any merged ranges.
          *
@@ -456,7 +557,7 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          *  values are 'columns' to shrink the range at its left and right
          *  border, or 'rows' to shrink the range at its top and bottom border.
          *
-         * @returns {Object|Null}
+         * @returns {Range|Null}
          *  The address of the shrunk range; or null, if no valid range was
          *  left after shrinking (the range has been shrunk to zero
          *  columns/rows).
@@ -464,33 +565,33 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
         this.shrinkRangeFromMergedRanges = function (range, direction) {
 
             var // flat copy of the merged ranges, to prevent processing them multiple times
-                pendingRanges = _.clone(mergedRanges),
+                pendingRanges = mergedRanges.clone(),
                 // the length of the pending ranges array
                 pendingLength = pendingRanges.length,
-                // address array index for passed direction
-                addressIndex = (direction === 'columns') ? 0 : 1;
+                // the direction to shrink the range
+                columns = direction === 'columns';
 
             // loop iterator function must be defined outside the do-while loop
             function shrinkFromMergedRange(mergedRange, index) {
 
                 // do not process merged ranges that are fully contained in the current range
-                if (SheetUtils.rangeContainsRange(range, mergedRange)) { return; }
+                if (range.contains(mergedRange)) { return; }
                 // remove the merged range from the pending array
                 pendingRanges.splice(index, 1);
                 // nothing to do, if merged range is completely outside
-                if (!SheetUtils.rangeOverlapsRange(range, mergedRange)) { return; }
+                if (!range.overlaps(mergedRange)) { return; }
 
                 // shrink leading border of the range
-                if ((mergedRange.start[addressIndex] < range.start[addressIndex]) && (range.start[addressIndex] <= mergedRange.end[addressIndex])) {
-                    range.start[addressIndex] = mergedRange.end[addressIndex] + 1;
+                if ((mergedRange.getStart(columns) < range.getStart(columns)) && (range.getStart(columns) <= mergedRange.getEnd(columns))) {
+                    range.setStart(mergedRange.getEnd(columns) + 1, columns);
                 }
                 // shrink trailing border of the range
-                if ((mergedRange.start[addressIndex] <= range.end[addressIndex]) && (range.end[addressIndex] < mergedRange.end[addressIndex])) {
-                    range.end[addressIndex] = mergedRange.start[addressIndex] - 1;
+                if ((mergedRange.getStart(columns) <= range.getEnd(columns)) && (range.getEnd(columns) < mergedRange.getEnd(columns))) {
+                    range.setEnd(mergedRange.getStart(columns) - 1, columns);
                 }
 
                 // check if the range has been shrunk to null size
-                if (range.start[addressIndex] > range.end[addressIndex]) {
+                if (range.getStart(columns) > range.getEnd(columns)) {
                     range = null;
                     return Utils.BREAK;
                 }
@@ -510,100 +611,16 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
             return range;
         };
 
-        /**
-         * Merges the specified cell range, or removes any merged ranges from
-         * that range.
-         *
-         * @param {Object} range
-         *  The address of the range to be merged or unmerged.
-         *
-         * @param {String} type
-         *  The merge type. Must be one of: 'merge', 'horizontal', 'vertical',
-         *  or 'unmerge'.
-         *
-         * @returns {MergeCollection}
-         *  A reference to this instance.
-         */
-        this.mergeRange = function (range, type) {
-
-            var // all existing merged ranges deleted from the collection
-                deletedRanges = [],
-                // all merged ranges added to the collection
-                insertedRanges = [];
-
-            // inserts the passed range into the collection, and into the 'insertedRanges' array
-            function insertMergedRange(mergedRange) {
-
-                var // the array index of the passed range in the 'deletedRanges' array
-                    deletedIndex = Utils.findFirstIndex(deletedRanges, _.partial(_.isEqual, mergedRange));
-
-                // if the range has been deleted, remove it from the array, otherwise
-                // insert it into the 'insertedRanges' array (this prevents to notify
-                // ranges that have been unmerged *and* merged)
-                if (deletedIndex >= 0) {
-                    deletedRanges.splice(deletedIndex, 1);
-                } else {
-                    insertedRanges.push(mergedRange);
-                }
-
-                // push a copy of the range into the collection
-                mergedRanges.push(_.copy(mergedRange, true));
-            }
-
-            // always remove all merged ranges that overlap with the passed range,
-            // regardless of passed type (needed to merge over merged ranges)
-            mergedRanges = _.reject(mergedRanges, function (mergedRange) {
-                var overlaps = SheetUtils.rangeOverlapsRange(range, mergedRange);
-                if (overlaps) { deletedRanges.push(mergedRange); }
-                return overlaps;
-            });
-
-            // handle the different merge types
-            switch (type) {
-            case 'merge':
-                if (SheetUtils.getCellCount(range) > 1) {
-                    insertMergedRange(range);
-                }
-                break;
-            case 'horizontal':
-                if (SheetUtils.getColCount(range) > 1) {
-                    Utils.iterateRange(range.start[1], range.end[1] + 1, function (row) {
-                        insertMergedRange({ start: [range.start[0], row], end: [range.end[0], row] });
-                    });
-                }
-                break;
-            case 'vertical':
-                if (SheetUtils.getRowCount(range) > 1) {
-                    Utils.iterateRange(range.start[0], range.end[0] + 1, function (col) {
-                        insertMergedRange({ start: [col, range.start[1]], end: [col, range.end[1]] });
-                    });
-                }
-                break;
-            case 'unmerge':
-                // all merged ranges have been removed already, see above
-                break;
-            default:
-                Utils.warn('MergeCollection.mergeRange(): unknown merge type "' + type + '"');
-            }
-
-            // trigger change events with array of changed merged ranges
-            triggerEvent('delete:merged', deletedRanges);
-            triggerEvent('insert:merged', insertedRanges);
-
-            return this;
-        };
-
         /**
          * Transforms the passed merged range according to the specified column
          * or row operation. Does NOT modify any merged range in this merged
          * ranges collection.
          *
-         * @param {Object} mergedRange
+         * @param {Range} mergedRange
          *  The address of the merged range to be transformed.
          *
-         * @param {Object} operationInterval
-         *  The column/row interval of the operation, with the index properties
-         *  'first' and 'last'.
+         * @param {Interval} operationInterval
+         *  The column/row interval of the operation.
          *
          * @param {Boolean} insert
          *  Whether the specified column/row interval has been inserted into
@@ -613,7 +630,7 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
          *  Whether the specified interval is a column interval (true), or a
          *  row interval (false).
          *
-         * @returns {Object|Null}
+         * @returns {Range|Null}
          *  The address of the transformed merged range. If the range has been
          *  deleted completely while deleting columns/rows, or has been shifted
          *  outside the sheet completely while inserting columns/rows, or if
@@ -624,7 +641,7 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
             // transform the passed range without expanding the end of the range while inserting
             mergedRange = docModel.transformRange(mergedRange, operationInterval, insert, columns);
             // a transformed merged range covering a single cell is considered deleted
-            return (mergedRange && (SheetUtils.getCellCount(mergedRange) > 1)) ? mergedRange : null;
+            return (mergedRange && (mergedRange.cells() > 1)) ? mergedRange : null;
         };
 
         // initialization -----------------------------------------------------
@@ -632,14 +649,9 @@ define('io.ox/office/spreadsheet/model/mergecollection', [
         // update merged ranges after inserting/deleting columns or rows in the sheet
         sheetModel.registerTransformationHandler(transformationHandler);
 
-        // clone merged ranges passed as hidden argument by the clone() method
-        if (_.isArray(arguments[MergeCollection.length])) {
-            mergedRanges = _.copy(arguments[MergeCollection.length], true);
-        }
-
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = sheetModel = self = docModel = null;
+            self = docModel = sheetModel = null;
         });
 
     } // class MergeCollection
