@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -26,13 +26,19 @@ define('io.ox/office/spreadsheet/model/sheetoperationsgenerator', [
      *
      * @constructor
      *
-     * @extends SpreadsheetOperationsGenerator
+     * @extends OperationsGenerator
+     *
+     * @param {SheetModel} sheetModel
+     *  The sheet model this operations generator will be created for.
      */
-    function SheetOperationsGenerator(app, sheet) {
+    function SheetOperationsGenerator(sheetModel) {
+
+        var // the sheet index of the passed sheet
+            sheet = sheetModel.getIndex();
 
         // base constructor ---------------------------------------------------
 
-        OperationsGenerator.call(this, app);
+        OperationsGenerator.call(this, sheetModel.getDocModel());
 
         // public methods -----------------------------------------------------
 
@@ -64,9 +70,8 @@ define('io.ox/office/spreadsheet/model/sheetoperationsgenerator', [
          * @param {String} name
          *  The name of the operation.
          *
-         * @param {Object} interval
-         *  The zero-based column/row indexes of the interval, in the
-         *  properties 'first' and 'last'.
+         * @param {Interval} interval
+         *  The column/row interval to be inserted into the operation.
          *
          * @param {Object} [properties]
          *  Additional properties that will be stored in the operation.
@@ -97,7 +102,7 @@ define('io.ox/office/spreadsheet/model/sheetoperationsgenerator', [
          * @param {String} name
          *  The name of the operation.
          *
-         * @param {Number[]} address
+         * @param {Address} address
          *  The address of the cell targeted by the operation.
          *
          * @param {Object} [properties]
@@ -107,7 +112,7 @@ define('io.ox/office/spreadsheet/model/sheetoperationsgenerator', [
          *  The created operation object.
          */
         this.generateCellOperation = function (name, address, properties) {
-            properties = _.extend({ start: address }, properties);
+            properties = _.extend({ start: address.toJSON() }, properties);
             return this.generateSheetOperation(name, properties);
         };
 
@@ -120,7 +125,7 @@ define('io.ox/office/spreadsheet/model/sheetoperationsgenerator', [
          * @param {String} name
          *  The name of the operation.
          *
-         * @param {Object} range
+         * @param {Range} range
          *  The address of the cell range addressed by the operation.
          *
          * @param {Object} [properties]
@@ -138,9 +143,9 @@ define('io.ox/office/spreadsheet/model/sheetoperationsgenerator', [
          */
         this.generateRangeOperation = function (name, range, properties, options) {
             // add 'start' and 'end' properties to the operation
-            properties = _.extend({ start: range.start }, properties);
-            if (!_.isEqual(range.start, range.end) || Utils.getBooleanOption(options, 'complete', false)) {
-                properties.end = range.end;
+            properties = _.extend({ start: range.start.toJSON() }, properties);
+            if (!range.single() || Utils.getBooleanOption(options, 'complete', false)) {
+                properties.end = range.end.toJSON();
             }
             return this.generateSheetOperation(name, properties);
         };
@@ -176,7 +181,7 @@ define('io.ox/office/spreadsheet/model/sheetoperationsgenerator', [
          * @param {String} name
          *  The name of the operation.
          *
-         * @param {Number[]} position
+         * @param {Array<Number>} position
          *  The position of the drawing object in the sheet.
          *
          * @param {Object} [properties]
