@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -27,17 +27,14 @@ define('io.ox/office/spreadsheet/model/tablestylecollection', [
      *
      * @extends StyleCollection
      *
-     * @param {SpreadsheetApplication} app
-     *  The root application instance.
-     *
-     * @param {SpreadsheetDocumentStyles} documentStyles
-     *  Collection with the style containers of all style families.
+     * @param {SpreadsheetModel} docModel
+     *  The document model containing this instance.
      */
-    function TableStyleCollection(app, documentStyles) {
+    function TableStyleCollection(docModel) {
 
         // base constructor ---------------------------------------------------
 
-        StyleCollection.call(this, app, documentStyles, 'table', { families: 'character cell' });
+        StyleCollection.call(this, docModel, 'table', { families: 'character cell' });
 
     } // class TableStyleCollection
 
