@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -21,51 +21,51 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
 
     'use strict';
 
+    var // convenience shortcuts
+        Address = SheetUtils.Address;
+
     // class ValidationModel ==================================================
 
     /**
-     * Stores data validation settings for a single range in a specific sheet.
+     * Stores data validation settings for one or more cell ranges in a sheet.
      *
      * @constructor
      *
      * @extends AttributedModel
      *
-     * @param {SpreadsheetApplication} app
-     *  The application that contains this data validation range.
-     *
      * @param {SheetModel} sheetModel
-     *  The sheet model instance containing this data validation range.
+     *  The sheet model instance containing this instance.
      *
-     * @param {Array} ranges
+     * @param {RangeArray} targetRanges
      *  The addresses of all cell ranges that contain the data validation
      *  settings.
      *
      * @param {Object} [attrs]
      *  The data validation attribute set.
      */
-    var ValidationModel = AttributedModel.extend({ constructor: function (app, sheetModel, ranges, attrs) {
+    var ValidationModel = AttributedModel.extend({ constructor: function (sheetModel, targetRanges, attrs) {
 
         var // self reference
             self = this,
 
             // the spreadsheet document model
-            docModel = app.getModel(),
+            docModel = sheetModel.getDocModel(),
 
             // token array representing the validation attribute 'value1'
-            tokenArray1 = new TokenArray(app, sheetModel),
+            tokenArray1 = new TokenArray(sheetModel),
 
             // token array representing the validation attribute 'value2'
-            tokenArray2 = new TokenArray(app, sheetModel);
+            tokenArray2 = new TokenArray(sheetModel);
 
         // base constructor ---------------------------------------------------
 
-        AttributedModel.call(this, app, attrs, { families: 'validation' });
+        AttributedModel.call(this, docModel, attrs, { families: 'validation' });
 
         // private methods ----------------------------------------------------
 
         /**
          * Updates internal settings after the validation attributes of this
-         * collection entry have been changed.
+         * validation model have been changed.
          */
         function changeAttributesHandler(event, newAttributes, oldAttributes) {
 
@@ -75,12 +75,12 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
 
             // parse a changed formula expression in attribute 'value1'
             if (newAttributes.value1 !== oldAttributes.value1) {
-                tokenArray1.invokeSilently('parseFormula', newAttributes.value1);
+                tokenArray1.invokeSilently('parseFormula', 'op', newAttributes.value1);
             }
 
             // parse a changed formula expression in attribute 'value2'
             if (newAttributes.value2 !== oldAttributes.value2) {
-                tokenArray2.invokeSilently('parseFormula', newAttributes.value2);
+                tokenArray2.invokeSilently('parseFormula', 'op', newAttributes.value2);
             }
         }
 
@@ -104,7 +104,7 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
          */
         this.clone = function (targetModel) {
             // construct a new model instance and relocate the sheet references
-            var valModel = new ValidationModel(app, targetModel, ranges, this.getExplicitAttributes(true));
+            var valModel = new ValidationModel(targetModel, targetRanges, this.getExplicitAttributes(true));
             valModel.relocateSheet(sheetModel.getIndex(), targetModel.getIndex());
             return valModel;
         };
@@ -112,39 +112,40 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
         // public methods -----------------------------------------------------
 
         /**
-         * Returns the addresses of all cell ranges of this collection entry.
+         * Returns the addresses of all target cell ranges of this validation
+         * model.
          *
-         * @returns {Array}
+         * @returns {RangeArray}
          *  The addresses of all cell ranges.
          */
         this.getRanges = function () {
-            return _.copy(ranges, true);
+            return targetRanges.clone(true);
         };
 
         /**
-         * Returns whether the passed cell address is contained by the cell
-         * ranges of this collection entry.
+         * Returns whether the passed cell address is contained by the target
+         * cell ranges of this validation model.
          *
-         * @param {Number[]} address
+         * @param {Address} address
          *  The address of the cell to be checked.
          *
          * @returns {Boolean}
-         *  Whether the specified cell is contained in this collection entry.
+         *  Whether the specified cell is contained in this validation model.
          */
         this.containsCell = function (address) {
-            return SheetUtils.rangesContainCell(ranges, address);
+            return targetRanges.containsAddress(address);
         };
 
         /**
-         * Returns the reference addresses of this collection entry. The
-         * reference address is the top-left cell of the bounding range of all
-         * ranges of this collection entry.
+         * Returns the reference addresses of this validation model (the
+         * top-left cell of the bounding range of all target cell ranges of
+         * this validation model).
          *
-         * @returns {Number[]}
-         *  The address of the reference cell of this collection entry.
+         * @returns {Address}
+         *  The address of the reference cell of this validation model.
          */
         this.getRefAddress = function () {
-            return SheetUtils.getBoundingRange(ranges).start;
+            return targetRanges.empty() ? Address.A1.clone() : targetRanges.boundary().start;
         };
 
         /**
@@ -152,19 +153,21 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
          * according to the formula expression contained in the attribute
          * 'value1'.
          *
-         * @param {Number[]} targetAddress
+         * @param {Address} targetAddress
          *  The target address used to resolve relative references in the
          *  formula expression.
          *
          * @returns {jQuery.Promise}
-         *  The promise of a Deferred object that will be resolved with an
-         *  array of strings, if this collection entry is a data validation of
-         *  type 'list' or 'source', and the formula expression contained in
-         *  the attribute 'value1' could be resolved successfully to a list of
-         *  strings. Otherwise, the promise will be rejected. Additionally, the
-         *  promise will contain the method 'abort()' that is able to abort a
-         *  running server request. In this case, the promise will be rejected
-         *  immediately.
+         *  A promise that will be resolved with an array of strings and/or
+         *  null values, if this model is a data validation of type 'list' or
+         *  'source', and the formula expression contained in the attribute
+         *  'value1' could be resolved successfully to a list. The array
+         *  elements set to the value null represent cells that cannot be
+         *  formatted to a valid display string with their number format.
+         *  Otherwise, if no list could be generated, the promise will be
+         *  rejected. Additionally, the promise will contain the method abort()
+         *  that is able to abort a running server request. In this case, the
+         *  promise will be rejected immediately.
          */
         this.queryListValues = function (targetAddress) {
 
@@ -181,9 +184,13 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
 
             // resolve formula to source ranges (validation type 'source')
             function resolveSource() {
-                var refAddress = self.getRefAddress(),
-                    sourceRanges = tokenArray1.resolveRangeList({ refAddress: refAddress, targetAddress: targetAddress });
-                if (!_.isArray(sourceRanges)) { return $.Deferred().reject(); }
+
+                var // the address of the reference cell, used to resolve range addresses
+                    refAddress = self.getRefAddress(),
+                    // get the cell range addresses contained in the first token array
+                    sourceRanges = tokenArray1.resolveRangeList({ refAddress: refAddress, targetAddress: targetAddress, resolveNames: true });
+
+                if (sourceRanges.empty()) { return $.Deferred().reject(); }
                 sourceQuery = docModel.queryCellContents([sourceRanges], { hidden: true, maxCount: 1000 });
                 return sourceQuery.then(function (resultContents) {
                     return _.pluck(resultContents[0], 'display');
@@ -213,58 +220,64 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
         };
 
         /**
-         * Changes the addresses of all cell ranges of this collection entry.
+         * Changes the target cell ranges of this validation model.
          *
-         * @param {Array} newRanges
-         *  The addresses of the new cell ranges. The old ranges of this
-         *  collection entry will be replaced completely.
+         * @param {RangeArray} newRanges
+         *  The addresses of the new target cell ranges. The old target ranges
+         *  of this validation model will be replaced completely.
          *
          * @returns {ValidationModel}
          *  A reference to this instance.
          */
         this.setRanges = function (newRanges) {
-            ranges = _.copy(newRanges, true);
+            targetRanges = newRanges.merge();
             return this;
         };
 
         /**
-         * Reduces the cell ranges containing the data validation settings to
-         * the difference of these ranges and the passed ranges.
+         * Reduces the target cell ranges containing the data validation
+         * settings to the difference of these ranges and the passed ranges.
+         *
+         * @param {RangeArray} otherRanges
+         *  The addresses of the cell ranges to be removed from the target cell
+         *  ranges of this model.
          *
          * @returns {Boolean}
-         *  Whether the reduced ranges are still valid (not all deleted). If
-         *  false is returned, this collection entry must be deleted from the
-         *  validation collection.
+         *  Whether the reduced target cell ranges are still valid (not all
+         *  deleted). If false is returned, this validation model must be
+         *  deleted from the validation collection.
          */
         this.reduceRanges = function (otherRanges) {
-            ranges = SheetUtils.getRemainingRanges(ranges, otherRanges);
-            return ranges.length > 0;
+            targetRanges = targetRanges.difference(otherRanges);
+            return !targetRanges.empty();
         };
 
         /**
-         * Transforms the addresses of the cell ranges containing the data
-         * validation settings, after columns or rows have been inserted into
-         * or removed from the sheet.
+         * Transforms the target cell ranges of this validation model, after
+         * columns or rows have been inserted into or removed from the sheet.
          *
          * @returns {Boolean}
-         *  Whether the transformed ranges are still valid (not all deleted).
-         *  If false is returned, this collection entry must be deleted from
-         *  the validation collection.
+         *  Whether the transformed target cell ranges are still valid (not all
+         *  deleted). If false is returned, this validation model must be
+         *  deleted from the validation collection.
          */
         this.transformRanges = function (interval, insert, columns) {
-            ranges = docModel.transformRanges(ranges, interval, insert, columns, 'expandEnd');
-            return ranges.length > 0;
+            targetRanges = docModel.transformRanges(targetRanges, interval, insert, columns, 'expandEnd');
+            return !targetRanges.empty();
         };
 
         /**
          * Recalculates the formula expressions stored in the validation
          * attributes 'value1' and 'value2', after columns or rows have been
          * inserted into or deleted from any sheet in the document.
+         *
+         * @returns {ValidationModel}
+         *  A reference to this instance.
          */
         this.transformValues = function (sheet, interval, insert, columns) {
-            tokenArray1.transformFormula(sheet, interval, insert, columns);
-            tokenArray2.transformFormula(sheet, interval, insert, columns);
-            return true;
+            tokenArray1.transformRanges(sheet, interval, insert, columns);
+            tokenArray2.transformRanges(sheet, interval, insert, columns);
+            return this;
         };
 
         /**
@@ -283,8 +296,8 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
 
         // initialization -----------------------------------------------------
 
-        // unify the range addresses
-        ranges = SheetUtils.getUnifiedRanges(ranges);
+        // merge the target range addresses
+        targetRanges = targetRanges.merge();
 
         // parse the source formulas after changing value attributes
         this.on('change:attributes', changeAttributesHandler);
@@ -293,30 +306,20 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
         // update source values after one of the token array has changed (but prevent updating the token array again)
         tokenArray1.on('triggered', function () {
             self.off('change:attributes', changeAttributesHandler);
-            self.setAttributes({ validation: { value1: tokenArray1.getFormula() } });
+            self.setAttributes({ validation: { value1: tokenArray1.getFormula('op') } });
             self.on('change:attributes', changeAttributesHandler);
         });
         tokenArray2.on('triggered', function () {
             self.off('change:attributes', changeAttributesHandler);
-            self.setAttributes({ validation: { value2: tokenArray2.getFormula() } });
+            self.setAttributes({ validation: { value2: tokenArray2.getFormula('op') } });
             self.on('change:attributes', changeAttributesHandler);
         });
 
-        // adjust formulas according to sheet indexes passed as hidden argument to the c'tor
-        (function (args) {
-            var cloneData = args[ValidationModel.length];
-            if (_.isObject(cloneData)) {
-                tokenArray1.relocateSheet(cloneData.oldSheet, cloneData.newSheet);
-                tokenArray2.relocateSheet(cloneData.oldSheet, cloneData.newSheet);
-            }
-        }(arguments));
-
         // destroy all class members
         this.registerDestructor(function () {
             tokenArray1.destroy();
             tokenArray2.destroy();
-            app = sheetModel = null;
-            self = docModel = tokenArray1 = tokenArray2 = null;
+            self = docModel = sheetModel = tokenArray1 = tokenArray2 = null;
         });
 
     }}); // class ValidationModel
@@ -325,81 +328,111 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
 
     /**
      * Stores data validation settings in a specific sheet. Data validation can
-     * be used to restrict the values allowed to be entered in cell ranges, and
-     * to show tooltips and error messages.
+     * be used to restrict the values allowed to be entered in specific cell
+     * ranges, and to show tooltips and error messages.
      *
      * @constructor
      *
      * @extends ModelObject
      *
-     * @param {SpreadsheetApplication} app
-     *  The application that contains this collection instance.
-     *
      * @param {SheetModel} sheetModel
      *  The sheet model instance containing this collection.
      */
-    function ValidationCollection(app, sheetModel) {
+    function ValidationCollection(sheetModel) {
 
         var // the spreadsheet document model
-            docModel = app.getModel(),
+            docModel = sheetModel.getDocModel(),
 
-            // all collection entries with validation settings
-            entries = [];
+            // all model instances with validation settings
+            valModels = [];
 
         // base constructor ---------------------------------------------------
 
-        ModelObject.call(this, app);
+        ModelObject.call(this, docModel);
 
         // private methods ----------------------------------------------------
 
         /**
-         * Returns the collection entry that covers the specified cell.
+         * Returns the validation model that covers the specified cell.
          *
-         * @param {Number[]} address
+         * @param {Address} address
          *  The address of a cell.
          *
          * @returns {ValidationModel|Null}
-         *  The collection entry for the specified cell. If the cell is not
-         *  validated at all, returns null.
+         *  The validation for the specified cell. If the cell is not validated
+         *  at all, returns null.
          */
-        function getEntry(address) {
-            var entry = _.find(entries, function (entry2) {
-                return entry2.containsCell(address);
+        function findModelByAddress(address) {
+            var valModel = _.find(valModels, function (model) {
+                return model.containsCell(address);
             });
-            return entry ? entry : null;
+            return valModel ? valModel : null;
         }
 
         /**
-         * Invokes the specified method on all collection entries.
+         * Returns the validation model addressed by the property 'index' in
+         * the passed document operation.
+         *
+         * @param {SheetOperationContext} context
+         *  A wrapper representing a document operation for validations,
+         *  expected to contain an integer property 'index'.
+         *
+         * @returns {Object}
+         *  A descriptor for the validation model and its index, in the
+         *  following properties:
+         *  - {Number} index
+         *      The array index of the validation model (the value of the
+         *      'index' property in the passed operation).
+         *  - {ValidationModel} model
+         *      The validation model referenced by the document operation.
+         *
+         * @throws {OperationError}
+         *  If the operation does not contain a valid 'index' property, or if
+         *  the validation model with such an index does not exist.
+         */
+        function getModelData(context) {
+
+            // get the index from the operation
+            var index = context.getInt('index');
+
+            // check existence of the validation model
+            var valModel = valModels[index];
+            context.ensure(valModel, 'missing validation');
+
+            return { index: index, model: valModel };
+        }
+
+        /**
+         * Invokes the specified method on all validation models.
          *
          * @param {String} methodName
-         *  The name of the method to be invoked on each collection entry. If
-         *  the method returns the value false, the respective entry will be
+         *  The name of the method to be invoked on each validation model. If
+         *  the method returns the value false, the respective model will be
          *  removed from the collection.
          *
-         * @param {ValidationModel|Null} excludeEntry
-         *  A collection entry that will be skipped (its method will not be
+         * @param {ValidationModel|Null} excludeModel
+         *  A validation model that will be skipped (its method will not be
          *  invoked).
          *
          * @param {Any} ...
          *  Other parameters that will be passed to the specified method.
          */
-        function invokeEntryMethod(methodName, excludeEntry) {
-            var args = _.toArray(arguments).slice(invokeEntryMethod.length);
-            Utils.iterateArray(entries, function (entry, index) {
-                if ((entry !== excludeEntry) && (entry[methodName].apply(entry, args) === false)) {
-                    entry.destroy();
-                    entries.splice(index, 1);
+        function invokeModelMethod(methodName, excludeModel) {
+            var args = _.toArray(arguments).slice(invokeModelMethod.length);
+            Utils.iterateArray(valModels, function (valModel, index) {
+                if ((valModel !== excludeModel) && (valModel[methodName].apply(valModel, args) === false)) {
+                    valModel.destroy();
+                    valModels.splice(index, 1);
                 }
             }, { reverse: true }); // reverse to be able to delete in-place
         }
 
         /**
-         * Recalculates the position of all validation ranges, after columns or
-         * rows have been inserted into or deleted from the sheet.
+         * Recalculates the position of all validation target ranges, after
+         * columns or rows have been inserted into or deleted from the sheet.
          */
         function transformRanges(interval, insert, columns) {
-            invokeEntryMethod('transformRanges', null, interval, insert, columns);
+            invokeModelMethod('transformRanges', null, interval, insert, columns);
         }
 
         /**
@@ -408,30 +441,116 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
          * inserted into or deleted from any sheet in the document.
          */
         function transformValues(sheet, interval, insert, columns) {
-            invokeEntryMethod('transformValues', null, sheet, interval, insert, columns);
+            _.invoke(valModels, 'transformValues', sheet, interval, insert, columns);
         }
 
         // protected methods --------------------------------------------------
 
         /**
-         * Creates and returns a cloned instance of this collection for the
-         * specified sheet.
+         * Returns the internal contents of this collection, needed for cloning
+         * into another collection.
+         *
+         * @internal
+         *  Used during clone construction. DO NOT CALL from external code!
+         *
+         * @returns {Object}
+         *  The internal contents of this collection.
+         */
+        this.getCloneData = function () {
+            return { valModels: valModels };
+        };
+
+        /**
+         * Clones all contents from the passed collection into this collection.
          *
          * @internal
          *  Used by the class SheetModel during clone construction. DO NOT CALL
          *  from external code!
          *
-         * @param {SheetModel} targetModel
-         *  The model instance of the new cloned sheet that will own the clone
-         *  returned by this method.
+         * @param {ValidationCollection} collection
+         *  The source collection whose contents will be cloned into this
+         *  collection.
          *
          * @returns {ValidationCollection}
-         *  A clone of this collection, initialized for ownership by the passed
-         *  sheet model.
+         *  A reference to this instance.
          */
-        this.clone = function (targetModel) {
-            // construct a new collection, pass all own entries as hidden parameter
-            return new ValidationCollection(app, targetModel, entries);
+        this.cloneFrom = function (collection) {
+
+            var // the internal contents of the source collection
+                cloneData = collection.getCloneData();
+
+            // clone the contents of the source collection
+            valModels = _.invoke(cloneData.valModels, 'clone', sheetModel);
+
+            return this;
+        };
+
+        /**
+         * Callback handler for the document operation 'insertValidation'.
+         *
+         * @param {SheetOperationContext} context
+         *  A wrapper representing the 'insertValidation' document operation.
+         */
+        this.applyInsertOperation = function (context) {
+
+            // get and validate the insertion index
+            var index = context.getInt('index');
+            context.ensure((index >= 0) && (index <= valModels.length), 'invalid index');
+
+            // the new validation model
+            var valModel = new ValidationModel(sheetModel, context.getRangeArray(), { validation: context.operation });
+
+            // reduce ranges of existing models (this may result in removing models completely)
+            invokeModelMethod('reduceRanges', null, valModel.getRanges());
+
+            // insert the new validation model
+            valModels.splice(index, 0, valModel);
+
+            // TODO: notify listeners
+        };
+
+        /**
+         * Callback handler for the document operation 'deleteValidation'.
+         *
+         * @param {SheetOperationContext} context
+         *  A wrapper representing the 'deleteValidation' document operation.
+         */
+        this.applyDeleteOperation = function (context) {
+
+            // resolve the validation model addressed by the operation (throws on error)
+            var modelData = getModelData(context);
+
+            // destroy and remove the validation model
+            modelData.model.destroy();
+            valModels.splice(modelData.index, 1);
+
+            // TODO: notify listeners
+        };
+
+        /**
+         * Callback handler for the document operation 'changeValidation'.
+         *
+         * @param {SheetOperationContext} context
+         *  A wrapper representing the 'changeValidation' document operation.
+         */
+        this.applyChangeOperation = function (context) {
+
+            // resolve the validation model addressed by the operation (throws on error)
+            var modelData = getModelData(context);
+
+            // resolve optional 'ranges' property (throws on error), change target ranges
+            var targetRanges = context.getOptRangeArray();
+            if (targetRanges) {
+                // reduce ranges of other existing models (this may result in removing models completely)
+                invokeModelMethod('reduceRanges', modelData.model, targetRanges);
+                // set the new target ranges at the validation model
+                modelData.model.setRanges(targetRanges);
+            }
+
+            // change all other validation attributes
+            modelData.model.setAttributes({ validation: context.operation });
+
+            // TODO: notify listeners
         };
 
         // public methods -----------------------------------------------------
@@ -439,13 +558,13 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
         /**
          * Returns the validation settings for the specified cell.
          *
-         * @param {Number[]} address
+         * @param {Address} address
          *  The address of a cell.
          *
          * @returns {Object|Null}
          *  The validation settings of the specified cell, in the following
          *  properties:
-         *  - {Array} ranges
+         *  - {RangeArray} ranges
          *      The addresses of all related cell ranges containing the same
          *      validation settings.
          *  - {Object} attributes
@@ -453,10 +572,10 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
          *  If the cell is not validated at all, returns null.
          */
         this.getValidationSettings = function (address) {
-            var entry = getEntry(address);
-            return entry ? {
-                ranges: entry.getRanges(),
-                attributes: entry.getMergedAttributes().validation
+            var valModel = findModelByAddress(address);
+            return valModel ? {
+                ranges: valModel.getRanges(),
+                attributes: valModel.getMergedAttributes().validation
             } : null;
         };
 
@@ -464,144 +583,37 @@ define('io.ox/office/spreadsheet/model/validationcollection', [
          * Returns the list contents for a data validation of type 'list' or
          * 'source' at the specified cell address.
          *
-         * @param {Number[]} address
+         * @param {Address} address
          *  The address of a cell.
          *
          * @returns {jQuery.Promise}
          *  The promise of a Deferred object that will be resolved with an
-         *  array of strings, if this collection entry is a data validation of
-         *  type 'list' or 'source', and the formula expression contained in
-         *  the attribute 'value1' could be resolved successfully to a list of
+         *  array of strings, if the data validation at the address is of type
+         *  'list' or 'source', and the formula expression contained in the
+         *  attribute 'value1' could be resolved successfully to a list of
          *  strings. Otherwise, the promise will be rejected. Additionally, the
          *  promise will contain the method 'abort()' that is able to abort a
          *  running server request. In this case, the promise will be rejected
          *  immediately.
          */
         this.queryListValues = function (address) {
-            var entry = getEntry(address);
-            return entry ? entry.queryListValues(address) : $.Deferred().reject();
-        };
-
-        /**
-         * Restricts the valid values of a range of cells according to the
-         * specified settings.
-         *
-         * @param {Array} ranges
-         *  The addresses of all cell ranges that contain the data validation
-         *  settings.
-         *
-         * @param {Object} attrs
-         *  The validation attributes.
-         *
-         * @param {Number} [index]
-         *  The zero-based insertion index for the new collection entry. If
-         *  omitted, the entry will be appended to the collection.
-         *
-         * @returns {Boolean}
-         *  Whether the validation settings have been created successfully.
-         */
-        this.insertValidation = function (ranges, attrs, index) {
-
-            var // the new collection entry
-                entry = new ValidationModel(app, sheetModel, ranges, { validation: attrs });
-
-            // reduce ranges of existing entries (this may result in removing entries completely)
-            invokeEntryMethod('reduceRanges', null, entry.getRanges());
-
-            // insert the new collection entry
-            index = _.isNumber(index) ? Utils.minMax(index, 0, entries.length) : entries.length;
-            entries.splice(index, 0, entry);
-
-            // TODO: trigger change events
-            return true;
-        };
-
-        /**
-         * Changes the ranges and/or validation attributes of an existing
-         * collection entry.
-         *
-         * @param {Number} index
-         *  The zero-based index of the collection entry to be changed.
-         *
-         * @param {Array|Null} ranges
-         *  The new addresses of the cell ranges for the collection entry. If
-         *  set to null, the original ranges will not be modified.
-         *
-         * @param {Object|Null} attrs
-         *  The new validation attributes. If set to null, the current
-         *  attributes will not be modified.
-         *
-         * @returns {Boolean}
-         *  Whether the validation settings have been changed successfully.
-         */
-        this.changeValidation = function (index, ranges, attrs) {
-
-            var // the collection entry to be modified
-                entry = entries[index];
-
-            // validate passed collection index
-            if (!_.isObject(entry)) { return false; }
-
-            // change validation ranges
-            if (_.isArray(ranges) && (ranges.length > 0)) {
-
-                // reduce ranges of other existing entries (this may result in removing entries completely)
-                invokeEntryMethod('reduceRanges', entry, ranges);
-
-                // set the new ranges at the collection entry
-                entry.setRanges(ranges);
-            }
-
-            // change all other validation attributes
-            if (_.isObject(attrs)) {
-                entry.setAttributes({ validation: attrs });
-            }
-
-            return true;
-        };
-
-        /**
-         * Deletes validation settings from this collection.
-         *
-         * @param {Number} index
-         *  The zero-based index of the collection entry to be deleted.
-         *
-         * @returns {Boolean}
-         *  Whether the validation settings have been deleted successfully.
-         */
-        this.deleteValidation = function (index) {
-
-            var // the collection entry to be deleted
-                entry = entries[index];
-
-            // validate passed collection index
-            if (!_.isObject(entry)) { return false; }
-
-            // destroy and remove the collection entry
-            entry.destroy();
-            entries.splice(index, 1);
-
-            return true;
+            var valModel = findModelByAddress(address);
+            return valModel ? valModel.queryListValues(address) : $.Deferred().reject();
         };
 
         // initialization -----------------------------------------------------
 
-        // update ranges after inserting/deleting columns or rows in the own sheet
+        // update target ranges after inserting/deleting columns or rows in the own sheet
         sheetModel.registerTransformationHandler(transformRanges);
 
         // update source formulas after inserting/deleting columns or rows in any sheet
         docModel.registerTransformationHandler(transformValues);
 
-        // clone collection entries passed as hidden argument by the clone() method
-        if (_.isArray(arguments[ValidationCollection.length])) {
-            entries = _.invoke(arguments[ValidationCollection.length], 'clone', sheetModel);
-        }
-
         // destroy all class members
         this.registerDestructor(function () {
             docModel.unregisterTransformationHandler(transformValues);
-            _.invoke(entries, 'destroy');
-            app = sheetModel = docModel = entries = null;
+            _.invoke(valModels, 'destroy');
+            docModel = sheetModel = valModels = null;
         });
 
     } // class ValidationCollection
