@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -32,9 +32,6 @@ define('io.ox/office/spreadsheet/model/viewsettingsmixin', [
      *
      * @constructor
      *
-     * @param {SpreadsheetApplication} app
-     *  The application containing this document model.
-     *
      * @param {Object} definitions
      *  A map with definition objects for all supported view attributes,
      *  containing their default values, and other settings. Each definition
@@ -42,6 +39,9 @@ define('io.ox/office/spreadsheet/model/viewsettingsmixin', [
      *  properties:
      *  - {Any} definition.def
      *      The default value for the view attribute.
+     *  - {Function} [definition.equals=_.isEqual]
+     *      A predicate that returns whether two values of the view attribute
+     *      are equal.
      *  - {Function|RegExp} [definition.validate=_.identity]
      *      A callback function used to validate a new value for the attribute,
      *      or a regular expression that can be used to validate enumeration
@@ -51,24 +51,26 @@ define('io.ox/office/spreadsheet/model/viewsettingsmixin', [
      *      identity function (accepts all attribute values). Will be called in
      *      the context of the object instance containing the view attributes.
      */
-    function ViewSettingsMixin(app, definitions) {
+    function ViewSettingsMixin(definitions) {
 
         var // self reference
             self = this,
 
             // the current view attributes of this object
-            viewAttributes = {};
+            viewAttributes = Utils.mapProperties(definitions, function (definition) { return definition.def; });
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the values of all view attribute.
          *
          * @returns {Object}
-         *  A cloned map containing all view attribute values.
+         *  A map containing all view attribute values. For performance, this
+         *  is a reference to the original attribute map. The values in this
+         *  map MUST NOT be changed (use the setter methods)!
          */
         this.getViewAttributes = function () {
-            return _.copy(viewAttributes, true);
+            return viewAttributes;
         };
 
         /**
@@ -78,10 +80,12 @@ define('io.ox/office/spreadsheet/model/viewsettingsmixin', [
          *  The name of the view attribute.
          *
          * @returns {Any}
-         *  A clone of the attribute value.
+         *  The attribute value if available, otherwise null. For performance,
+         *  this is the original attribute value. Arrays or objects MUST NOT be
+         *  changed (use the setter methods)!
          */
         this.getViewAttribute = function (name) {
-            return (name in viewAttributes) ? _.copy(viewAttributes[name], true) : null;
+            return (name in viewAttributes) ? viewAttributes[name] : null;
         };
 
         /**
@@ -103,18 +107,17 @@ define('io.ox/office/spreadsheet/model/viewsettingsmixin', [
                 if (name in definitions) {
 
                     // execute validator
-                    if (_.isFunction(definitions[name].validator)) {
-                        value = definitions[name].validator.call(self, value);
-                    } else if (_.isRegExp(definitions[name].validator)) {
-                        if (!definitions[name].validator.test(value)) { value = Utils.BREAK; }
+                    var validator = definitions[name].validate;
+                    if (_.isFunction(validator)) {
+                        value = validator.call(self, value);
+                    } else if (_.isRegExp(validator)) {
+                        if (!validator.test(value)) { value = Utils.BREAK; }
                     }
 
                     // change attribute value
-                    if (!_.isUndefined(value) && (value !== Utils.BREAK)) {
-                        if (!_.isEqual(viewAttributes[name], value)) {
-                            viewAttributes[name] = _.copy(value, true);
-                            changedAttributes[name] = value;
-                        }
+                    var equality = definitions[name].equals || _.isEqual;
+                    if (!_.isUndefined(value) && (value !== Utils.BREAK) && !equality(viewAttributes[name], value)) {
+                        viewAttributes[name] = changedAttributes[name] = value;
                     }
                 }
             });
@@ -144,8 +147,9 @@ define('io.ox/office/spreadsheet/model/viewsettingsmixin', [
 
         // initialization -----------------------------------------------------
 
-        _.each(definitions, function (definition, name) {
-            viewAttributes[name] = _.copy(definition.def, true);
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            definitions = self = viewAttributes = null;
         });
 
     } // class ViewSettingsMixin
