@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -29,11 +29,30 @@ define('io.ox/office/spreadsheet/utils/operations', [
 
         INSERT_SHEET: 'insertSheet',
         DELETE_SHEET: 'deleteSheet',
+        CHANGE_SHEET: 'setSheetAttributes',
+        RENAME_SHEET: 'setSheetName',
         MOVE_SHEET: 'moveSheet',
         COPY_SHEET: 'copySheet',
 
-        SET_SHEET_NAME: 'setSheetName',
-        SET_SHEET_ATTRIBUTES: 'setSheetAttributes',
+        INSERT_CELLS: 'insertCells',
+        DELETE_CELLS: 'deleteCells',
+        MERGE_CELLS: 'mergeCells',
+
+        SET_CELL_CONTENTS: 'setCellContents',
+        FILL_CELL_RANGE: 'fillCellRange',
+        CLEAR_CELL_RANGE: 'clearCellRange',
+        AUTO_FILL: 'autoFill',
+
+        INSERT_HYPERLINK: 'insertHyperlink',
+        DELETE_HYPERLINK: 'deleteHyperlink',
+
+        INSERT_COLUMNS: 'insertColumns',
+        DELETE_COLUMNS: 'deleteColumns',
+        CHANGE_COLUMNS: 'setColumnAttributes',
+
+        INSERT_ROWS: 'insertRows',
+        DELETE_ROWS: 'deleteRows',
+        CHANGE_ROWS: 'setRowAttributes',
 
         INSERT_NAME: 'insertName',
         CHANGE_NAME: 'changeName',
@@ -48,22 +67,12 @@ define('io.ox/office/spreadsheet/utils/operations', [
         CHANGE_VALIDATION: 'changeValidation',
         DELETE_VALIDATION: 'deleteValidation',
 
-        INSERT_COLUMNS: 'insertColumns',
-        DELETE_COLUMNS: 'deleteColumns',
-        SET_COLUMN_ATTRIBUTES: 'setColumnAttributes',
-
-        INSERT_ROWS: 'insertRows',
-        DELETE_ROWS: 'deleteRows',
-        SET_ROW_ATTRIBUTES: 'setRowAttributes',
-
-        INSERT_CELLS: 'insertCells',
-        DELETE_CELLS: 'deleteCells',
-        MERGE_CELLS: 'mergeCells',
-
-        SET_CELL_CONTENTS: 'setCellContents',
-        FILL_CELL_RANGE: 'fillCellRange',
-        CLEAR_CELL_RANGE: 'clearCellRange',
-        AUTO_FILL: 'autoFill'
+        INSERT_COND_FORMAT: 'insertCondFormat',
+        CHANGE_COND_FORMAT: 'changeCondFormat',
+        DELETE_COND_FORMAT: 'deleteCondFormat',
+        INSERT_COND_FORMAT_RULE: 'insertCondFormatRule',
+        CHANGE_COND_FORMAT_RULE: 'changeCondFormatRule',
+        DELETE_COND_FORMAT_RULE: 'deleteCondFormatRule'
 
     });
 
