@@ -6,40 +6,33 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/spreadsheet/utils/paneutils', [
     'io.ox/office/tk/utils',
-    'io.ox/office/spreadsheet/utils/sheetutils'
-], function (Utils, SheetUtils) {
+    'io.ox/office/editframework/utils/border',
+    'io.ox/office/spreadsheet/utils/range'
+], function (Utils, Border, Range) {
 
     'use strict';
 
     var // configuration for pane sides (header panes)
         PANE_SIDE_DATA = {
-            left:   { nextSide: 'right',  panePos1: 'topLeft',    panePos2: 'bottomLeft',  columns: true,  leading: true,  anchorName: 'anchorLeft'   },
-            right:  { nextSide: 'left',   panePos1: 'topRight',   panePos2: 'bottomRight', columns: true,  leading: false, anchorName: 'anchorRight'  },
-            top:    { nextSide: 'bottom', panePos1: 'topLeft',    panePos2: 'topRight',    columns: false, leading: true,  anchorName: 'anchorTop'    },
+            left:   { nextSide: 'right',  panePos1: 'topLeft',    panePos2: 'bottomLeft',  columns: true,  leading: true,  anchorName: 'anchorLeft' },
+            right:  { nextSide: 'left',   panePos1: 'topRight',   panePos2: 'bottomRight', columns: true,  leading: false, anchorName: 'anchorRight' },
+            top:    { nextSide: 'bottom', panePos1: 'topLeft',    panePos2: 'topRight',    columns: false, leading: true,  anchorName: 'anchorTop' },
             bottom: { nextSide: 'top',    panePos1: 'bottomLeft', panePos2: 'bottomRight', columns: false, leading: false, anchorName: 'anchorBottom' }
         },
 
         // configuration for pane positions (grid panes)
         PANE_POS_DATA = {
-            topLeft:     { colSide: 'left',  rowSide: 'top',    nextColPos: 'topRight',    nextRowPos: 'bottomLeft'  },
+            topLeft:     { colSide: 'left',  rowSide: 'top',    nextColPos: 'topRight',    nextRowPos: 'bottomLeft' },
             topRight:    { colSide: 'right', rowSide: 'top',    nextColPos: 'topLeft',     nextRowPos: 'bottomRight' },
-            bottomLeft:  { colSide: 'left',  rowSide: 'bottom', nextColPos: 'bottomRight', nextRowPos: 'topLeft'     },
-            bottomRight: { colSide: 'right', rowSide: 'bottom', nextColPos: 'bottomLeft',  nextRowPos: 'topRight'    }
-        },
-
-        // maps predefined border width names to line width in 1/100 mm
-        BORDER_WIDTH_MAP = {
-            hair: Utils.convertLengthToHmm(0.5, 'px'),
-            thin: Utils.convertLengthToHmm(1, 'px'),
-            medium: Utils.convertLengthToHmm(2, 'px'),
-            thick: Utils.convertLengthToHmm(3, 'px')
+            bottomLeft:  { colSide: 'left',  rowSide: 'bottom', nextColPos: 'bottomRight', nextRowPos: 'topLeft' },
+            bottomRight: { colSide: 'right', rowSide: 'bottom', nextColPos: 'bottomLeft',  nextRowPos: 'topRight' }
         },
 
         // the names of all tracking events but 'tracking:start'
@@ -51,6 +44,20 @@ define('io.ox/office/spreadsheet/utils/paneutils', [
 
     // constants --------------------------------------------------------------
 
+    /**
+     * All available pane side identifiers, as array of strings.
+     *
+     * @constant
+     */
+    PaneUtils.ALL_PANE_SIDES = _.keys(PANE_SIDE_DATA);
+
+    /**
+     * All available pane position identifiers, as array of strings.
+     *
+     * @constant
+     */
+    PaneUtils.ALL_PANE_POSITIONS = _.keys(PANE_POS_DATA);
+
     /**
      * The minimum size of grid/header panes, in pixels.
      *
@@ -58,6 +65,21 @@ define('io.ox/office/spreadsheet/utils/paneutils', [
      */
     PaneUtils.MIN_PANE_SIZE = 10;
 
+    /**
+     * The ratio of the header/grid pane size to be rendered additionally
+     * before and after the visible area.
+     *
+     * @constant
+     */
+    PaneUtils.ADDITIONAL_SIZE_RATIO = Utils.RETINA ? 0.25 : Utils.COMPACT_DEVICE ? 0.3 : 0.5;
+
+    /**
+     * All predefined zoom factor steps.
+     *
+     * @constant
+     */
+    PaneUtils.ZOOM_FACTORS = [0.5, 0.75, 1, 1.25, 1.5, 1.75, 2, 2.5, 3, 4, 5, 6, 8];
+
     /**
      * Default settings for mouse/touch tracking in header and grid panes.
      *
@@ -234,47 +256,6 @@ define('io.ox/office/spreadsheet/utils/paneutils', [
         return 'left:' + rectangle.left + 'px;top:' + rectangle.top + 'px;width:' + rectangle.width + 'px;height:' + rectangle.height + 'px;';
     };
 
-    /**
-     * Returns the effective CSS text alignment of the passed cell data,
-     * according to the cell result value and the alignment formatting
-     * attributes.
-     *
-     * @param {Object} cellData
-     *  The cell descriptor object, as received from the cell collection.
-     *
-     * @param {Boolean} [editMode=false]
-     *  If set to true, formula cells will be left-aligned (used for example in
-     *  in-place cell edit mode).
-     *
-     * @returns {String}
-     *  The effective horizontal CSS text alignment ('left', 'center', 'right',
-     *  or 'justify').
-     */
-    PaneUtils.getCssTextAlignment = function (cellData, editMode) {
-
-        switch (cellData.attributes.cell.alignHor) {
-        case 'left':
-        case 'center':
-        case 'right':
-        case 'justify':
-            return cellData.attributes.cell.alignHor;
-        case 'auto':
-            // formulas in cell edit mode always left aligned, regardless of result type
-            if (editMode && _.isString(cellData.formula)) { return 'left'; }
-            // strings (left aligned)
-            if (_.isString(cellData.result)) { return 'left'; }
-            // numbers (right aligned)
-            if (_.isNumber(cellData.result)) { return 'right'; }
-            // booleans and error codes (centered)
-            if (_.isBoolean(cellData.result) || SheetUtils.isErrorCode(cellData.result)) { return 'center'; }
-            // empty cells (left aligned)
-            return 'left';
-        }
-
-        Utils.warn('PaneUtils.getTextAlignment(): unknown alignment attribute value "' + cellData.attributes.cell.alignHor + '"');
-        return 'left';
-    };
-
     /**
      * Returns the identifier of a predefined border style for the passed mixed
      * border object.
@@ -303,15 +284,8 @@ define('io.ox/office/spreadsheet/utils/paneutils', [
             return null;
         }
 
-        var // the width of the border lines, in pixels
-            pixels = Utils.convertHmmToLength(mixedBorder.width, 'px');
-
         // concatenate the line style and the line width into a single identifier
-        return mixedBorder.style + ':' + (
-            (pixels >= 2.5) ? 'thick' :
-            (pixels >= 1.5) ? 'medium' :
-            (pixels >= 0.75) ? 'thin' :
-            'hair');
+        return mixedBorder.style + ':' + Border.getPresetForWidth(mixedBorder.width);
     };
 
     /**
@@ -334,9 +308,9 @@ define('io.ox/office/spreadsheet/utils/paneutils', [
             // resulting border value
             border = { style: 'none', width: 0 };
 
-        if ((tokens.length === 2) && (tokens[0].length > 0) && (tokens[1] in BORDER_WIDTH_MAP)) {
+        if ((tokens.length === 2) && (tokens[0].length > 0) && (tokens[1].length > 0)) {
             border.style = tokens[0];
-            border.width = BORDER_WIDTH_MAP[tokens[1]];
+            border.width = Border.getWidthForPreset(tokens[1]);
         } else if (presetStyle !== 'none') {
             Utils.warn('PaneUtils.getBorderForPresetStyle(): invalid preset style "' + presetStyle + '"');
         }
@@ -443,8 +417,128 @@ define('io.ox/office/spreadsheet/utils/paneutils', [
         return 'select';
     };
 
+    // class HeaderBoundary ===================================================
+
+    /**
+     * Represents a bounding column/row interval in a header pane, together
+     * with the exact pixel position of the bounding area. This pixel position
+     * may not be aligned exactly to the start position of the first
+     * column/row, or the end position of the last column/row in the interval.
+     *
+     * @constructor
+     *
+     * @property {Interval|Null} interval
+     *  The column/row indexes of the bounding interval; or null, if this
+     *  instance represents an invalid (hidden) header boundary.
+     *
+     * @property {Object|Null} position
+     *  The exact position and size of the bounding area, in pixels (in the
+     *  properties 'offset' and 'size'). May result in a start position before
+     *  or inside the first column/row of the interval, or in an end position
+     *  after or inside the last column/row.
+     */
+    function HeaderBoundary(interval, position) {
+
+        if (interval && position) {
+            this.interval = interval.clone();
+            this.position = _.clone(position);
+        } else {
+            this.interval = this.position = null;
+        }
+
+    } // class HeaderBoundary
+
+    // public methods ---------------------------------------------------------
+
+    /**
+     * Returns a deep clone of this instance.
+     *
+     * @returns {HeaderBoundary}
+     *  A deep clone of this instance.
+     */
+    HeaderBoundary.prototype.clone = function () {
+        return new HeaderBoundary(this.interval, this.position);
+    };
+
+    /**
+     * Returns whether this instance contains a valid interval and position
+     * (neither of the properties is null).
+     *
+     * @returns {Boolean}
+     *  Whether this instance contains a valid interval and position.
+     */
+    HeaderBoundary.prototype.isValid = function () {
+        return _.isObject(this.interval) && _.isObject(this.position);
+    };
+
+    // class RangeBoundary ====================================================
+
+    /**
+     * Represents a bounding range in a grid pane, together with the exact
+     * pixel position of the bounding rectangle. This pixel position may not be
+     * aligned exactly to the outer borders of the bounding range.
+     *
+     * @constructor
+     *
+     * @property {Range|Null} range
+     *  The address of the bounding range; or null, if this instance represents
+     *  an invalid (hidden) range boundary.
+     *
+     * @property {Object|Null} rectangle
+     *  The exact pixel position of the bounding rectangle, in pixels (in the
+     *  properties 'left', 'top', 'width', and 'height'). May result in a start
+     *  position before or inside the start cell of the range, or in an end
+     *  position after or inside the last cell.
+     *
+     * @param {HeaderBoundary|Null} colBoundary
+     *  The column interval and position represented by this instance.
+     *
+     * @param {HeaderBoundary|Null} rowBoundary
+     *  The row interval and position represented by this instance.
+     */
+    function RangeBoundary(colBoundary, rowBoundary) {
+
+        if (colBoundary && colBoundary.isValid() && rowBoundary && rowBoundary.isValid()) {
+            this.range = Range.createFromIntervals(colBoundary.interval, rowBoundary.interval);
+            this.rectangle = { left: colBoundary.position.offset, top: rowBoundary.position.offset, width: colBoundary.position.size, height: rowBoundary.position.size };
+        } else {
+            this.range = this.rectangle = null;
+        }
+
+    } // class RangeBoundary
+
+    // public methods ---------------------------------------------------------
+
+    /**
+     * Returns a (deep) clone of this instance.
+     *
+     * @returns {RangeBoundary}
+     *  A (deep) clone of this instance.
+     */
+    RangeBoundary.prototype.clone = function () {
+        var clone = new RangeBoundary();
+        clone.range = this.range ? this.range.clone() : null;
+        clone.rectangle = _.clone(this.rectangle);
+        return clone;
+    };
+
+    /**
+     * Returns whether this instance contains a valid range address and pixel
+     * position (neither of the properties is null).
+     *
+     * @returns {Boolean}
+     *  Whether this instance contains a valid range address and rectangle.
+     */
+    RangeBoundary.prototype.isValid = function () {
+        return _.isObject(this.range) && _.isObject(this.rectangle);
+    };
+
     // exports ================================================================
 
+    // export helper classes
+    PaneUtils.HeaderBoundary = HeaderBoundary;
+    PaneUtils.RangeBoundary = RangeBoundary;
+
     return PaneUtils;
 
 });
