@@ -6,89 +6,107 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/view/chartcreator',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/spreadsheet/utils/operations',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/model/formula/tokenarray',
-     'io.ox/office/drawinglayer/view/chartstyleutil',
-     'gettext!io.ox/office/spreadsheet'
-    ], function (Utils, Operations, SheetUtils, TokenArray, ChartStyleUtil, gt) {
+define('io.ox/office/spreadsheet/view/chartcreator', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/utils/operations',
+    'io.ox/office/spreadsheet/model/formula/tokenarray',
+    'io.ox/office/drawinglayer/view/chartstyleutil',
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, SheetUtils, Operations, TokenArray, ChartStyleUtil, gt) {
 
     'use strict';
 
-    var // default width of new chart objects
+    var // convenience shortcuts
+        Address = SheetUtils.Address,
+        Range = SheetUtils.Range,
+        Range3D = SheetUtils.Range3D,
+        Range3DArray = SheetUtils.Range3DArray,
+
+        // default width of new chart objects
         CHART_DEFAULT_WIDTH = Utils.convertHmmToLength(15000, 'px', 1),
 
         // default height of new chart objects
         CHART_DEFAULT_HEIGHT = Utils.convertHmmToLength(10000, 'px', 1);
 
-    var DEFAULT_TEXTCOLOR = {type: 'auto'};
-    var DEFAULT_LINECOLOR = {transformations: [{type: 'lumMod', value: 15000}, {type: 'lumOff',value: 85000}],type: 'scheme', value: 'text1'};
-    var STANDARD_SHAPE = {type: 'solid', color: DEFAULT_LINECOLOR, width: 36};
-    var NONE_SHAPE = {type: 'none'};
-    var STANDARD_CHAR = {color: DEFAULT_TEXTCOLOR};
-    var HEAD_CHAR = {color: DEFAULT_TEXTCOLOR, fontSize: 16};
+    var DEFAULT_LINECOLOR = { transformations: [{ type: 'lumMod', value: 15000 }, { type: 'lumOff', value: 85000 }], type: 'scheme', value: 'text1' };
+    var STANDARD_SHAPE = { type: 'solid', color: DEFAULT_LINECOLOR, width: 36 };
+    var NONE_SHAPE = { type: 'none' };
+    var HEAD_CHAR = { fontSize: 16 };
 
     var INSERT_DEFAULTS = (function () {
 
-        var AXIS_ENABLED =  { axis: {label: true},  line: NONE_SHAPE, character: STANDARD_CHAR};
-        var AXIS_DISABLED = { axis: {label: false}, line: NONE_SHAPE};
-        var GRID_ENABLED =  { line: STANDARD_SHAPE};
-        var GRID_DISABLED = { line: NONE_SHAPE};
+        var AXIS_ENABLED =  { axis: { label: true },  line: NONE_SHAPE };
+        var AXIS_DISABLED = { axis: { label: false }, line: NONE_SHAPE };
+        var GRID_ENABLED =  { line: STANDARD_SHAPE };
+        var GRID_DISABLED = { line: NONE_SHAPE };
 
         return {
             normal: {
                 xAxis: AXIS_ENABLED,
                 xGrid: GRID_DISABLED,
                 yAxis: AXIS_ENABLED,
-                yGrid: GRID_ENABLED,
+                yGrid: GRID_ENABLED
             },
             xy: {
                 xAxis: AXIS_ENABLED,
                 xGrid: GRID_ENABLED,
                 yAxis: AXIS_ENABLED,
-                yGrid: GRID_ENABLED,
+                yGrid: GRID_ENABLED
             },
             pie: {
                 xAxis: AXIS_DISABLED,
                 xGrid: GRID_DISABLED,
                 yAxis: AXIS_DISABLED,
-                yGrid: GRID_DISABLED,
+                yGrid: GRID_DISABLED
             }
         };
     }());
 
-    function getFormula(app, sheet, from, to) {
+    function getFormula(docModel, sheet, from, to) {
         if (!from) {
             throw 'no from assigned';
         }
-        var range = { start: from, end: to || from };
-        var tokenArray = new TokenArray(app, null, { trigger: 'never' });
+        var range = new Range(from, to);
+        var tokenArray = new TokenArray(docModel, { temp: true });
         tokenArray.appendRange(range, { sheet: sheet, abs: true });
-        var formula = tokenArray.getFormula();
+        var formula = tokenArray.getFormula('op');
         tokenArray.destroy();
 
         return formula;
     }
 
-    function addDataSeries(app, generator, sourceSheet, position, series, keyFrom, keyTo, title, valueFrom, valueTo, bubbleFrom, bubbleTo, bubbleArray) {
+    function addDataSeries(docModel, generator, options) {
+
+        var sourceSheet = options.sourceSheet;
+        var position = options.position;
+        var series = options.series;
+        var keyFrom = options.keyFrom;
+        var keyTo = options.keyTo;
+        var title = options.title;
+        var valueFrom = options.valueFrom;
+        var valueTo = options.valueTo;
+        var bubbleFrom = options.bubbleFrom;
+        var bubbleTo = options.bubbleTo;
+        var bubbleArray = options.bubbleArray;
+        var lineAttrs = options.lineAttrs;
+
         var insert = {
-            values: getFormula(app, sourceSheet, valueFrom, valueTo)
+            values: getFormula(docModel, sourceSheet, valueFrom, valueTo)
         };
         if (title) {
-            insert.title = getFormula(app, sourceSheet, title);
+            insert.title = getFormula(docModel, sourceSheet, title);
         }
         if (keyFrom) {
-            insert.names = getFormula(app, sourceSheet, keyFrom, keyTo);
+            insert.names = getFormula(docModel, sourceSheet, keyFrom, keyTo);
         }
         if (bubbleFrom) {
-            insert.bubbles = getFormula(app, sourceSheet, bubbleFrom, bubbleTo);
+            insert.bubbles = getFormula(docModel, sourceSheet, bubbleFrom, bubbleTo);
         } else if (bubbleArray) {
             insert.bubbles = bubbleArray;
         }
@@ -96,14 +114,14 @@ define('io.ox/office/spreadsheet/view/chartcreator',
             series: series,
             attrs: {
                 series: insert,
-                line: {type: 'solid'},
-                fill: {type: 'solid'},
+                line: lineAttrs || { type: 'solid' },
+                fill: { type: 'solid' }
             }
         };
         generator.generateDrawingOperation(Operations.INSERT_CHART_DATASERIES, position, properties);
     }
 
-    function generateSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, axis, chart) {
+    function generateSeriesOperations(docModel, sourceRange, sourceSheet, generator, position, cells, axis, chart, lineAttrs) {
 
         if (_.isUndefined(chart.varyColors)) {
             chart.varyColors = true;
@@ -135,9 +153,9 @@ define('io.ox/office/spreadsheet/view/chartcreator',
 
         var seriesCount;
         if (chart.type && chart.type.indexOf('bubble') === 0) {
-            seriesCount = generateBubbleSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, useAxis);
+            seriesCount = generateBubbleSeriesOperations(docModel, sourceRange, sourceSheet, generator, position, cells, useAxis, lineAttrs);
         } else {
-            seriesCount = generateStandardSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, useAxis);
+            seriesCount = generateStandardSeriesOperations(docModel, sourceRange, sourceSheet, generator, position, cells, useAxis, lineAttrs);
         }
         if ((seriesCount === 1 && !chart.varyColors) || (chart.type === 'scatter2d' || chart.type === 'line2d')) {
             var send = {};
@@ -155,10 +173,10 @@ define('io.ox/office/spreadsheet/view/chartcreator',
      * makes an address array, needed for the code, which does not know if its row or column oriented
      */
     function makeAddress(activeAxis, otherAxis, activeValue, otherValue) {
-        var res = [];
-        res[activeAxis] = activeValue;
-        res[otherAxis] = otherValue;
-        return res;
+        var address = Address.A1.clone();
+        address[activeAxis] = activeValue;
+        address[otherAxis] = otherValue;
+        return address;
     }
 
     /**
@@ -170,13 +188,13 @@ define('io.ox/office/spreadsheet/view/chartcreator',
      *                         second is a Boolean for the cell right of the first cell
      *  @param activeAxis [1 or 0] column or row for the direction of any series data
      */
-    function generateStandardSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, activeAxis) {
+    function generateStandardSeriesOperations(docModel, sourceRange, sourceSheet, generator, position, cells, activeAxis, lineAttrs) {
 
         var start = sourceRange.start;
         var end = sourceRange.end;
 
         var max = end[activeAxis] - start[activeAxis];
-        var otherAxis = (activeAxis - 1) * -1;
+        var otherAxis = 1 - activeAxis;
 
         var keyFrom, keyTo;
 
@@ -208,7 +226,17 @@ define('io.ox/office/spreadsheet/view/chartcreator',
             }
             var valueTo = makeAddress(activeAxis, otherAxis, activeIndex, end[otherAxis]);
 
-            addDataSeries(app, generator, sourceSheet, position, i, keyFrom, keyTo, title, valueFrom, valueTo);
+            addDataSeries(docModel, generator, {
+                sourceSheet: sourceSheet,
+                position: position,
+                series: i,
+                keyFrom: keyFrom,
+                keyTo: keyTo,
+                title: title,
+                valueFrom: valueFrom,
+                valueTo: valueTo,
+                lineAttrs: lineAttrs
+            });
         }
         return max;
     }
@@ -225,7 +253,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
      *                         second is a Boolean for the cell right of the first cell
      *  @param activeAxis [1 or 0] column or row for the direction of any series data
      */
-    function generateBubbleSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, activeAxis) {
+    function generateBubbleSeriesOperations(docModel, sourceRange, sourceSheet, generator, position, cells, activeAxis, lineAttrs) {
 
         var start = sourceRange.start;
         var end = sourceRange.end;
@@ -274,7 +302,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
             var bubbleTo = null;
             var bubbleArray = null;
 
-            if (i===max-1) {
+            if (i === max - 1) {
                 bubbleArray = [];
                 var arraySize = 1 + (valueTo[otherAxis] - valueFrom[otherAxis]);
                 for (var j = 0; j < arraySize; j++) {
@@ -285,106 +313,126 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                 bubbleTo = makeAddress(activeAxis, otherAxis, valueTo[activeAxis] + 1, valueTo[otherAxis]);
             }
 
-            addDataSeries(app, generator, sourceSheet, position, size, keyFrom, keyTo, title, valueFrom, valueTo, bubbleFrom, bubbleTo, bubbleArray);
+            addDataSeries(docModel, generator, {
+                sourceSheet: sourceSheet,
+                position: position,
+                series: size,
+                keyFrom: keyFrom,
+                keyTo: keyTo,
+                title: title,
+                valueFrom: valueFrom,
+                valueTo: valueTo,
+                bubbleFrom: bubbleFrom,
+                bubbleTo: bubbleTo,
+                bubbleArray: bubbleArray,
+                lineAttrs: lineAttrs
+            });
             size++;
         }
         return size;
     }
 
-    function getContentForRange(app, sourceRange, sourceSheet, def, follow, forceColLabel, forceRowLabel) {
+    function getContentForRange(docModel, docView, sourceRange, sourceSheet, callback, forceColLabel, forceRowLabel) {
 
-        var sheetModel = app.getModel().getSheetModel(sourceSheet);
+        var sheetModel = docModel.getSheetModel(sourceSheet);
 
         var start = sourceRange.start;
-        var cols = SheetUtils.getColCount(sourceRange);
-        var rows = SheetUtils.getRowCount(sourceRange);
+        var cols = sourceRange.cols();
+        var rows = sourceRange.rows();
 
         if (cols > 50 || rows > 50) {
-            app.getView().yell({ type: 'warning', message: gt('It is not possible to create a chart out of more than 50 input cells.') });
-            if (def) {
-                def.reject();
+            docView.yell({ type: 'warning', message: gt('It is not possible to create a chart out of more than 50 input cells.') });
+            return $.Deferred().reject();
+        }
+
+        var rightEntry = sheetModel.getColCollection().getNextVisibleEntry(start[0] + 1);
+        var bottEntry = sheetModel.getRowCollection().getNextVisibleEntry(start[1] + 1);
+
+        var rightAddress = new Address(rightEntry ? rightEntry.index : start[0], start[1]);
+        var bottAddress = new Address(start[0], bottEntry ? bottEntry.index : start[1]);
+
+        var ranges = new Range3DArray(
+                Range3D.createFromAddress(bottAddress, sourceSheet),
+                Range3D.createFromAddress(rightAddress, sourceSheet),
+                Range3D.createFromAddress(start, sourceSheet)
+            );
+
+        var promise = docModel.queryCellContents([ranges], { attributes: true });
+
+        promise = promise.then(function (res) {
+
+            var bottom = false;
+            var right = false;
+            var axis = null;
+
+            // first array element of the result is an array with the contents of the three cells
+            var bottomCell = res[0][0];
+            var rightCell = res[0][1];
+            var startCell = res[0][2];
+
+            function isDate(cellContents) {
+                return (/^(date|time|datetime)$/).test(cellContents.format.cat);
             }
-        } else {
-            var bottEntry = sheetModel.getRowCollection().getNextVisibleEntry(start[1] + 1);
-            var rightEntry = sheetModel.getColCollection().getNextVisibleEntry(start[0] + 1);
 
-            var bottAddress = [start[0], bottEntry ? bottEntry.index : start[1]];
-            var rightAddress = [rightEntry ? rightEntry.index : start[0], start[1]];
+            function cellIsLabel(cell) {
+                if (!startCell) { return false; }
+                if (!startCell.display) { return true; } // empty strings (blank cells) or null values (invalid number format)
+                return isDate(cell) || (cell ? !_.isNumber(cell.result) : false);
+            }
 
-            var rangesArray = [[
-                    { sheet: sourceSheet, start: bottAddress, end: bottAddress },
-                    { sheet: sourceSheet, start: rightAddress, end: rightAddress },
-                    { sheet: sourceSheet, start: start, end: start }
-                ]];
-
-            app.getModel().queryCellContents(rangesArray, { attributes: true }).done(function (res) {
-
-                var bottom = false;
-                var right = false;
-                var axis = null;
-
-                // first array element of the result is an array with the contents of the three cells
-                var bottomCell = res[0][0];
-                var rightCell = res[0][1];
-                var startCell = res[0][2];
-
-                function isDate(cellContents) {
-                    return (/^(date|time|datetime)$/).test(cellContents.format.cat);
-                }
-
-                function cellIsLabel(cell) {
-                    if (!startCell) { return false; }
-                    if (!startCell.display.length) { return true; }
-                    return isDate(cell) || (cell ? !_.isNumber(cell.result) : false);
-                }
-
-                if (cols === 1 || rows === 1) {
-                    var firstString = startCell ? !_.isNumber(startCell.result) : false;
-                    if (cols === 1) {
-                        axis = 0;
-                        if (firstString) {
-                            right = true;
-                        }
-                    } else {
-                        axis = 1;
-                        if (firstString) {
-                            bottom = true;
-                        }
+            if (cols === 1 || rows === 1) {
+                var firstString = startCell ? !_.isNumber(startCell.result) : false;
+                if (cols === 1) {
+                    axis = 0;
+                    if (firstString) {
+                        right = true;
                     }
                 } else {
-                    bottom = cellIsLabel(bottomCell);
-                    right = cellIsLabel(rightCell);
+                    axis = 1;
+                    if (firstString) {
+                        bottom = true;
+                    }
                 }
+            } else {
+                bottom = cellIsLabel(bottomCell);
+                right = cellIsLabel(rightCell);
+            }
 
-                if (!_.isUndefined(forceColLabel)) {
-                    bottom = forceColLabel;
-                }
-                if (!_.isUndefined(forceRowLabel)) {
-                    right = forceRowLabel;
-                }
-                follow([bottom, right], axis);
-            });
-        }
+            if (!_.isUndefined(forceColLabel)) {
+                bottom = forceColLabel;
+            }
+            if (!_.isUndefined(forceRowLabel)) {
+                right = forceRowLabel;
+            }
+
+            return callback([bottom, right], axis);
+        });
     }
 
-    // class ChartCreator =====================================================
+    // static class ChartCreator ==============================================
+
+    var ChartCreator = {};
+
+    // static methods ---------------------------------------------------------
 
     /**
-     * ChartCreator makes an insert-Chart-Op and Insert-Chart-DataSeries chosen by current Selection
-     * and changes the Selection to a Multiselection by title, names and values
+     * Creates and inserts a new chart into the active sheet.
+     *
+     * @returns {jQuery.Promise}
+     *  A promise that will be resolved after the chart has been created and
+     *  inserted successfully, or that will be rejected on any error.
      */
-    function ChartCreator(app, chart) {
+    ChartCreator.createChart = function (app, chart) {
+
+        var docModel = app.getModel();
         var docView = app.getView();
         var gridPane = docView.getActiveGridPane();
 
-        var def = $.Deferred();
-
         var drawingCollection = docView.getDrawingCollection();
-        var selection = docView.getSelection();
 
-        var range = selection.ranges[selection.activeRange];
+        var range = docView.getActiveRange();
 
-        getContentForRange(app, range, docView.getActiveSheet(), def, function (cells, axis) {
+        return getContentForRange(docModel, docView, range, docView.getActiveSheet(), function (cells, axis) {
 
             var rect = gridPane.getVisibleRectangle();
 
@@ -395,11 +443,11 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                 height: CHART_DEFAULT_HEIGHT
             };
 
-            var attrs = drawingCollection.getAttributesForRectangle(clip);
+            var attrs = drawingCollection.getAttributeSetForRectangle(clip);
 
-            docView.insertDrawing('chart', attrs, function (generator, sheet, position) {
+            return docView.insertDrawing('chart', attrs, function (generator, sheet, position) {
 
-                var seriesCount = generateSeriesOperations(app, range, sheet, generator, position, cells, axis, chart);
+                var seriesCount = generateSeriesOperations(docModel, range, sheet, generator, position, cells, axis, chart);
 
                 var legendPos;
                 if (seriesCount === 1) {
@@ -409,7 +457,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                 } else {
                     legendPos = 'bottom';
                 }
-                generator.generateDrawingOperation(Operations.SET_CHART_LEGEND_ATTRIBUTES, position, { attrs: { legend: { pos: legendPos}, character: STANDARD_CHAR } });
+                generator.generateDrawingOperation(Operations.SET_CHART_LEGEND_ATTRIBUTES, position, { attrs: { legend: { pos: legendPos } } });
 
                 var defaults;
                 switch (chart.type) {
@@ -436,24 +484,11 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                 generator.generateDrawingOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, position, { axis: 'x', attrs: defaults.xGrid });
                 generator.generateDrawingOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, position, { axis: 'y', attrs: defaults.yGrid });
             });
-            def.resolve();
         });
-
-
-        /**
-         * the deferred Object handles noting at the Moment,
-         * its just for Future cases, when the situation could be changed
-         * @return {Jquery|Deferred}
-         */
-        this.deferred = function () {
-            return def;
-        };
-    }
-
+    };
 
     /**
      *
-     * @param app
      * @param range whole start to end area of the new series
      * @param sheet
      * @param chartModel is used for the position and id in the document and as fallback for chart-type-checks
@@ -461,57 +496,104 @@ define('io.ox/office/spreadsheet/view/chartcreator',
      * @param chartAttrs is used for changing other attributes of the chart and for the type info (bubble or not)
      * @param forceColLabel is set, it overwrites the internal logic for detecting first column as label
      * @param forceRowLabel is set, it overwrites the internal logic for detecting first column as label (headline)
+     *
+     * @returns {jQuery.Promise}
+     *  A promise that will be resolved after the chart has been changed
+     *  successfully, or that will be rejected on any error.
      */
-    ChartCreator.generateOperations = function (app, sourceRange, sourceSheet, chartModel, axis, chartAttrs, forceColLabel, forceRowLabel) {
+    ChartCreator.updateSeries = function (app, sourceRange, sourceSheet, chartModel, axis, chartAttrs, forceColLabel, forceRowLabel) {
 
-        var chart;
-        if (chartAttrs) {
-            chart = chartAttrs;
-        } else {
-            var attrs = chartModel.getMergedAttributes();
-            var type = attrs.chart.type;
+        if (!chartAttrs) {
+            var mergedAttributes = chartModel.getMergedAttributes();
             //CURVED is a workaround, filter has not all the information it needs!
-            chart = {
-                type: type,
-                curved: attrs.chart.curved
+            chartAttrs = {
+                type: mergedAttributes.chart.type,
+                curved: mergedAttributes.chart.curved
             };
         }
 
-        getContentForRange(app, sourceRange, sourceSheet, null, function (cells, overwrittenAxis) {
+        var docModel = app.getModel(),
+            docView = app.getView();
+
+        return getContentForRange(docModel, docView, sourceRange, sourceSheet, function (cells, overwrittenAxis) {
 
             //only overwritten if source range height is 1 oder width is 1
             if (overwrittenAxis === 0 || overwrittenAxis === 1) {
                 axis = overwrittenAxis;
             }
-            var docModel = app.getModel();
-            var generator = chartModel.getSheetModel().createOperationsGenerator();
             var position = chartModel.getSheetPosition();
+            var lineAttrs = null;
+            if (chartModel.isMarkerOnly()) {
+                lineAttrs = NONE_SHAPE;
+            }
 
-            _.times(chartModel.getSeriesCount(), function () {
-                generator.generateDrawingOperation(Operations.DELETE_CHART_DATASERIES, position, { series: 0 });
+            return chartModel.getSheetModel().createAndApplyOperations(function (generator) {
+
+                var seriesCount = generateSeriesOperations(docModel, sourceRange, sourceSheet, generator, position, cells, axis, chartAttrs, lineAttrs);
+
+                _.times(chartModel.getSeriesCount(), function () {
+                    generator.generateDrawingOperation(Operations.DELETE_CHART_DATASERIES, position, { series: seriesCount });
+                });
             });
-
-            generateSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, axis, chart);
-            docModel.applyOperations(generator.getOperations());
         }, forceColLabel, forceRowLabel);
 
     };
 
-    ChartCreator.getStandardShape = function() {
+    ChartCreator.generateOperationsFromModelData = function (app, generator, position, chartModelData) {
+        var chartAttrs = {
+            chart: chartModelData.attrs.chart,
+            fill: chartModelData.attrs.chart,
+            line: chartModelData.attrs.line
+        };
+        var axes = chartModelData.axes;
+        var series = chartModelData.series;
+        var title = chartModelData.title;
+        var legend = chartModelData.legend;
+
+        generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, position, { attrs: chartAttrs });
+
+        _.each(series, function (serie, index) {
+            var properties = {
+                series: index,
+                attrs: serie
+            };
+            generator.generateDrawingOperation(Operations.INSERT_CHART_DATASERIES, position, properties);
+        });
+
+        _.each(axes, function (axisData, name) {
+            if (!_.isEmpty(axisData.axis)) {
+                generator.generateDrawingOperation(Operations.SET_CHART_AXIS_ATTRIBUTES, position, { axis: name, attrs: axisData.axis });
+            }
+            if (!_.isEmpty(axisData.grid)) {
+                generator.generateDrawingOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, position, { axis: name, attrs: axisData.grid });
+            }
+            if (!_.isEmpty(axisData.title)) {
+                generator.generateDrawingOperation(Operations.SET_CHART_TITLE_ATTRIBUTES, position, { axis: name, attrs: axisData.title });
+            }
+        });
+
+        if (!_.isEmpty(title)) {
+            generator.generateDrawingOperation(Operations.SET_CHART_TITLE_ATTRIBUTES, position, { axis: 'main', attrs: title });
+        }
+
+        if (!_.isEmpty(legend)) {
+            generator.generateDrawingOperation(Operations.SET_CHART_LEGEND_ATTRIBUTES, position, { attrs: legend });
+        }
+
+    };
+
+    ChartCreator.getStandardShape = function () {
         return STANDARD_SHAPE;
     };
 
-    ChartCreator.getStandardChar = function() {
-        return STANDARD_CHAR;
-    };
-
-    ChartCreator.getHeadChar = function() {
+    ChartCreator.getHeadChar = function () {
         return HEAD_CHAR;
     };
 
-    ChartCreator.getNoneShape = function() {
+    ChartCreator.getNoneShape = function () {
         return NONE_SHAPE;
     };
+
     // exports ================================================================
 
     return ChartCreator;
