@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,10 +14,11 @@
 define('io.ox/office/spreadsheet/view/control/activesheetgroup', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/forms',
+    'io.ox/office/tk/utils/tracking',
     'io.ox/office/tk/control/radiogroup',
     'io.ox/office/spreadsheet/view/popup/sheetcontextmenu',
-    'gettext!io.ox/office/spreadsheet'
-], function (Utils, Forms, RadioGroup, SheetContextMenu, gt) {
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, Forms, Tracking, RadioGroup, SheetContextMenu, gt) {
 
     'use strict';
 
@@ -30,22 +31,30 @@ define('io.ox/office/spreadsheet/view/control/activesheetgroup', [
      * @constructor
      *
      * @extends RadioGroup
+     *
+     * @param {SpreadsheetView} docView
+     *  The spreadsheet view containing this instance.
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters. Will be passed to the base class RadioGroup.
      */
-    function ActiveSheetGroup(app) {
+    function ActiveSheetGroup(docView) {
 
         var // self reference
             self = this,
 
-            // get application model and view
-            docModel = null,
-            docView = null,
+            // the document model
+            docModel = docView.getDocModel(),
 
             // the context menu with additional sheet actions
             contextMenu = null,
 
             // the scroll button nodes
-            prevButton = $(Forms.createButtonMarkup({ icon: 'fa-angle-left', tooltip: gt('Scroll to the previous sheet') })).addClass('scroll-button'),
-            nextButton = $(Forms.createButtonMarkup({ icon: 'fa-angle-right', tooltip: gt('Scroll to the next sheet') })).addClass('scroll-button'),
+            prevButton = $(Forms.createButtonMarkup({ icon: 'fa-angle-left' })).addClass('scroll-button'),
+            nextButton = $(Forms.createButtonMarkup({ icon: 'fa-angle-right' })).addClass('scroll-button'),
+
+            // keep track of active sheet index (scroll to active after change)
+            activeIndex = -1,
 
             // index of the first and last sheet tab button currently visible
             firstIndex = 0,
@@ -197,10 +206,11 @@ define('io.ox/office/spreadsheet/view/control/activesheetgroup', [
 
             // scroll backwards, if the active sheet is located before the first visible;
             // or scroll forwards, if the active sheet is located after the last visible
-            if ((index < firstIndex) || (index > lastIndex)) {
+            if ((activeIndex !== index) && ((index < firstIndex) || (index > lastIndex))) {
                 firstIndex = index;
                 renderSheetButtons();
             }
+            activeIndex = index;
         }
 
         /**
@@ -208,7 +218,7 @@ define('io.ox/office/spreadsheet/view/control/activesheetgroup', [
          * renaming the sheet.
          */
         function doubleClickHandler() {
-            app.getController().executeItem('sheet/rename/dialog');
+            docView.executeControllerItem('sheet/rename/dialog');
         }
 
         /**
@@ -245,7 +255,7 @@ define('io.ox/office/spreadsheet/view/control/activesheetgroup', [
                 oldLastIndex = lastIndex;
 
             // move to next sheets until the index of the last sheet has changed
-            while ((0 <= lastIndex) && (lastIndex + 1 < sheetButtons.length) && (oldLastIndex === lastIndex)) {
+            while ((lastIndex >= 0) && (lastIndex + 1 < sheetButtons.length) && (oldLastIndex === lastIndex)) {
                 firstIndex += 1;
                 renderSheetButtons();
             }
@@ -299,7 +309,7 @@ define('io.ox/office/spreadsheet/view/control/activesheetgroup', [
 
             // do nothing in read-only mode
             if (!docModel.getEditMode()) {
-                $.cancelTracking();
+                Tracking.cancelTracking();
                 return;
             }
 
@@ -471,33 +481,27 @@ define('io.ox/office/spreadsheet/view/control/activesheetgroup', [
 
         // initialization -----------------------------------------------------
 
-        // initialization when application is ready (model/view available)
-        app.onInit(function () {
+        // set tool tips for scroll buttons
+        Forms.setToolTip(prevButton, { tooltip: gt('Scroll to the previous sheet') });
+        Forms.setToolTip(nextButton, { tooltip: gt('Scroll to the next sheet') });
 
-            // get application model and view instances
-            docModel = app.getModel();
-            docView = app.getView();
+        // register handler early for fast preview during import
+        this.listenTo(docView, 'change:sheets', fillList);
 
-            // register handler early for fast preview during import
-            self.listenTo(docView, 'change:sheets', fillList);
-
-            // rebuild the list of available sheets after it has been changed
-            self.listenTo(app.getImportPromise(), 'done', function () {
-                fillList();
-                self.listenTo(docModel, 'change:sheet:attributes', fillList); // e.g. sheet protection
-            });
-
-            // cancel button drag&drop when losing edit rights
-            self.listenTo(docModel, 'change:editmode', function (event, editMode) {
-                if (!editMode && dndButtonNode) {
-                    $.cancelTracking();
-                }
-            });
-
-            // create the context menu
-            contextMenu = new SheetContextMenu(app, self.getNode());
+        // rebuild the list of available sheets after it has been changed
+        this.listenTo(docView.getApp().getImportPromise(), 'done', function () {
+            fillList();
+            self.listenTo(docModel, 'change:sheet:attributes', fillList); // e.g. sheet protection
         });
 
+        // cancel button drag&drop when losing edit rights
+        this.listenTo(docModel, 'change:editmode', function (event, editMode) {
+            if (!editMode && dndButtonNode) { Tracking.cancelTracking(); }
+        });
+
+        // create the context menu
+        contextMenu = new SheetContextMenu(docView, this.getNode());
+
         // the update handler scrolls to the active sheet, after it has been changed
         this.registerUpdateHandler(updateHandler);
 
@@ -508,19 +512,18 @@ define('io.ox/office/spreadsheet/view/control/activesheetgroup', [
         this.getNode().on('dblclick', Forms.OPTION_BUTTON_SELECTOR, doubleClickHandler);
 
         // handle tracking events of scroll buttons
-        prevButton.enableTracking({ autoRepeat: true })
+        Tracking.enableTracking(prevButton, { autoRepeat: true })
             .on('tracking:start tracking:repeat', prevButtonTrackingHandler)
             .on('tracking:move', function (event) { event.stopPropagation(); })
             .on('tracking:end tracking:cancel', buttonTrackingEndHandler);
-        nextButton.enableTracking({ autoRepeat: true })
+        Tracking.enableTracking(nextButton, { autoRepeat: true })
             .on('tracking:start tracking:repeat', nextButtonTrackingHandler)
             .on('tracking:move', function (event) { event.stopPropagation(); })
             .on('tracking:end tracking:cancel', buttonTrackingEndHandler);
 
         // drag sheet buttons to a new position (not on touch devices)
-        if (!Modernizr.touch) {
-            this.getNode()
-                .enableTracking({ selector: Forms.OPTION_BUTTON_SELECTOR, cursor: 'ew-resize', autoScroll: 'horizontal', scrollInterval: 500, borderMargin: -30 })
+        if (!Utils.TOUCHDEVICE) {
+            Tracking.enableTracking(this.getNode(), { selector: Forms.OPTION_BUTTON_SELECTOR, cursor: 'ew-resize', autoScroll: 'horizontal', scrollInterval: 500, borderMargin: -30 })
                 .on('tracking:start', dndTrackingStartHandler)
                 .on('tracking:move', dndTrackingMoveHandler)
                 .on('tracking:scroll', dndTrackingScrollHandler)
@@ -531,7 +534,7 @@ define('io.ox/office/spreadsheet/view/control/activesheetgroup', [
         // destroy all class members on destruction
         this.registerDestructor(function () {
             contextMenu.destroy();
-            app = self = docModel = docView = contextMenu = null;
+            self = docModel = docView = contextMenu = null;
             prevButton = nextButton = dndTrackerNode = dndSheetNode = null;
         });
 
