@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,7 +14,7 @@
 define('io.ox/office/spreadsheet/view/control/activesheetlist', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/radiolist',
-    'gettext!io.ox/office/spreadsheet'
+    'gettext!io.ox/office/spreadsheet/main'
 ], function (Utils, RadioList, gt) {
 
     'use strict';
@@ -27,12 +27,21 @@ define('io.ox/office/spreadsheet/view/control/activesheetlist', [
      * @constructor
      *
      * @extends RadioList
+     *
+     * @param {SpreadsheetView} docView
+     *  The spreadsheet view containing this instance.
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters. Will be passed to the base class RadioList.
      */
-    function ActiveSheetList(app, initOptions) {
+    function ActiveSheetList(docView, initOptions) {
 
         var // self reference
             self = this,
 
+            // the document model
+            docModel = docView.getDocModel(),
+
             // whether to show the sheet name as label text
             showNames = Utils.getBooleanOption(initOptions, 'showNames', false);
 
@@ -53,7 +62,7 @@ define('io.ox/office/spreadsheet/view/control/activesheetlist', [
          */
         var fillList = this.createDebouncedMethod($.noop, function () {
             self.clearMenu();
-            app.getView().iterateVisibleSheets(function (sheetModel, index, sheetName) {
+            docView.iterateVisibleSheets(function (sheetModel, index, sheetName) {
                 self.createOptionButton(index, {
                     icon: sheetModel.isLocked() ? 'fa-lock' : null,
                     iconClasses: 'small-icon',
@@ -67,18 +76,16 @@ define('io.ox/office/spreadsheet/view/control/activesheetlist', [
         // initialization -----------------------------------------------------
 
         // registering handler at the view
-        app.onInit(function () {
-            self.listenTo(app.getView(), 'change:sheets', fillList);
-        });
+        this.listenTo(docView, 'change:sheets', fillList);
 
-        this.listenTo(app.getImportPromise(), 'done', function () {
+        this.listenTo(docView.getApp().getImportPromise(), 'done', function () {
             fillList();
-            self.listenTo(app.getModel(), 'change:sheet:attributes', fillList); // e.g. sheet protection
+            self.listenTo(docModel, 'change:sheet:attributes', fillList); // e.g. sheet protection
         });
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initOptions = self = null;
+            self = docModel = docView = initOptions = null;
         });
 
     } // class ActiveSheetList
