@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/spreadsheet/view/control/alignmentpicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/radiolist',
     'io.ox/office/spreadsheet/view/labels',
-    'gettext!io.ox/office/spreadsheet'
+    'gettext!io.ox/office/spreadsheet/main'
 ], function (Utils, RadioList, Labels, gt) {
 
     'use strict';
@@ -90,7 +90,7 @@ define('io.ox/office/spreadsheet/view/control/alignmentpicker', [
 
         // initialization -----------------------------------------------------
 
-        _.each(config.entries, function (entryData) {
+        config.entries.forEach(function (entryData) {
             this.createOptionButton(entryData.value, { icon: entryData.icon, label: config.labels[entryData.value] });
         }, this);
 
