@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -28,45 +28,48 @@ define('io.ox/office/spreadsheet/view/control/cellbordercolorpicker', [
      * @constructor
      *
      * @extends ColorPicker
+     *
+     * @param {SpreadsheetView} docView
+     *  The spreadsheet view containing this instance.
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters. Supports all options that are also supported by
+     *  the base class ColorPicker.
      */
-    function CellBorderColorPicker(app, initOptions) {
+    function CellBorderColorPicker(docView, initOptions) {
 
         var // self reference
             self = this;
 
         // base constructor ---------------------------------------------------
 
-        ColorPicker.call(this, app, getGridColor(), Utils.extendOptions({
-            icon: 'docs-cell-fill-color',
-            tooltip: Labels.BORDER_COLOR_LABEL
+        ColorPicker.call(this, docView, 'line', Utils.extendOptions({
+            icon: 'docs-cell-line-color',
+            tooltip: Labels.BORDER_COLOR_LABEL,
+            title: Labels.BORDER_COLOR_LABEL
         }, initOptions));
 
         // private methods ----------------------------------------------------
 
         /**
-         * Returns the effective color used for the 'automatic color' list
-         * entry of this color picker control (which is the grid color of the
-         * active sheet).
-         *
-         * @returns {Object}
-         *  The color used for the 'automatic color' list entry.
+         * Updates the color used for the 'automatic color' list entry of this
+         * color picker control (which is the grid color of the active sheet).
          */
-        function getGridColor() {
-            return app.getView().getEffectiveGridColor(Color.BLACK);
+        function updateAutoColor() {
+            self.setAutoColor(docView.getSheetModel().getGridColor());
         }
 
         // initialization -----------------------------------------------------
 
         // refresh the 'automatic' border color (depends on sheet grid color)
-        app.onInit(function () {
-            self.listenTo(app.getView(), 'change:sheet:viewattributes', function () {
-                self.setAutoColor(getGridColor());
-            });
-        });
+        this.listenTo(docView, 'change:sheet:viewattributes', updateAutoColor);
+
+        // update auto color once when import is done
+        docView.getApp().onImportSuccess(updateAutoColor);
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initOptions = self = null;
+            docView = initOptions = self = null;
         });
 
     } // class CellBorderColorPicker
