@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -28,8 +28,17 @@ define('io.ox/office/spreadsheet/view/control/cellbordermodepicker', [
      * @constructor
      *
      * @extends BorderModePicker
+     *
+     * @param {SpreadsheetView} docView
+     *  The spreadsheet view containing this instance.
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters. Supports all options that are also supported by
+     *  the base class BorderModePicker, except for the options 'showInsideHor'
+     *  and 'showInsideVert' which will be initialized internally to reflect
+     *  the current sheet selection.
      */
-    function CellBorderModePicker(app, initOptions) {
+    function CellBorderModePicker(docView, initOptions) {
 
         // base constructor ---------------------------------------------------
 
@@ -50,7 +59,7 @@ define('io.ox/office/spreadsheet/view/control/cellbordermodepicker', [
          *  Whether inner horizontal borders are currently visible.
          */
         function showInsideHorBorder() {
-            return app.getView().hasMultipleRowsSelected();
+            return docView.hasMultipleRowsSelected();
         }
 
         /**
@@ -61,14 +70,14 @@ define('io.ox/office/spreadsheet/view/control/cellbordermodepicker', [
          *  Whether inner vertical borders are currently visible.
          */
         function showInsideVertBorder() {
-            return app.getView().hasMultipleColumnsSelected();
+            return docView.hasMultipleColumnsSelected();
         }
 
         // initialization -----------------------------------------------------
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initOptions = null;
+            docView = initOptions = null;
         });
 
     } // class CellBorderModePicker
