@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,9 +15,9 @@ define('io.ox/office/spreadsheet/view/control/cellstylepicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/forms',
     'io.ox/office/editframework/view/control/stylesheetpicker',
-    'io.ox/office/spreadsheet/utils/paneutils',
-    'gettext!io.ox/office/spreadsheet'
-], function (Utils, Forms, StyleSheetPicker, PaneUtils, gt) {
+    'io.ox/office/spreadsheet/model/cellcollection',
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, Forms, StyleSheetPicker, CellCollection, gt) {
 
     'use strict';
 
@@ -29,15 +29,21 @@ define('io.ox/office/spreadsheet/view/control/cellstylepicker', [
      * @constructor
      *
      * @extends StyleSheetPicker
+     *
+     * @param {SpreadsheetView} docView
+     *  The document view containing this instance.
      */
-    function CellStylePicker(app) {
+    function CellStylePicker(docView) {
 
-        var // the collection of cell styles in the document
-            styleCollection = null;
+        var // the document model
+            docModel = docView.getDocModel(),
+
+            // the collection of cell styles in the document
+            cellStyles = docModel.getCellStyles();
 
         // base constructor ---------------------------------------------------
 
-        StyleSheetPicker.call(this, app, 'cell', {
+        StyleSheetPicker.call(this, docView, 'cell', {
             width: 150,
             icon: 'docs-table-style',
             tooltip: /*#. tool tip: predefined styles for spreadsheet cells */ gt('Cell style'),
@@ -58,51 +64,48 @@ define('io.ox/office/spreadsheet/view/control/cellstylepicker', [
         function createItemHandler(event, buttonNode, styleId) {
 
             var // the merged attributes of the passed cell style sheet
-                attributes = styleCollection.getStyleSheetAttributes(styleId),
+                attributeSet = cellStyles.getStyleAttributeSet(styleId),
                 // the character attributes
-                charAttrs = attributes.character,
+                charAttrs = attributeSet.character,
                 // the cell attributes
-                cellAttrs = attributes.cell,
+                cellAttrs = attributeSet.cell,
                 // the caption element to be formatted
                 captionNode = Forms.getCaptionNode(buttonNode),
+                // text orientation settings according to localized style name
+                textOrientation = CellCollection.getTextOrientation({ result: captionNode.text(), display: captionNode.text(), attributes: attributeSet }),
                 // an additional border node as overlay (to prevent different line height in caption)
                 borderMarkup = null;
 
             // TODO: use canvas for complex border styles
             borderMarkup = '<div class="borders" style="';
-            borderMarkup += 'background-color:' + styleCollection.getCssColor(cellAttrs.fillColor, 'fill') + ';';
-            borderMarkup += 'border-top:' + styleCollection.getCssBorder(cellAttrs.borderTop, { preview: true }) + ';';
-            borderMarkup += 'border-bottom:' + styleCollection.getCssBorder(cellAttrs.borderBottom, { preview: true }) + ';';
-            borderMarkup += 'border-left:' + styleCollection.getCssBorder(cellAttrs.borderLeft, { preview: true }) + ';';
-            borderMarkup += 'border-right:' + styleCollection.getCssBorder(cellAttrs.borderRight, { preview: true }) + ';';
+            borderMarkup += 'background-color:' + docModel.getCssColor(cellAttrs.fillColor, 'fill') + ';';
+            borderMarkup += 'border-top:' + docModel.getCssBorder(cellAttrs.borderTop, { preview: true }) + ';';
+            borderMarkup += 'border-bottom:' + docModel.getCssBorder(cellAttrs.borderBottom, { preview: true }) + ';';
+            borderMarkup += 'border-left:' + docModel.getCssBorder(cellAttrs.borderLeft, { preview: true }) + ';';
+            borderMarkup += 'border-right:' + docModel.getCssBorder(cellAttrs.borderRight, { preview: true }) + ';';
             borderMarkup += '"></div>';
             buttonNode.prepend(borderMarkup);
 
             // add relevant character and cell formatting
             captionNode.css({
-                fontFamily: styleCollection.getCssFontFamily(charAttrs.fontName),
+                fontFamily: docModel.getCssFontFamily(charAttrs.fontName),
                 fontSize: Utils.minMax(Utils.round(10 + (charAttrs.fontSize - 10) / 1.5, 0.1), 6, 22) + 'pt',
                 fontWeight: charAttrs.bold ? 'bold' : 'normal',
                 fontStyle: charAttrs.italic ? 'italic' : 'normal',
-                textDecoration: styleCollection.getCssTextDecoration(charAttrs),
-                color: styleCollection.getCssTextColor(charAttrs.color, [cellAttrs.fillColor]),
-                textAlign: PaneUtils.getCssTextAlignment({ result: 'a', attributes: attributes })
+                textDecoration: docModel.getCssTextDecoration(charAttrs),
+                color: docModel.getCssTextColor(charAttrs.color, [cellAttrs.fillColor]),
+                textAlign: textOrientation.cssTextAlign
             });
         }
 
         // initialization -----------------------------------------------------
 
-        // initialize class members when application is ready
-        app.onInit(function () {
-            styleCollection = app.getModel().getStyleCollection('cell');
-        });
-
         // add visible formatting of the cell styles to the list item buttons
         this.getMenu().on('create:item', createItemHandler);
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = styleCollection = null;
+            docView = docModel = cellStyles = null;
         });
 
     } // class CellStylePicker
