@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,9 +14,9 @@
 define('io.ox/office/spreadsheet/view/control/discretefiltergroup', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/checkgroup',
-    'io.ox/office/spreadsheet/model/formula/tokenutils',
-    'gettext!io.ox/office/spreadsheet'
-], function (Utils, CheckGroup, TokenUtils, gt) {
+    'io.ox/office/spreadsheet/model/formula/formulautils',
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, CheckGroup, FormulaUtils, gt) {
 
     'use strict';
 
@@ -34,8 +34,11 @@ define('io.ox/office/spreadsheet/view/control/discretefiltergroup', [
      * @constructor
      *
      * @extends CheckGroup
+     *
+     * @param {SpreadsheetView} docView
+     *  The spreadsheet view that contains this instance.
      */
-    function DiscreteFilterGroup(app) {
+    function DiscreteFilterGroup(docView) {
 
         var // self reference
             self = this;
@@ -58,8 +61,8 @@ define('io.ox/office/spreadsheet/view/control/discretefiltergroup', [
          * entries of this control.
          */
         function sortComparator(sortIndex1, sortIndex2) {
-            var result = TokenUtils.compareValues(sortIndex1.result, sortIndex2.result);
-            return (result === 0) ? TokenUtils.compareStrings(sortIndex1.display, sortIndex2.display) : result;
+            var result = FormulaUtils.compareValues(sortIndex1.result, sortIndex2.result);
+            return (result === 0) ? FormulaUtils.compareStrings(sortIndex1.display, sortIndex2.display) : result;
         }
 
         // public methods -----------------------------------------------------
@@ -68,30 +71,32 @@ define('io.ox/office/spreadsheet/view/control/discretefiltergroup', [
          * Collects and inserts the cell entries of a table column into this
          * control.
          *
-         * @param {TableModel} tableModel
-         *  The table model.
+         * @param {String} tableName
+         *  The name of the table model in the active sheet to be filtered.
          *
          * @param {Number} tableCol
          *  The zero-based column index, relative to the table range.
          *
          * @returns {jQuery.Promise}
-         *  The promise of a Deferred object that will be resolved when the
-         *  control has been initialized.
+         *  A promise that will be resolved when the control has been
+         *  initialized.
          */
-        this.initialize = function (tableModel, tableCol) {
-
-            var // promise representing the result of this method
-                promise = null;
+        this.initialize = function (tableName, tableCol) {
 
             this.clearOptionButtons();
 
+            // resolve the table model from the document
+            var tableModel = docView.getTableCollection().getTable(tableName);
+            if (!tableModel) {
+                Utils.warn('DiscreteFilterGroup.initialize(): table model not found');
+                return $.Deferred().reject();
+            }
+
             // unfiltered table column: show entries of all visible rows; filtered table column: show
             // entries for all data rows that are not filtered by other columns (but that may be hidden)
-            if (tableModel.isColumnFiltered(tableCol)) {
-                promise = tableModel.queryFilterResult({ excludeTableCol: tableCol });
-            } else {
-                promise = $.when(tableModel.getVisibleDataRows());
-            }
+            var promise = tableModel.isColumnFiltered(tableCol) ?
+                    tableModel.queryFilterResult({ excludeTableCol: tableCol }) :
+                    $.when(tableModel.getVisibleDataRows());
 
             // receive source data from document
             promise = promise.then(function (rowIntervals) {
@@ -111,7 +116,7 @@ define('io.ox/office/spreadsheet/view/control/discretefiltergroup', [
 
                     var // the label displayed in the list entry (preserve case)
                         // bug 36264: trim space characters but no other whitespace
-                        label = cellData.display.replace(/^ +| +$/g, ''),
+                        label = _.isString(cellData.display) ? cellData.display.replace(/^ +| +$/g, '') : '#######',
                         // the case-insensitive key used to detect duplicates
                         key = label.toLowerCase(),
                         // options for the list entry
@@ -137,7 +142,7 @@ define('io.ox/office/spreadsheet/view/control/discretefiltergroup', [
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = self = null;
+            self = docView = null;
         });
 
     } // class DiscreteFilterGroup
