@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,10 +14,11 @@
 define('io.ox/office/spreadsheet/view/control/formatcategorygroup', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/forms',
+    'io.ox/office/tk/locale/localedata',
     'io.ox/office/tk/control/radiogroup',
     'io.ox/office/spreadsheet/view/labels',
-    'gettext!io.ox/office/spreadsheet'
-], function (Utils, Forms, RadioGroup, Labels, gt) {
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, Forms, LocaleData, RadioGroup, Labels, gt) {
 
     'use strict';
 
@@ -30,10 +31,13 @@ define('io.ox/office/spreadsheet/view/control/formatcategorygroup', [
      *
      * @extends RadioGroup
      *
+     * @param {SpreadsheetView} docView
+     *  The document view containing this instance.
+     *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options of the base class RadioGroup.
      */
-    function FormatCategoryGroup(app, initOptions) {
+    function FormatCategoryGroup(docView, initOptions) {
 
         var // self reference
             self = this;
@@ -41,7 +45,6 @@ define('io.ox/office/spreadsheet/view/control/formatcategorygroup', [
         // base constructor ---------------------------------------------------
 
         RadioGroup.call(this, Utils.extendOptions({
-            tooltip: gt('Number formats'),
             toggleValue: 'standard'
         }, initOptions));
 
@@ -74,20 +77,12 @@ define('io.ox/office/spreadsheet/view/control/formatcategorygroup', [
         });
 
         // initialize the option buttons
-        this.listenTo(app.getImportPromise(), 'done', function () {
-
-            var // the number formatter providing the currency symbol for the currency button
-                numberFormatter = app.getModel().getNumberFormatter(),
-                // the currency symbol of the current language
-                currencySymbol = numberFormatter.getCurrencySymbol();
-
-            createOptionButton('currency', _.noI18n(currencySymbol));
-            createOptionButton('percent', _.noI18n('%'));
-        });
+        createOptionButton('currency', _.noI18n(LocaleData.CURRENCY));
+        createOptionButton('percent', _.noI18n('%'));
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initOptions = self = null;
+            self = docView = initOptions = null;
         });
 
     } // class FormatCategoryGroup
