@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/spreadsheet/view/control/formatcategorypicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/radiolist',
     'io.ox/office/spreadsheet/view/labels',
-    'gettext!io.ox/office/spreadsheet'
+    'gettext!io.ox/office/spreadsheet/main'
 ], function (Utils, RadioList, Labels, gt) {
 
     'use strict';
@@ -27,16 +27,19 @@ define('io.ox/office/spreadsheet/view/control/formatcategorypicker', [
 
     /**
      * A drop-down list control for choosing number format categories, that
-     * will load predefined format codes in the FormatCodePicker.
+     * will load predefined format codes in the format code picker control.
      *
      * @constructor
      *
      * @extends RadioList
      *
+     * @param {SpreadsheetView} docView
+     *  The document view containing this instance.
+     *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options of the base class RadioList.
      */
-    function FormatCategoryPicker(app, initOptions) {
+    function FormatCategoryPicker(docView, initOptions) {
 
         var // self reference
             self = this;
@@ -55,7 +58,7 @@ define('io.ox/office/spreadsheet/view/control/formatcategorypicker', [
         this.getNode().addClass('format-category-group');
 
         // initialize the drop-down list items
-        this.listenTo(app.getImportPromise(), 'done', function () {
+        this.listenTo(docView.getApp().getImportPromise(), 'done', function () {
             // insert all supported list items
             _.each(FORMAT_CATEGORIES, function (category) {
                 self.createOptionButton(category, { label: Labels.FORMAT_CATEGORY_LABELS[category] });
@@ -64,7 +67,7 @@ define('io.ox/office/spreadsheet/view/control/formatcategorypicker', [
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initOptions = self = null;
+            self = docView = initOptions = null;
         });
 
     } // class FormatCategoryPicker
