@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/spreadsheet/view/control/formatcodepicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/radiolist',
     'io.ox/office/spreadsheet/view/labels',
-    'gettext!io.ox/office/spreadsheet'
+    'gettext!io.ox/office/spreadsheet/main'
 ], function (Utils, RadioList, Labels, gt) {
 
     'use strict';
@@ -30,17 +30,19 @@ define('io.ox/office/spreadsheet/view/control/formatcodepicker', [
      *
      * @extends RadioList
      *
+     * @param {SpreadsheetView} docView
+     *  The document view containing this instance.
+     *
      * @param {Object} [initOptions]
      *  Optional parameters. Supports all options of the base class RadioList.
      */
-    function FormatCodePicker(app, initOptions) {
+    function FormatCodePicker(docView, initOptions) {
 
         var // self reference
             self = this,
 
-            // the model and view of the document
-            docModel = null,
-            docView = null,
+            // the number formatter of the document
+            numberFormatter = docView.getDocModel().getNumberFormatter(),
 
             // the category of the format codes currently shown in the list
             listCategory = null;
@@ -65,7 +67,7 @@ define('io.ox/office/spreadsheet/view/control/formatcodepicker', [
             var // the current number format category
                 currentCategory = docView.getNumberFormatCategory(),
                 // all codes for the current number format
-                categoryCodes = docModel.getNumberFormatter().getCategoryCodes(currentCategory),
+                categoryCodes = numberFormatter.getCategoryCodes(currentCategory),
                 // the active format code
                 formatCodeEntry = _.findWhere(categoryCodes, { value: value }),
                 //  menu instance of the radio list
@@ -75,9 +77,11 @@ define('io.ox/office/spreadsheet/view/control/formatcodepicker', [
             if (listCategory !== currentCategory) {
                 self.clearMenu();
                 _.each(categoryCodes, function (entry) {
+                    var parsedFormat = numberFormatter.parseNumberFormat({ code: entry.value }),
+                        negativeSection = parsedFormat.numberSections[1];
                     self.createOptionButton(entry.value, {
                         label: _.noI18n(entry.label),
-                        labelStyle: entry.red ? 'color:#f00000;' : null
+                        labelStyle: (negativeSection && negativeSection.colorName) ? ('color:#' + negativeSection.rgbColor + ';') : null
                     });
                 });
                 listCategory = currentCategory;
@@ -102,18 +106,12 @@ define('io.ox/office/spreadsheet/view/control/formatcodepicker', [
 
         // initialization -----------------------------------------------------
 
-        // initialize class members when application is ready
-        app.onInit(function () {
-            docModel = app.getModel();
-            docView = app.getView();
-        });
-
         // register custom update handler to update button caption
         this.registerUpdateHandler(updateHandler);
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initOptions = self = docModel = docView = null;
+            self = docView = initOptions = numberFormatter = null;
         });
 
     } // class FormatCodePicker
