@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/spreadsheet/view/control/mergecellspicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/radiolist',
     'io.ox/office/spreadsheet/utils/sheetutils',
-    'gettext!io.ox/office/spreadsheet'
+    'gettext!io.ox/office/spreadsheet/main'
 ], function (Utils, RadioList, SheetUtils, gt) {
 
     'use strict';
@@ -37,8 +37,15 @@ define('io.ox/office/spreadsheet/view/control/mergecellspicker', [
      * @constructor
      *
      * @extends RadioList
+     *
+     * @param {SpreadsheetView} docView
+     *  The document view containing this instance.
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters. Supports all options of the base class RadioList,
+     *  except 'highlight', 'splitValue', and 'updateCaptionMode'.
      */
-    function MergeCellsPicker(app, initOptions) {
+    function MergeCellsPicker(docView, initOptions) {
 
         var // self reference
             self = this,
@@ -81,23 +88,23 @@ define('io.ox/office/spreadsheet/view/control/mergecellspicker', [
         function updateVisibilityFlags() {
 
             var // the collection of merged ranges in the active sheet
-                mergeCollection = app.getView().getMergeCollection(),
+                mergeCollection = docView.getMergeCollection(),
                 // the selected cell ranges
-                ranges = app.getView().getSelectedRanges();
+                ranges = docView.getSelectedRanges();
 
             // show the 'merge' list item, if at least one range consists of more than a cell or a merged range
-            showMergeItem = _.any(ranges, function (range) {
-                var mergedRange = (SheetUtils.getCellCount(range) > 1) ? mergeCollection.getMergedRange(range.start) : null;
-                return !mergedRange || !SheetUtils.rangeContainsRange(mergedRange, range);
+            showMergeItem = ranges.some(function (range) {
+                var mergedRange = range.single() ? null : mergeCollection.getMergedRange(range.start);
+                return !mergedRange || !mergedRange.contains(range);
             });
 
             // show the 'horizontal' list item, if at least one range does not consist entirely of horizontally merged cells
-            showHMergeItem = _.any(ranges, function (range) {
+            showHMergeItem = ranges.some(function (range) {
                 return (range.start[0] < range.end[0]) && (range.start[1] < range.end[1]) && !mergeCollection.isHorizontallyMerged(range);
             });
 
             // show the 'vertical' list item, if at least one range does not consist entirely of vertically merged cells
-            showVMergeItem = _.any(ranges, function (range) {
+            showVMergeItem = ranges.some(function (range) {
                 return (range.start[0] < range.end[0]) && (range.start[1] < range.end[1]) && !mergeCollection.isVerticallyMerged(range);
             });
 
@@ -117,9 +124,9 @@ define('io.ox/office/spreadsheet/view/control/mergecellspicker', [
         this.registerUpdateHandler(updateHandler);
 
         // update state flags used for visibility of list items (to keep the 'visible' callback functions simple and fast)
-        this.listenTo(app.getImportPromise(), 'done', function () {
+        this.listenTo(docView.getApp().getImportPromise(), 'done', function () {
             updateVisibilityFlags();
-            self.listenTo(app.getView(), 'change:selection insert:merged delete:merged', updateVisibilityFlags);
+            self.listenTo(docView, 'change:selection insert:merged delete:merged', updateVisibilityFlags);
         });
 
     } // class MergeCellsPicker
