@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Michael Nimz <michael.nimz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/spreadsheet/view/control/sortmenubutton', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/button',
     'io.ox/office/baseframework/view/control/compoundsplitbutton',
-    'gettext!io.ox/office/spreadsheet'
+    'gettext!io.ox/office/spreadsheet/main'
 ], function (Utils, Button, CompoundSplitButton, gt) {
 
     'use strict';
@@ -28,18 +28,22 @@ define('io.ox/office/spreadsheet/view/control/sortmenubutton', [
      * @constructor
      *
      * @extends CompoundSplitButton
+     *
+     * @param {SpreadsheetView} docView
+     *  The document view containing this instance.
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters. Supports all options of the base class
+     *  CompoundSplitButton, except 'value', and 'updateCaptionMode'.
      */
-    function SortMenuButton(app, initOptions) {
+    function SortMenuButton(docView, initOptions) {
 
         var // self reference
-            self = this,
-
-            // the document view
-            docView = null;
+            self = this;
 
         // base constructor ---------------------------------------------------
 
-        CompoundSplitButton.call(this, app, Utils.extendOptions({
+        CompoundSplitButton.call(this, docView, Utils.extendOptions({
             icon: 'fa-sort-alpha-asc',
             tooltip: /*#. spreadsheet sorting: button tooltip for sorting */ gt.pgettext('sort', 'Sorting')
         }, initOptions, {
@@ -68,23 +72,18 @@ define('io.ox/office/spreadsheet/view/control/sortmenubutton', [
 
         // initialization -----------------------------------------------------
 
-        // initialize class members when application is ready
-        app.onInit(function () {
-            docView = app.getView();
-        });
-
         // create all drop-down menu items
         this.addGroup('cell/sort', new Button({ icon: 'fa-sort-alpha-asc', label: /*#. sort ascending */ gt.pgettext('sort', 'Ascending'), tooltip: gt.pgettext('sort', 'Sort from smallest to largest value'), value: 'ascending' }))
             .addGroup('cell/sort', new Button({ icon: 'fa-sort-alpha-desc', label: /*#. sort descending */ gt.pgettext('sort', 'Descending'), tooltip: gt.pgettext('sort', 'Sort from largest to smallest value'), value: 'descending' }))
             .addSeparator()
-            .addGroup('cell/sort/expanddialog', new Button({ label: /*#. sorting: label of the button opening a menu with custom sort options */ gt.pgettext('sort', 'Custom'), tooltip: /*#. tooltip for custom sort button */ gt.pgettext('sort', 'Settings for sorting') }));
+            .addGroup('cell/sort/menu/show', new Button({ label: /*#. sorting: label of the button opening a menu with custom sort options */ gt.pgettext('sort', 'Custom'), tooltip: /*#. tooltip for custom sort button */ gt.pgettext('sort', 'Settings for sorting') }));
 
         // update the icon in the drop-down button
         this.registerUpdateHandler(updateHandler);
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initOptions = self = docView = null;
+            self = docView = initOptions = null;
         });
 
     } // class SortMenuButton
