@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/spreadsheet/view/control/subtotallist', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/radiolist',
     'io.ox/office/spreadsheet/utils/sheetutils',
-    'gettext!io.ox/office/spreadsheet'
+    'gettext!io.ox/office/spreadsheet/main'
 ], function (Utils, RadioList, SheetUtils, gt) {
 
     'use strict';
@@ -23,20 +23,25 @@ define('io.ox/office/spreadsheet/view/control/subtotallist', [
     // class SubtotalList =====================================================
 
     /**
-     * The subtotal 'customizer'. A yet another drop down list for displaying
-     * desired subtotal value.
+     * A drop-down list control for displaying different subtotal values.
      *
      * @constructor
      *
      * @extends RadioList
+     *
+     * @param {SpreadsheetView} docView
+     *  The spreadsheet view containing this instance.
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters. Will be passed to the base class RadioList.
      */
-    function SubtotalList(app, initOptions) {
+    function SubtotalList(docView, initOptions) {
 
         var // self reference
             self = this,
 
             // the number formatter of the document
-            numberFormatter = null;
+            numberFormatter = docView.getDocModel().getNumberFormatter();
 
         // base constructor ---------------------------------------------------
 
@@ -52,10 +57,10 @@ define('io.ox/office/spreadsheet/view/control/subtotallist', [
          */
         function getCaptionOptions(type, value) {
 
-            var // contents and formatting of the active cell
-                activeCell = app.getView().getCellContents(),
+            var // contents and formatting of the active cell (instance of CellDescriptor)
+                cellDesc = docView.getActiveCellEntry(),
                 // whether the active cell is formatted as date or time
-                dateTime = /^(date|time|datetime)$/.test(activeCell.format.cat),
+                dateTime = /^(date|time|datetime)$/.test(cellDesc.format.cat),
                 // the resulting label text
                 label = '';
 
@@ -68,10 +73,11 @@ define('io.ox/office/spreadsheet/view/control/subtotallist', [
                     label = String(value);
                 } else if ((type === 'sum') && dateTime) {
                     // type 'sum' with date/time: show localized #N/A error
-                    label = app.convertErrorCodeToString(SheetUtils.ErrorCodes.NA);
+                    label = numberFormatter.formatValue(SheetUtils.ErrorCodes.NA);
                 } else {
                     // format number according to the active cell's number format
-                    label = numberFormatter.formatCellContent(value, activeCell.attributes);
+                    var formatCode = numberFormatter.resolveFormatCode(cellDesc.attributes.cell.numberFormat);
+                    label = numberFormatter.formatValue(value, formatCode) || String(value);
                 }
             }
 
@@ -114,8 +120,8 @@ define('io.ox/office/spreadsheet/view/control/subtotallist', [
          */
         function updateHandler(type) {
 
-            var // get fresh subtotals
-                subtotals = app.getView().getSubtotals();
+            // get fresh subtotals
+            var subtotals = docView.getSubtotals();
 
             // if selection only consists of string values, show cell count
             if (('numbers' in subtotals) && (subtotals.numbers === 0)) { type = 'cells'; }
@@ -129,11 +135,12 @@ define('io.ox/office/spreadsheet/view/control/subtotallist', [
         /**
          * Recreates all sheet entries in the drop-down menu.
          */
-        var fillList = this.createDebouncedMethod($.noop, function () {
+        function fillList() {
 
-            var // get fresh subtotals
-                subtotals = app.getView().getSubtotals();
+            // get fresh subtotals
+            var subtotals = docView.getSubtotals();
 
+            // creates an option button for the current subtotals
             function createOptionButton(section, type) {
                 self.createOptionButton(type, _.extend({ section: section }, getCaptionOptions(type, subtotals[type])));
             }
@@ -151,24 +158,35 @@ define('io.ox/office/spreadsheet/view/control/subtotallist', [
                 createOptionButton('numbers', 'max');
                 createOptionButton('numbers', 'average');
             }
-        }, { delay: 200 });
+        }
+
+        /**
+         * A debounced version of the method fillList().
+         */
+        var fillListDebounced = this.createDebouncedMethod(_.noop, fillList, { delay: 200 });
 
         // initialization -----------------------------------------------------
 
         this.registerUpdateHandler(updateHandler);
 
-        app.onInit(function () {
-            numberFormatter = app.getModel().getNumberFormatter();
-        });
+        // initialize the menu after successful import
+        this.listenTo(docView.getApp().getImportPromise(), 'done', function () {
 
-        this.listenTo(app.getImportPromise(), 'done', function () {
-            fillList();
-            self.listenTo(app.getView(), 'change:layoutdata', fillList);
+            // be lazy and do nothing as long as the menu is closed
+            self.getMenu().on({
+                'popup:beforeshow': function () {
+                    fillList();
+                    self.listenTo(docView, 'change:layoutdata', fillListDebounced);
+                },
+                'popup:hide': function () {
+                    self.stopListeningTo(docView, 'change:layoutdata', fillListDebounced);
+                }
+            });
         });
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = initOptions = self = numberFormatter = null;
+            self = docView = initOptions = numberFormatter = null;
         });
 
     } // class SubtotalList
