@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -24,12 +24,13 @@ define('io.ox/office/spreadsheet/view/controls', [
     'io.ox/office/spreadsheet/view/control/cellbordercolorpicker',
     'io.ox/office/spreadsheet/view/control/cellstylepicker',
     'io.ox/office/spreadsheet/view/control/mergecellspicker',
+    'io.ox/office/spreadsheet/view/control/namedrangesmenubutton',
     'io.ox/office/spreadsheet/view/control/sortmenubutton',
     'io.ox/office/spreadsheet/view/control/activesheetgroup',
     'io.ox/office/spreadsheet/view/control/activesheetlist',
     'io.ox/office/spreadsheet/view/control/subtotallist',
-    'gettext!io.ox/office/spreadsheet'
-], function (Utils, EditControls, SheetUtils, Labels, AlignmentPicker, FormatCategoryGroup, FormatCategoryPicker, FormatCodePicker, CellBorderModePicker, CellBorderColorPicker, CellStylePicker, MergeCellsPicker, SortMenuButton, ActiveSheetGroup, ActiveSheetList, SubtotalList, gt) {
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, EditControls, SheetUtils, Labels, AlignmentPicker, FormatCategoryGroup, FormatCategoryPicker, FormatCodePicker, CellBorderModePicker, CellBorderColorPicker, CellStylePicker, MergeCellsPicker, NamedRangesMenuButton, SortMenuButton, ActiveSheetGroup, ActiveSheetList, SubtotalList, gt) {
 
     'use strict';
 
@@ -38,8 +39,7 @@ define('io.ox/office/spreadsheet/view/controls', [
     /**
      * Provides different classes for GUI form controls. Collects all standard
      * control group classes defined in the toolkit and the base framework, and
-     * adds more controls along with caption texts for control groups, menu
-     * headers, and other GUI elements in a single map for convenience.
+     * adds more controls in a single map for convenience.
      *
      * @extends EditControls
      */
@@ -52,6 +52,7 @@ define('io.ox/office/spreadsheet/view/controls', [
         CellBorderColorPicker: CellBorderColorPicker,
         CellStylePicker: CellStylePicker,
         MergeCellsPicker: MergeCellsPicker,
+        NamedRangesMenuButton: NamedRangesMenuButton,
         SortMenuButton: SortMenuButton,
         ActiveSheetGroup: ActiveSheetGroup,
         ActiveSheetList: ActiveSheetList,
@@ -95,10 +96,6 @@ define('io.ox/office/spreadsheet/view/controls', [
             tooltip: columns ? gt('Column width') : gt('Row height'),
             min: 0,
             max: columns ? SheetUtils.MAX_COLUMN_WIDTH : SheetUtils.MAX_ROW_HEIGHT,
-            precision: 100,
-            smallStep: 100,
-            largeStep: 500,
-            roundStep: true,
             smallerVersion: { hide: true }
         });
 
@@ -114,9 +111,9 @@ define('io.ox/office/spreadsheet/view/controls', [
      *
      * @extends BorderStylePicker
      */
-    SpreadsheetControls.CellBorderStylePicker = EditControls.BorderStylePicker.extend({ constructor: function (app, initOptions) {
+    SpreadsheetControls.CellBorderStylePicker = EditControls.BorderStylePicker.extend({ constructor: function (docView, initOptions) {
 
-        EditControls.BorderStylePicker.call(this, app.isOOXML() ? Labels.BORDER_OOXML_PRESET_STYLES : Labels.BORDER_ODF_LINE_STYLES, initOptions);
+        EditControls.BorderStylePicker.call(this, docView.getApp().isOOXML() ? Labels.BORDER_OOXML_PRESET_STYLES : Labels.BORDER_ODF_LINE_STYLES, initOptions);
 
     }}); // class CellBorderStylePicker
 
@@ -148,9 +145,9 @@ define('io.ox/office/spreadsheet/view/controls', [
      *
      * @extends CompoundCheckBox
      */
-    SpreadsheetControls.FrozenSplitCheckBox = EditControls.CompoundCheckBox.extend({ constructor: function (app, initOptions) {
+    SpreadsheetControls.FrozenSplitCheckBox = EditControls.CompoundCheckBox.extend({ constructor: function (docView, initOptions) {
 
-        EditControls.CompoundCheckBox.call(this, app, Utils.extendOptions({
+        EditControls.CompoundCheckBox.call(this, docView, Utils.extendOptions({
             label: /*#. check box: split a spreadsheet into 2 or 4 different parts, the leading (left/upper) parts are frozen and cannot be scrolled */ gt.pgettext('sheet-split', 'Freeze sheet'),
             tooltip: gt.pgettext('sheet-split', 'Freeze the rows above and the columns left of the cursor')
         }, initOptions));
