@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,6 +14,7 @@
 define('io.ox/office/spreadsheet/view/headerpane', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/forms',
+    'io.ox/office/tk/utils/tracking',
     'io.ox/office/tk/object/triggerobject',
     'io.ox/office/tk/object/timermixin',
     'io.ox/office/spreadsheet/utils/sheetutils',
@@ -21,13 +22,14 @@ define('io.ox/office/spreadsheet/view/headerpane', [
     'io.ox/office/spreadsheet/view/popup/headercontextmenu',
     'io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
     'io.ox/office/spreadsheet/view/render/renderutils',
-    'gettext!io.ox/office/spreadsheet'
-], function (Utils, Forms, TriggerObject, TimerMixin, SheetUtils, PaneUtils, HeaderContextMenu, HeaderTrackingMixin, RenderUtils, gt) {
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, Forms, Tracking, TriggerObject, TimerMixin, SheetUtils, PaneUtils, HeaderContextMenu, HeaderTrackingMixin, RenderUtils, gt) {
 
     'use strict';
 
-    var // the ratio of the visible pane size to be added before and after
-        ADDITIONAL_SIZE_RATIO = Utils.COMPACT_DEVICE ? 0.3 : 0.5,
+    var // convenience shortcuts
+        Address = SheetUtils.Address,
+        Interval = SheetUtils.Interval,
 
         // margin at trailing border of the entire sheet area, in pixels
         SCROLL_AREA_MARGIN = 30,
@@ -50,40 +52,6 @@ define('io.ox/office/spreadsheet/view/headerpane', [
      * header panes, and top and bottom row header panes).
      *
      * Triggers the following events:
-     * - 'change:interval'
-     *      After the column/row interval shown by this header pane has been
-     *      changed. Event handlers receive the following parameters:
-     *      (1) {jQuery.Event} event
-     *          The jQuery event object.
-     *      (2) {Object} interval
-     *          The new column/row interval displayed in this header pane, with
-     *          the zero-based index properties 'first' and 'last'.
-     *      (3) {Object} [refresh]
-     *          The refresh strategy for the columns/rows in the new interval,
-     *          with the following properties:
-     *          - {Number} [refresh.index]
-     *              If specified, the layout of the columns/rows starting from
-     *              this column/row index has changed, e.g. after a sheet
-     *              operation such as inserting/deleting rows or columns, or
-     *              changing their format attributes (including size and
-     *              visibility).
-     *          - {Boolean} [refresh.geometry=false]
-     *              If set to true, the geometry of all columns, rows, and
-     *              cells (position, size, and text clipping) needs to be
-     *              refreshed completely (e.g. while zooming).
-     *          If this parameter is omitted, the visible interval has changed
-     *          but the columns/rows in the new interval did not change.
-     *      (4) {Object} [options]
-     *          Optional parameters:
-     *          - {Boolean} [options.direct=false]
-     *              If set to true, dependent updates must be performed
-     *              immediately to prevent display errors (such as jumping cell
-     *              contents while scrolling through the sheet).
-     * - 'hide:interval'
-     *      After this header pane has been hidden, and the old column/row
-     *      interval has been reset to nothing. Event listeners are expected to
-     *      reset or update additional settings according to the hidden header
-     *      pane.
      * - 'change:scrollsize'
      *      After the size of the scrollable area in this header pane has been
      *      changed. Event handlers receive the new scroll area size.
@@ -135,21 +103,20 @@ define('io.ox/office/spreadsheet/view/headerpane', [
      * @extends TriggerObject
      * @extends TimerMixin
      *
-     * @param {SpreadsheetApplication} app
-     *  The application that contains this header pane.
+     * @param {SpreadsheetView} docView
+     *  The spreadsheet view that contains this header pane.
      *
      * @param {String} paneSide
      *  The position of this header pane (one of 'left', 'right', 'top',
      *  'bottom', or 'corner').
      */
-    function HeaderPane(app, paneSide) {
+    function HeaderPane(docView, paneSide) {
 
         var // self reference
             self = this,
 
             // the spreadsheet model and view
-            docModel = null,
-            docView = null,
+            docModel = docView.getDocModel(),
 
             // whether this header pane contains columns
             columns = PaneUtils.isColumnSide(paneSide),
@@ -197,7 +164,6 @@ define('io.ox/office/spreadsheet/view/headerpane', [
 
         TriggerObject.call(this);
         TimerMixin.call(this);
-        HeaderTrackingMixin.call(this, app, paneSide, contentNode);
 
         // private methods ----------------------------------------------------
 
@@ -215,7 +181,7 @@ define('io.ox/office/spreadsheet/view/headerpane', [
                 focused = false;
 
             if (docView.hasFrozenSplit()) {
-                // always highlight all panes in frozen split mode
+                // always highlight all header panes in frozen split mode
                 focused = true;
             } else if (_.isString(focusPaneSide)) {
                 // a header pane is active (selection tracking): focus this
@@ -255,7 +221,7 @@ define('io.ox/office/spreadsheet/view/headerpane', [
             var // the selected ranges
                 ranges = docView.getSelectedRanges(),
                 // the merged ranges of selected columns/rows
-                selectionIntervals = SheetUtils.getIntervals(ranges, columns),
+                selectionIntervals = ranges.intervals(columns).merge(),
                 // current index into the intervals array
                 arrayIndex = 0;
 
@@ -275,9 +241,9 @@ define('io.ox/office/spreadsheet/view/headerpane', [
             }, undefined, { children: true });
 
             // on touch devices, update the resize handle for entire columns/rows
-            if (Modernizr.touch) {
-                if ((ranges.length === 1) && docModel.isFullRange(ranges[0], columns)) {
-                    contentNode.find('>.touchresizer').show().attr('data-index', ranges[0].end[columns ? 0 : 1]);
+            if (Utils.TOUCHDEVICE) {
+                if ((ranges.length === 1) && docModel.isFullRange(ranges.first(), columns)) {
+                    contentNode.find('>.touchresizer').show().attr('data-index', ranges.first().getEnd(columns));
                     updateTouchResizerPosition();
                 } else {
                     contentNode.find('>.touchresizer').hide();
@@ -348,10 +314,10 @@ define('io.ox/office/spreadsheet/view/headerpane', [
 
             // set font size and line height (column headers only, at cell level for row headers) at content node
             contentNode.css('font-size', docView.getHeaderFontSize() + 'pt');
-            if (columns) { contentNode.css('line-height', (rootNode.height() - 1) + 'px' ); }
+            if (columns) { contentNode.css('line-height', (rootNode.height() - 1) + 'px'); }
 
             // generate HTML mark-up text for all header cells
-            _.each(entries, function (entry) {
+            entries.forEach(function (entry) {
 
                 var // whether the current entry is hidden and precedes a visible entry
                     hidden = entry.size === 0,
@@ -367,11 +333,11 @@ define('io.ox/office/spreadsheet/view/headerpane', [
                     markup += '<div class="cell' + (prevHidden ? ' prev-hidden' : '') + '"';
                     markup += ' style="' + getPosition(entry.offset, entry.size);
                     if (!columns) { markup += 'line-height:' + (entry.size - 1) + 'px;'; }
-                    markup += '"' + dataAttr + '>' + SheetUtils.getIndexName(entry.index, columns) + '</div>';
+                    markup += '"' + dataAttr + '>' + Address.stringifyIndex(entry.index, columns) + '</div>';
                 }
 
                 // generate mark-up for the resizer node
-                if (!Modernizr.touch) {
+                if (!Utils.TOUCHDEVICE) {
                     markup += '<div class="resizer' + (hidden ? ' collapsed' : '') + '"';
                     markup += ' style="' + getPosition(entry.resizerOffset, RESIZER_SIZE) + '"';
                     markup += ' title="' + resizerTooltip + '"' + dataAttr + '></div>';
@@ -382,7 +348,7 @@ define('io.ox/office/spreadsheet/view/headerpane', [
             });
 
             // generate mark-up for global resizer node for touch devices
-            if (Modernizr.touch) {
+            if (Utils.TOUCHDEVICE) {
                 markup += '<div class="touchresizer">' + Forms.createIconMarkup(columns ? 'fa-arrows-h' : 'fa-arrows-v') + '</div>';
             }
 
@@ -450,14 +416,18 @@ define('io.ox/office/spreadsheet/view/headerpane', [
             // (prevent that header pane scrolls more than the grid panes can scroll)
             scrollPos = getValidScrollPos(newScrollPos);
 
-            var // the relative offset of the content node
-                contentOffset = position.offset - (scrollPos + hiddenSize);
+            var // the column/row collection
+                collection = self.getColRowCollection(),
+                // the relative offset of the content node
+                contentOffset = position.offset - (scrollPos + hiddenSize),
+                // maximum size (restrict to sheet size)
+                contentSize = Math.min(position.size, collection.getTotalSize() - position.offset);
 
             // set the position and size of the content node
             if (columns) {
-                contentNode.css({ left: contentOffset, width: position.size });
+                contentNode.css({ left: contentOffset, width: contentSize });
             } else {
-                contentNode.css({ top: contentOffset, height: position.size });
+                contentNode.css({ top: contentOffset, height: contentSize });
             }
 
             // notify listeners (do not check whether the scroll position really has
@@ -474,9 +444,8 @@ define('io.ox/office/spreadsheet/view/headerpane', [
          * rows according to the position of this header pane, and the current
          * split settings.
          *
-         * @returns {Object}
-         *  The complete column/row interval available in this header pane, in
-         *  the zero-based index properties 'first' and 'last'.
+         * @returns {Interval}
+         *  The complete column/row interval available in this header pane.
          */
         function getAvailableInterval() {
 
@@ -496,128 +465,15 @@ define('io.ox/office/spreadsheet/view/headerpane', [
             }
 
             // build the available interval according to the own position
-            return leading ? frozenInterval : { first: frozenInterval.last + 1, last: docModel.getMaxIndex(columns) };
-        }
-
-        /**
-         * Recalculates the current row/column interval and its position and
-         * size in the sheet, and renders all visible header cells and the
-         * selection on demand. Triggers a 'change:interval' event, if the
-         * visible interval has been changed, or if the column/row entries in
-         * the interval have changed according to the passed options.
-         *
-         * @param {Object} [refresh]
-         *  The refresh strategy for the columns/rows in the new interval. See
-         *  description of the 'change:interval' event above for details.
-         *
-         * @param {Object} [options]
-         *  Optional parameters passed to the 'change:interval' event. See the
-         *  event description above for details.
-         */
-        function updateAndRenderInterval(refresh, options) {
-
-            var // the column/row collection
-                collection = self.getColRowCollection(),
-                // the current scroll anchor from the sheet attributes
-                scrollAnchor = docView.getSheetViewAttribute(anchorName),
-                // the current effective scroll position
-                newScrollPos = getValidScrollPos(collection.convertScrollAnchorToPixel(scrollAnchor) - hiddenSize),
-                // the absolute position of the first visible pixel
-                visibleOffset = newScrollPos + hiddenSize,
-                // the absolute offset behind the last visible pixel
-                visibleEndOffset = visibleOffset + paneSize,
-                // the end offset of the available header cells
-                intervalEndOffset = position.offset + position.size;
-
-            // recalculate the interval on demand:
-            // - if there is currently no interval
-            // - if an explicit refresh descriptor has been passed
-            // - if the visible area is near the borders of the current interval
-            if (!interval || _.isObject(refresh) ||
-                ((position.offset > 0) && (visibleOffset - position.offset < paneSize * ADDITIONAL_SIZE_RATIO / 3)) ||
-                ((intervalEndOffset < collection.getTotalSize()) && (intervalEndOffset - visibleEndOffset < paneSize * ADDITIONAL_SIZE_RATIO / 3))
-            ) {
-
-                var // the available interval
-                    availableInterval = getAvailableInterval(),
-                    // the old (current) column/row interval
-                    oldInterval = interval,
-                    // the data of the new first column/row
-                    firstEntryData = collection.getEntryByOffset(visibleOffset - paneSize * ADDITIONAL_SIZE_RATIO, { pixel: true }),
-                    // the data of the new last column/row
-                    lastEntryData = collection.getEntryByOffset(visibleOffset + paneSize * (1 + ADDITIONAL_SIZE_RATIO), { pixel: true }),
-                    // whether to refresh the whole geometry of the columns/rows
-                    refreshGeometry = Utils.getBooleanOption(refresh, 'geometry', false),
-                    // index of the first changed column/row of a sheet operation
-                    refreshIndex = Utils.getIntegerOption(refresh, 'index', collection.getMaxIndex() + 1);
-
-                // calculate the new visible column/row interval
-                interval = {
-                    first: Math.max(availableInterval.first, (firstEntryData.offset === 0) ? 0 : firstEntryData.index),
-                    last: Math.min(availableInterval.last, (lastEntryData.offset + lastEntryData.size >= collection.getTotalSize()) ? collection.getMaxIndex() : lastEntryData.index)
-                };
-
-                // sanity check
-                if (interval.last < interval.first) {
-                    Utils.warn('HeaderPane.updateAndRenderInterval(): invalid interval: side=' + paneSide + ' interval=' + SheetUtils.getIntervalName(interval, columns));
-                    interval = null;
-                    return;
-                }
-
-                // start/end position of the interval (add all leading/trailing hidden columns/rows in the sheet)
-                position = { offset: firstEntryData.offset, size: lastEntryData.offset + lastEntryData.size - firstEntryData.offset };
-
-                // update scroll position according to the new interval
-                updateScrollPosition(newScrollPos);
-
-                // render all header cells and the selection if necessary (ignore column/row operations that have modified the trailing sheet area)
-                if (!_.isEqual(oldInterval, interval) || refreshGeometry || (refreshIndex <= interval.last)) {
-                    RenderUtils.takeTime('HeaderPane.updateAndRenderInterval(): side=' + paneSide + ' interval=' + SheetUtils.getIntervalName(interval, columns) + ' refresh=' + JSON.stringify(refresh), function () {
-
-                        // render header cells and the selection
-                        renderCells();
-                        renderSelection();
-
-                        // update bounding interval in the rendering cache of the active sheet
-                        // (use pane side position as identifier of the interval in the cache)
-                        docView.getRenderCache().updateBoundingInterval(paneSide, columns, _.clone(interval), refresh);
-
-                        // notify listeners
-                        self.trigger('change:interval', _.clone(interval), refresh, options);
-                    });
-                }
-            } else {
-                updateScrollPosition(newScrollPos);
-            }
-        }
-
-        /**
-         * Clears the current row/column interval. Triggers a 'hide:interval'
-         * event, if the interval was valid and visible before.
-         */
-        function hideInterval() {
-
-            // nothing to do, if the header pane is already hidden
-            if (!interval) { return; }
-
-            // reset members
-            interval = null;
-            scrollPos = scrollSize = 0;
-
-            // clear bounding interval in the rendering cache of the active sheet
-            // (use pane side position as identifier of the interval in the cache)
-            docView.getRenderCache().updateBoundingInterval(paneSide, columns, null);
-
-            // notify listeners
-            self.trigger('hide:interval');
+            return leading ? frozenInterval : new Interval(frozenInterval.last + 1, docModel.getMaxIndex(columns));
         }
 
         /**
          * Registers an event handler at the specified instance that will only
          * be invoked when this header pane is visible.
          */
-        function registerEventHandler(source, type, handler) {
-            source.on(type, function () {
+        function listenToWhenVisible(source, type, handler) {
+            self.listenTo(source, type, function () {
                 if (interval) {
                     handler.apply(self, _.toArray(arguments));
                 }
@@ -635,34 +491,23 @@ define('io.ox/office/spreadsheet/view/headerpane', [
                 updateFocusDisplay();
             }
 
-            // zoom or scroll position: update visible column/row interval
-            if (Utils.hasProperty(attributes, /^(zoom$|split)/)) {
-                // zooming and splitting: refresh cell positions and sizes
-                updateAndRenderInterval({ geometry: true });
-            } else if (anchorName in attributes) {
-                // scrolling: do not debounce updating the view (otherwise, cells are jumpy)
-                updateAndRenderInterval(null, { direct: true });
-            }
-
             // repaint the selected header cells
             if ('selection' in attributes) {
                 renderSelection();
             }
         }
 
-        /**
-         * Handles a finished column/row operation in the rendering cache
-         * (inserted, deleted, or modified columns or rows in the sheet), and
-         * recalculates the own visible column/row interval.
-         */
-        function renderCacheOperationHandler(event, col, row) {
-            var index = columns ? col : row;
-            if (_.isNumber(index)) {
-                updateAndRenderInterval({ index: index });
-            }
-        }
+        // public methods -----------------------------------------------------
 
-        // methods ------------------------------------------------------------
+        /**
+         * Returns the document view that contains this header pane.
+         *
+         * @returns {SpreadsheetView}
+         *  The document view that contains this header pane.
+         */
+        this.getDocView = function () {
+            return docView;
+        };
 
         /**
          * Returns the root DOM node of this header pane.
@@ -674,6 +519,17 @@ define('io.ox/office/spreadsheet/view/headerpane', [
             return rootNode;
         };
 
+        /**
+         * Returns the content node of this header pane, containing the cell
+         * nodes and resize tracking nodes.
+         *
+         * @returns {jQuery}
+         *  The content node of this header pane, as jQuery object.
+         */
+        this.getContentNode = function () {
+            return contentNode;
+        };
+
         /**
          * Returns the column or row collection of the active sheet, depending
          * on the direction of this header pane.
@@ -692,14 +548,13 @@ define('io.ox/office/spreadsheet/view/headerpane', [
          * @param {Object} settings
          *  The view settings of the pane side represented by this header pane.
          *  Supports the following properties:
-         *  @param {Boolean} settings.visible
-         *      Whether the header pane is visible.
          *  @param {Number} settings.offset
          *      The absolute offset of the header pane in the view root node,
          *      in pixels.
          *  @param {Number} settings.size
          *      The size of the header pane (width of column panes, height of
-         *      row panes), in pixels.
+         *      row panes), in pixels. The value zero will have the effect to
+         *      hide the entire header pane.
          *  @param {Boolean} settings.frozen
          *      Whether the header pane and the associated grid panes are
          *      frozen (not scrollable in the main direction).
@@ -718,11 +573,12 @@ define('io.ox/office/spreadsheet/view/headerpane', [
             hiddenSize = settings.hiddenSize;
 
             // initialize according to visibility
-            if (settings.visible) {
+            if (paneSize > 0) {
 
                 // show pane root node and initialize auto-scrolling
-                rootNode.show().enableTracking(Utils.extendOptions(PaneUtils.DEFAULT_TRACKING_OPTIONS, {
-                    autoScroll: settings.frozen ? false : (columns ? 'horizontal' : 'vertical'),
+                rootNode.show();
+                Tracking.enableTracking(rootNode, Utils.extendOptions(PaneUtils.DEFAULT_TRACKING_OPTIONS, {
+                    autoScroll: frozen ? false : (columns ? 'horizontal' : 'vertical'),
                     borderNode: rootNode
                 }));
 
@@ -732,32 +588,122 @@ define('io.ox/office/spreadsheet/view/headerpane', [
                     { top: settings.offset, width: docView.getHeaderWidth(), height: paneSize }
                 );
 
-                // recalculate the interval and header cells
-                updateAndRenderInterval();
-
-                // recalculate the scroll size (must be done deferred after the grid
-                // panes have been initialized, otherwise scroll position in trailing
-                // panes may be wrong when enabling a split view)
-                this.executeDelayed(function () {
-                    updateScrollAreaSize(scrollPos);
-                    updateScrollPosition(scrollPos);
-                });
-
                 // update CSS classes at root node for focus display
                 updateFocusDisplay();
 
             } else {
 
                 // hide pane root node and deinitialize auto-scrolling
-                rootNode.hide().disableTracking();
-
-                // reset the column/row interval, and notify listeners
-                hideInterval();
+                rootNode.hide();
+                Tracking.disableTracking(rootNode);
             }
 
             return this;
         };
 
+        /**
+         * Recalculates the current row/column interval and its position and
+         * size in the sheet, and renders all visible header cells and the
+         * selection on demand.
+         *
+         * @param {Object} [options]
+         *  Optional parameters:
+         *  @param {Boolean} [options.hide=false]
+         *      If set to true, the layer interval will be explicitly hidden.
+         *  @param {Boolean} [options.force=false]
+         *      If set to true, the layer interval will always be recalculated
+         *      regardless of the current scroll position. This option will be
+         *      ignored, if the option 'hide' has been set.
+         *
+         * @returns {HeaderBoundary|Null}
+         *  A descriptor containing the new layer interval and position (if the
+         *  properties in this object are null, the layer interval has been
+         *  hidden); or null, if nothing has changed in this header pane.
+         */
+        this.updateLayerInterval = function (options) {
+
+            // invalidates the layer interval and other members, and returns the descriptor for the caller
+            function invalidateIntervalAndReturn() {
+
+                // nothing to do, if the interval is already invalid
+                if (!interval) { return null; }
+
+                // reset members, return 'hide' flag
+                interval = position = null;
+                scrollPos = scrollSize = 0;
+                return new PaneUtils.HeaderBoundary();
+            }
+
+            // invalidate existing layer interval (header pane hidden, or 'hide' flag passed)
+            if ((paneSize === 0) || Utils.getBooleanOption(options, 'hide', false)) {
+                return invalidateIntervalAndReturn();
+            }
+
+            var // the column/row collection
+                collection = self.getColRowCollection(),
+                // the current scroll anchor from the sheet attributes
+                scrollAnchor = docView.getSheetViewAttribute(anchorName),
+                // the current effective scroll position
+                newScrollPos = getValidScrollPos(collection.convertScrollAnchorToPixel(scrollAnchor) - hiddenSize),
+                // the absolute position of the first visible pixel
+                visibleOffset = newScrollPos + hiddenSize,
+                // the absolute offset behind the last visible pixel
+                visibleEndOffset = visibleOffset + paneSize,
+                // the additional size before and after the visible area
+                additionalSize = frozen ? 0 : Math.ceil(paneSize * PaneUtils.ADDITIONAL_SIZE_RATIO),
+                // whether updating the interval is required due to passed refresh settings
+                forceRefresh = Utils.getBooleanOption(options, 'force', false);
+
+            // check if the layer interval does not need to be recalculated: layer interval exists, update not forced,
+            // and the visible area of the header pane is inside the layer interval (with a small distance)
+            if (interval && position && !forceRefresh && (frozen ||
+                (((position.offset <= 0) || (visibleOffset - position.offset >= additionalSize / 3)) &&
+                ((collection.getTotalSize() <= position.offset + position.size) || (position.offset + position.size - visibleEndOffset >= additionalSize / 3))
+            ))) {
+                updateScrollPosition(newScrollPos);
+                return null;
+            }
+
+            var // exact pixel size for the new interval with additional buffer size
+                // (round up to multiples of 100 to reduce expensive resizing while rendering)
+                boundSize = Utils.roundUp(paneSize + 2 * additionalSize, 100),
+                // exact pixel position for the new interval
+                boundOffset = visibleOffset - Math.round((boundSize - paneSize) / 2),
+                // the available interval
+                availableInterval = getAvailableInterval(),
+                // the data of the new first column/row
+                firstEntryData = collection.getEntryByOffset(boundOffset, { pixel: true }),
+                // the data of the new last column/row
+                lastEntryData = collection.getEntryByOffset(boundOffset + boundSize - 1, { pixel: true });
+
+            // calculate the new visible column/row interval
+            interval = new Interval(
+                Math.max(availableInterval.first, (firstEntryData.offset === 0) ? 0 : firstEntryData.index),
+                Math.min(availableInterval.last, (lastEntryData.offset + lastEntryData.size >= collection.getTotalSize()) ? collection.getMaxIndex() : lastEntryData.index)
+            );
+
+            // set the new exact pixel position
+            position = { offset: boundOffset, size: boundSize };
+
+            // sanity check
+            if (interval.last < interval.first) {
+                Utils.warn('HeaderPane.updateLayerInterval(): invalid interval: side=' + paneSide + ' interval=' + interval.stringifyAs(columns));
+                return invalidateIntervalAndReturn();
+            }
+
+            // update scroll size and position according to the new interval
+            updateScrollAreaSize(newScrollPos);
+            updateScrollPosition(newScrollPos);
+
+            // render header cells and the selection
+            renderCells();
+            renderSelection();
+
+            // return the new interval and position
+            RenderUtils.log('HeaderPane.updateLayerInterval(): side=' + paneSide + ' size=' + paneSize + ' interval=' + interval.stringifyAs(columns) + ' position=' + JSON.stringify(position));
+            return new PaneUtils.HeaderBoundary(interval, position);
+        };
+
         /**
          * Returns whether this header pane is currently visible (depending on
          * its position, and the view split settings of the active sheet).
@@ -766,7 +712,7 @@ define('io.ox/office/spreadsheet/view/headerpane', [
          *  Whether this header pane is currently visible.
          */
         this.isVisible = function () {
-            return _.isObject(interval);
+            return (paneSize > 0) && _.isObject(interval);
         };
 
         /**
@@ -777,7 +723,7 @@ define('io.ox/office/spreadsheet/view/headerpane', [
          * rows according to the position of this header pane, and the current
          * split settings.
          *
-         * @returns {Object|Null}
+         * @returns {Interval|Null}
          *  The complete column/row interval available in this header pane, in
          *  the zero-based index properties 'first' and 'last'; or null, if
          *  this header pane is currently hidden.
@@ -789,13 +735,12 @@ define('io.ox/office/spreadsheet/view/headerpane', [
         /**
          * Returns the column/row interval currently shown in this header pane.
          *
-         * @returns {Object|Null}
-         *  The column/row interval currently displayed in this header pane, in
-         *  the zero-based index properties 'first' and 'last'; or null, if
-         *  this header pane is currently hidden.
+         * @returns {Interval|Null}
+         *  The column/row interval currently displayed in this header pane; or
+         *  null, if this header pane is currently hidden.
          */
         this.getRenderInterval = function () {
-            return _.clone(interval);
+            return interval.clone();
         };
 
         /**
@@ -858,15 +803,17 @@ define('io.ox/office/spreadsheet/view/headerpane', [
                 collection = this.getColRowCollection(),
                 // the current scroll anchor
                 scrollAnchor = docView.getSheetViewAttribute(anchorName),
+                // the column/row index of the scroll anchor
+                scrollIndex = Math.floor(scrollAnchor),
                 // the layout data of the next column/row
                 nextEntry = null;
 
             // check if the cell is visible less than half, go to next cell in that case
-            if ((scrollAnchor.ratio > 0.5) || !collection.isEntryVisible(scrollAnchor.index)) {
-                nextEntry = collection.getNextVisibleEntry(scrollAnchor.index + 1);
+            if ((scrollAnchor % 1 > 0.5) || !collection.isEntryVisible(scrollIndex)) {
+                nextEntry = collection.getNextVisibleEntry(scrollIndex + 1);
             }
 
-            return nextEntry ? nextEntry.index : scrollAnchor.index;
+            return nextEntry ? nextEntry.index : scrollIndex;
         };
 
         /**
@@ -899,6 +846,9 @@ define('io.ox/office/spreadsheet/view/headerpane', [
                 updateScrollAreaSize(newScrollPos);
             }
 
+            // immediately update internal scroll position, this prevents unnecessary updates
+            scrollPos = newScrollPos;
+
             // update scroll anchor view properties, this will trigger change
             // handler from the view that repositions the content node
             docView.setSheetViewAttribute(anchorName, collection.getScrollAnchorByOffset(newScrollPos + hiddenSize, { pixel: true }));
@@ -997,42 +947,33 @@ define('io.ox/office/spreadsheet/view/headerpane', [
 
         // initialization -----------------------------------------------------
 
+        // tracking mix-in expects fully initialized header pane
+        HeaderTrackingMixin.call(this, paneSide);
+
         // marker for touch devices and browser types
-        Utils.addDeviceMarkers(rootNode);
+        Forms.addDeviceMarkers(rootNode);
 
-        // initialize class members
-        app.onInit(function () {
+        // create the context menu
+        contextMenu = new HeaderContextMenu(this, columns);
 
-            // the spreadsheet model and view
-            docModel = app.getModel();
-            docView = app.getView();
+        // update scroll area size according to the size of the used area in the sheet
+        this.listenTo(docView, 'change:usedarea', function () { updateScrollAreaSize(scrollPos); });
 
-            // create the context menu
-            contextMenu = new HeaderContextMenu(app, self, columns);
+        // listen to view events (only if this pane is visible)
+        listenToWhenVisible(docView, 'change:sheet:viewattributes', changeSheetViewAttributesHandler);
 
-            // hide the interval initially when the sheet changes
-            docView.on('change:activesheet', hideInterval);
-
-            // update scroll area size according to the size of the used area in the sheet
-            docView.on('change:usedarea', function () { updateScrollAreaSize(scrollPos); });
-
-            // listen to view events (only if this pane is visible)
-            registerEventHandler(docView, 'change:sheet:viewattributes', changeSheetViewAttributesHandler);
-            registerEventHandler(docView, 'cache:operation', renderCacheOperationHandler);
-
-            // update the position of the touch resizer node while tracking
-            if (Modernizr.touch) {
-                self.on({
-                    'resize:move': function (event, offset, size) { updateTouchResizerPosition(size); },
-                    'resize:end': function () { updateTouchResizerPosition(); }
-                });
-            }
-        });
+        // update the position of the touch resizer node while tracking
+        if (Utils.TOUCHDEVICE) {
+            self.on({
+                'resize:move': function (event, offset, size) { updateTouchResizerPosition(size); },
+                'resize:end': function () { updateTouchResizerPosition(); }
+            });
+        }
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
             contextMenu.destroy();
-            app = self = docModel = docView = contextMenu = null;
+            self = docModel = docView = contextMenu = null;
         });
 
     } // class HeaderPane
