@@ -6,19 +6,40 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/spreadsheet/view/labels', [
+    'io.ox/office/tk/utils',
     'io.ox/office/editframework/view/editlabels',
     'io.ox/office/spreadsheet/utils/sheetutils',
-    'gettext!io.ox/office/spreadsheet'
-], function (EditLabels, SheetUtils, gt) {
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, EditLabels, SheetUtils, gt) {
 
     'use strict';
 
+    var // UI labels for all supported function categories
+        FUNCTION_CATEGORIES = {
+            complex: /*#. Category for spreadsheet functions: complex numbers (e.g. IMSUM, IMSIN, IMPOWER) */ gt.pgettext('function-category', 'Complex'),
+            conversion: /*#. Category for spreadsheet functions: conversion (e.g. DEC2HEX, ROMAN, CONVERT) */ gt.pgettext('function-category', 'Conversion'),
+            database: /*#. Category for spreadsheet functions: database-like (e.g. DSUM, DCOUNT) */ gt.pgettext('function-category', 'Database'),
+            datetime: /*#. Category for spreadsheet functions: date/time calculation (e.g. TODAY, WEEKDAY) */ gt.pgettext('function-category', 'Date and time'),
+            engineering: /*#. Category for spreadsheet functions: engineering (e.g. BESSELI, ERF) */ gt.pgettext('function-category', 'Engineering'),
+            financial: /*#. Category for spreadsheet functions: financial functions (e.g. ACCRINT, COUPDAYS) */ gt.pgettext('function-category', 'Financial'),
+            information: /*#. Category for spreadsheet functions: information about cells (e.g. ISNUMBER, ISTEXT) */ gt.pgettext('function-category', 'Information'),
+            logical: /*#. Category for spreadsheet functions: logical functions (e.g. AND, OR, NOT, IF) */ gt.pgettext('function-category', 'Logical'),
+            math: /*#. Category for spreadsheet functions: common mathematical functions (e.g. SUM, SIN, ROUND) */ gt.pgettext('function-category', 'Math'),
+            matrix: /*#. Category for spreadsheet functions: matrix functions (e.g. MMULT, MINVERSE, MDETERM) */ gt.pgettext('function-category', 'Matrix'),
+            reference: /*#. Category for spreadsheet functions: cell references (e.g. ADDRESS, HLOOKUP, COLUMN) */ gt.pgettext('function-category', 'Reference'),
+            statistical: /*#. Category for spreadsheet functions: statistical functions (e.g. COUNT, STDEV, HYPGEOMDIST) */ gt.pgettext('function-category', 'Statistical'),
+            text: /*#. Category for spreadsheet functions: text functions (e.g. LEN, MID, REPLACE) */ gt.pgettext('function-category', 'Text')
+        };
+
+    var // convenience shortcuts
+        Address = SheetUtils.Address;
+
     // static class SpreadsheetLabels =========================================
 
     /**
@@ -168,6 +189,132 @@ define('io.ox/office/spreadsheet/view/labels', [
         dropDownVersion: { label: SpreadsheetLabels.FORMAT_PAINTER_LABEL }
     };
 
+    /**
+     * Standard options for a 'Insert rows' button.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.INSERT_ROWS_OPTIONS = {
+        label: gt('Insert rows')
+    };
+
+    /**
+     * Standard options for a 'Insert row' button.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.INSERT_ROW_OPTIONS = {
+        label: gt('Insert row')
+    };
+
+    /**
+     * Standard options for a 'Delete rows' button.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.DELETE_ROWS_OPTIONS = {
+        label: gt('Delete rows')
+    };
+
+    /**
+     * Standard options for a 'Delete row' button.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.DELETE_ROW_OPTIONS = {
+        label: gt('Delete row')
+    };
+
+    /**
+     * Standard options for 'Insert columns'.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.INSERT_COLUMNS_OPTIONS = {
+        label: gt('Insert columns')
+    };
+
+    /**
+     * Standard options for 'Insert column'.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.INSERT_COLUMN_OPTIONS = {
+        label: gt('Insert column')
+    };
+
+    /**
+     * Standard options for 'Delete columns'.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.DELETE_COLUMNS_OPTIONS = {
+        label: gt('Delete columns')
+    };
+
+    /**
+     * Standard options for 'Delete column'.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.DELETE_COLUMN_OPTIONS = {
+        label: gt('Delete column')
+    };
+
+    /**
+     * Standard options for 'filter function'.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.FILTER_OPTIONS = {
+        label: /*#. Button label: filter a cell range */ gt.pgettext('filter', 'Filter'),
+        tooltip: gt('Filter the selected cells'),
+        toggle: true
+    };
+
+    /**
+     * Standard options for 'sort function'.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.SORT_OPTIONS = {
+        label: /*#. sorting: label of the button opening a menu with custom sort options */ gt.pgettext('sort', 'Custom'),
+        tooltip: /*#. tooltip for custom sort button */ gt.pgettext('sort', 'Settings for sorting')
+    };
+
+    /**
+     * Standard options for 'clean function'.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.CLEAN_OPTIONS = {
+        label: gt('Clear contents')
+        //TODO: missing tooltip
+    };
+
+    // insert options ---------------------------------------------------------
+
+    /**
+     * Standard options for 'Insert sum'.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.INSERT_SUM_OPTIONS = {
+        label: gt('Sum'),
+        tooltip: /*#. automatically create a SUM function for selected cells */ gt('Sum automatically'),
+        value: 'SUM'
+    };
+
+    /**
+     * Standard options for 'Insert function'.
+     *
+     * @constant
+     */
+    SpreadsheetLabels.INSERT_FUNCTION_OPTIONS = {
+        label: gt('Function'),
+        tooltip: /*#. Insert a function at the current cursor location */ gt('Insert function at cursor location')
+    };
+
     // style sets for list controls -------------------------------------------
 
     /**
@@ -219,7 +366,7 @@ define('io.ox/office/spreadsheet/view/labels', [
             //#. UI label for a spreadsheet column, e.g. "Column A", "Column B", etc.
             //#. %1$s is the name of the column
             //#, c-format
-            gt('Column %1$s', _.noI18n(SheetUtils.getColName(col))));
+            gt('Column %1$s', _.noI18n(Address.stringifyCol(col))));
     };
 
     /**
@@ -234,29 +381,28 @@ define('io.ox/office/spreadsheet/view/labels', [
             //#. UI label for a spreadsheet row, e.g. "Row 1", "Row 2", etc.
             //#. %1$s is the name of the row
             //#, c-format
-            gt('Row %1$s', _.noI18n(SheetUtils.getRowName(row))));
+            gt('Row %1$s', _.noI18n(Address.stringifyRow(row))));
     };
 
-
     /**
-     * Returns the localized sheet name with the specified index.
+     * Returns the localized name of the specified function category.
      *
-     * @param {Number} sheet
-     *  The zero-based sheet index to be inserted into the sheet name. The
-     *  number inserted into the sheet name will be increased by one.
+     * @param {String} category
+     *  The internal identifier of a function category.
      *
      * @returns {String}
-     *  The generated sheet name (e.g. 'Sheet1', 'Sheet2', etc.).
+     *  The localized name of the specified function category.
      */
-    SpreadsheetLabels.getSheetName = function (sheet) {
-        return (
-            //#. Default sheet names in spreadsheet documents. Should be equal to the sheet
-            //#. names used in existing spreadsheet applications (Excel, OpenOffice Calc, ...).
-            //#. %1$d is the numeric index inside the sheet name
-            //#. Resulting names e.g. "Sheet1", "Sheet2", etc.
-            //#. No white-space between the "Sheet" label and sheet index!
-            //#, c-format
-            gt('Sheet%1$d', _.noI18n(sheet + 1)));
+    SpreadsheetLabels.getCategoryLabel = function (category) {
+
+        // return labels for valid categories
+        if (category in FUNCTION_CATEGORIES) {
+            return FUNCTION_CATEGORIES[category];
+        }
+
+        // assert an unknown category, return it capitalized
+        Utils.error('SpreadsheetLabels.getCategoryLabel(): unknown function category: "' + category + '"');
+        return _.noI18n(Utils.capitalize(category));
     };
 
     // exports ================================================================
