@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
@@ -14,13 +14,14 @@
 
 define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin', [
     'io.ox/office/tk/utils',
+    'io.ox/office/tk/utils/tracking',
     'io.ox/office/spreadsheet/utils/sheetutils',
     'io.ox/office/spreadsheet/utils/paneutils'
-], function (Utils, SheetUtils, PaneUtils) {
+], function (Utils, Tracking, SheetUtils, PaneUtils) {
 
     'use strict';
 
-    // class HeaderTrackingMixin ==============================================
+    // mix-in class HeaderTrackingMixin =======================================
 
     /**
      * Mix-in class for the class HeaderPane that implements all kinds of mouse
@@ -29,16 +30,10 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin', [
      *
      * @constructor
      *
-     * @param {SpreadsheetApplication} app
-     *  The application that contains this header pane.
-     *
      * @param {String} paneSide
      *  The pane side identifier of this header pane.
-     *
-     * @param {jQuery} contentNode
-     *  The content node with the header cell and resizer nodes
      */
-    function HeaderTrackingMixin(app, paneSide, contentNode) {
+    function HeaderTrackingMixin(paneSide) {
 
         var // self reference
             self = this,
@@ -47,8 +42,11 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin', [
             columns = PaneUtils.isColumnSide(paneSide),
 
             // the spreadsheet model and view
-            docModel = null,
-            docView = null,
+            docView = this.getDocView(),
+            docModel = docView.getDocModel(),
+
+            // the (scrolled) content node of the header pane
+            contentNode = this.getContentNode(),
 
             // the type of the current tracking
             trackingType = null,
@@ -66,7 +64,7 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin', [
         function editModeHandler(event, editMode) {
             // stop tracking if edit rights have been lost
             if (!editMode && !readOnlyTracking) {
-                $.cancelTracking();
+                Tracking.cancelTracking();
             }
         }
 
@@ -75,7 +73,7 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin', [
          */
         function changeSelectionHandler() {
             if (trackingType === 'resize') {
-                $.cancelTracking();
+                Tracking.cancelTracking();
             }
         }
 
@@ -158,6 +156,11 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin', [
                 self.scrollToEntry(currentIndex);
                 docView.setSheetViewAttribute('activePaneSide', null);
                 self.trigger('select:end', currentIndex, success);
+
+                // ios fires no native 'contextmenu'-event, so i have to trigger it
+                if (_.device('ios') && event.duration > 750) {
+                    contentNode.find('.cell[data-index="' + currentIndex + '"]').trigger('contextmenu');
+                }
             }
 
             // return the actual selectionTrackingHandler() method
@@ -165,7 +168,6 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin', [
                 switch (event.type) {
                 case 'tracking:start':
                     initializeTracking(event);
-                    event.preventDefault(); // prevent native scrolling on touch devices
                     break;
                 case 'tracking:move':
                     updateTracking(event);
@@ -216,7 +218,7 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin', [
                 minScrollPos = Math.max(0, entryData.offset - 20 - hiddenSize);
                 maxScrollPos = Math.max(minScrollPos, entryData.offset + MAX_SIZE + 20 - position.size - hiddenSize);
                 self.trigger(resizeEvent, entryData.offset, currentSize);
-                if (resizeEvent.isDefaultPrevented()) { $.cancelTracking(); }
+                if (resizeEvent.isDefaultPrevented()) { Tracking.cancelTracking(); }
             }
 
             // updates column/row resizing according to the passed tracking event
@@ -329,29 +331,21 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin', [
 
         // initialization -----------------------------------------------------
 
-        // initialize class members
-        app.onInit(function () {
+        // enable/disable operations that modify the document according to edit mode
+        this.listenTo(docModel, 'change:editmode', editModeHandler);
 
-            // the spreadsheet model and view instances
-            docModel = app.getModel();
-            docView = app.getView();
+        // listen to view events
+        this.listenTo(docView, 'change:selection', changeSelectionHandler);
 
-            // enable/disable operations that modify the document according to edit mode
-            docModel.on('change:editmode', editModeHandler);
+        // tracking for column/row selection with mouse/touch
+        this.getNode().on('tracking:start', trackingStartHandler);
 
-            // listen to view events
-            docView.on('change:selection', changeSelectionHandler);
-
-            // tracking for column/row selection with mouse/touch
-            self.getNode().on('tracking:start', trackingStartHandler);
-
-            // double click on resizer for column/row
-            self.getNode().on('dblclick', '.resizer', doubleClickHandler);
-        });
+        // double click on resizer for column/row
+        this.getNode().on('dblclick', '.resizer', doubleClickHandler);
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = contentNode = self = docModel = docView = null;
+            self = docModel = docView = contentNode = null;
         });
 
     } // class HeaderTrackingMixin
