@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Michael Nimz <michael.nimz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/spreadsheet/view/mixin/sortmixin', [
     'io.ox/office/spreadsheet/utils/config',
     'io.ox/office/spreadsheet/utils/sheetutils',
     'io.ox/office/spreadsheet/view/popup/sortlayermenu',
-    'gettext!io.ox/office/spreadsheet'
+    'gettext!io.ox/office/spreadsheet/main'
 ], function (Config, SheetUtils, SortLayerMenu, gt) {
 
     'use strict';
@@ -27,36 +27,30 @@ define('io.ox/office/spreadsheet/view/mixin/sortmixin', [
      * sorting selected ranges.
      *
      * @constructor
-     *
-     * @param {SpreadsheetApplication} app
-     *  The spreadsheet application instance.
      */
-    function SortMixin(app) {
+    function SortMixin() {
 
         var // self reference (the spreadsheet view)
             self = this,
 
             // custom sort menu
-            sortLayerMenu = new SortLayerMenu(app),
+            sortLayerMenu = new SortLayerMenu(this),
 
             // normal (toggle-)buttons, or custom sort
             type = null,
 
-            // selection
-            selection = null,
-
             // maximum number of allowed columns/rows
             maxColRow = 2000,
 
-            // selected ranges
-            ranges = null,
-
             // first selected range
             range = null,
 
             // range after expanding
             newRange = null,
 
+            // should the selected range be checked
+            doCheck = true,
+
             // state of sorting
             sortObj = {
                 custom: {
@@ -84,19 +78,23 @@ define('io.ox/office/spreadsheet/view/mixin/sortmixin', [
         // private methods ----------------------------------------------------
 
         /**
-          * Sends the sort-ajax-request to the backend
-          */
+         * Sends the sort-ajax-request to the backend
+         *
+         * @returns {jQuery.Promise}
+         *  A promise representing the AJAX request.
+         */
         function sendSortRequest() {
-            var options = sortObj[type],
-                arrDirections = ['vertical', 'horizontal'],
-                directionIndex = arrDirections.indexOf(options.direction),
-                colRowIndex = options.colrow || selection.activeCell[directionIndex];
 
-            app.sendActionRequest('sort', {
+            var options = sortObj[type],
+                columns = options.direction === 'vertical',
+                range = self.getSelectedRanges().first(),
+                colRowIndex = options.colrow || self.getActiveCell().get(columns);
+
+            return self.getApp().sendActionRequest('sort', {
                 locale:             Config.LOCALE,
                 sheet:              self.getActiveSheet(),
-                start:              range.start,
-                end:                range.end,
+                start:              range.start.toJSON(),
+                end:                range.end.toJSON(),
                 // options
                 headers:            options.hasheadline,
                 direction:          options.direction,
@@ -111,30 +109,28 @@ define('io.ox/office/spreadsheet/view/mixin/sortmixin', [
         }
 
         /**
-          * Expands the selection if needed
-          *
-          * @return {Boolean}
-          */
+         * Expands the selection if needed
+         *
+         * @return {Boolean}
+         */
         function expandSelection() {
+
             self.setSortOption('preventClose', true, type);
             self.setSortOption('preventReset', true, type);
 
-            // yell, if the new range has too many columns/rows included
-            if (SheetUtils.getColCount(newRange) > SheetUtils.MAX_CHANGE_COLS_COUNT) {
+            // yell, if the new range has too many columns/rows included (returns false)
+            if (newRange.cols() > SheetUtils.MAX_CHANGE_COLS_COUNT) {
                 return self.yellOnResult('cols:overflow');
             }
-            if (SheetUtils.getRowCount(newRange) > SheetUtils.MAX_CHANGE_ROWS_COUNT) {
+            if (newRange.rows() > SheetUtils.MAX_CHANGE_ROWS_COUNT) {
                 return self.yellOnResult('rows:overflow');
             }
 
             // select the new range
-            self.selectRange(newRange, { active: selection.activeCell });
-
-            // get new/current selection
-            selection = self.getSelection();
+            self.selectRange(newRange, { active: self.getActiveCell() });
 
             // the selected range
-            range = selection.ranges[0];
+            range = self.getSelectedRanges().first();
 
             return true;
         }
@@ -146,18 +142,17 @@ define('io.ox/office/spreadsheet/view/mixin/sortmixin', [
          */
         function possibleSelection() {
 
-            // get current selection
-            selection = self.getSelection();
-
-            // the selected range(s)
-            ranges = selection.ranges;
-            range = selection.ranges[0];
-
             // get out here, sheet is protected
-            if (!self.requireUnlockedActiveSheet()) {
+            if (!self.requireEditableSelection({ lockTables: 'header' })) {
                 return false;
             }
 
+            var // get current selection
+                ranges = self.getSelectedRanges();
+
+            // the selected range(s)
+            range = ranges.first();
+
             // get out here, if more than one range is selected
             if (ranges.length > 1) {
                 self.yellOnResult('sort:multiselect');
@@ -170,7 +165,7 @@ define('io.ox/office/spreadsheet/view/mixin/sortmixin', [
                 return false;
             }
 
-            if ((range.end[0] - range.start[0]) > maxColRow || (range.end[1] - range.start[1]) > maxColRow) {
+            if ((range.cols() > maxColRow) || (range.rows() > maxColRow)) {
                 self.yell({ type: 'info', message: gt('The command cannot be performed on more than 2000 columns or rows') });
                 return false;
             }
@@ -181,27 +176,35 @@ define('io.ox/office/spreadsheet/view/mixin/sortmixin', [
          * Check if the selection is expandable or actually must expand.
          */
         function checkSelectionRange() {
+
             newRange = self.getCellCollection().findContentRange(range);
 
             // the selection is only within one row/col. Ask the user whether expand selection or not.
-            if ((range.start[1] === range.end[1] || range.start[0] === range.end[0]) && !_.isEqual(range.start, range.end) && !_.isEqual(newRange, range)) {
+            if ((range.singleCol() !== range.singleRow()) && newRange.differs(range)) {
+
                 // ask the user to expand the current selection
-                return self.showExpandSelectionForSortingDialog()
-                    .done(function(){
-                        // expand the selection automatically to current used rectangle
-                        expandSelection();
-                    })
-                    .always(function () {
-                        if (type === 'custom') {
-                            sortLayerMenu.show();
-                        } else {
-                            sendSortRequest();
-                        }
-                    });
+                var promise = self.showQueryDialog(gt('Sorting'), gt('Expand the selection?'), { cancel: true });
+
+                return promise.done(function (answer) {
+
+                    // expand the selection automatically to current used rectangle
+                    if (answer) { expandSelection(); }
+
+                    if (type === 'custom') {
+                        sortLayerMenu.show();
+                        self.executeDelayed(function () {
+                            sortLayerMenu.grabFocus({ bottom: true });
+                        }, 100);
+                    } else {
+                        sendSortRequest();
+                    }
+                    // stop checking the selected range. The user has made a decision already for this selection.
+                    doCheck = false;
+                });
             }
 
             // only one cell is selected. Expand selection automatically (don't ask the user).
-            if (_.isEqual(range.start, range.end) && !_.isEqual(newRange, range)) {
+            if (range.single() && newRange.differs(range)) {
                 // expand the selection automatically to current used rectangle
                 return expandSelection();
             }
@@ -217,8 +220,8 @@ define('io.ox/office/spreadsheet/view/mixin/sortmixin', [
         this.getSortState = function (sorttype) {
             var type = sorttype || 'standard',
                 states = {
-                    'ascending': 'descending',
-                    'descending': 'ascending'
+                    ascending: 'descending',
+                    descending: 'ascending'
                 };
 
             if (this.getSortOption('state', type) === null) {
@@ -239,7 +242,7 @@ define('io.ox/office/spreadsheet/view/mixin/sortmixin', [
           */
         this.getSortOption = function (key, sorttype) {
             var type = sorttype || 'default';
-            return _.has(sortObj[type], key)?sortObj[type][key]:false;
+            return _.has(sortObj[type], key) ? sortObj[type][key] : false;
         };
 
         /**
@@ -298,81 +301,102 @@ define('io.ox/office/spreadsheet/view/mixin/sortmixin', [
                         preventDialog: false
                     }
                 };
-            _.each(defaults[sorttype], function(value, key){
+            _.each(defaults[sorttype], function (value, key) {
                 self.setSortOption(key, value, sorttype);
             });
 
-            app.getController().update();
+            this.getApp().getController().update();
         };
 
-
         /**
          * Checks the sort-environment. Decides, whether the selection should expand, or in doubt, asks the user.
-         *
-         * @returns {jQuery.Promise}
          */
-        this.showSortExtensionDialog = function () {
+        this.showSortMenu = function () {
             type = 'custom';
-            if (possibleSelection() && checkSelectionRange() === true) {
+            if (possibleSelection() && (checkSelectionRange() === true)) {
                 sortLayerMenu.show();
+                this.executeDelayed(function () {
+                    sortLayerMenu.grabFocus({ bottom: true });
+                }, 100);
             }
         };
 
         /**
-          * Prepares and executes the sorting
-          *
-          * @param {String} sorttype
-          *  The type on which currently be sorted ('standard', 'custom')
-          *
-          * @param {String} order
-          *  The order-method ('ascending', 'descending', ...)
-          */
+         * Prepares and executes the sorting
+         *
+         * @param {String} sorttype
+         *  The type on which currently be sorted ('standard', 'custom')
+         *
+         * @param {String} order
+         *  The order-method ('ascending', 'descending', ...)
+         *
+         * @returns {jQuery.Promise}
+         *  A promise that will be resolved after the selected cell range has
+         *  been sorted.
+         */
         this.sortCellRange = function (sorttype, order) {
-            // type of sorting (standard || custom)
+
+            var // self reference
+                self = this,
+                // set toggle button state
+                pushToggleState = false,
+                // expand-dialog
+                dialog = null;
+
+            // set "global" variable 'type' of sorting (standard || custom)
             type = sorttype;
 
             // set defaults for given type
             switch (type) {
-                case 'custom':
-                    if (this.getSortState(type) === null) {
-                        this.setSortOption('hasheadline', false, type);
+            case 'custom':
+                if (this.getSortState(type) === null) {
+                    this.setSortOption('hasheadline', false, type);
+                }
+                this.setSortOption('state', order, type);
+                break;
+
+            default:
+                // if the toggle button is clicked
+                if (order === 'toggle') {
+                    // get last state
+                    order = this.getSortState();
+                    // prepare updating the toggle button state
+                    pushToggleState = true;
+                }
+
+                // set selected order direction
+                this.setSortOption('order', order, type);
+                // reset headline state to automatic detection
+                this.setSortOption('hasheadline', 'auto', type);
+
+                // if the current selection isn't sortable
+                if (!possibleSelection()) { return $.when(); }
+
+                if (doCheck) {
+                    dialog = checkSelectionRange();
+                    if (dialog !== true) {
+                        return dialog.done(function () {
+                            // set new/current sort state to toggle button
+                            self.setSortOption('state', pushToggleState ? order : null, type);
+                        });
                     }
-                    this.setSortOption('state', order, type);
-                    break;
+                }
 
-                default:
-                    // if the toggle-button is clicked
-                    if (order === 'toggle') {
-                        // get last state
-                        order = this.getSortState();
-                        // and toggle it
-                        this.setSortOption('state', order, type);
+                // set new/current sort state to toggle button
+                this.setSortOption('state', pushToggleState ? order : null, type);
 
-                    // if sorting is directly selected in DropDown-Menu (asc or desc)
-                    } else {
-                        // reset (toggle-button-)state
-                        this.setSortOption('state', null, type);
-                    }
-
-                    // set selected order-direction
-                    this.setSortOption('order', order, type);
-                    // reset headline-state to automatic detection
-                    this.setSortOption('hasheadline', 'auto', type);
-
-                    if (!possibleSelection()) { return $.when(); }
-
-                    if (checkSelectionRange() !== true) { return $.when(); }
-
-                    break;
+                break;
             }
 
             // send the sort request to the server
-            sendSortRequest();
+            var promise = sendSortRequest();
 
             if (type === 'custom') {
                 self.resetSortOptions('custom');
                 sortLayerMenu.hide();
             }
+
+            return promise;
         };
 
         // initialization -----------------------------------------------------
@@ -380,12 +404,14 @@ define('io.ox/office/spreadsheet/view/mixin/sortmixin', [
         // reset the state, if the selection changes (sort settings won't be saved)
         this.on('change:selection', function () {
             self.resetSortOptions('standard');
+            // reset also the expand decision
+            doCheck = true;
         });
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
             sortLayerMenu.destroy();
-            app = self = sortLayerMenu = type = selection = ranges = range = newRange = sortObj = null;
+            self = sortLayerMenu = type = range = newRange = sortObj = null;
         });
 
     } // class SortMixin
