@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,9 +15,9 @@ define('io.ox/office/spreadsheet/view/popup/headercontextmenu', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/button',
     'io.ox/office/baseframework/view/popup/contextmenu',
-    'io.ox/office/spreadsheet/utils/sheetutils',
-    'gettext!io.ox/office/spreadsheet'
-], function (Utils, Button, ContextMenu, SheetUtils, gt) {
+    'io.ox/office/spreadsheet/view/labels',
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, Button, ContextMenu, Labels, gt) {
 
     'use strict';
 
@@ -30,23 +30,29 @@ define('io.ox/office/spreadsheet/view/popup/headercontextmenu', [
      * @constructor
      *
      * @extends ContextMenu
+     *
+     * @param {HeaderPane} headerPane
+     *  The header pane that contains this instance.
+     *
+     * @param {Boolean} columns
+     *  Whether this context menu is for columns (true), or for rows (false).
      */
-    function HeaderContextMenu(app, headerPane, columns) {
+    function HeaderContextMenu(headerPane, columns) {
 
         var // self reference
             self = this,
 
             // the document model and view
-            docModel = app.getModel(),
-            docView = app.getView();
+            docView = headerPane.getDocView(),
+            docModel = docView.getDocModel();
 
         // base constructor ---------------------------------------------------
 
-        ContextMenu.call(this, app, headerPane.getNode(), {
+        ContextMenu.call(this, docView, headerPane.getNode(), {
             selector: '.cell', // only on cell header nodes (not on resizers)
-            mouseDown: true, // open context menu on 'mousedown' instead of 'mouseup'
             // do not show context menu at all in locked sheets, or during cell edit mode
-            enableKey: ['sheet/unlocked', 'view/cell/editmode/off']
+            enableKey: 'sheet/operation/unlocked/cell',
+            delay: 200
         });
 
         // private methods ----------------------------------------------------
@@ -55,24 +61,38 @@ define('io.ox/office/spreadsheet/view/popup/headercontextmenu', [
          * Inserts column-specific menu entries into the context menu.
          */
         function initColumnMenu() {
-            self.addGroup('column/insert', new Button({ label: gt('Insert columns') }))
-                .addGroup('column/delete', new Button({ label: gt('Delete columns') }))
+            var selRanges = docView.getSelectedRanges(),
+                multiCols = (selRanges.colIntervals().size() > 1);
+
+            self.destroyAllGroups();
+
+            self.addGroup('column/insert', new Button((multiCols ? Labels.INSERT_COLUMNS_OPTIONS : Labels.INSERT_COLUMN_OPTIONS)))
+                .addGroup('column/delete', new Button((multiCols ? Labels.DELETE_COLUMNS_OPTIONS : Labels.DELETE_COLUMN_OPTIONS)))
                 .addSeparator()
                 .addGroup('column/width/optimal', new Button({ label: gt('Set optimal column width') }))
-                .addGroup('column/hide', new Button({ label: gt('Hide columns') }))
+                .addGroup('column/hide', new Button({ label: (multiCols ? gt('Hide columns') : gt('Hide column')) }))
                 .addGroup('column/show', new Button({ label: gt('Show hidden columns') }));
+
+            self.updateAllGroups();
         }
 
         /**
          * Inserts row-specific menu entries into the context menu.
          */
         function initRowMenu() {
-            self.addGroup('row/insert', new Button({ label: gt('Insert rows') }))
-                .addGroup('row/delete', new Button({ label: gt('Delete rows') }))
+            var selRanges = docView.getSelectedRanges(),
+                multiRows = (selRanges.rowIntervals().size() > 1);
+
+            self.destroyAllGroups();
+
+            self.addGroup('row/insert', new Button((multiRows ? Labels.INSERT_ROWS_OPTIONS : Labels.INSERT_ROW_OPTIONS)))
+                .addGroup('row/delete', new Button((multiRows ? Labels.DELETE_ROWS_OPTIONS : Labels.DELETE_ROW_OPTIONS)))
                 .addSeparator()
                 .addGroup('row/height/optimal', new Button({ label: gt('Set optimal row height') }))
-                .addGroup('row/hide', new Button({ label: gt('Hide rows') }))
+                .addGroup('row/hide', new Button({ label: (multiRows ? gt('Hide rows') : gt('Hide row')) }))
                 .addGroup('row/show', new Button({ label: gt('Show hidden rows') }));
+
+            self.updateAllGroups();
         }
 
         /**
@@ -84,43 +104,57 @@ define('io.ox/office/spreadsheet/view/popup/headercontextmenu', [
             var // the source event that caused to open the context menu
                 sourceEvent = event.sourceEvent,
                 // the index of the column/row to be selected
-                index = Utils.getElementAttributeAsInteger(sourceEvent.currentTarget, 'data-index', -1);
+                index = Utils.getElementAttributeAsInteger(sourceEvent.target, 'data-index', -1);
 
             // functor to check whether the column/row index is contained in a range address
             function rangeContainsIndex(range) {
-                return docModel.isFullRange(range, columns) && SheetUtils.rangeContainsIndex(range, index, columns);
+                return docModel.isFullRange(range, columns) && range.containsIndex(index, columns);
             }
 
-            // Do nothing if no valid cell node has been clicked, or when the event
-            // is of type 'contextmenu' (otherwise, the context menu will be reopened
-            // after moving the mouse with pressed right mouse button).
-            if ((index < 0) || (sourceEvent.type === 'contextmenu')) {
+            // Do nothing if no valid cell node has been clicked
+            if ((index < 0)) {
                 event.preventDefault();
                 return;
             }
 
             // select the clicked column/row if it is not contained in the selection
-            if (!_.any(docView.getSelectedRanges(), rangeContainsIndex)) {
+            if (!docView.getSelectedRanges().some(rangeContainsIndex)) {
                 docView.grabFocus();
                 headerPane.trigger('select:start', index, 'select');
                 headerPane.trigger('select:end', index, true);
             }
         }
 
+        // public methods ----
+        this.getXY = function (event) {
+            if (event.sourceEvent.pageX && event.sourceEvent.pageY) {
+                return {
+                    pageX: event.sourceEvent.pageX,
+                    pageY: event.sourceEvent.pageY
+                };
+            } else {
+                var xy = $(event.sourceEvent.target).offset();
+                return {
+                    pageX: (xy.left + ($(event.sourceEvent.target).width() / 2)),
+                    pageY: (xy.top + ($(event.sourceEvent.target).height() / 2))
+                };
+            }
+        };
+
         // initialization -----------------------------------------------------
 
-        // suppress system context menu on other nodes (resizers etc.)
-        headerPane.getNode().on('contextmenu', ':not(.cell)', false);
-
         // lazy initialization of the menu items according to the orientation of the header pane
-        this.one('popup:beforeshow', columns ? initColumnMenu : initRowMenu);
+        this.on('popup:beforeshow', columns ? initColumnMenu : initRowMenu);
 
         // preprocessing before the context menu will be shown
         this.on('contextmenu:prepare', contextMenuPrepareHandler);
 
+        // hide contextmenu if the user start scrolling
+        this.listenTo(docView, 'change:scrollpos', function () { self.hide(); });
+
         // destroy all class members
         this.registerDestructor(function () {
-            app = headerPane = self = docModel = docView = null;
+            self = headerPane = docModel = docView = null;
         });
 
     } // class HeaderContextMenu
