@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -30,30 +30,34 @@ define('io.ox/office/spreadsheet/view/popup/sheetcontextmenu', [
      * @constructor
      *
      * @extends ContextMenu
+     *
+     * @param {SpreadsheetView} docView
+     *  The spreadsheet view containing this instance.
      */
-    function SheetContextMenu(app, rootNode) {
+    function SheetContextMenu(docView, rootNode) {
 
         var // self reference
-            self = this;
+            self = this,
+            // app reference
+            app = docView.getApp();
 
         // base constructor ---------------------------------------------------
 
-        ContextMenu.call(this, app, rootNode, {
+        ContextMenu.call(this, docView, rootNode, {
             selector: Forms.OPTION_BUTTON_SELECTOR,
-            mouseDown: true, // open context menu on 'mousedown' instead of 'mouseup'
             // do not show context menu in read-only mode, or during cell edit mode
-            enableKey: ['document/editable', 'view/cell/editmode/off']
+            enableKey: ['document/editable', 'view/cell/editmode/off'],
+            delay: 200,
+            lazyInitHandler: initMenu
         });
 
         // private methods ----------------------------------------------------
-
         /**
          * Activate the clicked sheet, before the context menu will be shown.
          */
         function contextMenuPrepareHandler(event) {
-
             var // the clicked button element
-                sheetButton = $(event.sourceEvent.currentTarget);
+                sheetButton = $(event.sourceEvent.target);
 
             // click the button if it is not selected (activate the sheet)
             if (!Forms.isCheckedButtonNode(sheetButton)) {
@@ -61,27 +65,41 @@ define('io.ox/office/spreadsheet/view/popup/sheetcontextmenu', [
             }
         }
 
-        // initialization -----------------------------------------------------
-
-        // lazy initialization of the  menu items
-        this.one('popup:beforeshow', function () {
+        function initMenu() {
             self.addGroup('sheet/insert', new Button({ label: Labels.INSERT_SHEET_LABEL }))
                 .addSeparator()
-                .addGroup('sheet/rename/dialog', new Button({ label: Labels.RENAME_SHEET_LABEL }))
-                .addGroup('sheet/copy/dialog', new Button({ label: Labels.COPY_SHEET_LABEL }))
-                .addGroup('sheet/delete', new Button({ label: Labels.DELETE_SHEET_LABEL }))
+                .addGroup('sheet/rename/dialog', new Button({ label: Labels.RENAME_SHEET_LABEL }));
+
+            if (app.isOOXML()) {
+                self.addGroup('sheet/copy/dialog', new Button({ label: Labels.COPY_SHEET_LABEL }));
+            }
+
+            self.addGroup('sheet/delete', new Button({ label: Labels.DELETE_SHEET_LABEL }))
                 .addGroup('sheet/visible', new Button({ label: Labels.HIDE_SHEET_LABEL, value: false }))
                 .addSeparator()
                 .addGroup('sheet/reorder/dialog', new Button({ label: Labels.REORDER_SHEETS_LABEL }))
                 .addGroup('sheet/showall', new Button({ label: Labels.SHOW_SHEETS_LABEL }));
-        });
+        }
+
+        // public methods -----------------------------------------------------
+        this.getXY = function (event) {
+            return {
+                pageX: event.sourceEvent.pageX,
+                pageY: event.sourceEvent.pageY
+            };
+        };
+
+        // initialization -----------------------------------------------------
 
         // preprocessing before the context menu will be shown
         this.on('contextmenu:prepare', contextMenuPrepareHandler);
 
+        // hide contextmenu if the user start scrolling
+        this.listenTo(docView, 'change:scrollpos', function () { self.hide(); });
+
         // destroy all class members
         this.registerDestructor(function () {
-            app = rootNode = self = null;
+            self = docView = rootNode = null;
         });
 
     } // class SheetContextMenu
