@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -28,16 +28,19 @@ define('io.ox/office/spreadsheet/view/popup/signaturetooltip', [
      *
      * @extends ToolTip
      *
-     * @param {SpreadsheetApplication} app
-     *  The spreadsheet application instance.
+     * @param {SpreadsheetView} docView
+     *  The spreadsheet view containing this instance.
      *
      * @param {jQuery} textAreaNode
      *  The <textarea> element used as anchor for the tooltip node.
      */
-    function SignatureToolTip(app, textAreaNode) {
+    function SignatureToolTip(docView, textAreaNode) {
 
-        var// current list separator, with a trailing space character
-            SEP = Utils.escapeHTML(app.getListSeparator() + ' ');
+        var // the application instance
+            app = docView.getApp(),
+
+            // current list separator, with a trailing space character
+            SEP = null;
 
         // base constructor ---------------------------------------------------
 
@@ -74,7 +77,7 @@ define('io.ox/office/spreadsheet/view/popup/signaturetooltip', [
         function getParameterHelp(functionHelp, paramIndex) {
 
             var // whether the parameter is part of a repetition cycle
-                repeated = (0 <= functionHelp.repeatStart) && (functionHelp.repeatStart <= paramIndex),
+                repeated = (functionHelp.repeatStart >= 0) && (functionHelp.repeatStart <= paramIndex),
                 // index of the repetition cycle (used as suffix for parameter names)
                 cycleIndex = repeated ? Math.floor((paramIndex - functionHelp.repeatStart) / functionHelp.cycleLength) : -1,
                 // relative parameter index in current repetition cycle
@@ -148,7 +151,7 @@ define('io.ox/office/spreadsheet/view/popup/signaturetooltip', [
          * function.
          *
          * @param {String} funcName
-         *  The localized name of a spreadsheet function (case-insensitive).
+         *  The native (English) name of a sheet function (case-insensitive).
          *
          * @param {Number} paramIndex
          *  The zero-based index of a parameter of the specified function that
@@ -231,9 +234,14 @@ define('io.ox/office/spreadsheet/view/popup/signaturetooltip', [
 
         // initialization -----------------------------------------------------
 
+        // initialize the list separator shown between function parameters
+        app.onInitLocale(function () {
+            SEP = Utils.escapeHTML(app.getListSeparator() + ' ');
+        });
+
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = textAreaNode = null;
+            app = docView = textAreaNode = null;
         });
 
     } // class SignatureToolTip
