@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Michael Nimz <michael.nimz@open-xchange.com>
  */
@@ -17,10 +17,9 @@ define('io.ox/office/spreadsheet/view/popup/sortlayermenu', [
     'io.ox/office/tk/control/radiogroup',
     'io.ox/office/tk/control/radiolist',
     'io.ox/office/baseframework/view/popup/layermenu',
-    'io.ox/office/spreadsheet/utils/sheetutils',
     'io.ox/office/spreadsheet/view/labels',
-    'gettext!io.ox/office/spreadsheet'
-], function (Utils, CheckBox, RadioGroup, RadioList, LayerMenu, SheetUtils, Labels, gt) {
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, CheckBox, RadioGroup, RadioList, LayerMenu, Labels, gt) {
 
     'use strict';
 
@@ -32,33 +31,34 @@ define('io.ox/office/spreadsheet/view/popup/sortlayermenu', [
      * @constructor
      *
      * @extends LayerMenu
+     *
+     * @param {SpreadsheetView} docView
+     *  The spreadsheet view containing this instance.
      */
-    function SortLayerMenu(app) {
+    function SortLayerMenu(docView) {
 
         var // self reference
             self = this,
 
-            // the document view
-            docView = null,
-
             directionGroup = null,
             keyRadioList = null,
+            orderRadioList = null,
             currentDirection = 'vertical',
             maxCount = 2000;
 
         // base constructor ---------------------------------------------------
 
-        LayerMenu.call(this, app, /*#. menu title: custom sort options */ gt.pgettext('sort', 'Custom Sort'), {
+        LayerMenu.call(this, docView, /*#. menu title: custom sort options */ gt.pgettext('sort', 'Custom Sort'), {
             actionKey: 'cell/sort/custom',
-            actionValue: 'custom'
+            actionValue: 'custom',
+            rootContainerNode: docView.getApp().getWindowNode()
         });
 
         // private methods ----------------------------------------------------
 
         function fillRadioList(columns) {
 
-            var currentSelection = docView.getSelection(),
-                interval = SheetUtils.getInterval(currentSelection.ranges[0], columns),
+            var interval = docView.getSelectedRanges().first().interval(columns),
                 from = interval.first,
                 to = interval.last,
                 controllerDefault = null;
@@ -66,7 +66,7 @@ define('io.ox/office/spreadsheet/view/popup/sortlayermenu', [
             // remove (old) column/row values. New selection will have new values
             keyRadioList.getMenu().clearContents();
 
-            if (SheetUtils.getIntervalSize(interval) > maxCount) { return false; }
+            if (interval.size() > maxCount) { return false; }
 
             for (var index = from; index <= to; index += 1) {
                 // set default selection to the first entry
@@ -74,7 +74,7 @@ define('io.ox/office/spreadsheet/view/popup/sortlayermenu', [
 
                 keyRadioList.createOptionButton(index, { label: columns ? Labels.getColLabel(index) : Labels.getRowLabel(index) });
             }
-            app.getController().executeItem('cell/sort/colrow', controllerDefault);
+            docView.executeControllerItem('cell/sort/colrow', controllerDefault);
         }
 
         /**
@@ -103,6 +103,8 @@ define('io.ox/office/spreadsheet/view/popup/sortlayermenu', [
          * Fills the radio lists before the menu will be shown.
          */
         function popupBeforeShowHandler() {
+            currentDirection = 'vertical';
+            docView.executeControllerItem('cell/sort/direction', 'vertical');
             fillRadioList(true);
         }
 
@@ -118,47 +120,48 @@ define('io.ox/office/spreadsheet/view/popup/sortlayermenu', [
          * first time.
          */
         function setDefaultPosition() {
-            var pos = Utils.getNodePositionInPage(app.getView().getAppPaneNode());
+            var pos = Utils.getNodePositionInPage(docView.getAppPaneNode());
             self.setNodePosition(pos.left + (pos.width - self.getNode().outerWidth()) / 2, pos.top + 40);
         }
 
         // initialization -----------------------------------------------------
 
-        // initialize after application is ready
-        app.onInit(function () {
-            docView = app.getView();
+        // add base menu items
+        this.addGroup('cell/sort/hasheadline', new CheckBox({ label: /*#. sorting: selected range has headers which shouldn't be sorted */ gt.pgettext('sort', 'Selection has headers'), boxed: true }))
+            .addSectionLabel(/*#. sorting: the sort direction headline for 'top to bottom' or 'left to right' */ gt.pgettext('sort', 'Direction'))
+            .addGroup('cell/sort/direction', directionGroup = new RadioGroup({ label: gt.pgettext('sort', 'Direction'), radioIcons: true })
+                .createOptionButton('vertical', { label: /*#. sort direction 'Top to bottom' */ gt.pgettext('sort', 'Top to bottom') })
+                .createOptionButton('horizontal', { label: /*#. sort direction 'Left to right' */ gt.pgettext('sort', 'Left to right') }),
+            { inline: true })
+            .addSectionLabel(/*#. head line: 'sort by' options (list of column names, e.g. 'Column C') */ gt.pgettext('sort', 'Sort by'))
+            .addGroup('cell/sort/colrow', keyRadioList = new RadioList({ style: 'width: 150px;', updateCaptionMode: 'label' }), { inline: true })
+            .addGroup('cell/sort/order', orderRadioList = new RadioList({ style: 'width:150px;text-align:right;' })
+                .createOptionButton('ascending', { label: /*#. sort ascending */ gt.pgettext('sort', 'Ascending') })
+                .createOptionButton('descending', { label: /*#. sort descending */ gt.pgettext('sort', 'Descending') }),
+            { inline: true });
 
-            // add base menu items
-            self.addGroup('cell/sort/hasheadline', new CheckBox({ label: /*#. sorting: selected range has headers which shouldn't be sorted */ gt.pgettext('sort', 'Selection has headers'), boxed: true }))
-                .addSectionLabel(/*#. sorting: the sort direction headline for 'top to bottom' or 'left to right' */ gt.pgettext('sort', 'Direction'))
-                .addGroup('cell/sort/direction', directionGroup = new RadioGroup({ label: gt.pgettext('sort', 'Direction'), radioIcons: true })
-                    .createOptionButton('vertical', { label: /*#. sort direction 'Top to bottom' */ gt.pgettext('sort', 'Top to bottom') })
-                    .createOptionButton('horizontal', { label: /*#. sort direction 'Left to right' */ gt.pgettext('sort', 'Left to right') }),
-                { inline: true })
-                .addSectionLabel( /*#. head line: 'sort by' options (list of column names, e.g. 'Column C') */ gt.pgettext('sort', 'Sort by'))
-                .addGroup('cell/sort/colrow', keyRadioList = new RadioList({ style: 'width: 150px;', updateCaptionMode: 'label' }), { inline: true })
-                .addGroup('cell/sort/order', new RadioList({ style: 'width:150px;text-align:right;' })
-                    .createOptionButton('ascending', { label: /*#. sort ascending */ gt.pgettext('sort', 'Ascending') })
-                    .createOptionButton('descending', { label: /*#. sort descending */ gt.pgettext('sort', 'Descending') }),
-                { inline: true });
-
-            // refill the column/row selector on changing the direction
-            self.listenTo(directionGroup, 'group:change', changeDirectionHandler);
-
-            // reset the custom option 'preventClose' on 'change:selection'
-            // event and do nothing, otherwise hide the pop-up menu
-            self.listenTo(docView, 'change:selection', changeSelectionHandler);
-
-            // add event handlers for the menu
-            self.on({ 'popup:beforeshow': popupBeforeShowHandler, 'popup:hide': popupHideHandler });
-
-            // set default position on first display of the menu
-            self.one('popup:show', setDefaultPosition);
+        // refill the column/row selector on changing the direction
+        this.listenTo(directionGroup, 'group:change', changeDirectionHandler);
+        this.listenTo(orderRadioList, 'group:change', function () {
+            orderRadioList.getMenu().hide();
         });
+        this.listenTo(keyRadioList, 'group:change', function () {
+            keyRadioList.getMenu().hide();
+        });
+
+        // reset the custom option 'preventClose' on 'change:selection'
+        // event and do nothing, otherwise hide the pop-up menu
+        this.listenTo(docView, 'change:selection', changeSelectionHandler);
+
+        // add event handlers for the menu
+        this.on({ 'popup:beforeshow': popupBeforeShowHandler, 'popup:hide': popupHideHandler });
+
+        // set default position on first display of the menu
+        this.one('popup:show', setDefaultPosition);
 
         // destroy all class members
         this.registerDestructor(function () {
-            app = self = docView = null;
+            self = docView = null;
         });
 
     } // class SortLayerMenu
