@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -16,8 +16,9 @@ define('io.ox/office/spreadsheet/view/popup/validationlistmenu', [
     'io.ox/office/tk/keycodes',
     'io.ox/office/tk/forms',
     'io.ox/office/tk/popup/listmenu',
-    'gettext!io.ox/office/spreadsheet'
-], function (Utils, KeyCodes, Forms, ListMenu, gt) {
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'gettext!io.ox/office/spreadsheet/main'
+], function (Utils, KeyCodes, Forms, ListMenu, SheetUtils, gt) {
 
     'use strict';
 
@@ -30,8 +31,11 @@ define('io.ox/office/spreadsheet/view/popup/validationlistmenu', [
      * @constructor
      *
      * @extends ListMenu
+     *
+     * @param {SpreadsheetView} docView
+     *  The spreadsheet view that contains this instance.
      */
-    function ValidationListMenu(app) {
+    function ValidationListMenu(docView) {
 
         var // self reference
             self = this;
@@ -54,9 +58,9 @@ define('io.ox/office/spreadsheet/view/popup/validationlistmenu', [
             // always hide the pop-up menu
             self.hide();
 
-            // insert the value into the cell
+            // insert the value into the cell (via controller for busy screen etc.)
             if (_.isString(itemValue)) {
-                app.getView().setCellContents(itemValue);
+                docView.executeControllerItem('cell/active/value', itemValue);
             }
         }
 
@@ -67,7 +71,7 @@ define('io.ox/office/spreadsheet/view/popup/validationlistmenu', [
          * Called by grid pane instances from the click handler of cell
          * drop-down buttons.
          *
-         * @param {Number[]} address
+         * @param {Address} address
          *  The address of the cell containing the validation data.
          *
          * @returns {jQuery.Promise}
@@ -77,7 +81,7 @@ define('io.ox/office/spreadsheet/view/popup/validationlistmenu', [
         this.initialize = function (address) {
 
             var // the validation collection of the active sheet
-                validationCollection = app.getView().getValidationCollection(),
+                validationCollection = docView.getValidationCollection(),
                 // the resulting promise
                 promise = validationCollection.queryListValues(address);
 
@@ -89,13 +93,14 @@ define('io.ox/office/spreadsheet/view/popup/validationlistmenu', [
 
                 // insert the data into the list
                 self.clearContents();
-                _.each(values, function (value) {
-                    self.createItemNode(value, { label: _.noI18n(value) });
+                values.forEach(function (value) {
+                    var display = _.isString(value) ? value : '#######'; // TODO: Excel uses real cell results internally
+                    self.createItemNode(value || '', { label: _.noI18n(display) });
                 });
             });
 
             // reject with a specific error code causing a warning alert box
-            return promise.then(null, _.constant('validation:source'));
+            return promise.then(null, _.constant(SheetUtils.makeRejected('validation:source')));
         };
 
         /**
@@ -149,7 +154,7 @@ define('io.ox/office/spreadsheet/view/popup/validationlistmenu', [
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = self = null;
+            self = docView = null;
         });
 
     } // class ValidationListMenu
