@@ -6,39 +6,41 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
-define('io.ox/office/text/androidselection',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/text/utils/operations',
-     'io.ox/office/text/position',
-     'io.ox/office/text/dom'
-    ], function (Utils, KeyCodes, Forms, Operations, Position, DOM) {
+define('io.ox/office/text/androidselection', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/utils/fontdescriptor',
+    'io.ox/office/tk/utils/fontmetrics',
+    'io.ox/office/text/utils/operations',
+    'io.ox/office/text/position',
+    'io.ox/office/text/dom'
+], function (Utils, KeyCodes, Forms, FontDescriptor, FontMetrics, Operations, Position, DOM) {
 
     'use strict';
 
-    // mix-in class AndroidSelection =============================================
-
-    var DUMMYLEN = 20;
+    var DUMMYLEN = 50;
     var CURSOR = 'androidcursor';
     var IMEMODE = 'androidime';
     var BLINK = 'androidblink';
     var SELECTOR = '.' + CURSOR + ', .' + IMEMODE;
     var CLASSES = CURSOR + ' ' + IMEMODE;
 
-    var INITTEXT = Utils.repeatString(' ', DUMMYLEN);
+    //first letter is workaround for Camel-Case-Behavior on some devices (Bug 40066)
+    var INITTEXT = 'G' + Utils.repeatString(' ', DUMMYLEN - 1);
 
     var ENDCHARS = Position.getWordBoundEndChars();
 
-    var FONTATTRS = {
-        fontName: 'monospace',
-        fontSize: 20//points!
-    };
+    var FONT_DESCRIPTOR = new FontDescriptor('monospace', 20); // size in points
+    var LETTER_WIDTH = FontMetrics.getTextWidth(FONT_DESCRIPTOR, 'A');
+
+    // mix-in class AndroidSelection ==========================================
+
     /**
      *
      * @constructor
@@ -62,10 +64,15 @@ define('io.ox/office/text/androidselection',
 
         var restoreFN = null;
 
-        var letterWidth = 1;
-
         // private methods ----------------------------------------------------
 
+        function isSoftKeyboardOpen() {
+            if (Utils.isSoftKeyboardOpen()) {
+                //TODO: better check focus in textarea!!!
+                return (Utils.getInnerWindowHeight() / Utils.getScreenHeight()) < 0.7;
+            }
+        }
+
         function splitTextSpan(para, pos) {
             singleArray[0] = _.last(pos);
 
@@ -101,17 +108,17 @@ define('io.ox/office/text/androidselection',
 
         function getWordBoundary(para, pos, beginning, dontAddSpace) {
             var len = pos.length;
-            singleArray[0] = pos[len-1];
+            singleArray[0] = pos[len - 1];
 
             var result = _.clone(pos);
-            result[len-1] = Position.getWordBoundary(para[0], singleArray, beginning, dontAddSpace)[0];
+            result[len - 1] = Position.getWordBoundary(para[0], singleArray, beginning, dontAddSpace)[0];
             return result;
         }
 
         function changeSelection(event, selectionInfo) {
             var start = selectionInfo.start;
             var end = selectionInfo.end;
-            //console.error('changeSelection', start, end);
+
             var spans = null;
             var node = null;
 
@@ -119,9 +126,11 @@ define('io.ox/office/text/androidselection',
                 oldPara.removeClass(BLINK);
                 spans = oldPara.find(SELECTOR);
                 spans.removeClass(CLASSES);
-//                spans.each(function() {
-//                    CharacterStyles.mergeSiblingTextSpans(this, true);
-//                });
+                /*
+                spans.each(function () {
+                    TextUtils.mergeSiblingTextSpans(this, true);
+                });
+                */
                 oldPara = null;
             }
 
@@ -131,21 +140,21 @@ define('io.ox/office/text/androidselection',
 
                 var selType = window.getSelection().type;
                 if (!selType || selType === 'None') {
-                    selType = Utils.isSoftKeyboardOpen();
+                    selType = isSoftKeyboardOpen();
                 }
 
                 if (_.isEqual(start, end) && selType) {
 
-                    oldPara = $(Position.getDOMPosition(rootNode, _.initial(start)).node);
+                    oldPara = $(Position.getDOMPosition(app.getModel().getCurrentRootNode(), _.initial(start)).node);
                     oldPara.addClass(BLINK);
 
                     wordFrom = getWordBoundary(oldPara, start, true);
                     wordTo = getWordBoundary(oldPara, start, false, true);
                     if (_.isEqual(wordFrom, wordTo) ||  _.last(wordFrom) >= _.last(wordTo)) {
-                        start[start.length-1]--;
+                        start[start.length - 1] -= 1;
                         wordFrom = getWordBoundary(oldPara, start, true);
                         wordTo = getWordBoundary(oldPara, start, false, true);
-                        start[start.length-1]++;
+                        start[start.length - 1] += 1;
                     }
 
                     if (_.last(wordFrom) < _.last(wordTo)) {
@@ -160,7 +169,7 @@ define('io.ox/office/text/androidselection',
                         var len = 0;
                         var maxLen = _.last(wordTo) - _.last(wordFrom);
 
-                        while(len < maxLen && spanFrom) {
+                        while (len < maxLen && spanFrom) {
                             word += spanFrom.innerHTML;
                             len = word.length;
                             spanFrom.classList.add(IMEMODE);
@@ -172,10 +181,6 @@ define('io.ox/office/text/androidselection',
                         word = '';
                     }
 
-
-                    //console.warn('domPos.offset', _.last(start));
-
-
                     node = splitTextSpan(oldPara, start);
                     node = $(node);
                     node.addClass(CURSOR);
@@ -195,34 +200,37 @@ define('io.ox/office/text/androidselection',
             if (parent && parent.length) {
 
                 var zoom = app.getView().getZoomFactor();
-                //console.warn('handleTextAreaPos', oldStart, oldPara.children().length, oldPara);
-                var pos = Position.getPixelPositionFromDomPoint (rootNode, [_.last(oldStart)], zoom, oldPara);
-                var multi = zoom/100;
-
-//                console.warn('getPixelPositionFromDomPoint', pos, oldPara, _.last(oldStart), zoom);
-                div.css({
-                    left: (pos.left * multi - 5) + 'px',
-                    top: (pos.top * multi + pos.height * multi - 30) + 'px'
-                });
+                try {
+                    var pos = Position.getPixelPositionFromDomPoint(rootNode,
+                            [_.last(oldStart)], zoom, oldPara);
+                    var multi = zoom / 100;
+                    div.css({
+                        left: pos.left * multi - 5,
+                        top: pos.top * multi + pos.height * multi - 30
+                    });
+                } catch (e) {
+                    Utils.warn('problem with handleTextAreaPos() using Position.getPixelPositionFromDomPoint()', e.message, oldStart);
+                    resetTextarea();
+                }
             }
         }
 
         function isPassThrough(event, press) {
             if (event.ctrlKey || event.altKey || event.metaKey) { return true; }
             if (press) {
-                return event.keyCode === KeyCodes.ENTER || event.keyCode === KeyCodes.SPACE;
-            } else if(event.keyCode === KeyCodes.TAB){
+                return event.keyCode === KeyCodes.ENTER;
+            } else if (event.keyCode === KeyCodes.TAB) {
                 event.preventDefault();
                 return true;
-            } else if(event.shiftKey && DOM.isCursorKey(event.keyCode)) {
+            } else if (event.shiftKey && DOM.isCursorKey(event.keyCode)) {
                 restoreFN();
                 event.preventDefault();
                 return true;
-            } else if(event.keyCode === KeyCodes.BACKSPACE) {
+            } else if (event.keyCode === KeyCodes.BACKSPACE) {
                 event.preventDefault();
                 return true;
             } else {
-                return (DOM.isCursorKey(event.keyCode) || event.keyCode === KeyCodes.ENTER || event.keyCode === KeyCodes.DELETE || event.keyCode === KeyCodes.SPACE);
+                return (DOM.isCursorKey(event.keyCode) || event.keyCode === KeyCodes.ENTER || event.keyCode === KeyCodes.DELETE);
             }
         }
 
@@ -251,15 +259,13 @@ define('io.ox/office/text/androidselection',
                     var top = pos.top - w.scrollTop();
                     var left = pos.left + cursorSpan.width() - w.scrollLeft();
 
-                    var newOxo = Position.getOxoPositionFromPixelPosition(rootNode, left, top + height/2 + offset);
+                    var newOxo = Position.getOxoPositionFromPixelPosition(rootNode, left, top + height / 2 + offset);
 
                     if (!newOxo) {
                         if (oldStart.length > 2) {
                             return;
                         }
-                        newOxo = {
-                                start: []
-                        };
+                        newOxo = { start: [] };
                         if (event.keyCode === KeyCodes.UP_ARROW) {
                             newOxo.start[0] = Math.max(0, oldStart[0] - 1);
                         } else {
@@ -294,8 +300,6 @@ define('io.ox/office/text/androidselection',
 
                 if (text.length > DUMMYLEN) {
 
-
-
                     text = text.substring(DUMMYLEN, text.length);
 
                     if (text === word) {
@@ -311,7 +315,7 @@ define('io.ox/office/text/androidselection',
                         } else {
                             attrs = _.copy(attrs, true);
                         }
-                        attrs.changes = { 'inserted': changeTrack.getChangeTrackInfo() };
+                        attrs.changes = { inserted: changeTrack.getChangeTrackInfo() };
                     }
 
                     var selection = getCursorIndex() - DUMMYLEN;
@@ -327,14 +331,12 @@ define('io.ox/office/text/androidselection',
                     if (text.length - 1 === word.length && text.substring(0, text.length - 1) === word && !preAttrs) {
                         //optimized for forward typing
                         redoOperations.push({ name: Operations.TEXT_INSERT, start: _.clone(wordTo), text: text.substring(text.length - 1, text.length), attrs: attrs });
-                        undoOperations.push({ name: Operations.DELETE, start: _.clone(wordTo), end: Position.increaseLastIndex(wordTo, -1) });
-
+                        undoOperations.push({ name: Operations.DELETE, start: _.clone(wordTo), end: _.clone(wordTo) });
                     } else {
 
                         var insertOp = { name: Operations.TEXT_INSERT, start: _.clone(wordTo), text: text, attrs: attrs };
                         redoOperations.push(insertOp);
 
-
                         var deleteOp = null;
 
                         if (_.last(wordFrom) !== _.last(wordTo)) {
@@ -363,9 +365,6 @@ define('io.ox/office/text/androidselection',
                         spans = oldPara.children('span.' + IMEMODE + ', span.' + CURSOR);
                     }
 
-
-                    //console.warn('spancount befor', oldPara.children('span.' + IMEMODE).length, oldPara.children('span.' + CURSOR).length);
-
                     var cursorSpan = oldPara.find('.' + CURSOR);
                     cursorSpan.removeClass(CURSOR);
 
@@ -382,6 +381,10 @@ define('io.ox/office/text/androidselection',
                             cursorSpan.addClass(CURSOR);
                             handlePreselectedAttrs(attrs, cursorSpan);
                         } else if (!cursorSpan.is('span')) {
+                            if (!spans.length) {
+                                spans = oldPara.children('span:first-of-type');
+                                if (!spans.length) { spans = $('span'); }
+                            }
                             var newSpan = $(spans[0]).clone(true);
                             newSpan.insertAfter(cursorSpan);
                             newSpan.addClass(IMEMODE);
@@ -403,27 +406,32 @@ define('io.ox/office/text/androidselection',
 
                     } else if (selection === text.length) {
                         //cursor on end
-                        _.times(spanCount - 1, function(i) {
+                        _.times(spanCount - 1, function (i) {
                             var el = spans[i];
                             el.parentNode.removeChild(el);
                         });
-                        var span = spans[spanCount-1];
+                        var span = spans[spanCount - 1];
                         span.classList.add(CURSOR);
-                        fillTextInSpan(span, text);
 
                         if (leftNoIME) {
                             span.classList.remove(IMEMODE);
+                            if (text[selection - 1] === ' ') {
+                                text = text.substring(0, selection - 1) + '\xa0';
+                            }
                         }
+
+                        fillTextInSpan(span, text);
+
                         handlePreselectedAttrs(attrs, span);
                     } else {
-                        _.times(spanCount - 2, function(i) {
+                        _.times(spanCount - 2, function (i) {
                             var el = spans[i];
                             el.parentNode.removeChild(el);
                         });
-                        var right = spans[spanCount-1];
+                        var right = spans[spanCount - 1];
                         right.innerText = text.substring(selection, text.length);
 
-                        var left = spans[spanCount-2];
+                        var left = spans[spanCount - 2];
                         left.classList.add(CURSOR);
                         left.innerText = text.substring(0, selection);
 
@@ -465,10 +473,10 @@ define('io.ox/office/text/androidselection',
         function sendOperations(docModel, changeTrack, undoOperations, redoOperations) {
             if (changeTrack.isActiveChangeTracking()) {
                 var info = changeTrack.getChangeTrackInfo();
-                _.each(undoOperations, function(op) {
+                _.each(undoOperations, function (op) {
                     addChangeTrackInfo(op, info);
                 });
-                _.each(redoOperations, function(op) {
+                _.each(redoOperations, function (op) {
                     addChangeTrackInfo(op, info);
                 });
             }
@@ -487,7 +495,6 @@ define('io.ox/office/text/androidselection',
                 attrs = {};
             }
 
-
             var name;
             if (op.name === Operations.TEXT_INSERT) {
                 name = 'inserted';
@@ -501,15 +508,9 @@ define('io.ox/office/text/androidselection',
 
         function handlePreselectedAttrs(attrs) {
             if (attrs) {
-                attrs = {
-                        attributes: attrs
-                };
+                attrs = { attributes: attrs };
                 for (var i = 1; i < arguments.length; i++) {
-                    var span = arguments[i];
-                    if (!(span instanceof $)) {
-                        span = $(span);
-                    }
-                    span.data(attrs);
+                    $(arguments[i]).data(attrs);
                 }
             }
         }
@@ -520,16 +521,6 @@ define('io.ox/office/text/androidselection',
             textarea.focus();
             textarea.val(INITTEXT + word);
             Forms.setInputSelection(textarea, DUMMYLEN + (_.last(oldStart) - _.last(wordFrom)));
-
-            //workaround for broken initial focus
-            if (document.activeElement !== textarea[0]) {
-                _.defer(function() {
-                    textarea.focus();
-                    if (document.activeElement !== textarea[0]) {
-                        restoreFN();
-                    }
-                });
-            }
         }
 
         function getCursorIndex() {
@@ -545,11 +536,13 @@ define('io.ox/office/text/androidselection',
         }
 
         function setReadOnly() {
+            //console.error('setReadOnly');
             rootNode.css('-webkit-user-modify', 'read-only');
         }
 
         function setReadWrite() {
-            rootNode.css('-webkit-user-modify', 'read-write');
+            //console.error('setReadWrite');
+            rootNode.css('-webkit-user-modify', 'read-write-plaintext-only');
         }
 
         var processSelectUpdate = (function () {
@@ -558,9 +551,7 @@ define('io.ox/office/text/androidselection',
                 if (document.activeElement === rootNode[0]) {
                     //listener hangs directly on document, but we only want the events of editdiv
                     if (selecttype !== 'None') {
-//                        console.warn('selecttype', selecttype);
                         self.updateSelectionAfterBrowserSelection({ preserveFocus: false, forceTrigger: false });
-//                        console.warn('all', self.getStartPosition(), self.getEndPosition());
                         if (selecttype === 'Caret') {
                             setReadOnly();
                         }
@@ -585,9 +576,10 @@ define('io.ox/office/text/androidselection',
             };
             var notAlways =  _.throttle(always, 250);
 
-            return function(evt) {
+            return function (evt) {
 
                 var selecttype = window.getSelection().type;
+                //console.warn('selecttype', selecttype);
                 if (evt.type === 'selectstart') {
                     if (selecttype === 'Range') {
                         ///User seems to call "select all", but android chrome does not really support that featurem when contenteditable=true and begins with scrolling :(
@@ -600,10 +592,10 @@ define('io.ox/office/text/androidselection',
                 }
 
                 if (selecttype === 'Range') {
-                    if (Utils.isSoftKeyboardOpen()) {
+                    if (isSoftKeyboardOpen()) {
                         setReadOnly();
                         _.defer(setReadWrite);
-                    } else if(rootNode.css('-webkit-user-modify') === 'read-only') {
+                    } else if (rootNode.css('-webkit-user-modify') === 'read-only') {
                         setReadWrite();
                     }
                 }
@@ -617,7 +609,6 @@ define('io.ox/office/text/androidselection',
 
         self.on('changecursorinfo', changeSelection);
 
-
         blinkDeferred = app.repeatDelayed(function () {
             if (oldPara) {
                 if (document.activeElement && document.activeElement.tagName === 'INPUT') {
@@ -626,18 +617,15 @@ define('io.ox/office/text/androidselection',
                     oldPara.toggleClass(BLINK);
                 }
             }
-        }, { delay: 500 });
+        }, { delay: 500 }, 'Text: Selection: blinkDeferred');
 
         app.onImportSuccess(function () {
-            var view = app.getView();
+            var docView = app.getView();
 
             restoreFN = _.bind(self.restoreBrowserSelection, self);
 
-            self.restoreBrowserSelection = function() {
-                if ($(document.activeElement).is(Forms.BUTTON_SELECTOR)) {
-                    //emulate default behavior because menus are not closed
-                    Utils.trigger('change:focus', rootNode, document.activeElement, null);
-                }
+            self.restoreBrowserSelection = function () {
+                //sideEffect of passThrough, with original restoreFN textarea-focus gets lost
                 if (!_.isEqual(self.getStartPosition(), self.getEndPosition())) {
                     //console.warn('restoreFN', self.getStartPosition(), self.getEndPosition());
                     restoreFN();
@@ -645,7 +633,7 @@ define('io.ox/office/text/androidselection',
             };
 
             var docModel = app.getModel();
-            textarea.on('cut copy paste', function(evt) {
+            textarea.on('cut copy paste', function (evt) {
                 //console.warn('evt', evt.type);
                 docModel[evt.type](evt);
             });
@@ -659,15 +647,13 @@ define('io.ox/office/text/androidselection',
                 overflow: 'hidden'
             });
 
-            letterWidth = docModel.getFontCollection().getTextWidth('A', FONTATTRS);
-
             textarea.css({
                 position: 'absolute',
                 width: '10px',
-                letterSpacing: -letterWidth + 'px',
-                fontFamily: 'monospace',
-                fontSize: FONTATTRS.fontSize + 'pt',
-                lineHeight: FONTATTRS.fontSize + 'pt',
+                letterSpacing: -LETTER_WIDTH + 'px',
+                fontFamily: FONT_DESCRIPTOR.family,
+                fontSize: FONT_DESCRIPTOR.size + 'pt',
+                lineHeight: FONT_DESCRIPTOR.size + 'pt',
                 height: '30px',
                 color: 'transparent',
                 overflow: 'hidden',
@@ -687,93 +673,20 @@ define('io.ox/office/text/androidselection',
 
             docModel.listenTo($(document), 'selectstart selectionchange select', processSelectUpdate);
 
-            app.registerWindowResizeHandler(function () {
-                if (Utils.isSoftKeyboardOpen() && window.getSelection().type !== 'Range') {
-                    var boundRect = docModel.getSelection().getFocusPositionBoundRect();
-                    if (boundRect) {
-                        app.getView().scrollToPageRectangle(boundRect);
+            if (false) {
+                //TODO
+                app.registerWindowResizeHandler(function () {
+                    if (isSoftKeyboardOpen() && window.getSelection().type !== 'Range') {
+                        var boundRect = docModel.getSelection().getFocusPositionBoundRect();
+                        if (boundRect) {
+                            docView.scrollToPageRectangle(boundRect);
+                        }
                     }
-                }
-            } );
-
+                });
+            }
 
             app.registerGlobalEventHandler(window, 'orientationchange', resetTextarea);
-            docModel.listenTo(view, 'change:zoom', resetTextarea);
-
-
-
-            var grabFocus = view.grabFocus;
-            view.grabFocus = function() {
-                if (view.isVisible()) {
-                    if (Utils.isSoftKeyboardOpen()) {
-                        var windowNode = app.getWindow().nodes.outer;
-                        var btn = $(windowNode).find('a.button[aria-expanded=true]');
-                        if (!btn.length) {
-                            //fallback for combo-text-fields
-                            btn = $(windowNode).find('.group.dropdown-open a').first();
-                        }
-                        btn[Forms.DEFAULT_CLICK_TYPE]();
-
-                        if(document.activeElement !== textarea[0]){
-                            $(document.activeElement).blur();
-                        }
-                        selectTextArea();
-
-
-                    } else {
-                        grabFocus.apply(view);
-                    }
-                }
-            };
-
-            textarea.blur(function() {
-                if (Utils.isSoftKeyboardOpen() && (!document.activeElement || document.activeElement.tagName !== 'INPUT')) {
-                    _.defer(function() {
-                        if (document.activeElement.classList.contains('popup-content') || document.activeElement.classList.contains('button')) {
-                            textarea.addClass('focus-helper');
-                            selectTextArea();
-                            _.defer(function() {
-                                textarea.removeClass('focus-helper');
-                            });
-                        }
-                    });
-                }
-            });
-
-
-            /*
-            var lastTouch = null;
-            var lastTouchCounter = 0;
-
-
-            //double tab
-            rootNode.on('click', function(evt) {
-                if (!lastTouch) {
-                    lastTouch = evt.timeStamp;
-                    lastTouchCounter = 0;
-                }
-
-                var relTime = evt.timeStamp - lastTouch;
-                if (relTime < 1500) {
-                    lastTouchCounter++;
-                    if (lastTouchCounter === 2) {
-
-                        if (_.last(wordFrom) < _.last(wordTo)) {
-                            restoreFN();
-                            self.setTextSelection(wordFrom, wordTo);
-
-                        }
-
-                        lastTouch = null;
-                    }
-                } else if (relTime > 3000) {
-                    lastTouch = evt.timeStamp;
-                    lastTouchCounter = 1;
-                } else {
-                    lastTouch = null;
-                }
-            });
-            */
+            docModel.listenTo(docView, 'change:zoom', resetTextarea);
 
         });
 
@@ -785,8 +698,6 @@ define('io.ox/office/text/androidselection',
 
     } // class AndroidSelection
 
-
-
     // exports ================================================================
 
     return AndroidSelection;
