@@ -6,22 +6,22 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  * @author Malte Timmermann <malte.timmermann@open-xchange.com>
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
-define('io.ox/office/text/app/application',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/errorcode',
-     'io.ox/office/editframework/app/editapplication',
-     'io.ox/office/text/editor',
-     'io.ox/office/text/view/view',
-     'io.ox/office/text/app/controller',
-     'gettext!io.ox/office/text'
-    ], function (Utils, ErrorCode, EditApplication, TextModel, TextView, TextController, gt) {
+define('io.ox/office/text/app/application', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/baseframework/utils/errorcode',
+    'io.ox/office/editframework/app/editapplication',
+    'io.ox/office/text/editor',
+    'io.ox/office/text/view/view',
+    'io.ox/office/text/app/controller',
+    'gettext!io.ox/office/text/main'
+], function (Utils, ErrorCode, EditApplication, TextModel, TextView, TextController, gt) {
 
     'use strict';
 
@@ -36,26 +36,25 @@ define('io.ox/office/text/app/application',
      *
      * @param {Object} launchOptions
      *  All options passed to the core launcher (the ox.launch() method).
-     *
-     * @param {Object} [appOptions]
-     *  Static application options that have been passed to the static method
-     *  BaseApplication.createLauncher().
      */
-    var TextApplication = EditApplication.extend({ constructor: function (launchOptions, appOptions) {
+    var TextApplication = EditApplication.extend({ constructor: function (launchOptions) {
 
         var // self reference
             self = this;
 
         // base constructor ---------------------------------------------------
 
-        EditApplication.call(this, TextModel, TextView, TextController, launchOptions, appOptions, {
+        EditApplication.call(this, TextModel, TextView, TextController, launchOptions, {
             applyActionsDetached: true,
             postProcessHandler: postProcessDocument,
             postProcessHandlerStorage: postProcessHandlerStorage,
             fastEmptyLoadHandler: fastEmptyLoadHandler,
             optimizeOperationsHandler: optimizeOperations,
+            mergeCachedOperationsHandler: mergeCachedOperations,
+            prepareLoseEditRightsHandler: prepareLoseEditRights,
             operationFilter: operationFilter,
             importFailedHandler: importFailedHandler,
+            prepareFlushHandler: prepareFlushDocument,
             sendActionsDelay: 1000,
             localStorageApp: true,
             requiredStorageVersion: 2,
@@ -107,6 +106,19 @@ define('io.ox/office/text/app/application',
             }
         }
 
+        /**
+         * Preprocessing before the document will be flushed for downloading,
+         * printing, sending as mail, or closing.
+         *
+         * @param {String} reason
+         *  Can be 'download', 'email', or 'quit'
+         */
+        function prepareFlushDocument (reason) {
+            if (reason === 'download' || reason === 'email') {
+                self.getModel().getFieldManager().prepareDateTimeFieldsDownload();
+            }
+        }
+
         /**
          * Optimizing the actions (operations) before sending them to the server.
          *
@@ -120,6 +132,30 @@ define('io.ox/office/text/app/application',
             return self.getModel().optimizeActions(actionsBuffer);
         }
 
+        /**
+         * Merge cached operations buffer, with current action buffer, before registering.
+         *
+         * @param {Array} actionBuffer
+         *  Current array of operations.
+         *
+         * @param {Array} cacheBuffer
+         *  Cached array of operations, e.g operations for complex fields update
+         *  on document open are cached until document is modified.
+         *
+         * @returns {Array}
+         *
+         */
+        function mergeCachedOperations(actionBuffer, cacheBuffer) {
+            return cacheBuffer.concat(actionBuffer);
+        }
+
+        /**
+         * Trigger flushing of buffered operations on losing edit rights.
+         */
+        function prepareLoseEditRights() {
+            self.getModel().trigger('cacheBuffer:flush');
+        }
+
         /**
          * Load performance: Only selected operations need to be executed, if document
          * can be loaded from local storage.
@@ -156,7 +192,7 @@ define('io.ox/office/text/app/application',
 
         // destroy all class members
         this.registerDestructor(function () {
-            launchOptions = appOptions = null;
+            launchOptions = self = null;
         });
 
     }}); // class TextApplication
@@ -168,7 +204,7 @@ define('io.ox/office/text/app/application',
      * without parameters, to launch text applications.
      */
     TextApplication.createLauncher = function () {
-        return EditApplication.createLauncher('io.ox/office/text', TextApplication, { icon: 'fa-align-left' });
+        return EditApplication.createLauncher('io.ox/office/text', TextApplication, { icon: 'fa-align-left', trackingId: 'io.ox/text-editor' });
     };
 
     // exports ================================================================
