@@ -5,15 +5,15 @@
  * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Carsten Driesner <carsten.driesner@open-xchange.com>
  */
 
-define('io.ox/office/text/app/fileactions',
-    ['io.ox/office/editframework/app/fileactions',
-     'gettext!io.ox/office/files'
-    ], function (FileActions, gt) {
+define('io.ox/office/text/app/fileactions', [
+    'io.ox/office/editframework/app/fileactions',
+    'gettext!io.ox/office/text/actions'
+], function (FileActions, gt) {
 
     'use strict';
 
