@@ -5,12 +5,14 @@
  * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Oliver Specht <oliver.specht@open-xchange.com>
  */
 
-define('io.ox/office/text/app/mailactions', ['io.ox/office/editframework/app/mailactions'], function (MailActions) {
+define('io.ox/office/text/app/mailactions', [
+    'io.ox/office/editframework/app/mailactions'
+], function (MailActions) {
 
     'use strict';
 
