@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Mario Schroeder <mario.schroeder@open-xchange.com>
  */
@@ -76,9 +76,28 @@ define('io.ox/office/text/clipboardmixin', [
             return text.replace(/["\\]/g, '');
         }
 
-
         // methods ------------------------------------------------------------
 
+        // Text parser --------------------------------------------------------
+
+        /**
+         * Parses the pasted clipboard text.
+         *
+         * @param {String} text
+         *
+         * @returns {Array}
+         *  The clipboard data array to create operations from.
+         */
+        this.parseClipboardText = function (text) {
+            var paragraphs = text.split('\r\n');
+            var ops = [];
+            _.each(paragraphs, function (para) {
+                ops.push({ operation: Operations.PARA_INSERT });
+                ops.push({ operation: Operations.TEXT_INSERT, data: para });
+            });
+            return ops;
+        };
+
         // HTML parser --------------------------------------------------------
 
         /**
@@ -99,7 +118,6 @@ define('io.ox/office/text/clipboardmixin', [
                 // the definition off all MS list styles
                 msListDefinitions = null;
 
-
             (function findTextNodes(current, depth, listLevel, type) {
 
                 var // the length (of chars, tabs, drawings...) to be added to the oxo position
@@ -131,15 +149,15 @@ define('io.ox/office/text/clipboardmixin', [
                             for (var j = 0; j < splitted.length; j++) {
                                 if (splitted[j] === '\t') {
                                     // tab
-                                    result.push({operation: Operations.TAB_INSERT, depth: depth});
-                                    insertLength ++;
+                                    result.push({ operation: Operations.TAB_INSERT, depth: depth });
+                                    insertLength += 1;
                                 } else {
                                     // text
                                     // replace '\r' and '\n' with space to fix pastes from aoo,
                                     // replace invisible control characters and reduce space sequences to a single spaces
                                     text = splitted[j].replace(/[\r\n]/g, ' ').replace(/[\x00-\x1f]/g, '').replace(/ +/g, ' ');
                                     insertLength += text.length;
-                                    result.push({operation: Operations.TEXT_INSERT, data: text, depth: depth});
+                                    result.push({ operation: Operations.TEXT_INSERT, data: text, depth: depth });
                                 }
                             }
                         }
@@ -163,10 +181,16 @@ define('io.ox/office/text/clipboardmixin', [
                         // insert paragraph for <div>, heading tags and for
                         //      <br> tags not nested inside a <div>, <p> or <li>
                         //      <p> tags not nested inside a <li>
-                        if ($(child).is('div, h1, h2, h3, h4, h5, h6') ||
-                            $(child).is('br') && (!$(child.parentNode).is('p, div, li') || $(child.parentNode).hasClass('clipboard')) ||
-                            $(child).is('p') && !$(child.parentNode).is('li')) {
 
+                        if (($(child).is('div') || $(child).is('span') || $(child).is('br')) && $(child.parentNode).is('a')) {
+                            // these children in links lead to empty following paragraphs, that receive setAttributes operations
+                            nextLevel = false;
+
+                        } else if ($(child).is('div, h1, h2, h3, h4, h5, h6') ||
+                            ($(child).is('br') && (!$(child.parentNode).is('p, div, li')) ||
+                            ($(child.parentNode).hasClass('clipboard') && !$(child).is('style'))) ||
+                            ($(child).is('p') && !$(child.parentNode).is('li'))
+                        ) {
                             if (self.isMSListParagraph(child)) {
                                 // read MS list definition styles if not yet done
                                 if (!msListDefinitions) {
@@ -179,21 +203,21 @@ define('io.ox/office/text/clipboardmixin', [
                                 if (msListStyle) {
                                     if (!msListStyle.inserted) {
                                         // insert MS list style definition
-                                        result.push({operation: 'insertMSList', depth: depth, listLevel: msParagraphLevelInfo.level, data: msListStyle});
+                                        result.push({ operation: 'insertMSList', depth: depth, listLevel: msParagraphLevelInfo.level, data: msListStyle });
                                         msListStyle.inserted = true;
                                     }
 
                                     // insert paragraph and list element for current list level
-                                    result.push({operation: Operations.PARA_INSERT, depth: depth, listLevel: msParagraphLevelInfo.level});
-                                    result.push({operation: 'insertListElement', depth: depth, listLevel: msParagraphLevelInfo.level});
+                                    result.push({ operation: Operations.PARA_INSERT, depth: depth, listLevel: msParagraphLevelInfo.level });
+                                    result.push({ operation: 'insertListElement', depth: depth, listLevel: msParagraphLevelInfo.level });
 
                                 } else {
                                     // special handling for pasting list paragraphs without bullet or numbering
-                                    result.push({operation: 'insertListParagraph', depth: depth, listLevel: 0});
+                                    result.push({ operation: 'insertListParagraph', depth: depth, listLevel: 0 });
                                 }
 
                             } else {
-                                result.push({operation: Operations.PARA_INSERT, depth: depth, listLevel: listLevel});
+                                result.push({ operation: Operations.PARA_INSERT, depth: depth, listLevel: listLevel });
                             }
 
                         } else if ($(child).is('p') && $(child.parentNode).is('li') && ($(child.parentNode).children('p').first().get(0) !== child)) {
@@ -204,7 +228,7 @@ define('io.ox/office/text/clipboardmixin', [
                             //     <p><span>bar</span></p>              bar
                             //   </li>
                             // </ol>
-                            result.push({operation: 'insertListParagraph', depth: depth, listLevel: listLevel});
+                            result.push({ operation: 'insertListParagraph', depth: depth, listLevel: listLevel });
 
                         } else if ($(child).is('span')) {
                             // don't parse <span> elements with 'mso-list:Ignore' style
@@ -223,22 +247,24 @@ define('io.ox/office/text/clipboardmixin', [
                             if (ox.session === childData.sessionId && fileId === childData.fileId && childData.altsrc) {
                                 // The image URL points to a document and the copy&paste action
                                 // takes place inside the same editor instance, so we use relative document media URL.
-                                result.push({operation: Operations.DRAWING_INSERT, data: childData.altsrc, depth: depth});
+                                result.push({ operation: Operations.DRAWING_INSERT, data: childData.altsrc, depth: depth });
 
                             } else if (DOM.isDocumentImageNode(child)) {
                                 // The image URL points to a document, but copy&paste is done from one editor instance to another
                                 // and the image src has not been replaced with a base64 data URL (e.g. IE 9 does not support this).
                                 // So the base64 data URL can be generated from the image node as long as the session is valid.
-                                result.push({operation: Operations.DRAWING_INSERT,
-                                             data: DOM.getBase64FromImageNode(child, Image.getMimeTypeFromImageUri(DOM.getUrlParamFromImageNode(child, 'get_filename'))),
-                                             depth: depth});
+                                result.push({
+                                    operation: Operations.DRAWING_INSERT,
+                                    data: DOM.getBase64FromImageNode(child, Image.getMimeTypeFromImageUri(DOM.getUrlParamFromImageNode(child, 'get_filename'))),
+                                    depth: depth
+                                });
 
                             } else {
                                 // The image src is an external web URL or already a base64 data URL
-                                result.push({operation: Operations.DRAWING_INSERT, data: child.src, depth: depth});
+                                result.push({ operation: Operations.DRAWING_INSERT, data: child.src, depth: depth });
 
                             }
-                            insertLength ++;
+                            insertLength += 1;
                         } else if ($(child).is('style, title')) {
                             // don't parse <style> and <title> elements
                             nextLevel = false;
@@ -248,7 +274,7 @@ define('io.ox/office/text/clipboardmixin', [
                             // so we first parse the next recursion level
                             var len = findTextNodes(child, depth + 1, listLevel, type);
                             // and then add the hyperlink
-                            result.push({operation: 'insertHyperlink', data: child.href, length: len, depth: depth});
+                            result.push({ operation: 'insertHyperlink', data: child.href, length: len, depth: depth });
                             // we already traversed the tree, so don't do it again
                             nextLevel = false;
 
@@ -258,7 +284,7 @@ define('io.ox/office/text/clipboardmixin', [
                             // add current list level type
                             type[listLevel + 1] = $(child).css('list-style-type') || $(child).attr('type') || ($(child).is('ol') ? 'decimal' : 'disc');
                             // insert list entry for current list level
-                            result.push({operation: 'insertList', depth: depth, listLevel: listLevel, type: type});
+                            result.push({ operation: 'insertList', depth: depth, listLevel: listLevel, type: type });
                             // look for deeper list levels
                             insertLength += findTextNodes(child, depth + 1, listLevel + 1, type);
                             // we already traversed the tree, so don't do it again
@@ -266,8 +292,8 @@ define('io.ox/office/text/clipboardmixin', [
 
                         } else if ($(child).is('li')) {
                             // insert paragraph and list element for current list level
-                            result.push({operation: Operations.PARA_INSERT, depth: depth, listLevel: listLevel});
-                            result.push({operation: 'insertListElement', depth: depth, listLevel: listLevel, type: type});
+                            result.push({ operation: Operations.PARA_INSERT, depth: depth, listLevel: listLevel });
+                            result.push({ operation: 'insertListElement', depth: depth, listLevel: listLevel, type: type });
                         }
 
                         if (nextLevel) {
@@ -278,7 +304,7 @@ define('io.ox/office/text/clipboardmixin', [
 
                 return insertLength;
 
-            } (clipboard.get(0) /*node*/, 0 /*depth*/, -1 /*listLevel*/, undefined /*type*/));
+            }(/*node*/ clipboard.get(0), /*depth*/ 0, /*listLevel*/ -1, /*type*/ undefined));
 
             return result;
         };
@@ -325,7 +351,7 @@ define('io.ox/office/text/clipboardmixin', [
 
             node = (element instanceof $) ? element[0] : element;
             style = node && element.getAttribute('style');
-            if (style && style.indexOf('mso-list:Ignore') > -1 ) {
+            if (style && style.indexOf('mso-list:Ignore') > -1) {
                 return true;
             }
 
@@ -358,7 +384,7 @@ define('io.ox/office/text/clipboardmixin', [
                 parts;
 
             parts = regEx.exec(style);
-            if (!_.isArray(parts) || !parts[1]|| !parts[2]) { return null; }
+            if (!_.isArray(parts) || !parts[1] || !parts[2]) { return null; }
 
             return { styleId: parts[1].toLowerCase(), level: listLevelStringToZeroBasedLevel(parts[2]), lfo: parseInt(parts[3], 10) };
         };
@@ -514,7 +540,7 @@ define('io.ox/office/text/clipboardmixin', [
                         styleId = msParagraphLevelInfo.styleId;
                         listLevel = msParagraphLevelInfo.level;
 
-                        if (!hasListDefinition (result, styleId, listLevel)) {
+                        if (!hasListDefinition(result, styleId, listLevel)) {
 
                             // get alt text from image bullet, or level text from bullet / numbering
                             image = $(p).find('img');
@@ -528,7 +554,6 @@ define('io.ox/office/text/clipboardmixin', [
                             numberFormat = self.getNumberFormatFromListLabelText(text);
                             fontFamily = $(p).children().first().css('font-family');
 
-
                             if (!result[styleId]) {
                                 result[styleId] = {};
                             }
@@ -698,7 +723,6 @@ define('io.ox/office/text/clipboardmixin', [
             return result;
         };
 
-
         /**
          * Returns the number format for the given list label.
          *
@@ -732,7 +756,6 @@ define('io.ox/office/text/clipboardmixin', [
             return 'bullet';
         };
 
-
         /**
          * Returns a Unicode replacement for the given char for font family Symbol or Wingdings.
          *
