@@ -6,19 +6,20 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
 define('io.ox/office/text/drawingResize', [
     'io.ox/office/tk/utils',
+    'io.ox/office/tk/utils/tracking',
     'io.ox/office/editframework/utils/attributeutils',
     'io.ox/office/drawinglayer/view/drawingframe',
     'io.ox/office/text/utils/operations',
     'io.ox/office/text/dom',
     'io.ox/office/text/position'
-], function (Utils, AttributeUtils, DrawingFrame, Operations, DOM, Position) {
+], function (Utils, Tracking, AttributeUtils, DrawingFrame, Operations, DOM, Position) {
 
     'use strict';
 
@@ -45,8 +46,8 @@ define('io.ox/office/text/drawingResize', [
 
         var // the text editor
             editor = app.getModel(),
-            // the root node of the editor DOM
-            rootNode = editor.getNode(),
+            // the root node of the editor DOM, or if inside header/footer, one of their root nodes
+            rootNode = editor.getCurrentRootNode(),
             // the node containing the scroll bar
             scrollNode = app.getView().getContentRootNode(),
             // object containing information about movable and resizable
@@ -64,7 +65,10 @@ define('io.ox/office/text/drawingResize', [
             // the container element used to visualize the movement and resizing
             moveBox = null,
             //zoom factor in floating point notation
-            zoomFactor;
+            zoomFactor,
+            // target ID of currently active root node, if existing
+            target = editor.getActiveTarget(),
+            operationProperties = {};
 
         // common deinitialization after move or resize tracking
         function leaveTracking() {
@@ -90,7 +94,7 @@ define('io.ox/office/text/drawingResize', [
                 // right/bottom distance from drawing to event point (in px)
                 // rightDrawingDistance = 0, bottomDrawingDistance = 0,
                 // the container node of a page element that contains all top-level content nodes (paragraphs and tables)
-                pageContent = isDrawingLayerNode ? rootNode : DOM.getPageContentNode(rootNode),
+                pageContent = isDrawingLayerNode ? editor.getNode() : DOM.getPageContentNode(editor.getNode()),
                 // the width of the page content node
                 pageContentWidth = isDrawingLayerNode ? Math.round(pageContent.outerWidth(true)) : Math.round(pageContent.width()),
                 // the height of the page content node
@@ -122,9 +126,9 @@ define('io.ox/office/text/drawingResize', [
                 // updating whether the drawing is located in the drawing layer
                 isDrawingLayerNode = DOM.isDrawingLayerNode(drawingNode.parent());
 
-                // updating the width, because outerWidth is dependent from zoom level
+                // updating the width, because outerWidth is dependent from zoom level (also handling comment layer node correctly)
                 if (isDrawingLayerNode) {
-                    pageContentWidth = Math.round(pageContent.outerWidth(true));
+                    pageContentWidth = Math.round(pageContent.outerWidth(true) - (pageContent.hasClass(DOM.COMMENTMARGIN_CLASS) ? pageContent.data(DOM.COMMENTMARGIN_CLASS) : 0));
                     pageContentHeight = Math.round(pageContent.outerHeight(true));
                 }
 
@@ -139,19 +143,19 @@ define('io.ox/office/text/drawingResize', [
                 // the left offset of the page content node -> the drawing must be right of this position
                 minLeftPosition = Math.round(pageContent.offset().left);
                 // the top position of the page content, not the page itself
-                minTopPosition = Math.round(pageContent.offset().top);
+                minTopPosition = editor.isHeaderFooterEditState() ? Math.round(rootNode.offset().top)  : Math.round(pageContent.offset().top);
 
                 if (isDrawingLayerNode) {
                     // Forcing the drawing to stay on the page
                     // the left offset of the page content node plus its width -> the drawing must be left of this position
                     maxRightPosition = minLeftPosition + pageContentWidth - drawingWidth * zoomFactor;
                     // adding the height of the page content (can be only one paragraph) to the top position of the page content
-                    maxBottomPosition = minTopPosition + pageContentHeight - drawingHeight * zoomFactor;
+                    maxBottomPosition = minTopPosition + (editor.isHeaderFooterEditState() ? (isDrawingLayerNode ? rootNode.outerHeight(true) : rootNode.height()) :  pageContentHeight) - drawingHeight * zoomFactor;
                 } else {
                     // the left offset of the page content node plus its width -> the drawing must be left of this position
                     maxRightPosition = minLeftPosition + (pageContentWidth * zoomFactor);
                     // adding the height of the page content (can be only one paragraph) to the top position of the page content
-                    maxBottomPosition = minTopPosition + (pageContentHeight * zoomFactor);
+                    maxBottomPosition = minTopPosition + ((editor.isHeaderFooterEditState() ? (isDrawingLayerNode ? rootNode.outerHeight(true) : rootNode.height()) :  pageContentHeight) * zoomFactor);
                 }
             }
 
@@ -278,7 +282,13 @@ define('io.ox/office/text/drawingResize', [
                     // the page layout manager
                     pageLayout = null,
                     // the page number
-                    pageNumber = 1;
+                    pageNumber = 1,
+                    // the active root node, might be header or footer
+                    activeRootNode = editor.getCurrentRootNode(),
+                    // whether the active root node is a header or footer node
+                    isHeaderOrFooter = DOM.isHeaderOrFooter(activeRootNode),
+                    // an optional footer offset
+                    footerOffset = 0;
 
                 function adaptPositionIntoDocument(posX, posY) {
 
@@ -369,25 +379,24 @@ define('io.ox/office/text/drawingResize', [
                     if (paragraph) { searchFollowing = false; }
 
                     if (searchFollowing) {
-                        paragraph = iterateSelectorNodes(topNode, Utils.getDomNode(startNode), posX, posY, DOM.PARAGRAPH_NODE_SELECTOR, DrawingFrame.NODE_SELECTOR, {'reverse': false});
+                        paragraph = iterateSelectorNodes(topNode, Utils.getDomNode(startNode), posX, posY, DOM.PARAGRAPH_NODE_SELECTOR, DrawingFrame.NODE_SELECTOR, { reverse: false });
                     }
 
-                    if (paragraph) { searchPrevious = false; }
-                    else { searchPrevious = true; }
+                    searchPrevious = !paragraph;
 
                     if (searchPrevious) {
-                        paragraph = iterateSelectorNodes(topNode, Utils.getDomNode(startNode), posX, posY, DOM.PARAGRAPH_NODE_SELECTOR, DrawingFrame.NODE_SELECTOR, {'reverse': true});
+                        paragraph = iterateSelectorNodes(topNode, Utils.getDomNode(startNode), posX, posY, DOM.PARAGRAPH_NODE_SELECTOR, DrawingFrame.NODE_SELECTOR, { reverse: true });
                     }
 
                     // maybe the paragraph is in a table cell with a cell neighbor that is much higher -> use last paragraph in this cell
-                    if (! paragraph) {
-                        tableCell = iterateSelectorNodes(topNode, Utils.getDomNode(startNode), posX, posY, DOM.TABLE_CELLNODE_SELECTOR, DrawingFrame.NODE_SELECTOR, {'reverse': false});
+                    if (!paragraph) {
+                        tableCell = iterateSelectorNodes(topNode, Utils.getDomNode(startNode), posX, posY, DOM.TABLE_CELLNODE_SELECTOR, DrawingFrame.NODE_SELECTOR, { reverse: false });
 
-                        if (! tableCell) {
-                            tableCell = iterateSelectorNodes(topNode, Utils.getDomNode(startNode), posX, posY, DOM.TABLE_CELLNODE_SELECTOR, DrawingFrame.NODE_SELECTOR, {'reverse': true});
+                        if (!tableCell) {
+                            tableCell = iterateSelectorNodes(topNode, Utils.getDomNode(startNode), posX, posY, DOM.TABLE_CELLNODE_SELECTOR, DrawingFrame.NODE_SELECTOR, { reverse: true });
                         }
 
-                        if ((tableCell) && (DOM.isTableCellNode(tableCell)) && (! DOM.isExceededSizeTableNode($(tableCell).closest(DOM.TABLE_NODE_SELECTOR)))) { // check if table is exceeded size for #33275
+                        if (tableCell && DOM.isTableCellNode(tableCell) && !DOM.isExceededSizeTableNode($(tableCell).closest(DOM.TABLE_NODE_SELECTOR))) { // check if table is exceeded size for #33275
                             paragraph = DOM.getCellContentNode(tableCell)[0].lastChild;  // the last paragraph of the cell content
                         }
                     }
@@ -419,20 +428,10 @@ define('io.ox/office/text/drawingResize', [
 
                 // begin of stopMoveDrawing
 
-                if (! mouseMoved) {
+                if (!mouseMoved) {
                     return;
                 }
 
-//                // reading scrollPosition again. Maybe it was not updated or not updated completely.
-//                scrollX = scrollNode.scrollLeft() - startScrollX;
-//                scrollY = scrollNode.scrollTop() - startScrollY;
-//
-//                shiftX = (event.pageX - event.startX + scrollX) / zoomFactor;
-//                shiftY = (event.pageY - event.startY + scrollY) / zoomFactor;
-//
-//                // only move the moveBox, if the mouse was really moved (Fix for 28633)
-//                mouseMoved = ((shiftX !== 0) || (shiftY !== 0));
-
                 zoomFactor = app.getView().getZoomFactor() / 100;
 
                 // setting position of tracking:end event
@@ -461,20 +460,37 @@ define('io.ox/office/text/drawingResize', [
 
                     if (isDrawingLayerNode) {
 
-                        startPosition = Position.getPixelPositionToRootNodeOffset(rootNode, drawingNode, zoomFactor * 100);
+                        activeRootNode = editor.getCurrentRootNode();
+
+                        isHeaderOrFooter = DOM.isHeaderOrFooter(activeRootNode);
+
+                        startPosition = Position.getPixelPositionToRootNodeOffset(activeRootNode, drawingNode, zoomFactor * 100);
 
                         maxVertPageOffset = Utils.convertHmmToLength(app.getModel().getPageLayout().getPageAttribute('height'), 'px', 0.001) - drawingNode.height();
 
                         // checking the page number of the drawing
                         pageLayout = app.getModel().getPageLayout();
-                        pageNumber = pageLayout.getPageNumber(DOM.getDrawingPlaceHolderNode(drawingNode));
-                        if (pageNumber > 1) { startPosition.y = startPosition.y - Position.getVerticalPagePixelPosition(rootNode, pageLayout, pageNumber, zoomFactor * 100); }
+
+                        // checking the page number of the drawing
+                        pageNumber = isHeaderOrFooter ? 1 : pageNumber = pageLayout.getPageNumber(DOM.getDrawingPlaceHolderNode(drawingNode));
+
+                        if (pageNumber > 1) { startPosition.y = startPosition.y - Position.getVerticalPagePixelPosition(editor.getNode(), pageLayout, pageNumber, zoomFactor * 100); }
 
                         anchorHorOffset = Utils.convertLengthToHmm(startPosition.x + shiftX, 'px');
 
                         anchorVertOffset = startPosition.y + shiftY;
-                        if (anchorVertOffset < 0) { anchorVertOffset = 0; }  // forcing on the same page, no shift to page above
-                        else if (anchorVertOffset > maxVertPageOffset) { anchorVertOffset = maxVertPageOffset; }  // forcing on the same page, no shift to page below
+
+                        if (DOM.isFooterNode(activeRootNode)) {
+                            // if this is a footer, the height between page start and footer need to be added
+                            footerOffset = Utils.convertHmmToLength(pageLayout.getPageAttribute('height'), 'px', 1) - Utils.round($(activeRootNode).outerHeight(true), 1);
+                            anchorVertOffset += footerOffset;
+                        }
+
+                        if (anchorVertOffset < 0) {
+                            anchorVertOffset = 0; // forcing on the same page, no shift to page above
+                        } else if (anchorVertOffset > maxVertPageOffset) {
+                            anchorVertOffset = maxVertPageOffset; // forcing on the same page, no shift to page below
+                        }
 
                         anchorVertOffset = Utils.convertLengthToHmm(anchorVertOffset, 'px');
 
@@ -531,8 +547,11 @@ define('io.ox/office/text/drawingResize', [
                                 anchorHorOffset = oldAnchorHorOffset + moveX;
                                 anchorHorAlign = 'offset';
                                 anchorHorBase = 'column';
-                                if (anchorHorOffset < 0) { anchorHorOffset = 0; }
-                                else if (anchorHorOffset > (paraWidth - drawingWidth)) { anchorHorOffset = paraWidth - drawingWidth; }
+                                if (anchorHorOffset < 0) {
+                                    anchorHorOffset = 0;
+                                } else if (anchorHorOffset > (paraWidth - drawingWidth)) {
+                                    anchorHorOffset = paraWidth - drawingWidth;
+                                }
                             }
 
                             if (moveY !== 0) {
@@ -556,10 +575,10 @@ define('io.ox/office/text/drawingResize', [
                                 // Moving the drawing to the beginning of the paragraph
                                 // There might be no offset node before (offsetNodeHeight === 0) or there can be an offset node
                                 // that reaches the beginning of the paragraph or there can also be an offset node, that starts
-                                // in the middle of the paragraph. The latter can be achieved by swichting from inline to floated
+                                // in the middle of the paragraph. The latter can be achieved by switching from inline to floated
                                 // and then moving down the drawing inside the paragraph. If such an offset node is not high enough,
                                 // the drawing must be moved to the beginning of the paragraph behind other floated drawings.
-                                if ((moveY < 0) && ((- moveY) > offsetNodeHeight)) {
+                                if ((moveY < 0) && (-moveY > offsetNodeHeight)) {
                                     destPosition = _.clone(updatePosition);
                                     destPosition[destPosition.length - 1] = Position.getLeadingFloatingDrawingCount(paragraph, drawingTopY - (paragraph.offset().top / zoomFactor), drawingNode); //#33245 passing optional drawing node, to exclude itself from count
                                     moveImage = true;
@@ -631,23 +650,27 @@ define('io.ox/office/text/drawingResize', [
                         if (editor.getChangeTrack().isActiveChangeTracking() && doChangeTrackMove && !editor.getChangeTrack().isInsertNodeByCurrentAuthor(drawingNode)) {
                             // if change tracking is active, the old drawing position must be marked as 'deleted' ...
                             // -> this is necessary for in-line and non-in-line drawings
-                            generator.generateOperation(Operations.SET_ATTRIBUTES, {
-                                attrs: { changes: { 'removed': editor.getChangeTrack().getChangeTrackInfo() }},
+                            operationProperties = {
+                                attrs: { changes: { removed: editor.getChangeTrack().getChangeTrackInfo() }},
                                 start: updatePosition
-                            });
+                            };
+                            editor.extendPropertiesWithTarget(operationProperties, target);
+                            generator.generateOperation(Operations.SET_ATTRIBUTES, operationProperties);
                         } else if (DOM.isInlineDrawingNode(drawingNode)) {
                             // switching from in-line to floated mode (vertically aligned at paragraph)
-                            generator.generateOperation(Operations.SET_ATTRIBUTES, {
+                            operationProperties = {
                                 attrs: { drawing: { inline: false, anchorHorBase: 'column', anchorVertBase: 'paragraph', anchorHorAlign: 'right', anchorHorOffset: 0, textWrapMode: 'topAndBottom', textWrapSide: null } },
                                 start: updatePosition
-                            });
+                            };
+                            editor.extendPropertiesWithTarget(operationProperties, target);
+                            generator.generateOperation(Operations.SET_ATTRIBUTES, operationProperties);
                         }
                     }
 
                     // 2. Moving the image, if this is necessary
                     // -> In case of activated change tracking, this is only necessary, if the drawing was inserted
                     //    before by the current author. Never moving drawings located in the drawing layer node.
-                    if ((moveImage) && (!isDrawingLayerNode) && (! _.isEqual(updatePosition, destPosition))) {
+                    if (moveImage && !isDrawingLayerNode && !_.isEqual(updatePosition, destPosition)) {
 
                         // moving the drawing, if change tracking is NOT active or it is active
                         // and the current author has inserted the drawing before
@@ -663,16 +686,20 @@ define('io.ox/office/text/drawingResize', [
                                     // removing implicit paragraph node
                                     $(localParagraph).remove();
                                     // creating new paragraph explicitly
-                                    generator.generateOperation(Operations.PARA_INSERT, {start: localPosition});
+                                    operationProperties = { start: localPosition };
+                                    editor.extendPropertiesWithTarget(operationProperties, target);
+                                    generator.generateOperation(Operations.PARA_INSERT, operationProperties);
                                 }
                             }
 
                             // move operation
-                            generator.generateOperation(Operations.MOVE, {
+                            operationProperties = {
                                 start: updatePosition,
                                 end: updatePosition,
                                 to: destPosition
-                            });
+                            };
+                            editor.extendPropertiesWithTarget(operationProperties, target);
+                            generator.generateOperation(Operations.MOVE, operationProperties);
 
                             updatePosition = _.clone(destPosition); // for setting attributes required
                             if (moveDownwards) {
@@ -712,7 +739,9 @@ define('io.ox/office/text/drawingResize', [
                                     // removing implicit paragraph node
                                     $(localParagraph).remove();
                                     // creating new paragraph explicitly
-                                    generator.generateOperation(Operations.PARA_INSERT, {start: localPosition});
+                                    operationProperties = { start: localPosition };
+                                    editor.extendPropertiesWithTarget(operationProperties, target);
+                                    generator.generateOperation(Operations.PARA_INSERT, operationProperties);
                                 }
                             }
 
@@ -726,28 +755,32 @@ define('io.ox/office/text/drawingResize', [
                                 allDrawingAttrs.drawing.textWrapSide = null;
                             }
 
-                            allDrawingAttrs.changes = { 'inserted': editor.getChangeTrack().getChangeTrackInfo(), 'removed': null };
+                            allDrawingAttrs.changes = { inserted: editor.getChangeTrack().getChangeTrackInfo(), removed: null };
 
-                            generator.generateOperation(Operations.DRAWING_INSERT, {
-                                                            start: destPosition,
-                                                            type: DrawingFrame.getDrawingType(drawingNode),
-                                                            attrs: allDrawingAttrs
-                                                        });
+                            operationProperties = {
+                                start: destPosition,
+                                type: DrawingFrame.getDrawingType(drawingNode),
+                                attrs: allDrawingAttrs
+                            };
+                            editor.extendPropertiesWithTarget(operationProperties, target);
+                            generator.generateOperation(Operations.DRAWING_INSERT, operationProperties);
 
                             updatePosition = _.clone(destPosition); // for setting attributes required
                         }
 
                         // and finally setting the new position attributes via a setAttributes operation
-                        generator.generateOperation(Operations.SET_ATTRIBUTES, {
+                        operationProperties = {
                             attrs: { drawing: { anchorHorOffset: anchorHorOffset, anchorVertOffset: anchorVertOffset, anchorHorAlign: anchorHorAlign, anchorVertAlign: anchorVertAlign, anchorHorBase: anchorHorBase, anchorVertBase: anchorVertBase } },
                             start: updatePosition
-                        });
+                        };
+                        editor.extendPropertiesWithTarget(operationProperties, target);
+                        generator.generateOperation(Operations.SET_ATTRIBUTES, operationProperties);
 
                     }
                 }
 
                 // apply the operations (undo group is created automatically)
-                editor.applyOperations(generator.getOperations());
+                editor.applyOperations(generator);
 
                 if (imageMoved) {
                     // set new text selection after moving the drawing, this will repaint the selection
@@ -778,8 +811,7 @@ define('io.ox/office/text/drawingResize', [
                 // handling for text frame drawings, if the click happened inside the internal text frame element
                 if ($(event.target).closest(DrawingFrame.TEXTFRAME_NODE_SELECTOR).length > 0) {
 
-                    drawingNode
-                        .disableTracking()
+                    Tracking.disableTracking(drawingNode)
                         .off('tracking:start', trackingStartHandler)
                         .off(TRACKING_EVENT_NAMES);
 
@@ -883,7 +915,7 @@ define('io.ox/office/text/drawingResize', [
                 paragraph = drawingNode.parent();
                 maxDrawingWidth = paragraph.width();  // using only the paragraph width, no padding or margin (task 28867)
 
-                pageAttributes = editor.getStyleCollection('page').getElementAttributes(rootNode);
+                pageAttributes = editor.getStyleCollection('page').getElementAttributes(editor.getNode());
                 // reading page attributes, they are always available -> no need to check existence
                 maxDrawingHeight = pageAttributes.page.height - pageAttributes.page.marginTop - pageAttributes.page.marginBottom;
                 maxDrawingHeight = Utils.convertHmmToLength(maxDrawingHeight, 'px', 1);
@@ -1004,8 +1036,14 @@ define('io.ox/office/text/drawingResize', [
                     startPosition = null,
                     // the page layout manager
                     pageLayout = null,
+                    // the affected drawings with auto resize functionality
+                    allAutoResizeNodes = null,
                     // the page number
-                    pageNumber = 1;
+                    pageNumber = 1,
+                    // the active root node, this can be a header or footer
+                    activeRootNode = null,
+                    // an optional offset of the footer node in the page
+                    footerOffset = 0;
 
                 // Calculating new width and height
 
@@ -1028,15 +1066,18 @@ define('io.ox/office/text/drawingResize', [
                 // simplified handling for drawings in drawing layer
                 if (isDrawingLayerNode) {
 
+                    activeRootNode = editor.getCurrentRootNode();
+
                     zoomFactor = app.getView().getZoomFactor();
 
                     // is it necessary to update the upper left corner of the drawing?
-                    startPosition = Position.getPixelPositionToRootNodeOffset(rootNode, drawingNode, zoomFactor);
+                    startPosition = Position.getPixelPositionToRootNodeOffset(activeRootNode, drawingNode, zoomFactor);
+                    pageLayout = app.getModel().getPageLayout();
 
                     // checking the page number of the drawing
-                    pageLayout = app.getModel().getPageLayout();
-                    pageNumber = pageLayout.getPageNumber(DOM.getDrawingPlaceHolderNode(drawingNode));
-                    if (pageNumber > 1) { startPosition.y = startPosition.y - Position.getVerticalPagePixelPosition(rootNode, pageLayout, pageNumber, zoomFactor); }
+                    pageNumber = DOM.isHeaderOrFooter(activeRootNode) ? 1 : pageLayout.getPageNumber(DOM.getDrawingPlaceHolderNode(drawingNode));
+
+                    if (pageNumber > 1) { startPosition.y = startPosition.y - Position.getVerticalPagePixelPosition(editor.getNode(), pageLayout, pageNumber, zoomFactor); }
 
                     if (topResize) {
                         // comparing final height and old height
@@ -1045,6 +1086,12 @@ define('io.ox/office/text/drawingResize', [
                         attributes.anchorVertOffset = Utils.convertLengthToHmm(startPosition.y, 'px');
                     }
 
+                    if (DOM.isFooterNode(activeRootNode)) {
+                        // if this is a footer, the height between page start and footer need to be added
+                        footerOffset = pageLayout.getPageAttribute('height') - Utils.convertLengthToHmm($(activeRootNode).outerHeight(true), 'px');
+                        attributes.anchorVertOffset += footerOffset;
+                    }
+
                     if (leftResize) {
                         // comparing final width and old width
                         attributes.anchorHorOffset = Math.max(Utils.convertLengthToHmm(startPosition.x, 'px') - Utils.convertLengthToHmm((finalWidth - oldWidth), 'px'), 0);
@@ -1067,7 +1114,7 @@ define('io.ox/office/text/drawingResize', [
 
                         // moveY > 0 -> Drawing was increased -> Moving drawing upwards
                         // moveY < 0 -> Drawing was decreased -> Moving drawing downwards
-                        moveYHmm = - Utils.convertLengthToHmm(moveY, 'px');  // negative value
+                        moveYHmm = -Utils.convertLengthToHmm(moveY, 'px');  // negative value
                         // moveYHmm > 0 -> moving drawing downwards -> value can be added to existing vertical offset
                         // moveYHmm < 0 -> moving drawing upwards -> value can be added to existing vertical offset
 
@@ -1094,7 +1141,7 @@ define('io.ox/office/text/drawingResize', [
                         }
 
                         // Resizing the drawing to the beginning of the paragraph
-                        if ((moveYHmm < 0) && ((- moveYHmm) > offsetNodeHeight)) {
+                        if ((moveYHmm < 0) && (-moveYHmm > offsetNodeHeight)) {
                             upperDrawings = Position.getLeadingFloatingDrawingCount(paragraph, paragraphDeltaY);
                             if (_.last(updatePosition) > upperDrawings) {
                                 destPosition = _.clone(updatePosition);
@@ -1125,15 +1172,17 @@ define('io.ox/office/text/drawingResize', [
                             attributes.anchorVertBase = anchorVertBase;
 
                             // Generating the move operation
-                            if ((moveImage) && (! _.isEqual(updatePosition, destPosition))) {
+                            if (moveImage && !_.isEqual(updatePosition, destPosition)) {
                                 // moving image not in another paragraph using resize
                                 // -> checking for implicit paragraphs not required (like in stopMoveDrawing).
 
-                                generator.generateOperation(Operations.MOVE, {
+                                operationProperties = {
                                     start: updatePosition,
                                     end: updatePosition,
                                     to: destPosition
-                                });
+                                };
+                                editor.extendPropertiesWithTarget(operationProperties, target);
+                                generator.generateOperation(Operations.MOVE, operationProperties);
 
                                 updatePosition = _.clone(destPosition); // for setting attributes required
                                 if (moveDownwards) {
@@ -1153,8 +1202,25 @@ define('io.ox/office/text/drawingResize', [
 
                     // after resizing an automatic resized text frame in vertical direction by the user,
                     // the automatic resize will be removed
-                    if (_.isNumber(attributes.height) && DrawingFrame.isAutoResizeHeightDrawingFrame(drawingNode)) {
-                        allAttrs.shape = { autoResizeHeight: false };
+                    if (_.isNumber(attributes.height)) {
+                        if (DrawingFrame.isAutoResizeHeightDrawingFrame(drawingNode)) {
+                            allAttrs.shape = { autoResizeHeight: false };
+                        } else if (DrawingFrame.isGroupDrawingFrame(drawingNode)) {
+                            // disable autoResizeHeight for all children inside a resized group
+                            allAutoResizeNodes = drawingNode.find(DrawingFrame.AUTORESIZEHEIGHT_SELECTOR);
+                            if (allAutoResizeNodes.length > 0) {
+                                _.each(allAutoResizeNodes, function (resizeNode) {
+                                    var localDrawingNode = $(resizeNode).parent(),
+                                        localDrawingPos = Position.getOxoPosition(rootNode, localDrawingNode, 0);
+
+                                    generator.generateOperation(Operations.SET_ATTRIBUTES, {
+                                        start: localDrawingPos,
+                                        attrs: { shape: { autoResizeHeight: false }}
+                                    });
+
+                                });
+                            }
+                        }
                     }
 
                     // Resize of drawing is not supported yet in OOXML file format
@@ -1166,18 +1232,20 @@ define('io.ox/office/text/drawingResize', [
                         // adding the old attributes, author and date for change tracking
                         if (oldAttrs) {
                             oldAttrs = _.extend(oldAttrs, editor.getChangeTrack().getChangeTrackInfo());
-                            allAttrs.changes = { 'modified': oldAttrs };
+                            allAttrs.changes = { modified: oldAttrs };
                         }
                     }
 
-                    generator.generateOperation(Operations.SET_ATTRIBUTES, {
+                    operationProperties = {
                         start: updatePosition,
                         attrs: allAttrs
-                    });
+                    };
+                    editor.extendPropertiesWithTarget(operationProperties, target);
+                    generator.generateOperation(Operations.SET_ATTRIBUTES, operationProperties);
                 }
 
                 // Applying the operations (undo group is created automatically)
-                editor.applyOperations(generator.getOperations());
+                editor.applyOperations(generator);
 
                 // Triggering text frame border to be repainted
                 if (paragraph) { app.getModel().trigger('paragraphUpdate:after', paragraph); }
@@ -1283,38 +1351,35 @@ define('io.ox/office/text/drawingResize', [
         moveBox = selectionBox.children('.tracker');
 
         if (options.movable) {
-            drawingNode
-                .enableTracking({
-                    autoScroll: true,
-                    borderNode: scrollNode,
-                    borderMargin: -30,
-                    borderSize: 60,
-                    minSpeed: 10,
-                    maxSpeed: 250
-                })
-                .on('tracking:start', trackingStartHandler);
+            Tracking.enableTracking(drawingNode, {
+                autoScroll: true,
+                borderNode: scrollNode,
+                borderMargin: -30,
+                borderSize: 60,
+                minSpeed: 10,
+                maxSpeed: 250
+            });
+            drawingNode.on('tracking:start', trackingStartHandler);
         } else {
-            drawingNode
-                .disableTracking()
-                .off('tracking:start', trackingStartHandler);
+            Tracking.disableTracking(drawingNode);
+            drawingNode.off('tracking:start', trackingStartHandler);
         }
 
         // initialize resize tracking
+        var resizeHandles = selectionBox.find('>.resizers>[data-pos]');
         if (options.resizable) {
-            selectionBox.find('>.resizers>[data-pos]')
-                .enableTracking({
-                    autoScroll: true,
-                    borderNode: scrollNode,
-                    borderMargin: -30,
-                    borderSize: 60,
-                    minSpeed: 10,
-                    maxSpeed: 250
-                })
-                .on('tracking:start', trackingStartHandler);
+            Tracking.enableTracking(resizeHandles, {
+                autoScroll: true,
+                borderNode: scrollNode,
+                borderMargin: -30,
+                borderSize: 60,
+                minSpeed: 10,
+                maxSpeed: 250
+            });
+            resizeHandles.on('tracking:start', trackingStartHandler);
         } else {
-            selectionBox.children('>.resizers>[data-pos]')
-                .disableTracking()
-                .off('tracking:start', trackingStartHandler);
+            Tracking.disableTracking(resizeHandles);
+            resizeHandles.off('tracking:start', trackingStartHandler);
         }
     };
 
