@@ -6,29 +6,29 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Carsten Driesner <carsten.driesner@open-xchange.com>
  */
 
-define('io.ox/office/text/export',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/text/dom',
-     'io.ox/office/drawinglayer/view/imageutil'
-    ], function (Utils, DOM, Image) {
+define('io.ox/office/text/export', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/text/dom',
+    'io.ox/office/drawinglayer/view/imageutil'
+], function (Utils, DOM, Image) {
 
     'use strict';
 
-    var Export = {
-        copyStyles: {
-            paragraph: ['text-align', 'background-color', 'padding', 'margin', 'text-indent', 'margin', 'border-color', 'border-style', 'border-width'],
-            span: ['font-family', 'font-weight', 'text-decoration', 'background-color', 'color', 'font-size', 'vertical-align', 'line-height', 'lang'],
-            table: ['width', 'background-color'],
-            td: ['height', 'background-color', 'border-top-style', 'border-right-style', 'border-bottom-style', 'border-left-style', 'border-top-width', 'border-right-width', 'border-bottom-width', 'border-left-width', 'border-color', 'padding'],
-            img: ['width', 'height', 'border', 'margin', 'title']
-        }
+    var COPY_STYLES = {
+        paragraph: ['text-align', 'background-color', 'padding', 'margin', 'text-indent', 'margin', 'border-color', 'border-style', 'border-width'],
+        span: ['font-family', 'font-weight', 'text-decoration', 'background-color', 'color', 'font-size', 'vertical-align', 'line-height', 'lang'],
+        table: ['width', 'background-color'],
+        td: ['height', 'background-color', 'border-top-style', 'border-right-style', 'border-bottom-style', 'border-left-style', 'border-top-width', 'border-right-width', 'border-bottom-width', 'border-left-width', 'border-color', 'padding'],
+        img: ['width', 'height', 'border', 'margin', 'title']
     };
 
+    var Export = {};
+
     /**
      * Converts the internal HTML mark-up of the current selection to
      * normalized HTML mark-up that can be exported (e.g. to the clipboard).
@@ -52,9 +52,9 @@ define('io.ox/office/text/export',
         var resultDiv = $('<div>'),
             selection = editor.getSelection(),
             paragraphStyles = editor.getStyleCollection('paragraph'),
-            listStyles = editor.getDocumentStyles().getCustomCollection('lists'),
+            listStyles = editor.getListCollection(),
             paraStylesIds = createParagraphStyleIds(selection, paragraphStyles),
-            html = $('<div>').html(selection.getHtmlFromBrowserSelection()),
+            html = $('<span>').html(selection.getHtmlFromBrowserSelection()),
             clipboardId = Utils.getStringOption(options, 'clipboardId', ''),
             clipboardOperations = Utils.getArrayOption(options, 'clipboardOperations', []),
             metaElement;
@@ -73,7 +73,9 @@ define('io.ox/office/text/export',
                 metaElement.attr('data-ox-operations', JSON.stringify(clipboardOperations));
             }
         }
-
+        if (resultDiv.children().length === 1) {
+            resultDiv.append('<span>&#8203;</span>');
+        }
         return resultDiv.html();
     };
 
@@ -88,12 +90,13 @@ define('io.ox/office/text/export',
      *  The result html.
      */
     function createOptimizedStructure(source, result, paragraphStylesMap, listStyles) {
-        var node = $(source), optNode, outNode, tbody,
+
+        var outNode, tbody,
             trow, td, cellContent, colgroup, styles,
             optimized = false, listLevel = -1, listStyle, listLevels = [];
 
-        _(node.children()).each(function (node) {
-            optNode = $(node);
+        $(source).children().each(function () {
+            var optNode = $(this);
 
             if (DOM.isParagraphNode(optNode)) {
                 styles = paragraphStylesMap[optNode.attr('styleId')];
@@ -129,13 +132,13 @@ define('io.ox/office/text/export',
                     outNode.append(colgroup);
                 }
                 tbody = $('<tbody>');
-                _(optNode.find('tr')).each(function (row) {
+                optNode.find('tr').each(function () {
                     trow = $('<tr>');
-                    _($(row).find('td')).each(function (cell) {
+                    $(this).find('td').each(function () {
                         td = $('<td>');
-                        td.attr('colspan', cell.colSpan);
-                        copyStyles(cell, td, 'td');
-                        cellContent = $(cell).find('div.cellcontent');
+                        td.attr('colspan', this.colSpan);
+                        copyStyles(this, td, 'td');
+                        cellContent = $(this).find('div.cellcontent');
                         if (cellContent) {
                             createOptimizedStructure(cellContent, td, paragraphStylesMap, listStyles);
                         }
@@ -275,7 +278,7 @@ define('io.ox/office/text/export',
     function copyStyles(from, to, type) {
         var source = $(from), target = $(to);
 
-        _(Export.copyStyles[type]).each(function (style) {
+        _.each(COPY_STYLES[type], function (style) {
             target.css(style, source.css(style));
         });
     }
@@ -291,20 +294,16 @@ define('io.ox/office/text/export',
      */
     function optimizeParagraphForExport(paragraph, parentNode) {
 
-        _(paragraph.children()).each(function (node) {
-            var tmpNode = null, img = null, valign = 0;
+        paragraph.children().each(function () {
+            var node = $(this), tmpNode = null, img = null, valign = 0;
 
             if (DOM.isSpan(node)) {
-                valign = $(node).css('vertical-align');
+                valign = node.css('vertical-align');
                 // check for vertical-align which is used for sub/superscript
                 if (valign && valign !== 'baseline') {
                     valign = parseFloat(valign);
-                    if (valign < 0) {
-                        tmpNode = $('<sub>');
-                    } else {
-                        tmpNode = $('<sup>');
-                    }
-                    $(node).css('vertical-align', 'baseline');
+                    tmpNode = $((valign < 0) ? '<sub>' : '<sup>');
+                    node.css('vertical-align', 'baseline');
                     parentNode.append(tmpNode);
                     tmpNode.append(node);
                 } else {
@@ -312,7 +311,7 @@ define('io.ox/office/text/export',
                 }
             } else if (DOM.isImageNode(node)) {
                 tmpNode = $('<img>');
-                img = $(node).find('img');
+                img = node.find('img');
                 if (DOM.isDocumentImageNode(node)) {
                     tmpNode.attr('src', DOM.getBase64FromImageNode(node, Image.getMimeTypeFromImageUri(DOM.getUrlParamFromImageNode(node, 'get_filename'))));
                 } else {
@@ -327,14 +326,16 @@ define('io.ox/office/text/export',
             } else if (DOM.isTabNode(node)) {
                 tmpNode = $('<span>');
                 copyStyles(node, tmpNode, 'span');
-                if (node.textContent) {
-                    tmpNode.text(node.textContent);
+                if (node[0].textContent) {
+                    tmpNode.text(node[0].textContent);
                 } else {
                     DOM.ensureExistingTextNode(tmpNode);
                 }
                 parentNode.append(tmpNode);
-            } else if ($(node).is('br')) {
+            } else if (node.is('br')) {
                 parentNode.append(node);
+            } else if (DOM.isHardBreakNode(node)) {
+                parentNode.append($('<br>'));
             }
         });
     }
@@ -395,7 +396,6 @@ define('io.ox/office/text/export',
      *  Current selection.
      */
     function clearParagraphStyleIds(selection) {
-        var i, len, paras;
 
         // visit the paragraphs and tables covered by the text selection
         selection.iterateContentNodes(function (contentNode) {
@@ -405,11 +405,7 @@ define('io.ox/office/text/export',
                 $(contentNode).removeAttr('styleId');
             // entire table: generate complete operations array for the table
             } else if (DOM.isTableNode(contentNode)) {
-                paras = $(contentNode).find('div.p');
-                len = paras.length;
-                for (i = 0; i < len; i++) {
-                    $(paras[i]).removeAttr('styleId');
-                }
+                $(contentNode).find('div.p').removeAttr('styleId');
             } else {
                 return Utils.BREAK;
             }
