@@ -6,22 +6,22 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
-define('io.ox/office/text/format/drawingstyles',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/utils/attributeutils',
-     'io.ox/office/editframework/utils/border',
-     'io.ox/office/drawinglayer/model/drawingstylecollection',
-     'io.ox/office/drawinglayer/view/drawingframe',
-     'io.ox/office/text/drawingResize',
-     'io.ox/office/text/dom',
-     'io.ox/office/text/position'
-    ], function (Utils, AttributeUtils, Border, DrawingStyleCollection, DrawingFrame, DrawingResize, DOM, Position) {
+define('io.ox/office/text/format/drawingstyles', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/utils/attributeutils',
+    'io.ox/office/editframework/utils/border',
+    'io.ox/office/drawinglayer/model/drawingstylecollection',
+    'io.ox/office/drawinglayer/view/drawingframe',
+    'io.ox/office/text/drawingResize',
+    'io.ox/office/text/dom',
+    'io.ox/office/text/position'
+], function (Utils, AttributeUtils, Border, DrawingStyleCollection, DrawingFrame, DrawingResize, DOM, Position) {
 
     'use strict';
 
@@ -183,17 +183,14 @@ define('io.ox/office/text/format/drawingstyles',
      *
      * @extends DrawingStyleCollection
      *
-     * @param {TextApplication} app
-     *  The root application instance.
-     *
-     * @param {DocumentStyles} documentStyles
-     *  Collection with the style containers of all style families.
+     * @param {TextModel} docModel
+     *  The text document model containing instance.
      */
-    function TextDrawingStyles(app, documentStyles) {
+    function TextDrawingStyles(docModel) {
 
         // base constructor ---------------------------------------------------
 
-        DrawingStyleCollection.call(this, app, documentStyles, {
+        DrawingStyleCollection.call(this, docModel, {
             families: 'line fill image shape changes',
             formatHandler: updateDrawingFormatting
         });
@@ -290,13 +287,13 @@ define('io.ox/office/text/format/drawingstyles',
                 // maximum line height in paragraph
                 maxLineHeightInParagraph,
                 // the page node
-                pageNode = app.getModel().getNode(),
+                pageNode = docModel.getNode(),
                 // the current zoom factor in percent
-                zoomFactor = app.getView().getZoomFactor() / 100,
+                zoomFactor = docModel.getApp().getView().getZoomFactor() / 100,
                 // the height of the top margin in 1/100 mm
                 topMarginHeight = 0,
-                // the width of the page in 1/100 mm
-                pageWidth = Math.round(Utils.convertLengthToHmm(pageNode.outerWidth(true), 'px') / zoomFactor),
+                // the width of the page in 1/100 mm (handling the margin for a comment node correctly)
+                pageWidth = Math.round(Utils.convertLengthToHmm(pageNode.outerWidth(true) - (pageNode.hasClass(DOM.COMMENTMARGIN_CLASS) ? pageNode.data(DOM.COMMENTMARGIN_CLASS) : 0), 'px') / zoomFactor),
                 // whether the drawing is positioned absolutely
                 drawingPositionedToPage = false,
                 // the horizontal and vertical offsets in pixel relative to the page
@@ -304,9 +301,9 @@ define('io.ox/office/text/format/drawingstyles',
                 // an optional text indent of the paragraph
                 textIndent = 0,
                 // the drawing layer for absolutely positioned drawings
-                drawingLayer = app.getModel().getDrawingLayer(),
+                drawingLayer = docModel.getDrawingLayer(),
                 // whether the drawing is located in header or footer
-                inHeaderFooter = app.isImportFinished() ? drawing.hasClass(DOM.TARGET_NODE_CLASSNAME) : (DOM.getMarginalTargetNode(pageNode, drawing).length > 0),
+                inHeaderFooter = (drawing.hasClass(DOM.TARGET_NODE_CLASSNAME) || DOM.getMarginalTargetNode(pageNode, drawing).length > 0),
                 // whether the top offset shall be used as distance to the bottom (used in footers)
                 useBottomOffset = false,
                 // the header or footer root node for the drawing
@@ -315,7 +312,7 @@ define('io.ox/office/text/format/drawingstyles',
                 borderWidth = mergedAttributes.line.width,
                 // the half width of the border of the drawing
                 // half border width goes only for text frames, full width for images; cannot be less than 1px (or 26.4 hmm), to be compatible with canvas border, #36576
-                halfBorderWidth = DrawingFrame.isTextFrameShapeDrawingFrame(drawing) ? ((0.5 * borderWidth) > 26.5 ? (0.5 * borderWidth) : 26.5 ) : borderWidth,
+                halfBorderWidth = DrawingFrame.isTextFrameShapeDrawingFrame(drawing) ? ((0.5 * borderWidth) > 26.5 ? (0.5 * borderWidth) : 26.5) : borderWidth,
                 // current drawing width, in 1/100 mm
                 // -> since the border is drawn with a canvas, the drawing width (including border) needs to be adjusted
                 // -> the canvas draws half of the border width outside and half of it inside the drawing.
@@ -395,9 +392,8 @@ define('io.ox/office/text/format/drawingstyles',
                             break;
                         case 'offset':
                             leftOffset = drawingAttributes.anchorHorOffset;
-                            if (drawingAttributes.anchorHorOffset + drawingWidth > pageWidth) {
-                                leftOffset = pageWidth - drawingWidth;
-                            }
+                            // -> do not force an absolutely positioned drawing into the page!
+                            // if (drawingAttributes.anchorHorOffset + drawingWidth > pageWidth) { leftOffset = pageWidth - drawingWidth; }
                             break;
                         default:
                             leftOffset = 0;
@@ -568,7 +564,7 @@ define('io.ox/office/text/format/drawingstyles',
                 // checking the fill color (task 36385)
                 if (explicitDrawingAttrs && explicitDrawingAttrs.fill) {
 
-                    paragraphStyles = documentStyles.getStyleCollection('paragraph'),
+                    paragraphStyles = docModel.getStyleCollection('paragraph');
                     textFrameNode = DrawingFrame.getTextFrameNode(drawing);
 
                     Utils.iterateSelectedDescendantNodes(textFrameNode, DOM.PARAGRAPH_NODE_SELECTOR, function (paragraph) {
@@ -588,14 +584,14 @@ define('io.ox/office/text/format/drawingstyles',
                 drawing.removeClass('inline float left right').addClass('absolute grouped');  // absolute positioned drawing inside its group!
 
                 // set generic formatting to the drawing frame
-                DrawingFrame.updateFormatting(app, drawing, mergedAttributes);
+                DrawingFrame.updateFormatting(docModel.getApp(), drawing, mergedAttributes);
 
                 // simply do nothing, let the group make further attribute settings
                 return;
             }
 
             // setting marker for drawings in header or footer
-            if (inHeaderFooter && !app.isImportFinished()) { drawing.addClass(DOM.TARGET_NODE_CLASSNAME); }
+            if (inHeaderFooter) { drawing.addClass(DOM.TARGET_NODE_CLASSNAME); }
 
             // position
             if (drawingAttributes.inline) {
@@ -623,8 +619,8 @@ define('io.ox/office/text/format/drawingstyles',
                         DrawingFrame.clearSelection(drawing);
                         // BUG 36372: Text frame gets blue resize/selection-border wrongly (via undo)
                         //  previously check if the drawing was selected before pressing undo
-                        if (app.getModel().getSelection().isDrawingSelected(drawing)) {
-                            DrawingResize.drawDrawingSelection(app, drawing);
+                        if (docModel.getSelection().isDrawingSelected(drawing)) {
+                            DrawingResize.drawDrawingSelection(docModel.getApp(), drawing);
                         }
                     }
                 }
@@ -684,7 +680,7 @@ define('io.ox/office/text/format/drawingstyles',
                     // templates, that are located inside 'div.header-footer-placeholder'.
 
                     // whether the drawing needs to be moved to the drawing layer
-                    if (DOM.isParagraphNodeComplete(drawing.parent())) { drawingLayer.shiftDrawingIntoDrawingLayer(drawing, targetRootNode); }
+                    if (DOM.isParagraphNode(drawing.parent())) { drawingLayer.shiftDrawingIntoDrawingLayer(drawing, targetRootNode); }
 
                     drawing.removeClass('inline float left right').addClass('absolute');  // absolute positioned!
 
@@ -696,7 +692,7 @@ define('io.ox/office/text/format/drawingstyles',
                     if (drawingAttributes.anchorVertAlign === 'offset') {
 
                         // checking height of top margin, not relying on 'padding-top'
-                        topMarginHeight = (drawingAttributes.anchorVertBase === 'margin') ? (app.getModel().getPageLayout().getPageAttribute('marginTop') * zoomFactor) : 0;
+                        topMarginHeight = (drawingAttributes.anchorVertBase === 'margin') ? (docModel.getPageLayout().getPageAttribute('marginTop') * zoomFactor) : 0;
                         // offset to 'margin' can be negative, but not to 'page'
                         topOffset = (drawingAttributes.anchorVertBase === 'margin') ? drawingAttributes.anchorVertOffset : Math.max(drawingAttributes.anchorVertOffset, 0);
 
@@ -706,13 +702,21 @@ define('io.ox/office/text/format/drawingstyles',
                                 useBottomOffset = true;
                                 // the offset from the bottom of the page is the height of the page minus
                                 // the specified vertical offset minus the height of the drawing.
-                                topOffset = app.getModel().getPageLayout().getPageAttribute('height') - topOffset - topMarginHeight - Utils.convertLengthToHmm(drawing.height(), 'px');
+                                topOffset = docModel.getPageLayout().getPageAttribute('height') - topOffset - topMarginHeight - Utils.convertLengthToHmm(drawing.height(), 'px');
                             } else {
                                 // in the header, the page offset can be used without modifications
                                 topOffset += topMarginHeight;
                             }
                         } else {
                             topOffset += topMarginHeight;
+                            // try to shift drawing into document by ignoring margin (like word does in task 37152)
+                            // -> unfortunately the filter is not informed about this shift, so that the drawing
+                            // will not be visible in the downloaded docx file
+                            // if (drawingAttributes.anchorVertBase === 'margin' &&
+                            //    (topOffset > docModel.getPageLayout().getPageAttribute('height')) &&
+                            //    (topOffset - topMarginHeight < docModel.getPageLayout().getPageAttribute('height'))) {
+                            //     topOffset -= topMarginHeight;
+                            // }
                         }
                         if (topOffset < 0) { topOffset = 0; }
                     } else {
@@ -803,7 +807,7 @@ define('io.ox/office/text/format/drawingstyles',
                         maxLineHeightInParagraph = getMaximumLineHeightInParagraph(paragraph);
                         topMargin += maxLineHeightInParagraph;
                         if (drawing.prev().hasClass('offset')) { // if margin is increased, decrease height of offset node for that value, so that page height doesn't get over max size
-                            drawing.prev().height(drawing.prev().height() - Utils.convertHmmToLength(maxLineHeightInParagraph, 'px', 1 ));
+                            drawing.prev().height(drawing.prev().height() - Utils.convertHmmToLength(maxLineHeightInParagraph, 'px', 1));
                         }
                     }
 
@@ -826,24 +830,29 @@ define('io.ox/office/text/format/drawingstyles',
             }
 
             // change track attribute handling
-            app.getModel().getChangeTrack().updateChangeTrackAttributes(drawing, mergedAttributes);
+            docModel.getChangeTrack().updateChangeTrackAttributes(drawing, mergedAttributes);
 
             // set generic formatting to the drawing frame
-            DrawingFrame.updateFormatting(app, drawing, mergedAttributes);
+            DrawingFrame.updateFormatting(docModel.getApp(), drawing, mergedAttributes);
 
             // update formatting of all paragraphs inside the text frame content node (36385)
             if (DrawingFrame.isTextFrameShapeDrawingFrame(drawing)) { handleChildAttributes(drawing); }
 
             // set visibility of anchor
-            if (app.getModel().getSelection().isDrawingSelected(drawing)) {
-                app.getView().setVisibleDrawingAnchor(drawing, mergedAttributes);
+            if (docModel.getSelection().isDrawingSelected(drawing)) {
+                docModel.getApp().getView().setVisibleDrawingAnchor(drawing, mergedAttributes);
             }
         }
 
         // initialization -----------------------------------------------------
 
         // register the attribute definitions for the style family
-        documentStyles.registerAttributeDefinitions('drawing', DEFINITIONS);
+        docModel.registerAttributeDefinitions('drawing', DEFINITIONS);
+
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            docModel = null;
+        });
 
     } // class TextDrawingStyles
 
