@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Oliver Specht <oliver.specht@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/text/format/listcollection', [
     'io.ox/office/tk/utils',
     'io.ox/office/baseframework/model/modelobject',
     'io.ox/office/text/utils/operations',
-    'gettext!io.ox/office/text'
+    'gettext!io.ox/office/text/main'
 ], function (Utils, ModelObject, Operations, gt) {
 
     'use strict';
@@ -35,15 +35,15 @@ define('io.ox/office/text/format/listcollection', [
 
         // the default numbered list definition with different number formats per list level (as used in MSO2010)
         DEFAULT_NUMBERING_LIST_DEFINITION = {
-            listLevel0: { numberFormat: 'decimal',     listStartValue: 1, indentLeft: 1270,     indentFirstLine: -635, textAlign: 'left',  levelText: '%1.'},
-            listLevel1: { numberFormat: 'lowerLetter', listStartValue: 1, indentLeft: 2 * 1270, indentFirstLine: -635, textAlign: 'left',  levelText: '%2.'},
-            listLevel2: { numberFormat: 'upperLetter', listStartValue: 1, indentLeft: 3 * 1270, indentFirstLine: -635, textAlign: 'right', levelText: '%3.'},
-            listLevel3: { numberFormat: 'lowerRoman',  listStartValue: 1, indentLeft: 4 * 1270, indentFirstLine: -635, textAlign: 'left',  levelText: '%4.'},
-            listLevel4: { numberFormat: 'upperRoman',  listStartValue: 1, indentLeft: 5 * 1270, indentFirstLine: -635, textAlign: 'left',  levelText: '%5.'},
-            listLevel5: { numberFormat: 'decimal',     listStartValue: 1, indentLeft: 6 * 1270, indentFirstLine: -635, textAlign: 'right', levelText: '%6.'},
-            listLevel6: { numberFormat: 'lowerLetter', listStartValue: 1, indentLeft: 7 * 1270, indentFirstLine: -635, textAlign: 'left',  levelText: '%7.'},
-            listLevel7: { numberFormat: 'upperLetter', listStartValue: 1, indentLeft: 8 * 1270, indentFirstLine: -635, textAlign: 'left',  levelText: '%8.'},
-            listLevel8: { numberFormat: 'lowerRoman',  listStartValue: 1, indentLeft: 9 * 1270, indentFirstLine: -635, textAlign: 'right', levelText: '%9.'}
+            listLevel0: { numberFormat: 'decimal',     listStartValue: 1, indentLeft: 1270,     indentFirstLine: -635, textAlign: 'left',  levelText: '%1.' },
+            listLevel1: { numberFormat: 'lowerLetter', listStartValue: 1, indentLeft: 2 * 1270, indentFirstLine: -635, textAlign: 'left',  levelText: '%2.' },
+            listLevel2: { numberFormat: 'upperLetter', listStartValue: 1, indentLeft: 3 * 1270, indentFirstLine: -635, textAlign: 'right', levelText: '%3.' },
+            listLevel3: { numberFormat: 'lowerRoman',  listStartValue: 1, indentLeft: 4 * 1270, indentFirstLine: -635, textAlign: 'left',  levelText: '%4.' },
+            listLevel4: { numberFormat: 'upperRoman',  listStartValue: 1, indentLeft: 5 * 1270, indentFirstLine: -635, textAlign: 'left',  levelText: '%5.' },
+            listLevel5: { numberFormat: 'decimal',     listStartValue: 1, indentLeft: 6 * 1270, indentFirstLine: -635, textAlign: 'right', levelText: '%6.' },
+            listLevel6: { numberFormat: 'lowerLetter', listStartValue: 1, indentLeft: 7 * 1270, indentFirstLine: -635, textAlign: 'left',  levelText: '%7.' },
+            listLevel7: { numberFormat: 'upperLetter', listStartValue: 1, indentLeft: 8 * 1270, indentFirstLine: -635, textAlign: 'left',  levelText: '%8.' },
+            listLevel8: { numberFormat: 'lowerRoman',  listStartValue: 1, indentLeft: 9 * 1270, indentFirstLine: -635, textAlign: 'right', levelText: '%9.' }
         },
 
         // all predefined list styles for the GUI controls, mapped by list identifier
@@ -280,7 +280,7 @@ define('io.ox/office/text/format/listcollection', [
 
         if (value > 0) {
             var index = 0;
-            for (;index < 7; index++) {
+            for (; index < 7; index++) {
                 while (value >= romanValArr[index]) {
                     result += romanDigitsArr[index];
                     value -= romanValArr[index];
@@ -300,7 +300,7 @@ define('io.ox/office/text/format/listcollection', [
 
     function parseRoman(text) {
         var romanSmallArr = ['m', 'd', 'c', 'l', 'x', 'v', 'i'],
-        romanValArr = [1000, 500, 100,  50,  10,   5,   1],
+        romanValArr = [1000, 500, 100,  50,  10, 5, 1],
         ret = {},
         lowerText = text.toLowerCase(),
         startValue = 0;
@@ -308,11 +308,13 @@ define('io.ox/office/text/format/listcollection', [
         var index = 0, lastValue = 1000;
         for (; index < text.length; ++index) {
             var char = lowerText.charAt(index);
-            if (char === '.')
+            if (char === '.') {
                 break;
+            }
             var position = $.inArray(char, romanSmallArr);
-            if (position < 0)
+            if (position < 0) {
                 return {};
+            }
             var value = romanValArr[position];
             if (lastValue < value) {
                 startValue = startValue - lastValue + (value - lastValue);
@@ -354,10 +356,10 @@ define('io.ox/office/text/format/listcollection', [
      *
      * @extends ModelObject
      *
-     * @param {EditApplication} app
-     *  The root application instance.
+     * @param {TextModel} docModel
+     *  The document model containing this instance.
      */
-    function ListCollection(app, documentStyles) {
+    function ListCollection(docModel) {
 
         var // list definitions
             lists = [],
@@ -365,18 +367,24 @@ define('io.ox/office/text/format/listcollection', [
             // listIds used in insertList operation
             insertedListIds = {},
 
+            // saving the base styles id for every list style
+            // -> key value pairs: listStyleId: baseStyleId
+            baseStyleIDs = {},
+
+            // saving all list styles, that share the same base style
+            // -> key value pairs: baseStyleId: [listStyleId1, listStyleId2, ...]
+            allListStyleIDs = {},
+
             // defaults
             defaultNumberingNumId,
 
             defaultBulletNumId,
 
-            characterStyles = documentStyles.getStyleCollection('character'),
-
             odfFormat = false;
 
         // base constructor ---------------------------------------------------
 
-        ModelObject.call(this, app);
+        ModelObject.call(this, docModel);
 
         // private methods ----------------------------------------------------
 
@@ -396,28 +404,29 @@ define('io.ox/office/text/format/listcollection', [
         function formatNumberType(levelIndexes, ilvl, listDefinition, listParagraphIndex) {
             var ret = {},
                 seqNo = 0,
-                end = listDefinition.listUnifiedNumbering === true ? ilvl: 0,
+                end = (listDefinition.listUnifiedNumbering === true) ? ilvl : 0,
                 topLevelformat = listDefinition.listLevels[ilvl],
                 levelText = topLevelformat.levelText ? topLevelformat.levelText : '',
                 lvl = ilvl;
             ret.labelFollowedBy = listDefinition.listLevels[ilvl].labelFollowedBy;
             ret.tabpos = listDefinition.listLevels[ilvl].tabpos;
 
-            for (;lvl >= end; --lvl) {
+            for (; lvl >= end; --lvl) {
                 var levelFormat = listDefinition.listLevels[lvl];
                 if (listDefinition.listUnifiedNumbering === true) {
                     seqNo += listParagraphIndex;
-                    if (listDefinition.listLevels[0] && listDefinition.listLevels[0].listStartValue !== undefined)
+                    if (listDefinition.listLevels[0] && listDefinition.listLevels[0].listStartValue !== undefined) {
                         seqNo += listDefinition.listLevels[0].listStartValue - 1;
-                }
-                else {
-                    seqNo = levelIndexes === undefined ? 0 :
+                    }
+                } else {
+                    seqNo = (levelIndexes === undefined) ? 0 :
                         levelIndexes[lvl] + (levelFormat && levelFormat.listStartValue !== undefined ? levelFormat.listStartValue - 1 : 0);
                 }
                 var levelToken = '%' + (lvl + 1);
                 var indexpos = levelText.indexOf(levelToken);
-                if (indexpos < 0 && levelFormat.numberFormat !== 'bullet')
+                if (indexpos < 0 && levelFormat.numberFormat !== 'bullet') {
                     continue;
+                }
                 var replacetext = '';
                 switch (levelFormat.numberFormat) {
                 case 'bullet':
@@ -426,14 +435,13 @@ define('io.ox/office/text/format/listcollection', [
                             ret.imgsrc = levelFormat.levelPicBulletUri;
                             ret.imgwidth = levelFormat.width;
                             replacetext = '';
-                        }
-                        else {
+                        } else {
                             var charCode = levelFormat.levelText ? levelFormat.levelText.charCodeAt(0) : -1;
                             if (charCode > 0 && (charCode < 0xE000 || charCode > 0xF8FF)) {
                                 replacetext = levelFormat.levelText;
-                            }
-                            else
+                            } else {
                                 replacetext = charCode === 0xf0b7 ? '\u2022' : '\u25cf';
+                            }
                         }
                     }
                     break;
@@ -448,16 +456,17 @@ define('io.ox/office/text/format/listcollection', [
                     levelText = '';
                     break;
                 }
-                else if (topLevelformat.numberFormat === 'bullet') {
+                if (topLevelformat.numberFormat === 'bullet') {
                     levelText = replacetext;
                     break;
                 }
-                else
-                    levelText = levelText.replace(levelToken, replacetext);
+                levelText = levelText.replace(levelToken, replacetext);
             }
             ret.color = topLevelformat.color;
-            if (!ret.color && topLevelformat.styleId)
-                ret.color = characterStyles.getStyleSheetAttributes(topLevelformat.styleId).character.color;
+            if (!ret.color && topLevelformat.styleId) {
+                var characterStyles = docModel.getStyleCollection('character');
+                ret.color = characterStyles.getStyleAttributeSet(topLevelformat.styleId).character.color;
+            }
             ret.text = levelText;
             return ret;
         }
@@ -479,13 +488,24 @@ define('io.ox/office/text/format/listcollection', [
          */
         this.insertList = function (operation) {
 
-            var newPosition = lists.length;
-            lists[newPosition] = {};
-            var list = lists[newPosition];
+            var list = {};
+
+            lists.push(list);
             list.listIdentifier = operation.listStyleId;
             list.listUnifiedNumbering = operation.listUnifiedNumbering;
-            //list.listStyleId = listStyleId;
             list.listLevels = [];
+
+            // handling the base style id (required for task 40792)
+            if (operation.baseStyleId) {
+                // saving base style direct in the list object
+                list.baseStyleId = operation.baseStyleId;
+                // creating key value pair from list style to base style
+                baseStyleIDs[operation.listStyleId] = operation.baseStyleId;
+                // creating list with all list styles belonging to one base style
+                if (!allListStyleIDs[operation.baseStyleId]) { allListStyleIDs[operation.baseStyleId] = []; }
+                allListStyleIDs[operation.baseStyleId].push(operation.listStyleId);
+            }
+
             if (operation.listDefinition) {
                 list.listLevels[0] = operation.listDefinition.listLevel0;
                 list.listLevels[1] = operation.listDefinition.listLevel1;
@@ -496,23 +516,42 @@ define('io.ox/office/text/format/listcollection', [
                 list.listLevels[6] = operation.listDefinition.listLevel6;
                 list.listLevels[7] = operation.listDefinition.listLevel7;
                 list.listLevels[8] = operation.listDefinition.listLevel8;
-                if (operation.listDefinition.listLevel9)
+                if (operation.listDefinition.listLevel9) {
                     list.listLevels[9] = operation.listDefinition.listLevel9;
+                }
                 if (operation.listDefinition.defaultlist) {
-                    if (operation.listDefinition.defaultlist === 'bullet')
+                    if (operation.listDefinition.defaultlist === 'bullet') {
                         defaultBulletNumId = operation.listStyleId;
-                    else
+                    } else {
                         defaultNumberingNumId = operation.listStyleId;
+                    }
                 } else {
                     if (defaultBulletNumId === undefined) {
-                        if (isDefinitionEqual(DEFAULT_BULLET_LIST_DEFINITION, operation.listDefinition) === true)
+                        if (isDefinitionEqual(DEFAULT_BULLET_LIST_DEFINITION, operation.listDefinition) === true) {
                             defaultBulletNumId = operation.listStyleId;
+                        }
                     }
                     if (defaultNumberingNumId === undefined) {
-                        if (isDefinitionEqual(DEFAULT_NUMBERING_LIST_DEFINITION, operation.listDefinition) === true)
+                        if (isDefinitionEqual(DEFAULT_NUMBERING_LIST_DEFINITION, operation.listDefinition) === true) {
                             defaultNumberingNumId = operation.listStyleId;
+                        }
                     }
                 }
+
+                //wingdings bullets, had problem in clipborad and now also in the filter!
+                //so we use mario clipboard replace utils to fix it!
+                //getUnicodeReplacementForChar() from clipboardmixin
+                //Bug 38545
+                _.each(list.listLevels, function (listDefinition, index) {
+                    var levelText = listDefinition.levelText;
+                    var replace = docModel.getUnicodeReplacementForChar(listDefinition.levelText, listDefinition.fontName);
+                    if (replace !== levelText && _.isObject(replace)) {
+                        var newDef = _.clone(listDefinition);
+                        newDef.levelText = replace.levelText;
+                        newDef.fontName = replace.fontFamily;
+                        list.listLevels[index] = newDef;
+                    }
+                });
             }
 
             // notify listeners
@@ -534,10 +573,12 @@ define('io.ox/office/text/format/listcollection', [
             for (; index < lists.length; ++index) {
                 if (lists[index].listIdentifier === listIdentifier) {
                     lists.splice(index, 1);
-                    if (defaultNumberingNumId === listIdentifier)
+                    if (defaultNumberingNumId === listIdentifier) {
                         defaultNumberingNumId = undefined;
-                    if (defaultBulletNumId === listIdentifier)
+                    }
+                    if (defaultBulletNumId === listIdentifier) {
                         defaultBulletNumId = undefined;
+                    }
                     break;
                 }
             }
@@ -724,7 +765,6 @@ define('io.ox/office/text/format/listcollection', [
                 }
                 levelCount = Math.min(htmlTypes.length, _.size(newOperation.listDefinition));
 
-
                 for (listLevel = 0; listLevel < levelCount; listLevel++) {
 
                     switch (htmlTypes[listLevel]) {
@@ -868,15 +908,19 @@ define('io.ox/office/text/format/listcollection', [
          */
         this.getListOperationFromListStyleId = function (listStyleId) {
 
-            var newOperation = { name: Operations.INSERT_LIST, listDefinition: {} },
+            var newOperation = null,
                 list = this.getList(listStyleId);
 
             if (list) {
-                newOperation.listStyleId = list.listIdentifier;
+
+                newOperation = { name: Operations.INSERT_LIST, listDefinition: {}, listStyleId: list.listIdentifier };
 
                 _(list.listLevels).each(function (levelData, id) {
                     newOperation.listDefinition['listLevel' + id] = levelData;
                 });
+
+                // also adding the base style id, if available
+                if (list.baseStyleId) { newOperation.baseStyleId = list.baseStyleId; }
             }
 
             return newOperation;
@@ -918,10 +962,11 @@ define('io.ox/office/text/format/listcollection', [
          *
          */
         this.mergeListStyle = function (existingListStyle, modifyingStyle, level) {
-            var operationAndStyle = {listOperation: null, listStyleId: null},
+
+            var operationAndStyle = { listOperation: null, listStyleId: null },
                 sourceLevel = null,
                 sourceListDefinition;
-                //newOperation = { name: Operations.INSERT_LIST, listStyleId: listId };
+
             if ((modifyingStyle in PREDEFINED_LISTSTYLES)) {
                 operationAndStyle.listOperation = { name: Operations.INSERT_LIST, listStyleId: this.getFreeListId() };
                 operationAndStyle.listOperation.listDefinition = _.copy(this.getList(existingListStyle), true);
@@ -940,9 +985,6 @@ define('io.ox/office/text/format/listcollection', [
                         operationAndStyle.listOperation = null;
                     }
                 });
-
-
-
             } else {
                 operationAndStyle.listStyleId = existingListStyle;
             }
@@ -998,7 +1040,7 @@ define('io.ox/office/text/format/listcollection', [
 
             return _.every(_.range(10), function (level) {
                 var listLevelDef = currentList.listLevels[level];
-                return ((! listLevelDef) || ((listLevelDef.numberFormat === 'bullet') || (listLevelDef.numberFormat === 'none')));
+                return (!listLevelDef || ((listLevelDef.numberFormat === 'bullet') || (listLevelDef.numberFormat === 'none')));
             });
         };
 
@@ -1110,8 +1152,9 @@ define('io.ox/office/text/format/listcollection', [
             var ilvl = 0;
             for (; ilvl < 9; ++ilvl) {
                 var levelFormat = list.listLevels[ilvl];
-                if (levelFormat.paraStyle === paraStyle)
+                if (levelFormat.paraStyle === paraStyle) {
                     return ilvl;
+                }
             }
             return -1;
         };
@@ -1127,8 +1170,9 @@ define('io.ox/office/text/format/listcollection', [
                 if (levelFormat.paraStyle === paraStyle) {
                     var ret = '';
                     if (prev) {
-                        if (ilvl > 0 && list.listLevels[ilvl - 1].paraStyle)
+                        if (ilvl > 0 && list.listLevels[ilvl - 1].paraStyle) {
                             ret = list.listLevels[ilvl - 1].paraStyle;
+                        }
                     } else if (ilvl < 8 && list.listLevels[ilvl + 1].paraStyle) {
                         ret = list.listLevels[ilvl + 1].paraStyle;
                     }
@@ -1138,10 +1182,44 @@ define('io.ox/office/text/format/listcollection', [
             return -1;
         };
 
+        /**
+         * Receiving a list of list style ids, that have the same base style id as a specified list
+         * style. If there is no base style at all, undefined is returned.
+         *
+         * @param {String} listStyleId
+         *  The list style id for that the list of all list styles with same base style id shall be returned.
+         *
+         * @returns {String[]|undefined}
+         *  Returns an array with all list style IDs, that share the same base style as the specified
+         *  list style. Or undefined, if there is no common base style at all.
+         */
+        this.getAllListStylesWithSameBaseStyle = function (listStyleId) {
+            return baseStyleIDs[listStyleId] && allListStyleIDs[baseStyleIDs[listStyleId]];
+        };
+
+        /**
+         * Receiving the base style id for a specified list style id. If there is no base style at all,
+         * undefined is returned.
+         *
+         * @param {String} listStyleId
+         *  The list style id for that the base style id shall be returned.
+         *
+         * @returns {String|undefined}
+         *  Returns the base style ID for the specified list style id. Or undefined, if there is no base
+         *  style at all.
+         */
+        this.getBaseStyleIdFromListStyle = function (listStyleId) {
+            return baseStyleIDs[listStyleId];
+        };
+
         // initialization -----------------------------------------------------
 
-        app.onImportSuccess(function () {
-            odfFormat = documentStyles.getDocumentAttributes().fileFormat === 'odf';
+        docModel.getApp().onImportSuccess(function () {
+            odfFormat = docModel.getDocumentAttributes().fileFormat === 'odf';
+        });
+
+        this.registerDestructor(function () {
+            docModel = lists = null;
         });
 
     } // class ListCollection
