@@ -6,15 +6,15 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/text/format/pagestyles',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/model/stylecollection'
-    ], function (Utils, StyleCollection) {
+define('io.ox/office/text/format/pagestyles', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/model/stylecollection'
+], function (Utils, StyleCollection) {
 
     'use strict';
 
@@ -31,6 +31,11 @@ define('io.ox/office/text/format/pagestyles',
              */
             height: { def: 29700 },
 
+            /**
+             * the actual paper format (portrait or landscape)
+             */
+            orientation: { def: 'portrait' },
+
             /**
              * Margin between left page border and editing area, in 1/100 of
              * millimeters.
@@ -157,22 +162,24 @@ define('io.ox/office/text/format/pagestyles',
      *
      * @extends StyleCollection
      *
-     * @param {TextApplication} app
-     *  The root application instance.
-     *
-     * @param {DocumentStyles} documentStyles
-     *  Collection with the style containers of all style families.
+     * @param {TextModel} docModel
+     *  The text document model containing instance.
      */
-    function PageStyles(app, documentStyles) {
+    function PageStyles(docModel) {
 
         // base constructor ---------------------------------------------------
 
-        StyleCollection.call(this, app, documentStyles, 'page', { formatHandler: updatePageFormatting });
+        StyleCollection.call(this, docModel, 'page', { formatHandler: updatePageFormatting });
 
         // initialization -----------------------------------------------------
 
         // register the attribute definitions for the style family
-        documentStyles.registerAttributeDefinitions('page', DEFINITIONS);
+        docModel.registerAttributeDefinitions('page', DEFINITIONS);
+
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            docModel = null;
+        });
 
     } // class PageStyles
 
