@@ -6,19 +6,20 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/text/format/paragraphstyles',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/utils/color',
-     'io.ox/office/editframework/utils/border',
-     'io.ox/office/editframework/utils/lineheight',
-     'io.ox/office/editframework/model/stylecollection',
-     'io.ox/office/text/dom'
-    ], function (Utils, Color, Border, LineHeight, StyleCollection, DOM) {
+define('io.ox/office/text/format/paragraphstyles', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/utils/attributeutils',
+    'io.ox/office/editframework/utils/color',
+    'io.ox/office/editframework/utils/border',
+    'io.ox/office/editframework/utils/lineheight',
+    'io.ox/office/editframework/model/stylecollection',
+    'io.ox/office/text/dom'
+], function (Utils, AttributeUtils, Color, Border, LineHeight, StyleCollection, DOM) {
 
     'use strict';
 
@@ -82,7 +83,7 @@ define('io.ox/office/text/format/paragraphstyles',
                         newTabstops = tabStops1 ? (additionalTabstops ? tabStops1.concat(additionalTabstops) : tabStops1) : (additionalTabstops ? additionalTabstops : []);
 
                     // Make entries in newTabstops unique
-                    newTabstops = _.unique(newTabstops, false, function (tabstop) {
+                    newTabstops = _.unique(newTabstops, function (tabstop) {
                         return tabstop.pos;
                     });
 
@@ -134,8 +135,9 @@ define('io.ox/office/text/format/paragraphstyles',
                         }
                         element.addClass('manual-page-break');
                     }
-                },
+                }
             },
+
             pageBreakAfter: {
                 def: false,
                 format: function (element, value) {
@@ -145,7 +147,7 @@ define('io.ox/office/text/format/paragraphstyles',
                         }
                         element.addClass('manual-pb-after');
                     }
-                },
+                }
             }
         },
 
@@ -201,7 +203,7 @@ define('io.ox/office/text/format/paragraphstyles',
      */
     function insertTabFillChar(spanNode, fillChar, width) {
 
-        var // multiplier for a better average calulation
+        var // multiplier for a better average calculation
             multiplier = 15,
             // multiplier fill characters, used to calculate average character width
             checkString = Utils.repeatString(fillChar, multiplier),
@@ -216,13 +218,13 @@ define('io.ox/office/text/format/paragraphstyles',
 
         // insert the fill string into the element
         spanNode.contents().remove().end().text(fillString);
-        if (! fillString) { DOM.ensureExistingTextNode(spanNode); }
+        if (!fillString) { DOM.ensureExistingTextNode(spanNode); }
 
         // shorten fill string by one character, if element is too wide (e.g. due to rounding errors)
         if ((fillString.length > 1) && (Utils.convertLengthToHmm(spanNode.width(), 'px') >= width)) {
             shortFillString = fillString.slice(0, -1);
             spanNode.contents().remove().end().text(shortFillString);
-            if (! shortFillString) { DOM.ensureExistingTextNode(spanNode); }
+            if (!shortFillString) { DOM.ensureExistingTextNode(spanNode); }
         }
 
         // workaround for iOS & Safari mobile - needs a text node directly within
@@ -245,22 +247,17 @@ define('io.ox/office/text/format/paragraphstyles',
      *
      * @extends StyleCollection
      *
-     * @param {TextApplication} app
-     *  The root application instance.
-     *
-     * @param {TextDocumentStyles} documentStyles
-     *  Collection with the style containers of all style families.
+     * @param {TextModel} docModel
+     *  The text document model containing instance.
      */
-    function ParagraphStyles(app, documentStyles) {
+    function ParagraphStyles(docModel) {
 
         var // self reference
-            self = this,
-            // the view instance
-            view = null;
+            self = this;
 
         // base constructor ---------------------------------------------------
 
-        StyleCollection.call(this, app, documentStyles, 'paragraph', {
+        StyleCollection.call(this, docModel, 'paragraph', {
             families: 'character changes',
             parentResolvers: PARENT_RESOLVERS,
             formatHandler: updateParagraphFormatting,
@@ -289,7 +286,7 @@ define('io.ox/office/text/format/paragraphstyles',
         function updateTabStops(paragraph, mergedAttributes) {
 
             var // default tab stop width from document settings
-                defaultTabStop = documentStyles.getDocumentAttributes().defaultTabStop,
+                defaultTabStop = docModel.getDocumentAttributes().defaultTabStop,
                 // paragraph tab stop definitions
                 paraTabStops = mergedAttributes.paragraph.tabStops,
                 // effective left margin of the paragraph
@@ -314,7 +311,7 @@ define('io.ox/office/text/format/paragraphstyles',
                 // active paragraph tab stops
                 activeParaTabStops = null,
                 // zoom factor in floating point notation
-                zoomFactor = view.getZoomFactor() / 100,
+                zoomFactor = docModel.getApp().getView().getZoomFactor() / 100,
                 // if paraTabStops has left and right oriented tabs, mixedTabs is set by the right-tab. if it is set it overwrites behaviro of text-aligment and para-margin
                 mixedTab = getMixedTab(paraTabStops);
 
@@ -325,11 +322,10 @@ define('io.ox/office/text/format/paragraphstyles',
                 topFirst = firstNode.length ? Utils.convertLengthToHmm(firstNode.position().top, 'px') : 0;
                 heightFirst = firstNode.length ? Utils.convertLengthToHmm(firstNode.height(), 'px') : 0;
 
-
                 //reset all tab sizes, for a much cleaner new calculating
                 //(for example changing text-alignment from justify to left)
                 paragraph.children(DOM.TAB_NODE_SELECTOR).each(function () {
-                   $(this).css('width', '0px');
+                    $(this).css('width', 0);
                 });
 
                 paragraph.children(DOM.TAB_NODE_SELECTOR).each(function () {
@@ -365,6 +361,16 @@ define('io.ox/office/text/format/paragraphstyles',
                         // insert index for virtual first line indent tab stop
                         indentFirstLineTabStopIndex = 0;
 
+                    var calcLeftTab = function () {
+                        return Math.max(0, tabStop.pos - (leftHMM % tabStop.pos));
+                    };
+                    var calcRightTab = function () {
+                        return calcRightAlignTabstop(tabStop, tabNode, leftHMM);
+                    };
+                    var calcMiddleTab = function () {
+                        return 0.5 * (calcLeftTab() + calcRightTab());
+                    };
+
                     if (mergedAttributes.paragraph.alignment === 'center') {
                         // for a centered paragraph tab stops are calculated where the first character is treated as leftmost (0) position
                         zeroLeft = firstNode.length ? -(marginLeft + Utils.convertLengthToHmm(firstNode.position().left, 'px')) : -marginLeft;
@@ -372,7 +378,7 @@ define('io.ox/office/text/format/paragraphstyles',
                         zeroLeft = marginLeft;
                     }
 
-                     // calculate left bound position of the tab node in 1/100th mm, including zoom factor for browsers using transf.scale
+                    // calculate left bound position of the tab node in 1/100th mm, including zoom factor for browsers using transf.scale
                     leftHMM = zeroLeft + Utils.convertLengthToHmm(pos.left / zoomFactor, 'px');
 
                     if (prevTabEndPos) {
@@ -384,8 +390,9 @@ define('io.ox/office/text/format/paragraphstyles',
                         // code.
                         if (tabNode.prev().length && tabNode.prev().text().length === 0) {
                             var checkPrevPrevNode = tabNode.prev().length ? tabNode.prev().prev() : null;
-                            if (checkPrevPrevNode.length && DOM.isTabNode(checkPrevPrevNode) && prevTabEndPos.top === topHMM)
-                                leftHMM = prevTabEndPos.right;
+                            if (checkPrevPrevNode.length && DOM.isTabNode(checkPrevPrevNode) && prevTabEndPos.top === topHMM) {
+                                leftHMM = prevTabEndPos.center || prevTabEndPos.right;
+                            }
                         }
                     }
 
@@ -413,13 +420,12 @@ define('io.ox/office/text/format/paragraphstyles',
                             }) + 1;
                             // create a copy of the paragraph tab stops and add the first indent tab stop at
                             // the correct position to the active paragraph tab stop array
-                            indentFirstLineTabStop = {value: 'left', pos: indentFirstLineTabStopValue, fillChar: NBSP, processed: false };
+                            indentFirstLineTabStop = { value: 'left', pos: indentFirstLineTabStopValue, fillChar: NBSP, processed: false };
                             activeParaTabStops = paraTabStops.slice();
                             activeParaTabStops.splice(indentFirstLineTabStopIndex, 0, indentFirstLineTabStop);
                             // reset width - the tab stop position must be calculated using all tab stop positions
                             width = 0;
-                        }
-                        else {
+                        } else {
                             // reset text within span
                             tabSpan.contents().remove();
                             tabSpan[0].appendChild(document.createTextNode(''));
@@ -437,7 +443,9 @@ define('io.ox/office/text/format/paragraphstyles',
                     // Paragraph tab stops. Only paragraph tab stop can have a fill character and
                     // define a new alignment
                     if (width <= 1) {
-                        tabStop = _(activeParaTabStops).find(function (tab) { return ((leftHMM + 10) < tab.pos) && !tab.processed; });
+                        tabStop = _(activeParaTabStops).find(function (tab) {
+                            return ((leftHMM + 10) < tab.pos) && !tab.processed;
+                        });
                         if (tabStop) {
 
                             tabStopType = tabStop.value || 'left';
@@ -445,11 +453,23 @@ define('io.ox/office/text/format/paragraphstyles',
                             // calculate tab stop size based on tab stop properties
                             if (tabStopType === 'left') {
                                 // left bound tab stop
-                                width = Math.max(0, tabStop.pos - (leftHMM % tabStop.pos));
-                            }
-                            else if (tabStopType === 'right') {
+                                width = calcLeftTab();
+                            } else if (tabStopType === 'right') {
                                 // right bound tab stop
-                                width = calcRightAlignTabstop(tabStop, tabNode, leftHMM);
+                                width = calcRightTab();
+                                // Ignore this tab stop if width is zero. Don't use the default
+                                // tab stop which is only active to left bound tab stops!
+                                ignore = (width === 0);
+
+                                if (!ignore && _.indexOf(activeParaTabStops, tabStop) === activeParaTabStops.length - 1) {
+                                    if (!_.browser.IE) {
+                                        paragraph.css('width', 'calc(100% + 5mm)');
+                                    }
+                                }
+
+                            } else if (tabStopType === 'center') {
+                                // right bound tab stop combinded with left bound tab stop
+                                width = calcMiddleTab();
                                 // Ignore this tab stop if width is zero. Don't use the default
                                 // tab stop which is only active to left bound tab stops!
                                 ignore = (width === 0);
@@ -479,13 +499,11 @@ define('io.ox/office/text/format/paragraphstyles',
                         }
                         prevTabEndPos = { right: leftHMM + width, top: topHMM };
                     }
-
                     // always set a width to the tab div (even an ignored tab div needs to set the size to zero)
-                    // small offset for FF & IE
-                    tabNode.css('width', (width / 100 - 0.01) + 'mm');
+                    tabNode.css('width', (width / 100) + 'mm');
                 });
 
-                calculateMarginForMixedTab(documentStyles, paragraph, mergedAttributes, mixedTab);
+                calculateMarginForMixedTab(docModel, paragraph, mergedAttributes, mixedTab);
 
                 // reset processed flag for tab stops again
                 paraTabStops.forEach(function (tab) {
@@ -500,14 +518,14 @@ define('io.ox/office/text/format/paragraphstyles',
          * so we calculate if right tab is outside of the margin, and if true,
          * we reduce the margin-right of the paragraph
          */
-        function calculateMarginForMixedTab(documentStyles, paragraph, paraAttributes, mixedTab) {
+        function calculateMarginForMixedTab(docModel, paragraph, paraAttributes, mixedTab) {
             paragraph.css('margin-right', null);
 
             if (!mixedTab) {
                 return;
             }
 
-            var pageAttrs = documentStyles.getDefaultAttributes('page').page;
+            var pageAttrs = docModel.getDefaultAttributes('page').page;
             var maxParaWidth = pageAttrs.width - pageAttrs.marginLeft - pageAttrs.marginRight;
             var indentRight = paraAttributes.paragraph.indentRight;
             var currentParaWidth = mixedTab.pos + indentRight;
@@ -519,13 +537,13 @@ define('io.ox/office/text/format/paragraphstyles',
         }
 
         function getMixedTab(paraTabStops) {
-            var leftTab = _.find(paraTabStops, function(tab) {
+            var leftTab = _.find(paraTabStops, function (tab) {
                 return !tab.value || tab.value === 'left';
             });
-            var rightTab = _.find(paraTabStops, function(tab) {
-                return tab.value === 'right';
+            var rightTab = _.find(paraTabStops, function (tab) {
+                return tab.value === 'center' || tab.value === 'right';
             });
-            if(leftTab && rightTab) {
+            if (leftTab && rightTab) {
                 return rightTab;
             }
         }
@@ -592,7 +610,7 @@ define('io.ox/office/text/format/paragraphstyles',
             var // the paragraph attributes of the passed attribute map
                 paragraphAttributes = mergedAttributes.paragraph,
                 // the character styles/formatter
-                characterStyles = documentStyles.getStyleCollection('character'),
+                characterStyles = docModel.getStyleCollection('character'),
 
                 leftMargin = 0,
                 rightMargin = 0,
@@ -604,13 +622,30 @@ define('io.ox/office/text/format/paragraphstyles',
                 prevAttributes = (prevParagraph.length > 0) ? self.getElementAttributes(prevParagraph) : { paragraph: {} },
 
                 nextParagraph = paragraph.next(),
-                nextAttributes = (nextParagraph.length > 0) ? self.getElementAttributes(nextParagraph) : { paragraph: {} };
+                nextAttributes = (nextParagraph.length > 0) ? self.getElementAttributes(nextParagraph) : { paragraph: {} },
+                collectedAttrs =  {},
+                prevCollectedAttrs =  {},
+                textIndent = 0,
+                explicitParaAttributes = null;
 
             function setBorder(border, position, addSpace, para) {
                 var space = addSpace + ((border && border.space) ? border.space : 0),
-                    localPara = para || paragraph;
-                localPara.css('padding-' + position, Utils.convertHmmToCssLength(space, 'px', 1));
-                localPara.css('border-' + position, self.getCssBorder(border));
+                    paddingPosition,
+                    borderPosition;
+
+                if (para) {
+                    para.css('padding-' + position, Utils.convertHmmToCssLength(space, 'px', 1));
+                    para.css('border-' + position, self.getCssBorder(border));
+
+                } else {
+                    position = position.charAt(0).toUpperCase() + position.slice(1);
+                    paddingPosition = 'padding' + position;
+                    borderPosition = 'border' + position;
+
+                    collectedAttrs[paddingPosition] = Utils.convertHmmToCssLength(space, 'px', 1);
+                    collectedAttrs[borderPosition] = self.getCssBorder(border);
+                }
+
                 return -space;
             }
 
@@ -644,16 +679,22 @@ define('io.ox/office/text/format/paragraphstyles',
                 rightMargin += rightMarginTmp;
             }
 
-
             var topMargin = paragraphAttributes.marginTop;
             var bottomMargin = paragraphAttributes.marginBottom;
 
+            // inside comments the paragraph bottom margin must be 0 (even if this was not applied via operation)
+            // and the paragraph must be left aligned
+            if (!docModel.getApp().isODF() && !docModel.getCommentLayer().isEmpty() && DOM.isNodeInsideComment(paragraph)) {
+                bottomMargin = 0;
+                collectedAttrs.textAlign = 'left';
+            }
+
             // top border is not set if previous paragraph uses the same border settings
             if (isMergeBorders(paragraphAttributes, prevAttributes.paragraph) && !isSetPageBreakAttribute(paragraphAttributes)) { // #36293
                 setBorder({ style: 'none' }, 'top', topMargin);
-                prevParagraph.css({
-                    paddingBottom: this.getCssBorder(paragraphAttributes.borderInside.space + bottomMargin),
-                    borderBottom: this.getCssBorder(paragraphAttributes.borderInside),
+                _.extend(prevCollectedAttrs, {
+                    paddingBottom: self.getCssBorder(paragraphAttributes.borderInside.space + bottomMargin),
+                    borderBottom: self.getCssBorder(paragraphAttributes.borderInside),
                     marginBottom: 0
                 });
                 topMargin = 0;
@@ -665,17 +706,18 @@ define('io.ox/office/text/format/paragraphstyles',
             if (isMergeBorders(paragraphAttributes, nextAttributes.paragraph) && !isSetPageBreakAttribute(nextAttributes.paragraph)) { // #36293
                 setBorder(paragraphAttributes.borderInside, 'bottom');
                 setBorder(paragraphAttributes.borderInside, 'top', 0, nextParagraph);  // Task 30473: Updating merged borders
-                prevParagraph.css('padding-bottom', this.getCssBorder(bottomMargin));
+                prevCollectedAttrs.paddingBottom = self.getCssBorder(bottomMargin);
                 bottomMargin = 0;
             } else {
                 setBorder(paragraphAttributes.borderBottom, 'bottom', 0);
             }
 
             //calculate list indents
+            var listLabel = $(paragraph).children(DOM.LIST_LABEL_NODE_SELECTOR);
             var listStyleId = paragraphAttributes.listStyleId;
             if (listStyleId.length) {
                 var listLevel = paragraphAttributes.listLevel,
-                     lists = documentStyles.getCustomCollection('lists');
+                    lists = docModel.getListCollection();
                 if (listLevel < 0) {
                     // is a numbering level assigned to the current paragraph style?
                     listLevel = lists.findIlvl(listStyleId, mergedAttributes.styleId);
@@ -688,41 +730,59 @@ define('io.ox/office/text/format/paragraphstyles',
                         leftPadding += listObject.indent - listObject.firstLine;
                         leftMargin += listObject.firstLine;
                     }
-                    var listLabel = $(paragraph).children(DOM.LIST_LABEL_NODE_SELECTOR);
                     if (listLabel.length) {
                         var listSpan = listLabel.children('span');
-                        if (listObject.fontSize)
+                        if (listObject.fontSize) {
                             listSpan.css('font-size', listObject.fontSize + 'pt');
-                        if (listObject.color)
+                        }
+                        if (listObject.color) {
                             listSpan.css('color', self.getCssTextColor(listObject.color, [paragraphAttributes.fillColor, listObject.fillColor]));
+                        }
                     }
+                } else {
+                    //fix for Bug 37594 some list-helper dont disappear
+                    listLabel.detach();
                 }
+            } else {
+                //fix for Bug 37594 some list-helper dont disappear
+                listLabel.detach();
             }
 
-            // paragraph margin attributes - not applied if paragraph is in a list
-            var textIndent = 0;
+            // paragraph margin attributes - also applying to paragraphs in a list, if they are defined as explicit attribute
+            // -> handle both cases correctly: 40792 and 41118
             if (listStyleId === '') {
-                leftMargin += paragraphAttributes.indentLeft;
-                rightMargin += paragraphAttributes.indentRight;
+                leftMargin += paragraphAttributes.indentLeft ? paragraphAttributes.indentLeft : 0;
+                rightMargin += paragraphAttributes.indentRight ? paragraphAttributes.indentRight : 0;
                 textIndent = paragraphAttributes.indentFirstLine ? paragraphAttributes.indentFirstLine : 0;
-                paragraph.css('text-indent', textIndent / 100 + 'mm');
+                collectedAttrs.textIndent = textIndent / 100 + 'mm';
+            } else {
+                // but explicit attributes need to be handled (40792)
+                explicitParaAttributes = AttributeUtils.getExplicitAttributes(paragraph);
+                explicitParaAttributes = (explicitParaAttributes && explicitParaAttributes.paragraph) || {};
+                if (explicitParaAttributes.indentLeft) { leftMargin += explicitParaAttributes.indentLeft; }
+                if (explicitParaAttributes.indentRight) { rightMargin += explicitParaAttributes.indentRight; }
+                if (explicitParaAttributes.indentFirstLine) {
+                    textIndent += explicitParaAttributes.indentRight;
+                    collectedAttrs.textIndent = textIndent / 100 + 'mm';
+                }
             }
 
             if (textIndent < 0) {
                 leftPadding -= textIndent;
                 leftMargin += textIndent;
             }
-            paragraph.css({
+
+            _.extend(collectedAttrs, {
                 paddingLeft: (leftPadding / 100) + 'mm',
                 // now set left/right margins
                 marginLeft: (leftMargin / 100) + 'mm',
                 marginRight: (rightMargin / 100) + 'mm',
-                'text-indent': (textIndent / 100) + 'mm'
+                textIndent: (textIndent / 100) + 'mm'
             });
 
             // Overwrite of margin left for lists in draft mode (conversion from fixed unit mm to %)
             if (Utils.SMALL_DEVICE && paragraph.data('draftRatio')) {
-                $(paragraph).css('margin-left',  (parseInt($(paragraph).css('margin-left')) * paragraph.data('draftRatio')) + '%');
+                collectedAttrs.marginLeft = (parseInt($(paragraph).css('margin-left')) * paragraph.data('draftRatio')) + '%';
             }
 
             //no distance between paragraph using the same style if contextualSpacing is set
@@ -730,33 +790,40 @@ define('io.ox/office/text/format/paragraphstyles',
                 noDistanceToNext = paragraphAttributes.contextualSpacing && (mergedAttributes.styleId === nextAttributes.styleId);
             if (noDistanceToPrev) {
                 //remove bottom margin from previous paragraph
-                prevParagraph.css('margin-bottom', 0 + 'mm');
-                paragraph.css('padding-top', 0 + 'mm');
+                prevCollectedAttrs.marginBottom = 0 + 'mm';
+                collectedAttrs.paddingTop = 0 + 'mm';
                 topMargin = 0;
             }
             if (noDistanceToNext) {
-                paragraph.css('padding-bottom', 0 + 'mm');
+                collectedAttrs.paddingBottom = 0 + 'mm';
                 bottomMargin = 0;
             }
-
-            paragraph.css({
+            _.extend(collectedAttrs, {
                 marginTop: (topMargin / 100) + 'mm',
                 marginBottom: (bottomMargin / 100) + 'mm'
             });
 
             // taking care of implicit paragraph nodes behind tables after loading the document
             // (also handling final empty paragraphs behind tables inside table cell (36537)).
-            if (DOM.isIncreasableParagraph(paragraph) && (!app.isImportFinished() || !app.getModel().selectionInNode(paragraph))) {
-                paragraph.css('height', 0);
+            if (DOM.isIncreasableParagraph(paragraph) && (!docModel.getApp().isImportFinished() || !docModel.selectionInNode(paragraph))) {
+                collectedAttrs.height = 0;
             }
 
+            if (!_.browser.IE) {
+                collectedAttrs.width = null;
+            }
+
+            // apply collected attributes at the end
+            prevParagraph.css(prevCollectedAttrs);
+            paragraph.css(collectedAttrs);
+
             // update the size of all tab stops in this paragraph (but only if the paragraph contains tabs (Performance))
             if (paragraph.find(DOM.TAB_NODE_SELECTOR).length > 0) {
                 updateTabStops(paragraph, mergedAttributes);
             }
 
             // change track attribute handling
-            app.getModel().getChangeTrack().updateChangeTrackAttributes(paragraph, mergedAttributes);
+            docModel.getChangeTrack().updateChangeTrackAttributes(paragraph, mergedAttributes);
         }
 
         /**
@@ -773,7 +840,7 @@ define('io.ox/office/text/format/paragraphstyles',
         function setParagraphPreviewFormatting(paragraph, mergedAttributes) {
 
             var // the character styles/formatter
-                characterStyles = documentStyles.getStyleCollection('character');
+                characterStyles = docModel.getStyleCollection('character');
 
             // format the text spans contained in the paragraph element
             paragraph.children().each(function () {
@@ -803,11 +870,11 @@ define('io.ox/office/text/format/paragraphstyles',
         // initialization -----------------------------------------------------
 
         // register the attribute definitions for the style family
-        documentStyles.registerAttributeDefinitions('paragraph', DEFINITIONS);
+        docModel.registerAttributeDefinitions('paragraph', DEFINITIONS);
 
-        //initialization after construction
-        app.onInit(function () {
-            view = app.getView();
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            docModel = null;
         });
 
     } // class ParagraphStyles
