@@ -6,21 +6,21 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
-define('io.ox/office/text/format/tablestyles',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/utils/color',
-     'io.ox/office/editframework/utils/border',
-     'io.ox/office/editframework/utils/lineheight',
-     'io.ox/office/editframework/utils/attributeutils',
-     'io.ox/office/editframework/model/stylecollection',
-     'io.ox/office/text/dom',
-     'io.ox/office/text/table'
-    ], function (Utils, Color, Border, LineHeight, AttributeUtils, StyleCollection, DOM, Table) {
+define('io.ox/office/text/format/tablestyles', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/utils/color',
+    'io.ox/office/editframework/utils/border',
+    'io.ox/office/editframework/utils/lineheight',
+    'io.ox/office/editframework/utils/attributeutils',
+    'io.ox/office/editframework/model/stylecollection',
+    'io.ox/office/text/dom',
+    'io.ox/office/text/table'
+], function (Utils, Color, Border, LineHeight, AttributeUtils, StyleCollection, DOM, Table) {
 
     'use strict';
 
@@ -34,13 +34,15 @@ define('io.ox/office/text/format/tablestyles',
                 def: 'auto',
                 format: function (element, width) {
                     var parentWidth = 0;
+                    // TODO: Adding support for width in percent
+                    // -> better for changes of page size or page orientation
                     if (width === 'auto') {
                         element.css('width', '100%');
                     } else {
                         // adapt size of table to its parent (36132,36163)
                         width = Utils.convertHmmToLength(width, 'px', 1);
                         parentWidth = $(element).parent().width();
-                        (parentWidth < width) ? element.css('width', '100%') : element.css('width', width + 'px');
+                        element.css('width', (parentWidth < width) ? '100%' : (width + 'px'));
                     }
                 }
             },
@@ -54,8 +56,9 @@ define('io.ox/office/text/format/tablestyles',
                     element.css('background-color', this.getCssColor(color, 'fill'));
                 },
                 preview: function (element, color) {
-                    if (!Color.isTransparentColor(color, 'fill')) {
-                        element.css('background-color', this.getCssColor(color, 'fill'));
+                    var colorDesc = this.resolveColor(Color.parseJSON(color), 'fill');
+                    if (colorDesc.a > 0) {
+                        element.css('background-color', colorDesc.css);
                     }
                 }
             },
@@ -145,20 +148,17 @@ define('io.ox/office/text/format/tablestyles',
      *
      * @extends StyleCollection
      *
-     * @param {TextApplication} app
-     *  The root application instance.
-     *
-     * @param {DocumentStyles} documentStyles
-     *  Collection with the style containers of all style families.
+     * @param {TextModel} docModel
+     *  The text document model containing instance.
      */
-    function TableStyles(app, documentStyles) {
+    function TableStyles(docModel) {
 
         var // self reference
             self = this;
 
         // base constructor ---------------------------------------------------
 
-        StyleCollection.call(this, app, documentStyles, 'table', {
+        StyleCollection.call(this, docModel, 'table', {
             families: 'cell paragraph character changes',
             styleAttributesResolver: resolveTableStyleAttributes,
             elementAttributesResolver: resolveTableElementAttributes,
@@ -270,7 +270,7 @@ define('io.ox/office/text/format/tablestyles',
                 if (_.isObject(conditionalAttributes)) {
 
                     // copy all attributes from the style sheet to the result object
-                    documentStyles.extendAttributes(attributes, conditionalAttributes);
+                    docModel.extendAttributes(attributes, conditionalAttributes);
 
                     // copy inner borders to outer cell borders, if cell is located inside the current table area
                     updateOuterCellBorder(conditionalAttributes, 'borderTop', 'borderInsideHor', !isVerticalBand || isFirstRow);
@@ -390,13 +390,13 @@ define('io.ox/office/text/format/tablestyles',
         function updateTableFormatting(table, mergedAttributes, async) {
 
             var // the table styles/formatter
-                tableStyles = documentStyles.getStyleCollection('table'),
+                tableStyles = docModel.getStyleCollection('table'),
                 // the table row styles/formatter
-                tableRowStyles = documentStyles.getStyleCollection('row'),
+                tableRowStyles = docModel.getStyleCollection('row'),
                 // the table cell styles/formatter
-                tableCellStyles = documentStyles.getStyleCollection('cell'),
+                tableCellStyles = docModel.getStyleCollection('cell'),
                 // the paragraph styles/formatter
-                paragraphStyles = documentStyles.getStyleCollection('paragraph'),
+                paragraphStyles = docModel.getStyleCollection('paragraph'),
                 // the column widths
                 tableGrid = mergedAttributes.table.tableGrid,
                 // the explicit table attributes
@@ -424,7 +424,7 @@ define('io.ox/office/text/format/tablestyles',
                 var isSimple = true,
                     styleSheetAttributeMap = tableStyles.getStyleSheetAttributeMap(mergedAttributes.styleId);
                 _.each(GEOMETRIC_TABLE_ATTRIBUTES, function (attribute) {
-                    if ((attribute in styleSheetAttributeMap) && (! _.contains(tableAttributes.exclude, attribute))) {
+                    if ((attribute in styleSheetAttributeMap) && !_.contains(tableAttributes.exclude, attribute)) {
                         isSimple = false;
                     }
                 });
@@ -524,7 +524,7 @@ define('io.ox/office/text/format/tablestyles',
 
             // Performance check for very slow devices. In this case an asynchronous update is
             // required, so that the user gets faster feed back
-            async = async || (Utils.IPAD && Utils.SLOW_DEVICE);
+            async = async || (Utils.IOS && Utils.SLOW_DEVICE);
 
             // Performance: Reducing the number of rows, because upper rows require no update
             if (_.isNumber(minUpdateRow)) {
@@ -532,17 +532,17 @@ define('io.ox/office/text/format/tablestyles',
             }
 
             // change track attribute handling
-            app.getModel().getChangeTrack().updateChangeTrackAttributes(table, mergedAttributes);
+            docModel.getChangeTrack().updateChangeTrackAttributes(table, mergedAttributes);
 
             // update rows, cells, ... , but only if required
             if (async) {
                 // process the table asynchronously
-                def = self.iterateArraySliced(rowNodes, updateRowFormatting);
+                def = self.iterateArraySliced(rowNodes, updateRowFormatting, { infoString: 'Text: Table: updateRowFormatting' });
             } else {
                 // update all rows at once (but asynchronous, so that for example a new column is visible immediately)
                 def = self.executeDelayed(function () {
                     rowNodes.each(function () { updateRowFormatting(this); });
-                });
+                }, undefined, 'Text: updateTableFormatting');
             }
 
             return def.promise();
@@ -562,7 +562,7 @@ define('io.ox/office/text/format/tablestyles',
         function updateTablePreviewFormatting(table/*, mergedAttributes*/) {
 
             var // the table cell styles/formatter
-                tableCellStyles = documentStyles.getStyleCollection('cell');
+                tableCellStyles = docModel.getStyleCollection('cell');
 
             // iterate over all cells in the table to set the cell attributes
             DOM.getTableCells(table).each(function () {
@@ -575,7 +575,12 @@ define('io.ox/office/text/format/tablestyles',
         // initialization -----------------------------------------------------
 
         // register the attribute definitions for the style family
-        documentStyles.registerAttributeDefinitions('table', DEFINITIONS);
+        docModel.registerAttributeDefinitions('table', DEFINITIONS);
+
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            docModel = self = null;
+        });
 
     } // class TableStyles
 
@@ -627,7 +632,7 @@ define('io.ox/office/text/format/tablestyles',
             borderRight:      borderRight,
             borderTop:        borderTop,
             borderBottom:     borderBottom,
-            borderInsideHor : borderInsideHor,
+            borderInsideHor:  borderInsideHor,
             borderInsideVert: borderInsideVert
         };
 
@@ -638,7 +643,7 @@ define('io.ox/office/text/format/tablestyles',
     TableStyles.getLightShadingTableStyleAttributes = function (colorValue) {
 
         var SINGLE_BORDER_LINE = { style: 'single', color: { type: 'scheme', value: colorValue }, width: 35 },
-            BAND_COLOR = { type: 'scheme', value: colorValue, transformations: [{ type: 'tint', value: 24706}] },
+            BAND_COLOR = { type: 'scheme', value: colorValue, transformations: [{ type: 'tint', value: 24706 }] },
             ROW_CELL_ATTRIBUTES = { borderLeft: Border.NONE, borderTop: SINGLE_BORDER_LINE, borderRight: Border.NONE, borderBottom: SINGLE_BORDER_LINE, borderInsideHor: Border.NONE, borderInsideVert: Border.NONE },
             BAND_CELL_ATTRIBUTES = { borderLeft: Border.NONE, borderRight: Border.NONE, borderInsideHor: Border.NONE, borderInsideVert: Border.NONE, fillColor: BAND_COLOR };
 
