@@ -6,12 +6,14 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/text/main', ['io.ox/office/text/app/application'], function (TextApplication) {
+define('io.ox/office/text/main', [
+    'io.ox/office/text/app/application'
+], function (TextApplication) {
 
     'use strict';
 
