@@ -6,22 +6,22 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/text/operationsgenerator',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/utils/attributeutils',
-     'io.ox/office/editframework/model/operationsgenerator',
-     'io.ox/office/drawinglayer/view/drawingframe',
-     'io.ox/office/text/utils/operations',
-     'io.ox/office/text/dom',
-     'io.ox/office/text/position',
-     'io.ox/office/drawinglayer/view/imageutil'
-    ], function (Utils, AttributeUtils, OperationsGenerator, DrawingFrame, Operations, DOM, Position, Image) {
+define('io.ox/office/text/operationsgenerator', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/utils/attributeutils',
+    'io.ox/office/editframework/model/operationsgenerator',
+    'io.ox/office/drawinglayer/view/drawingframe',
+    'io.ox/office/text/utils/operations',
+    'io.ox/office/text/dom',
+    'io.ox/office/text/position',
+    'io.ox/office/drawinglayer/view/imageutil'
+], function (Utils, AttributeUtils, OperationsGenerator, DrawingFrame, Operations, DOM, Position, Image) {
 
     'use strict';
 
@@ -35,20 +35,17 @@ define('io.ox/office/text/operationsgenerator',
      *
      * @extends OperationsGenerator
      *
-     * @param {TextApplication} app
-     *  The application instance containing this generator.
+     * @param {TextModel} app
+     *  The document model containing this generator.
      */
-    function TextOperationsGenerator(app) {
+    function TextOperationsGenerator(docModel) {
 
         var // self reference
-            self = this,
-
-            // the document styles (style sheet containers etc.)
-            documentStyles = app.getModel().getDocumentStyles();
+            self = this;
 
         // base constructor ---------------------------------------------------
 
-        OperationsGenerator.call(this, app);
+        OperationsGenerator.call(this, docModel);
 
         // methods ------------------------------------------------------------
 
@@ -73,6 +70,8 @@ define('io.ox/office/text/operationsgenerator',
          *  @param {String} [options.ignoreFamily]
          *      If specified, this property will be ignored in the generated
          *      operation.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
          *
          * @returns {Object}
          *  The created operation object.
@@ -82,7 +81,9 @@ define('io.ox/office/text/operationsgenerator',
             var // explicit attributes of the passed node
                 attributes = AttributeUtils.getExplicitAttributes(element),
                 // a specified property, that shall be ignored in the setAttributes operation
-                ignoreFamily = Utils.getStringOption(options, 'ignoreFamily', '');
+                ignoreFamily = Utils.getStringOption(options, 'ignoreFamily', ''),
+                // a specified property, that shall send target string with operation
+                target = Utils.getStringOption(options, 'target', '');
 
             // deleting the specified property from the attributes
             if (attributes && ignoreFamily.length > 0 && attributes[ignoreFamily]) { delete attributes[ignoreFamily]; }
@@ -93,7 +94,7 @@ define('io.ox/office/text/operationsgenerator',
             }
 
             // push the operation
-            return this.generateOperation(name, operationOptions);
+            return this.generateOperation(name, extendPropertiesWithTarget(operationOptions, target));
         };
 
         /**
@@ -121,6 +122,8 @@ define('io.ox/office/text/operationsgenerator',
          *  @param {String} [options.ignoreFamily]
          *      If specified, this property will be ignored in the generated
          *      'setAttributes' operation.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
          *
          * @returns {OperationsGenerator}
          *  A reference to this instance.
@@ -134,24 +137,26 @@ define('io.ox/office/text/operationsgenerator',
                 // the style families for generated null attributes
                 clearFamily = Utils.getStringOption(options, 'clearFamily', ''),
                 // a specified property, that shall be ignored in the setAttributes operation
-                ignoreFamily = Utils.getStringOption(options, 'ignoreFamily', '');
+                ignoreFamily = Utils.getStringOption(options, 'ignoreFamily', ''),
+                // a specified property, that shall send target string with operation
+                target = Utils.getStringOption(options, 'target', '');
 
             // deleting the specified property from the attributes
             if (elementAttributes && ignoreFamily.length > 0 && elementAttributes[ignoreFamily]) { delete elementAttributes[ignoreFamily]; }
 
             // insert null values for all attributes registered for the specified style family
             if (clearFamily.length > 0) {
-                operationOptions.attrs = documentStyles.getStyleCollection(clearFamily).buildNullAttributes();
+                operationOptions.attrs = docModel.getStyleCollection(clearFamily).buildNullAttributes();
             } else {
                 operationOptions.attrs = {};
             }
 
             // merge the explicit attributes of the passed element
-            documentStyles.extendAttributes(operationOptions.attrs, elementAttributes);
+            docModel.extendAttributes(operationOptions.attrs, elementAttributes);
 
             // no attributes, no operation
             if (!_.isEmpty(operationOptions.attrs)) {
-                this.generateOperation(Operations.SET_ATTRIBUTES, operationOptions);
+                this.generateOperation(Operations.SET_ATTRIBUTES, extendPropertiesWithTarget(operationOptions, target));
             }
 
             return this;
@@ -193,6 +198,8 @@ define('io.ox/office/text/operationsgenerator',
          *      generated for the child nodes will start at this offset. If
          *      omitted, the original node offset will be used in the logical
          *      positions.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
          *
          * @returns {TextOperationsGenerator}
          *  A reference to this instance.
@@ -209,6 +216,8 @@ define('io.ox/office/text/operationsgenerator',
                 lastTextOperation = null,
                 // a specified property, that shall be ignored in the setAttributes operation
                 ignoreFamily = Utils.getStringOption(options, 'ignoreFamily', ''),
+                // a specified property, that shall send target string with operation
+                target = Utils.getStringOption(options, 'target', ''),
                 // formatting ranges for text portions, must be applied after the contents
                 attributeRanges = [],
                 // attributes passed to the first insert operation to clear all formatting
@@ -228,7 +237,7 @@ define('io.ox/office/text/operationsgenerator',
 
             // clear all attributes of the first inserted text span
             if (Utils.getBooleanOption(options, 'clear', false)) {
-                clearAttributes = documentStyles.getStyleCollection('character').buildNullAttributes();
+                clearAttributes = docModel.getStyleCollection('character').buildNullAttributes();
             }
 
             // process all content nodes in the paragraph and create operations
@@ -244,8 +253,10 @@ define('io.ox/office/text/operationsgenerator',
                     endPosition = Position.appendNewIndex(position, endIndex),
                     // text of a portion span
                     text = null,
-                    // type of the drawing or the hard break or the field
+                    // type of the drawing or the hard break or the field or the range marker
                     type = null,
+                    // id and position of a text marker range
+                    rangeId = null, rangePos = null,
                     // the field attributes
                     attrs = {};
 
@@ -258,7 +269,9 @@ define('io.ox/office/text/operationsgenerator',
                     if (lastTextOperation) {
                         lastTextOperation.text += text;
                     } else {
-                        lastTextOperation = generateOperationWithClearAttributes(Operations.TEXT_INSERT, { start: startPosition, text: text });
+                        if (text.length > 0) {
+                            lastTextOperation = generateOperationWithClearAttributes(Operations.TEXT_INSERT, extendPropertiesWithTarget({ start: startPosition, text: text }, target));
+                        }
                     }
                     attributeRanges.push({ node: node, position: _.extend({ start: startPosition }, (text.length > 1) ? { end: endPosition } : undefined) });
 
@@ -268,52 +281,66 @@ define('io.ox/office/text/operationsgenerator',
                     lastTextOperation = null;
 
                     // operation to create a text field
-                    // TODO: field type
                     if (DOM.isFieldNode(node)) {
                         // extract text of all embedded spans representing the field
                         text = $(node).text();
                         attrs.field = {};
-                        $.each( $(node).data(),function( name, value) {
-                            if (name !== 'type')
+                        $.each($(node).data(), function (name, value) {
+                            if (name !== 'type') {
                                 attrs.field[name] = value;
-                            else
+                            } else {
                                 type = value;
-                         });
-                        self.generateOperation(Operations.FIELD_INSERT, { start: startPosition, representation: text, type: type, attrs: attrs });
+                            }
+                        });
+                        self.generateOperation(Operations.FIELD_INSERT, extendPropertiesWithTarget({ start: startPosition, representation: text, type: type, attrs: attrs }, target));
                         // attributes are contained in the embedded span elements
                         attributeRanges.push({ node: node.firstChild, position: { start: startPosition } });
-                    }
 
                     // operation to create a tabulator
-                    else if (DOM.isTabNode(node)) {
-                        generateOperationWithClearAttributes(Operations.TAB_INSERT, { start: startPosition });
+                    } else if (DOM.isTabNode(node)) {
+                        generateOperationWithClearAttributes(Operations.TAB_INSERT, extendPropertiesWithTarget({ start: startPosition }, options.target));
                         // attributes are contained in the embedded span elements
                         attributeRanges.push({ node: node.firstChild, position: { start: startPosition } });
-                    }
 
                     // operation to create a hard break
-                    else if (DOM.isHardBreakNode(node)) {
+                    } else if (DOM.isHardBreakNode(node)) {
                         // reading type of hard break from the node
                         type = $(node).data('type');
-                        generateOperationWithClearAttributes(Operations.HARDBREAK_INSERT, { start: startPosition, type: type });
+                        generateOperationWithClearAttributes(Operations.HARDBREAK_INSERT, extendPropertiesWithTarget({ start: startPosition, type: type }, target));
                         // attributes are contained in the embedded span elements
                         attributeRanges.push({ node: node.firstChild, position: { start: startPosition } });
-                    }
 
                     // operation to create a drawing (including its attributes)
-                    else if (DrawingFrame.isDrawingFrame(node) || DOM.isDrawingPlaceHolderNode(node)) {
+                    } else if (DrawingFrame.isDrawingFrame(node) || DOM.isDrawingPlaceHolderNode(node)) {
                         // switching to node in drawing layer
                         if (DOM.isDrawingPlaceHolderNode(node)) { node = DOM.getDrawingPlaceHolderNode(node); }
                         // skip drawing nodes that cannot be restored (inserting one space because of position counting inside paragraph)
                         if (DOM.isUnrestorableDrawingNode(node)) {
-                            generateOperationWithClearAttributes(Operations.TEXT_INSERT, { start: startPosition, text: ' ' });
+                            generateOperationWithClearAttributes(Operations.TEXT_INSERT, extendPropertiesWithTarget({ start: startPosition, text: ' ' }, target));
                             return;
                         }
                         // generate operations for the drawing
-                        this.generateDrawingOperations(node, startPosition, { ignoreFamily: ignoreFamily });
-                    }
+                        this.generateDrawingOperations(node, startPosition, { ignoreFamily: ignoreFamily, target: options.target });
 
-                    else {
+                    } else if (DOM.isCommentPlaceHolderNode(node)) {
+                        // switching to node in comment layer
+                        node = DOM.getCommentPlaceHolderNode(node);
+
+                        // generate operations for the drawing
+                        this.generateCommentOperations(node, startPosition, { ignoreFamily: ignoreFamily, target: options.target });
+
+                    } else if (DOM.isRangeMarkerNode(node)) {
+
+                        rangeId = DOM.getRangeMarkerId(node);
+                        rangePos = DOM.isRangeMarkerStartNode(node) ? 'start' : (DOM.isRangeMarkerEndNode(node) ? 'end' : undefined);
+                        type = DOM.getRangeMarkerType(node);
+                        self.generateOperation(Operations.RANGE_INSERT, extendPropertiesWithTarget({ start: startPosition, id: rangeId, type: type, position: rangePos }, options.target));
+
+                    } else if (DOM.isComplexFieldNode(node)) {
+
+                        self.generateOperation(Operations.COMPLEXFIELD_INSERT, extendPropertiesWithTarget({ start: startPosition, id: DOM.getComplexFieldId(node), instruction: DOM.getComplexFieldInstruction(node) }, options.target));
+
+                    } else {
                         Utils.error('TextOperationsGenerator.generateParagraphChildOperations(): unknown content node');
                         return Utils.BREAK;
                     }
@@ -331,7 +358,7 @@ define('io.ox/office/text/operationsgenerator',
             // operations would clone the attributes of the last text portion
             // instead of creating a clean text node as expected in this case.
             _(attributeRanges).each(function (range) {
-                this.generateSetAttributesOperation(range.node, range.position, { ignoreFamily: ignoreFamily });
+                this.generateSetAttributesOperation(range.node, range.position, { ignoreFamily: ignoreFamily, target: target });
             }, this);
 
             return this;
@@ -353,6 +380,8 @@ define('io.ox/office/text/operationsgenerator',
          *  Optional parameters:
          *  @param {String} [options.ignoreFamily]
          *      If specified, this property will be ignored.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
          *
          * @returns {TextOperationsGenerator}
          *  A reference to this instance.
@@ -382,6 +411,8 @@ define('io.ox/office/text/operationsgenerator',
          *  Optional parameters:
          *  @param {String} [options.ignoreFamily]
          *      If specified, this property will be ignored.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
          *
          * @returns {TextOperationsGenerator}
          *  A reference to this instance.
@@ -410,6 +441,8 @@ define('io.ox/office/text/operationsgenerator',
          *  Optional parameters:
          *  @param {String} [options.ignoreFamily]
          *      If specified, this property will be ignored.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
          *
          * @returns {TextOperationsGenerator}
          *  A reference to this instance.
@@ -444,6 +477,8 @@ define('io.ox/office/text/operationsgenerator',
          *  Optional parameters:
          *  @param {String} [options.ignoreFamily]
          *      If specified, this property will be ignored.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
          *
          * @returns {TextOperationsGenerator}
          *  A reference to this instance.
@@ -478,6 +513,8 @@ define('io.ox/office/text/operationsgenerator',
          *  Optional parameters:
          *  @param {String} [options.ignoreFamily]
          *      If specified, this property will be ignored.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
          *
          * @returns {TextOperationsGenerator}
          *  A reference to this instance.
@@ -499,13 +536,54 @@ define('io.ox/office/text/operationsgenerator',
                     this.generateDrawingGroupChildOperations(drawingNode, position, options);
                 } else if (type === 'shape' && DrawingFrame.isTextFrameShapeDrawingFrame(drawingNode)) {
                     // generate operations for the contents of the text frame
-                    this.generateContentOperations(drawingNode, position);
+                    this.generateContentOperations(drawingNode, position, options);
                 }
             }
 
             return this;
         };
 
+        /**
+         * Generates all operations needed to recreate the passed comment.
+         *
+         * @param {HTMLElement|jQuery} commentNode
+         *  The comment element whose contents will be converted to operations.
+         *  If this object is a jQuery collection, uses the first node it contains.
+         *
+         * @param {Number[]} position
+         *  The logical position of the passed drawing node. The generated
+         *  operations will contain positions starting with this address.
+         *
+         * @param {Object} [options]
+         *  Optional parameters:
+         *  @param {String} [options.ignoreFamily]
+         *      If specified, this property will be ignored.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
+         *
+         * @returns {TextOperationsGenerator}
+         *  A reference to this instance.
+         */
+        this.generateCommentOperations = function (commentNode, position, options) {
+
+            // restoring author, date and id
+            var id = DOM.getTargetContainerId(commentNode),
+                author = DOM.getCommentAuthor(commentNode),
+                uid = DOM.getCommentAuthorUid(commentNode),
+                date = DOM.getCommentDate(commentNode);
+
+            this.generateOperationWithAttributes(commentNode, Operations.COMMENT_INSERT, { start: position, id: id, author: author, uid: uid, date: date }, options);
+
+            // using the target for the created operations
+            var localOptions = options || {};
+            localOptions.target = id;
+
+            // generate operations for the contents of the text frame
+            this.generateContentOperations(commentNode, [], localOptions);
+
+            return this;
+        };
+
         /**
          * Generates a usable image operation for an internal & external URL.
          *
@@ -521,6 +599,8 @@ define('io.ox/office/text/operationsgenerator',
          *  @param {String} [options.ignoreFamily]
          *      If specified, this property will be ignored in the generated
          *      operation.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
          *
          * @returns {Object}
          *  The created operation object.
@@ -561,11 +641,13 @@ define('io.ox/office/text/operationsgenerator',
          *  Optional parameters:
          *  @param {String} [options.ignoreFamily]
          *      If specified, this property will be ignored.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
          *
          * @returns {TextOperationsGenerator}
          *  A reference to this instance.
          */
-        this.generateDrawingGroupChildOperations = function (drawingGroupNode, position , options) {
+        this.generateDrawingGroupChildOperations = function (drawingGroupNode, position, options) {
 
             // generate operations for all drawings inside the drawing group
             position = Position.appendNewIndex(position);
@@ -599,6 +681,8 @@ define('io.ox/office/text/operationsgenerator',
          *  Optional parameters:
          *  @param {String} [options.ignoreFamily]
          *      If specified, this property will be ignored.
+         *  @param {String} [options.target]
+         *      If specified, this property will be send as target with operation.
          *
          * @returns {TextOperationsGenerator}
          *  A reference to this instance.
@@ -622,6 +706,8 @@ define('io.ox/office/text/operationsgenerator',
                     if (DOM.isExceededSizeTableNode(node)) { return; }
                     // operations to create a table with its structure and contents
                     this.generateTableOperations(node, position, options);
+                } else if (DOM.isMarginalDrawingLayerNode(node)) {
+                    // skip over text drawing layer in header/footer
                 } else {
                     Utils.error('TextOperationsGenerator.generateContentOperations(): unexpected node "' + Utils.getNodeName(node) + '" at position ' + JSON.stringify(position) + '.');
                     // continue with next child node (do not increase position)
@@ -636,6 +722,25 @@ define('io.ox/office/text/operationsgenerator',
             return this;
         };
 
+        /**
+         * Utility method for extending object with target property,
+         * but only if its defined.
+         *
+         * @param{Object} object
+         *   object that is being extended
+         * @param{String[]} target
+         *   Id referencing to header or footer node
+         *
+         * @returns {Object} enhanced object
+         *
+         */
+        function extendPropertiesWithTarget (object, target) {
+            if (target) {
+                object.target = target;
+            }
+            return object;
+        }
+
     } // class TextOperationsGenerator
 
     // exports ================================================================
