@@ -6,19 +6,19 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
-define('io.ox/office/text/table',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/utils/attributeutils',
-     'io.ox/office/text/utils/config',
-     'io.ox/office/text/utils/operations',
-     'io.ox/office/text/dom',
-     'io.ox/office/text/position'
-    ], function (Utils, AttributeUtils, Config, Operations, DOM, Position) {
+define('io.ox/office/text/table', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/utils/attributeutils',
+    'io.ox/office/text/utils/config',
+    'io.ox/office/text/utils/operations',
+    'io.ox/office/text/dom',
+    'io.ox/office/text/position'
+], function (Utils, AttributeUtils, Config, Operations, DOM, Position) {
 
     'use strict';
 
@@ -48,8 +48,8 @@ define('io.ox/office/text/table',
 
         for (var i = 0; i < ranges.length; i++) {
             if (i > 0) {
-                if (! _.contains(startOffsets, ranges[i].start.offset)) { return false; }
-                if (! _.contains(endOffsets, ranges[i].end.offset)) { return false; }
+                if (!_.contains(startOffsets, ranges[i].start.offset)) { return false; }
+                if (!_.contains(endOffsets, ranges[i].end.offset)) { return false; }
             }
             startOffsets.push(ranges[i].start.offset);
             endOffsets.push(ranges[i].end.offset);
@@ -73,7 +73,7 @@ define('io.ox/office/text/table',
 
         for (var i = 0; i < ranges.length; i++) {
             if (i > 0) {
-                if (! _.contains(allRows, ranges[i].start.node)) { return false; }
+                if (!_.contains(allRows, ranges[i].start.node)) { return false; }
             }
             allRows.push(ranges[i].start.node);
         }
@@ -110,10 +110,10 @@ define('io.ox/office/text/table',
         var ranges = browserSelection.ranges,
             expand = true;
 
-        if ((ranges.length > 1) && (! isSingleColumnSelection(ranges))) {
+        if ((ranges.length > 1) && !isSingleColumnSelection(ranges)) {
 
             if (active.start.offset === ranges[0].start.offset) {
-                if (! backwards) {
+                if (!backwards) {
                     expand = false;
                 }
             } else if (active.end.offset ===  ranges[ranges.length - 1].end.offset) {
@@ -155,9 +155,9 @@ define('io.ox/office/text/table',
         var ranges = browserSelection.ranges,
             expand = true;
 
-        if ((ranges.length > 1) && (! isSingleRowSelection(ranges))) {
+        if ((ranges.length > 1) && !isSingleRowSelection(ranges)) {
             if (active.start.node === ranges[0].start.node) {
-                if (! backwards) {
+                if (!backwards) {
                     expand = false;
                 }
             } else if (active.end.node === ranges[ranges.length - 1].end.node) {
@@ -275,7 +275,7 @@ define('io.ox/office/text/table',
             var // deep copy of complete attribute map
                 attributes = _.copy($(cell).data('attributes'), true);
 
-            if ( _.isObject(attributes) && ('cell' in attributes)) {
+            if (_.isObject(attributes) && ('cell' in attributes)) {
                 _.each(attributes.cell, function (attribute, name) {
                     // we are interested only in border attributes, kick out others
                     if (_.indexOf(['borderBottom', 'borderLeft', 'borderRight', 'borderTop'], name) === -1) {
@@ -405,7 +405,7 @@ define('io.ox/office/text/table',
         var tableGrid = _.copy(Table.getTableGrid(startnode, tablePos), true),
             tableWidth = 0;
 
-        if (! tableGrid) {
+        if (!tableGrid) {
             Utils.error('Table.getTableGridWithNewColumn(): Unable to get existing table grid');
             return;
         }
@@ -569,7 +569,7 @@ define('io.ox/office/text/table',
         // with the specified grid position.
         // In insertColumn, cells shall always be added.
 
-        if ((! foundCell) && (! defaultToLastCell)) {
+        if (!foundCell && !defaultToLastCell) {
             cellPosition = -1;
         }
 
@@ -1016,14 +1016,13 @@ define('io.ox/office/text/table',
                             endPoint = new DOM.Point(siblingRow[0], allRangeOffsets[i][1]);
                             ranges.unshift(new DOM.Range(startPoint, endPoint));
                         }
-                    // } else {
-                    //     // Adding content before the table!
                     }
+                    // else: Adding content before the table!
                 }
             } else {
 
                 for (var i = 0; i < ranges.length; i++) {
-                    if (! (DOM.Range.equalRangeOffsets(activeCellRange, ranges[i]))) {
+                    if (!DOM.Range.equalRangeOffsets(activeCellRange, ranges[i])) {
                         newRanges.push(ranges[i]);
                     }
                 }
@@ -1061,15 +1060,14 @@ define('io.ox/office/text/table',
                             endPoint = new DOM.Point(siblingRow, allRangeOffsets[i][1]);
                             ranges.push(new DOM.Range(startPoint, endPoint));
                         }
-                    // } else {
-                    //     // Adding content behind the table
                     }
+                    // else: Adding content behind the table
                 }
 
             } else {
 
                 for (var i = 0; i < ranges.length; i++) {
-                    if (! (DOM.Range.equalRangeOffsets(activeCellRange, ranges[i]))) {
+                    if (!DOM.Range.equalRangeOffsets(activeCellRange, ranges[i])) {
                         newRanges.push(ranges[i]);
                     }
                 }
@@ -1211,12 +1209,12 @@ define('io.ox/office/text/table',
 
                 if (positionsInSameTable) {
 
-                    multiCellSelection = ! Position.hasSameParentComponent(start, end, 2);
+                    multiCellSelection = !Position.hasSameParentComponent(start, end, 2);
 
                     if (multiCellSelection) {
                         // only selections from beginning of cell to beginning of cell are allowed
                         returnObj = Position.getLastPositionInPrevCell(startnode, _.clone(start));
-                        if (! returnObj.beginOfTable) {
+                        if (!returnObj.beginOfTable) {
                             selection.setTextSelection(returnObj.position, end);
                             preventDefault = true;
                         } else {
@@ -1233,7 +1231,7 @@ define('io.ox/office/text/table',
                         // creating a cell selection, if the begin of the cell is reached
                         if (Position.isFirstPositionInTableCell(startnode, start)) {
                             returnObj = Position.getLastPositionInPrevCell(startnode, _.clone(start));
-                            if (! returnObj.beginOfTable) {
+                            if (!returnObj.beginOfTable) {
                                 selection.setTextSelection(returnObj.position, end, { backwards: true });
                                 preventDefault = true;
                             } else {
@@ -1247,7 +1245,7 @@ define('io.ox/office/text/table',
                 } else if (Position.isPositionInTable(startnode, start)) {
                     // only selections from beginning of cell to beginning of cell are allowed
                     returnObj = Position.getLastPositionInPrevCell(startnode, _.clone(start));
-                    if (! returnObj.beginOfTable) {
+                    if (!returnObj.beginOfTable) {
                         selection.setTextSelection(returnObj.position, end);
                         preventDefault = true;
                     } else {
@@ -1272,22 +1270,17 @@ define('io.ox/office/text/table',
 
         } else {
 
-            if (backwardSelection) {
-
-                // decreasing the selection with shift - cursor right
-
-
-            } else {
+            if (!backwardSelection) {
                 // increasing the selection with shift - cursor right
 
                 if (positionsInSameTable) {
 
-                    multiCellSelection = ! Position.hasSameParentComponent(start, end, 2);
+                    multiCellSelection = !Position.hasSameParentComponent(start, end, 2);
 
                     if (multiCellSelection) {
                         // only selections from beginning of cell to beginning of cell are allowed
                         returnObj = Position.getFirstPositionInNextCell(startnode, _.clone(end));
-                        if (! returnObj.endOfTable) {
+                        if (!returnObj.endOfTable) {
                             selection.setTextSelection(start, returnObj.position);
                             preventDefault = true;
                         } else {
@@ -1305,7 +1298,7 @@ define('io.ox/office/text/table',
                         if (Position.isLastPositionInTableCell(startnode, end)) {
                             start = Position.getFirstPositionInCurrentCell(startnode, start);
                             returnObj = Position.getFirstPositionInNextCell(startnode, start);
-                            if (! returnObj.endOfTable) {
+                            if (!returnObj.endOfTable) {
                                 selection.setTextSelection(start, returnObj.position);
                                 preventDefault = true;
                             } else {
@@ -1318,7 +1311,7 @@ define('io.ox/office/text/table',
                 } else if (Position.isPositionInTable(startnode, end)) {
                     // only selections from beginning of cell to beginning of cell are allowed
                     returnObj = Position.getFirstPositionInNextCell(startnode, _.clone(end));
-                    if (! returnObj.endOfTable) {
+                    if (!returnObj.endOfTable) {
                         selection.setTextSelection(start, returnObj.position);
                         preventDefault = true;
                     } else {
@@ -1559,30 +1552,27 @@ define('io.ox/office/text/table',
      * if a new table style is assigned to a table. Only if all 'hard' attributes are
      * removed, the style will be visible.
      *
+     * @param {TextModel} docModel
+     *  The text model instance.
+     *
+     * @param {Node} rootNode
+     *  The root node corresponding to the logical position.
+     *
      * @param {HTMLTableElement|jQuery} tableNode
      * The table node element, whose attributes shall be deleted.
      *
      * @param {Number[]} position
      *  The logical position of the table.
      *
-     * @param {DocumentStyles} documentStyles
-     *  The document style sheet container.
-     *
-     * @param {Node} rootNode
-     *  The root node corresponding to the logical position.
-     *
-     * @param {Generator} generator
+     * @param {OperationsGenerator} generator
      *  The operations generator
-     *
-     * @param {TextModel} model
-     *  The text model instance.
      */
-    Table.removeTableAttributes = function (tableNode, position, documentStyles, rootNode, generator, model) {
+    Table.removeTableAttributes = function (docModel, rootNode, tableNode, position, generator) {
 
         var // the options for the table operation
             tableOperationOptions = {},
             // remove all explicit table attributes
-            tableAttributes = documentStyles.buildNullAttributes('table');
+            tableAttributes = docModel.buildNullAttributes('table');
 
         // exclude some table attributes
         delete tableAttributes.table.width;
@@ -1605,7 +1595,7 @@ define('io.ox/office/text/table',
             var // the options for the cell operation
                 cellOperationOptions = {},
                 // remove all explicit cell attributes
-                cellAttributes = documentStyles.buildNullAttributes('cell'),
+                cellAttributes = docModel.buildNullAttributes('cell'),
                 // the old element attributes
                 oldAttrs = null;
 
@@ -1621,13 +1611,13 @@ define('io.ox/office/text/table',
             cellOperationOptions.attrs = cellAttributes;
 
             // Defining the oldAttrs, if change tracking is active
-            if (model.getChangeTrack().isActiveChangeTracking()) {
+            if (docModel.getChangeTrack().isActiveChangeTracking()) {
                 // Expanding operation for change tracking with old explicit attributes
-                oldAttrs = model.getChangeTrack().getOldNodeAttributes(this);
+                oldAttrs = docModel.getChangeTrack().getOldNodeAttributes(this);
                 // adding the old attributes for change tracking
                 if (oldAttrs) {
-                    oldAttrs = _.extend(oldAttrs, model.getChangeTrack().getChangeTrackInfo());
-                    cellOperationOptions.attrs.changes = { 'modified': oldAttrs };
+                    oldAttrs = _.extend(oldAttrs, docModel.getChangeTrack().getChangeTrackInfo());
+                    cellOperationOptions.attrs.changes = { modified: oldAttrs };
                 }
             }
 
@@ -1693,7 +1683,7 @@ define('io.ox/office/text/table',
                     // adding the old attributes for change tracking
                     if (oldAttrs) {
                         oldAttrs = _.extend(oldAttrs, model.getChangeTrack().getChangeTrackInfo());
-                        operationOptions.attrs.changes = { 'modified': oldAttrs };
+                        operationOptions.attrs.changes = { modified: oldAttrs };
                     }
                 }
 
@@ -1783,7 +1773,7 @@ define('io.ox/office/text/table',
                     // adding the old attributes for change tracking
                     if (oldAttrs) {
                         oldAttrs = _.extend(oldAttrs, model.getChangeTrack().getChangeTrackInfo());
-                        operationOptions.attrs.changes = { 'modified': oldAttrs };
+                        operationOptions.attrs.changes = { modified: oldAttrs };
                     }
                 }
 
