@@ -6,18 +6,19 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
-define('io.ox/office/text/tableResize',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/text/utils/operations',
-     'io.ox/office/text/dom',
-     'io.ox/office/text/position',
-     'io.ox/office/text/table'
-    ], function (Utils, Operations, DOM, Position, Table) {
+define('io.ox/office/text/tableResize', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/utils/tracking',
+    'io.ox/office/text/utils/operations',
+    'io.ox/office/text/dom',
+    'io.ox/office/text/position',
+    'io.ox/office/text/table'
+], function (Utils, Tracking, Operations, DOM, Position, Table) {
 
     'use strict';
 
@@ -122,8 +123,7 @@ define('io.ox/office/text/tableResize',
                 // the start scroll position
                 startScrollX = 0, startScrollY = 0,
                 //maximum height of page content in pixels (without margins)
-                pageMaxHeight = editor.getPageLayout().getDefPageActiveHeight({convertToPixel : true}) - 5; // -5 px because of difference in rounding and offsetHeight, position top in IE, FF
-
+                pageMaxHeight = editor.getPageLayout().getDefPageActiveHeight({ convertToPixel: true }) - 5; // -5 px because of difference in rounding and offsetHeight, position top in IE, FF
 
             /**
              * Calculates the required data, when mouse down happens on resize node.
@@ -136,7 +136,7 @@ define('io.ox/office/text/tableResize',
                 startX = event.pageX;
                 startY = event.pageY;
 
-                if ((! startX) && (event.originalEvent.pageX) && (event.originalEvent.pageY)) {
+                if (!startX && event.originalEvent.pageX && event.originalEvent.pageY) {
                     startX = event.originalEvent.pageX;
                     startY = event.originalEvent.pageY;
                 }
@@ -147,16 +147,16 @@ define('io.ox/office/text/tableResize',
                 startX = startX - leftDistance;
                 startY = startY - topDistance;
 
-                if ($(resizeNode).is('div.resize.right')) {
+                if (resizeNode.is('div.resize.right')) {
                     verticalResize = true;
-                } else if ($(resizeNode).is('div.resize.bottom')) {
+                } else if (resizeNode.is('div.resize.bottom')) {
                     horizontalResize = true;
                 }
 
                 // calculating maximum resize values
-                cellNode = $(resizeNode).closest('td, th');
-                rowNode =  $(resizeNode).closest('tr');
-                tableNode = $(resizeNode).closest('table');
+                cellNode = resizeNode.closest('td, th');
+                rowNode =  resizeNode.closest('tr');
+                tableNode = resizeNode.closest('table');
 
                 if (verticalResize) {
                     $(resizeLine).css({ width: '1px', height: '100%', left: startX, top: '0px' });
@@ -164,14 +164,17 @@ define('io.ox/office/text/tableResize',
 
                     // calculating maxLeftShift and maxRightShift
                     lastCell = cellNode[0].nextSibling ? false : true;
-                    tablePosition = Position.getOxoPosition(editor.getNode(), tableNode.get(0), 0);
+                    tablePosition = Position.getOxoPosition(editor.getCurrentRootNode(), tableNode.get(0), 0);
                     tableNodeAttrs = editor.getStyleCollection('table').getElementAttributes(tableNode).table;
                     oldTableGrid = tableNodeAttrs.tableGrid;
                     oldTableWidth = tableNodeAttrs.width;
                     maxTableWidth = tableNode.parent().width();
 
-                    if (oldTableWidth === 'auto') { oldTableWidth = tableNode.outerWidth(); }
-                    else { oldTableWidth = Utils.convertHmmToLength(oldTableWidth, 'px', 1); }
+                    if (oldTableWidth === 'auto') {
+                        oldTableWidth = tableNode.outerWidth();
+                    } else {
+                        oldTableWidth = Utils.convertHmmToLength(oldTableWidth, 'px', 1);
+                    }
 
                     // converting from relational grid to pixel grid
                     for (var i = 0; i < oldTableGrid.length; i++) { gridSum += oldTableGrid[i]; }
@@ -184,11 +187,14 @@ define('io.ox/office/text/tableResize',
                     if (cellNode.find(DOM.LIST_LABEL_NODE_SELECTOR).length > 0) { minColumnWidth *= 4; }
 
                     maxLeftShift = pixelGrid[shiftedGrid] * zoomFactor;
-                    if (! lastCell) { maxRightShift = pixelGrid[shiftedGrid + 1] * zoomFactor; }
-                    else { maxRightShift = (maxTableWidth - oldTableWidth) * zoomFactor; }
+                    if (!lastCell) {
+                        maxRightShift = pixelGrid[shiftedGrid + 1] * zoomFactor;
+                    } else {
+                        maxRightShift = (maxTableWidth - oldTableWidth) * zoomFactor;
+                    }
 
                 } else if (horizontalResize) {
-                    $(resizeLine).css({ width: '100%', height: '1px', left: '0px', top: startY});
+                    $(resizeLine).css({ width: '100%', height: '1px', left: 0, top: startY });
                     officemaindiv.append(resizeLine);
                     // calculating maxTopShift
                     maxTopShift = cellNode.outerHeight() * zoomFactor;
@@ -196,8 +202,8 @@ define('io.ox/office/text/tableResize',
                     maxBottomShift = (pageMaxHeight - cellNode.outerHeight()) * zoomFactor;
                 }
 
-                editor.getNode().css('cursor', $(resizeNode).css('cursor'));  // setting cursor for increasing drawing
-                $(resizeLine).css('cursor', $(resizeNode).css('cursor'));  // setting cursor for increasing drawing
+                editor.getCurrentRootNode().css('cursor', resizeNode.css('cursor'));  // setting cursor for increasing drawing
+                $(resizeLine).css('cursor', resizeNode.css('cursor'));  // setting cursor for increasing drawing
 
                 startScrollX = scrollNode.scrollLeft();
                 startScrollY = scrollNode.scrollTop();
@@ -215,7 +221,7 @@ define('io.ox/office/text/tableResize',
                 currentX = event.pageX;
                 currentY = event.pageY;
 
-                if ((! currentX) && (event.originalEvent.pageX)) {
+                if (!currentX && event.originalEvent.pageX) {
                     currentX = event.originalEvent.pageX;
                     currentY = event.originalEvent.pageY;
                 }
@@ -230,7 +236,7 @@ define('io.ox/office/text/tableResize',
 
                     maxRightValue = startX + maxRightShift;
                     minLeftValue = startX - maxLeftShift + (minColumnWidth * zoomFactor);
-                    if (! lastCell) { maxRightValue -= (minColumnWidth * zoomFactor); }
+                    if (!lastCell) { maxRightValue -= (minColumnWidth * zoomFactor); }
 
                     shiftX = currentX;
                     shiftY = 0;
@@ -245,7 +251,7 @@ define('io.ox/office/text/tableResize',
                     shiftX = 0;
                     shiftY = currentY;
 
-                    if ((shiftY - startY) <= - maxTopShift) {
+                    if ((shiftY - startY) <= -maxTopShift) {
                         shiftY = startY - maxTopShift;
                     }
                     if ((shiftY - startY) >= maxBottomShift) {
@@ -254,7 +260,7 @@ define('io.ox/office/text/tableResize',
                 }
 
                 if ((_.isNumber(shiftX)) && (_.isNumber(shiftY))) {
-                    $(resizeLine).css({'left': shiftX - scrollX, 'top': shiftY - scrollY});
+                    $(resizeLine).css({ left: shiftX - scrollX, top: shiftY - scrollY });
                 }
             }
 
@@ -279,17 +285,12 @@ define('io.ox/office/text/tableResize',
                 scrollY = scrollNode.scrollTop() - startScrollY;
             }
 
-
             /**
              * Calculates the required data, when mouse up happens and generates
              * operations.
              *
              * @param {Event} event
              *  The event object.
-             *
-             * @param {HTMLElement} resizeNode
-             *  The resize node, that will be displayed temporarely
-             *
              */
             function stopResizeTable(event) {
 
@@ -300,7 +301,9 @@ define('io.ox/office/text/tableResize',
                     // the old attributes
                     oldAttrs = null,
                     // the attributes for the setAttributes operation
-                    attrs = null;
+                    attrs = null,
+                    target = editor.getActiveTarget(),
+                    operationProperties = {};
 
                 // taking care of scroll node
                 scrollX = scrollNode.scrollLeft() - startScrollX;
@@ -310,7 +313,7 @@ define('io.ox/office/text/tableResize',
                 currentX = event.pageX;
                 currentY = event.pageY;
 
-                if ((! currentX) && (event.originalEvent.changedTouches[0].pageX)) {
+                if (!currentX && event.originalEvent.changedTouches[0].pageX) {
                     currentX = event.originalEvent.changedTouches[0].pageX;
                     currentY = event.originalEvent.changedTouches[0].pageY;
                 }
@@ -327,7 +330,7 @@ define('io.ox/office/text/tableResize',
 
                         maxRightValue = startX + maxRightShift;
                         minLeftValue = startX - maxLeftShift + (minColumnWidth * zoomFactor);
-                        if (! lastCell) { maxRightValue -= (minColumnWidth * zoomFactor); }
+                        if (!lastCell) { maxRightValue -= (minColumnWidth * zoomFactor); }
 
                         if (currentX >= maxRightValue) {
                             currentX = maxRightValue;
@@ -341,14 +344,14 @@ define('io.ox/office/text/tableResize',
 
                         // -> shifting the border
                         pixelGrid[shiftedGrid] += shiftX;
-                        if (! lastCell) { pixelGrid[shiftedGrid + 1] -= shiftX; }
+                        if (!lastCell) { pixelGrid[shiftedGrid + 1] -= shiftX; }
 
                         // converting modified pixel grid to new relation table grid
                         for (var i = 0; i < pixelGrid.length; i++) {
                             tableGrid.push(Math.round(gridSum * pixelGrid[i] / newTableWidth));  // only ints
                         }
 
-                        if ((! lastCell) && (editor.getStyleCollection('table').getElementAttributes(tableNode).table.width === 'auto')) {
+                        if (!lastCell && (editor.getStyleCollection('table').getElementAttributes(tableNode).table.width === 'auto')) {
                             newTableWidth = 'auto';
                         } else {
                             newTableWidth = Utils.convertLengthToHmm(newTableWidth, 'px');
@@ -363,20 +366,24 @@ define('io.ox/office/text/tableResize',
                             // adding the old attributes, author and date for change tracking
                             if (oldAttrs) {
                                 oldAttrs = _.extend(oldAttrs, editor.getChangeTrack().getChangeTrackInfo());
-                                attrs.changes = { 'modified': oldAttrs };
+                                attrs.changes = { modified: oldAttrs };
                             }
                         }
-
-                        generator.generateOperation(Operations.SET_ATTRIBUTES, {
+                        operationProperties = {
                             attrs: attrs,
                             start: tablePosition
-                        });
+                        };
+                        if (target) {
+                            operationProperties.target = target;
+                        }
+
+                        generator.generateOperation(Operations.SET_ATTRIBUTES, operationProperties);
 
                         // Performance: Mark the table with information about vertical resize, so that no full update of cell
                         // attributes is required, but only a recalculation of tab stops.
                         tableNode.data('tableResize', true);
 
-                        editor.applyOperations(generator.getOperations());
+                        editor.applyOperations(generator);
 
                         tableNode.removeData('tableResize');
                     }
@@ -391,7 +398,7 @@ define('io.ox/office/text/tableResize',
                         if (rowHeight > pageMaxHeight) { rowHeight = pageMaxHeight; }
 
                         newRowHeight = Utils.convertLengthToHmm(rowHeight, 'px');
-                        rowPosition = Position.getOxoPosition(editor.getNode(), rowNode.get(0), 0);
+                        rowPosition = Position.getOxoPosition(editor.getCurrentRootNode(), rowNode.get(0), 0);
 
                         // the row attributes for the setAttributes operation
                         attrs = { row: { height: newRowHeight } };
@@ -402,16 +409,21 @@ define('io.ox/office/text/tableResize',
                             // adding the old attributes, author and date for change tracking
                             if (oldAttrs) {
                                 oldAttrs = _.extend(oldAttrs, editor.getChangeTrack().getChangeTrackInfo());
-                                attrs.changes = { 'modified': oldAttrs };
+                                attrs.changes = { modified: oldAttrs };
                             }
                         }
 
-                        generator.generateOperation(Operations.SET_ATTRIBUTES, {
+                        operationProperties = {
                             attrs: attrs,
                             start: rowPosition
-                        });
+                        };
+                        if (target) {
+                            operationProperties.target = target;
+                        }
 
-                        editor.applyOperations(generator.getOperations());
+                        generator.generateOperation(Operations.SET_ATTRIBUTES, operationProperties);
+
+                        editor.applyOperations(generator);
                     }
                 }
 
@@ -422,23 +434,22 @@ define('io.ox/office/text/tableResize',
              */
             function finalizeResizeTable() {
                 // leaveTracking();
-                $(resizeNode).off(ALL_TRACKING_EVENT_NAMES);
+                resizeNode.off(ALL_TRACKING_EVENT_NAMES);
                 view.scrollToChildNode(resizeNode);
 
                 // removing the resize line
                 officemaindiv.children('div.resizeline').remove();
 
                 // Resetting cursor, using css file again
-                editor.getNode().css('cursor', '');
+                editor.getCurrentRootNode().css('cursor', '');
 
                 // Resetting variables
                 shiftX = shiftY = scrollX = scrollY = 0;
 
                 // removing class
-                $(resizeNode).removeClass('resizeActive');
+                resizeNode.removeClass('resizeActive');
             }
 
-
             // return the actual drawingMoveHandler() method
             return function (event) {
 
@@ -472,26 +483,27 @@ define('io.ox/office/text/tableResize',
          *  The 'tracking:start' event, that starts the resizing of the table.
          */
         function trackingStartHandler(event) {
-            $(resizeNode).off(TRACKING_EVENT_NAMES);
-            $(resizeNode).on(TRACKING_EVENT_NAMES, tableResizeHandler);
+            resizeNode.off(TRACKING_EVENT_NAMES);
+            resizeNode.on(TRACKING_EVENT_NAMES, tableResizeHandler);
 
             tableResizeHandler.call(this, event);
         }
 
-        // mark this resize node, that it is already prepared for tracking
-        $(resizeNode).addClass('resizeActive');
+        // ensure to work on a jQuery collection
+        resizeNode = $(resizeNode);
 
-        resizeNode
-        .enableTracking({
+        // mark this resize node, that it is already prepared for tracking
+        resizeNode.addClass('resizeActive');
+
+        Tracking.enableTracking(resizeNode, {
             autoScroll: true,
             borderNode: scrollNode,
             borderMargin: -30,
             borderSize: 60,
             minSpeed: 10,
             maxSpeed: 250
-        })
-        .on('tracking:start', trackingStartHandler);
-
+        });
+        resizeNode.on('tracking:start', trackingStartHandler);
     };
 
     // exports ================================================================
