@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
@@ -35,8 +35,11 @@ define('io.ox/office/text/utils/operations', [
         TEXT_INSERT: 'insertText',
         DRAWING_INSERT: 'insertDrawing',
         FIELD_INSERT: 'insertField',
+        COMPLEXFIELD_INSERT: 'insertComplexField',
         TAB_INSERT: 'insertTab',
         HARDBREAK_INSERT: 'insertHardBreak',
+        COMMENT_INSERT: 'insertComment',
+        RANGE_INSERT: 'insertRange',
 
         PARA_INSERT: 'insertParagraph',
         PARA_SPLIT: 'splitParagraph',
@@ -54,7 +57,8 @@ define('io.ox/office/text/utils/operations', [
 
         INSERT_LIST: 'insertListStyle',
         DELETE_LIST: 'deleteListStyle',
-        INSERT_HEADER_FOOTER: 'insertHeaderFooter'
+        INSERT_HEADER_FOOTER: 'insertHeaderFooter',
+        DELETE_HEADER_FOOTER: 'deleteHeaderFooter'
 
     });
 
