@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,7 +14,7 @@
 define('io.ox/office/text/view/control/drawingpositionpicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/control/radiolist',
-    'gettext!io.ox/office/text'
+    'gettext!io.ox/office/text/main'
 ], function (Utils, RadioList, gt) {
 
     'use strict';
