@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -29,10 +29,16 @@ define('io.ox/office/text/view/control/liststylepicker', [
      *
      * @extends RadioList
      */
-    function ListStylePicker(app, predefinedListStyles, initOptions) {
+    function ListStylePicker(docView, predefinedListStyles, initOptions) {
 
         var // self reference
-            self = this;
+            self = this,
+
+            // CSS font family of the 'Cambria' font (with fall-back fonts)
+            fontFamily = docView.getDocModel().getCssFontFamily('Cambria'),
+
+            // font attributes for the drop-down list entries
+            labelStyle = 'font-family:' + fontFamily + ';font-size:18px;min-width:18px;';
 
         // base constructor ---------------------------------------------------
 
@@ -46,38 +52,34 @@ define('io.ox/office/text/view/control/liststylepicker', [
 
         // initialization -----------------------------------------------------
 
-        // initialize class members
-        app.onInit(function () {
-
-            var // CSS font family of the 'Cambria' font (with fall-back fonts)
-                fontFamily = app.getModel().getFontCollection().getCssFontFamily('Cambria'),
-                // font attributes for the drop-down list entries
-                labelStyle = 'font-family:' + fontFamily + ';font-size:18px;min-width:18px;';
-
-            // create the list entries
-            self.createMenuSection('default').createMenuSection('special', { gridColumns: 1 });
-            _.each(predefinedListStyles, function (listStyle, listStyleId) {
-                if (Utils.trimAndCleanString(listStyle.listLabel).length > 0) {
-                    self.createOptionButton(listStyleId, {
-                        section: 'default',
-                        label: _.noI18n(listStyle.listLabel),
-                        labelStyle: labelStyle,
-                        tooltip: _.noI18n(listStyle.tooltip),
-                        dataValue: listStyle.listKey,
-                        sortIndex: listStyleId
-                    });
-                } else {
-                    self.createOptionButton(listStyleId, {
-                        section: 'special',
-                        label: _.noI18n(listStyle.tooltip),
-                        dataValue: listStyle.listKey
-                    });
-                }
-            });
+        // create the list entries
+        self.createMenuSection('default').createMenuSection('special', { gridColumns: 1 });
+        _.each(predefinedListStyles, function (listStyle, listStyleId) {
+            if (Utils.trimAndCleanString(listStyle.listLabel).length > 0) {
+                self.createOptionButton(listStyleId, {
+                    section: 'default',
+                    label: _.noI18n(listStyle.listLabel),
+                    labelStyle: labelStyle,
+                    tooltip: _.noI18n(listStyle.tooltip),
+                    dataValue: listStyle.listKey,
+                    sortIndex: listStyleId
+                });
+            } else {
+                self.createOptionButton(listStyleId, {
+                    section: 'special',
+                    label: _.noI18n(listStyle.tooltip),
+                    dataValue: listStyle.listKey
+                });
+            }
         });
 
         this.getMenuNode().addClass('list-style-picker');
 
+        // destroy class members
+        this.registerDestructor(function () {
+            self = docView = predefinedListStyles = initOptions = null;
+        });
+
     } // class ListStylePicker
 
     // exports ================================================================
