@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/text/view/control/tablestylepicker', [
     'io.ox/office/tk/utils',
     'io.ox/office/editframework/view/control/stylesheetpicker',
     'io.ox/office/text/dom',
-    'gettext!io.ox/office/text'
+    'gettext!io.ox/office/text/main'
 ], function (Utils, StyleSheetPicker, DOM, gt) {
 
     'use strict';
@@ -29,17 +29,17 @@ define('io.ox/office/text/view/control/tablestylepicker', [
      *
      * @extends StyleSheetPicker
      */
-    function TableStylePicker(app) {
+    function TableStylePicker(docView) {
 
         var // self reference
             self = this,
 
             // table style sheets
-            tableStyles = null;
+            tableStyles = docView.getDocModel().getStyleCollection('table');
 
         // base constructors --------------------------------------------------
 
-        StyleSheetPicker.call(this, app, 'table', {
+        StyleSheetPicker.call(this, docView, 'table', {
             width: 200,
             icon: 'docs-table-style',
             tooltip: /*#. tool tip: predefined styles for text tables */ gt('Table style'),
@@ -94,8 +94,7 @@ define('io.ox/office/text/view/control/tablestylepicker', [
                         styleId: tableInfo.styleId,
                         table: { exclude: ['lastRow', 'lastCol', 'bandsVert'] }
                     }, { preview: true });
-                    tableInfo.tableNode.find(DOM.PARAGRAPH_NODE_SELECTOR).css({ margin: 0 });  // no margins in micro tables allowed
-                });
+                }, { infoString: 'Text: TableStylePicker: updateTableFormatting' });
 
                 // forget reference to the timer, when all tables have been formatted
                 formatTimer.always(function () { formatTimer = null; });
@@ -122,17 +121,12 @@ define('io.ox/office/text/view/control/tablestylepicker', [
 
         // initialization -----------------------------------------------------
 
-        // initialize class members
-        app.onInit(function () {
-            tableStyles = app.getModel().getStyleCollection('table');
-        });
-
         // register a handler that inserts a table element into each list item
-        this.getMenu().on('create:item', createItemHandler);
+        this.listenTo(this.getMenu(), 'create:item', createItemHandler);
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = self = tableStyles = null;
+            self = docView = tableStyles = null;
         });
 
     } // class TableStylePicker
