@@ -6,15 +6,14 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define.async('io.ox/office/tk/config', [
-    'io.ox/core/gettext',
+define('io.ox/office/tk/config', [
     'settings!io.ox/office'
-], function (GetText, settings) {
+], function (settings) {
 
     'use strict';
 
@@ -51,6 +50,23 @@ define.async('io.ox/office/tk/config', [
         return _.isString(key) ? ((key in documentsConfig) ? documentsConfig[key] : def) : documentsConfig;
     };
 
+    /**
+     * Sets a value of a specific configuration property.
+     *
+     * @param {String} [key]
+     *  The key of a specific configuration property. If the key is missing in
+     *  the configuration, it will be added.
+     *
+     * @param {Any} [value]
+     *  A value to be set in association with the provided key.
+     */
+    Config.set = function (key, value) {
+        if (_.isString(key)) {
+            documentsConfig[key] = value;
+            settings.set('module', documentsConfig).save();
+        }
+    };
+
     /**
      * Returns the state of a specific Boolean configuration property.
      *
@@ -74,7 +90,7 @@ define.async('io.ox/office/tk/config', [
      * the browser URL.
      *
      * @param {String} flag
-     *  The name of the flag.
+     *  The name of the URL anchor flag.
      *
      * @param {Boolean} [def=false]
      *  The default state, if the URL does not contain the specified option.
@@ -87,9 +103,29 @@ define.async('io.ox/office/tk/config', [
         return (/^(1|true)$/i).test(value) || ((/^(0|false)$/i).test(value) ? false : (def === true));
     };
 
+    /**
+     * Returns the state of the specified Boolean flag in the anchor part of
+     * the browser URL, but only if the application runs in debug mode (see
+     * constant Config.DEBUG for details). Without debug mode, this method
+     * simply returns the passed default value.
+     *
+     * @param {String} flag
+     *  The name of the URL anchor flag.
+     *
+     * @param {Boolean} [def=false]
+     *  The default state, if the URL does not contain the specified option, or
+     *  if the application is not in debug mode.
+     *
+     * @returns {Boolean}
+     *  The state of the specified flag in the anchor part of the URL.
+     */
+    Config.getDebugUrlFlag = function (flag, def) {
+        return Config.DEBUG ? Config.getUrlFlag(flag, def) : (def === true);
+    };
+
     /**
      * Returns whether a specific debug mode is enabled in all OX Documents
-     * applications. the debug mode will be enabled, if the passed server-side
+     * applications. The debug mode will be enabled, if the passed server-side
      * configuration property is set to true, and the passed URL hash option is
      * NOT explicitly set to false; or if the server-side configuration
      * property 'debugondemand' is set to true, and the passed URL hash option
@@ -117,21 +153,21 @@ define.async('io.ox/office/tk/config', [
      *
      * @constant
      */
-    Config.LOCALE = '';
+    Config.LOCALE = ox.language;
 
     /**
      * The lower-case language identifier of the current locale, e.g. 'en'.
      *
      * @constant
      */
-    Config.LANGUAGE = '';
+    Config.LANGUAGE = ox.language.split('_')[0];
 
     /**
      * The upper-case country identifier of the current locale, e.g. 'US'.
      *
      * @constant
      */
-    Config.COUNTRY = '';
+    Config.COUNTRY = ox.language.split('_')[1];
 
     /**
      * A Boolean flag specifying whether the debug mode is enabled in all OX
@@ -145,27 +181,8 @@ define.async('io.ox/office/tk/config', [
      */
     Config.DEBUG = Config.getDebugState('debugavailable', 'office:enable-debug');
 
-    // user data ==============================================================
-
-    // get locale data of the user currently logged in
-    var userPromise = GetText.language.done(function (locale) {
-
-        var // language and country identifier from the locale
-            matches = /^([a-z]+)(?:_([A-Z]+))?/.exec(locale);
-
-        // locale settings
-        if (_.isArray(matches)) {
-            Config.LOCALE = locale;
-            Config.LANGUAGE = matches[1] || '';
-            Config.COUNTRY = matches[2] || '';
-        } else {
-            window.console.error('Config: unrecognized locale "' + locale + '"');
-        }
-    });
-
     // exports ================================================================
 
-    // create the exported promise that will be resolved with the Config class
-    return userPromise.then(_.constant(Config), _.constant(Config));
+    return Config;
 
 });
