@@ -6,18 +6,21 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
+/* global blankshield */
+
 define('io.ox/office/tk/control/button', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/forms',
     'io.ox/office/tk/control/group',
     'io.ox/office/tk/control/captionmixin',
-    'io.ox/office/tk/control/widthmixin'
-], function (Utils, Forms, Group, CaptionMixin, WidthMixin) {
+    'io.ox/office/tk/control/widthmixin',
+    'io.ox/office/tk/keycodes'
+], function (Utils, Forms, Group, CaptionMixin, WidthMixin, KeyCodes) {
 
     'use strict';
 
@@ -81,7 +84,7 @@ define('io.ox/office/tk/control/button', [
             toggle = Utils.getBooleanOption(initOptions, 'toggle', false),
 
             // the fixed value to be sent when activating the button (not in toggle mode)
-            value = toggle ? null : Utils.getOption(initOptions, 'value'),
+            fixedValue = toggle ? null : Utils.getOption(initOptions, 'value'),
 
             // custom predicate callback for button highlighting
             highlightHandler = Utils.getFunctionOption(initOptions, 'highlight'),
@@ -127,31 +130,13 @@ define('io.ox/office/tk/control/button', [
             // can be used to change the value of an item independently)
         }
 
-        /**
-         * Modifies the button for the drop-down view
-         *
-         * @param {Object} dropDownVersion
-         *  The following options are supported:
-         *  @param {String} [dropDownVersion.label]
-         *      If specified, the button gets a label
-         */
-        function createDropDownVersion(dropDownVersion){
-            if (_.has(dropDownVersion, 'label')) {
-                buttonNode.find('.caption').append(
-                    Forms.createSpanMarkup(dropDownVersion.label, {
-                        classes: 'drop-down'
-                    })
-                );
-            }
-        }
-
         /**
          * Activates/Deactivates a smaller version of the Button
          *
          * @param {Boolean} value
          *  decides whether the Button should be small or wide
          */
-        function smallButton(value){
+        function smallButton(value) {
             // show the small version
             if (value === true) {
                 // set new CSS if exists
@@ -170,7 +155,7 @@ define('io.ox/office/tk/control/button', [
                 // re-set the old (saved) styles
                 if (_.has(smallerVersion, 'css') && !_.isNull(savedStyles.css)) {
                     buttonNode.removeAttr('style');
-                    buttonNode.attr('style',savedStyles.css);
+                    buttonNode.attr('style', savedStyles.css);
                     savedStyles.css = null;
                 // if there were old styles, remove the style attribute
                 } else if (_.has(smallerVersion, 'css')) {
@@ -198,12 +183,12 @@ define('io.ox/office/tk/control/button', [
          * Overwrites the base-methods (group.js) to
          * activate/deactivate the small version of the button
          */
-        this.activateSmallVersion = function(){
+        this.activateSmallVersion = function () {
             if (_.isObject(smallerVersion)) {
                 smallButton(true);
             }
         };
-        this.deactivateSmallVersion = function(){
+        this.deactivateSmallVersion = function () {
             if (_.isObject(smallerVersion)) {
                 smallButton(false);
             }
@@ -211,15 +196,17 @@ define('io.ox/office/tk/control/button', [
 
         // initialization -----------------------------------------------------
 
+        Forms.setToolTip(buttonNode, initOptions);
+
         // set the fixed value of push buttons (replace Group instances by a
         // anonymous function that returns the current group value)
         if (!toggle) {
-            if (value instanceof Group) {
-                value = (function (group) {
+            if (fixedValue instanceof Group) {
+                fixedValue = (function (group) {
                     return function () { return group.getValue(); };
-                }(value));
+                }(fixedValue));
             }
-            Forms.setButtonValue(buttonNode, value, initOptions);
+            Forms.setButtonValue(buttonNode, fixedValue, initOptions);
         }
 
         // show a caret
@@ -228,8 +215,10 @@ define('io.ox/office/tk/control/button', [
         }
 
         // creates the drop-down version of this button, if specified
-        if (dropDownVersion) {
-            createDropDownVersion(dropDownVersion);
+        if (dropDownVersion && _.isString(dropDownVersion.label)) {
+            buttonNode.find('.caption').append(
+                Forms.createSpanMarkup(dropDownVersion.label, { classes: 'drop-down' })
+            );
         }
 
         // insert the button into this group
@@ -238,20 +227,48 @@ define('io.ox/office/tk/control/button', [
         // update handler for selected state
         this.registerUpdateHandler(updateHandler);
 
-        buttonNode.on(Forms.DEFAULT_CLICK_TYPE, Utils.preventFocusChange);
-        Utils.addToPreventFocusChange(buttonNode);
+        if (_.isEmpty(buttonNode.attr('href'))) {
+            buttonNode.on(Forms.DEFAULT_CLICK_TYPE, Utils.preventFocusChange);
 
-        // button click handler (convert ENTER and SPACE keys to click events)
-        Forms.setButtonKeyHandler(buttonNode);
-        buttonNode.on(Forms.DEFAULT_CLICK_TYPE, function (event) {
-            var value = toggle ? !self.getValue() : Forms.getButtonValue(buttonNode);
-            self.triggerChange(value, { sourceEvent: event });
-        });
+            // button click handler (convert ENTER and SPACE keys to click events)
+            Forms.setButtonKeyHandler(buttonNode);
+
+            buttonNode.on(Forms.DEFAULT_CLICK_TYPE, function (event) {
+                // BUG #37143
+                // The button was clicked twice. It's a known jquery-tap issue called "GhostClick".
+                // Current solution: preventDefault to stop after first invocation.
+                if (_.browser.Android && Forms.DEFAULT_CLICK_TYPE === 'tap') { event.preventDefault(); }
+
+                var value = toggle ? !self.getValue() : Forms.getButtonValue(buttonNode);
+                self.triggerChange(value, { sourceEvent: event });
+            });
+
+            // handler for 'remote' events triggered at the group root node
+            this.getNode().on('remote', function (event, command) {
+                if (command === 'click') {
+                    buttonNode.trigger(Forms.DEFAULT_CLICK_TYPE);
+                }
+            });
+
+        } else {
+            buttonNode.on('keyup', function (event) {
+                if ((event.keyCode === KeyCodes.ENTER) || (event.keyCode === KeyCodes.SPACE)) {
+                    blankshield.open($(event.target).attr('href'), '_blank');
+                }
+            });
+
+            // handler for 'remote' events triggered at the group root node
+            this.getNode().on('remote', function (event, command) {
+                if (command === 'click') {
+                    blankshield.open($(event.target).attr('href'), '_blank');
+                }
+            });
+        }
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
             buttonNode.remove();
-            initOptions = self = buttonNode = value = highlightHandler = null;
+            initOptions = self = buttonNode = fixedValue = highlightHandler = null;
         });
 
     } // class Button
