@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -230,6 +230,7 @@ define('io.ox/office/tk/control/buttongroup', [
 
             // add the button value, and insert the button node
             Forms.setButtonValue(buttonNode, value, options);
+            Forms.setToolTip(buttonNode, options);
             this.addChildNodes(buttonNode);
 
             if (dropDownVersion) {
@@ -273,7 +274,7 @@ define('io.ox/office/tk/control/buttongroup', [
         // initialization -----------------------------------------------------
 
         // set a tool tip that will be used for the drop-down version of this control
-        Forms.setToolTip(this.getNode(), Utils.getStringOption(initOptions, 'tooltip'));
+        Forms.setToolTip(this.getNode(), initOptions);
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
