@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -60,7 +60,7 @@ define('io.ox/office/tk/control/checkbox', [
             attributes: { role: 'checkbox' } // default ARIA role
         }, initOptions, {
             toggle: true,
-            highlight: _.constant(false), // never highlight the check box
+            highlight: _.constant(false) // never highlight the check box
         }));
 
         // private methods ----------------------------------------------------
