@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,7 +15,7 @@ define('io.ox/office/tk/control/checkgroup', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/forms',
     'io.ox/office/tk/control/buttongroup',
-    'gettext!io.ox/office/main'
+    'gettext!io.ox/office/tk/main'
 ], function (Utils, Forms, ButtonGroup, gt) {
 
     'use strict';
@@ -133,7 +133,7 @@ define('io.ox/office/tk/control/checkgroup', [
             if (selectAllNode) {
                 var checkedCount = Forms.filterCheckedButtonNodes(optionButtons).length,
                     state = (checkedCount === 0) ? false : (checkedCount === optionButtons.length) ? true : null;
-                Forms.checkButtonNodes(selectAllNode, state, { design: design, ambiguous: false/*true*/ });
+                Forms.checkButtonNodes(selectAllNode, state, { design: design, ambiguous: false });
             }
         }
 
@@ -143,6 +143,10 @@ define('io.ox/office/tk/control/checkgroup', [
          */
         function buttonClickHandler(event) {
 
+            // Bug 37143: The button was clicked twice. It's a known jQuery-tap issue called "GhostClick".
+            // Current solution: prevent default action to stop after first invocation.
+            if (event.type === 'tap') { event.preventDefault(); }
+
             var // current value of this control
                 valueArray = self.getValue(),
                 // the value of the clicked button
