@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -69,11 +69,11 @@ define('io.ox/office/tk/control/combofield', [
          */
         function returnFocusToTextField() {
             // not on touch devices to prevent flickering keyboard etc.
-            if (!Modernizr.touch) {
+            if (!Utils.TOUCHDEVICE) {
                 // deferred to not confuse focus listeners due to mixed focus events triggered from mouse click and focus() call
                 self.executeDelayed(function () {
                     self.getInputNode().focus();
-                });
+                }, undefined, 'TK: ComboField');
             }
         }
 
@@ -87,13 +87,21 @@ define('io.ox/office/tk/control/combofield', [
         }
 
         /**
-         * Handles 'popup:show' events and moves the focus to the text field.
+         * Handles 'popup:show' events, moves the focus to the text field and disables the tooltip.
          */
         function popupShowHandler() {
+            Forms.disableToolTip(self.getNode());
             returnFocusToTextField();
             scrollToListItem(menu.getSelectedItemNodes().first());
         }
 
+        /**
+         * Handles 'popup:hide' events and enables the tooltip.
+         */
+        function popupHideHandler() {
+            Forms.enableToolTip(self.getNode());
+        }
+
         /**
          * Update handler that activates a list item if it matches the passed
          * value.
@@ -112,6 +120,7 @@ define('io.ox/office/tk/control/combofield', [
          * entry according to cursor keys.
          */
         function inputKeyDownHandler(event) {
+
             var // item node currently selected
                 buttonNode = menu.getSelectedItemNodes().first(),
                 // the <input> control
@@ -122,7 +131,7 @@ define('io.ox/office/tk/control/combofield', [
                 self.showMenu();
                 return;
             }
-            
+
             // open drop-down menu on specific navigation keys
             switch (event.keyCode) {
             case KeyCodes.UP_ARROW:
@@ -260,7 +269,7 @@ define('io.ox/office/tk/control/combofield', [
         });
 
         // register pop-up event handlers
-        menu.on({ 'popup:show': popupShowHandler, 'create:item': function () { self.refresh(); } });
+        menu.on({ 'popup:show': popupShowHandler, 'create:item': function () { self.refresh(); }, 'popup:hide': popupHideHandler });
 
         // register input control for focus handling (to keep menu visible)
         menu.registerFocusableNodes(this.getInputNode());
