@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -185,10 +185,7 @@ define('io.ox/office/tk/control/group', [
          * update handlers which will update the visual appearance of this
          * group.
          */
-        function updateValue(newValue, force) {
-
-            // do nothing, if the value does not change (unless forced)
-            if (!force && _.isEqual(newValue, groupValue)) { return; }
+        function updateValue(newValue) {
 
             var // the old value of this group (for event handlers)
                 oldValue = groupValue;
@@ -247,14 +244,14 @@ define('io.ox/office/tk/control/group', [
                 switch (sourceEvent.type) {
                 case 'tap':
                 case 'click':
-                        // Forms.setButtonKeyHandler() triggers 'click' events with 'keyCode' field
+                    // Forms.setButtonKeyHandler() triggers 'click' events with 'keyCode' field
                     eventOptions.sourceType = ('keyCode' in sourceEvent) ? 'keyboard' : 'click';
                     // click events from keyboard: keep focus on button after SPACE or TAB key (see Forms.setButtonKeyHandler() method)
                     eventOptions.preserveFocus = (sourceEvent.keyCode === KeyCodes.SPACE) || (sourceEvent.keyCode === KeyCodes.TAB);
 
                     //necessary on touchdevice that the focus goes back in the document so the keyboard stays open
-                    Utils.preventFocusChange(sourceEvent);
-
+                    eventOptions.softKeyboardWasOpen = Utils.preventFocusChange(sourceEvent);
+                    eventOptions.sourceEventType = sourceEvent.type;
                     break;
                 case 'keydown':
                 case 'keypress':
@@ -537,7 +534,7 @@ define('io.ox/office/tk/control/group', [
          *  A reference to this group.
          */
         this.refresh = this.createDebouncedMethod(Utils.NOOP, function () {
-            updateValue(groupValue, true);
+            updateValue(groupValue);
         });
 
         /**
