@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -50,11 +50,12 @@ define('io.ox/office/tk/control/menumixin', [
      *      when clicked. If omitted, the drop-down menu cannot be toggled
      *      using the mouse; the client implementation has to provide its own
      *      logic to show and hide the menu.
-     *  @param {HTMLElement|jQuery} [initOptions.ariaOwner=initOptions.button]
+     *  @param {HTMLElement|jQuery} [initOptions.ariaOwner]
      *      The contextual parent DOM element that acts as owner of the menu.
      *      Exposes the relationship between the parent and the menu element to
      *      assistive technologies where the DOM hierarchy doesn't represent
-     *      the relationship. Is defaulted to the drop-down button element.
+     *      the relationship. If omitted, the drop-down button element passed
+     *      with the option 'button' will be used instead.
      *  @param {Boolean} [initOptions.caret=true]
      *      If set to true or omitted, a caret symbol will be added to the
      *      right border of the drop-down button specified with the option
@@ -63,6 +64,9 @@ define('io.ox/office/tk/control/menumixin', [
      *      If set to true, the TAB key will move the browser focus through the
      *      form controls of the menu. By default, the TAB key will leave and
      *      close the menu.
+     *  @param {Boolean} [initOptions.autoCloseParent=true]
+     *      Whether to trigger a 'group:cancel' event after closing this menu
+     *      with a click on the menu button.
      */
     function MenuMixin(menu, initOptions) {
 
@@ -82,7 +86,10 @@ define('io.ox/office/tk/control/menumixin', [
             ariaOwner = $(Utils.getOption(initOptions, 'ariaOwner', menuButton)),
 
             // whether to use TAB key to navigate inside the menu
-            tabNavigate = Utils.getOption(initOptions, 'tabNavigate', false);
+            tabNavigate = Utils.getOption(initOptions, 'tabNavigate', false),
+
+            // whether to trigger a 'group:cancel' event when closing with mouse click
+            autoCloseParent = Utils.getBooleanOption(initOptions, 'autoCloseParent', true);
 
         // private methods ----------------------------------------------------
 
@@ -110,6 +117,40 @@ define('io.ox/office/tk/control/menumixin', [
             ariaOwner.attr('aria-expanded', false);
         }
 
+        /**
+         * Handles keyboard events from any control in the group object.
+         */
+        function groupKeyDownHandler(event) {
+            if (self.isEnabled() && Forms.isVisibleNode(menuButton)) {
+                if (KeyCodes.matchKeyCode(event, 'DOWN_ARROW', { alt: true }) || KeyCodes.matchKeyCode(event, 'UP_ARROW', { alt: true })) {
+                    event.target = menuButton[0];
+                    Forms.triggerClickForKey(event);
+                }
+            }
+        }
+
+        /**
+         * Handler for 'remote' events that can be triggered manually at the
+         * root node of the group to control the visibility of the drop-down
+         * menu from external code.
+         */
+        function groupRemoteHandler(event, command) {
+
+            switch (command) {
+            case 'show':
+                menu.show();
+                // keep menu open regardless of browser focus
+                if (menu.getAutoCloseMode()) {
+                    menu.setAutoCloseMode(false);
+                    menu.one('popup:hide', function () { menu.setAutoCloseMode(true); });
+                }
+                break;
+            case 'hide':
+                menu.hide();
+                break;
+            }
+        }
+
         /**
          * Handles click events from the drop-down button, and toggles the
          * drop-down menu.
@@ -130,23 +171,11 @@ define('io.ox/office/tk/control/menumixin', [
 
             // trigger 'group:cancel' event, if the menu has been closed with mouse click,
             // or after click on a disabled group, but never after key events
-            if (!menu.isVisible() && !_.isNumber(event.keyCode)) {
+            if (autoCloseParent && !menu.isVisible() && !_.isNumber(event.keyCode)) {
                 self.triggerCancel({ sourceEvent: event });
             }
         }
 
-        /**
-         * Handles keyboard events from any control in the group object.
-         */
-        function groupKeyDownHandler(event) {
-            if (self.isEnabled() && Forms.isVisibleNode(menuButton)) {
-                if (KeyCodes.matchKeyCode(event, 'DOWN_ARROW', { alt: true }) || KeyCodes.matchKeyCode(event, 'UP_ARROW', { alt: true })) {
-                    event.target = menuButton[0];
-                    Forms.triggerClickForKey(event);
-                }
-            }
-        }
-
         /**
          * Handles keyboard events in the focused drop-down button.
          */
@@ -268,8 +297,8 @@ define('io.ox/office/tk/control/menumixin', [
          * @returns {MenuMixin}
          *  A reference to this instance.
          */
-        this.clearMenu = function () {
-            menu.clearContents();
+        this.clearMenu = function (options) {
+            menu.clearContents(options);
             return this;
         };
 
@@ -286,12 +315,12 @@ define('io.ox/office/tk/control/menumixin', [
             menuButton.addClass('caret-button').append(Forms.createCaretMarkup('down'));
         }
 
-        // set the aria relation between button and menu with the 'aria-owns' attribute
-        // and use the aria-label of the button also for the menu
-        menuNode.attr({ id: menu.getUid(), 'aria-label': (ariaOwner.attr('aria-label') || null) });
-        ariaOwner.attr({ 'aria-owns': menu.getUid(), 'aria-haspopup': 'true', 'aria-expanded': 'false' });
+        // set the ARIA relation between button and menu with the 'aria-owns' attribute
+        // and use the ARIA label of the button also for the menu
+        menuNode.attr({ id: menu.getUid(), 'aria-label': ariaOwner.attr('aria-label') || null });
+        ariaOwner.attr({ 'aria-owns': menu.getUid(), 'aria-haspopup': true, 'aria-expanded': false });
 
-        // register menu node for focus handling (group remains focused state while focus is in menu node)
+        // register menu node for focus handling (group remains in focused state while focus is in menu node)
         this.registerFocusableNodes(menuNode);
 
         // register menu button for focus handling (menu remains visible while button is focused)
@@ -301,18 +330,15 @@ define('io.ox/office/tk/control/menumixin', [
         this.on('group:beforehide', function () { menu.hide(); });
         this.on('group:enable', function (event, state) { if (!state) { menu.hide(); } });
         groupNode.on('keydown', groupKeyDownHandler);
+        groupNode.on('remote', groupRemoteHandler);
 
-        Utils.addToPreventFocusChange(menuButton);
-        Utils.addToPreventFocusChange(ariaOwner);
-
+        // register event handlers for the menu button
         menuButton.on(Forms.DEFAULT_CLICK_TYPE, menuButtonClickHandler);
         menuButton.on('keydown', menuButtonKeyDownHandler);
 
         // register event handlers for the drop-down menu
-        menu.on({
-            'popup:show': popupShowHandler,
-            'popup:beforehide': popupBeforeHideHandler,
-        });
+        menu.on('popup:show', popupShowHandler);
+        menu.on('popup:beforehide', popupBeforeHideHandler);
         menuNode.on('keydown keypress keyup', menuKeyHandler);
 
         // destroy all class members on destruction
