@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -100,11 +100,22 @@ define('io.ox/office/tk/control/radiogroup', [
         // option button click handler (convert ENTER and SPACE keys to click events)
         Forms.setButtonKeyHandler(this.getNode());
         this.getNode().on(Forms.DEFAULT_CLICK_TYPE, Forms.OPTION_BUTTON_SELECTOR, function (event) {
-            var toggleClick = Forms.isCheckedButtonNode(this) && !_.isNull(toggleValue) && !_.isUndefined(toggleValue),
+
+            var // check whether a click on a selected button toggles state back
+                toggleClick = Forms.isCheckedButtonNode(this) && !_.isNull(toggleValue) && !_.isUndefined(toggleValue),
+                // the new control value
                 value = toggleClick ? toggleValue : Forms.getButtonValue(this);
+
             self.triggerChange(value, { sourceEvent: event });
         });
 
+        // handler for 'remote' events at the group root node to activate a specific option button
+        this.getNode().on('remote', function (event, command, value) {
+            if (command === 'click') {
+                self.findOptionButtons(value).first().trigger(Forms.DEFAULT_CLICK_TYPE);
+            }
+        });
+
         // destroy all class members on destruction
         this.registerDestructor(function () {
             initOptions = self = toggleValue = matcher = null;
