@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -184,7 +184,7 @@ define('io.ox/office/tk/control/radiolist', [
 
             // update visibility of the section nodes (hide sections without visible items)
             menu.getSectionNodes().each(function () {
-                var visible = $(this).find(Forms.BUTTON_SELECTOR + Forms.VISIBLE_SELECTOR).length > 0;
+                var visible = $(this).find(Forms.BUTTON_SELECTOR + Forms.VISIBLE_SELECTOR + ', ' + Forms.TITLE_SELECTOR).length > 0;
                 Forms.showNodes(this, visible);
             });
 
@@ -279,7 +279,7 @@ define('io.ox/office/tk/control/radiolist', [
                 // re-set the old (saved) styles
                 if (_.has(savedStyles, 'css') && !_.isNull(savedStyles.css)) {
                     captionButton.removeAttr('style');
-                    captionButton.attr('style',savedStyles.css);
+                    captionButton.attr('style', savedStyles.css);
                     savedStyles.css = null;
                 }
 
@@ -388,12 +388,12 @@ define('io.ox/office/tk/control/radiolist', [
          * Overwrites the base-methods (group.js) to
          * activate/deactivate the small version of the button
          */
-        this.activateSmallVersion = function(){
+        this.activateSmallVersion = function () {
             if (_.isObject(smallerVersion)) {
                 smallCaptionButton(true);
             }
         };
-        this.deactivateSmallVersion = function(){
+        this.deactivateSmallVersion = function () {
             if (_.isObject(smallerVersion)) {
                 smallCaptionButton(false);
             }
@@ -401,6 +401,9 @@ define('io.ox/office/tk/control/radiolist', [
 
         // initialization -----------------------------------------------------
 
+        Forms.setToolTip(splitButton, initOptions);
+        Forms.setToolTip(menuButton, menuButtonOptions);
+
         // add the split button to the group
         if (splitButton.length > 0) {
             Forms.setButtonValue(splitButton, splitValue, initOptions);
