@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,8 +15,10 @@ define('io.ox/office/tk/control/spinfield', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/keycodes',
     'io.ox/office/tk/forms',
-    'io.ox/office/tk/control/textfield'
-], function (Utils, KeyCodes, Forms, TextField) {
+    'io.ox/office/tk/utils/tracking',
+    'io.ox/office/tk/control/textfield',
+    'io.ox/office/settings/units'
+], function (Utils, KeyCodes, Forms, Tracking, TextField, Units) {
 
     'use strict';
 
@@ -38,18 +40,6 @@ define('io.ox/office/tk/control/spinfield', [
      *  be passed. In this case, a new instance of TextField.NumberValidator
      *  will be created automatically based on these options. Additionally, the
      *  following options are supported:
-     *  @param {Number} [initOptions.smallStep=1]
-     *      The distance of small steps that will be used to increase or
-     *      decrease the current value of the spin field when using the spin
-     *      buttons or the cursor keys.
-     *  @param {Number} [initOptions.largeStep=10]
-     *      The distance of large steps that will be used to increase or
-     *      decrease the current value of the spin field when using the PAGEUP
-     *      or PAGEPOWN key.
-     *  @param {Boolean} [initOptions.roundStep=false]
-     *      If set to true, and the current value of the spin field will be
-     *      changed, the value will be rounded to entire multiples of the step
-     *      distance. Otherwise, the fractional part will remain in the value.
      */
     function SpinField(initOptions) {
 
@@ -59,14 +49,8 @@ define('io.ox/office/tk/control/spinfield', [
             // the number validator of this spin field
             validator = Utils.getObjectOption(initOptions, 'validator') || new TextField.NumberValidator(initOptions),
 
-            // the distance for small steps
-            smallStep = Utils.getNumberOption(initOptions, 'smallStep', 1, 0),
-
-            // the distance for large steps
-            largeStep = Utils.getNumberOption(initOptions, 'largeStep', 10, 0),
-
-            // whether to round while applying steps
-            roundStep = Utils.getBooleanOption(initOptions, 'roundStep', false);
+            // whether to turn of spin controls
+            spinEnabled = Utils.getBooleanOption(initOptions, 'spinEnabled', true);
 
         // base constructors --------------------------------------------------
 
@@ -74,19 +58,28 @@ define('io.ox/office/tk/control/spinfield', [
 
         // private methods ----------------------------------------------------
 
+        function getSmallStep() {
+            return Units.getStandardUnitStep();
+        }
+
+        function getLargeStep() {
+            return getSmallStep() * 5;
+        }
+
         /**
          * Changes the current value of the spin field, according to the passed
-         * step and the rounding mode passed to the constructor.
+         * step of the general unit.
          */
         function changeValue(step) {
 
-            var // the current value of the spin field
-                oldValue = self.getFieldValue(),
-                // the current value, rounded to the previous/next border
-                roundedValue = (step < 0) ? Utils.roundDown(oldValue, -step) : Utils.roundUp(oldValue, step),
-                // the new value, depending on rounding mode
-                newValue = (!roundStep || (oldValue === roundedValue)) ? (oldValue + step) : roundedValue;
+            var //
+                unit = Units.getStandardUnit(),
+                // the current value of the spin field
+                oldValue = Utils.convertHmmToLength(self.getFieldValue(), unit),
+                // the new value
+                newValue = oldValue + step;
 
+            newValue = Utils.convertLengthToHmm(newValue, unit);
             // set new value, after restricting to minimum/maximum
             newValue = validator.restrictValue(newValue);
             self.setValue(newValue);
@@ -104,9 +97,9 @@ define('io.ox/office/tk/control/spinfield', [
             // enables or disables node tracking according to own enabled state
             function initializeTracking() {
                 if (self.isEnabled()) {
-                    spinButton.enableTracking({ autoRepeat: true });
+                    Tracking.enableTracking(spinButton, { autoRepeat: true });
                 } else {
-                    spinButton.disableTracking();
+                    Tracking.disableTracking(spinButton);
                 }
             }
 
@@ -119,8 +112,8 @@ define('io.ox/office/tk/control/spinfield', [
 
             // register tracking event listeners to change the current value
             spinButton.on({
-                'tracking:start tracking:repeat': function () { changeValue(increase ? smallStep : -smallStep); },
-                'tracking:end tracking:cancel': function () { self.executeDelayed(function () { self.getInputNode().focus(); }); }
+                'tracking:start tracking:repeat': function () { changeValue(increase ? getSmallStep() : -getSmallStep()); },
+                'tracking:end tracking:cancel': function () { self.executeDelayed(function () { self.getInputNode().focus(); }, undefined, 'TK: createSpinButton'); }
             });
 
             // enable/disable the button according to the current value
@@ -141,16 +134,16 @@ define('io.ox/office/tk/control/spinfield', [
 
             switch (event.keyCode) {
             case KeyCodes.UP_ARROW:
-                changeValue(smallStep);
+                changeValue(getSmallStep());
                 return false;
             case KeyCodes.DOWN_ARROW:
-                changeValue(-smallStep);
+                changeValue(-getSmallStep());
                 return false;
             case KeyCodes.PAGE_UP:
-                changeValue(largeStep);
+                changeValue(getLargeStep());
                 return false;
             case KeyCodes.PAGE_DOWN:
-                changeValue(-largeStep);
+                changeValue(-getLargeStep());
                 return false;
             }
         }
@@ -164,16 +157,18 @@ define('io.ox/office/tk/control/spinfield', [
 
         // initialization -----------------------------------------------------
 
-        // add special marker class used to adjust formatting, add spin buttons
-        this.getNode().addClass('spin-field').append(
-            $('<div>').addClass('spin-wrapper').append(createSpinButton(true), createSpinButton(false))
-        );
+        if (spinEnabled) {
+            // add special marker class used to adjust formatting, add spin buttons
+            this.getNode().addClass('spin-field').append(
+                $('<div>').addClass('spin-wrapper').append(createSpinButton(true), createSpinButton(false))
+            );
 
-        // add ARIA attributes (note: role 'spinbutton' would be correct but doesn't work with VoiceOver)
-        this.getInputNode().attr({ role: 'slider', 'aria-valuemin': validator.getMin(), 'aria-valuemax': validator.getMax() });
+            // add ARIA attributes (note: role 'spinbutton' would be correct but doesn't work with VoiceOver)
+            this.getInputNode().attr({ role: 'slider', 'aria-valuemin': validator.getMin(), 'aria-valuemax': validator.getMax() });
 
-        // register event handlers
-        this.getInputNode().on('keydown', inputKeyDownHandler);
+            // register event handlers
+            this.getInputNode().on('keydown', inputKeyDownHandler);
+        }
 
         // register input field update handler
         this.registerUpdateHandler(updateHandler);
