@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -15,10 +15,12 @@ define('io.ox/office/tk/control/textfield', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/keycodes',
     'io.ox/office/tk/forms',
+    'io.ox/office/tk/locale/localedata',
+    'io.ox/office/tk/locale/parser',
     'io.ox/office/tk/object/baseobject',
     'io.ox/office/tk/control/group',
     'io.ox/office/tk/control/widthmixin'
-], function (Utils, KeyCodes, Forms, BaseObject, Group, WidthMixin) {
+], function (Utils, KeyCodes, Forms, LocaleData, Parser, BaseObject, Group, WidthMixin) {
 
     'use strict';
 
@@ -301,12 +303,12 @@ define('io.ox/office/tk/control/textfield', [
         }
 
         /**
-         * Activates/Deactivates a smaller version of the textfield
+         * Activates/Deactivates a smaller version of the text field
          *
          * @param {Boolean} value
-         *  decides wether the textfield should be small or wide
+         *  decides whether the text field should be small or wide
          */
-        function smallTextfield(value){
+        function smallTextfield(value) {
             // show the small version
             if (value === true) {
                 // set new css if exists
@@ -323,7 +325,7 @@ define('io.ox/office/tk/control/textfield', [
                 // re-set the old (saved) styles
                 if (_.has(savedStyles, 'css') && !_.isNull(savedStyles.css)) {
                     wrapperNode.removeAttr('style');
-                    wrapperNode.attr('style',savedStyles.css);
+                    wrapperNode.attr('style', savedStyles.css);
                     savedStyles.css = null;
                 }
 
@@ -371,7 +373,7 @@ define('io.ox/office/tk/control/textfield', [
         /**
          * Is applying a new validator
          */
-        this.setValidator = function(value) {
+        this.setValidator = function (value) {
             validator = value;
             return this;
         };
@@ -407,12 +409,12 @@ define('io.ox/office/tk/control/textfield', [
          * Overwrites the base-methods (group.js) to
          * activate/deactivate the small version of the button
          */
-        this.activateSmallVersion = function(){
+        this.activateSmallVersion = function () {
             if (_.isObject(smallerVersion)) {
                 smallTextfield(true);
             }
         };
-        this.deactivateSmallVersion = function(){
+        this.deactivateSmallVersion = function () {
             if (_.isObject(smallerVersion)) {
                 smallTextfield(false);
             }
@@ -420,6 +422,8 @@ define('io.ox/office/tk/control/textfield', [
 
         // initialization -----------------------------------------------------
 
+        Forms.setToolTip(this.getNode(), initOptions);
+
         // add special marker class used to adjust formatting
         this.getNode().addClass('text-field');
 
@@ -604,7 +608,7 @@ define('io.ox/office/tk/control/textfield', [
             min = Utils.getNumberOption(initOptions, 'min', -Number.MAX_VALUE, -Number.MAX_VALUE, Number.MAX_VALUE),
             max = Utils.getNumberOption(initOptions, 'max', Number.MAX_VALUE, min, Number.MAX_VALUE),
             precision = Utils.getNumberOption(initOptions, 'precision', 1, 0),
-            regex = new RegExp('^' + ((min < 0) ? '-?' : '') + '[0-9]*' + ((precision !== Math.round(precision)) ? '([.,][0-9]*)?' : '') + '$');
+            regex = new RegExp('^' + ((min < 0) ? '-?' : '') + '[0-9]*' + ((precision !== Math.round(precision)) ? ('(' + _.escapeRegExp(LocaleData.DEC)  + '[0-9]*)?') : '') + '$');
 
         // base constructor ---------------------------------------------------
 
@@ -613,12 +617,11 @@ define('io.ox/office/tk/control/textfield', [
         // methods ------------------------------------------------------------
 
         this.valueToText = function (value) {
-            // TODO L10N of decimal separator
-            return _.isFinite(value) ? String(Utils.round(value, precision)) : '';
+            return _.isFinite(value) ? Parser.numberToString(Utils.round(value, precision)) : '';
         };
 
         this.textToValue = function (text) {
-            var value = parseFloat(text.replace(/,/g, '.'));
+            var value = Parser.stringToNumber(text);
             return (_.isFinite(value) && (min <= value) && (value <= max)) ? Utils.round(value, precision) : null;
         };
 
