@@ -6,15 +6,17 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
 define('io.ox/office/tk/control/unitfield', [
     'io.ox/office/tk/utils',
+    'io.ox/office/tk/locale/parser',
+    'io.ox/office/settings/units',
     'io.ox/office/tk/control/spinfield'
-], function (Utils, SpinField) {
+], function (Utils, Parser, Units, SpinField) {
 
     'use strict';
 
@@ -47,34 +49,37 @@ define('io.ox/office/tk/control/unitfield', [
         // methods ------------------------------------------------------------
 
         /**
-         * Returning a string for the number value that can be displayed
-         * in the unit field. This string always has to contain the value
-         * in the unit 'mm'.
+         * Converts the passed value to a display string with measurement unit.
          *
          * @param {Number} value
          *  The length in 1/100 mm.
          *
          * @returns {String}
-         *  The calculated length in 'mm' together with the string ' mm'. If
-         *  the value is invalid, an empty string is returned.
+         *  The display string for the passed value, with measurement unit. If
+         *  the value is invalid, an empty string will be returned.
          */
         this.valueToText = function (value) {
-            return _.isFinite(value) ? ((Utils.round(value, this.getPrecision()) / 100) + ' mm') : '';
+            if (_.isFinite(value)) {
+                var mm = Utils.round(value, this.getPrecision()) / 100;
+                var unit = Units.getStandardUnit();
+                var result = Utils.convertLength(mm, 'mm', unit, Units.getStandardUnitStep());
+                return Parser.numberToString(result) + ' ' + unit;
+            }
+            return '';
         };
 
         /**
-         * Calculating the length value from the text that was inserted into the
-         * side pane. The string has to contain a number (optionally with '-' and
-         * ',' or '.') and optionally a string that represents the unit. Supported
-         * unit strings are 'px', 'pc', 'pt', 'in', 'cm' and 'mm'. If this unit
-         * string is not defined or if it cannot be resolved to one of the listed
-         * units, 'mm' is used as default, to calculate the length in 1/100 mm.
+         * Converts the passed display string of a length with measurement unit
+         * to a number. Supported unit strings are 'px', 'pc', 'pt', 'in', 'cm'
+         * and 'mm'. If this unit string is not defined or cannot be resolved
+         * to one of the listed units, 'mm' is used as default, to calculate
+         * the length in 1/100 mm.
          *
          * @param {String} text
-         *  The string that was inserted into the unit field.
+         *  The display string of a length with measurement unit.
          *
          * @returns {Number|Null}
-         *  The length in 1/100 mm or null, if the number value is invalid.
+         *  The length in 1/100 mm or null, if the passed text is invalid.
          */
         this.textToValue = function (text) {
 
@@ -82,7 +87,7 @@ define('io.ox/office/tk/control/unitfield', [
                 value = matches[1] || '',
                 unit = (matches[3] || '').toLowerCase();
 
-            value = (value.length > 0) ? parseFloat(value.replace(/,/g, '.')) : null;
+            value = Parser.stringToNumber(value);
             if (_.isNumber(value) && _.isFinite(value)) {
                 unit = /^(px|pc|pt|in|cm|mm)$/.test(unit) ? unit : 'mm';
                 return this.restrictValue(Utils.convertLengthToHmm(value, unit));
@@ -92,7 +97,7 @@ define('io.ox/office/tk/control/unitfield', [
         };
 
         /**
-         * The validation functions, that controls the input into the unit field
+         * The validation functions that controls the input into the unit field
          * corresponding to the defined regular expression.
          *
          * @returns {Boolean}
@@ -106,7 +111,7 @@ define('io.ox/office/tk/control/unitfield', [
         // initialization -----------------------------------------------------
 
         // the regular expression for the unit field
-        regex = new RegExp('^(' + ((this.getMin() < 0) ? '-?' : '') + '[0-9]*([.,][0-9]*)?)\\s*([a-zA-Z]*)$');
+        regex = new RegExp('^(' + ((this.getMin() < 0) ? '[+-]?' : '\\+?') + '[0-9]*([.,][0-9]*)?)\\s*([a-zA-Z]*)$');
 
     }}); // class UnitValidator
 
