@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,7 +14,6 @@
 define('io.ox/office/tk/controls', [
     'io.ox/office/tk/control/group',
     'io.ox/office/tk/control/label',
-    'io.ox/office/tk/control/picture',
     'io.ox/office/tk/control/button',
     'io.ox/office/tk/control/checkbox',
     'io.ox/office/tk/control/radiogroup',
@@ -24,7 +23,7 @@ define('io.ox/office/tk/controls', [
     'io.ox/office/tk/control/unitfield',
     'io.ox/office/tk/control/combofield',
     'io.ox/office/tk/control/spinfield'
-], function (Group, Label, Picture, Button, CheckBox, RadioGroup, CheckGroup, RadioList, TextField, UnitField, ComboField, SpinField) {
+], function (Group, Label, Button, CheckBox, RadioGroup, CheckGroup, RadioList, TextField, UnitField, ComboField, SpinField) {
 
     'use strict';
 
@@ -38,7 +37,6 @@ define('io.ox/office/tk/controls', [
     var Controls = {
             Group: Group,
             Label: Label,
-            Picture: Picture,
             Button: Button,
             CheckBox: CheckBox,
             RadioGroup: RadioGroup,
