@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -16,20 +16,42 @@ define('io.ox/office/tk/dialogs', [
     'io.ox/office/tk/utils',
     'io.ox/office/tk/forms',
     'io.ox/office/tk/keycodes',
-    'gettext!io.ox/office/main'
-], function (CoreDialogs, Utils, Forms, KeyCodes, gt) {
+    'io.ox/office/tk/utils/driveutils',
+    'io.ox/office/tk/object/triggerobject',
+    'gettext!io.ox/office/tk/main'
+], function (CoreDialogs, Utils, Forms, KeyCodes, DriveUtils, TriggerObject, gt) {
 
     'use strict';
 
-    var // the number of visible modal dialogs
-        modalDialogs = 0;
+    var OK_LABEL = /*#. button label in a OK/Cancel dialog box */ gt('OK'),
+        CANCEL_LABEL = /*#. button label in a OK/Cancel dialog box */ gt('Cancel'),
+        YES_LABEL = /*#. button label in a Yes/No dialog */ gt('Yes'),
+        NO_LABEL = /*#. button label in a Yes/No dialog */ gt('No');
 
     // static class Dialogs ===================================================
 
+    /**
+     * Provides different types of generic dialogs. Collects all standard
+     * dialog classes defined in the toolkit in a single map for convenience.
+     */
     var Dialogs = {};
 
     // methods ----------------------------------------------------------------
 
+    /**
+     * Returns the root node of the modal dialog that is currently open. In
+     * case multiple dialogs are open (stacked), the root node of the top-most
+     * dialog will be returned.
+     *
+     * @returns {jQuery}
+     *  The root node of the modal dialog that is currently open, as jQuery
+     *  object. If no modal dialog is currently open, an empty jQuery
+     *  collection will be returned instead.
+     */
+    Dialogs.getActiveModalDialogNode = function () {
+        return $('.io-ox-dialog-popup').last();
+    };
+
     /**
      * Returns whether a modal dialog is currently open.
      *
@@ -37,7 +59,25 @@ define('io.ox/office/tk/dialogs', [
      *  Whether a modal dialog is currently open.
      */
     Dialogs.isModalDialogOpen = function () {
-        return modalDialogs > 0;
+        return Dialogs.getActiveModalDialogNode().length > 0;
+    };
+
+    /**
+     * Returns a dialog width suitable for the current device type, and for the
+     * specified desired width.
+     *
+     * @param {Number} width
+     *  The desired dialog width.
+     *
+     * @return {Number}
+     *  The effective width of the dialog, in pixels. Will not be greater than
+     *  95% of the current screen width. On small devices, the dialog width
+     *  returned by this method will be fixed to 95% of the screen width,
+     *  regardless of the passed width.
+     */
+    Dialogs.getBestDialogWidth = function (width) {
+        var maxWidth = Math.round(window.innerWidth * 0.95);
+        return Utils.SMALL_DEVICE ? maxWidth : Math.min(width, maxWidth);
     };
 
     // class ModalDialog ======================================================
@@ -54,7 +94,7 @@ define('io.ox/office/tk/dialogs', [
      *  dialog. The following options are supported:
      *  @param {String} [initOptions.title]
      *      The title of the dialog window that will be shown in a larger font.
-     *  @param {Number} [initOptions.width=400]
+     *  @param {Number} [initOptions.width=500]
      *      The width of the dialog, in pixels.
      *  @param {String} [initOptions.okLabel=gt('OK')]
      *      The label of the primary button that triggers the default action of
@@ -64,6 +104,19 @@ define('io.ox/office/tk/dialogs', [
      *      The label of the cancel button that closes the dialog without
      *      performing any action. If omitted, the button will be created with
      *      the translated default label 'Cancel'.
+     *  @param {Function} [initOptions.validator]
+     *      A callback function that will be invoked before closing the dialog.
+     *      Will be used to decide whether the current state of the dialog is
+     *      valid in terms of passing the result to the user by resolving the
+     *      promise returned by the public method show(). The callback function
+     *      receives the action identifier associated to the clicked button. It
+     *      must return a boolean value specifying whether the dialog state is
+     *      valid; or a promise that will be resolved or rejected later. If the
+     *      function returns true, or a promise that will be resolved, the show
+     *      promise will be resolved too. Otherwise, the show promise will be
+     *      rejected with the value 'invalid'. The callback function will NOT
+     *      be invoked when canceling the dialog. In this case, the show
+     *      promise will be rejected immediately with the value 'cancel'.
      *  @param {Boolean} [initOptions.async=false]
      *      Whether the dialog will be opened in asynchronous mode.
      */
@@ -75,18 +128,22 @@ define('io.ox/office/tk/dialogs', [
             // the title text
             title = Utils.getStringOption(initOptions, 'title'),
 
-            // the text label of the 'OK' button
-            okLabel = Utils.getStringOption(initOptions, 'okLabel', gt('OK')),
+            // the text label of the OK button
+            okLabel = Utils.getStringOption(initOptions, 'okLabel', OK_LABEL),
 
-            // the text label of the 'Cancel' button
-            cancelLabel = Utils.getStringOption(initOptions, 'cancelLabel', gt('Cancel'));
+            // the text label of the Cancel button
+            cancelLabel = Utils.getStringOption(initOptions, 'cancelLabel', CANCEL_LABEL),
+
+            // the validator invoked before closing the dialog
+            validator = Utils.getFunctionOption(initOptions, 'validator', _.constant(true));
 
         // base constructor ---------------------------------------------------
 
         CoreDialogs.ModalDialog.call(this, {
-            width: Utils.getIntegerOption(initOptions, 'width', 400),
+            width: Dialogs.getBestDialogWidth(Utils.getIntegerOption(initOptions, 'width', 500)),
             async: Utils.getBooleanOption(initOptions, 'async', false),
-            enter: inputEnterHandler
+            enter: inputEnterHandler,
+            focus: false
         });
 
         // private methods ----------------------------------------------------
@@ -106,14 +163,18 @@ define('io.ox/office/tk/dialogs', [
             return !triggerAction;
         }
 
+        /**
+         * Handles global key events in the entire dialog.
+         */
         function keyDownHandler(event) {
-            if (event.keyCode === KeyCodes.ENTER) {
+            // safety check: ignore key events triggered during clean-up ('self' already reset to null)
+            if (self && (event.keyCode === KeyCodes.ENTER)) {
                 inputEnterHandler();
                 return false;
             }
         }
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the button element from the footer of this dialog bound to
@@ -130,20 +191,20 @@ define('io.ox/office/tk/dialogs', [
         };
 
         /**
-         * Returns the 'OK' button of this dialog, as jQuery object.
+         * Returns the OK button of this dialog, as jQuery object.
          *
          * @returns {jQuery}
-         *  The button element of the 'OK' button.
+         *  The button element of the OK button.
          */
         this.getOkButton = function () {
             return this.getButton('ok');
         };
 
         /**
-         * Returns the 'Cancel' button of this dialog, as jQuery object.
+         * Returns the Cancel button of this dialog, as jQuery object.
          *
          * @returns {jQuery}
-         *  The button element of the 'Cancel' button.
+         *  The button element of the Cancel button.
          */
         this.getCancelButton = function () {
             return this.getButton('cancel');
@@ -163,17 +224,12 @@ define('io.ox/office/tk/dialogs', [
          *  A reference to this instance.
          */
         this.enableButton = function (action, state) {
-            var buttonNode = this.getButton(action);
-            if (state) {
-                buttonNode.removeAttr('disabled').removeAttr('aria-disabled');
-            } else {
-                buttonNode.attr({ disabled: 'disabled', 'aria-disabled': true });
-            }
+            Forms.enableBSButton(this.getButton(action), state);
             return this;
         };
 
         /**
-         * Enables or disables the 'OK' button element of this dialog.
+         * Enables or disables the OK button element of this dialog.
          *
          * @param {Boolean} state
          *  Whether to enable (true) or disable (false) the button.
@@ -186,7 +242,7 @@ define('io.ox/office/tk/dialogs', [
         };
 
         /**
-         * Enables or disables the 'Cancel' button element of this dialog.
+         * Enables or disables the Cancel button element of this dialog.
          *
          * @param {Boolean} state
          *  Whether to enable (true) or disable (false) the button.
@@ -206,63 +262,118 @@ define('io.ox/office/tk/dialogs', [
          *  has been closed.
          *
          * @param {Function} [callback]
-         *  A callback function that will be invoked after the dialog became
-         *  visible.
+         *  A callback function that will be invoked immediately after showing
+         *  the dialog.
          *
          * @returns {jQuery.Promise}
-         *  The promise of a Deferred object that will be resolved with the
-         *  action identifier, if any action button ('OK' button, alternative
-         *  buttons) has been pressed; or rejected, if the 'Cancel' button has
-         *  been pressed.
+         *  A promise that will be resolved with the action identifier, if any
+         *  action button (OK button, alternative buttons) has been pressed; or
+         *  rejected, if the Cancel button has been pressed. The action
+         *  identifier of the OK button is the string 'ok'.
          */
-        this.show = (function () {
-            var show = self.show;
-            return function () {
-                // work-around the weird behavior of the core dialog in asynchronous mode:
-                // - the promise returned by show() will not resolve/reject when triggering an action
-                // - the generic 'action' event does not send the action identifier
-                // solution is to process all dialog events and look for an action button to resolve/reject the own promise
-                var def = $.Deferred();
-                function actionHandler(event, action) {
-                    if (action === 'close') {
-                        // bug 32244: handle explicit "dialog.close()" calls
-                        self.close = $.noop; // prevent recursive calls from promise callbacks
-                        def.reject(action, self);
-                    } else if (self.getButton(action).length > 0) {
-                        def[(action === 'cancel') ? 'reject' : 'resolve'](action, self);
+        this.show = _.wrap(this.show, function (show, callback) {
+
+            // work-around the weird behavior of the core dialog in asynchronous mode:
+            // - the promise returned by show() will not resolve/reject when triggering an action
+            // - the generic 'action' event does not send the action identifier
+            // solution is to process all dialog events and look for an action button to resolve/reject the own promise
+            var def = $.Deferred();
+
+            // event handler for all events triggered by this dialog, resolves/rejects
+            // the own deferred object after activating an action button
+            function actionHandler(event, action) {
+
+                // bug 32244: handle explicit "dialog.close()" calls
+                if (action === 'close') {
+                    self.close = $.noop; // prevent recursive calls from promise callbacks
+                    def.reject(action, self);
+                    return;
+                }
+
+                // check that a button exists for the action identifier
+                if (self.getButton(action).length === 0) { return; }
+
+                // reject the promise on Cancel
+                if (action === 'cancel') {
+                    def.reject(action);
+                    return;
+                }
+
+                // invoke the validator callback function
+                var result = validator.call(self, action);
+                if (_.isObject(result) && _.isFunction(result.promise)) {
+                    result.done(function () { def.resolve(action); }).fail(function () { def.reject('invalid'); });
+                } else if (result === true) {
+                    def.resolve(action);
+                } else {
+                    def.reject('invalid');
+                }
+            }
+
+            // Clicking on the overlay node moves focus to document body. Add handlers
+            // for the TAB key and ESCAPE key to be able to return to the dialog, or
+            // to close it even without having the focus inside the dialog.
+            function globalKeyDownHandler(event) {
+                if (document.activeElement === document.body) {
+                    if (KeyCodes.matchKeyCode(event, 'TAB', { shift: null })) {
+                        Forms.grabFocus(self.getPopup());
+                        event.preventDefault();
+                    } else if (KeyCodes.matchKeyCode(event, 'ESCAPE')) {
+                        self.close();
                     }
                 }
-                modalDialogs += 1;
-                self.on('triggered', actionHandler);
-                def.always(function () { modalDialogs -= 1; self.off('triggered', actionHandler); self = null; });
-                // show the dialog, ignore the promise it returns
-                show.apply(self, arguments);
-                // return the own promise that will always be resolved/rejected
-                // (dialog remains still open in asynchronous mode)
-                return def.promise();
-            };
-        }());
+            }
+
+            // Bug 41397: When focus leaves a node with browser selection (e.g. a clipboard
+            // node, or a content-editable node), the browser selection MUST be destroyed.
+            // Otherwise, all keyboard events bubbling to the document body will cause to
+            // focus that previous element having the browser selection regardless whether
+            // it is actually focused. This may cause for example to be able to edit a
+            // document while a modal dialog is open.
+            Utils.clearBrowserSelection();
+
+            // register event handlers for the dialog lifetime
+            this.on('triggered', actionHandler);
+            $(document).on('keydown', globalKeyDownHandler);
+
+            // clean-up after closing the dialog
+            def.always(function () {
+                self.off('triggered', actionHandler);
+                $(document).off('keydown', globalKeyDownHandler);
+                self = null;
+            });
+
+            // show the dialog, but ignore the promise it returns
+            show.call(this, callback);
+
+            // return the own promise that will always be resolved/rejected
+            // (dialog remains still open in asynchronous mode)
+            return def.promise();
+        });
 
         // initialization -----------------------------------------------------
 
         // add own CSS marker class
         this.getPopup().addClass('io-ox-office-dialog');
-        Utils.addDeviceMarkers(this.getPopup());
+        Forms.addDeviceMarkers(this.getPopup());
 
         // add title
         if (_.isString(title)) {
             this.header($('<h4>', { id: 'dialog-title' }).text(title));
         }
 
-        // add 'OK' button
-        if (_.isString(okLabel)) {
-            this.addPrimaryButton('ok', okLabel, 'ok', { tabIndex: 1 });
-        }
+        // override all 'addSomeButton()' methods: set 'dataaction' parameter to 'action', add tabIndex option
+        ['addButton', 'addDangerButton', 'addSuccessButton', 'addWarningButton', 'addPrimaryButton', 'addAlternativeButton'].forEach(function (methodName) {
+            this[methodName] = _.wrap(this[methodName], function (baseMethod, action, label, options) {
+                return baseMethod.call(this, action, label, action, _.extend({ tabIndex: 1 }, options));
+            });
+        }, this);
 
-        // add 'Cancel' button
-        if (_.isString(cancelLabel)) {
-            this.addButton('cancel', cancelLabel, 'cancel', { tabIndex: 1 });
-        }
+        // add the OK button
+        this.addPrimaryButton('ok', okLabel);
+
+        // add the Cancel button
+        this.addButton('cancel', cancelLabel);
 
         // listen to key events for additional keyboard handling
         this.getBody().on('keydown', keyDownHandler);
@@ -339,7 +450,7 @@ define('io.ox/office/tk/dialogs', [
          * panel.
          */
         function focusActiveTabButton() {
-           tabBarNode.find('li.active a').first().focus();
+            tabBarNode.find('li.active a').first().focus();
         }
 
         /**
@@ -509,7 +620,20 @@ define('io.ox/office/tk/dialogs', [
     // class ModalQueryDialog =================================================
 
     /**
-     * Creates a simple dialog with a message text and 'Yes'/'No' buttons.
+     * Creates a simple dialog with a message text and Yes and No buttons.
+     * Optionally, the dialog may contain an additional Cancel button next to
+     * the Yes and No buttons.
+     *
+     * By default, the promise returned by the public method show() of an
+     * instance of this dialog will be resolved with the boolean value true
+     * when pressing the Yes button, and rejected when pressing the No button.
+     *
+     * If the dialog has been opened with an additional Cancel button (see
+     * constructor option 'cancel' below), the promise returned by the public
+     * method show() will behave differently: When pressing the Yes button, the
+     * promise will be resolved with the boolean value true. When pressing the
+     * No button, the promise will be resolved with the boolean value false.
+     * Otherwise (dialog canceled), the promise will be rejected.
      *
      * @constructor
      *
@@ -524,20 +648,61 @@ define('io.ox/office/tk/dialogs', [
      *  ModalDialog. The default value of the option 'okLabel' will be changed
      *  to the translated string 'Yes', and the default value of the option
      *  'cancelLabel' will be changed to the translated string 'No'.
+     *  Additionally, the following options are supported:
+     *  @param {Boolean} [initOptions.cancel=false]
+     *      Whether to show an additional Cancel button next to the Yes and No
+     *      buttons. See comments above about the behavioral changes for this
+     *      case.
      */
     Dialogs.ModalQueryDialog = Dialogs.ModalDialog.extend({ constructor: function (message, initOptions) {
 
+        var // whether to show an additional Cancel button
+            cancelMode = Utils.getBooleanOption(initOptions, 'cancel', false);
+
         // base constructor ---------------------------------------------------
 
         Dialogs.ModalDialog.call(this, Utils.extendOptions({
-            okLabel: gt('Yes'),
-            cancelLabel: gt('No')
-        }, initOptions));
+            width: 400
+        }, initOptions, {
+            okLabel: YES_LABEL,
+            cancelLabel: cancelMode ? null : NO_LABEL
+        }));
+
+        // public methods -----------------------------------------------------
+
+        /**
+         * Shows this dialog.
+         *
+         * @attention
+         *  The dialog can only be shown once. It will destroy itself after it
+         *  has been closed.
+         *
+         * @param {Function} [callback]
+         *  A callback function that will be invoked immediately after showing
+         *  the dialog.
+         *
+         * @returns {jQuery.Promise}
+         *  A promise that will be resolved with a boolean value. See the
+         *  description of the class constructor for details.
+         */
+        this.show = _.wrap(this.show, function (show, callback) {
+            return show.call(this, callback).then(function (action) { return action === 'ok'; });
+        });
 
         // initialization -----------------------------------------------------
 
         this.append($('<label>').text(message));
 
+        // add an own No button in Yes/No/Cancel mode
+        if (cancelMode) {
+            this.addButton('no', NO_LABEL);
+            // there is no better way to position a button
+            this.getButton('no').insertBefore(this.getOkButton());
+        }
+
+        // focus the OK button when showing the dialog
+        this.on('show', function () { this.getOkButton().focus(); });
+
     }}); // class ModalQueryDialog
 
     // class ModalInputDialog =================================================
@@ -553,6 +718,8 @@ define('io.ox/office/tk/dialogs', [
      *  Optional parameters that control the appearance and behavior of the
      *  dialog. Supports all options also supported by the base class
      *  ModalDialog. Additionally, the following options are supported:
+     *  @param {String} [initOptions.inputLabel]
+     *      A leading label for the input field.
      *  @param {String} [initOptions.value='']
      *      The initial value of the text field.
      *  @param {String} [initOptions.placeholder='']
@@ -561,20 +728,31 @@ define('io.ox/office/tk/dialogs', [
      *      If set to true, empty text will be considered valid. Otherwise, the
      *      primary button will be disabled as long as the text input field is
      *      empty.
+     *  @param {Boolean} [initOptions.maxLength]
+     *      If specified, the maximum number of characters that can be entered.
      */
     Dialogs.ModalInputDialog = Dialogs.ModalDialog.extend({ constructor: function (initOptions) {
 
         var // self reference
             self = this,
 
+            // maximum length allowed in the text field
+            maxLength = Utils.getIntegerOption(initOptions, 'maxLength', null, 1),
+
+            // whether to allow empty text
+            allowEmpty = Utils.getBooleanOption(initOptions, 'allowEmpty', false),
+
+            // the label text
+            inputLabel = Utils.getStringOption(initOptions, 'inputLabel', ''),
+
+            // the text label element
+            labelNode = inputLabel ? $('<label>').text(inputLabel) : $(),
+
             // the text field control
             inputNode = $('<input type="text" class="form-control" tabindex="1">').attr({
                 placeholder: Utils.getStringOption(initOptions, 'placeholder', ''),
                 value: Utils.getStringOption(initOptions, 'value', '')
-            }),
-
-            // whether to allow empty text
-            allowEmpty = Utils.getBooleanOption(initOptions, 'allowEmpty', false);
+            });
 
         // base constructor ---------------------------------------------------
 
@@ -592,6 +770,16 @@ define('io.ox/office/tk/dialogs', [
 
         // methods ------------------------------------------------------------
 
+        /**
+         * Returns the text input element, as jQuery object.
+         *
+         * @returns {jQuery}
+         *  The text input element, as jQuery object.
+         */
+        this.getInputNode = function () {
+            return inputNode;
+        };
+
         /**
          * Returns the text that is contained in the text input element.
          *
@@ -619,20 +807,153 @@ define('io.ox/office/tk/dialogs', [
         // initialization -----------------------------------------------------
 
         // add the text field
-        this.append(inputNode);
+        this.append(labelNode, inputNode);
+
+        // initialize maximum length
+        if (_.isNumber(maxLength)) {
+            inputNode.val(inputNode.val().substr(0, maxLength));
+            inputNode.attr('maxlength', maxLength);
+        }
 
         // enable/disable the primary button
         inputNode.on('input', updateOkButton);
         updateOkButton();
 
         // focus and select the text in the input field
-        this.on({
-            show: function () { inputNode.focus().select(); },
-            close: function () { inputNode.remove(); }
+        this.on('show', function () {
+            // us 102078426: dialogs on small devices should come up w/o keyboard,
+            // so the focus shoud not be in an input element
+            if (Utils.SMALL_DEVICE) {
+                self.getOkButton().focus();
+            } else {
+                inputNode.focus().select();
+            }
         });
 
     }}); // class ModalInputDialog
 
+    // class ModalDriveDialog =================================================
+
+    /**
+     * A custom dialog for Drive Utils
+     * DriveUtils are assigned as "driveUtil", you can take FolderPicker,
+     * FlatFolderPicker & FilePicker
+     *
+     * @extends ModalDialog
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters that control the appearance and behavior of the
+     *  dialog. Supports all options also supported by the base class
+     *  ModalDialog. Additionally, the following options are supported:
+     *  @param {String} [initOptions.preselect]
+     *      Drive-File-Id to preselect a folder
+     *  @param {String} [initOptions.placeholder='']
+     *      The place-holder text that will be shown in the empty text field.
+     *  @param {Object} [initOptions.driveUtil]
+     *      the assigned DriveUtil from the DriveUtils class
+     *      can be: FolderPicker, FlatFolderPicker & FilePicker
+     */
+    Dialogs.ModalDriveDialog = Dialogs.ModalDialog.extend({ constructor: function (initOptions) {
+
+        var self = this;
+
+        var driveUtil = Utils.getOption(initOptions, 'driveUtil');
+
+        // base constructor ---------------------------------------------------
+
+        Dialogs.ModalDialog.call(this, initOptions);
+
+        // public methods -----------------------------------------------------
+
+        /**
+         * @returns {Object}
+         *  returns the current file-selection of the DriveUtil
+         *  file can have folder_id (parentId), id (file-id) & filename,
+         *  but all are optional
+         */
+        this.getSelectedFile = function () {
+            return driveUtil.getSelectedFile();
+        };
+
+        // initialization -----------------------------------------------------
+
+        this.getPopup().find('.modal-body').append(driveUtil.getNode());
+
+        driveUtil.on('change', function () {
+            self.trigger('change');
+        });
+
+    }}); // class ModalDriveDialog
+
+    // class ModalFolderPicker ================================================
+
+    /**
+     * A dialog with DriveUtils' FolderPicker
+     *
+     * @constructor
+     *
+     * @extends ModalDriveDialog
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters that control the appearance and behavior of the
+     *  dialog. Supports all options also supported by the base class
+     *  ModalDriveDialog. Additionally, the following options are supported:
+     *  @param {String} [initOptions.preselect]
+     *      Drive-File-Id to preselect a folder
+     */
+    Dialogs.ModalFolderPicker = Dialogs.ModalDriveDialog.extend({ constructor: function (initOptions) {
+
+        var driveUtil = new DriveUtils.FolderPicker(initOptions);
+
+        // base constructor ---------------------------------------------------
+
+        Dialogs.ModalDriveDialog.call(this, Utils.extendOptions({
+            driveUtil: driveUtil
+        }, initOptions));
+
+        // public methods -----------------------------------------------------
+
+        /**
+         * @return {String}
+         *  returns the current folder_id
+         */
+        this.getSelectedFolderId = function () {
+            var selectedFile = this.getSelectedFile();
+            if (selectedFile) {
+                return selectedFile.folder_id;
+            }
+        };
+
+    }}); // class ModalFolderPicker
+
+    // class ModalFilePicker ==================================================
+
+    /**
+     * A dialog with DriveUtils' FilePicker
+     *
+     * @constructor
+     *
+     * @extends ModalDriveDialog
+     *
+     * @param {Object} [initOptions]
+     *  Optional parameters that control the appearance and behavior of the
+     *  dialog. Supports all options also supported by the base class
+     *  ModalDriveDialog. Additionally, the following options are supported:
+     *  @param {String} [initOptions.preselect]
+     *      Drive-File-Id to preselect a folder
+     */
+    Dialogs.ModalFilePicker = Dialogs.ModalDriveDialog.extend({ constructor: function (initOptions) {
+
+        var driveUtil = new DriveUtils.FilePicker(initOptions);
+
+        // base constructor ---------------------------------------------------
+
+        Dialogs.ModalDriveDialog.call(this, Utils.extendOptions({
+            driveUtil: driveUtil
+        }, initOptions));
+
+    }}); // class ModalFilePicker
+
     // exports ================================================================
 
     return Dialogs;
