@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -14,12 +14,16 @@
 define('io.ox/office/tk/forms', [
     'io.ox/office/tk/config',
     'io.ox/office/tk/utils',
-    'io.ox/office/tk/keycodes'
+    'io.ox/office/tk/keycodes',
+    'less!io.ox/office/tk/style'
 ], function (Config, Utils, KeyCodes) {
 
     'use strict';
 
-    var // CSS class name for caption elements
+    var // CSS classes added to localized bitmap icons
+        DOCS_ICON_CLASSES = 'lc-' + Config.LOCALE + ' lc-' + Config.LANGUAGE + (Utils.RETINA ? ' retina' : ''),
+
+        // CSS class name for caption elements
         CAPTION_CLASS = 'caption',
 
         // CSS selector for caption elements in control nodes
@@ -94,16 +98,6 @@ define('io.ox/office/tk/forms', [
         return styleMarkup;
     }
 
-    /**
-     * Converts the passed string to the mark-up of a 'title' element
-     * attribute, together with the according 'aria-label' attribute.
-     */
-    function createToolTipMarkup(tooltip) {
-        if (!_.isString(tooltip) || (tooltip.length === 0)) { return ''; }
-        tooltip = Utils.escapeHTML(tooltip);
-        return ' title="' + tooltip + '" aria-label="' + tooltip + '"';
-    }
-
     // static class Forms =====================================================
 
     var // the exported Forms class
@@ -178,6 +172,20 @@ define('io.ox/office/tk/forms', [
      */
     Forms.FOCUSABLE_SELECTOR = '[tabindex]:not([tabindex^="-"]):visible';
 
+    /**
+     * CSS class for title elements.
+     *
+     * @constant
+     */
+    Forms.TITLE_CLASS = 'title';
+
+    /**
+     * CSS selector for title elements.
+     *
+     * @constant
+     */
+    Forms.TITLE_SELECTOR = '.' + Forms.TITLE_CLASS;
+
     /**
      * CSS class for button elements.
      *
@@ -213,6 +221,22 @@ define('io.ox/office/tk/forms', [
      */
     Forms.DEFAULT_CLICK_TYPE = Utils.TOUCHDEVICE ? 'tap' : 'click';
 
+    /**
+     * CSS class for application bound tooltip.
+     * (Events will be handled by BaseApplication & AppTooltipMixin)
+     *
+     * @constant
+     */
+    Forms.APP_TOOLTIP_CLASS = 'app-tooltip';
+
+    /**
+     * CSS selector for application bound tooltip.
+     * (Events will be handled by BaseApplication)
+     *
+     * @constant
+     */
+    Forms.APP_TOOLTIP_SELECTOR = '.' + Forms.APP_TOOLTIP_CLASS;
+
     // generic helpers --------------------------------------------------------
 
     /**
@@ -227,17 +251,13 @@ define('io.ox/office/tk/forms', [
      *      The element attributes to be inserted into the DOM element. The
      *      attribute names can be given in camel-case notation. MUST NOT
      *      contain the attribute 'style' if the option 'style' (see below) is
-     *      in use. MUST NOT contain the attribute 'title' if the option
-     *      'tooltip' (see below) is in use.
+     *      in use.
      *  @param {String|Object} [options.style]
      *      Additional CSS style attributes for the control element. Will be
      *      inserted into the 'style' attribute of the element. Can be a string
      *      that will be inserted into the 'style' attribute as is, or can be
      *      an object that maps CSS properties to their values. The property
      *      names can be given in camel-case notation.
-     *  @param {String} [options.tooltip]
-     *      Tool tip text shown when the mouse hovers the DOM element. If
-     *      omitted, the element will not show a tool tip.
      *  @param {String} [options.content='']
      *      The HTML mark-up of the DOM element's contents.
      *
@@ -256,29 +276,104 @@ define('io.ox/office/tk/forms', [
 
         // add CSS styles and tool tip
         markup += createStyleMarkup(Utils.getOption(options, 'style'));
-        markup += createToolTipMarkup(Utils.getOption(options, 'tooltip'));
 
         // add inner HTML mark-up and close the element
         markup += '>' + Utils.getStringOption(options, 'content', '') + '</' + nodeName + '>';
         return markup;
     };
 
+    /**
+     * Adds device-specific CSS marker classes to the specified element. The
+     * following CSS classes will be added:
+     * - 'touch', if the device has a touch surface.
+     * - 'browser-firefox', if the current browser is Firefox.
+     * - 'browser-webkit', if the current browser is based on WebKit.
+     * - 'browser-chrome', if the current browser is Chrome.
+     * - 'browser-safari', if the current browser is Safari.
+     * - 'browser-ie', if the current browser is Internet Explorer.
+     *
+     * @param {HTMLElement|jQuery} node
+     *  A single DOM element, or a jQuery collection with one or more elements.
+     */
+    Forms.addDeviceMarkers = function (node) {
+
+        // convert to jQuery collection
+        node = $(node);
+
+        // marker for touch devices
+        node.toggleClass('touch', Utils.TOUCHDEVICE);
+
+        // marker for browser types
+        node.toggleClass('browser-firefox', !!_.browser.Firefox)
+            .toggleClass('browser-webkit', !!_.browser.WebKit)
+            .toggleClass('browser-chrome', !!_.browser.Chrome)
+            .toggleClass('browser-safari', !!_.browser.Safari)
+            .toggleClass('browser-ie', !!_.browser.IE);
+    };
+
     /**
      * Adds a tool tip to the specified elements.
      *
-     * @param {HTMLElement|jQuery} nodes
+     * @param {HTMLElement|jQuery} node
      *  The elements to be manipulated. If this object is a jQuery collection,
      *  sets the tool tip for all nodes it contains.
      *
-     * @param {String} [tooltip]
-     *  Tool tip text shown when the mouse hovers one of the nodes. If omitted,
-     *  the nodes will not show a tool tip anymore.
+     * @param {Object} [options]
+     *  Optional parameters:
+     *  @param {String} [options.tooltip='']
+     *      Tool tip text shown when the mouse hovers one of the nodes. If
+     *      omitted, the nodes will not show a tool tip anymore.
+     *  @param {Boolean} [options.forceTouch=false]
+     *      If set to true, the tooltip is set also on touch devices.
      */
-    Forms.setToolTip = function (nodes, tooltip) {
-        if (_.isString(tooltip) && tooltip) {
-            $(nodes).attr({ title: tooltip, 'aria-label': tooltip });
+    Forms.setToolTip = function (node, options) {
+
+        var tooltip = Utils.getStringOption(options, 'tooltip', ''),
+            forceTouch = null;
+
+        node = $(node);
+
+        if (!_.isString(tooltip) || (tooltip.length === 0)) {
+            node.removeAttr('data-original-title title aria-label');
+            Forms.disableToolTip(node);
         } else {
-            $(nodes).removeAttr('title aria-label');
+            forceTouch = Utils.getBooleanOption(options, 'forceTouch', false);
+
+            if (Utils.TOUCHDEVICE && !forceTouch) {
+                node.attr({ title: tooltip });
+            } else {
+                node.attr({
+                    'aria-label': tooltip,
+                    'data-original-title': tooltip
+                });
+                Forms.enableToolTip(node);
+            }
+        }
+    };
+
+    /**
+     * disables the tooltip for assigned node if node is inited as tooltip (Froms.setToolTip)
+     *
+     * @param {HTMLElement|jQuery} node
+     *  The elements to be manipulated. If this object is a jQuery collection,
+     *  changes the tool tip states for all nodes it contains.
+     */
+    Forms.disableToolTip = function (node) {
+        if (!Utils.TOUCHDEVICE) {
+            node.removeClass(Forms.APP_TOOLTIP_CLASS);
+        }
+    };
+
+    /**
+     * enables the tooltip for assigned node if node is inited as tooltip (Froms.setToolTip)
+     *
+     * @param {HTMLElement|jQuery} node
+     *  The elements to be manipulated. If this object is a jQuery collection,
+     *  changes the tool tip states for all nodes it contains.
+     */
+    Forms.enableToolTip = function (node) {
+        if (!Utils.TOUCHDEVICE) {
+            node.addClass(Forms.APP_TOOLTIP_CLASS);
         }
     };
 
@@ -657,13 +752,13 @@ define('io.ox/office/tk/forms', [
             focusableNodes = _.filter(focusableNodes, function (nodePosition) {
                 switch (direction) {
                 case 'left':
-                    return nodePosition.left + nodePosition.width <= focusPosition.left;
+                    return nodePosition.left + nodePosition.width <= focusPosition.left + 1; // add 1 pixel to avoid browser rounding effects
                 case 'right':
-                    return nodePosition.right + nodePosition.width <= focusPosition.right;
+                    return nodePosition.right + nodePosition.width <= focusPosition.right + 1;
                 case 'up':
-                    return nodePosition.top + nodePosition.height <= focusPosition.top;
+                    return nodePosition.top + nodePosition.height <= focusPosition.top + 1;
                 case 'down':
-                    return nodePosition.bottom + nodePosition.height <= focusPosition.bottom;
+                    return nodePosition.bottom + nodePosition.height <= focusPosition.bottom + 1;
                 }
             });
 
@@ -753,23 +848,9 @@ define('io.ox/office/tk/forms', [
      * @returns {String}
      *  The complete CSS classes for the icon element.
      */
-    Forms.getIconClasses = (function () {
-
-        var // the CSS classes added to localized bitmap icons
-            docsIconClasses = ' lc-' + Config.LOCALE;
-
-        if ((Config.LANGUAGE !== '') && (Config.COUNTRY !== '')) {
-            docsIconClasses += ' lc-' + Config.LANGUAGE;
-        }
-
-        if (Utils.RETINA) {
-            docsIconClasses += ' retina';
-        }
-
-        return function (icon) {
-            return (/^fa-/).test(icon) ? ('fa ' + icon) : (/^docs-/).test(icon) ? (icon + docsIconClasses) : icon;
-        };
-    }());
+    Forms.getIconClasses = function (icon) {
+        return (/^fa-/).test(icon) ? ('fa ' + icon) : (/^docs-/).test(icon) ? (icon + ' ' + DOCS_ICON_CLASSES) : icon;
+    };
 
     /**
      * Creates the HTML mark-up of an <i> DOM element representing an icon.
@@ -1238,6 +1319,8 @@ define('io.ox/office/tk/forms', [
             focusable = Utils.getBooleanOption(options, 'focusable', true),
             captionMarkup = Forms.createCaptionMarkup(options),
             contentMarkup = Utils.getStringOption(options, 'content', ''),
+            href = Utils.getStringOption(options, 'href', ''),
+            target = Utils.getStringOption(options, 'target', '_blank'),
             ariaOwns = Utils.getStringOption(options, 'aria-owns');
 
         // add more element attributes, return the mark-up
@@ -1247,6 +1330,17 @@ define('io.ox/office/tk/forms', [
         if (classes.length > 0) { attributes['class'] += ' ' + classes; }
         if (ariaOwns) {attributes.ariaOwns = ariaOwns; }
 
+        if (href.length > 0) {
+            attributes.href = href;
+            attributes.target = target;
+            // bug #47916# prevent content spoofing
+            // see: https://mathiasbynens.github.io/rel-noopener/
+            // and: https://github.com/danielstjules/blankshield
+            if (attributes.target === '_blank') {
+                attributes.rel = 'noopener';
+            }
+        }
+
         return Forms.createElementMarkup(nodeName, Utils.extendOptions(options, { attributes: attributes, content: captionMarkup + contentMarkup }));
     };
 
@@ -1496,7 +1590,9 @@ define('io.ox/office/tk/forms', [
     Forms.setButtonKeyHandler = function (nodes, options) {
 
         var // whether to listen to the TAB key
-            tab = Utils.getBooleanOption(options, 'tab', false);
+            tab = Utils.getBooleanOption(options, 'tab', false),
+            // target node of the last caught 'keydown' event
+            lastKeyDownTarget = null;
 
         // Bug 28528: ENTER key must be handled explicitly, <a> elements without
         // 'href' attribute do not trigger click events. The 'href' attribute
@@ -1506,9 +1602,26 @@ define('io.ox/office/tk/forms', [
         // handler for the 'keyup' events
         function keyHandler(event) {
 
+            var // whether the target of a 'keyup' event matches the target of preceding 'keydown'
+                isKeyUpWithMatchingTarget = false;
+
+            // Bug 37870: Remember target node of a 'keydown' event, check that 'keyup' event is for the
+            // same target. Needed to prevent triggering a click event when pressing ENTER one one node,
+            // the changing the browser focus to one of the nodes covered by this method, then receiving
+            // the 'keyup' event. This happens for example when pressing the ENTER key on the OK button
+            // of a modal dialog. The 'keydown' event causes to close the dialog which restores the old
+            // focus node, e.g. a button in a tool bar. The following 'keyup' event for the ENTER key
+            // would immediately trigger that button which will open the modal dialog again.
+            if (event.type === 'keydown') {
+                lastKeyDownTarget = event.target;
+            } else if (event.type === 'keyup') {
+                isKeyUpWithMatchingTarget = lastKeyDownTarget === event.target;
+                lastKeyDownTarget = null;
+            }
+
             // ENTER and SPACE key: wait for keyup event
             if ((event.keyCode === KeyCodes.ENTER) || (event.keyCode === KeyCodes.SPACE)) {
-                if (event.type === 'keyup') {
+                if (isKeyUpWithMatchingTarget) {
                     Forms.triggerClickForKey(event);
                 }
                 return false; // stop propagation of ALL keyboard events
@@ -1526,6 +1639,25 @@ define('io.ox/office/tk/forms', [
         $(nodes).not(Forms.BUTTON_SELECTOR).on('keydown keypress keyup', Forms.BUTTON_SELECTOR, keyHandler);
     };
 
+    // native (Bootstrap) button elements -------------------------------------
+
+    /**
+     * Enables or disables a simple Bootstrap <button> element.
+     *
+     * @param {HTMLElement|jQuery} buttonNode
+     *  The button elements to be manipulated.
+     *
+     * @param {Boolean} state
+     *  Whether to enable (true) or disable (false) the button elements.
+     */
+    Forms.enableBSButton = function (buttonNode, state) {
+        if (state) {
+            buttonNode.removeAttr('disabled').removeAttr('aria-disabled');
+        } else {
+            buttonNode.attr({ disabled: 'disabled', 'aria-disabled': true });
+        }
+    };
+
     // text input elements ----------------------------------------------------
 
     /**
@@ -1554,13 +1686,14 @@ define('io.ox/office/tk/forms', [
      */
     Forms.createInputMarkup = function (options) {
 
-        var type = Modernizr.touch ? Utils.getStringOption(options, 'keyboard', 'text') : 'text',
+        var type = Utils.TOUCHDEVICE ? Utils.getStringOption(options, 'keyboard', 'text') : 'text',
             attributes = Utils.getObjectOption(options, 'attributes', {}),
+            tabIndex = Utils.getIntegerOption(options, 'tabindex', 1),
             placeHolder = Utils.getStringOption(options, 'placeholder'),
             maxLength = Utils.getIntegerOption(options, 'maxLength');
 
         // add more element attributes, return the mark-up
-        attributes = _.extend({ type: type, tabindex: 1 }, attributes);
+        attributes = _.extend({ type: type, tabindex: tabIndex }, attributes);
         if (_.isString(placeHolder) && (placeHolder.length > 0)) { attributes.placeholder = placeHolder; }
         if (_.isNumber(maxLength) && (maxLength > 0)) { attributes.maxlength = maxLength; }
         return Forms.createElementMarkup('input', Utils.extendOptions(options, { attributes: attributes, content: null }));
@@ -1586,11 +1719,12 @@ define('io.ox/office/tk/forms', [
     Forms.createTextAreaMarkup = function (options) {
 
         var attributes = Utils.getObjectOption(options, 'attributes', {}),
+            tabIndex = Utils.getIntegerOption(options, 'tabindex', 1),
             placeHolder = Utils.getStringOption(options, 'placeholder'),
             maxLength = Utils.getIntegerOption(options, 'maxLength');
 
         // add more element attributes, return the mark-up
-        attributes = _.extend({ tabindex: 1 }, attributes);
+        attributes = _.extend({ tabindex: tabIndex }, attributes);
         if (_.isString(placeHolder) && (placeHolder.length > 0)) { attributes.placeholder = placeHolder; }
         if (_.isNumber(maxLength) && (maxLength > 0)) { attributes.maxlength = maxLength; }
         return Forms.createElementMarkup('textarea', Utils.extendOptions(options, { attributes: attributes, content: null }));
@@ -1662,6 +1796,290 @@ define('io.ox/office/tk/forms', [
         node.setSelectionRange(start + text.length, start + text.length);
     };
 
+    // simple or bootstrap elements -------------------------------------------
+
+    /**
+     * Creates a DropDown-Menu from/for the bootstrap-framework
+     *
+     * * @param {Array} arrEntries
+     *  An array with objects. Each
+     *  descriptor is an object with the following properties:
+     *  - {String} title
+     *      The text label for the list item.
+     *
+     * @param {Object} [options]
+     *  Optional parameters. The following options are
+     *  supported:
+     *  @param {String} [options.btnText='']
+     *      The button text which is visible on the dropdown-opener-button
+     *  @param {Boolean} [options.withCaret=true]
+     *      Show a caret besides the buttontext
+     *  @param {String} [options.classes='']
+     *      Additionally style classes for the surrounding div.
+     *  @param {String} [options.dropDirection='down']
+     *      The direction in which the caret shows and the popup will open
+     *
+     * @return {String}
+     *   String of the complete bootstrap dropdown-element
+     */
+    Forms.createDropDownMarkup = function (arrEntries, options) {
+
+        var dropDirection = Utils.getStringOption(options, 'dropDirection', 'down'),
+            classes = Utils.getStringOption(options, 'classes', '') + ((dropDirection === 'down') ? ' dropdown' : ' dropup'),
+
+            // caret
+            caret       = Utils.getBooleanOption(options, 'withCaret', true) ? Forms.createElementMarkup('span', { attributes: { class: 'caret' } }) : '',
+
+            // button
+            btnText     = Utils.getStringOption(options, 'btnText', ''),
+            btnNode     = Forms.createElementMarkup('button', { attributes: { 'data-toggle': 'dropdown', 'class': 'dropdown-toggle' }, content: btnText + caret }),
+
+            // list
+            listNode    = null,
+            listNodes   = null;
+
+        // add list-entries
+        listNodes = '';
+        _.each(arrEntries, function (entry) {
+            var title = Utils.getStringOption(entry, 'title', '[no-title]'),
+                link = Forms.createElementMarkup('a', { attributes: { role: 'menuitem', tabindex: '-1' }, content: title });
+
+            listNodes += Forms.createElementMarkup('li', { attributes: { role: 'presentation' }, content: link });
+        });
+
+        listNode = Forms.createElementMarkup('ul', { attributes: { role: 'menu', 'class': 'dropdown-menu' }, content: listNodes });
+
+        return Forms.createElementMarkup('div', { attributes: { 'class': classes }, content: btnNode + listNode });
+    };
+
+    /**
+     * Creates a simple <select> list control with an optional leading label.
+     *
+     * @param {Array} entries
+     *  An array with descriptor objects for all list entries. Each object MUST
+     *  contain the following properties:
+     *  - {Any} value
+     *      The value associated to the list item.
+     *  - {String} label
+     *      The text label for the list item.
+     *
+     * @param {Object} [options]
+     *  Optional parameters:
+     *  @param {String} [options.classes='']
+     *      Additionally style classes for the surrounding div.
+     *  @param {String} [options.name='']
+     *      The name of the <select> element.
+     *  @param {String} [options.label='']
+     *      The leading label for the <select> element. If empty, no label
+     *      element will be created.
+     *  @param {Number} [options.size=0]
+     *      The size of the <select> element (text lines). If omitted or set to
+     *      the value 0, the list will appear as drop-down list.
+     *  @param {Boolean} [options.multipleSelection=false]
+     *      Whether to make multiple entries selectable or only one.
+     *
+     * @return {String}
+     *   The HTML mark-up of the complete <select> element group.
+     */
+    Forms.createSelectBoxMarkup = function (entries, options) {
+
+        var // classes, which will be added to the surrounding div-element
+            classes = Utils.trimString(Utils.getStringOption(options, 'classes', '') + ' select-box'),
+
+            // the label text
+            textLabel   = Utils.getStringOption(options, 'label', false),
+            // the cleaned/trimed label text for attributes like 'id', 'name', ...
+            cleanLabel  = textLabel ? Utils.trimAndCleanString(textLabel.toLowerCase().replace(/\s+/g, '')) : false,
+
+            // flag to decide whether a multiple selection should be allowed
+            multiple    = Utils.getBooleanOption(options, 'multipleSelection', false) ? 'multiple' : '',
+            // the name of the selectBox
+            name        = Utils.getStringOption(options, 'name', false) || cleanLabel,
+
+            // helper
+            labelNode   = '',
+            listNode    = '',
+            listNodes   = null,
+
+            // object which contains all selectBox attributes
+            selectAttributes = {
+                size: Utils.getNumberOption(options, 'size', 0),
+                tabindex: Utils.getIntegerOption(options, 'tabindex', 1)
+            };
+
+        // collect list-elements
+        listNodes = '';
+        entries.forEach(function (entry) {
+            var value = Utils.getStringOption(entry, 'value', false),
+                label = Utils.getStringOption(entry, 'label', false);
+
+            if (label && value) {
+                listNodes += Forms.createElementMarkup('option', { attributes: { value: value }, content: label });
+            }
+        });
+
+        // add list-elements to select-node
+        if (cleanLabel) { selectAttributes.id = cleanLabel; }
+        if (multiple) { selectAttributes.multiple = ''; classes += ' multiple'; }
+        if (name) { selectAttributes.name = name.toLowerCase().replace(' ', ''); }
+        listNode = Forms.createElementMarkup('select', { attributes: selectAttributes, content: listNodes });
+
+        // create labelNode
+        if (textLabel) { labelNode = Forms.createElementMarkup('label', { attributes: { 'for': cleanLabel }, content: textLabel }); }
+
+        // add label + list to container and return it
+        return Forms.createElementMarkup('div', { attributes: { 'class': classes }, content: labelNode + listNode });
+    };
+
+    /**
+     * Creates a simple SpinField
+     *   with leading and/or tailing label
+     *
+     * @param {Object} [options]
+     *  Optional parameters. The following options are
+     *  supported:
+     *  @param {Float} options.value
+     *      Spin default value
+     *  @param {String} [options.classes='']
+     *      Additionally style classes for the surrounding div.
+     *  @param {String} [options.label='']
+     *      The label for the spinField. If empty, no label element will be
+     *      created
+     *  @param {Boolean} [options.tail=false]
+     *      The tailing label
+     *  @param {Float} [options.min=0.0]
+     *      Spin min-value
+     *  @param {Float} [options.max=10.0]
+     *      Spin max-value
+     *  @param {Float} [options.step=0.1]
+     *      Spin step-value
+     *  @param {Boolean} [options.required=false]
+     *      Is infill required
+     *  @param {String} [options.pattern]
+     *      The validation pattern
+     *
+     * @return {String}
+     *   String of the complete spinField-element
+     */
+    Forms.createSpinFieldMarkup = function (options) {
+
+        var // classes, which will be added to the surrounding div-element
+            classes = Utils.trimString(Utils.getStringOption(options, 'classes', '') + ' spin-field'),
+
+            // the leading label
+            labelNode = '',
+            // the label text
+            label       = Utils.getStringOption(options, 'label', false),
+            // the cleaned/trimed label text for attributes like 'id', 'name', ...
+            cleanLabel  = label ? Utils.trimAndCleanString(label.toLowerCase().replace(' ', '')) : false,
+
+            // the tailing label
+            tailNode = '',
+            // the tailing text
+            tail = Utils.getStringOption(options, 'tail', false),
+
+            // the spin field node
+            spinFieldNode = null,
+            spinFieldAttributes = {
+                min: Utils.getNumberOption(options, 'min', 0.0),
+                max: Utils.getNumberOption(options, 'max', 10.0),
+                step: Utils.getNumberOption(options, 'step', 0.1),
+                tabindex: Utils.getIntegerOption(options, 'tabindex', 1),
+                type: Utils.getStringOption(options, 'type', 'number'),
+                value: Utils.getNumberOption(options, 'value')
+            };
+
+        // prepare the spinField
+        if (cleanLabel) { spinFieldAttributes.id = cleanLabel; }
+        if (Utils.getBooleanOption(options, 'required', false)) { spinFieldAttributes.required = ''; }
+        if (Utils.getStringOption(options, 'pattern')) { spinFieldAttributes.pattern = Utils.getStringOption(options, 'pattern'); }
+        spinFieldNode = Forms.createElementMarkup('input', { attributes: spinFieldAttributes });
+
+        // prepare leading label
+        if (label) { labelNode = Forms.createElementMarkup('label', { attributes: { 'class': 'spin-field-label', 'for': cleanLabel }, content: label }); }
+
+        // prepare tailing label
+        if (tail) { tailNode = Forms.createElementMarkup('span', { attributes: { 'class': 'spin-field-tail' }, content: tail }); }
+
+        return Forms.createElementMarkup('div', { attributes: { 'class': classes }, content: labelNode + spinFieldNode + tailNode });
+    };
+
+    // debugging --------------------------------------------------------------
+
+    /**
+     * Creates multi-line text mark-up from the passed value that can be set as
+     * 'title' attribute at an HTML element.
+     *
+     * @param {Any} value
+     *  Any value to be converted to the tooltip content string. Objects and
+     *  arrays will be converted to text recursively.
+     *
+     * @returns {String}
+     *  The tooltip content string, as encoded HTML mark-up.
+     */
+    Forms.createLoggingTooltipMarkup = function (value) {
+
+        var // the resulting mark-up, as array of strings
+            markup = [],
+            // the current line indentation
+            indent = 0;
+
+        function isAtomic(val) {
+            return _.isNumber(val) || _.isBoolean(val) || _.isNull(val) || _.isUndefined(val);
+        }
+
+        function appendLine(text) {
+            markup.push(Utils.repeatString('&nbsp; &nbsp; ', indent) + Utils.escapeHTML(text));
+        }
+
+        function appendProperties(object) {
+            indent += 1;
+            var keys = (object instanceof Error) ? Object.getOwnPropertyNames(object) : Object.keys(object);
+            keys.sort().forEach(function (key) {
+                appendValue(key.replace(/"/g, '\\"') + ': ', object[key]);
+            });
+            indent -= 1;
+        }
+
+        function appendElements(array) {
+            indent += 1;
+            array.forEach(function (element, index) {
+                appendValue(index + ': ', element);
+            });
+            indent -= 1;
+        }
+
+        function appendValue(prefix, val) {
+            if (markup.length > 500) { return; } // prevent endless loops in cyclic references
+            if (isAtomic(val)) {
+                appendLine(prefix + String(val));
+            } else if (_.isString(val)) {
+                val = val.split(/\n/);
+                if (val.length === 1) {
+                    appendLine(prefix + '"' + val[0].replace(/"/g, '\\"') + '"');
+                } else {
+                    appendLine(prefix + 'String [' + val.length + ']');
+                    appendElements(val);
+                }
+            } else if (_.isRegExp(val)) {
+                appendLine(prefix + val);
+            } else if (_.isArray(val)) {
+                if ((val.length <= 8) && val.every(isAtomic)) {
+                    appendLine(prefix + '[' + val.map(String).join(', ') + ']');
+                } else {
+                    appendLine(prefix + 'Array [' + val.length + ']');
+                    appendElements(val);
+                }
+            } else if (_.isObject(val)) {
+                appendLine(prefix + (_.isFunction(val) ? ('Function [' + val.length + ']') : 'Object'));
+                appendProperties(val);
+            }
+        }
+
+        appendValue('', value);
+        return markup.join('&#10;');
+    };
+
     // exports ================================================================
 
     return Forms;
