@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -19,9 +19,6 @@ define('io.ox/office/tk/io', [
 
     'use strict';
 
-    var // static cache for JSON resources already loaded, mapped by resource module name
-        resourceCache = {};
-
     // static class IO ========================================================
 
     /**
@@ -45,44 +42,42 @@ define('io.ox/office/tk/io', [
     // public methods ---------------------------------------------------------
 
     /**
-     * Creates a function that returns a Deferred object which will be resolved
-     * or rejected depending on the result of the passed filter callback
-     * function. Intended to be used in Deferred chains to modify the state of
-     * a Deferred object depending on the result data.
+     * Creates a function that returns a promise which will be resolved or
+     * rejected depending on the result of the passed filter callback function.
+     * Intended to be used in promise chains to modify the state of a promise
+     * depending on the result data.
      *
      * @param {Function} filter
-     *  A function that receives and filters the response of a resolved
-     *  Deferred object as first parameter. If this function returns undefined,
-     *  the Deferred object created by the method will be rejected. Otherwise,
-     *  the Deferred object will be resolved with the return value of this
-     *  filter function.
+     *  A function that receives and filters the response of a resolved promise
+     *  as first parameter. If this function returns undefined, the promise
+     *  created by the method will be rejected. Otherwise, the promise will be
+     *  resolved with the return value of this filter function.
      *
      * @param {Object} [context]
      *  The context object that will be bound to the passed filter function.
      *
      * @returns {Function}
-     *  A new function that returns a Deferred object that has been resolved or
-     *  rejected depending on the result of the passed filter callback
-     *  function.
+     *  A new function that returns a promise that will be resolved or rejected
+     *  depending on the result of the passed filter callback function.
      */
     IO.createDeferredFilter = function (filter, context) {
 
-        // create and return a function that call the filter and returns a resolved/rejected Deferred object
+        // create and return a function that call the filter and returns a resolved/rejected promise
         return function (response) {
 
-            var // the result Deferred object
+            var // the resulting deferred object
                 def = $.Deferred(),
                 // call passed filter callback
                 value = _.isObject(response) ? filter.call(context, response) : undefined;
 
-            // resolve the Deferred object, if the value returned by the filter function is not undefined
+            // resolve the deferred object, if the value returned by the filter function is not undefined
             return _.isUndefined(value) ? def.reject(response) : def.resolve(value);
         };
     };
 
     /**
-     * Sends a request to the server and returns the promise of a Deferred
-     * object waiting for the response.
+     * Sends a request to the server and returns a promise waiting for the
+     * response.
      *
      * @param {String} module
      *  The name of the server module that will receive the request.
@@ -120,7 +115,7 @@ define('io.ox/office/tk/io', [
             resultFilter = Utils.getFunctionOption(options, 'resultFilter'),
             // properties passed to the server request
             requestProps = { module: module, params: params },
-            // the Deferred object representing the core AJAX request
+            // the promise representing the core AJAX request
             ajaxRequest = null,
             // the Promise returned by this method
             promise = null;
@@ -143,33 +138,33 @@ define('io.ox/office/tk/io', [
     };
 
     /**
-     * Reads the specified file and returns the promise of a Deferred object
-     * that will be resolved or rejected depending on the result of the read
-     * operation. The file will be converted to a data URL containing the file
-     * contents as Base-64 encoded data and passed to the resolved Promise.
+     * Reads the specified file and returns a promise that will be resolved or
+     * rejected depending on the result of the read operation. The file will be
+     * converted to a data URL containing the file contents as Base-64 encoded
+     * data and passed to the resolved promise.
      *
      * @param {File} fileDesc
      *  The descriptor of the file to be loaded.
      *
      * @returns {jQuery.Promise}
-     *  The Promise of a Deferred object that will be resolved with the result
-     *  object containing the data URL, or rejected if the read operation
-     *  failed. If the file size is known, the Promise will be notified about
-     *  the progress of the operation (a floating-point number between 0.0 and
-     *  1.0). Contains the additional method 'abort()' that allows to abort the
-     *  running file reader which rejects the promise. Calling this method has
-     *  no effect, if importing the file is finished already.
+     *  A promise that will be resolved with the result object containing the
+     *  data URL, or rejected if the read operation failed. If the file size is
+     *  known, the promise will be notified about the progress of the operation
+     *  (a floating-point number between 0.0 and 1.0). Contains the additional
+     *  method 'abort()' that allows to abort the running file reader which
+     *  rejects the promise. Calling this method has no effect, if importing
+     *  the file is finished already.
      */
     IO.readClientFileAsDataUrl = function (fileDesc) {
 
-        var // Deferred result object
+        var // resulting deferred object
             def = $.Deferred(),
             // create a browser file reader instance
             reader = window.FileReader ? new window.FileReader() : null;
 
         if (reader) {
 
-            // register the load event handler, Deferred object will be resolved with data URL
+            // register the load event handler, deferred object will be resolved with data URL
             reader.onload = function (event) {
                 if (event && event.target && _.isString(event.target.result)) {
                     def.resolve(event.target.result);
@@ -178,17 +173,17 @@ define('io.ox/office/tk/io', [
                 }
             };
 
-            // register error event handlers, Deferred object will be rejected
+            // register error event handlers, deferred object will be rejected
             reader.onerror = function () {
                 def.reject();
             };
 
-            // register abort event handlers, Deferred object will be rejected
+            // register abort event handlers, deferred object will be rejected
             reader.onabort = function () {
                 def.reject('abort');
             };
 
-            // register progress handler, Deferred object will be notified
+            // register progress handler, deferred object will be notified
             reader.onprogress = function (event) {
                 if (event.lengthComputable) {
                     def.notify(event.loaded / event.total);
@@ -215,75 +210,36 @@ define('io.ox/office/tk/io', [
     };
 
     /**
-     * Loads the specified JSON resource module for the current GUI locale. If
-     * there is no resource module available for the locale, tries to fall back
-     * to the plain language code, then to the locale 'en_US', then to the
-     * language code 'en'. All JSON resources loaded once will be cached
-     * internally, and will be returned immediately when calling this method
-     * again.
+     * Loads the specified JSON source file.
      *
-     * @param {String} modulePath
-     *  The module path to the directory containing the JSON resource modules.
-     *  The resource modules must be stored as RequireJS module source file
-     *  '[modulePath]/[LOCALE].json'.
+     * @param {String} fileName
+     *  The name of the file, relative to the application base path, as used by
+     *  RequireJS, without file extension (which is assumed to be 'json').
      *
      * @returns {jQuery.Promise}
-     *  The Promise of a Deferred object that will be resolved with the JSON
-     *  object loaded from the specified resource file, or that will be
-     *  rejected on any error (e.g., resource file not found, or invalid JSON
-     *  data).
+     *  A promise that will be resolved with the JSON data loaded from the
+     *  specified source file, or that will be rejected on any error (e.g.,
+     *  source file not found, or invalid JSON data).
      */
-    IO.loadResource = function (modulePath) {
+    IO.loadJSON = function (fileName) {
 
-        // tries to load the resource file for the passed locale
-        function requireResource(language, country) {
+        // use the text loader of RequireJS to load the JSON file
+        return require(['text!' + fileName + '.json']).then(function (response) {
 
-            return require(['text!' + modulePath + '/' + language + '.json']).then(function (response) {
+            // check response
+            if (!_.isString(response) || (response.length === 0)) {
+                Utils.error('IO.loadJSON(): invalid response');
+                return $.Deferred().reject(response);
+            }
 
-                var // the map with all resources for all countries
-                    resourceData = null;
-
-                // check response
-                if (!_.isString(response) || (response.length === 0)) {
-                    Utils.error('IO.loadResource(): invalid response');
-                    return $.Deferred().reject(response);
-                }
-
-                // parse JSON string to object
-                try {
-                    resourceData = JSON.parse(response);
-                } catch (ex) {
-                    Utils.exception(ex);
-                    return $.Deferred().reject(ex);
-                }
-
-                // validate resource data (must be a map with language and/or local identifiers
-                var keyPattern = new RegExp(_.escapeRegExp(language) + '(_[A-Z]{2})?');
-                _.each(resourceData, function (map, key) {
-                    if (!keyPattern.test(key)) {
-                        Utils.error('IO.loadResource(): invalid map key "' + key + '"');
-                    }
-                });
-
-                // merge country specific resources (e.g. 'en_GB') over generic language resources (e.g. 'en')
-                return Utils.extendOptions(resourceData[language], resourceData[language + '_' + country]);
-            });
-        }
-
-        // the (pending, resolved, or rejected) Promise is stored in the cache
-        if (!(modulePath in resourceCache)) {
-
-            resourceCache[modulePath] =
-                // first, try to load the resource module for the current locale
-                requireResource(Config.LANGUAGE, Config.COUNTRY)
-                // on error, fall back to en_US (unless English is already the current language)
-                .then(null, function (response) {
-                    return (Config.LANGUAGE === 'en') ? response : requireResource('en', 'US');
-                });
-        }
-
-        // return the Promise containing the localized resource object
-        return resourceCache[modulePath];
+            // parse JSON string to object
+            try {
+                return JSON.parse(response);
+            } catch (ex) {
+                Utils.exception(ex);
+                return $.Deferred().reject(ex);
+            }
+        });
     };
 
     /**
