@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -43,7 +43,7 @@ define('io.ox/office/tk/object/baseobject', [
          * Registers a destructor callback function that will be invoked when
          * the method BaseObject.destroy() has been called.
          *
-         * @attention
+         * @internal
          *  Intended to be used by the internal implementations of sub classes.
          *  DO NOT CALL from external code!
          *
@@ -61,7 +61,7 @@ define('io.ox/office/tk/object/baseobject', [
             return this;
         };
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the globally unique identifier for this instance.
@@ -183,7 +183,8 @@ define('io.ox/office/tk/object/baseobject', [
 
                 // detach all matching event listeners for the passed object
                 _.each(listeners, function (data) {
-                    if ((source === data.source) && (!evtTypes || _.contains(evtTypes, data.type)) && (!evtListener || (evtListener === data.listener))) {
+                    // === equals check insufficient for jquery elements -> jq.is()
+                    if ((data.source instanceof $ ? data.source.is(source) : data.source === source) && (!evtTypes || _.contains(evtTypes, data.type)) && (!evtListener || (evtListener === data.listener))) {
                         source.off(data.type, data.listener);
                     }
                 });
