@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -21,8 +21,11 @@ define('io.ox/office/tk/popup/basemenu', [
 
     'use strict';
 
-    var // whether pop-up nodes remain open when losing focus (for debugging)
-        STICKY_POPUPS = Config.DEBUG && Config.getUrlFlag('office:sticky-popups');
+    var // events registered at the document while the menu is open
+        DOCUMENT_EVENTS = 'mousedown touchstart',
+
+        // whether pop-up nodes remain open when losing focus (for debugging)
+        DEBUG_STICKY_POPUPS = Config.getDebugUrlFlag('office:sticky-popups');
 
     // class BaseMenu =========================================================
 
@@ -89,10 +92,7 @@ define('io.ox/office/tk/popup/basemenu', [
             focusableNodeResolvers = [],
 
             // filter selector for preferred focusable elements
-            preferFocusFilter = Utils.getOption(initOptions, 'preferFocusFilter'),
-
-            // events registered at the document while the menu is open
-            DOCUMENT_EVENTS = 'mousedown touchstart';
+            preferFocusFilter = Utils.getOption(initOptions, 'preferFocusFilter');
 
         // base constructor ---------------------------------------------------
 
@@ -129,7 +129,7 @@ define('io.ox/office/tk/popup/basemenu', [
         function hideMenuOnFocusNode(node) {
             if (autoClose && !(node && isFocusableNode(node))) {
                 // do not close the menu, if the debug URL flag 'office:sticky-popups' is set (but still return true)
-                if (!STICKY_POPUPS) { self.hide(); }
+                if (!DEBUG_STICKY_POPUPS) { self.hide(); }
                 return true;
             }
             return false;
@@ -141,7 +141,17 @@ define('io.ox/office/tk/popup/basemenu', [
          */
         function autoFocusContentNode() {
             if (autoFocus && !Utils.isSoftKeyboardOpen()) {
-                self.getContentNode().focus();
+
+                // workaround for US 101274476: self.getContentNode().focus()
+                // should not be called on 1. iPhones iOS safari, 2. in landscape mode,
+                // 3. with the navigation bar (URL) visible. Otherwise the screen jumps
+                // and popups can't be opened (e.g. try to open the font size menu).
+                // This workaround is not perfect, because on rare occasions, when
+                // you switch fast between buttons, the popup can lose it's focus.
+                // In all other cases self.getContentNode().focus(); is called like before.
+                if (!(Utils.isLandscape() && Utils.SMALL_DEVICE && Utils.IOS)) {
+                    self.getContentNode().focus();
+                }
             }
         }
 
@@ -190,7 +200,7 @@ define('io.ox/office/tk/popup/basemenu', [
                 if (self.isVisible()) {
                     self.listenTo($(document), DOCUMENT_EVENTS, globalClickHandler);
                 }
-            });
+            }, undefined, 'TK: popupShowHandler');
         }
 
         /**
@@ -294,6 +304,60 @@ define('io.ox/office/tk/popup/basemenu', [
             return this;
         };
 
+        /**
+         * Returns whether the pop-up menu will be focused after it has been
+         * opened. See constructor option 'autoFocus' for details.
+         *
+         * @returns {Boolean}
+         *  The current state of the auto-focus mode.
+         */
+        this.getAutoFocusMode = function () {
+            return autoFocus;
+        };
+
+        /**
+         * Specifies whether the pop-up menu will be focused after it has been
+         * opened. See constructor option 'autoFocus' for details.
+         *
+         * @param {Boolean} newAutoFocus
+         *  The new state of the auto-focus mode.
+         *
+         * @returns {BaseMenu}
+         *  A reference to this instance.
+         */
+        this.setAutoFocusMode = function (newAutoFocus) {
+            autoFocus = newAutoFocus;
+            return this;
+        };
+
+        /**
+         * Returns whether the menu will be closed automatically, when the
+         * browser focus leaves the pop-up menu node. See constructor option
+         * 'autoClose' for details.
+         *
+         * @returns {Boolean}
+         *  The current state of the auto-close mode.
+         */
+        this.getAutoCloseMode = function () {
+            return autoClose;
+        };
+
+        /**
+         * Specifies whether the menu will be closed automatically, when the
+         * browser focus leaves the pop-up menu node. See constructor option
+         * 'autoClose' for details.
+         *
+         * @param {Boolean} newAutoClose
+         *  The new state of the auto-close mode.
+         *
+         * @returns {BaseMenu}
+         *  A reference to this instance.
+         */
+        this.setAutoCloseMode = function (newAutoClose) {
+            autoClose = newAutoClose;
+            return this;
+        };
+
         // initialization -----------------------------------------------------
 
         // add basic pop-up menu styling
