@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -430,6 +430,12 @@ define('io.ox/office/tk/popup/basepopup', [
             windowArea.width = Math.max(Math.min(window.innerWidth, pageSize.width - keyboardWidth), 0);
             windowArea.height = Math.max(Math.min(window.innerHeight, pageSize.height - keyboardHeight), 0);
 
+            if (Utils.IOS) {
+                //workaround for flickering and self closing popups (comboundmenus)
+                //window.pageYOffset is higher than zero because the body is moved for a short moment.
+                windowArea.top = 0;
+            }
+
             // intersect window area with the bounding box, if existing
             // (results in null, if bounding box is not part of the window area)
             boundingBox = boundingBox ? Utils.getIntersectionRectangle(boundingBox, windowArea) : windowArea;
@@ -772,7 +778,7 @@ define('io.ox/office/tk/popup/basepopup', [
                 }
 
                 // on touch devices, get the size of the virtual keyboard
-                if (Utils.IPAD && (lastActiveElement !== document.activeElement)) {
+                if (Utils.IOS && (lastActiveElement !== document.activeElement)) {
                     lastActiveElement = document.activeElement;
                     needsLayout = true;
                     keyboardWidth = keyboardHeight = 0;
@@ -875,7 +881,7 @@ define('io.ox/office/tk/popup/basepopup', [
 
             // start a timer that regularly refreshes the position of the pop-up node, and
             // hides the pop-up node automatically when the anchor node becomes inaccessible
-            refreshTimer = this.repeatDelayed(function () { refreshNode({ autoHide: true }); }, Modernizr.touch ? 200 : 50);
+            refreshTimer = this.repeatDelayed(function () { refreshNode({ autoHide: true }); }, _.browser.Android ? 200 : 50);
 
             return this;
         };
@@ -941,11 +947,18 @@ define('io.ox/office/tk/popup/basepopup', [
          * @returns {BasePopup}
          *  A reference to this instance.
          */
-        this.clearContents = function () {
+        this.clearContents = function (options) {
+            var except = Utils.getStringOption(options, 'except');
+
             if (this.hasContents()) {
                 this.trigger('popup:beforeclear');
-                contentNode.empty();
+                if (!_.isEmpty(except)) {
+                    contentNode.contents(':not(' + except + ')').remove();
+                } else {
+                    contentNode.empty();
+                }
                 this.refresh();
+                this.trigger('popup:afterclear');
             }
             return this;
         };
@@ -1012,7 +1025,6 @@ define('io.ox/office/tk/popup/basepopup', [
             return busyNode.parent().length > 0;
         };
 
-
         /**
          * Switches this pop-up node to busy mode, showing a busy spinner only,
          * regardless of the actual contents.
@@ -1151,6 +1163,16 @@ define('io.ox/office/tk/popup/basepopup', [
             return this.refreshImmediately();
         };
 
+        /**
+         * Changes the internal anchorBorders, on which the popup opens.
+         *
+         * @param {Array} newAnchorBorder
+         *  The new anchorBorders. The default value for this is ['bottom', 'top'].
+         */
+        this.setAnchorBorders = function (newAnchorBorder) {
+            anchorBorders = newAnchorBorder;
+        };
+
         /**
          * Changes the current position of the pop-up node directly. The pop-up
          * node will remain at the specified position, instead of being aligned
@@ -1171,60 +1193,13 @@ define('io.ox/office/tk/popup/basepopup', [
             return this.refreshImmediately();
         };
 
-        /**
-         * Sets the position of the pop-up node according to the passed CSS
-         * positioning attributes. The resulting position of the pop-up node
-         * will be adjusted according to the current visible area of the
-         * browser window.
-         *
-         * @internal
-         *  Intended to be used by sub classes of this class for custom
-         *  rendering of the pop-up node.
-         *
-         * @deprecated
-         *
-         * @param {Object} cssProps
-         *  The CSS positioning properties for the pop-up node, relative to the
-         *  document page. Must contain one horizontal offset (either 'left' or
-         *  'right'), and one vertical offset (either 'top' or 'bottom').
-         *  Optionally, may contain the new size ('width' and/or 'height') of
-         *  the pop-up node. May contain any other CSS properties.
-         *
-         * @returns {BasePopup}
-         *  A reference to this instance.
-         */
-        this._setNodePosition = function (cssProps) {
-
-            var // available area in the document page
-                boundingBox = resolveBoundingBox(true),
-                // the outer width and height of the pop-up node (may be missing in passed properties)
-                nodeWidth = _.isNumber(cssProps.width) ? (cssProps.width + rootNode.outerWidth() - rootNode.width()) : rootNode.outerWidth(),
-                nodeHeight = _.isNumber(cssProps.height) ? (cssProps.height + rootNode.outerHeight() - rootNode.height()) : rootNode.outerHeight();
-
-            // restrict all existing positioning attributes to the visible area
-            if (boundingBox) {
-                cssProps.left = _.isNumber(cssProps.left) ? Math.max(boundingBox.left, Math.min(cssProps.left, boundingBox.left + boundingBox.width - nodeWidth)) : '';
-                cssProps.right = _.isNumber(cssProps.right) ? Math.max(boundingBox.right, Math.min(cssProps.right, boundingBox.right + boundingBox.width - nodeWidth)) : '';
-                cssProps.top = _.isNumber(cssProps.top) ? Math.max(boundingBox.top, Math.min(cssProps.top, boundingBox.top + boundingBox.height - nodeHeight)) : '';
-                cssProps.bottom = _.isNumber(cssProps.bottom) ? Math.max(boundingBox.bottom, Math.min(cssProps.bottom, boundingBox.bottom + boundingBox.height - nodeHeight)) : '';
-                rootNode.css(cssProps);
-            }
-
-            return this;
-        };
-
         // initialization -----------------------------------------------------
 
         // additional CSS classes passed to constructor
         rootNode.addClass(Utils.getStringOption(initOptions, 'classes', ''));
 
-        // disable context menu
-        rootNode.on('contextmenu', function (event) {
-            if (!$(event.target).is('input,textarea')) { return false; }
-        });
-
         // disable dragging of controls (otherwise, it is possible to drag buttons and other controls around)
-        rootNode.on('contextmenu drop dragstart dragenter dragexit dragover dragleave', false);
+        rootNode.on('drop dragstart dragenter dragexit dragover dragleave', false);
 
         // ARIA role
         if (ariaRole.length > 0) {
