@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2016 OX Software GmbH.
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -112,7 +112,11 @@ define('io.ox/office/tk/popup/listmenu', [
             sorted = Utils.getBooleanOption(initOptions, 'sortItems', false),
 
             // default number of columns in each section node ('grid' design mode)
-            defColumns = Utils.getIntegerOption(initOptions, 'gridColumns', 10, 1);
+            defColumns = Utils.getIntegerOption(initOptions, 'gridColumns', 10, 1),
+
+            // show the title-text in the drop down
+            showTitle = Utils.getBooleanOption(initOptions, 'showTitle', true),
+            title = Utils.getStringOption(initOptions, 'title', '');
 
         // base constructor ---------------------------------------------------
 
@@ -236,6 +240,31 @@ define('io.ox/office/tk/popup/listmenu', [
             }
         }
 
+        function createTitle (value, options) {
+
+            var // the root node of the menu section
+                sectionNode = self.createSectionNode(Utils.getStringOption(options, 'section', '')),
+                titleNode = $('<span>', { class: 'title' }).text(value);
+
+            sectionNode.addClass('title');
+
+            // make section node visible (initially hidden)
+            Forms.showNodes(sectionNode, true);
+
+            // create a new row element for the button if required
+            var rowNode = sectionNode.find('>.grid-row').last();
+            if ((rowNode.length === 0) || (rowNode.children().length === sectionNode.data('grid-columns'))) {
+                rowNode = $('<div class="grid-row" role="row">').appendTo(sectionNode);
+            }
+            // append the button to the row (wrap into a helper node for ARIA)
+            var cellNode = $('<div class="grid-cell" role="gridcell">').attr('aria-label', titleNode.attr('aria-label') || '').append(titleNode);
+            rowNode.append(cellNode);
+
+            // notify listeners
+            // this.trigger('create:item', titleNode, value, options);
+            return titleNode;
+        }
+
         // public methods -----------------------------------------------------
 
         /**
@@ -387,6 +416,7 @@ define('io.ox/office/tk/popup/listmenu', [
 
             // initialize button node
             Forms.setButtonValue(buttonNode, value, options);
+            Forms.setToolTip(buttonNode, options);
             buttonNode.data('index', Utils.getOption(options, 'sortIndex', null));
             buttonNode.attr('role', 'option');
 
@@ -713,6 +743,13 @@ define('io.ox/office/tk/popup/listmenu', [
 
         // initialization -----------------------------------------------------
 
+        this.one('popup:beforeshow', function () {
+            if (showTitle && title) {
+                self.createSectionNode('title', { gridColumns: 1 });
+                createTitle(title, { section: 'title' });
+            }
+        });
+
         // add CSS marker for the item design
         this.getNode().addClass('list-menu').attr({ 'data-design': design, role: 'listbox' });
 
