@@ -7,6 +7,8 @@ module.exports = function(config) {
     // base path that will be used to resolve all patterns (eg. files, exclude)
     basePath: 'build/',
 
+    browserNoActivityTimeout: 50000, // default: 10000
+
     // frameworks to use
     // available frameworks: https://npmjs.org/browse/keyword/karma-adapter
     frameworks: ['mocha', 'requirejs', 'chai', 'sinon'],
@@ -18,8 +20,8 @@ module.exports = function(config) {
       '../bower_components/jquery/dist/jquery.js',
       'spec/lib/**/*.js',
       'spec/test-all.js',
-      {pattern: 'apps/**/*.js', included: false},
-      {pattern: 'spec/**/*_spec.js', included: false}
+      { pattern: 'apps/**/*.{js,json}', included: false },
+      { pattern: 'spec/**/*_spec.js', included: false }
     ],
 
 
